<?php

namespace HS\Install\System\Checks;

use Illuminate\Database\DatabaseManager;
use HS\Install\System\SystemCheckInterface;

class SqlServerFullTextInstalled implements SystemCheckInterface
{
    /**
     * @var \Illuminate\Database\DatabaseManager
     */
    protected $db;

    /**
     * If system check is valid (OK).
     * @var bool
     */
    protected $valid;

    /**
     * Error.
     * @var string
     */
    protected $error;

    public function __construct(DatabaseManager $db)
    {
        $this->db = $db;
    }

    /**
     * This is required to pass.
     * @return bool
     */
    public function required()
    {
        return true;
    }

    /**
     * If we cannot connect to the DB
     * we get an exception here.
     * @return bool
     */
    public function valid()
    {
        // if we aren't using sqlsrv then just pretend it works.
        if (config('database.default') != 'sqlsrv') {
            return true;
        }

        if (is_bool($this->valid)) {
            return $this->valid;
        }

        try {
            $result = $this->db->select("SELECT FullTextServiceProperty('IsFullTextInstalled') as fulltext");
            $result = $result[0];
            if ($result->fulltext == "1") {
                $this->valid = true;
            } else {
                $this->valid = false;
            }
        } catch (\Exception $e) {
            $this->error = $e->getMessage();
            $this->valid = false;
        }

        if ($this->valid === false) {
            $this->error = 'SQL Server Full Text Indexing is Not Installed';
        }

        return $this->valid;
    }

    /**
     * @return mixed
     */
    public function getError()
    {
        return $this->error;
    }
}
