/*  Prototype JavaScript framework, version 1.7
 *  (c) 2005-2010 Sam Stephenson
 *
 *  Prototype is freely distributable under the terms of an MIT-style license.
 *  For details, see the Prototype web site: http://www.prototypejs.org/
 *
 *--------------------------------------------------------------------------*/

var Prototype = {

  Version: '1.7',

  Browser: (function(){
    var ua = navigator.userAgent;
    var isOpera = Object.prototype.toString.call(window.opera) == '[object Opera]';
    return {
      IE:             !!window.attachEvent && !isOpera,
      Opera:          isOpera,
      WebKit:         ua.indexOf('AppleWebKit/') > -1,
      Gecko:          ua.indexOf('Gecko') > -1 && ua.indexOf('KHTML') === -1,
      MobileSafari:   /Apple.*Mobile/.test(ua)
    }
  })(),

  BrowserFeatures: {
    XPath: !!document.evaluate,

    SelectorsAPI: !!document.querySelector,

    ElementExtensions: (function() {
      var constructor = window.Element || window.HTMLElement;
      return !!(constructor && constructor.prototype);
    })(),
    SpecificElementExtensions: (function() {
      if (typeof window.HTMLDivElement !== 'undefined')
        return true;

      var div = document.createElement('div'),
          form = document.createElement('form'),
          isSupported = false;

      if (div['__proto__'] && (div['__proto__'] !== form['__proto__'])) {
        isSupported = true;
      }

      div = form = null;

      return isSupported;
    })()
  },

  ScriptFragment: '<script[^>]*>([\\S\\s]*?)<\/script>',
  JSONFilter: /^\/\*-secure-([\s\S]*)\*\/\s*$/,

  emptyFunction: function() { },

  K: function(x) { return x }
};

if (Prototype.Browser.MobileSafari)
  Prototype.BrowserFeatures.SpecificElementExtensions = false;
/* Based on Alex Arnell's inheritance implementation. */

var Class = (function() {

  var IS_DONTENUM_BUGGY = (function(){
    for (var p in { toString: 1 }) {
      if (p === 'toString') return false;
    }
    return true;
  })();

  function subclass() {};
  function create() {
    var parent = null, properties = $A(arguments);
    if (Object.isFunction(properties[0]))
      parent = properties.shift();

    function klass() {
      this.initialize.apply(this, arguments);
    }

    Object.extend(klass, Class.Methods);
    klass.superclass = parent;
    klass.subclasses = [];

    if (parent) {
      subclass.prototype = parent.prototype;
      klass.prototype = new subclass;
      parent.subclasses.push(klass);
    }

    for (var i = 0, length = properties.length; i < length; i++)
      klass.addMethods(properties[i]);

    if (!klass.prototype.initialize)
      klass.prototype.initialize = Prototype.emptyFunction;

    klass.prototype.constructor = klass;
    return klass;
  }

  function addMethods(source) {
    var ancestor   = this.superclass && this.superclass.prototype,
        properties = Object.keys(source);

    if (IS_DONTENUM_BUGGY) {
      if (source.toString != Object.prototype.toString)
        properties.push("toString");
      if (source.valueOf != Object.prototype.valueOf)
        properties.push("valueOf");
    }

    for (var i = 0, length = properties.length; i < length; i++) {
      var property = properties[i], value = source[property];
      if (ancestor && Object.isFunction(value) &&
          value.argumentNames()[0] == "$super") {
        var method = value;
        value = (function(m) {
          return function() { return ancestor[m].apply(this, arguments); };
        })(property).wrap(method);

        value.valueOf = method.valueOf.bind(method);
        value.toString = method.toString.bind(method);
      }
      this.prototype[property] = value;
    }

    return this;
  }

  return {
    create: create,
    Methods: {
      addMethods: addMethods
    }
  };
})();
(function() {

  var _toString = Object.prototype.toString,
      NULL_TYPE = 'Null',
      UNDEFINED_TYPE = 'Undefined',
      BOOLEAN_TYPE = 'Boolean',
      NUMBER_TYPE = 'Number',
      STRING_TYPE = 'String',
      OBJECT_TYPE = 'Object',
      FUNCTION_CLASS = '[object Function]',
      BOOLEAN_CLASS = '[object Boolean]',
      NUMBER_CLASS = '[object Number]',
      STRING_CLASS = '[object String]',
      ARRAY_CLASS = '[object Array]',
      DATE_CLASS = '[object Date]',
      NATIVE_JSON_STRINGIFY_SUPPORT = window.JSON &&
        typeof JSON.stringify === 'function' &&
        JSON.stringify(0) === '0' &&
        typeof JSON.stringify(Prototype.K) === 'undefined';

  function Type(o) {
    switch(o) {
      case null: return NULL_TYPE;
      case (void 0): return UNDEFINED_TYPE;
    }
    var type = typeof o;
    switch(type) {
      case 'boolean': return BOOLEAN_TYPE;
      case 'number':  return NUMBER_TYPE;
      case 'string':  return STRING_TYPE;
    }
    return OBJECT_TYPE;
  }

  function extend(destination, source) {
    for (var property in source)
      destination[property] = source[property];
    return destination;
  }

  function inspect(object) {
    try {
      if (isUndefined(object)) return 'undefined';
      if (object === null) return 'null';
      return object.inspect ? object.inspect() : String(object);
    } catch (e) {
      if (e instanceof RangeError) return '...';
      throw e;
    }
  }

  function toJSON(value) {
    return Str('', { '': value }, []);
  }

  function Str(key, holder, stack) {
    var value = holder[key],
        type = typeof value;

    if (Type(value) === OBJECT_TYPE && typeof value.toJSON === 'function') {
      value = value.toJSON(key);
    }

    var _class = _toString.call(value);

    switch (_class) {
      case NUMBER_CLASS:
      case BOOLEAN_CLASS:
      case STRING_CLASS:
        value = value.valueOf();
    }

    switch (value) {
      case null: return 'null';
      case true: return 'true';
      case false: return 'false';
    }

    type = typeof value;
    switch (type) {
      case 'string':
        return value.inspect(true);
      case 'number':
        return isFinite(value) ? String(value) : 'null';
      case 'object':

        for (var i = 0, length = stack.length; i < length; i++) {
          if (stack[i] === value) { throw new TypeError(); }
        }
        stack.push(value);

        var partial = [];
        if (_class === ARRAY_CLASS) {
          for (var i = 0, length = value.length; i < length; i++) {
            var str = Str(i, value, stack);
            partial.push(typeof str === 'undefined' ? 'null' : str);
          }
          partial = '[' + partial.join(',') + ']';
        } else {
          var keys = Object.keys(value);
          for (var i = 0, length = keys.length; i < length; i++) {
            var key = keys[i], str = Str(key, value, stack);
            if (typeof str !== "undefined") {
               partial.push(key.inspect(true)+ ':' + str);
             }
          }
          partial = '{' + partial.join(',') + '}';
        }
        stack.pop();
        return partial;
    }
  }

  function stringify(object) {
    return JSON.stringify(object);
  }

  function toQueryString(object) {
    return $H(object).toQueryString();
  }

  function toHTML(object) {
    return object && object.toHTML ? object.toHTML() : String.interpret(object);
  }

  function keys(object) {
    if (Type(object) !== OBJECT_TYPE) { throw new TypeError(); }
    var results = [];
    for (var property in object) {
      if (object.hasOwnProperty(property)) {
        results.push(property);
      }
    }
    return results;
  }

  function values(object) {
    var results = [];
    for (var property in object)
      results.push(object[property]);
    return results;
  }

  function clone(object) {
    return extend({ }, object);
  }

  function isElement(object) {
    return !!(object && object.nodeType == 1);
  }

  function isArray(object) {
    return _toString.call(object) === ARRAY_CLASS;
  }

  var hasNativeIsArray = (typeof Array.isArray == 'function')
    && Array.isArray([]) && !Array.isArray({});

  if (hasNativeIsArray) {
    isArray = Array.isArray;
  }

  function isHash(object) {
    return object instanceof Hash;
  }

  function isFunction(object) {
    return _toString.call(object) === FUNCTION_CLASS;
  }

  function isString(object) {
    return _toString.call(object) === STRING_CLASS;
  }

  function isNumber(object) {
    return _toString.call(object) === NUMBER_CLASS;
  }

  function isDate(object) {
    return _toString.call(object) === DATE_CLASS;
  }

  function isUndefined(object) {
    return typeof object === "undefined";
  }

  extend(Object, {
    extend:        extend,
    inspect:       inspect,
    toJSON:        NATIVE_JSON_STRINGIFY_SUPPORT ? stringify : toJSON,
    toQueryString: toQueryString,
    toHTML:        toHTML,
    keys:          Object.keys || keys,
    values:        values,
    clone:         clone,
    isElement:     isElement,
    isArray:       isArray,
    isHash:        isHash,
    isFunction:    isFunction,
    isString:      isString,
    isNumber:      isNumber,
    isDate:        isDate,
    isUndefined:   isUndefined
  });
})();
Object.extend(Function.prototype, (function() {
  var slice = Array.prototype.slice;

  function update(array, args) {
    var arrayLength = array.length, length = args.length;
    while (length--) array[arrayLength + length] = args[length];
    return array;
  }

  function merge(array, args) {
    array = slice.call(array, 0);
    return update(array, args);
  }

  function argumentNames() {
    var names = this.toString().match(/^[\s\(]*function[^(]*\(([^)]*)\)/)[1]
      .replace(/\/\/.*?[\r\n]|\/\*(?:.|[\r\n])*?\*\//g, '')
      .replace(/\s+/g, '').split(',');
    return names.length == 1 && !names[0] ? [] : names;
  }

  function bind(context) {
    if (arguments.length < 2 && Object.isUndefined(arguments[0])) return this;
    var __method = this, args = slice.call(arguments, 1);
    return function() {
      var a = merge(args, arguments);
      return __method.apply(context, a);
    }
  }

  function bindAsEventListener(context) {
    var __method = this, args = slice.call(arguments, 1);
    return function(event) {
      var a = update([event || window.event], args);
      return __method.apply(context, a);
    }
  }

  function curry() {
    if (!arguments.length) return this;
    var __method = this, args = slice.call(arguments, 0);
    return function() {
      var a = merge(args, arguments);
      return __method.apply(this, a);
    }
  }

  function delay(timeout) {
    var __method = this, args = slice.call(arguments, 1);
    timeout = timeout * 1000;
    return window.setTimeout(function() {
      return __method.apply(__method, args);
    }, timeout);
  }

  function defer() {
    var args = update([0.01], arguments);
    return this.delay.apply(this, args);
  }

  function wrap(wrapper) {
    var __method = this;
    return function() {
      var a = update([__method.bind(this)], arguments);
      return wrapper.apply(this, a);
    }
  }

  function methodize() {
    if (this._methodized) return this._methodized;
    var __method = this;
    return this._methodized = function() {
      var a = update([this], arguments);
      return __method.apply(null, a);
    };
  }

  return {
    argumentNames:       argumentNames,
    bind:                bind,
    bindAsEventListener: bindAsEventListener,
    curry:               curry,
    delay:               delay,
    defer:               defer,
    wrap:                wrap,
    methodize:           methodize
  }
})());



(function(proto) {


  function toISOString() {
    return this.getUTCFullYear() + '-' +
      (this.getUTCMonth() + 1).toPaddedString(2) + '-' +
      this.getUTCDate().toPaddedString(2) + 'T' +
      this.getUTCHours().toPaddedString(2) + ':' +
      this.getUTCMinutes().toPaddedString(2) + ':' +
      this.getUTCSeconds().toPaddedString(2) + 'Z';
  }


  function toJSON() {
    return this.toISOString();
  }

  if (!proto.toISOString) proto.toISOString = toISOString;
  if (!proto.toJSON) proto.toJSON = toJSON;

})(Date.prototype);


RegExp.prototype.match = RegExp.prototype.test;

RegExp.escape = function(str) {
  return String(str).replace(/([.*+?^=!:${}()|[\]\/\\])/g, '\\$1');
};
var PeriodicalExecuter = Class.create({
  initialize: function(callback, frequency) {
    this.callback = callback;
    this.frequency = frequency;
    this.currentlyExecuting = false;

    this.registerCallback();
  },

  registerCallback: function() {
    this.timer = setInterval(this.onTimerEvent.bind(this), this.frequency * 1000);
  },

  execute: function() {
    this.callback(this);
  },

  stop: function() {
    if (!this.timer) return;
    clearInterval(this.timer);
    this.timer = null;
  },

  onTimerEvent: function() {
    if (!this.currentlyExecuting) {
      try {
        this.currentlyExecuting = true;
        this.execute();
        this.currentlyExecuting = false;
      } catch(e) {
        this.currentlyExecuting = false;
        throw e;
      }
    }
  }
});
Object.extend(String, {
  interpret: function(value) {
    return value == null ? '' : String(value);
  },
  specialChar: {
    '\b': '\\b',
    '\t': '\\t',
    '\n': '\\n',
    '\f': '\\f',
    '\r': '\\r',
    '\\': '\\\\'
  }
});

Object.extend(String.prototype, (function() {
  var NATIVE_JSON_PARSE_SUPPORT = window.JSON &&
    typeof JSON.parse === 'function' &&
    JSON.parse('{"test": true}').test;

  function prepareReplacement(replacement) {
    if (Object.isFunction(replacement)) return replacement;
    var template = new Template(replacement);
    return function(match) { return template.evaluate(match) };
  }

  function gsub(pattern, replacement) {
    var result = '', source = this, match;
    replacement = prepareReplacement(replacement);

    if (Object.isString(pattern))
      pattern = RegExp.escape(pattern);

    if (!(pattern.length || pattern.source)) {
      replacement = replacement('');
      return replacement + source.split('').join(replacement) + replacement;
    }

    while (source.length > 0) {
      if (match = source.match(pattern)) {
        result += source.slice(0, match.index);
        result += String.interpret(replacement(match));
        source  = source.slice(match.index + match[0].length);
      } else {
        result += source, source = '';
      }
    }
    return result;
  }

  function sub(pattern, replacement, count) {
    replacement = prepareReplacement(replacement);
    count = Object.isUndefined(count) ? 1 : count;

    return this.gsub(pattern, function(match) {
      if (--count < 0) return match[0];
      return replacement(match);
    });
  }

  function scan(pattern, iterator) {
    this.gsub(pattern, iterator);
    return String(this);
  }

  function truncate(length, truncation) {
    length = length || 30;
    truncation = Object.isUndefined(truncation) ? '...' : truncation;
    return this.length > length ?
      this.slice(0, length - truncation.length) + truncation : String(this);
  }

  function strip() {
    return this.replace(/^\s+/, '').replace(/\s+$/, '');
  }

  function stripTags() {
    return this.replace(/<\w+(\s+("[^"]*"|'[^']*'|[^>])+)?>|<\/\w+>/gi, '');
  }

  function stripScripts() {
    return this.replace(new RegExp(Prototype.ScriptFragment, 'img'), '');
  }

  function extractScripts() {
    var matchAll = new RegExp(Prototype.ScriptFragment, 'img'),
        matchOne = new RegExp(Prototype.ScriptFragment, 'im');
    return (this.match(matchAll) || []).map(function(scriptTag) {
      return (scriptTag.match(matchOne) || ['', ''])[1];
    });
  }

  function evalScripts() {
    return this.extractScripts().map(function(script) { return eval(script) });
  }

  function escapeHTML() {
    return this.replace(/&/g,'&amp;').replace(/</g,'&lt;').replace(/>/g,'&gt;');
  }

  function unescapeHTML() {
    return this.stripTags().replace(/&lt;/g,'<').replace(/&gt;/g,'>').replace(/&amp;/g,'&');
  }


  function toQueryParams(separator) {
    var match = this.strip().match(/([^?#]*)(#.*)?$/);
    if (!match) return { };

    return match[1].split(separator || '&').inject({ }, function(hash, pair) {
      if ((pair = pair.split('='))[0]) {
        var key = decodeURIComponent(pair.shift()),
            value = pair.length > 1 ? pair.join('=') : pair[0];

        if (value != undefined) value = decodeURIComponent(value);

        if (key in hash) {
          if (!Object.isArray(hash[key])) hash[key] = [hash[key]];
          hash[key].push(value);
        }
        else hash[key] = value;
      }
      return hash;
    });
  }

  function toArray() {
    return this.split('');
  }

  function succ() {
    return this.slice(0, this.length - 1) +
      String.fromCharCode(this.charCodeAt(this.length - 1) + 1);
  }

  function times(count) {
    return count < 1 ? '' : new Array(count + 1).join(this);
  }

  function camelize() {
    return this.replace(/-+(.)?/g, function(match, chr) {
      return chr ? chr.toUpperCase() : '';
    });
  }

  function capitalize() {
    return this.charAt(0).toUpperCase() + this.substring(1).toLowerCase();
  }

  function underscore() {
    return this.replace(/::/g, '/')
               .replace(/([A-Z]+)([A-Z][a-z])/g, '$1_$2')
               .replace(/([a-z\d])([A-Z])/g, '$1_$2')
               .replace(/-/g, '_')
               .toLowerCase();
  }

  function dasherize() {
    return this.replace(/_/g, '-');
  }

  function inspect(useDoubleQuotes) {
    var escapedString = this.replace(/[\x00-\x1f\\]/g, function(character) {
      if (character in String.specialChar) {
        return String.specialChar[character];
      }
      return '\\u00' + character.charCodeAt().toPaddedString(2, 16);
    });
    if (useDoubleQuotes) return '"' + escapedString.replace(/"/g, '\\"') + '"';
    return "'" + escapedString.replace(/'/g, '\\\'') + "'";
  }

  function unfilterJSON(filter) {
    return this.replace(filter || Prototype.JSONFilter, '$1');
  }

  function isJSON() {
    var str = this;
    if (str.blank()) return false;
    str = str.replace(/\\(?:["\\\/bfnrt]|u[0-9a-fA-F]{4})/g, '@');
    str = str.replace(/"[^"\\\n\r]*"|true|false|null|-?\d+(?:\.\d*)?(?:[eE][+\-]?\d+)?/g, ']');
    str = str.replace(/(?:^|:|,)(?:\s*\[)+/g, '');
    return (/^[\],:{}\s]*$/).test(str);
  }

  function evalJSON(sanitize) {
    var json = this.unfilterJSON(),
        cx = /[\u0000\u00ad\u0600-\u0604\u070f\u17b4\u17b5\u200c-\u200f\u2028-\u202f\u2060-\u206f\ufeff\ufff0-\uffff]/g;
    if (cx.test(json)) {
      json = json.replace(cx, function (a) {
        return '\\u' + ('0000' + a.charCodeAt(0).toString(16)).slice(-4);
      });
    }
    try {
      if (!sanitize || json.isJSON()) return eval('(' + json + ')');
    } catch (e) { }
    throw new SyntaxError('Badly formed JSON string: ' + this.inspect());
  }

  function parseJSON() {
    var json = this.unfilterJSON();
    return JSON.parse(json);
  }

  function include(pattern) {
    return this.indexOf(pattern) > -1;
  }

  function startsWith(pattern) {
    return this.lastIndexOf(pattern, 0) === 0;
  }

  function endsWith(pattern) {
    var d = this.length - pattern.length;
    return d >= 0 && this.indexOf(pattern, d) === d;
  }

  function empty() {
    return this == '';
  }

  function blank() {
    return /^\s*$/.test(this);
  }

  function interpolate(object, pattern) {
    return new Template(this, pattern).evaluate(object);
  }

  return {
    gsub:           gsub,
    sub:            sub,
    scan:           scan,
    truncate:       truncate,
    strip:          String.prototype.trim || strip,
    stripTags:      stripTags,
    stripScripts:   stripScripts,
    extractScripts: extractScripts,
    evalScripts:    evalScripts,
    escapeHTML:     escapeHTML,
    unescapeHTML:   unescapeHTML,
    toQueryParams:  toQueryParams,
    parseQuery:     toQueryParams,
    toArray:        toArray,
    succ:           succ,
    times:          times,
    camelize:       camelize,
    capitalize:     capitalize,
    underscore:     underscore,
    dasherize:      dasherize,
    inspect:        inspect,
    unfilterJSON:   unfilterJSON,
    isJSON:         isJSON,
    evalJSON:       NATIVE_JSON_PARSE_SUPPORT ? parseJSON : evalJSON,
    include:        include,
    startsWith:     startsWith,
    endsWith:       endsWith,
    empty:          empty,
    blank:          blank,
    interpolate:    interpolate
  };
})());

var Template = Class.create({
  initialize: function(template, pattern) {
    this.template = template.toString();
    this.pattern = pattern || Template.Pattern;
  },

  evaluate: function(object) {
    if (object && Object.isFunction(object.toTemplateReplacements))
      object = object.toTemplateReplacements();

    return this.template.gsub(this.pattern, function(match) {
      if (object == null) return (match[1] + '');

      var before = match[1] || '';
      if (before == '\\') return match[2];

      var ctx = object, expr = match[3],
          pattern = /^([^.[]+|\[((?:.*?[^\\])?)\])(\.|\[|$)/;

      match = pattern.exec(expr);
      if (match == null) return before;

      while (match != null) {
        var comp = match[1].startsWith('[') ? match[2].replace(/\\\\]/g, ']') : match[1];
        ctx = ctx[comp];
        if (null == ctx || '' == match[3]) break;
        expr = expr.substring('[' == match[3] ? match[1].length : match[0].length);
        match = pattern.exec(expr);
      }

      return before + String.interpret(ctx);
    });
  }
});
Template.Pattern = /(^|.|\r|\n)(#\{(.*?)\})/;

var $break = { };

var Enumerable = (function() {
  function each(iterator, context) {
    var index = 0;
    try {
      this._each(function(value) {
        iterator.call(context, value, index++);
      });
    } catch (e) {
      if (e != $break) throw e;
    }
    return this;
  }

  function eachSlice(number, iterator, context) {
    var index = -number, slices = [], array = this.toArray();
    if (number < 1) return array;
    while ((index += number) < array.length)
      slices.push(array.slice(index, index+number));
    return slices.collect(iterator, context);
  }

  function all(iterator, context) {
    iterator = iterator || Prototype.K;
    var result = true;
    this.each(function(value, index) {
      result = result && !!iterator.call(context, value, index);
      if (!result) throw $break;
    });
    return result;
  }

  function any(iterator, context) {
    iterator = iterator || Prototype.K;
    var result = false;
    this.each(function(value, index) {
      if (result = !!iterator.call(context, value, index))
        throw $break;
    });
    return result;
  }

  function collect(iterator, context) {
    iterator = iterator || Prototype.K;
    var results = [];
    this.each(function(value, index) {
      results.push(iterator.call(context, value, index));
    });
    return results;
  }

  function detect(iterator, context) {
    var result;
    this.each(function(value, index) {
      if (iterator.call(context, value, index)) {
        result = value;
        throw $break;
      }
    });
    return result;
  }

  function findAll(iterator, context) {
    var results = [];
    this.each(function(value, index) {
      if (iterator.call(context, value, index))
        results.push(value);
    });
    return results;
  }

  function grep(filter, iterator, context) {
    iterator = iterator || Prototype.K;
    var results = [];

    if (Object.isString(filter))
      filter = new RegExp(RegExp.escape(filter));

    this.each(function(value, index) {
      if (filter.match(value))
        results.push(iterator.call(context, value, index));
    });
    return results;
  }

  function include(object) {
    if (Object.isFunction(this.indexOf))
      if (this.indexOf(object) != -1) return true;

    var found = false;
    this.each(function(value) {
      if (value == object) {
        found = true;
        throw $break;
      }
    });
    return found;
  }

  function inGroupsOf(number, fillWith) {
    fillWith = Object.isUndefined(fillWith) ? null : fillWith;
    return this.eachSlice(number, function(slice) {
      while(slice.length < number) slice.push(fillWith);
      return slice;
    });
  }

  function inject(memo, iterator, context) {
    this.each(function(value, index) {
      memo = iterator.call(context, memo, value, index);
    });
    return memo;
  }

  function invoke(method) {
    var args = $A(arguments).slice(1);
    return this.map(function(value) {
      return value[method].apply(value, args);
    });
  }

  function max(iterator, context) {
    iterator = iterator || Prototype.K;
    var result;
    this.each(function(value, index) {
      value = iterator.call(context, value, index);
      if (result == null || value >= result)
        result = value;
    });
    return result;
  }

  function min(iterator, context) {
    iterator = iterator || Prototype.K;
    var result;
    this.each(function(value, index) {
      value = iterator.call(context, value, index);
      if (result == null || value < result)
        result = value;
    });
    return result;
  }

  function partition(iterator, context) {
    iterator = iterator || Prototype.K;
    var trues = [], falses = [];
    this.each(function(value, index) {
      (iterator.call(context, value, index) ?
        trues : falses).push(value);
    });
    return [trues, falses];
  }

  function pluck(property) {
    var results = [];
    this.each(function(value) {
      results.push(value[property]);
    });
    return results;
  }

  function reject(iterator, context) {
    var results = [];
    this.each(function(value, index) {
      if (!iterator.call(context, value, index))
        results.push(value);
    });
    return results;
  }

  function sortBy(iterator, context) {
    return this.map(function(value, index) {
      return {
        value: value,
        criteria: iterator.call(context, value, index)
      };
    }).sort(function(left, right) {
      var a = left.criteria, b = right.criteria;
      return a < b ? -1 : a > b ? 1 : 0;
    }).pluck('value');
  }

  function toArray() {
    return this.map();
  }

  function zip() {
    var iterator = Prototype.K, args = $A(arguments);
    if (Object.isFunction(args.last()))
      iterator = args.pop();

    var collections = [this].concat(args).map($A);
    return this.map(function(value, index) {
      return iterator(collections.pluck(index));
    });
  }

  function size() {
    return this.toArray().length;
  }

  function inspect() {
    return '#<Enumerable:' + this.toArray().inspect() + '>';
  }









  return {
    each:       each,
    eachSlice:  eachSlice,
    all:        all,
    every:      all,
    any:        any,
    some:       any,
    collect:    collect,
    map:        collect,
    detect:     detect,
    findAll:    findAll,
    select:     findAll,
    filter:     findAll,
    grep:       grep,
    include:    include,
    member:     include,
    inGroupsOf: inGroupsOf,
    inject:     inject,
    invoke:     invoke,
    max:        max,
    min:        min,
    partition:  partition,
    pluck:      pluck,
    reject:     reject,
    sortBy:     sortBy,
    toArray:    toArray,
    entries:    toArray,
    zip:        zip,
    size:       size,
    inspect:    inspect,
    find:       detect
  };
})();

function $A(iterable) {
  if (!iterable) return [];
  if ('toArray' in Object(iterable)) return iterable.toArray();
  var length = iterable.length || 0, results = new Array(length);
  while (length--) results[length] = iterable[length];
  return results;
}


function $w(string) {
  if (!Object.isString(string)) return [];
  string = string.strip();
  return string ? string.split(/\s+/) : [];
}

Array.from = $A;


(function() {
  var arrayProto = Array.prototype,
      slice = arrayProto.slice,
      _each = arrayProto.forEach; // use native browser JS 1.6 implementation if available

  function each(iterator, context) {
    for (var i = 0, length = this.length >>> 0; i < length; i++) {
      if (i in this) iterator.call(context, this[i], i, this);
    }
  }
  if (!_each) _each = each;

  function clear() {
    this.length = 0;
    return this;
  }

  function first() {
    return this[0];
  }

  function last() {
    return this[this.length - 1];
  }

  function compact() {
    return this.select(function(value) {
      return value != null;
    });
  }

  function flatten() {
    return this.inject([], function(array, value) {
      if (Object.isArray(value))
        return array.concat(value.flatten());
      array.push(value);
      return array;
    });
  }

  function without() {
    var values = slice.call(arguments, 0);
    return this.select(function(value) {
      return !values.include(value);
    });
  }

  function reverse(inline) {
    return (inline === false ? this.toArray() : this)._reverse();
  }

  function uniq(sorted) {
    return this.inject([], function(array, value, index) {
      if (0 == index || (sorted ? array.last() != value : !array.include(value)))
        array.push(value);
      return array;
    });
  }

  function intersect(array) {
    return this.uniq().findAll(function(item) {
      return array.detect(function(value) { return item === value });
    });
  }


  function clone() {
    return slice.call(this, 0);
  }

  function size() {
    return this.length;
  }

  function inspect() {
    return '[' + this.map(Object.inspect).join(', ') + ']';
  }

  function indexOf(item, i) {
    i || (i = 0);
    var length = this.length;
    if (i < 0) i = length + i;
    for (; i < length; i++)
      if (this[i] === item) return i;
    return -1;
  }

  function lastIndexOf(item, i) {
    i = isNaN(i) ? this.length : (i < 0 ? this.length + i : i) + 1;
    var n = this.slice(0, i).reverse().indexOf(item);
    return (n < 0) ? n : i - n - 1;
  }

  function concat() {
    var array = slice.call(this, 0), item;
    for (var i = 0, length = arguments.length; i < length; i++) {
      item = arguments[i];
      if (Object.isArray(item) && !('callee' in item)) {
        for (var j = 0, arrayLength = item.length; j < arrayLength; j++)
          array.push(item[j]);
      } else {
        array.push(item);
      }
    }
    return array;
  }

  Object.extend(arrayProto, Enumerable);

  if (!arrayProto._reverse)
    arrayProto._reverse = arrayProto.reverse;

  Object.extend(arrayProto, {
    _each:     _each,
    clear:     clear,
    first:     first,
    last:      last,
    compact:   compact,
    flatten:   flatten,
    without:   without,
    reverse:   reverse,
    uniq:      uniq,
    intersect: intersect,
    clone:     clone,
    toArray:   clone,
    size:      size,
    inspect:   inspect
  });

  var CONCAT_ARGUMENTS_BUGGY = (function() {
    return [].concat(arguments)[0][0] !== 1;
  })(1,2)

  if (CONCAT_ARGUMENTS_BUGGY) arrayProto.concat = concat;

  if (!arrayProto.indexOf) arrayProto.indexOf = indexOf;
  if (!arrayProto.lastIndexOf) arrayProto.lastIndexOf = lastIndexOf;
})();
function $H(object) {
  return new Hash(object);
};

var Hash = Class.create(Enumerable, (function() {
  function initialize(object) {
    this._object = Object.isHash(object) ? object.toObject() : Object.clone(object);
  }


  function _each(iterator) {
    for (var key in this._object) {
      var value = this._object[key], pair = [key, value];
      pair.key = key;
      pair.value = value;
      iterator(pair);
    }
  }

  function set(key, value) {
    return this._object[key] = value;
  }

  function get(key) {
    if (this._object[key] !== Object.prototype[key])
      return this._object[key];
  }

  function unset(key) {
    var value = this._object[key];
    delete this._object[key];
    return value;
  }

  function toObject() {
    return Object.clone(this._object);
  }



  function keys() {
    return this.pluck('key');
  }

  function values() {
    return this.pluck('value');
  }

  function index(value) {
    var match = this.detect(function(pair) {
      return pair.value === value;
    });
    return match && match.key;
  }

  function merge(object) {
    return this.clone().update(object);
  }

  function update(object) {
    return new Hash(object).inject(this, function(result, pair) {
      result.set(pair.key, pair.value);
      return result;
    });
  }

  function toQueryPair(key, value) {
    if (Object.isUndefined(value)) return key;
    return key + '=' + encodeURIComponent(String.interpret(value));
  }

  function toQueryString() {
    return this.inject([], function(results, pair) {
      var key = encodeURIComponent(pair.key), values = pair.value;

      if (values && typeof values == 'object') {
        if (Object.isArray(values)) {
          var queryValues = [];
          for (var i = 0, len = values.length, value; i < len; i++) {
            value = values[i];
            queryValues.push(toQueryPair(key, value));
          }
          return results.concat(queryValues);
        }
      } else results.push(toQueryPair(key, values));
      return results;
    }).join('&');
  }

  function inspect() {
    return '#<Hash:{' + this.map(function(pair) {
      return pair.map(Object.inspect).join(': ');
    }).join(', ') + '}>';
  }

  function clone() {
    return new Hash(this);
  }

  return {
    initialize:             initialize,
    _each:                  _each,
    set:                    set,
    get:                    get,
    unset:                  unset,
    toObject:               toObject,
    toTemplateReplacements: toObject,
    keys:                   keys,
    values:                 values,
    index:                  index,
    merge:                  merge,
    update:                 update,
    toQueryString:          toQueryString,
    inspect:                inspect,
    toJSON:                 toObject,
    clone:                  clone
  };
})());

Hash.from = $H;
Object.extend(Number.prototype, (function() {
  function toColorPart() {
    return this.toPaddedString(2, 16);
  }

  function succ() {
    return this + 1;
  }

  function times(iterator, context) {
    $R(0, this, true).each(iterator, context);
    return this;
  }

  function toPaddedString(length, radix) {
    var string = this.toString(radix || 10);
    return '0'.times(length - string.length) + string;
  }

  function abs() {
    return Math.abs(this);
  }

  function round() {
    return Math.round(this);
  }

  function ceil() {
    return Math.ceil(this);
  }

  function floor() {
    return Math.floor(this);
  }

  return {
    toColorPart:    toColorPart,
    succ:           succ,
    times:          times,
    toPaddedString: toPaddedString,
    abs:            abs,
    round:          round,
    ceil:           ceil,
    floor:          floor
  };
})());

function $R(start, end, exclusive) {
  return new ObjectRange(start, end, exclusive);
}

var ObjectRange = Class.create(Enumerable, (function() {
  function initialize(start, end, exclusive) {
    this.start = start;
    this.end = end;
    this.exclusive = exclusive;
  }

  function _each(iterator) {
    var value = this.start;
    while (this.include(value)) {
      iterator(value);
      value = value.succ();
    }
  }

  function include(value) {
    if (value < this.start)
      return false;
    if (this.exclusive)
      return value < this.end;
    return value <= this.end;
  }

  return {
    initialize: initialize,
    _each:      _each,
    include:    include
  };
})());



var Abstract = { };


var Try = {
  these: function() {
    var returnValue;

    for (var i = 0, length = arguments.length; i < length; i++) {
      var lambda = arguments[i];
      try {
        returnValue = lambda();
        break;
      } catch (e) { }
    }

    return returnValue;
  }
};

var Ajax = {
  getTransport: function() {
    return Try.these(
      function() {return new XMLHttpRequest()},
      function() {return new ActiveXObject('Msxml2.XMLHTTP')},
      function() {return new ActiveXObject('Microsoft.XMLHTTP')}
    ) || false;
  },

  activeRequestCount: 0
};

Ajax.Responders = {
  responders: [],

  _each: function(iterator) {
    this.responders._each(iterator);
  },

  register: function(responder) {
    if (!this.include(responder))
      this.responders.push(responder);
  },

  unregister: function(responder) {
    this.responders = this.responders.without(responder);
  },

  dispatch: function(callback, request, transport, json) {
    this.each(function(responder) {
      if (Object.isFunction(responder[callback])) {
        try {
          responder[callback].apply(responder, [request, transport, json]);
        } catch (e) { }
      }
    });
  }
};

Object.extend(Ajax.Responders, Enumerable);

Ajax.Responders.register({
  onCreate:   function() { Ajax.activeRequestCount++ },
  onComplete: function() { Ajax.activeRequestCount-- }
});
Ajax.Base = Class.create({
  initialize: function(options) {
    this.options = {
      method:       'post',
      asynchronous: true,
      contentType:  'application/x-www-form-urlencoded',
      encoding:     'UTF-8',
      parameters:   '',
      evalJSON:     true,
      evalJS:       true
    };
    Object.extend(this.options, options || { });

    this.options.method = this.options.method.toLowerCase();

    if (Object.isHash(this.options.parameters))
      this.options.parameters = this.options.parameters.toObject();
  }
});
Ajax.Request = Class.create(Ajax.Base, {
  _complete: false,

  initialize: function($super, url, options) {
    $super(options);
    this.transport = Ajax.getTransport();
    this.request(url);
  },

  request: function(url) {
    this.url = url;
    this.method = this.options.method;
    var params = Object.isString(this.options.parameters) ?
          this.options.parameters :
          Object.toQueryString(this.options.parameters);

    if (!['get', 'post'].include(this.method)) {
      params += (params ? '&' : '') + "_method=" + this.method;
      this.method = 'post';
    }

    if (params && this.method === 'get') {
      this.url += (this.url.include('?') ? '&' : '?') + params;
    }

    this.parameters = params.toQueryParams();

    try {
      var response = new Ajax.Response(this);
      if (this.options.onCreate) this.options.onCreate(response);
      Ajax.Responders.dispatch('onCreate', this, response);

      this.transport.open(this.method.toUpperCase(), this.url,
        this.options.asynchronous);

      if (this.options.asynchronous) this.respondToReadyState.bind(this).defer(1);

      this.transport.onreadystatechange = this.onStateChange.bind(this);
      this.setRequestHeaders();

      this.body = this.method == 'post' ? (this.options.postBody || params) : null;
      this.transport.send(this.body);

      /* Force Firefox to handle ready state 4 for synchronous requests */
      if (!this.options.asynchronous && this.transport.overrideMimeType)
        this.onStateChange();

    }
    catch (e) {
      this.dispatchException(e);
    }
  },

  onStateChange: function() {
    var readyState = this.transport.readyState;
    if (readyState > 1 && !((readyState == 4) && this._complete))
      this.respondToReadyState(this.transport.readyState);
  },

  setRequestHeaders: function() {
    var headers = {
      'X-Requested-With': 'XMLHttpRequest',
      'X-Prototype-Version': Prototype.Version,
      'Accept': 'text/javascript, text/html, application/xml, text/xml, */*'
    };

    if (this.method == 'post') {
      headers['Content-type'] = this.options.contentType +
        (this.options.encoding ? '; charset=' + this.options.encoding : '');

      /* Force "Connection: close" for older Mozilla browsers to work
       * around a bug where XMLHttpRequest sends an incorrect
       * Content-length header. See Mozilla Bugzilla #246651.
       */
      if (this.transport.overrideMimeType &&
          (navigator.userAgent.match(/Gecko\/(\d{4})/) || [0,2005])[1] < 2005)
            headers['Connection'] = 'close';
    }

    if (typeof this.options.requestHeaders == 'object') {
      var extras = this.options.requestHeaders;

      if (Object.isFunction(extras.push))
        for (var i = 0, length = extras.length; i < length; i += 2)
          headers[extras[i]] = extras[i+1];
      else
        $H(extras).each(function(pair) { headers[pair.key] = pair.value });
    }

    for (var name in headers)
      this.transport.setRequestHeader(name, headers[name]);
  },

  success: function() {
    var status = this.getStatus();
    return !status || (status >= 200 && status < 300) || status == 304;
  },

  getStatus: function() {
    try {
      if (this.transport.status === 1223) return 204;
      return this.transport.status || 0;
    } catch (e) { return 0 }
  },

  respondToReadyState: function(readyState) {
    var state = Ajax.Request.Events[readyState], response = new Ajax.Response(this);

    if (state == 'Complete') {
      try {
        this._complete = true;
        (this.options['on' + response.status]
         || this.options['on' + (this.success() ? 'Success' : 'Failure')]
         || Prototype.emptyFunction)(response, response.headerJSON);
      } catch (e) {
        this.dispatchException(e);
      }

      var contentType = response.getHeader('Content-type');
      if (this.options.evalJS == 'force'
          || (this.options.evalJS && this.isSameOrigin() && contentType
          && contentType.match(/^\s*(text|application)\/(x-)?(java|ecma)script(;.*)?\s*$/i)))
        this.evalResponse();
    }

    try {
      (this.options['on' + state] || Prototype.emptyFunction)(response, response.headerJSON);
      Ajax.Responders.dispatch('on' + state, this, response, response.headerJSON);
    } catch (e) {
      this.dispatchException(e);
    }

    if (state == 'Complete') {
      this.transport.onreadystatechange = Prototype.emptyFunction;
    }
  },

  isSameOrigin: function() {
    var m = this.url.match(/^\s*https?:\/\/[^\/]*/);
    return !m || (m[0] == '#{protocol}//#{domain}#{port}'.interpolate({
      protocol: location.protocol,
      domain: document.domain,
      port: location.port ? ':' + location.port : ''
    }));
  },

  getHeader: function(name) {
    try {
      return this.transport.getResponseHeader(name) || null;
    } catch (e) { return null; }
  },

  evalResponse: function() {
    try {
      return eval((this.transport.responseText || '').unfilterJSON());
    } catch (e) {
      this.dispatchException(e);
    }
  },

  dispatchException: function(exception) {
    (this.options.onException || Prototype.emptyFunction)(this, exception);
    Ajax.Responders.dispatch('onException', this, exception);
  }
});

Ajax.Request.Events =
  ['Uninitialized', 'Loading', 'Loaded', 'Interactive', 'Complete'];








Ajax.Response = Class.create({
  initialize: function(request){
    this.request = request;
    var transport  = this.transport  = request.transport,
        readyState = this.readyState = transport.readyState;

    if ((readyState > 2 && !Prototype.Browser.IE) || readyState == 4) {
      this.status       = this.getStatus();
      this.statusText   = this.getStatusText();
      this.responseText = String.interpret(transport.responseText);
      this.headerJSON   = this._getHeaderJSON();
    }

    if (readyState == 4) {
      var xml = transport.responseXML;
      this.responseXML  = Object.isUndefined(xml) ? null : xml;
      this.responseJSON = this._getResponseJSON();
    }
  },

  status:      0,

  statusText: '',

  getStatus: Ajax.Request.prototype.getStatus,

  getStatusText: function() {
    try {
      return this.transport.statusText || '';
    } catch (e) { return '' }
  },

  getHeader: Ajax.Request.prototype.getHeader,

  getAllHeaders: function() {
    try {
      return this.getAllResponseHeaders();
    } catch (e) { return null }
  },

  getResponseHeader: function(name) {
    return this.transport.getResponseHeader(name);
  },

  getAllResponseHeaders: function() {
    return this.transport.getAllResponseHeaders();
  },

  _getHeaderJSON: function() {
    var json = this.getHeader('X-JSON');
    if (!json) return null;
    json = decodeURIComponent(escape(json));
    try {
      return json.evalJSON(this.request.options.sanitizeJSON ||
        !this.request.isSameOrigin());
    } catch (e) {
      this.request.dispatchException(e);
    }
  },

  _getResponseJSON: function() {
    var options = this.request.options;
    if (!options.evalJSON || (options.evalJSON != 'force' &&
      !(this.getHeader('Content-type') || '').include('application/json')) ||
        this.responseText.blank())
          return null;
    try {
      return this.responseText.evalJSON(options.sanitizeJSON ||
        !this.request.isSameOrigin());
    } catch (e) {
      this.request.dispatchException(e);
    }
  }
});

Ajax.Updater = Class.create(Ajax.Request, {
  initialize: function($super, container, url, options) {
    this.container = {
      success: (container.success || container),
      failure: (container.failure || (container.success ? null : container))
    };

    options = Object.clone(options);
    var onComplete = options.onComplete;
    options.onComplete = (function(response, json) {
      this.updateContent(response.responseText);
      if (Object.isFunction(onComplete)) onComplete(response, json);
    }).bind(this);

    $super(url, options);
  },

  updateContent: function(responseText) {
    var receiver = this.container[this.success() ? 'success' : 'failure'],
        options = this.options;

    if (!options.evalScripts) responseText = responseText.stripScripts();

    if (receiver = $(receiver)) {
      if (options.insertion) {
        if (Object.isString(options.insertion)) {
          var insertion = { }; insertion[options.insertion] = responseText;
          receiver.insert(insertion);
        }
        else options.insertion(receiver, responseText);
      }
      else receiver.update(responseText);
    }
  }
});

Ajax.PeriodicalUpdater = Class.create(Ajax.Base, {
  initialize: function($super, container, url, options) {
    $super(options);
    this.onComplete = this.options.onComplete;

    this.frequency = (this.options.frequency || 2);
    this.decay = (this.options.decay || 1);

    this.updater = { };
    this.container = container;
    this.url = url;

    this.start();
  },

  start: function() {
    this.options.onComplete = this.updateComplete.bind(this);
    this.onTimerEvent();
  },

  stop: function() {
    this.updater.options.onComplete = undefined;
    clearTimeout(this.timer);
    (this.onComplete || Prototype.emptyFunction).apply(this, arguments);
  },

  updateComplete: function(response) {
    if (this.options.decay) {
      this.decay = (response.responseText == this.lastText ?
        this.decay * this.options.decay : 1);

      this.lastText = response.responseText;
    }
    this.timer = this.onTimerEvent.bind(this).delay(this.decay * this.frequency);
  },

  onTimerEvent: function() {
    this.updater = new Ajax.Updater(this.container, this.url, this.options);
  }
});


function $(element) {
  if (arguments.length > 1) {
    for (var i = 0, elements = [], length = arguments.length; i < length; i++)
      elements.push($(arguments[i]));
    return elements;
  }
  if (Object.isString(element))
    element = document.getElementById(element);
  return Element.extend(element);
}

if (Prototype.BrowserFeatures.XPath) {
  document._getElementsByXPath = function(expression, parentElement) {
    var results = [];
    var query = document.evaluate(expression, $(parentElement) || document,
      null, XPathResult.ORDERED_NODE_SNAPSHOT_TYPE, null);
    for (var i = 0, length = query.snapshotLength; i < length; i++)
      results.push(Element.extend(query.snapshotItem(i)));
    return results;
  };
}

/*--------------------------------------------------------------------------*/

if (!Node) var Node = { };

if (!Node.ELEMENT_NODE) {
  Object.extend(Node, {
    ELEMENT_NODE: 1,
    ATTRIBUTE_NODE: 2,
    TEXT_NODE: 3,
    CDATA_SECTION_NODE: 4,
    ENTITY_REFERENCE_NODE: 5,
    ENTITY_NODE: 6,
    PROCESSING_INSTRUCTION_NODE: 7,
    COMMENT_NODE: 8,
    DOCUMENT_NODE: 9,
    DOCUMENT_TYPE_NODE: 10,
    DOCUMENT_FRAGMENT_NODE: 11,
    NOTATION_NODE: 12
  });
}



(function(global) {
  function shouldUseCache(tagName, attributes) {
    if (tagName === 'select') return false;
    if ('type' in attributes) return false;
    return true;
  }

  var HAS_EXTENDED_CREATE_ELEMENT_SYNTAX = (function(){
    try {
      var el = document.createElement('<input name="x">');
      return el.tagName.toLowerCase() === 'input' && el.name === 'x';
    }
    catch(err) {
      return false;
    }
  })();

  var element = global.Element;

  global.Element = function(tagName, attributes) {
    attributes = attributes || { };
    tagName = tagName.toLowerCase();
    var cache = Element.cache;

    if (HAS_EXTENDED_CREATE_ELEMENT_SYNTAX && attributes.name) {
      tagName = '<' + tagName + ' name="' + attributes.name + '">';
      delete attributes.name;
      return Element.writeAttribute(document.createElement(tagName), attributes);
    }

    if (!cache[tagName]) cache[tagName] = Element.extend(document.createElement(tagName));

    var node = shouldUseCache(tagName, attributes) ?
     cache[tagName].cloneNode(false) : document.createElement(tagName);

    return Element.writeAttribute(node, attributes);
  };

  Object.extend(global.Element, element || { });
  if (element) global.Element.prototype = element.prototype;

})(this);

Element.idCounter = 1;
Element.cache = { };

Element._purgeElement = function(element) {
  var uid = element._prototypeUID;
  if (uid) {
    Element.stopObserving(element);
    element._prototypeUID = void 0;
    delete Element.Storage[uid];
  }
}

Element.Methods = {
  visible: function(element) {
    return $(element).style.display != 'none';
  },

  toggle: function(element) {
    element = $(element);
    Element[Element.visible(element) ? 'hide' : 'show'](element);
    return element;
  },

  hide: function(element) {
    element = $(element);
    element.style.display = 'none';
    return element;
  },

  show: function(element) {
    element = $(element);
    element.style.display = '';
    return element;
  },

  remove: function(element) {
    element = $(element);
    element.parentNode.removeChild(element);
    return element;
  },

  update: (function(){

    var SELECT_ELEMENT_INNERHTML_BUGGY = (function(){
      var el = document.createElement("select"),
          isBuggy = true;
      el.innerHTML = "<option value=\"test\">test</option>";
      if (el.options && el.options[0]) {
        isBuggy = el.options[0].nodeName.toUpperCase() !== "OPTION";
      }
      el = null;
      return isBuggy;
    })();

    var TABLE_ELEMENT_INNERHTML_BUGGY = (function(){
      try {
        var el = document.createElement("table");
        if (el && el.tBodies) {
          el.innerHTML = "<tbody><tr><td>test</td></tr></tbody>";
          var isBuggy = typeof el.tBodies[0] == "undefined";
          el = null;
          return isBuggy;
        }
      } catch (e) {
        return true;
      }
    })();

    var LINK_ELEMENT_INNERHTML_BUGGY = (function() {
      try {
        var el = document.createElement('div');
        el.innerHTML = "<link>";
        var isBuggy = (el.childNodes.length === 0);
        el = null;
        return isBuggy;
      } catch(e) {
        return true;
      }
    })();

    var ANY_INNERHTML_BUGGY = SELECT_ELEMENT_INNERHTML_BUGGY ||
     TABLE_ELEMENT_INNERHTML_BUGGY || LINK_ELEMENT_INNERHTML_BUGGY;

    var SCRIPT_ELEMENT_REJECTS_TEXTNODE_APPENDING = (function () {
      var s = document.createElement("script"),
          isBuggy = false;
      try {
        s.appendChild(document.createTextNode(""));
        isBuggy = !s.firstChild ||
          s.firstChild && s.firstChild.nodeType !== 3;
      } catch (e) {
        isBuggy = true;
      }
      s = null;
      return isBuggy;
    })();


    function update(element, content) {
      element = $(element);
      var purgeElement = Element._purgeElement;

      var descendants = element.getElementsByTagName('*'),
       i = descendants.length;
      while (i--) purgeElement(descendants[i]);

      if (content && content.toElement)
        content = content.toElement();

      if (Object.isElement(content))
        return element.update().insert(content);

      content = Object.toHTML(content);

      var tagName = element.tagName.toUpperCase();

      if (tagName === 'SCRIPT' && SCRIPT_ELEMENT_REJECTS_TEXTNODE_APPENDING) {
        element.text = content;
        return element;
      }

      if (ANY_INNERHTML_BUGGY) {
        if (tagName in Element._insertionTranslations.tags) {
          while (element.firstChild) {
            element.removeChild(element.firstChild);
          }
          Element._getContentFromAnonymousElement(tagName, content.stripScripts())
            .each(function(node) {
              element.appendChild(node)
            });
        } else if (LINK_ELEMENT_INNERHTML_BUGGY && Object.isString(content) && content.indexOf('<link') > -1) {
          while (element.firstChild) {
            element.removeChild(element.firstChild);
          }
          var nodes = Element._getContentFromAnonymousElement(tagName, content.stripScripts(), true);
          nodes.each(function(node) { element.appendChild(node) });
        }
        else {
          element.innerHTML = content.stripScripts();
        }
      }
      else {
        element.innerHTML = content.stripScripts();
      }

      content.evalScripts.bind(content).defer();
      return element;
    }

    return update;
  })(),

  replace: function(element, content) {
    element = $(element);
    if (content && content.toElement) content = content.toElement();
    else if (!Object.isElement(content)) {
      content = Object.toHTML(content);
      var range = element.ownerDocument.createRange();
      range.selectNode(element);
      content.evalScripts.bind(content).defer();
      content = range.createContextualFragment(content.stripScripts());
    }
    element.parentNode.replaceChild(content, element);
    return element;
  },

  insert: function(element, insertions) {
    element = $(element);

    if (Object.isString(insertions) || Object.isNumber(insertions) ||
        Object.isElement(insertions) || (insertions && (insertions.toElement || insertions.toHTML)))
          insertions = {bottom:insertions};

    var content, insert, tagName, childNodes;

    for (var position in insertions) {
      content  = insertions[position];
      position = position.toLowerCase();
      insert = Element._insertionTranslations[position];

      if (content && content.toElement) content = content.toElement();
      if (Object.isElement(content)) {
        insert(element, content);
        continue;
      }

      content = Object.toHTML(content);

      tagName = ((position == 'before' || position == 'after')
        ? element.parentNode : element).tagName.toUpperCase();

      childNodes = Element._getContentFromAnonymousElement(tagName, content.stripScripts());

      if (position == 'top' || position == 'after') childNodes.reverse();
      childNodes.each(insert.curry(element));

      content.evalScripts.bind(content).defer();
    }

    return element;
  },

  wrap: function(element, wrapper, attributes) {
    element = $(element);
    if (Object.isElement(wrapper))
      $(wrapper).writeAttribute(attributes || { });
    else if (Object.isString(wrapper)) wrapper = new Element(wrapper, attributes);
    else wrapper = new Element('div', wrapper);
    if (element.parentNode)
      element.parentNode.replaceChild(wrapper, element);
    wrapper.appendChild(element);
    return wrapper;
  },

  inspect: function(element) {
    element = $(element);
    var result = '<' + element.tagName.toLowerCase();
    $H({'id': 'id', 'className': 'class'}).each(function(pair) {
      var property = pair.first(),
          attribute = pair.last(),
          value = (element[property] || '').toString();
      if (value) result += ' ' + attribute + '=' + value.inspect(true);
    });
    return result + '>';
  },

  recursivelyCollect: function(element, property, maximumLength) {
    element = $(element);
    maximumLength = maximumLength || -1;
    var elements = [];

    while (element = element[property]) {
      if (element.nodeType == 1)
        elements.push(Element.extend(element));
      if (elements.length == maximumLength)
        break;
    }

    return elements;
  },

  ancestors: function(element) {
    return Element.recursivelyCollect(element, 'parentNode');
  },

  descendants: function(element) {
    return Element.select(element, "*");
  },

  firstDescendant: function(element) {
    element = $(element).firstChild;
    while (element && element.nodeType != 1) element = element.nextSibling;
    return $(element);
  },

  immediateDescendants: function(element) {
    var results = [], child = $(element).firstChild;
    while (child) {
      if (child.nodeType === 1) {
        results.push(Element.extend(child));
      }
      child = child.nextSibling;
    }
    return results;
  },

  previousSiblings: function(element, maximumLength) {
    return Element.recursivelyCollect(element, 'previousSibling');
  },

  nextSiblings: function(element) {
    return Element.recursivelyCollect(element, 'nextSibling');
  },

  siblings: function(element) {
    element = $(element);
    return Element.previousSiblings(element).reverse()
      .concat(Element.nextSiblings(element));
  },

  match: function(element, selector) {
    element = $(element);
    if (Object.isString(selector))
      return Prototype.Selector.match(element, selector);
    return selector.match(element);
  },

  up: function(element, expression, index) {
    element = $(element);
    if (arguments.length == 1) return $(element.parentNode);
    var ancestors = Element.ancestors(element);
    return Object.isNumber(expression) ? ancestors[expression] :
      Prototype.Selector.find(ancestors, expression, index);
  },

  down: function(element, expression, index) {
    element = $(element);
    if (arguments.length == 1) return Element.firstDescendant(element);
    return Object.isNumber(expression) ? Element.descendants(element)[expression] :
      Element.select(element, expression)[index || 0];
  },

  previous: function(element, expression, index) {
    element = $(element);
    if (Object.isNumber(expression)) index = expression, expression = false;
    if (!Object.isNumber(index)) index = 0;

    if (expression) {
      return Prototype.Selector.find(element.previousSiblings(), expression, index);
    } else {
      return element.recursivelyCollect("previousSibling", index + 1)[index];
    }
  },

  next: function(element, expression, index) {
    element = $(element);
    if (Object.isNumber(expression)) index = expression, expression = false;
    if (!Object.isNumber(index)) index = 0;

    if (expression) {
      return Prototype.Selector.find(element.nextSiblings(), expression, index);
    } else {
      var maximumLength = Object.isNumber(index) ? index + 1 : 1;
      return element.recursivelyCollect("nextSibling", index + 1)[index];
    }
  },


  select: function(element) {
    element = $(element);
    var expressions = Array.prototype.slice.call(arguments, 1).join(', ');
    return Prototype.Selector.select(expressions, element);
  },

  adjacent: function(element) {
    element = $(element);
    var expressions = Array.prototype.slice.call(arguments, 1).join(', ');
    return Prototype.Selector.select(expressions, element.parentNode).without(element);
  },

  identify: function(element) {
    element = $(element);
    var id = Element.readAttribute(element, 'id');
    if (id) return id;
    do { id = 'anonymous_element_' + Element.idCounter++ } while ($(id));
    Element.writeAttribute(element, 'id', id);
    return id;
  },

  readAttribute: function(element, name) {
    element = $(element);
    if (Prototype.Browser.IE) {
      var t = Element._attributeTranslations.read;
      if (t.values[name]) return t.values[name](element, name);
      if (t.names[name]) name = t.names[name];
      if (name.include(':')) {
        return (!element.attributes || !element.attributes[name]) ? null :
         element.attributes[name].value;
      }
    }
    return element.getAttribute(name);
  },

  writeAttribute: function(element, name, value) {
    element = $(element);
    var attributes = { }, t = Element._attributeTranslations.write;

    if (typeof name == 'object') attributes = name;
    else attributes[name] = Object.isUndefined(value) ? true : value;

    for (var attr in attributes) {
      name = t.names[attr] || attr;
      value = attributes[attr];
      if (t.values[attr]) name = t.values[attr](element, value);
      if (value === false || value === null)
        element.removeAttribute(name);
      else if (value === true)
        element.setAttribute(name, name);
      else element.setAttribute(name, value);
    }
    return element;
  },

  getHeight: function(element) {
    return Element.getDimensions(element).height;
  },

  getWidth: function(element) {
    return Element.getDimensions(element).width;
  },

  classNames: function(element) {
    return new Element.ClassNames(element);
  },

  hasClassName: function(element, className) {
    if (!(element = $(element))) return;
    var elementClassName = element.className;
    return (elementClassName.length > 0 && (elementClassName == className ||
      new RegExp("(^|\\s)" + className + "(\\s|$)").test(elementClassName)));
  },

  addClassName: function(element, className) {
    if (!(element = $(element))) return;
    if (!Element.hasClassName(element, className))
      element.className += (element.className ? ' ' : '') + className;
    return element;
  },

  removeClassName: function(element, className) {
    if (!(element = $(element))) return;
    element.className = element.className.replace(
      new RegExp("(^|\\s+)" + className + "(\\s+|$)"), ' ').strip();
    return element;
  },

  toggleClassName: function(element, className) {
    if (!(element = $(element))) return;
    return Element[Element.hasClassName(element, className) ?
      'removeClassName' : 'addClassName'](element, className);
  },

  cleanWhitespace: function(element) {
    element = $(element);
    var node = element.firstChild;
    while (node) {
      var nextNode = node.nextSibling;
      if (node.nodeType == 3 && !/\S/.test(node.nodeValue))
        element.removeChild(node);
      node = nextNode;
    }
    return element;
  },

  empty: function(element) {
    return $(element).innerHTML.blank();
  },

  descendantOf: function(element, ancestor) {
    element = $(element), ancestor = $(ancestor);

    if (element.compareDocumentPosition)
      return (element.compareDocumentPosition(ancestor) & 8) === 8;

    if (ancestor.contains)
      return ancestor.contains(element) && ancestor !== element;

    while (element = element.parentNode)
      if (element == ancestor) return true;

    return false;
  },

  scrollTo: function(element) {
    element = $(element);
    var pos = Element.cumulativeOffset(element);
    window.scrollTo(pos[0], pos[1]);
    return element;
  },

  getStyle: function(element, style) {
    element = $(element);
    style = style == 'float' ? 'cssFloat' : style.camelize();
    var value = element.style[style];
    if (!value || value == 'auto') {
      var css = document.defaultView.getComputedStyle(element, null);
      value = css ? css[style] : null;
    }
    if (style == 'opacity') return value ? parseFloat(value) : 1.0;
    return value == 'auto' ? null : value;
  },

  getOpacity: function(element) {
    return $(element).getStyle('opacity');
  },

  setStyle: function(element, styles) {
    element = $(element);
    var elementStyle = element.style, match;
    if (Object.isString(styles)) {
      element.style.cssText += ';' + styles;
      return styles.include('opacity') ?
        element.setOpacity(styles.match(/opacity:\s*(\d?\.?\d*)/)[1]) : element;
    }
    for (var property in styles)
      if (property == 'opacity') element.setOpacity(styles[property]);
      else
        elementStyle[(property == 'float' || property == 'cssFloat') ?
          (Object.isUndefined(elementStyle.styleFloat) ? 'cssFloat' : 'styleFloat') :
            property] = styles[property];

    return element;
  },

  setOpacity: function(element, value) {
    element = $(element);
    element.style.opacity = (value == 1 || value === '') ? '' :
      (value < 0.00001) ? 0 : value;
    return element;
  },

  makePositioned: function(element) {
    element = $(element);
    var pos = Element.getStyle(element, 'position');
    if (pos == 'static' || !pos) {
      element._madePositioned = true;
      element.style.position = 'relative';
      if (Prototype.Browser.Opera) {
        element.style.top = 0;
        element.style.left = 0;
      }
    }
    return element;
  },

  undoPositioned: function(element) {
    element = $(element);
    if (element._madePositioned) {
      element._madePositioned = undefined;
      element.style.position =
        element.style.top =
        element.style.left =
        element.style.bottom =
        element.style.right = '';
    }
    return element;
  },

  makeClipping: function(element) {
    element = $(element);
    if (element._overflow) return element;
    element._overflow = Element.getStyle(element, 'overflow') || 'auto';
    if (element._overflow !== 'hidden')
      element.style.overflow = 'hidden';
    return element;
  },

  undoClipping: function(element) {
    element = $(element);
    if (!element._overflow) return element;
    element.style.overflow = element._overflow == 'auto' ? '' : element._overflow;
    element._overflow = null;
    return element;
  },

  clonePosition: function(element, source) {
    var options = Object.extend({
      setLeft:    true,
      setTop:     true,
      setWidth:   true,
      setHeight:  true,
      offsetTop:  0,
      offsetLeft: 0
    }, arguments[2] || { });

    source = $(source);
    var p = Element.viewportOffset(source), delta = [0, 0], parent = null;

    element = $(element);

    if (Element.getStyle(element, 'position') == 'absolute') {
      parent = Element.getOffsetParent(element);
      delta = Element.viewportOffset(parent);
    }

    if (parent == document.body) {
      delta[0] -= document.body.offsetLeft;
      delta[1] -= document.body.offsetTop;
    }

    if (options.setLeft)   element.style.left  = (p[0] - delta[0] + options.offsetLeft) + 'px';
    if (options.setTop)    element.style.top   = (p[1] - delta[1] + options.offsetTop) + 'px';
    if (options.setWidth)  element.style.width = source.offsetWidth + 'px';
    if (options.setHeight) element.style.height = source.offsetHeight + 'px';
    return element;
  }
};

Object.extend(Element.Methods, {
  getElementsBySelector: Element.Methods.select,

  childElements: Element.Methods.immediateDescendants
});

Element._attributeTranslations = {
  write: {
    names: {
      className: 'class',
      htmlFor:   'for'
    },
    values: { }
  }
};

if (Prototype.Browser.Opera) {
  Element.Methods.getStyle = Element.Methods.getStyle.wrap(
    function(proceed, element, style) {
      switch (style) {
        case 'height': case 'width':
          if (!Element.visible(element)) return null;

          var dim = parseInt(proceed(element, style), 10);

          if (dim !== element['offset' + style.capitalize()])
            return dim + 'px';

          var properties;
          if (style === 'height') {
            properties = ['border-top-width', 'padding-top',
             'padding-bottom', 'border-bottom-width'];
          }
          else {
            properties = ['border-left-width', 'padding-left',
             'padding-right', 'border-right-width'];
          }
          return properties.inject(dim, function(memo, property) {
            var val = proceed(element, property);
            return val === null ? memo : memo - parseInt(val, 10);
          }) + 'px';
        default: return proceed(element, style);
      }
    }
  );

  Element.Methods.readAttribute = Element.Methods.readAttribute.wrap(
    function(proceed, element, attribute) {
      if (attribute === 'title') return element.title;
      return proceed(element, attribute);
    }
  );
}

else if (Prototype.Browser.IE) {
  Element.Methods.getStyle = function(element, style) {
    element = $(element);
    style = (style == 'float' || style == 'cssFloat') ? 'styleFloat' : style.camelize();
    var value = element.style[style];
    if (!value && element.currentStyle) value = element.currentStyle[style];

    if (style == 'opacity') {
      if (value = (element.getStyle('filter') || '').match(/alpha\(opacity=(.*)\)/))
        if (value[1]) return parseFloat(value[1]) / 100;
      return 1.0;
    }

    if (value == 'auto') {
      if ((style == 'width' || style == 'height') && (element.getStyle('display') != 'none'))
        return element['offset' + style.capitalize()] + 'px';
      return null;
    }
    return value;
  };

  Element.Methods.setOpacity = function(element, value) {
    function stripAlpha(filter){
      return filter.replace(/alpha\([^\)]*\)/gi,'');
    }
    element = $(element);
    var currentStyle = element.currentStyle;
    if ((currentStyle && !currentStyle.hasLayout) ||
      (!currentStyle && element.style.zoom == 'normal'))
        element.style.zoom = 1;

    var filter = element.getStyle('filter'), style = element.style;
    if (value == 1 || value === '') {
      (filter = stripAlpha(filter)) ?
        style.filter = filter : style.removeAttribute('filter');
      return element;
    } else if (value < 0.00001) value = 0;
    style.filter = stripAlpha(filter) +
      'alpha(opacity=' + (value * 100) + ')';
    return element;
  };

  Element._attributeTranslations = (function(){

    var classProp = 'className',
        forProp = 'for',
        el = document.createElement('div');

    el.setAttribute(classProp, 'x');

    if (el.className !== 'x') {
      el.setAttribute('class', 'x');
      if (el.className === 'x') {
        classProp = 'class';
      }
    }
    el = null;

    el = document.createElement('label');
    el.setAttribute(forProp, 'x');
    if (el.htmlFor !== 'x') {
      el.setAttribute('htmlFor', 'x');
      if (el.htmlFor === 'x') {
        forProp = 'htmlFor';
      }
    }
    el = null;

    return {
      read: {
        names: {
          'class':      classProp,
          'className':  classProp,
          'for':        forProp,
          'htmlFor':    forProp
        },
        values: {
          _getAttr: function(element, attribute) {
            return element.getAttribute(attribute);
          },
          _getAttr2: function(element, attribute) {
            return element.getAttribute(attribute, 2);
          },
          _getAttrNode: function(element, attribute) {
            var node = element.getAttributeNode(attribute);
            return node ? node.value : "";
          },
          _getEv: (function(){

            var el = document.createElement('div'), f;
            el.onclick = Prototype.emptyFunction;
            var value = el.getAttribute('onclick');

            if (String(value).indexOf('{') > -1) {
              f = function(element, attribute) {
                attribute = element.getAttribute(attribute);
                if (!attribute) return null;
                attribute = attribute.toString();
                attribute = attribute.split('{')[1];
                attribute = attribute.split('}')[0];
                return attribute.strip();
              };
            }
            else if (value === '') {
              f = function(element, attribute) {
                attribute = element.getAttribute(attribute);
                if (!attribute) return null;
                return attribute.strip();
              };
            }
            el = null;
            return f;
          })(),
          _flag: function(element, attribute) {
            return $(element).hasAttribute(attribute) ? attribute : null;
          },
          style: function(element) {
            return element.style.cssText.toLowerCase();
          },
          title: function(element) {
            return element.title;
          }
        }
      }
    }
  })();

  Element._attributeTranslations.write = {
    names: Object.extend({
      cellpadding: 'cellPadding',
      cellspacing: 'cellSpacing'
    }, Element._attributeTranslations.read.names),
    values: {
      checked: function(element, value) {
        element.checked = !!value;
      },

      style: function(element, value) {
        element.style.cssText = value ? value : '';
      }
    }
  };

  Element._attributeTranslations.has = {};

  $w('colSpan rowSpan vAlign dateTime accessKey tabIndex ' +
      'encType maxLength readOnly longDesc frameBorder').each(function(attr) {
    Element._attributeTranslations.write.names[attr.toLowerCase()] = attr;
    Element._attributeTranslations.has[attr.toLowerCase()] = attr;
  });

  (function(v) {
    Object.extend(v, {
      href:        v._getAttr2,
      src:         v._getAttr2,
      type:        v._getAttr,
      action:      v._getAttrNode,
      disabled:    v._flag,
      checked:     v._flag,
      readonly:    v._flag,
      multiple:    v._flag,
      onload:      v._getEv,
      onunload:    v._getEv,
      onclick:     v._getEv,
      ondblclick:  v._getEv,
      onmousedown: v._getEv,
      onmouseup:   v._getEv,
      onmouseover: v._getEv,
      onmousemove: v._getEv,
      onmouseout:  v._getEv,
      onfocus:     v._getEv,
      onblur:      v._getEv,
      onkeypress:  v._getEv,
      onkeydown:   v._getEv,
      onkeyup:     v._getEv,
      onsubmit:    v._getEv,
      onreset:     v._getEv,
      onselect:    v._getEv,
      onchange:    v._getEv
    });
  })(Element._attributeTranslations.read.values);

  if (Prototype.BrowserFeatures.ElementExtensions) {
    (function() {
      function _descendants(element) {
        var nodes = element.getElementsByTagName('*'), results = [];
        for (var i = 0, node; node = nodes[i]; i++)
          if (node.tagName !== "!") // Filter out comment nodes.
            results.push(node);
        return results;
      }

      Element.Methods.down = function(element, expression, index) {
        element = $(element);
        if (arguments.length == 1) return element.firstDescendant();
        return Object.isNumber(expression) ? _descendants(element)[expression] :
          Element.select(element, expression)[index || 0];
      }
    })();
  }

}

else if (Prototype.Browser.Gecko && /rv:1\.8\.0/.test(navigator.userAgent)) {
  Element.Methods.setOpacity = function(element, value) {
    element = $(element);
    element.style.opacity = (value == 1) ? 0.999999 :
      (value === '') ? '' : (value < 0.00001) ? 0 : value;
    return element;
  };
}

else if (Prototype.Browser.WebKit) {
  Element.Methods.setOpacity = function(element, value) {
    element = $(element);
    element.style.opacity = (value == 1 || value === '') ? '' :
      (value < 0.00001) ? 0 : value;

    if (value == 1)
      if (element.tagName.toUpperCase() == 'IMG' && element.width) {
        element.width++; element.width--;
      } else try {
        var n = document.createTextNode(' ');
        element.appendChild(n);
        element.removeChild(n);
      } catch (e) { }

    return element;
  };
}

if ('outerHTML' in document.documentElement) {
  Element.Methods.replace = function(element, content) {
    element = $(element);

    if (content && content.toElement) content = content.toElement();
    if (Object.isElement(content)) {
      element.parentNode.replaceChild(content, element);
      return element;
    }

    content = Object.toHTML(content);
    var parent = element.parentNode, tagName = parent.tagName.toUpperCase();

    if (Element._insertionTranslations.tags[tagName]) {
      var nextSibling = element.next(),
          fragments = Element._getContentFromAnonymousElement(tagName, content.stripScripts());
      parent.removeChild(element);
      if (nextSibling)
        fragments.each(function(node) { parent.insertBefore(node, nextSibling) });
      else
        fragments.each(function(node) { parent.appendChild(node) });
    }
    else element.outerHTML = content.stripScripts();

    content.evalScripts.bind(content).defer();
    return element;
  };
}

Element._returnOffset = function(l, t) {
  var result = [l, t];
  result.left = l;
  result.top = t;
  return result;
};

Element._getContentFromAnonymousElement = function(tagName, html, force) {
  var div = new Element('div'),
      t = Element._insertionTranslations.tags[tagName];

  var workaround = false;
  if (t) workaround = true;
  else if (force) {
    workaround = true;
    t = ['', '', 0];
  }

  if (workaround) {
    div.innerHTML = '&nbsp;' + t[0] + html + t[1];
    div.removeChild(div.firstChild);
    for (var i = t[2]; i--; ) {
      div = div.firstChild;
    }
  }
  else {
    div.innerHTML = html;
  }
  return $A(div.childNodes);
};

Element._insertionTranslations = {
  before: function(element, node) {
    element.parentNode.insertBefore(node, element);
  },
  top: function(element, node) {
    element.insertBefore(node, element.firstChild);
  },
  bottom: function(element, node) {
    element.appendChild(node);
  },
  after: function(element, node) {
    element.parentNode.insertBefore(node, element.nextSibling);
  },
  tags: {
    TABLE:  ['<table>',                '</table>',                   1],
    TBODY:  ['<table><tbody>',         '</tbody></table>',           2],
    TR:     ['<table><tbody><tr>',     '</tr></tbody></table>',      3],
    TD:     ['<table><tbody><tr><td>', '</td></tr></tbody></table>', 4],
    SELECT: ['<select>',               '</select>',                  1]
  }
};

(function() {
  var tags = Element._insertionTranslations.tags;
  Object.extend(tags, {
    THEAD: tags.TBODY,
    TFOOT: tags.TBODY,
    TH:    tags.TD
  });
})();

Element.Methods.Simulated = {
  hasAttribute: function(element, attribute) {
    attribute = Element._attributeTranslations.has[attribute] || attribute;
    var node = $(element).getAttributeNode(attribute);
    return !!(node && node.specified);
  }
};

Element.Methods.ByTag = { };

Object.extend(Element, Element.Methods);

(function(div) {

  if (!Prototype.BrowserFeatures.ElementExtensions && div['__proto__']) {
    window.HTMLElement = { };
    window.HTMLElement.prototype = div['__proto__'];
    Prototype.BrowserFeatures.ElementExtensions = true;
  }

  div = null;

})(document.createElement('div'));

Element.extend = (function() {

  function checkDeficiency(tagName) {
    if (typeof window.Element != 'undefined') {
      var proto = window.Element.prototype;
      if (proto) {
        var id = '_' + (Math.random()+'').slice(2),
            el = document.createElement(tagName);
        proto[id] = 'x';
        var isBuggy = (el[id] !== 'x');
        delete proto[id];
        el = null;
        return isBuggy;
      }
    }
    return false;
  }

  function extendElementWith(element, methods) {
    for (var property in methods) {
      var value = methods[property];
      if (Object.isFunction(value) && !(property in element))
        element[property] = value.methodize();
    }
  }

  var HTMLOBJECTELEMENT_PROTOTYPE_BUGGY = checkDeficiency('object');

  if (Prototype.BrowserFeatures.SpecificElementExtensions) {
    if (HTMLOBJECTELEMENT_PROTOTYPE_BUGGY) {
      return function(element) {
        if (element && typeof element._extendedByPrototype == 'undefined') {
          var t = element.tagName;
          if (t && (/^(?:object|applet|embed)$/i.test(t))) {
            extendElementWith(element, Element.Methods);
            extendElementWith(element, Element.Methods.Simulated);
            extendElementWith(element, Element.Methods.ByTag[t.toUpperCase()]);
          }
        }
        return element;
      }
    }
    return Prototype.K;
  }

  var Methods = { }, ByTag = Element.Methods.ByTag;

  var extend = Object.extend(function(element) {
    if (!element || typeof element._extendedByPrototype != 'undefined' ||
        element.nodeType != 1 || element == window) return element;

    var methods = Object.clone(Methods),
        tagName = element.tagName.toUpperCase();

    if (ByTag[tagName]) Object.extend(methods, ByTag[tagName]);

    extendElementWith(element, methods);

    element._extendedByPrototype = Prototype.emptyFunction;
    return element;

  }, {
    refresh: function() {
      if (!Prototype.BrowserFeatures.ElementExtensions) {
        Object.extend(Methods, Element.Methods);
        Object.extend(Methods, Element.Methods.Simulated);
      }
    }
  });

  extend.refresh();
  return extend;
})();

if (document.documentElement.hasAttribute) {
  Element.hasAttribute = function(element, attribute) {
    return element.hasAttribute(attribute);
  };
}
else {
  Element.hasAttribute = Element.Methods.Simulated.hasAttribute;
}

Element.addMethods = function(methods) {
  var F = Prototype.BrowserFeatures, T = Element.Methods.ByTag;

  if (!methods) {
    Object.extend(Form, Form.Methods);
    Object.extend(Form.Element, Form.Element.Methods);
    Object.extend(Element.Methods.ByTag, {
      "FORM":     Object.clone(Form.Methods),
      "INPUT":    Object.clone(Form.Element.Methods),
      "SELECT":   Object.clone(Form.Element.Methods),
      "TEXTAREA": Object.clone(Form.Element.Methods),
      "BUTTON":   Object.clone(Form.Element.Methods)
    });
  }

  if (arguments.length == 2) {
    var tagName = methods;
    methods = arguments[1];
  }

  if (!tagName) Object.extend(Element.Methods, methods || { });
  else {
    if (Object.isArray(tagName)) tagName.each(extend);
    else extend(tagName);
  }

  function extend(tagName) {
    tagName = tagName.toUpperCase();
    if (!Element.Methods.ByTag[tagName])
      Element.Methods.ByTag[tagName] = { };
    Object.extend(Element.Methods.ByTag[tagName], methods);
  }

  function copy(methods, destination, onlyIfAbsent) {
    onlyIfAbsent = onlyIfAbsent || false;
    for (var property in methods) {
      var value = methods[property];
      if (!Object.isFunction(value)) continue;
      if (!onlyIfAbsent || !(property in destination))
        destination[property] = value.methodize();
    }
  }

  function findDOMClass(tagName) {
    var klass;
    var trans = {
      "OPTGROUP": "OptGroup", "TEXTAREA": "TextArea", "P": "Paragraph",
      "FIELDSET": "FieldSet", "UL": "UList", "OL": "OList", "DL": "DList",
      "DIR": "Directory", "H1": "Heading", "H2": "Heading", "H3": "Heading",
      "H4": "Heading", "H5": "Heading", "H6": "Heading", "Q": "Quote",
      "INS": "Mod", "DEL": "Mod", "A": "Anchor", "IMG": "Image", "CAPTION":
      "TableCaption", "COL": "TableCol", "COLGROUP": "TableCol", "THEAD":
      "TableSection", "TFOOT": "TableSection", "TBODY": "TableSection", "TR":
      "TableRow", "TH": "TableCell", "TD": "TableCell", "FRAMESET":
      "FrameSet", "IFRAME": "IFrame"
    };
    if (trans[tagName]) klass = 'HTML' + trans[tagName] + 'Element';
    if (window[klass]) return window[klass];
    klass = 'HTML' + tagName + 'Element';
    if (window[klass]) return window[klass];
    klass = 'HTML' + tagName.capitalize() + 'Element';
    if (window[klass]) return window[klass];

    var element = document.createElement(tagName),
        proto = element['__proto__'] || element.constructor.prototype;

    element = null;
    return proto;
  }

  var elementPrototype = window.HTMLElement ? HTMLElement.prototype :
   Element.prototype;

  if (F.ElementExtensions) {
    copy(Element.Methods, elementPrototype);
    copy(Element.Methods.Simulated, elementPrototype, true);
  }

  if (F.SpecificElementExtensions) {
    for (var tag in Element.Methods.ByTag) {
      var klass = findDOMClass(tag);
      if (Object.isUndefined(klass)) continue;
      copy(T[tag], klass.prototype);
    }
  }

  Object.extend(Element, Element.Methods);
  delete Element.ByTag;

  if (Element.extend.refresh) Element.extend.refresh();
  Element.cache = { };
};


document.viewport = {

  getDimensions: function() {
    return { width: this.getWidth(), height: this.getHeight() };
  },

  getScrollOffsets: function() {
    return Element._returnOffset(
      window.pageXOffset || document.documentElement.scrollLeft || document.body.scrollLeft,
      window.pageYOffset || document.documentElement.scrollTop  || document.body.scrollTop);
  }
};

(function(viewport) {
  var B = Prototype.Browser, doc = document, element, property = {};

  function getRootElement() {
    if (B.WebKit && !doc.evaluate)
      return document;

    if (B.Opera && window.parseFloat(window.opera.version()) < 9.5)
      return document.body;

    return document.documentElement;
  }

  function define(D) {
    if (!element) element = getRootElement();

    property[D] = 'client' + D;

    viewport['get' + D] = function() { return element[property[D]] };
    return viewport['get' + D]();
  }

  viewport.getWidth  = define.curry('Width');

  viewport.getHeight = define.curry('Height');
})(document.viewport);


Element.Storage = {
  UID: 1
};

Element.addMethods({
  getStorage: function(element) {
    if (!(element = $(element))) return;

    var uid;
    if (element === window) {
      uid = 0;
    } else {
      if (typeof element._prototypeUID === "undefined")
        element._prototypeUID = Element.Storage.UID++;
      uid = element._prototypeUID;
    }

    if (!Element.Storage[uid])
      Element.Storage[uid] = $H();

    return Element.Storage[uid];
  },

  store: function(element, key, value) {
    if (!(element = $(element))) return;

    if (arguments.length === 2) {
      Element.getStorage(element).update(key);
    } else {
      Element.getStorage(element).set(key, value);
    }

    return element;
  },

  retrieve: function(element, key, defaultValue) {
    if (!(element = $(element))) return;
    var hash = Element.getStorage(element), value = hash.get(key);

    if (Object.isUndefined(value)) {
      hash.set(key, defaultValue);
      value = defaultValue;
    }

    return value;
  },

  clone: function(element, deep) {
    if (!(element = $(element))) return;
    var clone = element.cloneNode(deep);
    clone._prototypeUID = void 0;
    if (deep) {
      var descendants = Element.select(clone, '*'),
          i = descendants.length;
      while (i--) {
        descendants[i]._prototypeUID = void 0;
      }
    }
    return Element.extend(clone);
  },

  purge: function(element) {
    if (!(element = $(element))) return;
    var purgeElement = Element._purgeElement;

    purgeElement(element);

    var descendants = element.getElementsByTagName('*'),
     i = descendants.length;

    while (i--) purgeElement(descendants[i]);

    return null;
  }
});

(function() {

  function toDecimal(pctString) {
    var match = pctString.match(/^(\d+)%?$/i);
    if (!match) return null;
    return (Number(match[1]) / 100);
  }

  function getPixelValue(value, property, context) {
    var element = null;
    if (Object.isElement(value)) {
      element = value;
      value = element.getStyle(property);
    }

    if (value === null) {
      return null;
    }

    if ((/^(?:-)?\d+(\.\d+)?(px)?$/i).test(value)) {
      return window.parseFloat(value);
    }

    var isPercentage = value.include('%'), isViewport = (context === document.viewport);

    if (/\d/.test(value) && element && element.runtimeStyle && !(isPercentage && isViewport)) {
      var style = element.style.left, rStyle = element.runtimeStyle.left;
      element.runtimeStyle.left = element.currentStyle.left;
      element.style.left = value || 0;
      value = element.style.pixelLeft;
      element.style.left = style;
      element.runtimeStyle.left = rStyle;

      return value;
    }

    if (element && isPercentage) {
      context = context || element.parentNode;
      var decimal = toDecimal(value);
      var whole = null;
      var position = element.getStyle('position');

      var isHorizontal = property.include('left') || property.include('right') ||
       property.include('width');

      var isVertical =  property.include('top') || property.include('bottom') ||
        property.include('height');

      if (context === document.viewport) {
        if (isHorizontal) {
          whole = document.viewport.getWidth();
        } else if (isVertical) {
          whole = document.viewport.getHeight();
        }
      } else {
        if (isHorizontal) {
          whole = $(context).measure('width');
        } else if (isVertical) {
          whole = $(context).measure('height');
        }
      }

      return (whole === null) ? 0 : whole * decimal;
    }

    return 0;
  }

  function toCSSPixels(number) {
    if (Object.isString(number) && number.endsWith('px')) {
      return number;
    }
    return number + 'px';
  }

  function isDisplayed(element) {
    var originalElement = element;
    while (element && element.parentNode) {
      var display = element.getStyle('display');
      if (display === 'none') {
        return false;
      }
      element = $(element.parentNode);
    }
    return true;
  }

  var hasLayout = Prototype.K;
  if ('currentStyle' in document.documentElement) {
    hasLayout = function(element) {
      if (!element.currentStyle.hasLayout) {
        element.style.zoom = 1;
      }
      return element;
    };
  }

  function cssNameFor(key) {
    if (key.include('border')) key = key + '-width';
    return key.camelize();
  }

  Element.Layout = Class.create(Hash, {
    initialize: function($super, element, preCompute) {
      $super();
      this.element = $(element);

      Element.Layout.PROPERTIES.each( function(property) {
        this._set(property, null);
      }, this);

      if (preCompute) {
        this._preComputing = true;
        this._begin();
        Element.Layout.PROPERTIES.each( this._compute, this );
        this._end();
        this._preComputing = false;
      }
    },

    _set: function(property, value) {
      return Hash.prototype.set.call(this, property, value);
    },

    set: function(property, value) {
      throw "Properties of Element.Layout are read-only.";
    },

    get: function($super, property) {
      var value = $super(property);
      return value === null ? this._compute(property) : value;
    },

    _begin: function() {
      if (this._prepared) return;

      var element = this.element;
      if (isDisplayed(element)) {
        this._prepared = true;
        return;
      }

      var originalStyles = {
        position:   element.style.position   || '',
        width:      element.style.width      || '',
        visibility: element.style.visibility || '',
        display:    element.style.display    || ''
      };

      element.store('prototype_original_styles', originalStyles);

      var position = element.getStyle('position'),
       width = element.getStyle('width');

      if (width === "0px" || width === null) {
        element.style.display = 'block';
        width = element.getStyle('width');
      }

      var context = (position === 'fixed') ? document.viewport :
       element.parentNode;

      element.setStyle({
        position:   'absolute',
        visibility: 'hidden',
        display:    'block'
      });

      var positionedWidth = element.getStyle('width');

      var newWidth;
      if (width && (positionedWidth === width)) {
        newWidth = getPixelValue(element, 'width', context);
      } else if (position === 'absolute' || position === 'fixed') {
        newWidth = getPixelValue(element, 'width', context);
      } else {
        var parent = element.parentNode, pLayout = $(parent).getLayout();

        newWidth = pLayout.get('width') -
         this.get('margin-left') -
         this.get('border-left') -
         this.get('padding-left') -
         this.get('padding-right') -
         this.get('border-right') -
         this.get('margin-right');
      }

      element.setStyle({ width: newWidth + 'px' });

      this._prepared = true;
    },

    _end: function() {
      var element = this.element;
      var originalStyles = element.retrieve('prototype_original_styles');
      element.store('prototype_original_styles', null);
      element.setStyle(originalStyles);
      this._prepared = false;
    },

    _compute: function(property) {
      var COMPUTATIONS = Element.Layout.COMPUTATIONS;
      if (!(property in COMPUTATIONS)) {
        throw "Property not found.";
      }

      return this._set(property, COMPUTATIONS[property].call(this, this.element));
    },

    toObject: function() {
      var args = $A(arguments);
      var keys = (args.length === 0) ? Element.Layout.PROPERTIES :
       args.join(' ').split(' ');
      var obj = {};
      keys.each( function(key) {
        if (!Element.Layout.PROPERTIES.include(key)) return;
        var value = this.get(key);
        if (value != null) obj[key] = value;
      }, this);
      return obj;
    },

    toHash: function() {
      var obj = this.toObject.apply(this, arguments);
      return new Hash(obj);
    },

    toCSS: function() {
      var args = $A(arguments);
      var keys = (args.length === 0) ? Element.Layout.PROPERTIES :
       args.join(' ').split(' ');
      var css = {};

      keys.each( function(key) {
        if (!Element.Layout.PROPERTIES.include(key)) return;
        if (Element.Layout.COMPOSITE_PROPERTIES.include(key)) return;

        var value = this.get(key);
        if (value != null) css[cssNameFor(key)] = value + 'px';
      }, this);
      return css;
    },

    inspect: function() {
      return "#<Element.Layout>";
    }
  });

  Object.extend(Element.Layout, {
    PROPERTIES: $w('height width top left right bottom border-left border-right border-top border-bottom padding-left padding-right padding-top padding-bottom margin-top margin-bottom margin-left margin-right padding-box-width padding-box-height border-box-width border-box-height margin-box-width margin-box-height'),

    COMPOSITE_PROPERTIES: $w('padding-box-width padding-box-height margin-box-width margin-box-height border-box-width border-box-height'),

    COMPUTATIONS: {
      'height': function(element) {
        if (!this._preComputing) this._begin();

        var bHeight = this.get('border-box-height');
        if (bHeight <= 0) {
          if (!this._preComputing) this._end();
          return 0;
        }

        var bTop = this.get('border-top'),
         bBottom = this.get('border-bottom');

        var pTop = this.get('padding-top'),
         pBottom = this.get('padding-bottom');

        if (!this._preComputing) this._end();

        return bHeight - bTop - bBottom - pTop - pBottom;
      },

      'width': function(element) {
        if (!this._preComputing) this._begin();

        var bWidth = this.get('border-box-width');
        if (bWidth <= 0) {
          if (!this._preComputing) this._end();
          return 0;
        }

        var bLeft = this.get('border-left'),
         bRight = this.get('border-right');

        var pLeft = this.get('padding-left'),
         pRight = this.get('padding-right');

        if (!this._preComputing) this._end();

        return bWidth - bLeft - bRight - pLeft - pRight;
      },

      'padding-box-height': function(element) {
        var height = this.get('height'),
         pTop = this.get('padding-top'),
         pBottom = this.get('padding-bottom');

        return height + pTop + pBottom;
      },

      'padding-box-width': function(element) {
        var width = this.get('width'),
         pLeft = this.get('padding-left'),
         pRight = this.get('padding-right');

        return width + pLeft + pRight;
      },

      'border-box-height': function(element) {
        if (!this._preComputing) this._begin();
        var height = element.offsetHeight;
        if (!this._preComputing) this._end();
        return height;
      },

      'border-box-width': function(element) {
        if (!this._preComputing) this._begin();
        var width = element.offsetWidth;
        if (!this._preComputing) this._end();
        return width;
      },

      'margin-box-height': function(element) {
        var bHeight = this.get('border-box-height'),
         mTop = this.get('margin-top'),
         mBottom = this.get('margin-bottom');

        if (bHeight <= 0) return 0;

        return bHeight + mTop + mBottom;
      },

      'margin-box-width': function(element) {
        var bWidth = this.get('border-box-width'),
         mLeft = this.get('margin-left'),
         mRight = this.get('margin-right');

        if (bWidth <= 0) return 0;

        return bWidth + mLeft + mRight;
      },

      'top': function(element) {
        var offset = element.positionedOffset();
        return offset.top;
      },

      'bottom': function(element) {
        var offset = element.positionedOffset(),
         parent = element.getOffsetParent(),
         pHeight = parent.measure('height');

        var mHeight = this.get('border-box-height');

        return pHeight - mHeight - offset.top;
      },

      'left': function(element) {
        var offset = element.positionedOffset();
        return offset.left;
      },

      'right': function(element) {
        var offset = element.positionedOffset(),
         parent = element.getOffsetParent(),
         pWidth = parent.measure('width');

        var mWidth = this.get('border-box-width');

        return pWidth - mWidth - offset.left;
      },

      'padding-top': function(element) {
        return getPixelValue(element, 'paddingTop');
      },

      'padding-bottom': function(element) {
        return getPixelValue(element, 'paddingBottom');
      },

      'padding-left': function(element) {
        return getPixelValue(element, 'paddingLeft');
      },

      'padding-right': function(element) {
        return getPixelValue(element, 'paddingRight');
      },

      'border-top': function(element) {
        return getPixelValue(element, 'borderTopWidth');
      },

      'border-bottom': function(element) {
        return getPixelValue(element, 'borderBottomWidth');
      },

      'border-left': function(element) {
        return getPixelValue(element, 'borderLeftWidth');
      },

      'border-right': function(element) {
        return getPixelValue(element, 'borderRightWidth');
      },

      'margin-top': function(element) {
        return getPixelValue(element, 'marginTop');
      },

      'margin-bottom': function(element) {
        return getPixelValue(element, 'marginBottom');
      },

      'margin-left': function(element) {
        return getPixelValue(element, 'marginLeft');
      },

      'margin-right': function(element) {
        return getPixelValue(element, 'marginRight');
      }
    }
  });

  if ('getBoundingClientRect' in document.documentElement) {
    Object.extend(Element.Layout.COMPUTATIONS, {
      'right': function(element) {
        var parent = hasLayout(element.getOffsetParent());
        var rect = element.getBoundingClientRect(),
         pRect = parent.getBoundingClientRect();

        return (pRect.right - rect.right).round();
      },

      'bottom': function(element) {
        var parent = hasLayout(element.getOffsetParent());
        var rect = element.getBoundingClientRect(),
         pRect = parent.getBoundingClientRect();

        return (pRect.bottom - rect.bottom).round();
      }
    });
  }

  Element.Offset = Class.create({
    initialize: function(left, top) {
      this.left = left.round();
      this.top  = top.round();

      this[0] = this.left;
      this[1] = this.top;
    },

    relativeTo: function(offset) {
      return new Element.Offset(
        this.left - offset.left,
        this.top  - offset.top
      );
    },

    inspect: function() {
      return "#<Element.Offset left: #{left} top: #{top}>".interpolate(this);
    },

    toString: function() {
      return "[#{left}, #{top}]".interpolate(this);
    },

    toArray: function() {
      return [this.left, this.top];
    }
  });

  function getLayout(element, preCompute) {
    return new Element.Layout(element, preCompute);
  }

  function measure(element, property) {
    return $(element).getLayout().get(property);
  }

  function getDimensions(element) {
    element = $(element);
    var display = Element.getStyle(element, 'display');

    if (display && display !== 'none') {
      return { width: element.offsetWidth, height: element.offsetHeight };
    }

    var style = element.style;
    var originalStyles = {
      visibility: style.visibility,
      position:   style.position,
      display:    style.display
    };

    var newStyles = {
      visibility: 'hidden',
      display:    'block'
    };

    if (originalStyles.position !== 'fixed')
      newStyles.position = 'absolute';

    Element.setStyle(element, newStyles);

    var dimensions = {
      width:  element.offsetWidth,
      height: element.offsetHeight
    };

    Element.setStyle(element, originalStyles);

    return dimensions;
  }

  function getOffsetParent(element) {
    element = $(element);

    if (isDocument(element) || isDetached(element) || isBody(element) || isHtml(element))
      return $(document.body);

    var isInline = (Element.getStyle(element, 'display') === 'inline');
    if (!isInline && element.offsetParent) return $(element.offsetParent);

    while ((element = element.parentNode) && element !== document.body) {
      if (Element.getStyle(element, 'position') !== 'static') {
        return isHtml(element) ? $(document.body) : $(element);
      }
    }

    return $(document.body);
  }


  function cumulativeOffset(element) {
    element = $(element);
    var valueT = 0, valueL = 0;
    if (element.parentNode) {
      do {
        valueT += element.offsetTop  || 0;
        valueL += element.offsetLeft || 0;
        element = element.offsetParent;
      } while (element);
    }
    return new Element.Offset(valueL, valueT);
  }

  function positionedOffset(element) {
    element = $(element);

    var layout = element.getLayout();

    var valueT = 0, valueL = 0;
    do {
      valueT += element.offsetTop  || 0;
      valueL += element.offsetLeft || 0;
      element = element.offsetParent;
      if (element) {
        if (isBody(element)) break;
        var p = Element.getStyle(element, 'position');
        if (p !== 'static') break;
      }
    } while (element);

    valueL -= layout.get('margin-top');
    valueT -= layout.get('margin-left');

    return new Element.Offset(valueL, valueT);
  }

  function cumulativeScrollOffset(element) {
    var valueT = 0, valueL = 0;
    do {
      valueT += element.scrollTop  || 0;
      valueL += element.scrollLeft || 0;
      element = element.parentNode;
    } while (element);
    return new Element.Offset(valueL, valueT);
  }

  function viewportOffset(forElement) {
    element = $(element);
    var valueT = 0, valueL = 0, docBody = document.body;

    var element = forElement;
    do {
      valueT += element.offsetTop  || 0;
      valueL += element.offsetLeft || 0;
      if (element.offsetParent == docBody &&
        Element.getStyle(element, 'position') == 'absolute') break;
    } while (element = element.offsetParent);

    element = forElement;
    do {
      if (element != docBody) {
        valueT -= element.scrollTop  || 0;
        valueL -= element.scrollLeft || 0;
      }
    } while (element = element.parentNode);
    return new Element.Offset(valueL, valueT);
  }

  function absolutize(element) {
    element = $(element);

    if (Element.getStyle(element, 'position') === 'absolute') {
      return element;
    }

    var offsetParent = getOffsetParent(element);
    var eOffset = element.viewportOffset(),
     pOffset = offsetParent.viewportOffset();

    var offset = eOffset.relativeTo(pOffset);
    var layout = element.getLayout();

    element.store('prototype_absolutize_original_styles', {
      left:   element.getStyle('left'),
      top:    element.getStyle('top'),
      width:  element.getStyle('width'),
      height: element.getStyle('height')
    });

    element.setStyle({
      position: 'absolute',
      top:    offset.top + 'px',
      left:   offset.left + 'px',
      width:  layout.get('width') + 'px',
      height: layout.get('height') + 'px'
    });

    return element;
  }

  function relativize(element) {
    element = $(element);
    if (Element.getStyle(element, 'position') === 'relative') {
      return element;
    }

    var originalStyles =
     element.retrieve('prototype_absolutize_original_styles');

    if (originalStyles) element.setStyle(originalStyles);
    return element;
  }

  if (Prototype.Browser.IE) {
    getOffsetParent = getOffsetParent.wrap(
      function(proceed, element) {
        element = $(element);

        if (isDocument(element) || isDetached(element) || isBody(element) || isHtml(element))
          return $(document.body);

        var position = element.getStyle('position');
        if (position !== 'static') return proceed(element);

        element.setStyle({ position: 'relative' });
        var value = proceed(element);
        element.setStyle({ position: position });
        return value;
      }
    );

    positionedOffset = positionedOffset.wrap(function(proceed, element) {
      element = $(element);
      if (!element.parentNode) return new Element.Offset(0, 0);
      var position = element.getStyle('position');
      if (position !== 'static') return proceed(element);

      var offsetParent = element.getOffsetParent();
      if (offsetParent && offsetParent.getStyle('position') === 'fixed')
        hasLayout(offsetParent);

      element.setStyle({ position: 'relative' });
      var value = proceed(element);
      element.setStyle({ position: position });
      return value;
    });
  } else if (Prototype.Browser.Webkit) {
    cumulativeOffset = function(element) {
      element = $(element);
      var valueT = 0, valueL = 0;
      do {
        valueT += element.offsetTop  || 0;
        valueL += element.offsetLeft || 0;
        if (element.offsetParent == document.body)
          if (Element.getStyle(element, 'position') == 'absolute') break;

        element = element.offsetParent;
      } while (element);

      return new Element.Offset(valueL, valueT);
    };
  }


  Element.addMethods({
    getLayout:              getLayout,
    measure:                measure,
    getDimensions:          getDimensions,
    getOffsetParent:        getOffsetParent,
    cumulativeOffset:       cumulativeOffset,
    positionedOffset:       positionedOffset,
    cumulativeScrollOffset: cumulativeScrollOffset,
    viewportOffset:         viewportOffset,
    absolutize:             absolutize,
    relativize:             relativize
  });

  function isBody(element) {
    return element.nodeName.toUpperCase() === 'BODY';
  }

  function isHtml(element) {
    return element.nodeName.toUpperCase() === 'HTML';
  }

  function isDocument(element) {
    return element.nodeType === Node.DOCUMENT_NODE;
  }

  function isDetached(element) {
    return element !== document.body &&
     !Element.descendantOf(element, document.body);
  }

  if ('getBoundingClientRect' in document.documentElement) {
    Element.addMethods({
      viewportOffset: function(element) {
        element = $(element);
        if (isDetached(element)) return new Element.Offset(0, 0);

        var rect = element.getBoundingClientRect(),
         docEl = document.documentElement;
        return new Element.Offset(rect.left - docEl.clientLeft,
         rect.top - docEl.clientTop);
      }
    });
  }
})();
window.$$ = function() {
  var expression = $A(arguments).join(', ');
  return Prototype.Selector.select(expression, document);
};

Prototype.Selector = (function() {

  function select() {
    throw new Error('Method "Prototype.Selector.select" must be defined.');
  }

  function match() {
    throw new Error('Method "Prototype.Selector.match" must be defined.');
  }

  function find(elements, expression, index) {
    index = index || 0;
    var match = Prototype.Selector.match, length = elements.length, matchIndex = 0, i;

    for (i = 0; i < length; i++) {
      if (match(elements[i], expression) && index == matchIndex++) {
        return Element.extend(elements[i]);
      }
    }
  }

  function extendElements(elements) {
    for (var i = 0, length = elements.length; i < length; i++) {
      Element.extend(elements[i]);
    }
    return elements;
  }


  var K = Prototype.K;

  return {
    select: select,
    match: match,
    find: find,
    extendElements: (Element.extend === K) ? K : extendElements,
    extendElement: Element.extend
  };
})();
/*!
 * Sizzle CSS Selector Engine - v1.0
 *  Copyright 2009, The Dojo Foundation
 *  Released under the MIT, BSD, and GPL Licenses.
 *  More information: http://sizzlejs.com/
 */
(function(){

var chunker = /((?:\((?:\([^()]+\)|[^()]+)+\)|\[(?:\[[^[\]]*\]|['"][^'"]*['"]|[^[\]'"]+)+\]|\\.|[^ >+~,(\[\\]+)+|[>+~])(\s*,\s*)?((?:.|\r|\n)*)/g,
	done = 0,
	toString = Object.prototype.toString,
	hasDuplicate = false,
	baseHasDuplicate = true;

[0, 0].sort(function(){
	baseHasDuplicate = false;
	return 0;
});

var Sizzle = function(selector, context, results, seed) {
	results = results || [];
	var origContext = context = context || document;

	if ( context.nodeType !== 1 && context.nodeType !== 9 ) {
		return [];
	}

	if ( !selector || typeof selector !== "string" ) {
		return results;
	}

	var parts = [], m, set, checkSet, check, mode, extra, prune = true, contextXML = isXML(context),
		soFar = selector;

	while ( (chunker.exec(""), m = chunker.exec(soFar)) !== null ) {
		soFar = m[3];

		parts.push( m[1] );

		if ( m[2] ) {
			extra = m[3];
			break;
		}
	}

	if ( parts.length > 1 && origPOS.exec( selector ) ) {
		if ( parts.length === 2 && Expr.relative[ parts[0] ] ) {
			set = posProcess( parts[0] + parts[1], context );
		} else {
			set = Expr.relative[ parts[0] ] ?
				[ context ] :
				Sizzle( parts.shift(), context );

			while ( parts.length ) {
				selector = parts.shift();

				if ( Expr.relative[ selector ] )
					selector += parts.shift();

				set = posProcess( selector, set );
			}
		}
	} else {
		if ( !seed && parts.length > 1 && context.nodeType === 9 && !contextXML &&
				Expr.match.ID.test(parts[0]) && !Expr.match.ID.test(parts[parts.length - 1]) ) {
			var ret = Sizzle.find( parts.shift(), context, contextXML );
			context = ret.expr ? Sizzle.filter( ret.expr, ret.set )[0] : ret.set[0];
		}

		if ( context ) {
			var ret = seed ?
				{ expr: parts.pop(), set: makeArray(seed) } :
				Sizzle.find( parts.pop(), parts.length === 1 && (parts[0] === "~" || parts[0] === "+") && context.parentNode ? context.parentNode : context, contextXML );
			set = ret.expr ? Sizzle.filter( ret.expr, ret.set ) : ret.set;

			if ( parts.length > 0 ) {
				checkSet = makeArray(set);
			} else {
				prune = false;
			}

			while ( parts.length ) {
				var cur = parts.pop(), pop = cur;

				if ( !Expr.relative[ cur ] ) {
					cur = "";
				} else {
					pop = parts.pop();
				}

				if ( pop == null ) {
					pop = context;
				}

				Expr.relative[ cur ]( checkSet, pop, contextXML );
			}
		} else {
			checkSet = parts = [];
		}
	}

	if ( !checkSet ) {
		checkSet = set;
	}

	if ( !checkSet ) {
		throw "Syntax error, unrecognized expression: " + (cur || selector);
	}

	if ( toString.call(checkSet) === "[object Array]" ) {
		if ( !prune ) {
			results.push.apply( results, checkSet );
		} else if ( context && context.nodeType === 1 ) {
			for ( var i = 0; checkSet[i] != null; i++ ) {
				if ( checkSet[i] && (checkSet[i] === true || checkSet[i].nodeType === 1 && contains(context, checkSet[i])) ) {
					results.push( set[i] );
				}
			}
		} else {
			for ( var i = 0; checkSet[i] != null; i++ ) {
				if ( checkSet[i] && checkSet[i].nodeType === 1 ) {
					results.push( set[i] );
				}
			}
		}
	} else {
		makeArray( checkSet, results );
	}

	if ( extra ) {
		Sizzle( extra, origContext, results, seed );
		Sizzle.uniqueSort( results );
	}

	return results;
};

Sizzle.uniqueSort = function(results){
	if ( sortOrder ) {
		hasDuplicate = baseHasDuplicate;
		results.sort(sortOrder);

		if ( hasDuplicate ) {
			for ( var i = 1; i < results.length; i++ ) {
				if ( results[i] === results[i-1] ) {
					results.splice(i--, 1);
				}
			}
		}
	}

	return results;
};

Sizzle.matches = function(expr, set){
	return Sizzle(expr, null, null, set);
};

Sizzle.find = function(expr, context, isXML){
	var set, match;

	if ( !expr ) {
		return [];
	}

	for ( var i = 0, l = Expr.order.length; i < l; i++ ) {
		var type = Expr.order[i], match;

		if ( (match = Expr.leftMatch[ type ].exec( expr )) ) {
			var left = match[1];
			match.splice(1,1);

			if ( left.substr( left.length - 1 ) !== "\\" ) {
				match[1] = (match[1] || "").replace(/\\/g, "");
				set = Expr.find[ type ]( match, context, isXML );
				if ( set != null ) {
					expr = expr.replace( Expr.match[ type ], "" );
					break;
				}
			}
		}
	}

	if ( !set ) {
		set = context.getElementsByTagName("*");
	}

	return {set: set, expr: expr};
};

Sizzle.filter = function(expr, set, inplace, not){
	var old = expr, result = [], curLoop = set, match, anyFound,
		isXMLFilter = set && set[0] && isXML(set[0]);

	while ( expr && set.length ) {
		for ( var type in Expr.filter ) {
			if ( (match = Expr.match[ type ].exec( expr )) != null ) {
				var filter = Expr.filter[ type ], found, item;
				anyFound = false;

				if ( curLoop == result ) {
					result = [];
				}

				if ( Expr.preFilter[ type ] ) {
					match = Expr.preFilter[ type ]( match, curLoop, inplace, result, not, isXMLFilter );

					if ( !match ) {
						anyFound = found = true;
					} else if ( match === true ) {
						continue;
					}
				}

				if ( match ) {
					for ( var i = 0; (item = curLoop[i]) != null; i++ ) {
						if ( item ) {
							found = filter( item, match, i, curLoop );
							var pass = not ^ !!found;

							if ( inplace && found != null ) {
								if ( pass ) {
									anyFound = true;
								} else {
									curLoop[i] = false;
								}
							} else if ( pass ) {
								result.push( item );
								anyFound = true;
							}
						}
					}
				}

				if ( found !== undefined ) {
					if ( !inplace ) {
						curLoop = result;
					}

					expr = expr.replace( Expr.match[ type ], "" );

					if ( !anyFound ) {
						return [];
					}

					break;
				}
			}
		}

		if ( expr == old ) {
			if ( anyFound == null ) {
				throw "Syntax error, unrecognized expression: " + expr;
			} else {
				break;
			}
		}

		old = expr;
	}

	return curLoop;
};

var Expr = Sizzle.selectors = {
	order: [ "ID", "NAME", "TAG" ],
	match: {
		ID: /#((?:[\w\u00c0-\uFFFF-]|\\.)+)/,
		CLASS: /\.((?:[\w\u00c0-\uFFFF-]|\\.)+)/,
		NAME: /\[name=['"]*((?:[\w\u00c0-\uFFFF-]|\\.)+)['"]*\]/,
		ATTR: /\[\s*((?:[\w\u00c0-\uFFFF-]|\\.)+)\s*(?:(\S?=)\s*(['"]*)(.*?)\3|)\s*\]/,
		TAG: /^((?:[\w\u00c0-\uFFFF\*-]|\\.)+)/,
		CHILD: /:(only|nth|last|first)-child(?:\((even|odd|[\dn+-]*)\))?/,
		POS: /:(nth|eq|gt|lt|first|last|even|odd)(?:\((\d*)\))?(?=[^-]|$)/,
		PSEUDO: /:((?:[\w\u00c0-\uFFFF-]|\\.)+)(?:\((['"]*)((?:\([^\)]+\)|[^\2\(\)]*)+)\2\))?/
	},
	leftMatch: {},
	attrMap: {
		"class": "className",
		"for": "htmlFor"
	},
	attrHandle: {
		href: function(elem){
			return elem.getAttribute("href");
		}
	},
	relative: {
		"+": function(checkSet, part, isXML){
			var isPartStr = typeof part === "string",
				isTag = isPartStr && !/\W/.test(part),
				isPartStrNotTag = isPartStr && !isTag;

			if ( isTag && !isXML ) {
				part = part.toUpperCase();
			}

			for ( var i = 0, l = checkSet.length, elem; i < l; i++ ) {
				if ( (elem = checkSet[i]) ) {
					while ( (elem = elem.previousSibling) && elem.nodeType !== 1 ) {}

					checkSet[i] = isPartStrNotTag || elem && elem.nodeName === part ?
						elem || false :
						elem === part;
				}
			}

			if ( isPartStrNotTag ) {
				Sizzle.filter( part, checkSet, true );
			}
		},
		">": function(checkSet, part, isXML){
			var isPartStr = typeof part === "string";

			if ( isPartStr && !/\W/.test(part) ) {
				part = isXML ? part : part.toUpperCase();

				for ( var i = 0, l = checkSet.length; i < l; i++ ) {
					var elem = checkSet[i];
					if ( elem ) {
						var parent = elem.parentNode;
						checkSet[i] = parent.nodeName === part ? parent : false;
					}
				}
			} else {
				for ( var i = 0, l = checkSet.length; i < l; i++ ) {
					var elem = checkSet[i];
					if ( elem ) {
						checkSet[i] = isPartStr ?
							elem.parentNode :
							elem.parentNode === part;
					}
				}

				if ( isPartStr ) {
					Sizzle.filter( part, checkSet, true );
				}
			}
		},
		"": function(checkSet, part, isXML){
			var doneName = done++, checkFn = dirCheck;

			if ( !/\W/.test(part) ) {
				var nodeCheck = part = isXML ? part : part.toUpperCase();
				checkFn = dirNodeCheck;
			}

			checkFn("parentNode", part, doneName, checkSet, nodeCheck, isXML);
		},
		"~": function(checkSet, part, isXML){
			var doneName = done++, checkFn = dirCheck;

			if ( typeof part === "string" && !/\W/.test(part) ) {
				var nodeCheck = part = isXML ? part : part.toUpperCase();
				checkFn = dirNodeCheck;
			}

			checkFn("previousSibling", part, doneName, checkSet, nodeCheck, isXML);
		}
	},
	find: {
		ID: function(match, context, isXML){
			if ( typeof context.getElementById !== "undefined" && !isXML ) {
				var m = context.getElementById(match[1]);
				return m ? [m] : [];
			}
		},
		NAME: function(match, context, isXML){
			if ( typeof context.getElementsByName !== "undefined" ) {
				var ret = [], results = context.getElementsByName(match[1]);

				for ( var i = 0, l = results.length; i < l; i++ ) {
					if ( results[i].getAttribute("name") === match[1] ) {
						ret.push( results[i] );
					}
				}

				return ret.length === 0 ? null : ret;
			}
		},
		TAG: function(match, context){
			return context.getElementsByTagName(match[1]);
		}
	},
	preFilter: {
		CLASS: function(match, curLoop, inplace, result, not, isXML){
			match = " " + match[1].replace(/\\/g, "") + " ";

			if ( isXML ) {
				return match;
			}

			for ( var i = 0, elem; (elem = curLoop[i]) != null; i++ ) {
				if ( elem ) {
					if ( not ^ (elem.className && (" " + elem.className + " ").indexOf(match) >= 0) ) {
						if ( !inplace )
							result.push( elem );
					} else if ( inplace ) {
						curLoop[i] = false;
					}
				}
			}

			return false;
		},
		ID: function(match){
			return match[1].replace(/\\/g, "");
		},
		TAG: function(match, curLoop){
			for ( var i = 0; curLoop[i] === false; i++ ){}
			return curLoop[i] && isXML(curLoop[i]) ? match[1] : match[1].toUpperCase();
		},
		CHILD: function(match){
			if ( match[1] == "nth" ) {
				var test = /(-?)(\d*)n((?:\+|-)?\d*)/.exec(
					match[2] == "even" && "2n" || match[2] == "odd" && "2n+1" ||
					!/\D/.test( match[2] ) && "0n+" + match[2] || match[2]);

				match[2] = (test[1] + (test[2] || 1)) - 0;
				match[3] = test[3] - 0;
			}

			match[0] = done++;

			return match;
		},
		ATTR: function(match, curLoop, inplace, result, not, isXML){
			var name = match[1].replace(/\\/g, "");

			if ( !isXML && Expr.attrMap[name] ) {
				match[1] = Expr.attrMap[name];
			}

			if ( match[2] === "~=" ) {
				match[4] = " " + match[4] + " ";
			}

			return match;
		},
		PSEUDO: function(match, curLoop, inplace, result, not){
			if ( match[1] === "not" ) {
				if ( ( chunker.exec(match[3]) || "" ).length > 1 || /^\w/.test(match[3]) ) {
					match[3] = Sizzle(match[3], null, null, curLoop);
				} else {
					var ret = Sizzle.filter(match[3], curLoop, inplace, true ^ not);
					if ( !inplace ) {
						result.push.apply( result, ret );
					}
					return false;
				}
			} else if ( Expr.match.POS.test( match[0] ) || Expr.match.CHILD.test( match[0] ) ) {
				return true;
			}

			return match;
		},
		POS: function(match){
			match.unshift( true );
			return match;
		}
	},
	filters: {
		enabled: function(elem){
			return elem.disabled === false && elem.type !== "hidden";
		},
		disabled: function(elem){
			return elem.disabled === true;
		},
		checked: function(elem){
			return elem.checked === true;
		},
		selected: function(elem){
			elem.parentNode.selectedIndex;
			return elem.selected === true;
		},
		parent: function(elem){
			return !!elem.firstChild;
		},
		empty: function(elem){
			return !elem.firstChild;
		},
		has: function(elem, i, match){
			return !!Sizzle( match[3], elem ).length;
		},
		header: function(elem){
			return /h\d/i.test( elem.nodeName );
		},
		text: function(elem){
			return "text" === elem.type;
		},
		radio: function(elem){
			return "radio" === elem.type;
		},
		checkbox: function(elem){
			return "checkbox" === elem.type;
		},
		file: function(elem){
			return "file" === elem.type;
		},
		password: function(elem){
			return "password" === elem.type;
		},
		submit: function(elem){
			return "submit" === elem.type;
		},
		image: function(elem){
			return "image" === elem.type;
		},
		reset: function(elem){
			return "reset" === elem.type;
		},
		button: function(elem){
			return "button" === elem.type || elem.nodeName.toUpperCase() === "BUTTON";
		},
		input: function(elem){
			return /input|select|textarea|button/i.test(elem.nodeName);
		}
	},
	setFilters: {
		first: function(elem, i){
			return i === 0;
		},
		last: function(elem, i, match, array){
			return i === array.length - 1;
		},
		even: function(elem, i){
			return i % 2 === 0;
		},
		odd: function(elem, i){
			return i % 2 === 1;
		},
		lt: function(elem, i, match){
			return i < match[3] - 0;
		},
		gt: function(elem, i, match){
			return i > match[3] - 0;
		},
		nth: function(elem, i, match){
			return match[3] - 0 == i;
		},
		eq: function(elem, i, match){
			return match[3] - 0 == i;
		}
	},
	filter: {
		PSEUDO: function(elem, match, i, array){
			var name = match[1], filter = Expr.filters[ name ];

			if ( filter ) {
				return filter( elem, i, match, array );
			} else if ( name === "contains" ) {
				return (elem.textContent || elem.innerText || "").indexOf(match[3]) >= 0;
			} else if ( name === "not" ) {
				var not = match[3];

				for ( var i = 0, l = not.length; i < l; i++ ) {
					if ( not[i] === elem ) {
						return false;
					}
				}

				return true;
			}
		},
		CHILD: function(elem, match){
			var type = match[1], node = elem;
			switch (type) {
				case 'only':
				case 'first':
					while ( (node = node.previousSibling) )  {
						if ( node.nodeType === 1 ) return false;
					}
					if ( type == 'first') return true;
					node = elem;
				case 'last':
					while ( (node = node.nextSibling) )  {
						if ( node.nodeType === 1 ) return false;
					}
					return true;
				case 'nth':
					var first = match[2], last = match[3];

					if ( first == 1 && last == 0 ) {
						return true;
					}

					var doneName = match[0],
						parent = elem.parentNode;

					if ( parent && (parent.sizcache !== doneName || !elem.nodeIndex) ) {
						var count = 0;
						for ( node = parent.firstChild; node; node = node.nextSibling ) {
							if ( node.nodeType === 1 ) {
								node.nodeIndex = ++count;
							}
						}
						parent.sizcache = doneName;
					}

					var diff = elem.nodeIndex - last;
					if ( first == 0 ) {
						return diff == 0;
					} else {
						return ( diff % first == 0 && diff / first >= 0 );
					}
			}
		},
		ID: function(elem, match){
			return elem.nodeType === 1 && elem.getAttribute("id") === match;
		},
		TAG: function(elem, match){
			return (match === "*" && elem.nodeType === 1) || elem.nodeName === match;
		},
		CLASS: function(elem, match){
			return (" " + (elem.className || elem.getAttribute("class")) + " ")
				.indexOf( match ) > -1;
		},
		ATTR: function(elem, match){
			var name = match[1],
				result = Expr.attrHandle[ name ] ?
					Expr.attrHandle[ name ]( elem ) :
					elem[ name ] != null ?
						elem[ name ] :
						elem.getAttribute( name ),
				value = result + "",
				type = match[2],
				check = match[4];

			return result == null ?
				type === "!=" :
				type === "=" ?
				value === check :
				type === "*=" ?
				value.indexOf(check) >= 0 :
				type === "~=" ?
				(" " + value + " ").indexOf(check) >= 0 :
				!check ?
				value && result !== false :
				type === "!=" ?
				value != check :
				type === "^=" ?
				value.indexOf(check) === 0 :
				type === "$=" ?
				value.substr(value.length - check.length) === check :
				type === "|=" ?
				value === check || value.substr(0, check.length + 1) === check + "-" :
				false;
		},
		POS: function(elem, match, i, array){
			var name = match[2], filter = Expr.setFilters[ name ];

			if ( filter ) {
				return filter( elem, i, match, array );
			}
		}
	}
};

var origPOS = Expr.match.POS;

for ( var type in Expr.match ) {
	Expr.match[ type ] = new RegExp( Expr.match[ type ].source + /(?![^\[]*\])(?![^\(]*\))/.source );
	Expr.leftMatch[ type ] = new RegExp( /(^(?:.|\r|\n)*?)/.source + Expr.match[ type ].source );
}

var makeArray = function(array, results) {
	array = Array.prototype.slice.call( array, 0 );

	if ( results ) {
		results.push.apply( results, array );
		return results;
	}

	return array;
};

try {
	Array.prototype.slice.call( document.documentElement.childNodes, 0 );

} catch(e){
	makeArray = function(array, results) {
		var ret = results || [];

		if ( toString.call(array) === "[object Array]" ) {
			Array.prototype.push.apply( ret, array );
		} else {
			if ( typeof array.length === "number" ) {
				for ( var i = 0, l = array.length; i < l; i++ ) {
					ret.push( array[i] );
				}
			} else {
				for ( var i = 0; array[i]; i++ ) {
					ret.push( array[i] );
				}
			}
		}

		return ret;
	};
}

var sortOrder;

if ( document.documentElement.compareDocumentPosition ) {
	sortOrder = function( a, b ) {
		if ( !a.compareDocumentPosition || !b.compareDocumentPosition ) {
			if ( a == b ) {
				hasDuplicate = true;
			}
			return 0;
		}

		var ret = a.compareDocumentPosition(b) & 4 ? -1 : a === b ? 0 : 1;
		if ( ret === 0 ) {
			hasDuplicate = true;
		}
		return ret;
	};
} else if ( "sourceIndex" in document.documentElement ) {
	sortOrder = function( a, b ) {
		if ( !a.sourceIndex || !b.sourceIndex ) {
			if ( a == b ) {
				hasDuplicate = true;
			}
			return 0;
		}

		var ret = a.sourceIndex - b.sourceIndex;
		if ( ret === 0 ) {
			hasDuplicate = true;
		}
		return ret;
	};
} else if ( document.createRange ) {
	sortOrder = function( a, b ) {
		if ( !a.ownerDocument || !b.ownerDocument ) {
			if ( a == b ) {
				hasDuplicate = true;
			}
			return 0;
		}

		var aRange = a.ownerDocument.createRange(), bRange = b.ownerDocument.createRange();
		aRange.setStart(a, 0);
		aRange.setEnd(a, 0);
		bRange.setStart(b, 0);
		bRange.setEnd(b, 0);
		var ret = aRange.compareBoundaryPoints(Range.START_TO_END, bRange);
		if ( ret === 0 ) {
			hasDuplicate = true;
		}
		return ret;
	};
}

(function(){
	var form = document.createElement("div"),
		id = "script" + (new Date).getTime();
	form.innerHTML = "<a name='" + id + "'/>";

	var root = document.documentElement;
	root.insertBefore( form, root.firstChild );

	if ( !!document.getElementById( id ) ) {
		Expr.find.ID = function(match, context, isXML){
			if ( typeof context.getElementById !== "undefined" && !isXML ) {
				var m = context.getElementById(match[1]);
				return m ? m.id === match[1] || typeof m.getAttributeNode !== "undefined" && m.getAttributeNode("id").nodeValue === match[1] ? [m] : undefined : [];
			}
		};

		Expr.filter.ID = function(elem, match){
			var node = typeof elem.getAttributeNode !== "undefined" && elem.getAttributeNode("id");
			return elem.nodeType === 1 && node && node.nodeValue === match;
		};
	}

	root.removeChild( form );
	root = form = null; // release memory in IE
})();

(function(){

	var div = document.createElement("div");
	div.appendChild( document.createComment("") );

	if ( div.getElementsByTagName("*").length > 0 ) {
		Expr.find.TAG = function(match, context){
			var results = context.getElementsByTagName(match[1]);

			if ( match[1] === "*" ) {
				var tmp = [];

				for ( var i = 0; results[i]; i++ ) {
					if ( results[i].nodeType === 1 ) {
						tmp.push( results[i] );
					}
				}

				results = tmp;
			}

			return results;
		};
	}

	div.innerHTML = "<a href='#'></a>";
	if ( div.firstChild && typeof div.firstChild.getAttribute !== "undefined" &&
			div.firstChild.getAttribute("href") !== "#" ) {
		Expr.attrHandle.href = function(elem){
			return elem.getAttribute("href", 2);
		};
	}

	div = null; // release memory in IE
})();

if ( document.querySelectorAll ) (function(){
	var oldSizzle = Sizzle, div = document.createElement("div");
	div.innerHTML = "<p class='TEST'></p>";

	if ( div.querySelectorAll && div.querySelectorAll(".TEST").length === 0 ) {
		return;
	}

	Sizzle = function(query, context, extra, seed){
		context = context || document;

		if ( !seed && context.nodeType === 9 && !isXML(context) ) {
			try {
				return makeArray( context.querySelectorAll(query), extra );
			} catch(e){}
		}

		return oldSizzle(query, context, extra, seed);
	};

	for ( var prop in oldSizzle ) {
		Sizzle[ prop ] = oldSizzle[ prop ];
	}

	div = null; // release memory in IE
})();

if ( document.getElementsByClassName && document.documentElement.getElementsByClassName ) (function(){
	var div = document.createElement("div");
	div.innerHTML = "<div class='test e'></div><div class='test'></div>";

	if ( div.getElementsByClassName("e").length === 0 )
		return;

	div.lastChild.className = "e";

	if ( div.getElementsByClassName("e").length === 1 )
		return;

	Expr.order.splice(1, 0, "CLASS");
	Expr.find.CLASS = function(match, context, isXML) {
		if ( typeof context.getElementsByClassName !== "undefined" && !isXML ) {
			return context.getElementsByClassName(match[1]);
		}
	};

	div = null; // release memory in IE
})();

function dirNodeCheck( dir, cur, doneName, checkSet, nodeCheck, isXML ) {
	var sibDir = dir == "previousSibling" && !isXML;
	for ( var i = 0, l = checkSet.length; i < l; i++ ) {
		var elem = checkSet[i];
		if ( elem ) {
			if ( sibDir && elem.nodeType === 1 ){
				elem.sizcache = doneName;
				elem.sizset = i;
			}
			elem = elem[dir];
			var match = false;

			while ( elem ) {
				if ( elem.sizcache === doneName ) {
					match = checkSet[elem.sizset];
					break;
				}

				if ( elem.nodeType === 1 && !isXML ){
					elem.sizcache = doneName;
					elem.sizset = i;
				}

				if ( elem.nodeName === cur ) {
					match = elem;
					break;
				}

				elem = elem[dir];
			}

			checkSet[i] = match;
		}
	}
}

function dirCheck( dir, cur, doneName, checkSet, nodeCheck, isXML ) {
	var sibDir = dir == "previousSibling" && !isXML;
	for ( var i = 0, l = checkSet.length; i < l; i++ ) {
		var elem = checkSet[i];
		if ( elem ) {
			if ( sibDir && elem.nodeType === 1 ) {
				elem.sizcache = doneName;
				elem.sizset = i;
			}
			elem = elem[dir];
			var match = false;

			while ( elem ) {
				if ( elem.sizcache === doneName ) {
					match = checkSet[elem.sizset];
					break;
				}

				if ( elem.nodeType === 1 ) {
					if ( !isXML ) {
						elem.sizcache = doneName;
						elem.sizset = i;
					}
					if ( typeof cur !== "string" ) {
						if ( elem === cur ) {
							match = true;
							break;
						}

					} else if ( Sizzle.filter( cur, [elem] ).length > 0 ) {
						match = elem;
						break;
					}
				}

				elem = elem[dir];
			}

			checkSet[i] = match;
		}
	}
}

var contains = document.compareDocumentPosition ?  function(a, b){
	return a.compareDocumentPosition(b) & 16;
} : function(a, b){
	return a !== b && (a.contains ? a.contains(b) : true);
};

var isXML = function(elem){
	return elem.nodeType === 9 && elem.documentElement.nodeName !== "HTML" ||
		!!elem.ownerDocument && elem.ownerDocument.documentElement.nodeName !== "HTML";
};

var posProcess = function(selector, context){
	var tmpSet = [], later = "", match,
		root = context.nodeType ? [context] : context;

	while ( (match = Expr.match.PSEUDO.exec( selector )) ) {
		later += match[0];
		selector = selector.replace( Expr.match.PSEUDO, "" );
	}

	selector = Expr.relative[selector] ? selector + "*" : selector;

	for ( var i = 0, l = root.length; i < l; i++ ) {
		Sizzle( selector, root[i], tmpSet );
	}

	return Sizzle.filter( later, tmpSet );
};


window.Sizzle = Sizzle;

})();

Prototype._original_property = window.Sizzle;

;(function(engine) {
  var extendElements = Prototype.Selector.extendElements;

  function select(selector, scope) {
    return extendElements(engine(selector, scope || document));
  }

  function match(element, selector) {
    return engine.matches(selector, [element]).length == 1;
  }

  Prototype.Selector.engine = engine;
  Prototype.Selector.select = select;
  Prototype.Selector.match = match;
})(Sizzle);

window.Sizzle = Prototype._original_property;
delete Prototype._original_property;

var Form = {
  reset: function(form) {
    form = $(form);
    form.reset();
    return form;
  },

  serializeElements: function(elements, options) {
    if (typeof options != 'object') options = { hash: !!options };
    else if (Object.isUndefined(options.hash)) options.hash = true;
    var key, value, submitted = false, submit = options.submit, accumulator, initial;

    if (options.hash) {
      initial = {};
      accumulator = function(result, key, value) {
        if (key in result) {
          if (!Object.isArray(result[key])) result[key] = [result[key]];
          result[key].push(value);
        } else result[key] = value;
        return result;
      };
    } else {
      initial = '';
      accumulator = function(result, key, value) {
        return result + (result ? '&' : '') + encodeURIComponent(key) + '=' + encodeURIComponent(value);
      }
    }

    return elements.inject(initial, function(result, element) {
      if (!element.disabled && element.name) {
        key = element.name; value = $(element).getValue();
        if (value != null && element.type != 'file' && (element.type != 'submit' || (!submitted &&
            submit !== false && (!submit || key == submit) && (submitted = true)))) {
          result = accumulator(result, key, value);
        }
      }
      return result;
    });
  }
};

Form.Methods = {
  serialize: function(form, options) {
    return Form.serializeElements(Form.getElements(form), options);
  },

  getElements: function(form) {
    var elements = $(form).getElementsByTagName('*'),
        element,
        arr = [ ],
        serializers = Form.Element.Serializers;
    for (var i = 0; element = elements[i]; i++) {
      arr.push(element);
    }
    return arr.inject([], function(elements, child) {
      if (serializers[child.tagName.toLowerCase()])
        elements.push(Element.extend(child));
      return elements;
    })
  },

  getInputs: function(form, typeName, name) {
    form = $(form);
    var inputs = form.getElementsByTagName('input');

    if (!typeName && !name) return $A(inputs).map(Element.extend);

    for (var i = 0, matchingInputs = [], length = inputs.length; i < length; i++) {
      var input = inputs[i];
      if ((typeName && input.type != typeName) || (name && input.name != name))
        continue;
      matchingInputs.push(Element.extend(input));
    }

    return matchingInputs;
  },

  disable: function(form) {
    form = $(form);
    Form.getElements(form).invoke('disable');
    return form;
  },

  enable: function(form) {
    form = $(form);
    Form.getElements(form).invoke('enable');
    return form;
  },

  findFirstElement: function(form) {
    var elements = $(form).getElements().findAll(function(element) {
      return 'hidden' != element.type && !element.disabled;
    });
    var firstByIndex = elements.findAll(function(element) {
      return element.hasAttribute('tabIndex') && element.tabIndex >= 0;
    }).sortBy(function(element) { return element.tabIndex }).first();

    return firstByIndex ? firstByIndex : elements.find(function(element) {
      return /^(?:input|select|textarea)$/i.test(element.tagName);
    });
  },

  focusFirstElement: function(form) {
    form = $(form);
    var element = form.findFirstElement();
    if (element) element.activate();
    return form;
  },

  request: function(form, options) {
    form = $(form), options = Object.clone(options || { });

    var params = options.parameters, action = form.readAttribute('action') || '';
    if (action.blank()) action = window.location.href;
    options.parameters = form.serialize(true);

    if (params) {
      if (Object.isString(params)) params = params.toQueryParams();
      Object.extend(options.parameters, params);
    }

    if (form.hasAttribute('method') && !options.method)
      options.method = form.method;

    return new Ajax.Request(action, options);
  }
};

/*--------------------------------------------------------------------------*/


Form.Element = {
  focus: function(element) {
    $(element).focus();
    return element;
  },

  select: function(element) {
    $(element).select();
    return element;
  }
};

Form.Element.Methods = {

  serialize: function(element) {
    element = $(element);
    if (!element.disabled && element.name) {
      var value = element.getValue();
      if (value != undefined) {
        var pair = { };
        pair[element.name] = value;
        return Object.toQueryString(pair);
      }
    }
    return '';
  },

  getValue: function(element) {
    element = $(element);
    var method = element.tagName.toLowerCase();
    return Form.Element.Serializers[method](element);
  },

  setValue: function(element, value) {
    element = $(element);
    var method = element.tagName.toLowerCase();
    Form.Element.Serializers[method](element, value);
    return element;
  },

  clear: function(element) {
    $(element).value = '';
    return element;
  },

  present: function(element) {
    return $(element).value != '';
  },

  activate: function(element) {
    element = $(element);
    try {
      element.focus();
      if (element.select && (element.tagName.toLowerCase() != 'input' ||
          !(/^(?:button|reset|submit)$/i.test(element.type))))
        element.select();
    } catch (e) { }
    return element;
  },

  disable: function(element) {
    element = $(element);
    element.disabled = true;
    return element;
  },

  enable: function(element) {
    element = $(element);
    element.disabled = false;
    return element;
  }
};

/*--------------------------------------------------------------------------*/

var Field = Form.Element;

var $F = Form.Element.Methods.getValue;

/*--------------------------------------------------------------------------*/

Form.Element.Serializers = (function() {
  function input(element, value) {
    switch (element.type.toLowerCase()) {
      case 'checkbox':
      case 'radio':
        return inputSelector(element, value);
      default:
        return valueSelector(element, value);
    }
  }

  function inputSelector(element, value) {
    if (Object.isUndefined(value))
      return element.checked ? element.value : null;
    else element.checked = !!value;
  }

  function valueSelector(element, value) {
    if (Object.isUndefined(value)) return element.value;
    else element.value = value;
  }

  function select(element, value) {
    if (Object.isUndefined(value))
      return (element.type === 'select-one' ? selectOne : selectMany)(element);

    var opt, currentValue, single = !Object.isArray(value);
    for (var i = 0, length = element.length; i < length; i++) {
      opt = element.options[i];
      currentValue = this.optionValue(opt);
      if (single) {
        if (currentValue == value) {
          opt.selected = true;
          return;
        }
      }
      else opt.selected = value.include(currentValue);
    }
  }

  function selectOne(element) {
    var index = element.selectedIndex;
    return index >= 0 ? optionValue(element.options[index]) : null;
  }

  function selectMany(element) {
    var values, length = element.length;
    if (!length) return null;

    for (var i = 0, values = []; i < length; i++) {
      var opt = element.options[i];
      if (opt.selected) values.push(optionValue(opt));
    }
    return values;
  }

  function optionValue(opt) {
    return Element.hasAttribute(opt, 'value') ? opt.value : opt.text;
  }

  return {
    input:         input,
    inputSelector: inputSelector,
    textarea:      valueSelector,
    select:        select,
    selectOne:     selectOne,
    selectMany:    selectMany,
    optionValue:   optionValue,
    button:        valueSelector
  };
})();

/*--------------------------------------------------------------------------*/


Abstract.TimedObserver = Class.create(PeriodicalExecuter, {
  initialize: function($super, element, frequency, callback) {
    $super(callback, frequency);
    this.element   = $(element);
    this.lastValue = this.getValue();
  },

  execute: function() {
    var value = this.getValue();
    if (Object.isString(this.lastValue) && Object.isString(value) ?
        this.lastValue != value : String(this.lastValue) != String(value)) {
      this.callback(this.element, value);
      this.lastValue = value;
    }
  }
});

Form.Element.Observer = Class.create(Abstract.TimedObserver, {
  getValue: function() {
    return Form.Element.getValue(this.element);
  }
});

Form.Observer = Class.create(Abstract.TimedObserver, {
  getValue: function() {
    return Form.serialize(this.element);
  }
});

/*--------------------------------------------------------------------------*/

Abstract.EventObserver = Class.create({
  initialize: function(element, callback) {
    this.element  = $(element);
    this.callback = callback;

    this.lastValue = this.getValue();
    if (this.element.tagName.toLowerCase() == 'form')
      this.registerFormCallbacks();
    else
      this.registerCallback(this.element);
  },

  onElementEvent: function() {
    var value = this.getValue();
    if (this.lastValue != value) {
      this.callback(this.element, value);
      this.lastValue = value;
    }
  },

  registerFormCallbacks: function() {
    Form.getElements(this.element).each(this.registerCallback, this);
  },

  registerCallback: function(element) {
    if (element.type) {
      switch (element.type.toLowerCase()) {
        case 'checkbox':
        case 'radio':
          Event.observe(element, 'click', this.onElementEvent.bind(this));
          break;
        default:
          Event.observe(element, 'change', this.onElementEvent.bind(this));
          break;
      }
    }
  }
});

Form.Element.EventObserver = Class.create(Abstract.EventObserver, {
  getValue: function() {
    return Form.Element.getValue(this.element);
  }
});

Form.EventObserver = Class.create(Abstract.EventObserver, {
  getValue: function() {
    return Form.serialize(this.element);
  }
});
(function() {

  var Event = {
    KEY_BACKSPACE: 8,
    KEY_TAB:       9,
    KEY_RETURN:   13,
    KEY_ESC:      27,
    KEY_LEFT:     37,
    KEY_UP:       38,
    KEY_RIGHT:    39,
    KEY_DOWN:     40,
    KEY_DELETE:   46,
    KEY_HOME:     36,
    KEY_END:      35,
    KEY_PAGEUP:   33,
    KEY_PAGEDOWN: 34,
    KEY_INSERT:   45,

    cache: {}
  };

  var docEl = document.documentElement;
  var MOUSEENTER_MOUSELEAVE_EVENTS_SUPPORTED = 'onmouseenter' in docEl
    && 'onmouseleave' in docEl;



  var isIELegacyEvent = function(event) { return false; };

  if (window.attachEvent) {
    if (window.addEventListener) {
      isIELegacyEvent = function(event) {
        return !(event instanceof window.Event);
      };
    } else {
      isIELegacyEvent = function(event) { return true; };
    }
  }

  var _isButton;

  function _isButtonForDOMEvents(event, code) {
    return event.which ? (event.which === code + 1) : (event.button === code);
  }

  var legacyButtonMap = { 0: 1, 1: 4, 2: 2 };
  function _isButtonForLegacyEvents(event, code) {
    return event.button === legacyButtonMap[code];
  }

  function _isButtonForWebKit(event, code) {
    switch (code) {
      case 0: return event.which == 1 && !event.metaKey;
      case 1: return event.which == 2 || (event.which == 1 && event.metaKey);
      case 2: return event.which == 3;
      default: return false;
    }
  }

  if (window.attachEvent) {
    if (!window.addEventListener) {
      _isButton = _isButtonForLegacyEvents;
    } else {
      _isButton = function(event, code) {
        return isIELegacyEvent(event) ? _isButtonForLegacyEvents(event, code) :
         _isButtonForDOMEvents(event, code);
      }
    }
  } else if (Prototype.Browser.WebKit) {
    _isButton = _isButtonForWebKit;
  } else {
    _isButton = _isButtonForDOMEvents;
  }

  function isLeftClick(event)   { return _isButton(event, 0) }

  function isMiddleClick(event) { return _isButton(event, 1) }

  function isRightClick(event)  { return _isButton(event, 2) }

  function element(event) {
    event = Event.extend(event);

    var node = event.target, type = event.type,
     currentTarget = event.currentTarget;

    if (currentTarget && currentTarget.tagName) {
      if (type === 'load' || type === 'error' ||
        (type === 'click' && currentTarget.tagName.toLowerCase() === 'input'
          && currentTarget.type === 'radio'))
            node = currentTarget;
    }

    if (node.nodeType == Node.TEXT_NODE)
      node = node.parentNode;

    return Element.extend(node);
  }

  function findElement(event, expression) {
    var element = Event.element(event);

    if (!expression) return element;
    while (element) {
      if (Object.isElement(element) && Prototype.Selector.match(element, expression)) {
        return Element.extend(element);
      }
      element = element.parentNode;
    }
  }

  function pointer(event) {
    return { x: pointerX(event), y: pointerY(event) };
  }

  function pointerX(event) {
    var docElement = document.documentElement,
     body = document.body || { scrollLeft: 0 };

    return event.pageX || (event.clientX +
      (docElement.scrollLeft || body.scrollLeft) -
      (docElement.clientLeft || 0));
  }

  function pointerY(event) {
    var docElement = document.documentElement,
     body = document.body || { scrollTop: 0 };

    return  event.pageY || (event.clientY +
       (docElement.scrollTop || body.scrollTop) -
       (docElement.clientTop || 0));
  }


  function stop(event) {
    Event.extend(event);
    event.preventDefault();
    event.stopPropagation();

    event.stopped = true;
  }


  Event.Methods = {
    isLeftClick:   isLeftClick,
    isMiddleClick: isMiddleClick,
    isRightClick:  isRightClick,

    element:     element,
    findElement: findElement,

    pointer:  pointer,
    pointerX: pointerX,
    pointerY: pointerY,

    stop: stop
  };

  var methods = Object.keys(Event.Methods).inject({ }, function(m, name) {
    m[name] = Event.Methods[name].methodize();
    return m;
  });

  if (window.attachEvent) {
    function _relatedTarget(event) {
      var element;
      switch (event.type) {
        case 'mouseover':
        case 'mouseenter':
          element = event.fromElement;
          break;
        case 'mouseout':
        case 'mouseleave':
          element = event.toElement;
          break;
        default:
          return null;
      }
      return Element.extend(element);
    }

    var additionalMethods = {
      stopPropagation: function() { this.cancelBubble = true },
      preventDefault:  function() { this.returnValue = false },
      inspect: function() { return '[object Event]' }
    };

    Event.extend = function(event, element) {
      if (!event) return false;

      if (!isIELegacyEvent(event)) return event;

      if (event._extendedByPrototype) return event;
      event._extendedByPrototype = Prototype.emptyFunction;

      var pointer = Event.pointer(event);

      Object.extend(event, {
        target: event.srcElement || element,
        relatedTarget: _relatedTarget(event),
        pageX:  pointer.x,
        pageY:  pointer.y
      });

      Object.extend(event, methods);
      Object.extend(event, additionalMethods);

      return event;
    };
  } else {
    Event.extend = Prototype.K;
  }

  if (window.addEventListener) {
    Event.prototype = window.Event.prototype || document.createEvent('HTMLEvents').__proto__;
    Object.extend(Event.prototype, methods);
  }

  function _createResponder(element, eventName, handler) {
    var registry = Element.retrieve(element, 'prototype_event_registry');

    if (Object.isUndefined(registry)) {
      CACHE.push(element);
      registry = Element.retrieve(element, 'prototype_event_registry', $H());
    }

    var respondersForEvent = registry.get(eventName);
    if (Object.isUndefined(respondersForEvent)) {
      respondersForEvent = [];
      registry.set(eventName, respondersForEvent);
    }

    if (respondersForEvent.pluck('handler').include(handler)) return false;

    var responder;
    if (eventName.include(":")) {
      responder = function(event) {
        if (Object.isUndefined(event.eventName))
          return false;

        if (event.eventName !== eventName)
          return false;

        Event.extend(event, element);
        handler.call(element, event);
      };
    } else {
      if (!MOUSEENTER_MOUSELEAVE_EVENTS_SUPPORTED &&
       (eventName === "mouseenter" || eventName === "mouseleave")) {
        if (eventName === "mouseenter" || eventName === "mouseleave") {
          responder = function(event) {
            Event.extend(event, element);

            var parent = event.relatedTarget;
            while (parent && parent !== element) {
              try { parent = parent.parentNode; }
              catch(e) { parent = element; }
            }

            if (parent === element) return;

            handler.call(element, event);
          };
        }
      } else {
        responder = function(event) {
          Event.extend(event, element);
          handler.call(element, event);
        };
      }
    }

    responder.handler = handler;
    respondersForEvent.push(responder);
    return responder;
  }

  function _destroyCache() {
    for (var i = 0, length = CACHE.length; i < length; i++) {
      Event.stopObserving(CACHE[i]);
      CACHE[i] = null;
    }
  }

  var CACHE = [];

  if (Prototype.Browser.IE)
    window.attachEvent('onunload', _destroyCache);

  if (Prototype.Browser.WebKit)
    window.addEventListener('unload', Prototype.emptyFunction, false);


  var _getDOMEventName = Prototype.K,
      translations = { mouseenter: "mouseover", mouseleave: "mouseout" };

  if (!MOUSEENTER_MOUSELEAVE_EVENTS_SUPPORTED) {
    _getDOMEventName = function(eventName) {
      return (translations[eventName] || eventName);
    };
  }

  function observe(element, eventName, handler) {
    element = $(element);

    var responder = _createResponder(element, eventName, handler);

    if (!responder) return element;

    if (eventName.include(':')) {
      if (element.addEventListener)
        element.addEventListener("dataavailable", responder, false);
      else {
        element.attachEvent("ondataavailable", responder);
        element.attachEvent("onlosecapture", responder);
      }
    } else {
      var actualEventName = _getDOMEventName(eventName);

      if (element.addEventListener)
        element.addEventListener(actualEventName, responder, false);
      else
        element.attachEvent("on" + actualEventName, responder);
    }

    return element;
  }

  function stopObserving(element, eventName, handler) {
    element = $(element);

    var registry = Element.retrieve(element, 'prototype_event_registry');
    if (!registry) return element;

    if (!eventName) {
      registry.each( function(pair) {
        var eventName = pair.key;
        stopObserving(element, eventName);
      });
      return element;
    }

    var responders = registry.get(eventName);
    if (!responders) return element;

    if (!handler) {
      responders.each(function(r) {
        stopObserving(element, eventName, r.handler);
      });
      return element;
    }

    var i = responders.length, responder;
    while (i--) {
      if (responders[i].handler === handler) {
        responder = responders[i];
        break;
      }
    }
    if (!responder) return element;

    if (eventName.include(':')) {
      if (element.removeEventListener)
        element.removeEventListener("dataavailable", responder, false);
      else {
        element.detachEvent("ondataavailable", responder);
        element.detachEvent("onlosecapture", responder);
      }
    } else {
      var actualEventName = _getDOMEventName(eventName);
      if (element.removeEventListener)
        element.removeEventListener(actualEventName, responder, false);
      else
        element.detachEvent('on' + actualEventName, responder);
    }

    registry.set(eventName, responders.without(responder));

    return element;
  }

  function fire(element, eventName, memo, bubble) {
    element = $(element);

    if (Object.isUndefined(bubble))
      bubble = true;

    if (element == document && document.createEvent && !element.dispatchEvent)
      element = document.documentElement;

    var event;
    if (document.createEvent) {
      event = document.createEvent('HTMLEvents');
      event.initEvent('dataavailable', bubble, true);
    } else {
      event = document.createEventObject();
      event.eventType = bubble ? 'ondataavailable' : 'onlosecapture';
    }

    event.eventName = eventName;
    event.memo = memo || { };

    if (document.createEvent)
      element.dispatchEvent(event);
    else
      element.fireEvent(event.eventType, event);

    return Event.extend(event);
  }

  Event.Handler = Class.create({
    initialize: function(element, eventName, selector, callback) {
      this.element   = $(element);
      this.eventName = eventName;
      this.selector  = selector;
      this.callback  = callback;
      this.handler   = this.handleEvent.bind(this);
    },

    start: function() {
      Event.observe(this.element, this.eventName, this.handler);
      return this;
    },

    stop: function() {
      Event.stopObserving(this.element, this.eventName, this.handler);
      return this;
    },

    handleEvent: function(event) {
      var element = Event.findElement(event, this.selector);
      if (element) this.callback.call(this.element, event, element);
    }
  });

  function on(element, eventName, selector, callback) {
    element = $(element);
    if (Object.isFunction(selector) && Object.isUndefined(callback)) {
      callback = selector, selector = null;
    }

    return new Event.Handler(element, eventName, selector, callback).start();
  }

  Object.extend(Event, Event.Methods);

  Object.extend(Event, {
    fire:          fire,
    observe:       observe,
    stopObserving: stopObserving,
    on:            on
  });

  Element.addMethods({
    fire:          fire,

    observe:       observe,

    stopObserving: stopObserving,

    on:            on
  });

  Object.extend(document, {
    fire:          fire.methodize(),

    observe:       observe.methodize(),

    stopObserving: stopObserving.methodize(),

    on:            on.methodize(),

    loaded:        false
  });

  if (window.Event) Object.extend(window.Event, Event);
  else window.Event = Event;
})();

(function() {
  /* Support for the DOMContentLoaded event is based on work by Dan Webb,
     Matthias Miller, Dean Edwards, John Resig, and Diego Perini. */

  var timer;

  function fireContentLoadedEvent() {
    if (document.loaded) return;
    if (timer) window.clearTimeout(timer);
    document.loaded = true;
    document.fire('dom:loaded');
  }

  function checkReadyState() {
    if (document.readyState === 'complete') {
      document.stopObserving('readystatechange', checkReadyState);
      fireContentLoadedEvent();
    }
  }

  function pollDoScroll() {
    try { document.documentElement.doScroll('left'); }
    catch(e) {
      timer = pollDoScroll.defer();
      return;
    }
    fireContentLoadedEvent();
  }

  if (document.addEventListener) {
    document.addEventListener('DOMContentLoaded', fireContentLoadedEvent, false);
  } else {
    document.observe('readystatechange', checkReadyState);
    if (window == top)
      timer = pollDoScroll.defer();
  }

  Event.observe(window, 'load', fireContentLoadedEvent);
})();


Element.addMethods();
/*------------------------------- DEPRECATED -------------------------------*/

Hash.toQueryString = Object.toQueryString;

var Toggle = { display: Element.toggle };

Element.Methods.childOf = Element.Methods.descendantOf;

var Insertion = {
  Before: function(element, content) {
    return Element.insert(element, {before:content});
  },

  Top: function(element, content) {
    return Element.insert(element, {top:content});
  },

  Bottom: function(element, content) {
    return Element.insert(element, {bottom:content});
  },

  After: function(element, content) {
    return Element.insert(element, {after:content});
  }
};

var $continue = new Error('"throw $continue" is deprecated, use "return" instead');

var Position = {
  includeScrollOffsets: false,

  prepare: function() {
    this.deltaX =  window.pageXOffset
                || document.documentElement.scrollLeft
                || document.body.scrollLeft
                || 0;
    this.deltaY =  window.pageYOffset
                || document.documentElement.scrollTop
                || document.body.scrollTop
                || 0;
  },

  within: function(element, x, y) {
    if (this.includeScrollOffsets)
      return this.withinIncludingScrolloffsets(element, x, y);
    this.xcomp = x;
    this.ycomp = y;
    this.offset = Element.cumulativeOffset(element);

    return (y >= this.offset[1] &&
            y <  this.offset[1] + element.offsetHeight &&
            x >= this.offset[0] &&
            x <  this.offset[0] + element.offsetWidth);
  },

  withinIncludingScrolloffsets: function(element, x, y) {
    var offsetcache = Element.cumulativeScrollOffset(element);

    this.xcomp = x + offsetcache[0] - this.deltaX;
    this.ycomp = y + offsetcache[1] - this.deltaY;
    this.offset = Element.cumulativeOffset(element);

    return (this.ycomp >= this.offset[1] &&
            this.ycomp <  this.offset[1] + element.offsetHeight &&
            this.xcomp >= this.offset[0] &&
            this.xcomp <  this.offset[0] + element.offsetWidth);
  },

  overlap: function(mode, element) {
    if (!mode) return 0;
    if (mode == 'vertical')
      return ((this.offset[1] + element.offsetHeight) - this.ycomp) /
        element.offsetHeight;
    if (mode == 'horizontal')
      return ((this.offset[0] + element.offsetWidth) - this.xcomp) /
        element.offsetWidth;
  },


  cumulativeOffset: Element.Methods.cumulativeOffset,

  positionedOffset: Element.Methods.positionedOffset,

  absolutize: function(element) {
    Position.prepare();
    return Element.absolutize(element);
  },

  relativize: function(element) {
    Position.prepare();
    return Element.relativize(element);
  },

  realOffset: Element.Methods.cumulativeScrollOffset,

  offsetParent: Element.Methods.getOffsetParent,

  page: Element.Methods.viewportOffset,

  clone: function(source, target, options) {
    options = options || { };
    return Element.clonePosition(target, source, options);
  }
};

/*--------------------------------------------------------------------------*/

if (!document.getElementsByClassName) document.getElementsByClassName = function(instanceMethods){
  function iter(name) {
    return name.blank() ? null : "[contains(concat(' ', @class, ' '), ' " + name + " ')]";
  }

  instanceMethods.getElementsByClassName = Prototype.BrowserFeatures.XPath ?
  function(element, className) {
    className = className.toString().strip();
    var cond = /\s/.test(className) ? $w(className).map(iter).join('') : iter(className);
    return cond ? document._getElementsByXPath('.//*' + cond, element) : [];
  } : function(element, className) {
    className = className.toString().strip();
    var elements = [], classNames = (/\s/.test(className) ? $w(className) : null);
    if (!classNames && !className) return elements;

    var nodes = $(element).getElementsByTagName('*');
    className = ' ' + className + ' ';

    for (var i = 0, child, cn; child = nodes[i]; i++) {
      if (child.className && (cn = ' ' + child.className + ' ') && (cn.include(className) ||
          (classNames && classNames.all(function(name) {
            return !name.toString().blank() && cn.include(' ' + name + ' ');
          }))))
        elements.push(Element.extend(child));
    }
    return elements;
  };

  return function(className, parentElement) {
    return $(parentElement || document.body).getElementsByClassName(className);
  };
}(Element.Methods);

/*--------------------------------------------------------------------------*/

Element.ClassNames = Class.create();
Element.ClassNames.prototype = {
  initialize: function(element) {
    this.element = $(element);
  },

  _each: function(iterator) {
    this.element.className.split(/\s+/).select(function(name) {
      return name.length > 0;
    })._each(iterator);
  },

  set: function(className) {
    this.element.className = className;
  },

  add: function(classNameToAdd) {
    if (this.include(classNameToAdd)) return;
    this.set($A(this).concat(classNameToAdd).join(' '));
  },

  remove: function(classNameToRemove) {
    if (!this.include(classNameToRemove)) return;
    this.set($A(this).without(classNameToRemove).join(' '));
  },

  toString: function() {
    return $A(this).join(' ');
  }
};

Object.extend(Element.ClassNames.prototype, Enumerable);

/*--------------------------------------------------------------------------*/

(function() {
  window.Selector = Class.create({
    initialize: function(expression) {
      this.expression = expression.strip();
    },

    findElements: function(rootElement) {
      return Prototype.Selector.select(this.expression, rootElement);
    },

    match: function(element) {
      return Prototype.Selector.match(element, this.expression);
    },

    toString: function() {
      return this.expression;
    },

    inspect: function() {
      return "#<Selector: " + this.expression + ">";
    }
  });

  Object.extend(Selector, {
    matchElements: function(elements, expression) {
      var match = Prototype.Selector.match,
          results = [];

      for (var i = 0, length = elements.length; i < length; i++) {
        var element = elements[i];
        if (match(element, expression)) {
          results.push(Element.extend(element));
        }
      }
      return results;
    },

    findElement: function(elements, expression, index) {
      index = index || 0;
      var matchIndex = 0, element;
      for (var i = 0, length = elements.length; i < length; i++) {
        element = elements[i];
        if (Prototype.Selector.match(element, expression) && index === matchIndex++) {
          return Element.extend(element);
        }
      }
    },

    findChildElements: function(element, expressions) {
      var selector = expressions.toArray().join(', ');
      return Prototype.Selector.select(selector, element || document);
    }
  });
})();
/*!
 * jQuery JavaScript Library v3.6.0
 * https://jquery.com/
 *
 * Includes Sizzle.js
 * https://sizzlejs.com/
 *
 * Copyright OpenJS Foundation and other contributors
 * Released under the MIT license
 * https://jquery.org/license
 *
 * Date: 2021-03-02T17:08Z
 */(function(e,t){"use strict";typeof module=="object"&&typeof module.exports=="object"?module.exports=e.document?t(e,!0):function(e){if(!e.document)throw new Error("jQuery requires a window with a document");return t(e)}:t(e)})(typeof window!="undefined"?window:this,function(s,tD){"use strict";p=[],e5=Object.getPrototypeOf,v=p.slice,eg=p.flat?function(e){return p.flat.call(e)}:function(e){return p.concat.apply([],e)},Y=p.push,H=p.indexOf,I={},e6=I.toString,V=I.hasOwnProperty,ef=V.toString,td=ef.call(Object),i={},n=function(e){return typeof e=="function"&&typeof e.nodeType!="number"&&typeof e.item!="function"},_=function(e){return e!=null&&e===e.window},o=s.document,tl={type:!0,src:!0,nonce:!0,noModule:!0};function ta(a,e,t){t=t||o;var n,i,s=t.createElement("script");if(s.text=a,e)for(n in tl)i=e[n]||e.getAttribute&&e.getAttribute(n),i&&s.setAttribute(n,i);t.head.appendChild(s).parentNode.removeChild(s)}function k(e){return e==null?e+"":typeof e=="object"||typeof e=="function"?I[e6.call(e)]||"object":typeof e}eb="3.6.0",e=function(t,n){return new e.fn.init(t,n)},e.fn=e.prototype={jquery:eb,constructor:e,length:0,toArray:function(){return v.call(this)},get:function(e){return e==null?v.call(this):e<0?this[e+this.length]:this[e]},pushStack:function(n){var t=e.merge(this.constructor(),n);return t.prevObject=this,t},each:function(t){return e.each(this,t)},map:function(t){return this.pushStack(e.map(this,function(e,n){return t.call(e,n,e)}))},slice:function(){return this.pushStack(v.apply(this,arguments))},first:function(){return this.eq(0)},last:function(){return this.eq(-1)},even:function(){return this.pushStack(e.grep(this,function(t,e){return(e+1)%2}))},odd:function(){return this.pushStack(e.grep(this,function(t,e){return e%2}))},eq:function(t){var n=this.length,e=+t+(t<0?n:0);return this.pushStack(e>=0&&e<n?[this[e]]:[])},end:function(){return this.prevObject||this.constructor()},push:Y,sort:p.sort,splice:p.splice},e.extend=e.fn.extend=function(){var s,i,a,r,c,l,t=arguments[0]||{},o=1,u=arguments.length,d=!1;for(typeof t=="boolean"&&(d=t,t=arguments[o]||{},o++),typeof t!="object"&&!n(t)&&(t={}),o===u&&(t=this,o--);o<u;o++)if((l=arguments[o])!=null)for(i in l){if(s=l[i],i==="__proto__"||t===s)continue;d&&s&&(e.isPlainObject(s)||(r=Array.isArray(s)))?(a=t[i],r&&!Array.isArray(a)?c=[]:!r&&!e.isPlainObject(a)?c={}:c=a,r=!1,t[i]=e.extend(d,c,s)):s!==void 0&&(t[i]=s)}return t},e.extend({expando:"jQuery"+(eb+Math.random()).replace(/\D/g,""),isReady:!0,error:function(e){throw new Error(e)},noop:function(){},isPlainObject:function(t){var e,n;return!!t&&e6.call(t)==="[object Object]"&&(e=e5(t),!e||(n=V.call(e,"constructor")&&e.constructor,typeof n=="function"&&ef.call(n)===td))},isEmptyObject:function(e){var t;for(t in e)return!1;return!0},globalEval:function(t,e,n){ta(t,{nonce:e&&e.nonce},n)},each:function(t,n){var s,e=0;if(em(t)){for(s=t.length;e<s;e++)if(n.call(t[e],e,t[e])===!1)break}else for(e in t)if(n.call(t[e],e,t[e])===!1)break;return t},makeArray:function(t,s){var n=s||[];return t!=null&&(em(Object(t))?e.merge(n,typeof t=="string"?[t]:t):Y.call(n,t)),n},inArray:function(t,e,n){return e==null?-1:H.call(e,t,n)},merge:function(e,n){for(var o=+n.length,t=0,s=e.length;t<o;t++)e[s++]=n[t];return e.length=s,e},grep:function(t,o,i){for(var n,s=[],e=0,a=t.length,r=!i;e<a;e++)n=!o(t[e],e),n!==r&&s.push(t[e]);return s},map:function(n,o,i){var t,a,e=0,s=[];if(em(n))for(a=n.length;e<a;e++)t=o(n[e],e,i),t!=null&&s.push(t);else for(e in n)t=o(n[e],e,i),t!=null&&s.push(t);return eg(s)},guid:1,support:i}),typeof Symbol=="function"&&(e.fn[Symbol.iterator]=p[Symbol.iterator]),e.each("Boolean Number String Function Array Date RegExp Object Error Symbol".split(" "),function(t,e){I["[object "+e+"]"]=e.toLowerCase()});function em(e){var t=!!e&&"length"in e&&e.length,s=k(e);return!n(e)&&!_(e)&&(s==="array"||t===0||typeof t=="number"&&t>0&&t-1 in e)}b=function(P){var e,t,o,a,r,c,p,g,y,O,x,C,E,A,S,T,$,X,Q,i="sizzle"+1*new Date,d=P.document,h=0,eg=0,Z=N(),J=N(),ee=N(),F=N(),q=function(e,t){return e===t&&(x=!0),0},eh={}.hasOwnProperty,b=[],e3=b.pop,eu=b.push,v=b.push,ec=b.slice,_=function(t,n){for(var e=0,s=t.length;e<s;e++)if(t[e]===n)return e;return-1},H="checked|selected|async|autofocus|autoplay|controls|defer|disabled|hidden|ismap|loop|multiple|open|readonly|required|scoped",n="[\\x20\\t\\r\\n\\f]",j="(?:\\\\[\\da-fA-F]{1,6}"+n+"?|\\\\[^\\r\\n\\f]|[\\w-]|[^\0-\\x7f])+",G="\\["+n+"*("+j+")(?:"+n+"*([*^$|!~]?=)"+n+"*(?:'((?:\\\\.|[^\\\\'])*)'|\"((?:\\\\.|[^\\\\\"])*)\"|("+j+"))|)"+n+"*\\]",Y=":("+j+")(?:\\(("+"('((?:\\\\.|[^\\\\'])*)'|\"((?:\\\\.|[^\\\\\"])*)\")|"+"((?:\\\\.|[^\\\\()[\\]]|"+G+")*)|"+".*"+")\\)|)",ed=new RegExp(n+"+","g"),M=new RegExp("^"+n+"+|((?:^|[^\\\\])(?:\\\\.)*)"+n+"+$","g"),e1=new RegExp("^"+n+"*,"+n+"*"),en=new RegExp("^"+n+"*([>+~]|"+n+")"+n+"*"),em=new RegExp(n+"|>"),e8=new RegExp(Y),e6=new RegExp("^"+j+"$"),L={ID:new RegExp("^#("+j+")"),CLASS:new RegExp("^\\.("+j+")"),TAG:new RegExp("^("+j+"|[*])"),ATTR:new RegExp("^"+G),PSEUDO:new RegExp("^"+Y),CHILD:new RegExp("^:(only|first|last|nth|nth-last)-(child|of-type)(?:\\("+n+"*(even|odd|(([+-]|)(\\d*)n|)"+n+"*(?:([+-]|)"+n+"*(\\d+)|))"+n+"*\\)|)","i"),bool:new RegExp("^(?:"+H+")$","i"),needsContext:new RegExp("^"+n+"*[>+~]|:(even|odd|eq|gt|lt|nth|first|last)(?:\\("+n+"*((?:-\\d)?\\d*)"+n+"*\\)|)(?=[^-]|$)","i")},el=/HTML$/i,e7=/^(?:input|select|textarea|button)$/i,e5=/^h\d$/i,k=/^[^{]+\{\s*\[native \w/,ef=/^(?:#([\w-]+)|(\w+)|\.([\w-]+))$/,K=/[+~]/,f=new RegExp("\\\\[\\da-fA-F]{1,6}"+n+"?|\\\\([^\\r\\n\\f])","g"),m=function(n,t){var e="0x"+n.slice(1)-65536;return t?t:e<0?String.fromCharCode(e+65536):String.fromCharCode(e>>10|55296,e&1023|56320)},es=/([\0-\x1f\x7f]|^-?\d)|^-$|[^\0-\x1f\x7f-\uFFFF\w-]/g,eo=function(e,t){return t?e==="\0"?"\uFFFD":e.slice(0,-1)+"\\"+e.charCodeAt(e.length-1).toString(16)+" ":"\\"+e},ei=function(){p()},e2=D(function(e){return e.disabled===!0&&e.nodeName.toLowerCase()==="fieldset"},{dir:"parentNode",next:"legend"});try{v.apply(b=ec.call(d.childNodes),d.childNodes),b[d.childNodes.length].nodeType}catch(e){v={apply:b.length?function(e,t){eu.apply(e,ec.call(t))}:function(e,n){for(var t=e.length,s=0;e[t++]=n[s++];);e.length=t-1}}}function s(s,e,n,j){var r,l,d,m,f,g,b,h=e&&e.ownerDocument,u=e?e.nodeType:9;if(n=n||[],typeof s!="string"||!s||u!==1&&u!==9&&u!==11)return n;if(!j&&(p(e),e=e||t,c)){if(u!==11&&(f=ef.exec(s)))if(l=f[1]){if(u===9)if(d=e.getElementById(l)){if(d.id===l)return n.push(d),n}else return n;else if(h&&(d=h.getElementById(l))&&E(e,d)&&d.id===l)return n.push(d),n}else if(f[2])return v.apply(n,e.getElementsByTagName(s)),n;else if((l=f[3])&&o.getElementsByClassName&&e.getElementsByClassName)return v.apply(n,e.getElementsByClassName(l)),n;if(o.qsa&&!F[s+" "]&&(!a||!a.test(s))&&(u!==1||e.nodeName.toLowerCase()!=="object")){if(b=s,h=e,u===1&&(em.test(s)||en.test(s))){for(h=K.test(s)&&U(e.parentNode)||e,(h!==e||!o.scope)&&((r=e.getAttribute("id"))?r=r.replace(es,eo):e.setAttribute("id",r=i)),m=C(s),g=m.length;g--;)m[g]=(r?"#"+r:":scope")+" "+z(m[g]);b=m.join(",")}try{return v.apply(n,h.querySelectorAll(b)),n}catch(e){F(s,!0)}finally{r===i&&e.removeAttribute("id")}}}return Q(s.replace(M,"$1"),e,n,j)}function N(){var n=[];function t(s,o){return n.push(s+" ")>e.cacheLength&&delete t[n.shift()],t[s+" "]=o}return t}function u(e){return e[i]=!0,e}function l(n){var e=t.createElement("fieldset");try{return!!n(e)}catch(e){return!1}finally{e.parentNode&&e.parentNode.removeChild(e),e=null}}function I(s,o){for(var t=s.split("|"),n=t.length;n--;)e.attrHandle[t[n]]=o}function er(t,n){var e=n&&t,s=e&&t.nodeType===1&&n.nodeType===1&&t.sourceIndex-n.sourceIndex;if(s)return s;if(e)for(;e=e.nextSibling;)if(e===n)return-1;return t?1:-1}function e4(e){return function(t){var n=t.nodeName.toLowerCase();return n==="input"&&t.type===e}}function e0(e){return function(t){var n=t.nodeName.toLowerCase();return(n==="input"||n==="button")&&t.type===e}}function ea(e){return function(t){return"form"in t?t.parentNode&&t.disabled===!1?"label"in t?"label"in t.parentNode?t.parentNode.disabled===e:t.disabled===e:t.isDisabled===e||t.isDisabled!==!e&&e2(t)===e:t.disabled===e:"label"in t&&t.disabled===e}}function w(e){return u(function(t){return t=+t,u(function(n,a){for(var s,o=e([],n.length,t),i=o.length;i--;)n[s=o[i]]&&(n[s]=!(a[s]=n[s]))})})}function U(e){return e&&typeof e.getElementsByTagName!="undefined"&&e}o=s.support={},X=s.isXML=function(e){var n=e&&e.namespaceURI,t=e&&(e.ownerDocument||e).documentElement;return!el.test(n||t&&t.nodeName||"HTML")},p=s.setDocument=function(h){var s,p,u=h?h.ownerDocument||h:d;return u==t||u.nodeType!==9||!u.documentElement?t:(t=u,r=t.documentElement,c=!X(t),d!=t&&(s=t.defaultView)&&s.top!==s&&(s.addEventListener?s.addEventListener("unload",ei,!1):s.attachEvent&&s.attachEvent("onunload",ei)),o.scope=l(function(e){return r.appendChild(e).appendChild(t.createElement("div")),typeof e.querySelectorAll!="undefined"&&!e.querySelectorAll(":scope fieldset div").length}),o.attributes=l(function(e){return e.className="i",!e.getAttribute("className")}),o.getElementsByTagName=l(function(e){return e.appendChild(t.createComment("")),!e.getElementsByTagName("*").length}),o.getElementsByClassName=k.test(t.getElementsByClassName),o.getById=l(function(e){return r.appendChild(e).id=i,!t.getElementsByName||!t.getElementsByName(i).length}),o.getById?(e.filter.ID=function(e){var t=e.replace(f,m);return function(e){return e.getAttribute("id")===t}},e.find.ID=function(n,e){if(typeof e.getElementById!="undefined"&&c){var t=e.getElementById(n);return t?[t]:[]}}):(e.filter.ID=function(e){var t=e.replace(f,m);return function(e){var n=typeof e.getAttributeNode!="undefined"&&e.getAttributeNode("id");return n&&n.value===t}},e.find.ID=function(n,s){if(typeof s.getElementById!="undefined"&&c){var e,o,i,t=s.getElementById(n);if(t){if(e=t.getAttributeNode("id"),e&&e.value===n)return[t];for(i=s.getElementsByName(n),o=0;t=i[o++];)if(e=t.getAttributeNode("id"),e&&e.value===n)return[t]}return[]}}),e.find.TAG=o.getElementsByTagName?function(t,e){if(typeof e.getElementsByTagName!="undefined")return e.getElementsByTagName(t);if(o.qsa)return e.querySelectorAll(t)}:function(t,o){var e,n=[],i=0,s=o.getElementsByTagName(t);if(t==="*"){for(;e=s[i++];)e.nodeType===1&&n.push(e);return n}return s},e.find.CLASS=o.getElementsByClassName&&function(t,e){if(typeof e.getElementsByClassName!="undefined"&&c)return e.getElementsByClassName(t)},y=[],a=[],(o.qsa=k.test(t.querySelectorAll))&&(l(function(e){var s;r.appendChild(e).innerHTML="<a id='"+i+"'></a>"+"<select id='"+i+"-\r\\' msallowcapture=''>"+"<option selected=''></option></select>",e.querySelectorAll("[msallowcapture^='']").length&&a.push("[*^$]="+n+"*(?:''|\"\")"),e.querySelectorAll("[selected]").length||a.push("\\["+n+"*(?:value|"+H+")"),e.querySelectorAll("[id~="+i+"-]").length||a.push("~="),s=t.createElement("input"),s.setAttribute("name",""),e.appendChild(s),e.querySelectorAll("[name='']").length||a.push("\\["+n+"*name"+n+"*="+n+"*(?:''|\"\")"),e.querySelectorAll(":checked").length||a.push(":checked"),e.querySelectorAll("a#"+i+"+*").length||a.push(".#.+[+~]"),e.querySelectorAll("\\"),a.push("[\\r\\n\\f]")}),l(function(e){e.innerHTML="<a href='' disabled='disabled'></a><select disabled='disabled'><option/></select>";var s=t.createElement("input");s.setAttribute("type","hidden"),e.appendChild(s).setAttribute("name","D"),e.querySelectorAll("[name=d]").length&&a.push("name"+n+"*[*^$|!~]?="),e.querySelectorAll(":enabled").length!==2&&a.push(":enabled",":disabled"),r.appendChild(e).disabled=!0,e.querySelectorAll(":disabled").length!==2&&a.push(":enabled",":disabled"),e.querySelectorAll("*,:x"),a.push(",.*:")})),(o.matchesSelector=k.test(A=r.matches||r.webkitMatchesSelector||r.mozMatchesSelector||r.oMatchesSelector||r.msMatchesSelector))&&l(function(e){o.disconnectedMatch=A.call(e,"*"),A.call(e,"[s!='']:x"),y.push("!=",Y)}),a=a.length&&new RegExp(a.join("|")),y=y.length&&new RegExp(y.join("|")),p=k.test(r.compareDocumentPosition),E=p||k.test(r.contains)?function(e,n){var s=e.nodeType===9?e.documentElement:e,t=n&&n.parentNode;return e===t||!!(t&&t.nodeType===1&&(s.contains?s.contains(t):e.compareDocumentPosition&&e.compareDocumentPosition(t)&16))}:function(t,e){if(e)for(;e=e.parentNode;)if(e===t)return!0;return!1},q=p?function(e,n){if(e===n)return x=!0,0;var s=!e.compareDocumentPosition-!n.compareDocumentPosition;return s||(s=(e.ownerDocument||e)==(n.ownerDocument||n)?e.compareDocumentPosition(n):1,s&1||!o.sortDetached&&n.compareDocumentPosition(e)===s?e==t||e.ownerDocument==d&&E(d,e)?-1:n==t||n.ownerDocument==d&&E(d,n)?1:g?_(g,e)-_(g,n):0:s&4?-1:1)}:function(s,o){if(s===o)return x=!0,0;var e,n=0,r=s.parentNode,c=o.parentNode,i=[s],a=[o];if(!r||!c)return s==t?-1:o==t?1:r?-1:c?1:g?_(g,s)-_(g,o):0;if(r===c)return er(s,o);for(e=s;e=e.parentNode;)i.unshift(e);for(e=o;e=e.parentNode;)a.unshift(e);for(;i[n]===a[n];)n++;return n?er(i[n],a[n]):i[n]==d?-1:a[n]==d?1:0},t)},s.matches=function(e,t){return s(e,null,null,t)},s.matchesSelector=function(n,e){if(p(n),o.matchesSelector&&c&&!F[e+" "]&&(!y||!y.test(e))&&(!a||!a.test(e)))try{var i=A.call(n,e);if(i||o.disconnectedMatch||n.document&&n.document.nodeType!==11)return i}catch(t){F(e,!0)}return s(e,t,null,[n]).length>0},s.contains=function(e,n){return(e.ownerDocument||e)!=t&&p(e),E(e,n)},s.attr=function(n,s){(n.ownerDocument||n)!=t&&p(n);var a=e.attrHandle[s.toLowerCase()],i=a&&eh.call(e.attrHandle,s.toLowerCase())?a(n,s,!c):void 0;return i!==void 0?i:o.attributes||!c?n.getAttribute(s):(i=n.getAttributeNode(s))&&i.specified?i.value:null},s.escape=function(e){return(e+"").replace(es,eo)},s.error=function(e){throw new Error("Syntax error, unrecognized expression: "+e)},s.uniqueSort=function(e){var s,i=[],t=0,n=0;if(x=!o.detectDuplicates,g=!o.sortStable&&e.slice(0),e.sort(q),x){for(;s=e[n++];)s===e[n]&&(t=i.push(n));for(;t--;)e.splice(i[t],1)}return g=null,e},S=s.getText=function(e){var s,n="",o=0,t=e.nodeType;if(t){if(t===1||t===9||t===11){if(typeof e.textContent=="string")return e.textContent;for(e=e.firstChild;e;e=e.nextSibling)n+=S(e)}else if(t===3||t===4)return e.nodeValue}else for(;s=e[o++];)n+=S(s);return n},e=s.selectors={cacheLength:50,createPseudo:u,match:L,attrHandle:{},find:{},relative:{">":{dir:"parentNode",first:!0}," ":{dir:"parentNode"},"+":{dir:"previousSibling",first:!0},"~":{dir:"previousSibling"}},preFilter:{ATTR:function(e){return e[1]=e[1].replace(f,m),e[3]=(e[3]||e[4]||e[5]||"").replace(f,m),e[2]==="~="&&(e[3]=" "+e[3]+" "),e.slice(0,4)},CHILD:function(e){return e[1]=e[1].toLowerCase(),e[1].slice(0,3)==="nth"?(e[3]||s.error(e[0]),e[4]=+(e[4]?e[5]+(e[6]||1):2*(e[3]==="even"||e[3]==="odd")),e[5]=+(e[7]+e[8]||e[3]==="odd")):e[3]&&s.error(e[0]),e},PSEUDO:function(e){var n,t=!e[6]&&e[2];return L.CHILD.test(e[0])?null:(e[3]?e[2]=e[4]||e[5]||"":t&&e8.test(t)&&(n=C(t,!0))&&(n=t.indexOf(")",t.length-n)-t.length)&&(e[0]=e[0].slice(0,n),e[2]=t.slice(0,n)),e.slice(0,3))}},filter:{TAG:function(e){var t=e.replace(f,m).toLowerCase();return e==="*"?function(){return!0}:function(e){return e.nodeName&&e.nodeName.toLowerCase()===t}},CLASS:function(e){var t=Z[e+" "];return t||(t=new RegExp("(^|"+n+")"+e+"("+n+"|$)"))&&Z(e,function(e){return t.test(typeof e.className=="string"&&e.className||typeof e.getAttribute!="undefined"&&e.getAttribute("class")||"")})},ATTR:function(n,t,e){return function(i){var o=s.attr(i,n);return o==null?t==="!=":!t||(o+="",t==="="?o===e:t==="!="?o!==e:t==="^="?e&&o.indexOf(e)===0:t==="*="?e&&o.indexOf(e)>-1:t==="$="?e&&o.slice(-e.length)===e:t==="~="?(" "+o.replace(ed," ")+" ").indexOf(e)>-1:t==="|="&&(o===e||o.slice(0,e.length+1)===e+"-"))}},CHILD:function(e,r,c,n,o){var a=e.slice(0,3)!=="nth",s=e.slice(-4)!=="last",t=r==="of-type";return n===1&&o===0?function(e){return!!e.parentNode}:function(f,_,y){var r,l,d,u,m,p,g=a!==s?"nextSibling":"previousSibling",v=f.parentNode,j=t&&f.nodeName.toLowerCase(),b=!y&&!t,c=!1;if(v){if(a){for(;g;){for(r=f;r=r[g];)if(t?r.nodeName.toLowerCase()===j:r.nodeType===1)return!1;p=g=e==="only"&&!p&&"nextSibling"}return!0}if(p=[s?v.firstChild:v.lastChild],s&&b){for(r=v,d=r[i]||(r[i]={}),m=d[r.uniqueID]||(d[r.uniqueID]={}),u=m[e]||[],l=u[0]===h&&u[1],c=l&&u[2],r=l&&v.childNodes[l];r=++l&&r&&r[g]||(c=l=0)||p.pop();)if(r.nodeType===1&&++c&&r===f){m[e]=[h,l,c];break}}else if(b&&(r=f,d=r[i]||(r[i]={}),m=d[r.uniqueID]||(d[r.uniqueID]={}),u=m[e]||[],l=u[0]===h&&u[1],c=l),c===!1)for(;r=++l&&r&&r[g]||(c=l=0)||p.pop();)if((t?r.nodeName.toLowerCase()===j:r.nodeType===1)&&++c&&(b&&(d=r[i]||(r[i]={}),m=d[r.uniqueID]||(d[r.uniqueID]={}),m[e]=[h,c]),r===f))break;return c-=o,c===n||c%n===0&&c/n>=0}}},PSEUDO:function(t,o){var a,n=e.pseudos[t]||e.setFilters[t.toLowerCase()]||s.error("unsupported pseudo: "+t);return n[i]?n(o):n.length>1?(a=[t,t,"",o],e.setFilters.hasOwnProperty(t.toLowerCase())?u(function(e,a){for(var t,s=n(e,o),i=s.length;i--;)t=_(e,s[i]),e[t]=!(a[t]=s[i])}):function(e){return n(e,0,a)}):n}},pseudos:{not:u(function(s){var e=[],n=[],t=$(s.replace(M,"$1"));return t[i]?u(function(n,o,r,i){for(var s,a=t(n,null,i,[]),e=n.length;e--;)(s=a[e])&&(n[e]=!(o[e]=s))}):function(s,i,o){return e[0]=s,t(e,null,o,n),e[0]=null,!n.pop()}}),has:u(function(e){return function(t){return s(e,t).length>0}}),contains:u(function(e){return e=e.replace(f,m),function(t){return(t.textContent||S(t)).indexOf(e)>-1}}),lang:u(function(e){return e6.test(e||"")||s.error("unsupported lang: "+e),e=e.replace(f,m).toLowerCase(),function(t){var n;do if(n=c?t.lang:t.getAttribute("xml:lang")||t.getAttribute("lang"))return n=n.toLowerCase(),n===e||n.indexOf(e+"-")===0;while((t=t.parentNode)&&t.nodeType===1)return!1}}),target:function(t){var e=P.location&&P.location.hash;return e&&e.slice(1)===t.id},root:function(e){return e===r},focus:function(e){return e===t.activeElement&&(!t.hasFocus||t.hasFocus())&&!!(e.type||e.href||~e.tabIndex)},enabled:ea(!1),disabled:ea(!0),checked:function(e){var t=e.nodeName.toLowerCase();return t==="input"&&!!e.checked||t==="option"&&!!e.selected},selected:function(e){return e.parentNode&&e.parentNode.selectedIndex,e.selected===!0},empty:function(e){for(e=e.firstChild;e;e=e.nextSibling)if(e.nodeType<6)return!1;return!0},parent:function(t){return!e.pseudos.empty(t)},header:function(e){return e5.test(e.nodeName)},input:function(e){return e7.test(e.nodeName)},button:function(e){var t=e.nodeName.toLowerCase();return t==="input"&&e.type==="button"||t==="button"},text:function(e){var t;return e.nodeName.toLowerCase()==="input"&&e.type==="text"&&((t=e.getAttribute("type"))==null||t.toLowerCase()==="text")},first:w(function(){return[0]}),last:w(function(t,e){return[e-1]}),eq:w(function(n,t,e){return[e<0?e+t:e]}),even:w(function(t,n){for(var e=0;e<n;e+=2)t.push(e);return t}),odd:w(function(t,n){for(var e=1;e<n;e+=2)t.push(e);return t}),lt:w(function(n,t,e){for(var s=e<0?e+t:e>t?t:e;--s>=0;)n.push(s);return n}),gt:w(function(t,n,e){for(var s=e<0?e+n:e;++s<n;)t.push(s);return t})}},e.pseudos.nth=e.pseudos.eq;for(O in{radio:!0,checkbox:!0,file:!0,password:!0,image:!0})e.pseudos[O]=e4(O);for(O in{submit:!0,reset:!0})e.pseudos[O]=e0(O);function et(){}et.prototype=e.filters=e.pseudos,e.setFilters=new et,C=s.tokenize=function(a,d){var t,n,o,i,r,c,l,u=J[a+" "];if(u)return d?0:u.slice(0);for(t=a,r=[],l=e.preFilter;t;){(!o||(n=e1.exec(t)))&&(n&&(t=t.slice(n[0].length)||t),r.push(c=[])),o=!1,(n=en.exec(t))&&(o=n.shift(),c.push({value:o,type:n[0].replace(M," ")}),t=t.slice(o.length));for(i in e.filter)(n=L[i].exec(t))&&(!l[i]||(n=l[i](n)))&&(o=n.shift(),c.push({value:o,type:i,matches:n}),t=t.slice(o.length));if(!o)break}return d?t.length:t?s.error(a):J(a,r).slice(0)};function z(t){for(var e=0,s=t.length,n="";e<s;e++)n+=t[e].value;return n}function D(t,n,c){var e=n.dir,s=n.next,o=s||e,a=c&&o==="parentNode",r=eg++;return n.first?function(n,s,o){for(;n=n[e];)if(n.nodeType===1||a)return t(n,s,o);return!1}:function(n,f,l){var c,d,u,m=[h,r];if(l){for(;n=n[e];)if((n.nodeType===1||a)&&t(n,f,l))return!0}else for(;n=n[e];)if(n.nodeType===1||a)if(u=n[i]||(n[i]={}),d=u[n.uniqueID]||(u[n.uniqueID]={}),s&&s===n.nodeName.toLowerCase())n=n[e]||n;else if((c=d[o])&&c[0]===h&&c[1]===r)return m[2]=c[2];else if(d[o]=m,m[2]=t(n,f,l))return!0;return!1}}function V(e){return e.length>1?function(n,s,o){for(var t=e.length;t--;)if(!e[t](n,s,o))return!1;return!0}:e[0]}function ep(o,t,n){for(var e=0,i=t.length;e<i;e++)s(o,t[e],n);return n}function R(n,i,s,a,r){for(var t,o=[],e=0,c=n.length,l=i!=null;e<c;e++)(t=n[e])&&(!s||s(t,a,r))&&(o.push(t),l&&i.push(e));return o}function B(n,o,s,t,e,a){return t&&!t[i]&&(t=B(t)),e&&!e[i]&&(e=B(e,a)),u(function(u,d,l,h){var a,r,c,p=[],f=[],g=d.length,b=u||ep(o||"*",l.nodeType?[l]:l,[]),m=n&&(u||!o)?R(b,p,n,l,h):b,i=s?e||(u?n:g||t)?[]:d:m;if(s&&s(m,i,l,h),t)for(r=R(i,f),t(r,[],l,h),a=r.length;a--;)(c=r[a])&&(i[f[a]]=!(m[f[a]]=c));if(u){if(e||n){if(e){for(r=[],a=i.length;a--;)(c=i[a])&&r.push(m[a]=c);e(null,i=[],r,h)}for(a=i.length;a--;)(c=i[a])&&(r=e?_(u,c):p[a])>-1&&(u[r]=!(d[r]=c))}}else i=R(i===d?i.splice(g,i.length):i),e?e(null,d,i,h):v.apply(d,i)})}function W(t){for(var s,o,r,c=t.length,l=e.relative[t[0].type],d=l||e.relative[" "],n=l?1:0,u=D(function(e){return e===r},d,!0),h=D(function(e){return _(r,e)>-1},d,!0),a=[function(n,e,t){var s=!l&&(t||e!==T)||((r=e).nodeType?u(n,e,t):h(n,e,t));return r=null,s}];n<c;n++)if(o=e.relative[t[n].type])a=[D(V(a),o)];else{if(o=e.filter[t[n].type].apply(null,t[n].matches),o[i]){for(s=++n;s<c;s++)if(e.relative[t[s].type])break;return B(n>1&&V(a),n>1&&z(t.slice(0,n-1).concat({value:t[n-2].type===" "?"*":""})).replace(M,"$1"),o,n<s&&W(t.slice(n,s)),s<c&&W(t=t.slice(s)),s<c&&z(t))}a.push(o)}return V(a)}function ev(i,n){var o=n.length>0,a=i.length>0,r=function(b,f,w,y,u){var l,m,j,g=0,r="0",_=b&&[],d=[],O=T,x=b||a&&e.find.TAG("*",u),C=h+=O==null?1:Math.random()||.1,E=x.length;for(u&&(T=f==t||f||u);r!==E&&(l=x[r])!=null;r++){if(a&&l){for(j=0,!f&&l.ownerDocument!=t&&(p(l),w=!c);m=i[j++];)if(m(l,f||t,w)){y.push(l);break}u&&(h=C)}o&&((l=!m&&l)&&g--,b&&_.push(l))}if(g+=r,o&&r!==g){for(j=0;m=n[j++];)m(_,d,f,w);if(b){if(g>0)for(;r--;)_[r]||d[r]||(d[r]=e3.call(y));d=R(d)}v.apply(y,d),u&&!b&&d.length>0&&g+n.length>1&&s.uniqueSort(y)}return u&&(h=C,T=O),_};return o?u(r):r}return $=s.compile=function(t,n){var s,o=[],a=[],e=ee[t+" "];if(!e){for(n||(n=C(t)),s=n.length;s--;)e=W(n[s]),e[i]?o.push(e):a.push(e);e=ee(t,ev(a,o)),e.selector=t}return e},Q=s.select=function(n,t,o,i){var s,a,r,u,h,d=typeof n=="function"&&n,l=!i&&C(n=d.selector||n);if(o=o||[],l.length===1){if(s=l[0]=l[0].slice(0),s.length>2&&(a=s[0]).type==="ID"&&t.nodeType===9&&c&&e.relative[s[1].type]){if(t=(e.find.ID(a.matches[0].replace(f,m),t)||[])[0],!t)return o;d&&(t=t.parentNode),n=n.slice(s.shift().value.length)}for(r=L.needsContext.test(n)?0:s.length;r--;){if(a=s[r],e.relative[u=a.type])break;if((h=e.find[u])&&(i=h(a.matches[0].replace(f,m),K.test(s[0].type)&&U(t.parentNode)||t))){if(s.splice(r,1),n=i.length&&z(s),!n)return v.apply(o,i),o;break}}}return(d||$(n,l))(i,t,!c,o,!t||K.test(n)&&U(t.parentNode)||t),o},o.sortStable=i.split("").sort(q).join("")===i,o.detectDuplicates=!!x,p(),o.sortDetached=l(function(e){return e.compareDocumentPosition(t.createElement("fieldset"))&1}),l(function(e){return e.innerHTML="<a href='#'></a>",e.firstChild.getAttribute("href")==="#"})||I("type|href|height|width",function(t,e,n){if(!n)return t.getAttribute(e,e.toLowerCase()==="type"?1:2)}),(!o.attributes||!l(function(e){return e.innerHTML="<input/>",e.firstChild.setAttribute("value",""),e.firstChild.getAttribute("value")===""}))&&I("value",function(e,n,t){if(!t&&e.nodeName.toLowerCase()==="input")return e.defaultValue}),l(function(e){return e.getAttribute("disabled")==null})||I(H,function(n,e,s){var t;if(!s)return n[e]===!0?e.toLowerCase():(t=n.getAttributeNode(e))&&t.specified?t.value:null}),s}(s),e.find=b,e.expr=b.selectors,e.expr[":"]=e.expr.pseudos,e.uniqueSort=e.unique=b.uniqueSort,e.text=b.getText,e.isXMLDoc=b.isXML,e.contains=b.contains,e.escapeSelector=b.escape,O=function(t,o,n){for(var s=[],i=n!==void 0;(t=t[o])&&t.nodeType!==9;)if(t.nodeType===1){if(i&&e(t).is(n))break;s.push(t)}return s},eh=function(e,n){for(var t=[];e;e=e.nextSibling)e.nodeType===1&&e!==n&&t.push(e);return t},e0=e.expr.match.needsContext;function d(e,t){return e.nodeName&&e.nodeName.toLowerCase()===t.toLowerCase()}e4=/^<([a-z][^/\0>:\x20\t\r\n\f]*)[\x20\t\r\n\f]*\/?>(?:<\/\1>|)$/i;function e1(s,t,o){return n(t)?e.grep(s,function(e,n){return!!t.call(e,n,e)!==o}):t.nodeType?e.grep(s,function(e){return e===t!==o}):typeof t!="string"?e.grep(s,function(e){return H.call(t,e)>-1!==o}):e.filter(t,s,o)}e.filter=function(t,n,o){var s=n[0];return o&&(t=":not("+t+")"),n.length===1&&s.nodeType===1?e.find.matchesSelector(s,t)?[s]:[]:e.find.matches(t,e.grep(n,function(e){return e.nodeType===1}))},e.fn.extend({find:function(s){var t,n,o=this.length,i=this;if(typeof s!="string")return this.pushStack(e(s).filter(function(){for(t=0;t<o;t++)if(e.contains(i[t],this))return!0}));n=this.pushStack([]);for(t=0;t<o;t++)e.find(s,i[t],n);return o>1?e.uniqueSort(n):n},filter:function(e){return this.pushStack(e1(this,e||[],!1))},not:function(e){return this.pushStack(e1(this,e||[],!0))},is:function(t){return!!e1(this,typeof t=="string"&&e0.test(t)?e(t):t||[],!1).length}}),to=/^(?:\s*(<[\w\W]+>)[^>]*|#([\w-]+))$/,ts=e.fn.init=function(s,t,a){var i,r;if(!s)return this;if(a=a||ti,typeof s=="string")if(s[0]==="<"&&s[s.length-1]===">"&&s.length>=3?i=[null,s,null]:i=to.exec(s),i&&(i[1]||!t)){if(i[1]){if(t=t instanceof e?t[0]:t,e.merge(this,e.parseHTML(i[1],t&&t.nodeType?t.ownerDocument||t:o,!0)),e4.test(i[1])&&e.isPlainObject(t))for(i in t)n(this[i])?this[i](t[i]):this.attr(i,t[i]);return this}return r=o.getElementById(i[2]),r&&(this[0]=r,this.length=1),this}else return!t||t.jquery?(t||a).find(s):this.constructor(t).find(s);else if(s.nodeType)return this[0]=s,this.length=1,this;else if(n(s))return a.ready!==void 0?a.ready(s):s(e);return e.makeArray(s,this)},ts.prototype=e.fn,ti=e(o),tn=/^(?:parents|prev(?:Until|All))/,te={children:!0,contents:!0,next:!0,prev:!0},e.fn.extend({has:function(n){var t=e(n,this),s=t.length;return this.filter(function(){for(var n=0;n<s;n++)if(e.contains(this,t[n]))return!0})},closest:function(n,a){var t,o=0,r=this.length,s=[],i=typeof n!="string"&&e(n);if(!e0.test(n))for(;o<r;o++)for(t=this[o];t&&t!==a;t=t.parentNode)if(t.nodeType<11&&(i?i.index(t)>-1:t.nodeType===1&&e.find.matchesSelector(t,n))){s.push(t);break}return this.pushStack(s.length>1?e.uniqueSort(s):s)},index:function(t){return t?typeof t=="string"?H.call(e(t),this[0]):H.call(this,t.jquery?t[0]:t):this[0]&&this[0].parentNode?this.first().prevAll().length:-1},add:function(t,n){return this.pushStack(e.uniqueSort(e.merge(this.get(),e(t,n))))},addBack:function(e){return this.add(e==null?this.prevObject:this.prevObject.filter(e))}});function eZ(e,t){for(;(e=e[t])&&e.nodeType!==1;);return e}e.each({parent:function(t){var e=t.parentNode;return e&&e.nodeType!==11?e:null},parents:function(e){return O(e,"parentNode")},parentsUntil:function(e,n,t){return O(e,"parentNode",t)},next:function(e){return eZ(e,"nextSibling")},prev:function(e){return eZ(e,"previousSibling")},nextAll:function(e){return O(e,"nextSibling")},prevAll:function(e){return O(e,"previousSibling")},nextUntil:function(e,n,t){return O(e,"nextSibling",t)},prevUntil:function(e,n,t){return O(e,"previousSibling",t)},siblings:function(e){return eh((e.parentNode||{}).firstChild,e)},children:function(e){return eh(e.firstChild)},contents:function(t){return t.contentDocument!=null&&e5(t.contentDocument)?t.contentDocument:(d(t,"template")&&(t=t.content||t),e.merge([],t.childNodes))}},function(t,n){e.fn[t]=function(i,o){var s=e.map(this,n,i);return t.slice(-5)!=="Until"&&(o=i),o&&typeof o=="string"&&(s=e.filter(o,s)),this.length>1&&(te[t]||e.uniqueSort(s),tn.test(t)&&s.reverse()),this.pushStack(s)}}),h=/[^\x20\t\r\n\f]+/g;function tz(n){var t={};return e.each(n.match(h)||[],function(n,e){t[e]=!0}),t}e.Callbacks=function(o){o=typeof o=="string"?tz(o):e.extend({},o);var s,a,c,u,t=[],r=[],i=-1,d=function(){for(a=a||o.once,u=c=!0;r.length;i=-1)for(s=r.shift();++i<t.length;)t[i].apply(s[0],s[1])===!1&&o.stopOnFalse&&(i=t.length,s=!1);o.memory||(s=!1),c=!1,a&&(s?t=[]:t="")},l={add:function(){return t&&(s&&!c&&(i=t.length-1,r.push(s)),function s(i){e.each(i,function(i,e){n(e)?(!o.unique||!l.has(e))&&t.push(e):e&&e.length&&k(e)!=="string"&&s(e)})}(arguments),s&&!c&&d()),this},remove:function(){return e.each(arguments,function(o,s){for(var n;(n=e.inArray(s,t,n))>-1;)t.splice(n,1),n<=i&&i--}),this},has:function(n){return n?e.inArray(n,t)>-1:t.length>0},empty:function(){return t&&(t=[]),this},disable:function(){return a=r=[],t=s="",this},disabled:function(){return!t},lock:function(){return a=r=[],!s&&!c&&(t=s=""),this},locked:function(){return!!a},fireWith:function(t,e){return a||(e=e||[],e=[t,e.slice?e.slice():e],r.push(e),c||d()),this},fire:function(){return l.fireWith(this,arguments),this},fired:function(){return!!u}};return l};function N(e){return e}function Z(e){throw e}function eG(e,s,o,i){var t;try{e&&n(t=e.promise)?t.call(e).done(s).fail(o):e&&n(t=e.then)?t.call(e,s,o):s.apply(void 0,[e].slice(i))}catch(e){o.apply(void 0,[e])}}e.extend({Deferred:function(a){var o=[["notify","progress",e.Callbacks("memory"),e.Callbacks("memory"),2],["resolve","done",e.Callbacks("once memory"),e.Callbacks("once memory"),0,"resolved"],["reject","fail",e.Callbacks("once memory"),e.Callbacks("once memory"),1,"rejected"]],r="pending",i={state:function(){return r},always:function(){return t.done(arguments).fail(arguments),this},catch:function(e){return i.then(null,e)},pipe:function(){var s=arguments;return e.Deferred(function(i){e.each(o,function(a,e){var o=n(s[e[4]])&&s[e[4]];t[e[1]](function(){var t=o&&o.apply(this,arguments);t&&n(t.promise)?t.promise().progress(i.notify).done(i.resolve).fail(i.reject):i[e[0]+"With"](this,o?[t]:arguments)})}),s=null}).promise()},then:function(a,r,c){var t=0;function i(r,o,c,a){return function(){var l=this,d=arguments,h=function(){var e,s;if(r<t)return;if(e=c.apply(l,d),e===o.promise())throw new TypeError("Thenable self-resolution");s=e&&(typeof e=="object"||typeof e=="function")&&e.then,n(s)?a?s.call(e,i(t,o,N,a),i(t,o,Z,a)):(t++,s.call(e,i(t,o,N,a),i(t,o,Z,a),i(t,o,N,o.notifyWith))):(c!==N&&(l=void 0,d=[e]),(a||o.resolveWith)(l,d))},u=a?h:function(){try{h()}catch(n){e.Deferred.exceptionHook&&e.Deferred.exceptionHook(n,u.stackTrace),r+1>=t&&(c!==Z&&(l=void 0,d=[n]),o.rejectWith(l,d))}};r?u():(e.Deferred.getStackHook&&(u.stackTrace=e.Deferred.getStackHook()),s.setTimeout(u))}}return e.Deferred(function(e){o[0][3].add(i(0,e,n(c)?c:N,e.notifyWith)),o[1][3].add(i(0,e,n(a)?a:N)),o[2][3].add(i(0,e,n(r)?r:Z))}).promise()},promise:function(t){return t!=null?e.extend(t,i):i}},t={};return e.each(o,function(s,e){var n=e[2],a=e[5];i[e[1]]=n.add,a&&n.add(function(){r=a},o[3-s][2].disable,o[3-s][3].disable,o[0][2].lock,o[0][3].lock),n.add(e[3].fire),t[e[0]]=function(){return t[e[0]+"With"](this===t?void 0:this,arguments),this},t[e[0]+"With"]=n.fireWith}),i.promise(t),a&&a.call(t,t),t},when:function(c){var i=arguments.length,t=i,r=Array(t),o=v.call(arguments),s=e.Deferred(),a=function(e){return function(t){r[e]=this,o[e]=arguments.length>1?v.call(arguments):t,--i||s.resolveWith(r,o)}};if(i<=1&&(eG(c,s.done(a(t)).resolve,s.reject,!i),s.state()==="pending"||n(o[t]&&o[t].then)))return s.then();for(;t--;)eG(o[t],a(t),s.reject);return s.promise()}}),eY=/^(Eval|Internal|Range|Reference|Syntax|Type|URI)Error$/,e.Deferred.exceptionHook=function(e,t){s.console&&s.console.warn&&e&&eY.test(e.name)&&s.console.warn("jQuery.Deferred exception: "+e.message,e.stack,t)},e.readyException=function(e){s.setTimeout(function(){throw e})},ee=e.Deferred(),e.fn.ready=function(t){return ee.then(t).catch(function(t){e.readyException(t)}),this},e.extend({isReady:!1,readyWait:1,ready:function(t){if(t===!0?--e.readyWait:e.isReady)return;if(e.isReady=!0,t!==!0&&--e.readyWait>0)return;ee.resolveWith(o,[e])}}),e.ready.then=ee.then;function q(){o.removeEventListener("DOMContentLoaded",q),s.removeEventListener("load",q),e.ready()}o.readyState==="complete"||o.readyState!=="loading"&&!o.documentElement.doScroll?s.setTimeout(e.ready):(o.addEventListener("DOMContentLoaded",q),s.addEventListener("load",q)),g=function(s,t,i,a,l,d,r){var o=0,u=s.length,c=i==null;if(k(i)==="object"){l=!0;for(o in i)g(s,t,o,i[o],!0,d,r)}else if(a!==void 0&&(l=!0,n(a)||(r=!0),c&&(r?(t.call(s,a),t=null):(c=t,t=function(t,s,n){return c.call(e(t),n)})),t))for(;o<u;o++)t(s[o],i,r?a:a.call(s[o],o,t(s[o],i)));return l?s:c?t.call(s):u?t(s[0],i):d},eq=/^-ms-/,eU=/-([a-z])/g;function tT(t,e){return e.toUpperCase()}function m(e){return e.replace(eq,"ms-").replace(eU,tT)}S=function(e){return e.nodeType===1||e.nodeType===9||!+e.nodeType};function P(){this.expando=e.expando+P.uid++}P.uid=1,P.prototype={cache:function(e){var t=e[this.expando];return t||(t={},S(e)&&(e.nodeType?e[this.expando]=t:Object.defineProperty(e,this.expando,{value:t,configurable:!0}))),t},set:function(s,e,o){var t,n=this.cache(s);if(typeof e=="string")n[m(e)]=o;else for(t in e)n[m(t)]=e[t];return n},get:function(e,t){return t===void 0?this.cache(e):e[this.expando]&&e[this.expando][m(t)]},access:function(n,e,t){return e===void 0||e&&typeof e=="string"&&t===void 0?this.get(n,e):(this.set(n,e,t),t!==void 0?t:e)},remove:function(n,t){var o,s=n[this.expando];if(s===void 0)return;if(t!==void 0)for(Array.isArray(t)?t=t.map(m):(t=m(t),t=t in s?[t]:t.match(h)||[]),o=t.length;o--;)delete s[t[o]];(t===void 0||e.isEmptyObject(s))&&(n.nodeType?n[this.expando]=void 0:delete n[this.expando])},hasData:function(n){var t=n[this.expando];return t!==void 0&&!e.isEmptyObject(t)}},t=new P,a=new P,eI=/^(?:\{[\w\W]*\}|\[[\w\W]*\])$/,eH=/[A-Z]/g;function tF(e){return e==="true"||e!=="false"&&(e==="null"?null:e===+e+""?+e:eI.test(e)?JSON.parse(e):e)}function ez(t,n,e){var s;if(e===void 0&&t.nodeType===1)if(s="data-"+n.replace(eH,"-$&").toLowerCase(),e=t.getAttribute(s),typeof e=="string"){try{e=tF(e)}catch(e){}a.set(t,n,e)}else e=void 0;return e}e.extend({hasData:function(e){return a.hasData(e)||t.hasData(e)},data:function(e,t,n){return a.access(e,t,n)},removeData:function(e,t){a.remove(e,t)},_data:function(e,n,s){return t.access(e,n,s)},_removeData:function(e,n){t.remove(e,n)}}),e.fn.extend({data:function(n,c){var s,o,i,e=this[0],r=e&&e.attributes;if(n===void 0){if(this.length&&(i=a.get(e),e.nodeType===1&&!t.get(e,"hasDataAttrs"))){for(o=r.length;o--;)r[o]&&(s=r[o].name,s.indexOf("data-")===0&&(s=m(s.slice(5)),ez(e,s,i[s])));t.set(e,"hasDataAttrs",!0)}return i}return typeof n=="object"?this.each(function(){a.set(this,n)}):g(this,function(s){var t;if(e&&s===void 0){if(t=a.get(e,n),t!==void 0)return t;if(t=ez(e,n),t!==void 0)return t;return}this.each(function(){a.set(this,n,s)})},null,c,arguments.length>1,null,!0)},removeData:function(e){return this.each(function(){a.remove(this,e)})}}),e.extend({queue:function(i,s,o){var n;if(i)return s=(s||"fx")+"queue",n=t.get(i,s),o&&(!n||Array.isArray(o)?n=t.access(i,s,e.makeArray(o)):n.push(o)),n||[]},dequeue:function(n,t){t=t||"fx";var s=e.queue(n,t),a=s.length,o=s.shift(),i=e._queueHooks(n,t),r=function(){e.dequeue(n,t)};o==="inprogress"&&(o=s.shift(),a--),o&&(t==="fx"&&s.unshift("inprogress"),delete i.stop,o.call(n,r,i)),!a&&i&&i.empty.fire()},_queueHooks:function(n,o){var s=o+"queueHooks";return t.get(n,s)||t.access(n,s,{empty:e.Callbacks("once memory").add(function(){t.remove(n,[o+"queue",s])})})}}),e.fn.extend({queue:function(t,n){var s=2;return typeof t!="string"&&(n=t,t="fx",s--),arguments.length<s?e.queue(this[0],t):n===void 0?this:this.each(function(){var s=e.queue(this,t,n);e._queueHooks(this,t),t==="fx"&&s[0]!=="inprogress"&&e.dequeue(this,t)})},dequeue:function(t){return this.each(function(){e.dequeue(this,t)})},clearQueue:function(e){return this.queue(e||"fx",[])},promise:function(n,i){var s,a=1,r=e.Deferred(),o=this,c=this.length,l=function(){--a||r.resolveWith(o,[o])};for(typeof n!="string"&&(i=n,n=void 0),n=n||"fx";c--;)s=t.get(o[c],n+"queueHooks"),s&&s.empty&&(a++,s.empty.add(l));return l(),r.promise(i)}}),es=/[+-]?(?:\d*\.|)\d+(?:[eE][+-]?\d+|)/.source,C=new RegExp("^(?:([+-])=|)("+es+")([a-z%]*)$","i"),f=["Top","Right","Bottom","Left"],j=o.documentElement,x=function(t){return e.contains(t.ownerDocument,t)},t0={composed:!0},j.getRootNode&&(x=function(t){return e.contains(t.ownerDocument,t)||t.getRootNode(t0)===t.ownerDocument}),$=function(t,n){return t=n||t,t.style.display==="none"||t.style.display===""&&x(t)&&e.css(t,"display")==="none"};function eF(r,o,n,a){var c,l,d=20,u=a?function(){return a.cur()}:function(){return e.css(r,o,"")},i=u(),s=n&&n[3]||(e.cssNumber[o]?"":"px"),t=r.nodeType&&(e.cssNumber[o]||s!=="px"&&+i)&&C.exec(e.css(r,o));if(t&&t[3]!==s){for(i=i/2,s=s||t[3],t=+i||1;d--;)e.style(r,o,t+s),(1-c)*(1-(c=u()/i||.5))<=0&&(d=0),t=t/c;t=t*2,e.style(r,o,t+s),n=n||[]}return n&&(t=+t||+i||0,l=n[1]?t+(n[1]+1)*n[2]:+n[2],a&&(a.unit=s,a.start=t,a.end=l)),l}eu={};function tM(o){var n,i=o.ownerDocument,s=o.nodeName,t=eu[s];return t||(n=i.body.appendChild(i.createElement(s)),t=e.css(n,"display"),n.parentNode.removeChild(n),t==="none"&&(t="block"),eu[s]=t,t)}function A(o,r){for(var n,i,s=[],e=0,a=o.length;e<a;e++){if(n=o[e],!n.style)continue;i=n.style.display,r?(i==="none"&&(s[e]=t.get(n,"display")||null,s[e]||(n.style.display="")),n.style.display===""&&$(n)&&(s[e]=tM(n))):i!=="none"&&(s[e]="none",t.set(n,"display",i))}for(e=0;e<a;e++)s[e]!=null&&(o[e].style.display=s[e]);return o}e.fn.extend({show:function(){return A(this,!0)},hide:function(){return A(this)},toggle:function(t){return typeof t=="boolean"?t?this.show():this.hide():this.each(function(){$(this)?e(this).show():e(this).hide()})}}),E=/^(?:checkbox|radio)$/i,e2=/<([a-z][^/\0>\x20\t\r\n\f]*)/i,e3=/^$|^module$|\/(?:java|ecma)script/i,function(){var n=o.createDocumentFragment(),e=n.appendChild(o.createElement("div")),t=o.createElement("input");t.setAttribute("type","radio"),t.setAttribute("checked","checked"),t.setAttribute("name","t"),e.appendChild(t),i.checkClone=e.cloneNode(!0).cloneNode(!0).lastChild.checked,e.innerHTML="<textarea>x</textarea>",i.noCloneChecked=!!e.cloneNode(!0).lastChild.defaultValue,e.innerHTML="<option></option>",i.option=!!e.lastChild}(),l={thead:[1,"<table>","</table>"],col:[2,"<table><colgroup>","</colgroup></table>"],tr:[2,"<table><tbody>","</tbody></table>"],td:[3,"<table><tbody><tr>","</tr></tbody></table>"],_default:[0,"",""]},l.tbody=l.tfoot=l.colgroup=l.caption=l.thead,l.th=l.td,i.option||(l.optgroup=l.option=[1,"<select multiple='multiple'>","</select>"]);function r(t,n){var s;return typeof t.getElementsByTagName!="undefined"?s=t.getElementsByTagName(n||"*"):typeof t.querySelectorAll!="undefined"?s=t.querySelectorAll(n||"*"):s=[],n===void 0||n&&d(t,n)?e.merge([t],s):s}function ej(n,s){for(var e=0,o=n.length;e<o;e++)t.set(n[e],"globalEval",!s||t.get(s[e],"globalEval"))}eM=/<|&#?\w+;/;function eS(u,d,g,p,f){for(var t,n,i,c,h,m,o=d.createDocumentFragment(),a=[],s=0,v=u.length;s<v;s++)if(t=u[s],t||t===0)if(k(t)==="object")e.merge(a,t.nodeType?[t]:t);else if(eM.test(t)){for(n=n||o.appendChild(d.createElement("div")),m=(e2.exec(t)||["",""])[1].toLowerCase(),c=l[m]||l._default,n.innerHTML=c[1]+e.htmlPrefilter(t)+c[2],i=c[0];i--;)n=n.lastChild;e.merge(a,n.childNodes),n=o.firstChild,n.textContent=""}else a.push(d.createTextNode(t));for(o.textContent="",s=0;t=a[s++];){if(p&&e.inArray(t,p)>-1){f&&f.push(t);continue}if(h=x(t),n=r(o.appendChild(t),"script"),h&&ej(n),g)for(i=0;t=n[i++];)e3.test(t.type||"")&&g.push(t)}return o}ep=/^([^.]*)(?:\.(.+)|)/;function T(){return!0}function z(){return!1}function tS(e,t){return e===tE()===(t==="focus")}function tE(){try{return o.activeElement}catch(e){}}function e9(o,i,n,s,t,c){var a,r;if(typeof i=="object"){typeof n!="string"&&(s=s||n,n=void 0);for(r in i)e9(o,r,n,s,i[r],c);return o}if(s==null&&t==null?(t=n,s=n=void 0):t==null&&(typeof n=="string"?(t=s,s=void 0):(t=s,s=n,n=void 0)),t===!1)t=z;else if(!t)return o;return c===1&&(a=t,t=function(t){return e().off(t),a.apply(this,arguments)},t.guid=a.guid||(a.guid=e.guid++)),o.each(function(){e.event.add(this,i,t,s,n)})}e.event={global:{},add:function(a,u,s,b,o){var n,i,r,c,l,d,m,p,g,v,y,f=t.get(a);if(!S(a))return;for(s.handler&&(l=s,s=l.handler,o=l.selector),o&&e.find.matchesSelector(j,o),s.guid||(s.guid=e.guid++),(m=f.events)||(m=f.events=Object.create(null)),(d=f.handle)||(d=f.handle=function(t){return typeof e!="undefined"&&e.event.triggered!==t.type?e.event.dispatch.apply(a,arguments):void 0}),u=(u||"").match(h)||[""],p=u.length;p--;){if(g=ep.exec(u[p])||[],n=y=g[1],v=(g[2]||"").split(".").sort(),!n)continue;i=e.event.special[n]||{},n=(o?i.delegateType:i.bindType)||n,i=e.event.special[n]||{},r=e.extend({type:n,origType:y,data:b,handler:s,guid:s.guid,selector:o,needsContext:o&&e.expr.match.needsContext.test(o),namespace:v.join(".")},l),(c=m[n])||(c=m[n]=[],c.delegateCount=0,(!i.setup||i.setup.call(a,b,v,d)===!1)&&a.addEventListener&&a.addEventListener(n,d)),i.add&&(i.add.call(a,r),r.handler.guid||(r.handler.guid=s.guid)),o?c.splice(c.delegateCount++,0,r):c.push(r),e.event.global[n]=!0}},remove:function(a,l,p,c,j){var n,s,o,i,r,d,u,f,g,v,b,m=t.hasData(a)&&t.get(a);if(!m||!(d=m.events))return;for(l=(l||"").match(h)||[""],u=l.length;u--;){if(o=ep.exec(l[u])||[],n=v=o[1],g=(o[2]||"").split(".").sort(),!n){for(n in d)e.event.remove(a,n+l[u],p,c,!0);continue}for(i=e.event.special[n]||{},n=(c?i.delegateType:i.bindType)||n,r=d[n]||[],o=o[2]&&new RegExp("(^|\\.)"+g.join("\\.(?:.*\\.|)")+"(\\.|$)"),b=f=r.length;f--;)s=r[f],(j||v===s.origType)&&(!p||p.guid===s.guid)&&(!o||o.test(s.namespace))&&(!c||c===s.selector||c==="**"&&s.selector)&&(r.splice(f,1),s.selector&&r.delegateCount--,i.remove&&i.remove.call(a,s));b&&!r.length&&((!i.teardown||i.teardown.call(a,g,m.handle)===!1)&&e.removeEvent(a,n,m.handle),delete d[n])}e.isEmptyObject(d)&&t.remove(a,"handle events")},dispatch:function(u){var s,o,i,r,l,d,c=new Array(arguments.length),n=e.event.fix(u),h=(t.get(this,"events")||Object.create(null))[n.type]||[],a=e.event.special[n.type]||{};c[0]=n;for(s=1;s<arguments.length;s++)c[s]=arguments[s];if(n.delegateTarget=this,a.preDispatch&&a.preDispatch.call(this,n)===!1)return;for(l=e.event.handlers.call(this,n,h),s=0;(i=l[s++])&&!n.isPropagationStopped();)for(n.currentTarget=i.elem,d=0;(o=i.handlers[d++])&&!n.isImmediatePropagationStopped();)(!n.rnamespace||o.namespace===!1||n.rnamespace.test(o.namespace))&&(n.handleObj=o,n.data=o.data,r=((e.event.special[o.origType]||{}).handle||o.handler).apply(i.elem,c),r!==void 0&&(n.result=r)===!1&&(n.preventDefault(),n.stopPropagation()));return a.postDispatch&&a.postDispatch.call(this,n),n.result},handlers:function(s,o){var n,i,a,r,l,d=[],c=o.delegateCount,t=s.target;if(c&&t.nodeType&&!(s.type==="click"&&s.button>=1))for(;t!==this;t=t.parentNode||this)if(t.nodeType===1&&(s.type!=="click"||t.disabled!==!0)){a=[],r={};for(i=0;i<c;i++)l=o[i],n=l.selector+" ",r[n]===void 0&&(r[n]=l.needsContext?e(n,this).index(t)>-1:e.find(n,this,null,[t]).length),r[n]&&a.push(l);a.length&&d.push({elem:t,handlers:a})}return t=this,c<o.length&&d.push({elem:t,handlers:o.slice(c)}),d},addProp:function(t,s){Object.defineProperty(e.Event.prototype,t,{enumerable:!0,configurable:!0,get:n(s)?function(){if(this.originalEvent)return s(this.originalEvent)}:function(){if(this.originalEvent)return this.originalEvent[t]},set:function(e){Object.defineProperty(this,t,{enumerable:!0,configurable:!0,writable:!0,value:e})}})},fix:function(t){return t[e.expando]?t:new e.Event(t)},special:{load:{noBubble:!0},click:{setup:function(t){var e=this||t;return E.test(e.type)&&e.click&&d(e,"input")&&J(e,"click",T),!1},trigger:function(t){var e=this||t;return E.test(e.type)&&e.click&&d(e,"input")&&J(e,"click"),!0},_default:function(n){var e=n.target;return E.test(e.type)&&e.click&&d(e,"input")&&t.get(e,"click")||d(e,"a")}},beforeunload:{postDispatch:function(e){e.result!==void 0&&e.originalEvent&&(e.originalEvent.returnValue=e.result)}}}};function J(s,n,o){if(!o){t.get(s,n)===void 0&&e.event.add(s,n,T);return}t.set(s,n,!1),e.event.add(s,n,{namespace:!1,handler:function(a){var i,r,s=t.get(this,n);if(a.isTrigger&1&&this[n]){if(s.length)(e.event.special[n]||{}).delegateType&&a.stopPropagation();else if(s=v.call(arguments),t.set(this,n,s),r=o(this,n),this[n](),i=t.get(this,n),s!==i||r?t.set(this,n,!1):i={},s!==i)return a.stopImmediatePropagation(),a.preventDefault(),i&&i.value}else s.length&&(t.set(this,n,{value:e.event.trigger(e.extend(s[0],e.Event.prototype),s.slice(1),this)}),a.stopImmediatePropagation())}})}e.removeEvent=function(e,t,n){e.removeEventListener&&e.removeEventListener(t,n)},e.Event=function(t,n){if(!(this instanceof e.Event))return new e.Event(t,n);t&&t.type?(this.originalEvent=t,this.type=t.type,this.isDefaultPrevented=t.defaultPrevented||t.defaultPrevented===void 0&&t.returnValue===!1?T:z,this.target=t.target&&t.target.nodeType===3?t.target.parentNode:t.target,this.currentTarget=t.currentTarget,this.relatedTarget=t.relatedTarget):this.type=t,n&&e.extend(this,n),this.timeStamp=t&&t.timeStamp||Date.now(),this[e.expando]=!0},e.Event.prototype={constructor:e.Event,isDefaultPrevented:z,isPropagationStopped:z,isImmediatePropagationStopped:z,isSimulated:!1,preventDefault:function(){var e=this.originalEvent;this.isDefaultPrevented=T,e&&!this.isSimulated&&e.preventDefault()},stopPropagation:function(){var e=this.originalEvent;this.isPropagationStopped=T,e&&!this.isSimulated&&e.stopPropagation()},stopImmediatePropagation:function(){var e=this.originalEvent;this.isImmediatePropagationStopped=T,e&&!this.isSimulated&&e.stopImmediatePropagation(),this.stopPropagation()}},e.each({altKey:!0,bubbles:!0,cancelable:!0,changedTouches:!0,ctrlKey:!0,detail:!0,eventPhase:!0,metaKey:!0,pageX:!0,pageY:!0,shiftKey:!0,view:!0,char:!0,code:!0,charCode:!0,key:!0,keyCode:!0,button:!0,buttons:!0,clientX:!0,clientY:!0,offsetX:!0,offsetY:!0,pointerId:!0,pointerType:!0,screenX:!0,screenY:!0,targetTouches:!0,toElement:!0,touches:!0,which:!0},e.event.addProp),e.each({focus:"focusin",blur:"focusout"},function(t,n){e.event.special[t]={setup:function(){return J(this,t,tS),!1},trigger:function(){return J(this,t),!0},_default:function(){return!0},delegateType:n}}),e.each({mouseenter:"mouseover",mouseleave:"mouseout",pointerenter:"pointerover",pointerleave:"pointerout"},function(n,t){e.event.special[n]={delegateType:t,bindType:t,handle:function(n){var o,i=this,s=n.relatedTarget,a=n.handleObj;return(!s||s!==i&&!e.contains(i,s))&&(n.type=a.origType,o=a.handler.apply(this,arguments),n.type=t),o}}}),e.fn.extend({on:function(e,t,n,s){return e9(this,e,t,n,s)},one:function(e,t,n,s){return e9(this,e,t,n,s,1)},off:function(t,s,o){var n,i;if(t&&t.preventDefault&&t.handleObj)return n=t.handleObj,e(t.delegateTarget).off(n.namespace?n.origType+"."+n.namespace:n.origType,n.selector,n.handler),this;if(typeof t=="object"){for(i in t)this.off(i,s,t[i]);return this}return(s===!1||typeof s=="function")&&(o=s,s=void 0),o===!1&&(o=z),this.each(function(){e.event.remove(this,t,o,s)})}});var tC=/<script|<style|<link/i,tx=/checked\s*(?:[^=]|=\s*.checked.)/i,tO=/^\s*<!(?:\[CDATA\[|--)|(?:\]\]|--)>\s*$/g;function ew(t,n){return d(t,"table")&&d(n.nodeType!==11?n:n.firstChild,"tr")?e(t).children("tbody")[0]||t:t}function tw(e){return e.type=(e.getAttribute("type")!==null)+"/"+e.type,e}function t_(e){return(e.type||"").slice(0,5)==="true/"?e.type=e.type.slice(5):e.removeAttribute("type"),e}function eC(s,o){var n,i,r,c,l,d,u;if(o.nodeType!==1)return;if(t.hasData(s)&&(l=t.get(s),n=l.events,n)){t.remove(o,"handle events");for(i in n)for(r=0,c=n[i].length;r<c;r++)e.event.add(o,i,n[i][r])}a.hasData(s)&&(d=a.access(s),u=e.extend({},d),a.set(o,u))}function ty(e,t){var n=t.nodeName.toLowerCase();n==="input"&&E.test(e.type)?t.checked=e.checked:(n==="input"||n==="textarea")&&(t.defaultValue=e.defaultValue)}function D(c,d,v,g){d=eg(d);var s,a,l,h,f,p,o=0,u=c.length,j=u-1,m=d[0],b=n(m);if(b||u>1&&typeof m=="string"&&!i.checkClone&&tx.test(m))return c.each(function(e){var t=c.eq(e);b&&(d[0]=m.call(this,e,t.html())),D(t,d,v,g)});if(u&&(l=eS(d,c[0].ownerDocument,!1,c,g),p=l.firstChild,l.childNodes.length===1&&(l=p),p||g)){for(a=e.map(r(l,"script"),tw),f=a.length;o<u;o++)s=l,o!==j&&(s=e.clone(s,!0,!0),f&&e.merge(a,r(s,"script"))),v.call(c[o],s,o);if(f){h=a[a.length-1].ownerDocument,e.map(a,t_);for(o=0;o<f;o++)s=a[o],e3.test(s.type||"")&&!t.access(s,"globalEval")&&e.contains(h,s)&&(s.src&&(s.type||"").toLowerCase()!=="module"?e._evalUrl&&!s.noModule&&e._evalUrl(s.src,{nonce:s.nonce||s.getAttribute("nonce")},h):ta(s.textContent.replace(tO,""),s,h))}}return c}function eA(n,s,o){for(var t,a=s?e.filter(s,n):n,i=0;(t=a[i])!=null;i++)!o&&t.nodeType===1&&e.cleanData(r(t)),t.parentNode&&(o&&x(t)&&ej(r(t,"script")),t.parentNode.removeChild(t));return n}e.extend({htmlPrefilter:function(e){return e},clone:function(n,d,l){var t,s,o,c,a=n.cloneNode(!0),u=x(n);if(!i.noCloneChecked&&(n.nodeType===1||n.nodeType===11)&&!e.isXMLDoc(n)){s=r(a),o=r(n);for(t=0,c=o.length;t<c;t++)ty(o[t],s[t])}if(d)if(l){o=o||r(n),s=s||r(a);for(t=0,c=o.length;t<c;t++)eC(o[t],s[t])}else eC(n,a);return s=r(a,"script"),s.length>0&&ej(s,!u&&r(n,"script")),a},cleanData:function(r){for(var n,s,o,c=e.event.special,i=0;(n=r[i])!==void 0;i++)if(S(n)){if(s=n[t.expando]){if(s.events)for(o in s.events)c[o]?e.event.remove(n,o):e.removeEvent(n,o,s.handle);n[t.expando]=void 0}n[a.expando]&&(n[a.expando]=void 0)}}}),e.fn.extend({detach:function(e){return eA(this,e,!0)},remove:function(e){return eA(this,e)},text:function(t){return g(this,function(t){return t===void 0?e.text(this):this.empty().each(function(){(this.nodeType===1||this.nodeType===11||this.nodeType===9)&&(this.textContent=t)})},null,t,arguments.length)},append:function(){return D(this,arguments,function(e){if(this.nodeType===1||this.nodeType===11||this.nodeType===9){var t=ew(this,e);t.appendChild(e)}})},prepend:function(){return D(this,arguments,function(e){if(this.nodeType===1||this.nodeType===11||this.nodeType===9){var t=ew(this,e);t.insertBefore(e,t.firstChild)}})},before:function(){return D(this,arguments,function(e){this.parentNode&&this.parentNode.insertBefore(e,this)})},after:function(){return D(this,arguments,function(e){this.parentNode&&this.parentNode.insertBefore(e,this.nextSibling)})},empty:function(){for(var t,n=0;(t=this[n])!=null;n++)t.nodeType===1&&(e.cleanData(r(t,!1)),t.textContent="");return this},clone:function(t,n){return t=t??!1,n=n??t,this.map(function(){return e.clone(this,t,n)})},html:function(t){return g(this,function(t){var n=this[0]||{},s=0,o=this.length;if(t===void 0&&n.nodeType===1)return n.innerHTML;if(typeof t=="string"&&!tC.test(t)&&!l[(e2.exec(t)||["",""])[1].toLowerCase()]){t=e.htmlPrefilter(t);try{for(;s<o;s++)n=this[s]||{},n.nodeType===1&&(e.cleanData(r(n,!1)),n.innerHTML=t);n=0}catch(e){}}n&&this.empty().append(t)},null,t,arguments.length)},replaceWith:function(){var t=[];return D(this,arguments,function(s){var n=this.parentNode;e.inArray(this,t)<0&&(e.cleanData(r(this)),n&&n.replaceChild(s,this))},t)}}),e.each({appendTo:"append",prependTo:"prepend",insertBefore:"before",insertAfter:"after",replaceAll:"replaceWith"},function(t,n){e.fn[t]=function(r){for(var s,o=[],i=e(r),a=i.length-1,t=0;t<=a;t++)s=t===a?this:this.clone(!0),e(i[t])[n](s),Y.apply(o,s.get());return this.pushStack(o)}}),X=new RegExp("^("+es+")(?!px)[a-z%]+$","i"),B=function(t){var e=t.ownerDocument.defaultView;return(!e||!e.opener)&&(e=s),e.getComputedStyle(t)},ed=function(t,n,i){var e,s,o={};for(e in n)o[e]=t.style[e],t.style[e]=n[e];s=i.call(t);for(e in n)t.style[e]=o[e];return s},eT=new RegExp(f.join("|"),"i"),function(){function n(){if(!t)return;r.style.cssText="position:absolute;left:-11111px;width:60px;margin-top:1px;padding:0;border:0",t.style.cssText="position:relative;display:block;box-sizing:border-box;overflow:scroll;margin:auto;border:1px;padding:1px;width:60%;top:1%",j.appendChild(r).appendChild(t);var e=s.getComputedStyle(t),h=e.top!=="1%",u=a(e.marginLeft)===12;t.style.right="60%",d=a(e.right)===36,m=a(e.width)===36,t.style.position="absolute",l=a(t.offsetWidth/3)===12,j.removeChild(r),t=null}function a(e){return Math.round(parseFloat(e))}var c,l,d,u,h,m,r=o.createElement("div"),t=o.createElement("div");if(!t.style)return;t.style.backgroundClip="content-box",t.cloneNode(!0).style.backgroundClip="",i.clearCloneStyle=t.style.backgroundClip==="content-box",e.extend(i,{boxSizingReliable:function(){return n(),m},pixelBoxStyles:function(){return n(),d},pixelPosition:function(){return n(),h},reliableMarginLeft:function(){return n(),u},scrollboxSize:function(){return n(),l},reliableTrDimensions:function(){var e,t,n,i;return c==null&&(t=o.createElement("table"),e=o.createElement("tr"),n=o.createElement("div"),t.style.cssText="position:absolute;left:-11111px;border-collapse:separate",e.style.cssText="border:1px solid",e.style.height="1px",n.style.height="9px",n.style.display="block",j.appendChild(t).appendChild(e).appendChild(n),i=s.getComputedStyle(e),c=parseInt(i.height,10)+parseInt(i.borderTopWidth,10)+parseInt(i.borderBottomWidth,10)===e.offsetHeight,j.removeChild(t)),c}})}();function W(o,a,s){var t,r,c,l,n=o.style;return s=s||B(o),s&&(t=s.getPropertyValue(a)||s[a],t===""&&!x(o)&&(t=e.style(o,a)),!i.pixelBoxStyles()&&X.test(t)&&eT.test(a)&&(r=n.width,c=n.minWidth,l=n.maxWidth,n.minWidth=n.maxWidth=n.width=t,t=s.width,n.width=r,n.minWidth=c,n.maxWidth=l)),t!==void 0?t+"":t}function eD(e,t){return{get:function(){if(e()){delete this.get;return}return(this.get=t).apply(this,arguments)}}}var eN=["Webkit","Moz","ms"],eL=o.createElement("div").style,eR={};function t7(e){for(var n=e[0].toUpperCase()+e.slice(1),t=eN.length;t--;)if(e=eN[t]+n,e in eL)return e}function en(t){var n=e.cssProps[t]||eR[t];return n?n:t in eL?t:eR[t]=t7(t)||t}var tp=/^(none|table(?!-c[ea]).+)/,eB=/^--/,t8={position:"absolute",visibility:"hidden",display:"block"},e$={letterSpacing:"0",fontWeight:"400"};function eW(s,t,n){var e=C.exec(t);return e?Math.max(0,e[2]-(n||0))+(e[3]||"px"):t}function ea(s,a,i,r,o,c){var t=a==="width"?1:0,l=0,n=0;if(i===(r?"border":"content"))return 0;for(;t<4;t+=2)i==="margin"&&(n+=e.css(s,i+f[t],!0,o)),r?(i==="content"&&(n-=e.css(s,"padding"+f[t],!0,o)),i!=="margin"&&(n-=e.css(s,"border"+f[t]+"Width",!0,o))):(n+=e.css(s,"padding"+f[t],!0,o),i!=="padding"?n+=e.css(s,"border"+f[t]+"Width",!0,o):l+=e.css(s,"border"+f[t]+"Width",!0,o));return!r&&c>=0&&(n+=Math.max(0,Math.ceil(s["offset"+a[0].toUpperCase()+a.slice(1)]-c-n-l-.5))||0),n}function eK(t,a,r){var s=B(t),u=!i.boxSizingReliable()||r,o=u&&e.css(t,"boxSizing",!1,s)==="border-box",c=o,n=W(t,a,s),l="offset"+a[0].toUpperCase()+a.slice(1);if(X.test(n)){if(!r)return n;n="auto"}return(!i.boxSizingReliable()&&o||!i.reliableTrDimensions()&&d(t,"tr")||n==="auto"||!parseFloat(n)&&e.css(t,"display",!1,s)==="inline")&&t.getClientRects().length&&(o=e.css(t,"boxSizing",!1,s)==="border-box",c=l in t,c&&(n=t[l])),n=parseFloat(n)||0,n+ea(t,a,r||(o?"border":"content"),c,s,n)+"px"}e.extend({cssHooks:{opacity:{get:function(t,n){if(n){var e=W(t,"opacity");return e===""?"1":e}}}},cssNumber:{animationIterationCount:!0,columnCount:!0,fillOpacity:!0,flexGrow:!0,flexShrink:!0,fontWeight:!0,gridArea:!0,gridColumn:!0,gridColumnEnd:!0,gridColumnStart:!0,gridRow:!0,gridRowEnd:!0,gridRowStart:!0,lineHeight:!0,opacity:!0,order:!0,orphans:!0,widows:!0,zIndex:!0,zoom:!0},cssProps:{},style:function(s,n,t,u){if(!s||s.nodeType===3||s.nodeType===8||!s.style)return;var o,a,c,l=m(n),d=eB.test(n),r=s.style;if(d||(n=en(l)),a=e.cssHooks[n]||e.cssHooks[l],t!==void 0){if(c=typeof t,c==="string"&&(o=C.exec(t))&&o[1]&&(t=eF(s,n,o),c="number"),t==null||t!==t)return;c==="number"&&!d&&(t+=o&&o[3]||(e.cssNumber[l]?"":"px")),!i.clearCloneStyle&&t===""&&n.indexOf("background")===0&&(r[n]="inherit"),(!a||!("set"in a)||(t=a.set(s,t,u))!==void 0)&&(d?r.setProperty(n,t):r[n]=t)}else return a&&"get"in a&&(o=a.get(s,!1,u))!==void 0?o:r[n]},css:function(a,n,o,c){var t,s,i,r=m(n),l=eB.test(n);return l||(n=en(r)),s=e.cssHooks[n]||e.cssHooks[r],s&&"get"in s&&(t=s.get(a,!0,o)),t===void 0&&(t=W(a,n,c)),t==="normal"&&n in e$&&(t=e$[n]),o===""||o?(i=parseFloat(t),o===!0||isFinite(i)?i||0:t):t}}),e.each(["height","width"],function(n,t){e.cssHooks[t]={get:function(n,o,s){if(o)return tp.test(e.css(n,"display"))&&(!n.getClientRects().length||!n.getBoundingClientRect().width)?ed(n,t8,function(){return eK(n,t,s)}):eK(n,t,s)},set:function(n,o,a){var c,s=B(n),l=!i.scrollboxSize()&&s.position==="absolute",u=l||a,d=u&&e.css(n,"boxSizing",!1,s)==="border-box",r=a?ea(n,t,a,d,s):0;return d&&l&&(r-=Math.ceil(n["offset"+t[0].toUpperCase()+t.slice(1)]-parseFloat(s[t])-ea(n,t,"border",!1,s)-.5)),r&&(c=C.exec(o))&&(c[3]||"px")!=="px"&&(n.style[t]=o,o=e.css(n,t)),eW(n,o,r)}}}),e.cssHooks.marginLeft=eD(i.reliableMarginLeft,function(e,t){if(t)return(parseFloat(W(e,"marginLeft"))||e.getBoundingClientRect().left-ed(e,{marginLeft:0},function(){return e.getBoundingClientRect().left}))+"px"}),e.each({margin:"",padding:"",border:"Width"},function(t,n){e.cssHooks[t+n]={expand:function(s){for(var e=0,i={},o=typeof s=="string"?s.split(" "):[s];e<4;e++)i[t+f[e]+n]=o[e]||o[e-2]||o[0];return i}},t!=="margin"&&(e.cssHooks[t+n].set=eW)}),e.fn.extend({css:function(t,n){return g(this,function(n,t,o){var i,a,r={},s=0;if(Array.isArray(t)){for(i=B(n),a=t.length;s<a;s++)r[t[s]]=e.css(n,t[s],!1,i);return r}return o!==void 0?e.style(n,t,o):e.css(n,t)},t,n,arguments.length>1)}});function c(e,t,n,s,o){return new c.prototype.init(e,t,n,s,o)}e.Tween=c,c.prototype={constructor:c,init:function(n,s,t,o,i,a){this.elem=n,this.prop=t,this.easing=i||e.easing._default,this.options=s,this.start=this.now=this.cur(),this.end=o,this.unit=a||(e.cssNumber[t]?"":"px")},cur:function(){var e=c.propHooks[this.prop];return e&&e.get?e.get(this):c.propHooks._default.get(this)},run:function(t){var n,s=c.propHooks[this.prop];return this.options.duration?this.pos=n=e.easing[this.easing](t,this.options.duration*t,0,1,this.options.duration):this.pos=n=t,this.now=(this.end-this.start)*n+this.start,this.options.step&&this.options.step.call(this.elem,this.now,this),s&&s.set?s.set(this):c.propHooks._default.set(this),this}},c.prototype.init.prototype=c.prototype,c.propHooks={_default:{get:function(t){var n;return t.elem.nodeType!==1||t.elem[t.prop]!=null&&t.elem.style[t.prop]==null?t.elem[t.prop]:(n=e.css(t.elem,t.prop,""),!n||n==="auto"?0:n)},set:function(t){e.fx.step[t.prop]?e.fx.step[t.prop](t):t.elem.nodeType===1&&(e.cssHooks[t.prop]||t.elem.style[en(t.prop)]!=null)?e.style(t.elem,t.prop,t.now+t.unit):t.elem[t.prop]=t.now}}},c.propHooks.scrollTop=c.propHooks.scrollLeft={set:function(e){e.elem.nodeType&&e.elem.parentNode&&(e.elem[e.prop]=e.now)}},e.easing={linear:function(e){return e},swing:function(e){return.5-Math.cos(e*Math.PI)/2},_default:"swing"},e.fx=c.prototype.init,e.fx.step={},eX=/^(?:toggle|show|hide)$/,t4=/queueHooks$/;function ec(){Q&&(o.hidden===!1&&s.requestAnimationFrame?s.requestAnimationFrame(ec):s.setTimeout(ec,e.fx.interval),e.fx.tick())}function eJ(){return s.setTimeout(function(){R=void 0}),R=Date.now()}function K(n,t){var s,o=0,e={height:n};for(t=t?1:0;o<4;o+=2-t)s=f[o],e["margin"+s]=e["padding"+s]=n;return t&&(e.opacity=e.width=n),e}function tt(o,t,i){for(var n,s=(u.tweeners[t]||[]).concat(u.tweeners["*"]),e=0,a=s.length;e<a;e++)if(n=s[e].call(i,t,o))return n}function th(n,u,h){var s,i,r,c,l,p,g,v,b="width"in u||"height"in u,m=this,f={},a=n.style,d=n.nodeType&&$(n),o=t.get(n,"fxshow");h.queue||(c=e._queueHooks(n,"fx"),c.unqueued==null&&(c.unqueued=0,v=c.empty.fire,c.empty.fire=function(){c.unqueued||v()}),c.unqueued++,m.always(function(){m.always(function(){c.unqueued--,e.queue(n,"fx").length||c.empty.fire()})}));for(s in u)if(p=u[s],eX.test(p)){if(delete u[s],g=g||p==="toggle",p===(d?"hide":"show"))if(p==="show"&&o&&o[s]!==void 0)d=!0;else continue;f[s]=o&&o[s]||e.style(n,s)}if(r=!e.isEmptyObject(u),!r&&e.isEmptyObject(f))return;b&&n.nodeType===1&&(h.overflow=[a.overflow,a.overflowX,a.overflowY],i=o&&o.display,i==null&&(i=t.get(n,"display")),l=e.css(n,"display"),l==="none"&&(i?l=i:(A([n],!0),i=n.style.display||i,l=e.css(n,"display"),A([n]))),(l==="inline"||l==="inline-block"&&i!=null)&&e.css(n,"float")==="none"&&(r||(m.done(function(){a.display=i}),i==null&&(l=a.display,i=l==="none"?"":l)),a.display="inline-block")),h.overflow&&(a.overflow="hidden",m.always(function(){a.overflow=h.overflow[0],a.overflowX=h.overflow[1],a.overflowY=h.overflow[2]})),r=!1;for(s in f)r||(o?"hidden"in o&&(d=o.hidden):o=t.access(n,"fxshow",{display:i}),g&&(o.hidden=!d),d&&A([n],!0),m.done(function(){d||A([n]),t.remove(n,"fxshow");for(s in f)e.style(n,s,f[s])})),r=tt(d?o[s]:0,s,m),s in o||(o[s]=r.start,d&&(r.end=r.start,r.start=0))}function t3(s,r){var t,n,o,i,a;for(t in s)if(o=m(t),i=r[o],n=s[t],Array.isArray(n)&&(i=n[1],n=s[t]=n[0]),t!==o&&(s[o]=n,delete s[t]),a=e.cssHooks[o],a&&"expand"in a){n=a.expand(n),delete s[o];for(t in n)t in s||(s[t]=n[t],r[t]=i)}else r[o]=i}function u(s,d,a){var i,r,c=0,m=u.prefilters.length,o=e.Deferred().always(function(){delete h.elem}),h=function(){if(r)return!1;for(var c=R||eJ(),e=Math.max(0,t.startTime+t.duration-c),l=e/t.duration||0,n=1-l,i=0,a=t.tweens.length;i<a;i++)t.tweens[i].run(n);return o.notifyWith(s,[t,n,e]),n<1&&a?e:(a||o.notifyWith(s,[t,1,0]),o.resolveWith(s,[t]),!1)},t=o.promise({elem:s,props:e.extend({},d),opts:e.extend(!0,{specialEasing:{},easing:e.easing._default},a),originalProperties:d,originalOptions:a,startTime:R||eJ(),duration:a.duration,tweens:[],createTween:function(n,i){var o=e.Tween(s,t.opts,n,i,t.opts.specialEasing[n]||t.opts.easing);return t.tweens.push(o),o},stop:function(e){var n=0,i=e?t.tweens.length:0;if(r)return this;for(r=!0;n<i;n++)t.tweens[n].run(1);return e?(o.notifyWith(s,[t,1,0]),o.resolveWith(s,[t,e])):o.rejectWith(s,[t,e]),this}}),l=t.props;for(t3(l,t.opts.specialEasing);c<m;c++)if(i=u.prefilters[c].call(t,s,l,t.opts),i)return n(i.stop)&&(e._queueHooks(t.elem,t.opts.queue).stop=i.stop.bind(i)),i;return e.map(l,tt,t),n(t.opts.start)&&t.opts.start.call(s,t),t.progress(t.opts.progress).done(t.opts.done,t.opts.complete).fail(t.opts.fail).always(t.opts.always),e.fx.timer(e.extend(h,{elem:s,anim:t,queue:t.opts.queue})),t}e.Animation=e.extend(u,{tweeners:{"*":[function(t,n){var e=this.createTween(t,n);return eF(e.elem,t,C.exec(n),e),e}]},tweener:function(e,o){n(e)?(o=e,e=["*"]):e=e.match(h);for(var t,s=0,i=e.length;s<i;s++)t=e[s],u.tweeners[t]=u.tweeners[t]||[],u.tweeners[t].unshift(o)},prefilters:[th],prefilter:function(e,t){t?u.prefilters.unshift(e):u.prefilters.push(e)}}),e.speed=function(s,o,i){var t=s&&typeof s=="object"?e.extend({},s):{complete:i||!i&&o||n(s)&&s,duration:s,easing:i&&o||o&&!n(o)&&o};return e.fx.off?t.duration=0:typeof t.duration!="number"&&(t.duration in e.fx.speeds?t.duration=e.fx.speeds[t.duration]:t.duration=e.fx.speeds._default),(t.queue==null||t.queue===!0)&&(t.queue="fx"),t.old=t.complete,t.complete=function(){n(t.old)&&t.old.call(this),t.queue&&e.dequeue(this,t.queue)},t},e.fn.extend({fadeTo:function(e,t,n,s){return this.filter($).css("opacity",0).show().end().animate({opacity:t},e,n,s)},animate:function(i,a,r,c){var o=e.isEmptyObject(i),s=e.speed(a,r,c),n=function(){var n=u(this,e.extend({},i),s);(o||t.get(this,"finish"))&&n.stop(!0)};return n.finish=n,o||s.queue===!1?this.each(n):this.queue(s.queue,n)},stop:function(n,o,s){var i=function(e){var t=e.stop;delete e.stop,t(s)};return typeof n!="string"&&(s=o,o=n,n=void 0),o&&this.queue(n||"fx",[]),this.each(function(){var c=!0,o=n!=null&&n+"queueHooks",r=e.timers,a=t.get(this);if(o)a[o]&&a[o].stop&&i(a[o]);else for(o in a)a[o]&&a[o].stop&&t4.test(o)&&i(a[o]);for(o=r.length;o--;)r[o].elem===this&&(n==null||r[o].queue===n)&&(r[o].anim.stop(s),c=!1,r.splice(o,1));(c||!s)&&e.dequeue(this,n)})},finish:function(n){return n!==!1&&(n=n||"fx"),this.each(function(){var s,a=t.get(this),o=a[n+"queue"],r=a[n+"queueHooks"],i=e.timers,c=o?o.length:0;a.finish=!0,e.queue(this,n,[]),r&&r.stop&&r.stop.call(this,!0);for(s=i.length;s--;)i[s].elem===this&&i[s].queue===n&&(i[s].anim.stop(!0),i.splice(s,1));for(s=0;s<c;s++)o[s]&&o[s].finish&&o[s].finish.call(this);delete a.finish})}}),e.each(["toggle","show","hide"],function(s,t){var n=e.fn[t];e.fn[t]=function(e,s,o){return e==null||typeof e=="boolean"?n.apply(this,arguments):this.animate(K(t,!0),e,s,o)}}),e.each({slideDown:K("show"),slideUp:K("hide"),slideToggle:K("toggle"),fadeIn:{opacity:"show"},fadeOut:{opacity:"hide"},fadeToggle:{opacity:"toggle"}},function(t,n){e.fn[t]=function(e,t,s){return this.animate(n,e,t,s)}}),e.timers=[],e.fx.tick=function(){var s,t=0,n=e.timers;for(R=Date.now();t<n.length;t++)s=n[t],!s()&&n[t]===s&&n.splice(t--,1);n.length||e.fx.stop(),R=void 0},e.fx.timer=function(t){e.timers.push(t),e.fx.start()},e.fx.interval=13,e.fx.start=function(){if(Q)return;Q=!0,ec()},e.fx.stop=function(){Q=null},e.fx.speeds={slow:600,fast:200,_default:400},e.fn.delay=function(t,n){return t=e.fx?e.fx.speeds[t]||t:t,n=n||"fx",this.queue(n,function(e,n){var o=s.setTimeout(e,t);n.stop=function(){s.clearTimeout(o)}})},function(){var e=o.createElement("input"),t=o.createElement("select"),n=t.appendChild(o.createElement("option"));e.type="checkbox",i.checkOn=e.value!=="",i.optSelected=n.selected,e=o.createElement("input"),e.value="t",e.type="radio",i.radioValue=e.value==="t"}(),L=e.expr.attrHandle,e.fn.extend({attr:function(t,n){return g(this,e.attr,t,n,arguments.length>1)},removeAttr:function(t){return this.each(function(){e.removeAttr(this,t)})}}),e.extend({attr:function(t,n,i){var s,o,a=t.nodeType;if(a===3||a===8||a===2)return;if(typeof t.getAttribute=="undefined")return e.prop(t,n,i);if((a!==1||!e.isXMLDoc(t))&&(o=e.attrHooks[n.toLowerCase()]||(e.expr.match.bool.test(n)?t2:void 0)),i!==void 0){if(i===null){e.removeAttr(t,n);return}return o&&"set"in o&&(s=o.set(t,i,n))!==void 0?s:(t.setAttribute(n,i+""),i)}return o&&"get"in o&&(s=o.get(t,n))!==null?s:(s=e.find.attr(t,n),s??void 0)},attrHooks:{type:{set:function(e,t){if(!i.radioValue&&t==="radio"&&d(e,"input")){var n=e.value;return e.setAttribute("type",t),n&&(e.value=n),t}}}},removeAttr:function(e,t){var n,o=0,s=t&&t.match(h);if(s&&e.nodeType===1)for(;n=s[o++];)e.removeAttribute(n)}}),t2={set:function(n,s,t){return s===!1?e.removeAttr(n,t):n.setAttribute(t,t),t}},e.each(e.expr.match.bool.source.match(/\w+/g),function(s,t){var n=L[t]||e.find.attr;L[t]=function(a,s,o){var t,i,e=s.toLowerCase();return o||(i=L[e],L[e]=t,t=n(a,s,o)!=null?e:null,L[e]=i),t}}),tr=/^(?:input|select|textarea|button)$/i,tc=/^(?:a|area)$/i,e.fn.extend({prop:function(t,n){return g(this,e.prop,t,n,arguments.length>1)},removeProp:function(t){return this.each(function(){delete this[e.propFix[t]||t]})}}),e.extend({prop:function(s,t,a){var n,o,i=s.nodeType;if(i===3||i===8||i===2)return;return(i!==1||!e.isXMLDoc(s))&&(t=e.propFix[t]||t,n=e.propHooks[t]),a!==void 0?n&&"set"in n&&(o=n.set(s,a,t))!==void 0?o:s[t]=a:n&&"get"in n&&(o=n.get(s,t))!==null?o:s[t]},propHooks:{tabIndex:{get:function(t){var n=e.find.attr(t,"tabindex");return n?parseInt(n,10):tr.test(t.nodeName)||tc.test(t.nodeName)&&t.href?0:-1}}},propFix:{for:"htmlFor",class:"className"}}),i.optSelected||(e.propHooks.selected={get:function(t){var e=t.parentNode;return e&&e.parentNode&&e.parentNode.selectedIndex,null},set:function(t){var e=t.parentNode;e&&(e.selectedIndex,e.parentNode&&e.parentNode.selectedIndex)}}),e.each(["tabIndex","readOnly","maxLength","cellSpacing","cellPadding","rowSpan","colSpan","useMap","frameBorder","contentEditable"],function(){e.propFix[this.toLowerCase()]=this});function y(e){var t=e.match(h)||[];return t.join(" ")}function w(e){return e.getAttribute&&e.getAttribute("class")||""}function e7(e){return Array.isArray(e)?e:typeof e=="string"?e.match(h)||[]:[]}e.fn.extend({addClass:function(o){var t,s,i,a,r,c,l,d=0;if(n(o))return this.each(function(t){e(this).addClass(o.call(this,t,w(this)))});if(i=e7(o),i.length)for(;s=this[d++];)if(a=w(s),t=s.nodeType===1&&" "+y(a)+" ",t){for(l=0;r=i[l++];)t.indexOf(" "+r+" ")<0&&(t+=r+" ");c=y(t),a!==c&&s.setAttribute("class",c)}return this},removeClass:function(o){var t,s,i,a,r,c,l,d=0;if(n(o))return this.each(function(t){e(this).removeClass(o.call(this,t,w(this)))});if(!arguments.length)return this.attr("class","");if(i=e7(o),i.length)for(;s=this[d++];)if(a=w(s),t=s.nodeType===1&&" "+y(a)+" ",t){for(l=0;r=i[l++];)for(;t.indexOf(" "+r+" ")>-1;)t=t.replace(" "+r+" "," ");c=y(t),a!==c&&s.setAttribute("class",c)}return this},toggleClass:function(s,o){var i=typeof s,a=i==="string"||Array.isArray(s);return typeof o=="boolean"&&a?o?this.addClass(s):this.removeClass(s):n(s)?this.each(function(t){e(this).toggleClass(s.call(this,t,w(this),o),o)}):this.each(function(){if(a){r=0,o=e(this),c=e7(s);for(var n,o,r,c;n=c[r++];)o.hasClass(n)?o.removeClass(n):o.addClass(n)}else(s===void 0||i==="boolean")&&(n=w(this),n&&t.set(this,"__className__",n),this.setAttribute&&this.setAttribute("class",n||s===!1?"":t.get(this,"__className__")||""))})},hasClass:function(t){for(var e,s=0,n=" "+t+" ";e=this[s++];)if(e.nodeType===1&&(" "+y(w(e))+" ").indexOf(n)>-1)return!0;return!1}}),t1=/\r/g,e.fn.extend({val:function(i){var t,s,a,o=this[0];if(!arguments.length){if(o)return t=e.valHooks[o.type]||e.valHooks[o.nodeName.toLowerCase()],t&&"get"in t&&(s=t.get(o,"value"))!==void 0?s:(s=o.value,typeof s=="string"?s.replace(t1,""):s??"");return}return a=n(i),this.each(function(s){var n;if(this.nodeType!==1)return;a?n=i.call(this,s,e(this).val()):n=i,n==null?n="":typeof n=="number"?n+="":Array.isArray(n)&&(n=e.map(n,function(e){return e==null?"":e+""})),t=e.valHooks[this.type]||e.valHooks[this.nodeName.toLowerCase()],(!t||!("set"in t)||t.set(this,n,"value")===void 0)&&(this.value=n)})}}),e.extend({valHooks:{option:{get:function(t){var n=e.find.attr(t,"value");return n??y(e.text(t))}},select:{get:function(a){var t,n,i,r=a.options,o=a.selectedIndex,s=a.type==="select-one",c=s?null:[],l=s?o+1:r.length;for(o<0?n=l:n=s?o:0;n<l;n++)if(t=r[n],(t.selected||n===o)&&!t.disabled&&(!t.parentNode.disabled||!d(t.parentNode,"optgroup"))){if(i=e(t).val(),s)return i;c.push(i)}return c},set:function(n,r){for(var t,s,o=n.options,i=e.makeArray(r),a=o.length;a--;)t=o[a],(t.selected=e.inArray(e.valHooks.option.get(t),i)>-1)&&(s=!0);return s||(n.selectedIndex=-1),i}}}}),e.each(["radio","checkbox"],function(){e.valHooks[this]={set:function(t,n){if(Array.isArray(n))return t.checked=e.inArray(e(t).val(),n)>-1}},i.checkOn||(e.valHooks[this].get=function(e){return e.getAttribute("value")===null?"on":e.value})}),i.focusin="onfocusin"in s,er=/^(?:focusinfocus|focusoutblur)$/,ey=function(e){e.stopPropagation()},e.extend(e.event,{trigger:function(i,h,r,v){var l,d,u,m,p,b,j,g=[r||o],a=V.call(i,"type")?i.type:i,f=V.call(i,"namespace")?i.namespace.split("."):[],c=p=l=r=r||o;if(r.nodeType===3||r.nodeType===8)return;if(er.test(a+e.event.triggered))return;if(a.indexOf(".")>-1&&(f=a.split("."),a=f.shift(),f.sort()),m=a.indexOf(":")<0&&"on"+a,i=i[e.expando]?i:new e.Event(a,typeof i=="object"&&i),i.isTrigger=v?2:3,i.namespace=f.join("."),i.rnamespace=i.namespace?new RegExp("(^|\\.)"+f.join("\\.(?:.*\\.|)")+"(\\.|$)"):null,i.result=void 0,i.target||(i.target=r),h=h==null?[i]:e.makeArray(h,[i]),d=e.event.special[a]||{},!v&&d.trigger&&d.trigger.apply(r,h)===!1)return;if(!v&&!d.noBubble&&!_(r)){for(j=d.delegateType||a,er.test(j+a)||(c=c.parentNode);c;c=c.parentNode)g.push(c),l=c;l===(r.ownerDocument||o)&&g.push(l.defaultView||l.parentWindow||s)}for(b=0;(c=g[b++])&&!i.isPropagationStopped();)p=c,i.type=b>1?j:d.bindType||a,u=(t.get(c,"events")||Object.create(null))[i.type]&&t.get(c,"handle"),u&&u.apply(c,h),u=m&&c[m],u&&u.apply&&S(c)&&(i.result=u.apply(c,h),i.result===!1&&i.preventDefault());return i.type=a,!v&&!i.isDefaultPrevented()&&(!d._default||d._default.apply(g.pop(),h)===!1)&&S(r)&&m&&n(r[a])&&!_(r)&&(l=r[m],l&&(r[m]=null),e.event.triggered=a,i.isPropagationStopped()&&p.addEventListener(a,ey),r[a](),i.isPropagationStopped()&&p.removeEventListener(a,ey),e.event.triggered=void 0,l&&(r[m]=l)),i.result},simulate:function(t,n,s){var o=e.extend(new e.Event,s,{type:t,isSimulated:!0});e.event.trigger(o,null,n)}}),e.fn.extend({trigger:function(t,n){return this.each(function(){e.event.trigger(t,n,this)})},triggerHandler:function(n,s){var t=this[0];if(t)return e.event.trigger(n,s,t,!0)}}),i.focusin||e.each({focus:"focusin",blur:"focusout"},function(s,n){var o=function(t){e.event.simulate(n,t.target,e.event.fix(t))};e.event.special[n]={setup:function(){var e=this.ownerDocument||this.document||this,i=t.access(e,n);i||e.addEventListener(s,o,!0),t.access(e,n,(i||0)+1)},teardown:function(){var e=this.ownerDocument||this.document||this,i=t.access(e,n)-1;i?t.access(e,n,i):(e.removeEventListener(s,o,!0),t.remove(e,n))}}}),M=s.location,ei={guid:Date.now()},U=/\?/,e.parseXML=function(n){var t,o;if(!n||typeof n!="string")return null;try{t=(new s.DOMParser).parseFromString(n,"text/xml")}catch(e){}return o=t&&t.getElementsByTagName("parsererror")[0],(!t||o)&&e.error("Invalid XML: "+(o?e.map(o.childNodes,function(e){return e.textContent}).join("\n"):n)),t};var tm=/\[\]$/,eV=/\r?\n/g,tf=/^(?:submit|button|image|reset|file)$/i,t6=/^(?:input|select|textarea|keygen)/i;function eo(n,t,s,o){var i;if(Array.isArray(t))e.each(t,function(t,e){s||tm.test(n)?o(n,e):eo(n+"["+(typeof e=="object"&&e!=null?t:"")+"]",e,s,o)});else if(!s&&k(t)==="object")for(i in t)eo(n+"["+i+"]",t[i],s,o);else o(n,t)}e.param=function(t,a){var s,o=[],i=function(s,e){var t=n(e)?e():e;o[o.length]=encodeURIComponent(s)+"="+encodeURIComponent(t??"")};if(t==null)return"";if(Array.isArray(t)||t.jquery&&!e.isPlainObject(t))e.each(t,function(){i(this.name,this.value)});else for(s in t)eo(s,t[s],a,i);return o.join("&")},e.fn.extend({serialize:function(){return e.param(this.serializeArray())},serializeArray:function(){return this.map(function(){var t=e.prop(this,"elements");return t?e.makeArray(t):this}).filter(function(){var t=this.type;return this.name&&!e(this).is(":disabled")&&t6.test(this.nodeName)&&!tf.test(t)&&(this.checked||!E.test(t))}).map(function(s,n){var t=e(this).val();return t==null?null:Array.isArray(t)?e.map(t,function(e){return{name:n.name,value:e.replace(eV,"\r\n")}}):{name:n.name,value:t.replace(eV,"\r\n")}}).get()}});var e,t,n,o,i,a,l,h,f,p,g,v,b,j,_,O,x,C,E,S,M,F,L,R,H,I,B,V,$,U,Y,G,X,Q,e5,eg,e6,ef,td,tl,eb,eh,e0,e4,ti,to,ts,tn,te,eY,ee,eq,eU,eI,eH,es,t0,eu,e2,e3,eM,ep,ed,eT,eX,t4,t2,tr,tc,t1,er,ey,ei,tg=/%20/g,t5=/#.*$/,tN=/([?&])_=[^&]*/,tv=/^(.*?):[ \t]*([^\r\n]*)$/mg,t9=/^(?:about|app|app-storage|.+-extension|file|res|widget):$/,tb=/^(?:GET|HEAD)$/,tj=/^\/\//,eE={},el={},eO="*/".concat("*"),e8=o.createElement("a"),ek,et,eQ,tu,eP;e8.href=M.href;function e_(e){return function(s,o){typeof s!="string"&&(o=s,s="*");var t,i=0,a=s.toLowerCase().match(h)||[];if(n(o))for(;t=a[i++];)t[0]==="+"?(t=t.slice(1)||"*",(e[t]=e[t]||[]).unshift(o)):(e[t]=e[t]||[]).push(o)}}function ex(i,n,a,r){var s={},o=i===el;function t(c){var l;return s[c]=!0,e.each(i[c]||[],function(c,i){var e=i(n,a,r);if(typeof e=="string"&&!o&&!s[e])return n.dataTypes.unshift(e),t(e),!1;if(o)return!(l=e)}),l}return t(n.dataTypes[0])||!s["*"]&&t("*")}function ev(s,o){var t,n,i=e.ajaxSettings.flatOptions||{};for(t in o)o[t]!==void 0&&((i[t]?s:n||(n={}))[t]=o[t]);return n&&e.extend(!0,s,n),s}function tk(s,c,r){for(var t,n,o,a,i=s.contents,e=s.dataTypes;e[0]==="*";)e.shift(),o===void 0&&(o=s.mimeType||c.getResponseHeader("Content-Type"));if(o)for(t in i)if(i[t]&&i[t].test(o)){e.unshift(t);break}if(e[0]in r)n=e[0];else{for(t in r){if(!e[0]||s.converters[t+" "+e[0]]){n=t;break}a||(a=t)}n=n||a}if(n)return n!==e[0]&&e.unshift(n),r[n]}function tA(n,s,l,d){var e,t,o,a,r,i={},c=n.dataTypes.slice();if(c[1])for(t in n.converters)i[t.toLowerCase()]=n.converters[t];for(e=c.shift();e;)if(n.responseFields[e]&&(l[n.responseFields[e]]=s),!o&&d&&n.dataFilter&&(s=n.dataFilter(s,n.dataType)),o=e,e=c.shift(),e)if(e==="*")e=o;else if(o!=="*"&&o!==e){if(t=i[o+" "+e]||i["* "+e],!t)for(r in i)if(a=r.split(" "),a[1]===e&&(t=i[o+" "+a[0]]||i["* "+a[0]],t)){t===!0?t=i[r]:i[r]!==!0&&(e=a[0],c.unshift(a[1]));break}if(t!==!0)if(t&&n.throws)s=t(s);else try{s=t(s)}catch(n){return{state:"parsererror",error:t?n:"No conversion from "+o+" to "+e}}}return{state:"success",data:s}}return e.extend({active:0,lastModified:{},etag:{},ajaxSettings:{url:M.href,type:"GET",isLocal:t9.test(M.protocol),global:!0,processData:!0,async:!0,contentType:"application/x-www-form-urlencoded; charset=UTF-8",accepts:{"*":eO,text:"text/plain",html:"text/html",xml:"application/xml, text/xml",json:"application/json, text/javascript"},contents:{xml:/\bxml\b/,html:/\bhtml/,json:/\bjson\b/},responseFields:{xml:"responseXML",text:"responseText",json:"responseJSON"},converters:{"* text":String,"text html":!0,"text json":JSON.parse,"text xml":e.parseXML},flatOptions:{url:!0,context:!0}},ajaxSetup:function(t,n){return n?ev(ev(t,e.ajaxSettings),n):ev(e.ajaxSettings,t)},ajaxPrefilter:e_(eE),ajaxTransport:e_(el),ajax:function(g,r){typeof g=="object"&&(r=g,g=void 0),r=r||{};var i,a,l,d,u,m,p,b,_,w,t=e.ajaxSetup({},r),c=t.context||t,j=t.context&&(c.nodeType||c.jquery)?e(c):e.event,y=e.Deferred(),E=e.Callbacks("once memory"),f=t.statusCode||{},O={},x={},C="canceled",n={readyState:0,getResponseHeader:function(t){if(a){if(!u){u={};for(var e;e=tv.exec(w);)u[e[1].toLowerCase()+" "]=(u[e[1].toLowerCase()+" "]||[]).concat(e[2])}e=u[t.toLowerCase()+" "]}return e==null?null:e.join(", ")},getAllResponseHeaders:function(){return a?w:null},setRequestHeader:function(e,t){return a==null&&(e=x[e.toLowerCase()]=x[e.toLowerCase()]||e,O[e]=t),this},overrideMimeType:function(e){return a==null&&(t.mimeType=e),this},statusCode:function(e){var t;if(e)if(a)n.always(e[n.status]);else for(t in e)f[t]=[f[t],e[t]];return this},abort:function(t){var e=t||C;return d&&d.abort(e),v(0,e),this}};if(y.promise(n),t.url=((g||t.url||M.href)+"").replace(tj,M.protocol+"//"),t.type=r.method||r.type||t.method||t.type,t.dataTypes=(t.dataType||"*").toLowerCase().match(h)||[""],t.crossDomain==null){l=o.createElement("a");try{l.href=t.url,l.href=l.href,t.crossDomain=e8.protocol+"//"+e8.host!==l.protocol+"//"+l.host}catch(e){t.crossDomain=!0}}if(t.data&&t.processData&&typeof t.data!="string"&&(t.data=e.param(t.data,t.traditional)),ex(eE,t,r,n),a)return n;m=e.event&&t.global,m&&e.active++===0&&e.event.trigger("ajaxStart"),t.type=t.type.toUpperCase(),t.hasContent=!tb.test(t.type),i=t.url.replace(t5,""),t.hasContent?t.data&&t.processData&&(t.contentType||"").indexOf("application/x-www-form-urlencoded")===0&&(t.data=t.data.replace(tg,"+")):(p=t.url.slice(i.length),t.data&&(t.processData||typeof t.data=="string")&&(i+=(U.test(i)?"&":"?")+t.data,delete t.data),t.cache===!1&&(i=i.replace(tN,"$1"),p=(U.test(i)?"&":"?")+"_="+ei.guid+++p),t.url=i+p),t.ifModified&&(e.lastModified[i]&&n.setRequestHeader("If-Modified-Since",e.lastModified[i]),e.etag[i]&&n.setRequestHeader("If-None-Match",e.etag[i])),(t.data&&t.hasContent&&t.contentType!==!1||r.contentType)&&n.setRequestHeader("Content-Type",t.contentType),n.setRequestHeader("Accept",t.dataTypes[0]&&t.accepts[t.dataTypes[0]]?t.accepts[t.dataTypes[0]]+(t.dataTypes[0]!=="*"?", "+eO+"; q=0.01":""):t.accepts["*"]);for(b in t.headers)n.setRequestHeader(b,t.headers[b]);if(t.beforeSend&&(t.beforeSend.call(c,n,t)===!1||a))return n.abort();if(C="abort",E.add(t.complete),n.done(t.success),n.fail(t.error),d=ex(el,t,r,n),d){if(n.readyState=1,m&&j.trigger("ajaxSend",[n,t]),a)return n;t.async&&t.timeout>0&&(_=s.setTimeout(function(){n.abort("timeout")},t.timeout));try{a=!1,d.send(O,v)}catch(e){if(a)throw e;v(-1,e)}}else v(-1,"No Transport");function v(o,v,b,O){var l,u,h,p,g,r=v;if(a)return;a=!0,_&&s.clearTimeout(_),d=void 0,w=O||"",n.readyState=o>0?4:0,l=o>=200&&o<300||o===304,b&&(u=tk(t,n,b)),!l&&e.inArray("script",t.dataTypes)>-1&&e.inArray("json",t.dataTypes)<0&&(t.converters["text script"]=function(){}),u=tA(t,u,n,l),l?(t.ifModified&&(h=n.getResponseHeader("Last-Modified"),h&&(e.lastModified[i]=h),h=n.getResponseHeader("etag"),h&&(e.etag[i]=h)),o===204||t.type==="HEAD"?r="nocontent":o===304?r="notmodified":(r=u.state,g=u.data,p=u.error,l=!p)):(p=r,(o||!r)&&(r="error",o<0&&(o=0))),n.status=o,n.statusText=(v||r)+"",l?y.resolveWith(c,[g,r,n]):y.rejectWith(c,[n,r,p]),n.statusCode(f),f=void 0,m&&j.trigger(l?"ajaxSuccess":"ajaxError",[n,t,l?g:p]),E.fireWith(c,[n,r]),m&&(j.trigger("ajaxComplete",[n,t]),--e.active||e.event.trigger("ajaxStop"))}return n},getJSON:function(t,n,s){return e.get(t,n,s,"json")},getScript:function(t,n){return e.get(t,void 0,n,"script")}}),e.each(["get","post"],function(s,t){e[t]=function(o,s,i,a){return n(s)&&(a=a||i,i=s,s=void 0),e.ajax(e.extend({url:o,type:t,dataType:a,data:s,success:i},e.isPlainObject(o)&&o))}}),e.ajaxPrefilter(function(e){var t;for(t in e.headers)t.toLowerCase()==="content-type"&&(e.contentType=e.headers[t]||"")}),e._evalUrl=function(t,n,s){return e.ajax({url:t,type:"GET",dataType:"script",cache:!0,async:!1,global:!1,converters:{"text script":function(){}},dataFilter:function(t){e.globalEval(t,n,s)}})},e.fn.extend({wrapAll:function(t){var s;return this[0]&&(n(t)&&(t=t.call(this[0])),s=e(t,this[0].ownerDocument).eq(0).clone(!0),this[0].parentNode&&s.insertBefore(this[0]),s.map(function(){for(var e=this;e.firstElementChild;)e=e.firstElementChild;return e}).append(this)),this},wrapInner:function(t){return n(t)?this.each(function(n){e(this).wrapInner(t.call(this,n))}):this.each(function(){var n=e(this),s=n.contents();s.length?s.wrapAll(t):n.append(t)})},wrap:function(t){var s=n(t);return this.each(function(n){e(this).wrapAll(s?t.call(this,n):t)})},unwrap:function(t){return this.parent(t).not("body").each(function(){e(this).replaceWith(this.childNodes)}),this}}),e.expr.pseudos.hidden=function(t){return!e.expr.pseudos.visible(t)},e.expr.pseudos.visible=function(e){return!!(e.offsetWidth||e.offsetHeight||e.getClientRects().length)},e.ajaxSettings.xhr=function(){try{return new s.XMLHttpRequest}catch(e){}},ek={0:200,1223:204},F=e.ajaxSettings.xhr(),i.cors=!!F&&"withCredentials"in F,i.ajax=F=!!F,e.ajaxTransport(function(e){var t,n;if(i.cors||F&&!e.crossDomain)return{send:function(a,r){var i,o=e.xhr();if(o.open(e.type,e.url,e.async,e.username,e.password),e.xhrFields)for(i in e.xhrFields)o[i]=e.xhrFields[i];e.mimeType&&o.overrideMimeType&&o.overrideMimeType(e.mimeType),!e.crossDomain&&!a["X-Requested-With"]&&(a["X-Requested-With"]="XMLHttpRequest");for(i in a)o.setRequestHeader(i,a[i]);t=function(e){return function(){t&&(t=n=o.onload=o.onerror=o.onabort=o.ontimeout=o.onreadystatechange=null,e==="abort"?o.abort():e==="error"?typeof o.status!="number"?r(0,"error"):r(o.status,o.statusText):r(ek[o.status]||o.status,o.statusText,(o.responseType||"text")!=="text"||typeof o.responseText!="string"?{binary:o.response}:{text:o.responseText},o.getAllResponseHeaders()))}},o.onload=t(),n=o.onerror=o.ontimeout=t("error"),o.onabort!==void 0?o.onabort=n:o.onreadystatechange=function(){o.readyState===4&&s.setTimeout(function(){t&&n()})},t=t("abort");try{o.send(e.hasContent&&e.data||null)}catch(e){if(t)throw e}},abort:function(){t&&t()}}}),e.ajaxPrefilter(function(e){e.crossDomain&&(e.contents.script=!1)}),e.ajaxSetup({accepts:{script:"text/javascript, application/javascript, application/ecmascript, application/x-ecmascript"},contents:{script:/\b(?:java|ecma)script\b/},converters:{"text script":function(t){return e.globalEval(t),t}}}),e.ajaxPrefilter("script",function(e){e.cache===void 0&&(e.cache=!1),e.crossDomain&&(e.type="GET")}),e.ajaxTransport("script",function(t){if(t.crossDomain||t.scriptAttrs){var n,s;return{send:function(a,i){s=e("<script>").attr(t.scriptAttrs||{}).prop({charset:t.scriptCharset,src:t.url}).on("load error",n=function(e){s.remove(),n=null,e&&i(e.type==="error"?404:200,e.type)}),o.head.appendChild(s[0])},abort:function(){n&&n()}}}}),et=[],G=/(=)\?(?=&|$)|\?\?/,e.ajaxSetup({jsonp:"callback",jsonpCallback:function(){var t=et.pop()||e.expando+"_"+ei.guid++;return this[t]=!0,t}}),e.ajaxPrefilter("json jsonp",function(t,c,l){var o,i,a,r=t.jsonp!==!1&&(G.test(t.url)?"url":typeof t.data=="string"&&(t.contentType||"").indexOf("application/x-www-form-urlencoded")===0&&G.test(t.data)&&"data");if(r||t.dataTypes[0]==="jsonp")return o=t.jsonpCallback=n(t.jsonpCallback)?t.jsonpCallback():t.jsonpCallback,r?t[r]=t[r].replace(G,"$1"+o):t.jsonp!==!1&&(t.url+=(U.test(t.url)?"&":"?")+t.jsonp+"="+o),t.converters["script json"]=function(){return a||e.error(o+" was not called"),a[0]},t.dataTypes[0]="json",i=s[o],s[o]=function(){a=arguments},l.always(function(){i===void 0?e(s).removeProp(o):s[o]=i,t[o]&&(t.jsonpCallback=c.jsonpCallback,et.push(o)),a&&n(i)&&i(a[0]),a=i=void 0}),"script"}),i.createHTMLDocument=function(){var e=o.implementation.createHTMLDocument("").body;return e.innerHTML="<form></form><form></form>",e.childNodes.length===2}(),e.parseHTML=function(a,t,c){if(typeof a!="string")return[];typeof t=="boolean"&&(c=t,t=!1);var n,s,r;return t||(i.createHTMLDocument?(t=o.implementation.createHTMLDocument(""),r=t.createElement("base"),r.href=o.location.href,t.head.appendChild(r)):t=o),n=e4.exec(a),s=!c&&[],n?[t.createElement(n[1])]:(n=eS([a],t,s),s&&s.length&&e(s).remove(),e.merge([],n.childNodes))},e.fn.load=function(s,t,o){var i,c,l,a=this,r=s.indexOf(" ");return r>-1&&(i=y(s.slice(r)),s=s.slice(0,r)),n(t)?(o=t,t=void 0):t&&typeof t=="object"&&(c="POST"),a.length>0&&e.ajax({url:s,type:c||"GET",dataType:"html",data:t}).done(function(t){l=arguments,a.html(i?e("<div>").append(e.parseHTML(t)).find(i):t)}).always(o&&function(e,t){a.each(function(){o.apply(this,l||[e.responseText,t,e])})}),this},e.expr.pseudos.animated=function(t){return e.grep(e.timers,function(e){return t===e.elem}).length},e.offset={setOffset:function(s,t,f){var o,a,r,c,l,h,m,d=e.css(s,"position"),u=e(s),i={};d==="static"&&(s.style.position="relative"),o=u.offset(),r=e.css(s,"top"),l=e.css(s,"left"),m=(d==="absolute"||d==="fixed")&&(r+l).indexOf("auto")>-1,m?(c=u.position(),h=c.top,a=c.left):(h=parseFloat(r)||0,a=parseFloat(l)||0),n(t)&&(t=t.call(s,f,e.extend({},o))),t.top!=null&&(i.top=t.top-o.top+h),t.left!=null&&(i.left=t.left-o.left+a),"using"in t?t.using.call(s,i):u.css(i)}},e.fn.extend({offset:function(o){if(arguments.length)return o===void 0?this:this.each(function(t){e.offset.setOffset(this,o,t)});var n,s,t=this[0];if(!t)return;return t.getClientRects().length?(n=t.getBoundingClientRect(),s=t.ownerDocument.defaultView,{top:n.top+s.pageYOffset,left:n.left+s.pageXOffset}):{top:0,left:0}},position:function(){if(!this[0])return;var t,o,i,n=this[0],s={top:0,left:0};if(e.css(n,"position")==="fixed")o=n.getBoundingClientRect();else{for(o=this.offset(),i=n.ownerDocument,t=n.offsetParent||i.documentElement;t&&(t===i.body||t===i.documentElement)&&e.css(t,"position")==="static";)t=t.parentNode;t&&t!==n&&t.nodeType===1&&(s=e(t).offset(),s.top+=e.css(t,"borderTopWidth",!0),s.left+=e.css(t,"borderLeftWidth",!0))}return{top:o.top-s.top-e.css(n,"marginTop",!0),left:o.left-s.left-e.css(n,"marginLeft",!0)}},offsetParent:function(){return this.map(function(){for(var t=this.offsetParent;t&&e.css(t,"position")==="static";)t=t.offsetParent;return t||j})}}),e.each({scrollLeft:"pageXOffset",scrollTop:"pageYOffset"},function(t,n){var s="pageYOffset"===n;e.fn[t]=function(e){return g(this,function(t,i,o){var e;if(_(t)?e=t:t.nodeType===9&&(e=t.defaultView),o===void 0)return e?e[n]:t[i];e?e.scrollTo(s?e.pageXOffset:o,s?o:e.pageYOffset):t[i]=o},t,e,arguments.length)}}),e.each(["top","left"],function(n,t){e.cssHooks[t]=eD(i.pixelPosition,function(s,n){if(n)return n=W(s,t),X.test(n)?e(s).position()[t]+"px":n})}),e.each({Height:"height",Width:"width"},function(t,n){e.each({padding:"inner"+t,content:n,"":"outer"+t},function(s,o){e.fn[o]=function(i,c){var a=arguments.length&&(s||typeof i!="boolean"),r=s||(i===!0||c===!0?"margin":"border");return g(this,function(n,i,a){var s;return _(n)?o.indexOf("outer")===0?n["inner"+t]:n.document.documentElement["client"+t]:n.nodeType===9?(s=n.documentElement,Math.max(n.body["scroll"+t],s["scroll"+t],n.body["offset"+t],s["offset"+t],s["client"+t])):a===void 0?e.css(n,i,r):e.style(n,i,a,r)},n,a?i:void 0,a)}})}),e.each(["ajaxStart","ajaxStop","ajaxComplete","ajaxError","ajaxSuccess","ajaxSend"],function(n,t){e.fn[t]=function(e){return this.on(t,e)}}),e.fn.extend({bind:function(e,t,n){return this.on(e,null,t,n)},unbind:function(e,t){return this.off(e,null,t)},delegate:function(e,t,n,s){return this.on(t,e,n,s)},undelegate:function(e,t,n){return arguments.length===1?this.off(e,"**"):this.off(t,e||"**",n)},hover:function(e,t){return this.mouseenter(e).mouseleave(t||e)}}),e.each(("blur focus focusin focusout resize scroll click dblclick mousedown mouseup mousemove mouseover mouseout mouseenter mouseleave change select submit keydown keypress keyup contextmenu").split(" "),function(n,t){e.fn[t]=function(e,n){return arguments.length>0?this.on(t,null,e,n):this.trigger(t)}}),eQ=/^[\s\uFEFF\xA0]+|[\s\uFEFF\xA0]+$/g,e.proxy=function(t,s){var o,i,a;return typeof s=="string"&&(i=t[s],s=t,t=i),n(t)?(a=v.call(arguments,2),o=function(){return t.apply(s||this,a.concat(v.call(arguments)))},o.guid=t.guid=t.guid||e.guid++,o):void 0},e.holdReady=function(t){t?e.readyWait++:e.ready(!0)},e.isArray=Array.isArray,e.parseJSON=JSON.parse,e.nodeName=d,e.isFunction=n,e.isWindow=_,e.camelCase=m,e.type=k,e.now=Date.now,e.isNumeric=function(t){var n=e.type(t);return(n==="number"||n==="string")&&!isNaN(t-parseFloat(t))},e.trim=function(e){return e==null?"":(e+"").replace(eQ,"")},typeof define=="function"&&define.amd&&define("jquery",[],function(){return e}),tu=s.jQuery,eP=s.$,e.noConflict=function(t){return s.$===e&&(s.$=eP),t&&s.jQuery===e&&(s.jQuery=tu),e},typeof tD=="undefined"&&(s.jQuery=s.$=e),e}),function(e){e.fn.hoverIntent=function(r,c){var n,o,i,a,t={sensitivity:7,interval:100,timeout:0},t=e.extend(t,c?{over:r,out:c}:r),s=function(e){o=e.pageX,i=e.pageY},l=function(c,r){if(r.hoverIntent_t=clearTimeout(r.hoverIntent_t),Math.abs(a-o)+Math.abs(n-i)<t.sensitivity)return e(r).unbind("mousemove",s),r.hoverIntent_s=1,t.over.apply(r,[c]);a=o,n=i,r.hoverIntent_t=setTimeout(function(){l(c,r)},t.interval)},u=function(n,e){return e.hoverIntent_t=clearTimeout(e.hoverIntent_t),e.hoverIntent_s=0,t.out.apply(e,[n])},d=function(r){var i=jQuery.extend({},r),o=this;o.hoverIntent_t&&(o.hoverIntent_t=clearTimeout(o.hoverIntent_t)),r.type=="mouseenter"?(a=i.pageX,n=i.pageY,e(o).bind("mousemove",s),o.hoverIntent_s!=1&&(o.hoverIntent_t=setTimeout(function(){l(i,o)},t.interval))):(e(o).unbind("mousemove",s),o.hoverIntent_s==1&&(o.hoverIntent_t=setTimeout(function(){u(i,o)},t.timeout)))};return this.bind("mouseenter",d).bind("mouseleave",d)}}(jQuery),jQuery.cookie=function(n,s,e){if(typeof s!="undefined"){e=e||{},s===null&&(s='',e.expires=-1);var t,o,i,a,r,c,l,u,d='';e.expires&&(typeof e.expires=="number"||e.expires.toUTCString)&&(typeof e.expires=="number"?(t=new Date,t.setTime(t.getTime()+e.expires*24*60*60*1e3)):t=e.expires,d="; expires="+t.toUTCString()),u=e.path?"; path="+e.path:'',c=e.domain?"; domain="+e.domain:'',l=e.secure?"; secure":'',document.cookie=[n,"=",encodeURIComponent(s),d,u,c,l].join('')}else{if(r=null,document.cookie&&document.cookie!=''){i=document.cookie.split(";");for(o=0;o<i.length;o++)if(a=jQuery.trim(i[o]),a.substring(0,n.length+1)==n+"="){r=decodeURIComponent(a.substring(n.length+1));break}}return r}},function(t,e){'use strict';typeof define=="function"&&define.amd?define(e):typeof exports!="undefined"?module.exports=e():t.simpleStorage=e()}(this,function(){'use strict';var m="0.1.3",e=!1,o=0,i=!1,n=null;function h(){window.localStorage.setItem("__simpleStorageInitTest","tmpval"),window.localStorage.removeItem("__simpleStorageInitTest"),l(),s(),u(),"addEventListener"in window&&window.addEventListener("pageshow",function(e){e.persisted&&r()},!1),i=!0}function u(){"addEventListener"in window?window.addEventListener("storage",r,!1):document.attachEvent("onstorage",r)}function r(){try{l()}catch(e){i=!1;return}s()}function l(){var t=localStorage.getItem("simpleStorage");try{e=JSON.parse(t)||{}}catch(t){e={}}o=d()}function t(){try{localStorage.setItem("simpleStorage",JSON.stringify(e)),o=d()}catch(e){return e}return!0}function d(){var e=localStorage.getItem("simpleStorage");return e?String(e).length:0}function s(){var o,i,a,l,u,r=1/0,d=0;if(clearTimeout(n),!e||!e.__simpleStorage_meta||!e.__simpleStorage_meta.TTL)return;l=+new Date,i=e.__simpleStorage_meta.TTL.keys||[],a=e.__simpleStorage_meta.TTL.expire||{};for(o=0,u=i.length;o<u;o++)if(a[i[o]]<=l)d++,delete e[i[o]],delete a[i[o]];else{a[i[o]]<r&&(r=a[i[o]]);break}r!=1/0&&(n=setTimeout(s,Math.min(r-l,2147483647))),d&&(i.splice(0,d),c(),t())}function a(o,a){var t,i,r=+new Date,l=!1;if(a=Number(a)||0,a!==0)if(e.hasOwnProperty(o)){if(e.__simpleStorage_meta||(e.__simpleStorage_meta={}),e.__simpleStorage_meta.TTL||(e.__simpleStorage_meta.TTL={expire:{},keys:[]}),e.__simpleStorage_meta.TTL.expire[o]=r+a,e.__simpleStorage_meta.TTL.expire.hasOwnProperty(o))for(t=0,i=e.__simpleStorage_meta.TTL.keys.length;t<i;t++)e.__simpleStorage_meta.TTL.keys[t]==o&&e.__simpleStorage_meta.TTL.keys.splice(t);for(t=0,i=e.__simpleStorage_meta.TTL.keys.length;t<i;t++)if(e.__simpleStorage_meta.TTL.expire[e.__simpleStorage_meta.TTL.keys[t]]>r+a){e.__simpleStorage_meta.TTL.keys.splice(t,0,o),l=!0;break}l||e.__simpleStorage_meta.TTL.keys.push(o)}else return!1;else if(e&&e.__simpleStorage_meta&&e.__simpleStorage_meta.TTL){if(e.__simpleStorage_meta.TTL.expire.hasOwnProperty(o)){delete e.__simpleStorage_meta.TTL.expire[o];for(t=0,i=e.__simpleStorage_meta.TTL.keys.length;t<i;t++)if(e.__simpleStorage_meta.TTL.keys[t]==o){e.__simpleStorage_meta.TTL.keys.splice(t,1);break}}c()}return clearTimeout(n),e&&e.__simpleStorage_meta&&e.__simpleStorage_meta.TTL&&e.__simpleStorage_meta.TTL.keys.length&&(n=setTimeout(s,Math.min(Math.max(e.__simpleStorage_meta.TTL.expire[e.__simpleStorage_meta.TTL.keys[0]]-r,0),2147483647))),!0}function c(){var s,t=!1,n=!1;if(!e||!e.__simpleStorage_meta)return t;e.__simpleStorage_meta.TTL&&!e.__simpleStorage_meta.TTL.keys.length&&(delete e.__simpleStorage_meta.TTL,t=!0);for(s in e.__simpleStorage_meta)if(e.__simpleStorage_meta.hasOwnProperty(s)){n=!0;break}return n||(delete e.__simpleStorage_meta,t=!0),t}try{h()}catch(e){}return{version:m,canUse:function(){return!!i},set:function(n,s,o){if(n=="__simpleStorage_meta")return!1;if(!e)return!1;if(typeof s=="undefined")return this.deleteKey(n);o=o||{};try{s=JSON.parse(JSON.stringify(s))}catch(e){return e}return e[n]=s,a(n,o.TTL||0),t()},get:function(t){if(!e)return!1;if(e.hasOwnProperty(t)&&t!="__simpleStorage_meta"&&this.getTTL(t))return e[t]},deleteKey:function(n){return!!e&&n in e&&(delete e[n],a(n,0),t())},setTTL:function(n,s){return!!e&&(a(n,s),t())},getTTL:function(t){var n;return!!e&&!!e.hasOwnProperty(t)&&(e.__simpleStorage_meta&&e.__simpleStorage_meta.TTL&&e.__simpleStorage_meta.TTL.expire&&e.__simpleStorage_meta.TTL.expire.hasOwnProperty(t)?(n=Math.max(e.__simpleStorage_meta.TTL.expire[t]-+new Date||0,0),n||!1):1/0)},flush:function(){if(!e)return!1;e={};try{return localStorage.removeItem("simpleStorage"),!0}catch(e){return e}},index:function(){if(!e)return!1;var t,n=[];for(t in e)e.hasOwnProperty(t)&&t!="__simpleStorage_meta"&&n.push(t);return n},storageSize:function(){return o}}}),function(){var o,i,a,r,c,n=[].slice,l={}.hasOwnProperty,d=function(e,t){for(var n in t)l.call(t,n)&&(e[n]=t[n]);function s(){this.constructor=e}return s.prototype=t.prototype,e.prototype=new s,e.__super__=t.prototype,e},t=function(){},s=function(){function e(){}return e.prototype.addEventListener=e.prototype.on,e.prototype.on=function(e,t){return this._callbacks=this._callbacks||{},this._callbacks[e]||(this._callbacks[e]=[]),this._callbacks[e].push(t),this},e.prototype.emit=function(){var e,t,s,o,a=arguments[0],i=2<=arguments.length?n.call(arguments,1):[];if(this._callbacks=this._callbacks||{},e=this._callbacks[a],e)for(t=0,o=e.length;t<o;t++)s=e[t],s.apply(this,i);return this},e.prototype.removeListener=e.prototype.off,e.prototype.removeAllListeners=e.prototype.off,e.prototype.removeEventListener=e.prototype.off,e.prototype.off=function(s,a){var e,t,n,o,i;if(!this._callbacks||arguments.length===0)return this._callbacks={},this;if(e=this._callbacks[s],!e)return this;if(arguments.length===1)return delete this._callbacks[s],this;for(t=n=0,i=e.length;n<i;t=++n)if(o=e[t],o===a){e.splice(t,1);break}return this},e}(),e=function(c){var o,i;d(e,c),e.prototype.Emitter=s,e.prototype.events=["drop","dragstart","dragend","dragenter","dragover","dragleave","addedfile","addedfiles","removedfile","thumbnail","error","errormultiple","processing","processingmultiple","uploadprogress","totaluploadprogress","sending","sendingmultiple","success","successmultiple","canceled","canceledmultiple","complete","completemultiple","reset","maxfilesexceeded","maxfilesreached","queuecomplete"],e.prototype.defaultOptions={url:null,method:"post",withCredentials:!1,parallelUploads:2,uploadMultiple:!1,maxFilesize:256,paramName:"file",createImageThumbnails:!0,maxThumbnailFilesize:10,thumbnailWidth:120,thumbnailHeight:120,filesizeBase:1e3,maxFiles:null,params:{},clickable:!0,ignoreHiddenFiles:!0,acceptedFiles:null,acceptedMimeTypes:null,autoProcessQueue:!0,autoQueue:!0,addRemoveLinks:!1,previewsContainer:null,hiddenInputContainer:"body",capture:null,renameFilename:null,dictDefaultMessage:"Drop files here to upload",dictFallbackMessage:"Your browser does not support drag'n'drop file uploads.",dictFallbackText:"Please use the fallback form below to upload your files like in the olden days.",dictFileTooBig:"File is too big ({{filesize}}MiB). Max filesize: {{maxFilesize}}MiB.",dictInvalidFileType:"You can't upload files of this type.",dictResponseError:"Server responded with {{statusCode}} code.",dictCancelUpload:"Cancel upload",dictCancelUploadConfirmation:"Are you sure you want to cancel this upload?",dictRemoveFile:"Remove file",dictRemoveFileConfirmation:null,dictMaxFilesExceeded:"You can not upload any more files.",accept:function(t,e){return e()},init:function(){return t},forceFallback:!1,fallback:function(){var t,n,s,o,i,a;this.element.className=""+this.element.className+" dz-browser-not-supported",i=this.element.getElementsByTagName("div");for(o=0,a=i.length;o<a;o++)if(s=i[o],/(^| )dz-message($| )/.test(s.className)){n=s,s.className="dz-message";continue}return n||(n=e.createElement('<div class="dz-message"><span></span></div>'),this.element.appendChild(n)),t=n.getElementsByTagName("span")[0],t&&(t.textContent!=null?t.textContent=this.options.dictFallbackMessage:t.innerText!=null&&(t.innerText=this.options.dictFallbackMessage)),this.element.appendChild(this.getFallbackForm())},resize:function(t){var n,e={srcX:0,srcY:0,srcWidth:t.width,srcHeight:t.height},s=t.width/t.height;return e.optWidth=this.options.thumbnailWidth,e.optHeight=this.options.thumbnailHeight,e.optWidth==null&&e.optHeight==null?(e.optWidth=e.srcWidth,e.optHeight=e.srcHeight):e.optWidth==null?e.optWidth=s*e.optHeight:e.optHeight==null&&(e.optHeight=1/s*e.optWidth),n=e.optWidth/e.optHeight,t.height<e.optHeight||t.width<e.optWidth?(e.trgHeight=e.srcHeight,e.trgWidth=e.srcWidth):s>n?(e.srcHeight=t.height,e.srcWidth=e.srcHeight*n):(e.srcWidth=t.width,e.srcHeight=e.srcWidth/n),e.srcX=(t.width-e.srcWidth)/2,e.srcY=(t.height-e.srcHeight)/2,e},drop:function(){return this.element.classList.remove("dz-drag-hover")},dragstart:t,dragend:function(){return this.element.classList.remove("dz-drag-hover")},dragenter:function(){return this.element.classList.add("dz-drag-hover")},dragover:function(){return this.element.classList.add("dz-drag-hover")},dragleave:function(){return this.element.classList.remove("dz-drag-hover")},paste:t,reset:function(){return this.element.classList.remove("dz-started")},addedfile:function(t){var n,s,o,i,a,r,c,l,d,u,h,m,f;if(this.element===this.previewsContainer&&this.element.classList.add("dz-started"),this.previewsContainer){t.previewElement=e.createElement(this.options.previewTemplate.trim()),t.previewTemplate=t.previewElement,this.previewsContainer.appendChild(t.previewElement),c=t.previewElement.querySelectorAll("[data-dz-name]");for(s=0,f=c.length;s<f;s++)i=c[s],i.textContent=this._renameFilename(t.name);r=t.previewElement.querySelectorAll("[data-dz-size]");for(o=0,u=r.length;o<u;o++)i=r[o],i.innerHTML=this.filesize(t.size);this.options.addRemoveLinks&&(t._removeLink=e.createElement('<a class="dz-remove" href="javascript:undefined;" data-dz-remove>'+this.options.dictRemoveFile+"</a>"),t.previewElement.appendChild(t._removeLink)),m=function(n){return function(s){return s.preventDefault(),s.stopPropagation(),t.status===e.UPLOADING?e.confirm(n.options.dictCancelUploadConfirmation,function(){return n.removeFile(t)}):n.options.dictRemoveFileConfirmation?e.confirm(n.options.dictRemoveFileConfirmation,function(){return n.removeFile(t)}):n.removeFile(t)}}(this),a=t.previewElement.querySelectorAll("[data-dz-remove]"),l=[];for(n=0,d=a.length;n<d;n++)h=a[n],l.push(h.addEventListener("click",m));return l}},removedfile:function(e){var t;return e.previewElement&&(t=e.previewElement)!=null&&t.parentNode.removeChild(e.previewElement),this._updateMaxFilesReachedClass()},thumbnail:function(e,i){var t,n,s,o;if(e.previewElement){e.previewElement.classList.remove("dz-file-preview"),s=e.previewElement.querySelectorAll("[data-dz-thumbnail]");for(t=0,o=s.length;t<o;t++)n=s[t],n.alt=e.name,n.src=i;return setTimeout(function(){return function(){return e.previewElement.classList.add("dz-image-preview")}}(this),1)}},error:function(n,e){var t,s,o,i,a;if(n.previewElement){n.previewElement.classList.add("dz-error"),typeof e!="String"&&e.error&&(e=e.error),s=n.previewElement.querySelectorAll("[data-dz-errormessage]"),o=[];for(t=0,a=s.length;t<a;t++)i=s[t],o.push(i.textContent=e);return o}},errormultiple:t,processing:function(e){if(e.previewElement&&(e.previewElement.classList.add("dz-processing"),e._removeLink))return e._removeLink.textContent=this.options.dictCancelUpload},processingmultiple:t,uploadprogress:function(i,a){var e,t,n,s,o;if(i.previewElement){s=i.previewElement.querySelectorAll("[data-dz-uploadprogress]"),e=[];for(n=0,o=s.length;n<o;n++)t=s[n],t.nodeName==="PROGRESS"?e.push(t.value=a):e.push(t.style.width=""+a+"%");return e}},totaluploadprogress:t,sending:t,sendingmultiple:t,success:function(e){if(e.previewElement)return e.previewElement.classList.add("dz-success")},successmultiple:t,canceled:function(e){return this.emit("error",e,"Upload canceled.")},canceledmultiple:t,complete:function(e){if(e._removeLink&&(e._removeLink.textContent=this.options.dictRemoveFile),e.previewElement)return e.previewElement.classList.add("dz-complete")},completemultiple:t,maxfilesexceeded:t,maxfilesreached:t,queuecomplete:t,addedfiles:t,previewTemplate:'<div class="dz-preview dz-file-preview">\n  <div class="dz-image"><img data-dz-thumbnail /></div>\n  <div class="dz-details">\n    <div class="dz-size"><span data-dz-size></span></div>\n    <div class="dz-filename"><span data-dz-name></span></div>\n  </div>\n  <div class="dz-progress"><span class="dz-upload" data-dz-uploadprogress></span></div>\n  <div class="dz-error-message"><span data-dz-errormessage></span></div>\n  <div class="dz-success-mark">\n    <svg width="54px" height="54px" viewBox="0 0 54 54" version="1.1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" xmlns:sketch="http://www.bohemiancoding.com/sketch/ns">\n      <title>Check</title>\n      <defs></defs>\n      <g id="Page-1" stroke="none" stroke-width="1" fill="none" fill-rule="evenodd" sketch:type="MSPage">\n        <path d="M23.5,31.8431458 L17.5852419,25.9283877 C16.0248253,24.3679711 13.4910294,24.366835 11.9289322,25.9289322 C10.3700136,27.4878508 10.3665912,30.0234455 11.9283877,31.5852419 L20.4147581,40.0716123 C20.5133999,40.1702541 20.6159315,40.2626649 20.7218615,40.3488435 C22.2835669,41.8725651 24.794234,41.8626202 26.3461564,40.3106978 L43.3106978,23.3461564 C44.8771021,21.7797521 44.8758057,19.2483887 43.3137085,17.6862915 C41.7547899,16.1273729 39.2176035,16.1255422 37.6538436,17.6893022 L23.5,31.8431458 Z M27,53 C41.3594035,53 53,41.3594035 53,27 C53,12.6405965 41.3594035,1 27,1 C12.6405965,1 1,12.6405965 1,27 C1,41.3594035 12.6405965,53 27,53 Z" id="Oval-2" stroke-opacity="0.198794158" stroke="#747474" fill-opacity="0.816519475" fill="#FFFFFF" sketch:type="MSShapeGroup"></path>\n      </g>\n    </svg>\n  </div>\n  <div class="dz-error-mark">\n    <svg width="54px" height="54px" viewBox="0 0 54 54" version="1.1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" xmlns:sketch="http://www.bohemiancoding.com/sketch/ns">\n      <title>Error</title>\n      <defs></defs>\n      <g id="Page-1" stroke="none" stroke-width="1" fill="none" fill-rule="evenodd" sketch:type="MSPage">\n        <g id="Check-+-Oval-2" sketch:type="MSLayerGroup" stroke="#747474" stroke-opacity="0.198794158" fill="#FFFFFF" fill-opacity="0.816519475">\n          <path d="M32.6568542,29 L38.3106978,23.3461564 C39.8771021,21.7797521 39.8758057,19.2483887 38.3137085,17.6862915 C36.7547899,16.1273729 34.2176035,16.1255422 32.6538436,17.6893022 L27,23.3431458 L21.3461564,17.6893022 C19.7823965,16.1255422 17.2452101,16.1273729 15.6862915,17.6862915 C14.1241943,19.2483887 14.1228979,21.7797521 15.6893022,23.3461564 L21.3431458,29 L15.6893022,34.6538436 C14.1228979,36.2202479 14.1241943,38.7516113 15.6862915,40.3137085 C17.2452101,41.8726271 19.7823965,41.8744578 21.3461564,40.3106978 L27,34.6568542 L32.6538436,40.3106978 C34.2176035,41.8744578 36.7547899,41.8726271 38.3137085,40.3137085 C39.8758057,38.7516113 39.8771021,36.2202479 38.3106978,34.6538436 L32.6568542,29 Z M27,53 C41.3594035,53 53,41.3594035 53,27 C53,12.6405965 41.3594035,1 27,1 C12.6405965,1 1,12.6405965 1,27 C1,41.3594035 12.6405965,53 27,53 Z" id="Oval-2" sketch:type="MSShapeGroup"></path>\n        </g>\n      </g>\n    </svg>\n  </div>\n</div>'},o=function(){var e,t,s,a,r,i=arguments[0],o=2<=arguments.length?n.call(arguments,1):[];for(e=0,r=o.length;e<r;e++){s=o[e];for(t in s)a=s[t],i[t]=a}return i};function e(a,n){var t,s,i;if(this.element=a,this.version=e.version,this.defaultOptions.previewTemplate=this.defaultOptions.previewTemplate.replace(/\n*/g,""),this.clickableElements=[],this.listeners=[],this.files=[],typeof this.element=="string"&&(this.element=document.querySelector(this.element)),!(this.element&&this.element.nodeType!=null))throw new Error("Invalid dropzone element.");if(this.element.dropzone)throw new Error("Dropzone already attached.");if(e.instances.push(this),this.element.dropzone=this,s=(i=e.optionsForElement(this.element))!=null?i:{},this.options=o({},this.defaultOptions,s,n??{}),this.options.forceFallback||!e.isBrowserSupported())return this.options.fallback.call(this);if(this.options.url==null&&(this.options.url=this.element.getAttribute("action")),!this.options.url)throw new Error("No URL provided.");if(this.options.acceptedFiles&&this.options.acceptedMimeTypes)throw new Error("You can't provide both 'acceptedFiles' and 'acceptedMimeTypes'. 'acceptedMimeTypes' is deprecated.");this.options.acceptedMimeTypes&&(this.options.acceptedFiles=this.options.acceptedMimeTypes,delete this.options.acceptedMimeTypes),this.options.method=this.options.method.toUpperCase(),(t=this.getExistingFallback())&&t.parentNode&&t.parentNode.removeChild(t),this.options.previewsContainer!==!1&&(this.options.previewsContainer?this.previewsContainer=e.getElement(this.options.previewsContainer,"previewsContainer"):this.previewsContainer=this.element),this.options.clickable&&(this.options.clickable===!0?this.clickableElements=[this.element]:this.clickableElements=e.getElements(this.options.clickable,"clickable")),this.init()}return e.prototype.getAcceptedFiles=function(){var e,t,n,s=this.files,o=[];for(e=0,n=s.length;e<n;e++)t=s[e],t.accepted&&o.push(t);return o},e.prototype.getRejectedFiles=function(){var e,t,n,s=this.files,o=[];for(e=0,n=s.length;e<n;e++)t=s[e],t.accepted||o.push(t);return o},e.prototype.getFilesWithStatus=function(i){var e,t,n,s=this.files,o=[];for(e=0,n=s.length;e<n;e++)t=s[e],t.status===i&&o.push(t);return o},e.prototype.getQueuedFiles=function(){return this.getFilesWithStatus(e.QUEUED)},e.prototype.getUploadingFiles=function(){return this.getFilesWithStatus(e.UPLOADING)},e.prototype.getAddedFiles=function(){return this.getFilesWithStatus(e.ADDED)},e.prototype.getActiveFiles=function(){var t,n,s,o=this.files,i=[];for(n=0,s=o.length;n<s;n++)t=o[n],(t.status===e.UPLOADING||t.status===e.QUEUED)&&i.push(t);return i},e.prototype.init=function(){var t,n,s,o,i,a,r;this.element.tagName==="form"&&this.element.setAttribute("enctype","multipart/form-data"),this.element.classList.contains("dropzone")&&!this.element.querySelector(".dz-message")&&this.element.appendChild(e.createElement('<div class="dz-default dz-message"><span>'+this.options.dictDefaultMessage+"</span></div>")),this.clickableElements.length&&(o=function(e){return function(){return e.hiddenFileInput&&e.hiddenFileInput.parentNode.removeChild(e.hiddenFileInput),e.hiddenFileInput=document.createElement("input"),e.hiddenFileInput.setAttribute("type","file"),(e.options.maxFiles==null||e.options.maxFiles>1)&&e.hiddenFileInput.setAttribute("multiple","multiple"),e.hiddenFileInput.className="dz-hidden-input",e.options.acceptedFiles!=null&&e.hiddenFileInput.setAttribute("accept",e.options.acceptedFiles),e.options.capture!=null&&e.hiddenFileInput.setAttribute("capture",e.options.capture),e.hiddenFileInput.style.visibility="hidden",e.hiddenFileInput.style.position="absolute",e.hiddenFileInput.style.top="0",e.hiddenFileInput.style.left="0",e.hiddenFileInput.style.height="0",e.hiddenFileInput.style.width="0",document.querySelector(e.options.hiddenInputContainer).appendChild(e.hiddenFileInput),e.hiddenFileInput.addEventListener("change",function(){var n,s,i,t=e.hiddenFileInput.files;if(t.length)for(n=0,i=t.length;n<i;n++)s=t[n],e.addFile(s);return e.emit("addedfiles",t),o()})}}(this),o()),this.URL=(r=window.URL)!=null?r:window.webkitURL,i=this.events;for(n=0,a=i.length;n<a;n++)s=i[n],this.on(s,this.options[s]);return this.on("uploadprogress",function(e){return function(){return e.updateTotalUploadProgress()}}(this)),this.on("removedfile",function(e){return function(){return e.updateTotalUploadProgress()}}(this)),this.on("canceled",function(e){return function(t){return e.emit("complete",t)}}(this)),this.on("complete",function(e){return function(){if(e.getAddedFiles().length===0&&e.getUploadingFiles().length===0&&e.getQueuedFiles().length===0)return setTimeout(function(){return e.emit("queuecomplete")},0)}}(this)),t=function(e){return e.stopPropagation(),e.preventDefault?e.preventDefault():e.returnValue=!1},this.listeners=[{element:this.element,events:{dragstart:function(e){return function(t){return e.emit("dragstart",t)}}(this),dragenter:function(e){return function(n){return t(n),e.emit("dragenter",n)}}(this),dragover:function(e){return function(n){var s;try{s=n.dataTransfer.effectAllowed}catch(e){}return n.dataTransfer.dropEffect="move"===s||"linkMove"===s?"move":"copy",t(n),e.emit("dragover",n)}}(this),dragleave:function(e){return function(t){return e.emit("dragleave",t)}}(this),drop:function(e){return function(n){return t(n),e.drop(n)}}(this),dragend:function(e){return function(t){return e.emit("dragend",t)}}(this)}}],this.clickableElements.forEach(function(t){return function(n){return t.listeners.push({element:n,events:{click:function(s){return(n!==t.element||s.target===t.element||e.elementInside(s.target,t.element.querySelector(".dz-message")))&&t.hiddenFileInput.click(),!0}}})}}(this)),this.enable(),this.options.init.call(this)},e.prototype.destroy=function(){var t;return this.disable(),this.removeAllFiles(!0),((t=this.hiddenFileInput)!=null?t.parentNode:void 0)&&(this.hiddenFileInput.parentNode.removeChild(this.hiddenFileInput),this.hiddenFileInput=null),delete this.element.dropzone,e.instances.splice(e.instances.indexOf(this),1)},e.prototype.updateTotalUploadProgress=function(){var e,t,o,i,a,s=0,n=0,r=this.getActiveFiles();if(r.length){i=this.getActiveFiles();for(e=0,a=i.length;e<a;e++)t=i[e],s+=t.upload.bytesSent,n+=t.upload.total;o=100*s/n}else o=100;return this.emit("totaluploadprogress",o,n,s)},e.prototype._getParamName=function(e){return typeof this.options.paramName=="function"?this.options.paramName(e):""+this.options.paramName+(this.options.uploadMultiple?"["+e+"]":"")},e.prototype._renameFilename=function(e){return typeof this.options.renameFilename!="function"?e:this.options.renameFilename(e)},e.prototype.getFallbackForm=function(){var t,n,s,o;return(o=this.getExistingFallback())||(t='<div class="dz-fallback">',this.options.dictFallbackText&&(t+="<p>"+this.options.dictFallbackText+"</p>"),t+='<input type="file" name="'+this._getParamName(0)+'" '+(this.options.uploadMultiple?'multiple="multiple"':void 0)+' /><input type="submit" value="Upload!"></div>',s=e.createElement(t),this.element.tagName!=="FORM"?(n=e.createElement('<form action="'+this.options.url+'" enctype="multipart/form-data" method="'+this.options.method+'"></form>'),n.appendChild(s)):(this.element.setAttribute("enctype","multipart/form-data"),this.element.setAttribute("method",this.options.method)),n??s)},e.prototype.getExistingFallback=function(){var e,t,n,s,i=function(n){var e,t,s;for(e=0,s=n.length;e<s;e++)if(t=n[e],/(^| )fallback($| )/.test(t.className))return t},o=["div","form"];for(e=0,s=o.length;e<s;e++)if(n=o[e],t=i(this.element.getElementsByTagName(n)))return t},e.prototype.setupEventListeners=function(){var e,t,n,s,o,i=this.listeners,a=[];for(e=0,o=i.length;e<o;e++)t=i[e],a.push(function(){var e=t.events,o=[];for(n in e)s=e[n],o.push(t.element.addEventListener(n,s,!1));return o}());return a},e.prototype.removeEventListeners=function(){var e,t,n,s,o,i=this.listeners,a=[];for(e=0,o=i.length;e<o;e++)t=i[e],a.push(function(){var e=t.events,o=[];for(n in e)s=e[n],o.push(t.element.removeEventListener(n,s,!1));return o}());return a},e.prototype.disable=function(){var e,t,n,s,o;this.clickableElements.forEach(function(e){return e.classList.remove("dz-clickable")}),this.removeEventListeners(),t=this.files,n=[];for(e=0,o=t.length;e<o;e++)s=t[e],n.push(this.cancelUpload(s));return n},e.prototype.enable=function(){return this.clickableElements.forEach(function(e){return e.classList.add("dz-clickable")}),this.setupEventListeners()},e.prototype.filesize=function(n){var e,s,o,a,r,c,t=0,i="b";if(n>0){o=["TB","GB","MB","KB","b"];for(e=s=0,c=o.length;s<c;e=++s)if(a=o[e],r=Math.pow(this.options.filesizeBase,4-e)/10,n>=r){t=n/Math.pow(this.options.filesizeBase,4-e),i=a;break}t=Math.round(10*t)/10}return"<strong>"+t+"</strong> "+i},e.prototype._updateMaxFilesReachedClass=function(){return this.options.maxFiles!=null&&this.getAcceptedFiles().length>=this.options.maxFiles?(this.getAcceptedFiles().length===this.options.maxFiles&&this.emit("maxfilesreached",this.files),this.element.classList.add("dz-max-files-reached")):this.element.classList.remove("dz-max-files-reached")},e.prototype.drop=function(t){var e,n;if(!t.dataTransfer)return;this.emit("drop",t),n=t.dataTransfer.files,this.emit("addedfiles",n),n.length&&(e=t.dataTransfer.items,e&&e.length&&e[0].webkitGetAsEntry!=null?this._addFilesFromItems(e):this.handleFiles(n))},e.prototype.paste=function(e){var t,n;if((e!=null?(n=e.clipboardData)!=null?n.items:void 0:void 0)==null)return;if(this.emit("paste",e),t=e.clipboardData.items,t.length)return this._addFilesFromItems(t)},e.prototype.handleFiles=function(t){var e,n,s,o=[];for(e=0,s=t.length;e<s;e++)n=t[e],o.push(this.addFile(n));return o},e.prototype._addFilesFromItems=function(o){var e,n,s,i,t=[];for(s=0,i=o.length;s<i;s++)e=o[s],e.webkitGetAsEntry!=null&&(n=e.webkitGetAsEntry())?n.isFile?t.push(this.addFile(e.getAsFile())):n.isDirectory?t.push(this._addFilesFromDirectory(n,n.name)):t.push(void 0):e.getAsFile!=null?e.kind==null||e.kind==="file"?t.push(this.addFile(e.getAsFile())):t.push(void 0):t.push(void 0);return t},e.prototype._addFilesFromDirectory=function(n,e){var s=n.createReader(),o=function(e){return typeof console!="undefined"&&console!==null?typeof console.log=="function"?console.log(e):void 0:void 0},t=function(n){return function(){return s.readEntries(function(i){var s,o,a;if(i.length>0){for(o=0,a=i.length;o<a;o++)s=i[o],s.isFile?s.file(function(t){if(n.options.ignoreHiddenFiles&&t.name.substring(0,1)===".")return;return t.fullPath=""+e+"/"+t.name,n.addFile(t)}):s.isDirectory&&n._addFilesFromDirectory(s,""+e+"/"+s.name);t()}return null},o)}}(this);return t()},e.prototype.accept=function(t,n){return t.size>this.options.maxFilesize*1024*1024?n(this.options.dictFileTooBig.replace("{{filesize}}",Math.round(t.size/1024/10.24)/100).replace("{{maxFilesize}}",this.options.maxFilesize)):e.isValidFile(t,this.options.acceptedFiles)?this.options.maxFiles!=null&&this.getAcceptedFiles().length>=this.options.maxFiles?(n(this.options.dictMaxFilesExceeded.replace("{{maxFiles}}",this.options.maxFiles)),this.emit("maxfilesexceeded",t)):this.options.accept.call(this,t,n):n(this.options.dictInvalidFileType)},e.prototype.addFile=function(t){return t.upload={progress:0,total:t.size,bytesSent:0},this.files.push(t),t.status=e.ADDED,this.emit("addedfile",t),this._enqueueThumbnail(t),this.accept(t,function(e){return function(n){return n?(t.accepted=!1,e._errorProcessing([t],n)):(t.accepted=!0,e.options.autoQueue&&e.enqueueFile(t)),e._updateMaxFilesReachedClass()}}(this))},e.prototype.enqueueFiles=function(t){var e,n,s;for(e=0,s=t.length;e<s;e++)n=t[e],this.enqueueFile(n);return null},e.prototype.enqueueFile=function(t){if(t.status===e.ADDED&&t.accepted===!0){if(t.status=e.QUEUED,this.options.autoProcessQueue)return setTimeout(function(e){return function(){return e.processQueue()}}(this),0)}else throw new Error("This file can't be queued because it has already been processed or was rejected.")},e.prototype._thumbnailQueue=[],e.prototype._processingThumbnail=!1,e.prototype._enqueueThumbnail=function(e){if(this.options.createImageThumbnails&&e.type.match(/image.*/)&&e.size<=this.options.maxThumbnailFilesize*1024*1024)return this._thumbnailQueue.push(e),setTimeout(function(e){return function(){return e._processThumbnailQueue()}}(this),0)},e.prototype._processThumbnailQueue=function(){if(this._processingThumbnail||this._thumbnailQueue.length===0)return;return this._processingThumbnail=!0,this.createThumbnail(this._thumbnailQueue.shift(),function(e){return function(){return e._processingThumbnail=!1,e._processThumbnailQueue()}}(this))},e.prototype.removeFile=function(t){if(t.status===e.UPLOADING&&this.cancelUpload(t),this.files=r(this.files,t),this.emit("removedfile",t),this.files.length===0)return this.emit("reset")},e.prototype.removeAllFiles=function(n){var t,s,o,i;n==null&&(n=!1),o=this.files.slice();for(t=0,i=o.length;t<i;t++)s=o[t],(s.status!==e.UPLOADING||n)&&this.removeFile(s);return null},e.prototype.createThumbnail=function(e,n){var t=new FileReader;return t.onload=function(s){return function(){if(e.type==="image/svg+xml"){s.emit("thumbnail",e,t.result),n!=null&&n();return}return s.createThumbnailFromUrl(e,t.result,n)}}(this),t.readAsDataURL(e)},e.prototype.createThumbnailFromUrl=function(t,o,n,s){var e=document.createElement("img");return s&&(e.crossOrigin=s),e.onload=function(s){return function(){var o,i,r,c,l,d,u,h;if(t.width=e.width,t.height=e.height,o=s.options.resize.call(s,t),o.trgWidth==null&&(o.trgWidth=o.optWidth),o.trgHeight==null&&(o.trgHeight=o.optHeight),i=document.createElement("canvas"),r=i.getContext("2d"),i.width=o.trgWidth,i.height=o.trgHeight,a(r,e,(l=o.srcX)!=null?l:0,(d=o.srcY)!=null?d:0,o.srcWidth,o.srcHeight,(u=o.trgX)!=null?u:0,(h=o.trgY)!=null?h:0,o.trgWidth,o.trgHeight),c=i.toDataURL("image/png"),s.emit("thumbnail",t,c),n!=null)return n()}}(this),n!=null&&(e.onerror=n),e.src=o},e.prototype.processQueue=function(){if(t=this.options.parallelUploads,n=this.getUploadingFiles().length,s=n,n>=t)return;if(e=this.getQueuedFiles(),!(e.length>0))return;if(this.options.uploadMultiple)return this.processFiles(e.slice(0,t-n));for(var e,t,n,s;s<t;){if(!e.length)return;this.processFile(e.shift()),s++}},e.prototype.processFile=function(e){return this.processFiles([e])},e.prototype.processFiles=function(t){var n,s,o;for(s=0,o=t.length;s<o;s++)n=t[s],n.processing=!0,n.status=e.UPLOADING,this.emit("processing",n);return this.options.uploadMultiple&&this.emit("processingmultiple",t),this.uploadFiles(t)},e.prototype._getFilesWithXhr=function(t){var e,n;return n=function(){var n,s,o=this.files,i=[];for(n=0,s=o.length;n<s;n++)e=o[n],e.xhr===t&&i.push(e);return i}.call(this)},e.prototype.cancelUpload=function(t){var n,s,o,i,a,r,c;if(t.status===e.UPLOADING){n=this._getFilesWithXhr(t.xhr);for(o=0,a=n.length;o<a;o++)s=n[o],s.status=e.CANCELED;t.xhr.abort();for(i=0,r=n.length;i<r;i++)s=n[i],this.emit("canceled",s);this.options.uploadMultiple&&this.emit("canceledmultiple",n)}else((c=t.status)===e.ADDED||c===e.QUEUED)&&(t.status=e.CANCELED,this.emit("canceled",t),this.options.uploadMultiple&&this.emit("canceledmultiple",[t]));if(this.options.autoProcessQueue)return this.processQueue()},i=function(){var e=arguments[0],t=2<=arguments.length?n.call(arguments,1):[];return typeof e=="function"?e.apply(this,t):e},e.prototype.uploadFile=function(e){return this.uploadFiles([e])},e.prototype.uploadFiles=function(t){var s,a,r,c,l,d,u,h,m,f,p,g,v,b,j,y,_,w,O,x,C,E,k,A,S,M,F,T,z,D,N,L,R,n=new XMLHttpRequest;for(p=0,F=t.length;p<F;p++)s=t[p],s.xhr=n;N=i(this.options.method,t),R=i(this.options.url,t),n.open(N,R,!0),n.withCredentials=!!this.options.withCredentials,c=null,E=function(e){return function(){var o,i,a=[];for(o=0,i=t.length;o<i;o++)s=t[o],a.push(e._errorProcessing(t,c||e.options.dictResponseError.replace("{{statusCode}}",n.status),n));return a}}(this),v=function(e){return function(n){var o,i,a,r,c,l,d,u,h;if(n!=null){o=100*n.loaded/n.total;for(i=0,d=t.length;i<d;i++)s=t[i],s.upload={progress:o,total:n.total,bytesSent:n.loaded}}else{c=!0,o=100;for(a=0,u=t.length;a<u;a++)s=t[a],s.upload.progress===100&&s.upload.bytesSent===s.upload.total||(c=!1),s.upload.progress=o,s.upload.bytesSent=s.upload.total;if(c)return}l=[];for(r=0,h=t.length;r<h;r++)s=t[r],l.push(e.emit("uploadprogress",s,o,s.upload.bytesSent));return l}}(this),n.onload=function(s){return function(o){var i;if(t[0].status===e.CANCELED)return;if(n.readyState!==4)return;if(c=n.responseText,n.getResponseHeader("content-type")&&~n.getResponseHeader("content-type").indexOf("application/json"))try{c=JSON.parse(c)}catch(e){o=e,c="Invalid JSON response from server."}return v(),200<=(i=n.status)&&i<300?s._finished(t,c,o):E()}}(this),n.onerror=function(){return function(){if(t[0].status===e.CANCELED)return;return E()}}(this),D=(T=n.upload)!=null?T:n,D.onprogress=v,h={Accept:"application/json","Cache-Control":"no-cache","X-Requested-With":"XMLHttpRequest"},this.options.headers&&o(h,this.options.headers);for(w in h)y=h[w],y&&n.setRequestHeader(w,y);if(r=new FormData,this.options.params){_=this.options.params;for(x in _)z=_[x],r.append(x,z)}for(f=0,A=t.length;f<A;f++)s=t[f],this.emit("sending",s,n,r);if(this.options.uploadMultiple&&this.emit("sendingmultiple",t,n,r),this.element.tagName==="FORM"){b=this.element.querySelectorAll("input, textarea, select, button");for(m=0,S=b.length;m<S;m++)if(a=b[m],k=a.getAttribute("name"),C=a.getAttribute("type"),a.tagName==="SELECT"&&a.hasAttribute("multiple")){O=a.options;for(g=0,M=O.length;g<M;g++)j=O[g],j.selected&&r.append(k,j.value)}else(!C||(L=C.toLowerCase())!=="checkbox"&&L!=="radio"||a.checked)&&r.append(k,a.value)}for(d=l=0,u=t.length-1;0<=u?l<=u:l>=u;d=0<=u?++l:--l)r.append(this._getParamName(d),t[d],this._renameFilename(t[d].name));return this.submitRequest(n,r,t)},e.prototype.submitRequest=function(e,t){return e.send(t)},e.prototype._finished=function(t,o,i){var n,s,a;for(s=0,a=t.length;s<a;s++)n=t[s],n.status=e.SUCCESS,this.emit("success",n,o,i),this.emit("complete",n);if(this.options.uploadMultiple&&(this.emit("successmultiple",t,o,i),this.emit("completemultiple",t)),this.options.autoProcessQueue)return this.processQueue()},e.prototype._errorProcessing=function(t,o,i){var n,s,a;for(s=0,a=t.length;s<a;s++)n=t[s],n.status=e.ERROR,this.emit("error",n,o,i),this.emit("complete",n);if(this.options.uploadMultiple&&(this.emit("errormultiple",t,o,i),this.emit("completemultiple",t)),this.options.autoProcessQueue)return this.processQueue()},e}(s);e.version="4.3.0",e.options={},e.optionsForElement=function(t){return t.getAttribute("id")?e.options[c(t.getAttribute("id"))]:void 0},e.instances=[],e.forElement=function(e){if(typeof e=="string"&&(e=document.querySelector(e)),(e!=null?e.dropzone:void 0)==null)throw new Error("No Dropzone found for given element. This is probably because you're trying to access it before Dropzone had the time to initialize. Use the `init` option to setup any additional observers on your Dropzone.");return e.dropzone},e.autoDiscover=!0,e.discover=function(){var t,n,s,o,i,a;document.querySelectorAll?t=document.querySelectorAll(".dropzone"):(t=[],o=function(o){var e,n,i,s=[];for(e=0,i=o.length;e<i;e++)n=o[e],/(^| )dropzone($| )/.test(n.className)?s.push(t.push(n)):s.push(void 0);return s},o(document.getElementsByTagName("div")),o(document.getElementsByTagName("form"))),s=[];for(n=0,a=t.length;n<a;n++)i=t[n],e.optionsForElement(i)!==!1?s.push(new e(i)):s.push(void 0);return s},e.blacklistedBrowsers=[/opera.*Macintosh.*version\/12/i],e.isBrowserSupported=function(){var n,s,o,i,t=!0;if(window.File&&window.FileReader&&window.FileList&&window.Blob&&window.FormData&&document.querySelector)if("classList"in document.createElement("a")){{s=e.blacklistedBrowsers;for(n=0,i=s.length;n<i;n++)if(o=s[n],o.test(navigator.userAgent)){t=!1;continue}}}else t=!1;else t=!1;return t},r=function(n,i){var e,t,s,o=[];for(e=0,s=n.length;e<s;e++)t=n[e],t!==i&&o.push(t);return o},c=function(e){return e.replace(/[-_](\w)/g,function(e){return e.charAt(1).toUpperCase()})},e.createElement=function(t){var e=document.createElement("div");return e.innerHTML=t,e.childNodes[0]},e.elementInside=function(e,t){if(e===t)return!0;for(;e=e.parentNode;)if(e===t)return!0;return!1},e.getElement=function(e,n){var t;if(typeof e=="string"?t=document.querySelector(e):e.nodeType!=null&&(t=e),t==null)throw new Error("Invalid `"+n+"` option provided. Please provide a CSS selector or a plain HTML element.");return t},e.getElements=function(t,a){var e,n,s,o,i,r,c,l;if(t instanceof Array){e=[];try{for(s=0,c=t.length;s<c;s++)n=t[s],e.push(this.getElement(n,a))}catch(t){l=t,e=null}}else if(typeof t=="string"){e=[],i=document.querySelectorAll(t);for(o=0,r=i.length;o<r;o++)n=i[o],e.push(n)}else t.nodeType!=null&&(e=[t]);if(!(e!=null&&e.length))throw new Error("Invalid `"+a+"` option provided. Please provide a CSS selector, a plain HTML element or a list of those.");return e},e.confirm=function(t,n,e){if(window.confirm(t))return n();if(e!=null)return e()},e.isValidFile=function(s,t){var e,n,o,i,a;if(!t)return!0;t=t.split(","),o=s.type,i=o.replace(/\/.*$/,"");for(n=0,a=t.length;n<a;n++)if(e=t[n],e=e.trim(),e.charAt(0)==="."){if(s.name.toLowerCase().indexOf(e.toLowerCase(),s.name.length-e.length)!==-1)return!0}else if(/\/\*$/.test(e)){if(i===e.replace(/\/.*$/,""))return!0}else if(o===e)return!0;return!1},typeof jQuery!="undefined"&&jQuery!==null&&(jQuery.fn.dropzone=function(t){return this.each(function(){return new e(this,t)})}),typeof module!="undefined"&&module!==null?module.exports=e:window.Dropzone=e,e.ADDED="added",e.QUEUED="queued",e.ACCEPTED=e.QUEUED,e.UPLOADING="uploading",e.PROCESSING=e.UPLOADING,e.CANCELED="canceled",e.ERROR="error",e.SUCCESS="success",i=function(a){d=a.naturalWidth,t=a.naturalHeight,s=document.createElement("canvas"),s.width=1,s.height=t,o=s.getContext("2d"),o.drawImage(a,0,0),l=o.getImageData(0,0,1,t).data,n=0,i=t,e=t;for(var e,t,n,s,o,i,r,c,l,d;e>n;)c=l[(e-1)*4+3],c===0?i=e:n=e,e=i+n>>1;return r=e/t,r===0?1:r},a=function(t,e,n,s,o,a,r,c,l,d){var u=i(e);return t.drawImage(e,n,s,o,a,r,c,l,d/u)},o=function(t,l){var i=!1,c=!0,e=t.document,r=e.documentElement,o=e.addEventListener?"addEventListener":"attachEvent",d=e.addEventListener?"removeEventListener":"detachEvent",s=e.addEventListener?"":"on",n=function(o){if(o.type==="readystatechange"&&e.readyState!=="complete")return;if((o.type==="load"?t:e)[d](s+o.type,n,!1),!i&&(i=!0))return l.call(t,o.type||o)},a=function(){var e;try{r.doScroll("left")}catch(t){e=t,setTimeout(a,50);return}return n("poll")};if(e.readyState!=="complete"){if(e.createEventObject&&r.doScroll){try{c=!t.frameElement}catch(e){}c&&a()}return e[o](s+"DOMContentLoaded",n,!1),e[o](s+"readystatechange",n,!1),t[o](s+"load",n,!1)}},e._autoDiscoverFunction=function(){if(e.autoDiscover)return e.discover()},o(window,e._autoDiscoverFunction)}.call(this),function(e){e.idleTimer=function(t,n){typeof t=="object"?(i=t,t=null):typeof t=="number"&&(i={timeout:t},t=null),n=n||document;var i=e.extend({idle:!1,timeout:3e4,events:"mousemove keydown wheel DOMMouseScroll mousewheel mousedown touchstart touchmove MSPointerDown MSPointerMove"},i),s=e(n),o=s.data("idleTimerObj")||{},a=function(o){var s,t=e.data(n,"idleTimerObj")||{};t.idle=!t.idle,t.olddate=+new Date,s=e.Event((t.idle?"idle":"active")+".idleTimer"),e(n).trigger(s,[n,e.extend({},t),o])},c=function(s){var o,t=e.data(n,"idleTimerObj")||{};if(t.remaining!=null)return;if(s.type==="mousemove"){if(s.pageX===t.pageX&&s.pageY===t.pageY)return;if(typeof s.pageX=="undefined"&&typeof s.pageY=="undefined")return;if(o=+new Date-t.olddate,o<200)return}clearTimeout(t.tId),t.idle&&a(s),t.lastActive=+new Date,t.pageX=s.pageX,t.pageY=s.pageY,t.tId=setTimeout(a,t.timeout)},r=function(){var t=e.data(n,"idleTimerObj")||{};t.idle=t.idleBackup,t.olddate=+new Date,t.lastActive=t.olddate,t.remaining=null,clearTimeout(t.tId),t.idle||(t.tId=setTimeout(a,t.timeout))},l=function(){var t=e.data(n,"idleTimerObj")||{};if(t.remaining!=null)return;t.remaining=t.timeout-(+new Date-t.olddate),clearTimeout(t.tId)},d=function(){var t=e.data(n,"idleTimerObj")||{};if(t.remaining==null)return;t.idle||(t.tId=setTimeout(a,t.remaining)),t.remaining=null},u=function(){var t=e.data(n,"idleTimerObj")||{};clearTimeout(t.tId),s.removeData("idleTimerObj"),s.off("._idleTimer")},h=function(){var s,t=e.data(n,"idleTimerObj")||{};return t.idle?0:t.remaining!=null?t.remaining:(s=t.timeout-(+new Date-t.lastActive),s<0&&(s=0),s)};if(t===null&&typeof o.idle!="undefined")return r(),s;if(t===null);else if(t!==null&&typeof o.idle=="undefined")return!1;else if(t==="destroy")return u(),s;else if(t==="pause")return l(),s;else if(t==="resume")return d(),s;else if(t==="reset")return r(),s;else if(t==="getRemainingTime")return h();else if(t==="getElapsedTime")return+new Date-o.olddate;else if(t==="getLastActiveTime")return o.lastActive;else if(t==="isIdle")return o.idle;return s.on(e.trim((i.events+" ").split(" ").join("._idleTimer ")),function(e){c(e)}),o=e.extend({},{olddate:+new Date,lastActive:+new Date,idle:i.idle,idleBackup:i.idle,timeout:i.timeout,remaining:null,tId:null,pageX:null,pageY:null}),o.idle||(o.tId=setTimeout(a,o.timeout)),e.data(n,"idleTimerObj",o),s},e.fn.idleTimer=function(t){return this[0]?e.idleTimer(t,this[0]):this}}(jQuery),function(t,e){typeof define=="function"&&define.amd?define(["jquery"],e):e(t.jQuery)}(this,function(e){var t,n,c={seconds:0,editable:!1,restart:!1,duration:null,callback:function(){alert("Time up!")},startTimer:function(){},pauseTimer:function(){},resumeTimer:function(){},resetTimer:function(){},removeTimer:function(){},repeat:!1,countdown:!1,format:null,updateFrequency:1e3,state:"running"},s="html",i="stopped",a="running",y="paused";function h(t){var n=t.element;e(n).data("intr",setInterval(v.bind(t),t.options.updateFrequency)),e(n).data("isTimerRunning",!0)}function r(t){clearInterval(e(t.element).data("intr")),e(t.element).data("isTimerRunning",!1)}function v(){e(this.element).data("totalSeconds",l()-e(this.element).data("startTime")),o(this),e(this.element).data("duration")&&e(this.element).data("totalSeconds")%e(this.element).data("duration")===0&&(this.options.repeat||(e(this.element).data("duration",null),this.options.duration=null),this.options.countdown&&(r(this),this.options.countdown=!1,e(this.element).data("state",i)),this.options.callback())}function o(n){var t=n.element,o=e(t).data("totalSeconds");n.options.countdown&&e(t).data("duration")>0&&(o=e(t).data("duration")-e(t).data("totalSeconds")),e(t)[s](f(o,n)),e(t).data("seconds",o)}function g(n){var t=n.element;e(t).on("focus",function(){d(n)}),e(t).on("blur",function(){var i,o=e(t)[s]();o.indexOf("sec")>0?e(t).data("totalSeconds",Number(o.replace(/\ssec/g,''))):o.indexOf("min")>0?(o=o.replace(/\smin/g,''),i=o.split(":"),e(t).data("totalSeconds",Number(i[0]*60)+Number(i[1]))):o.match(/\d{1,2}:\d{2}:\d{2}/)&&(i=o.split(":"),e(t).data("totalSeconds",Number(i[0]*3600)+Number(i[1]*60)+Number(i[2]))),u(n)})}function l(){return Math.round((new Date).getTime()/1e3)}function _(e){var n,s=0,t=Math.floor(e/60);return e>=3600&&(s=Math.floor(e/3600)),e>=3600&&(t=Math.floor(e%3600/60)),t<10&&s>0&&(t="0"+t),n=e%60,n<10&&(t>0||s>0)&&(n="0"+n),{hours:s,minutes:t,seconds:n}}function f(o,n){var s,t='',e=_(o);return n.options.format?(s=[{identifier:"%h",value:e.hours,pad:!1},{identifier:"%m",value:e.minutes,pad:!1},{identifier:"%s",value:e.seconds,pad:!1},{identifier:"%H",value:parseInt(e.hours),pad:!0},{identifier:"%M",value:parseInt(e.minutes),pad:!0},{identifier:"%S",value:parseInt(e.seconds),pad:!0}],t=n.options.format,s.forEach(function(e){t=t.replace(new RegExp(e.identifier.replace(/([.*+?^=!:${}()|[\]/\\])/g,"\\$1"),"g"),e.pad?e.value<10?"0"+e.value:e.value:e.value)})):e.hours?t=e.hours+":"+e.minutes+":"+e.seconds:e.minutes?t=e.minutes+":"+e.seconds+" min":t=e.seconds+" sec",t}function m(e){if(!isNaN(Number(e)))return e;var n=e.match(/\d{1,2}h/),s=e.match(/\d{1,2}m/),o=e.match(/\d{1,2}s/),t=0;return e=e.toLowerCase(),n&&(t+=Number(n[0].replace("h",''))*3600),s&&(t+=Number(s[0].replace("m",''))*60),o&&(t+=Number(o[0].replace("s",''))),t}function p(t){var n=t.element;e(n).data("isTimerRunning")||(o(t),h(t),e(n).data("state",a),t.options.startTimer.bind(t).call())}function d(t){var n=t.element;e(n).data("isTimerRunning")&&(r(t),e(n).data("state",y),t.options.pauseTimer.bind(t).call())}function u(t){var n=t.element;e(n).data("isTimerRunning")||(e(n).data("startTime",l()-e(n).data("totalSeconds")),h(t),e(n).data("state",a),t.options.resumeTimer.bind(t).call())}function b(n){var t=n.element;e(t).data("startTime",0),e(t).data("totalSeconds",0),e(t).data("seconds",0),e(t).data("state",i),e(t).data("duration",n.options.duration),n.options.resetTimer.bind(n).call()}function j(n){var o=n.element;r(n),n.options.removeTimer.bind(n).call(),e(o).data("plugin_"+t,null),e(o).data("seconds",null),e(o).data("state",null),e(o)[s]('')}n=function(t,o){var n;this.options=c=e.extend(this.options,c,o),this.element=t,e(t).data("totalSeconds",c.seconds),e(t).data("startTime",l()-e(t).data("totalSeconds")),e(t).data("seconds",e(t).data("totalSeconds")),e(t).data("state",i),n=e(t).prop("tagName").toLowerCase(),(n==="input"||n==="textarea")&&(s="val"),this.options.duration&&(e(t).data("duration",m(this.options.duration)),this.options.duration=m(this.options.duration)),this.options.editable&&g(this)},n.prototype={start:function(){p(this)},pause:function(){d(this)},resume:function(){u(this)},reset:function(){b(this)},remove:function(){j(this)}},t="timer",e.fn[t]=function(s){return s=s||"start",this.each(function(){e.data(this,"plugin_"+t)instanceof n||e.data(this,"plugin_"+t,new n(this,s));var i=e.data(this,"plugin_"+t);typeof s=="string"&&typeof i[s]=="function"&&i[s].call(i),typeof s=="object"&&(i.options.state===a?i.start.call(i):o(i))})}});var $jq=jQuery.noConflict(),shortcutsON,calc_row,note_option_string,Builder,Effect,Droppables,Draggables,Draggable,SortableObserver,Sortable,Autocompleter,Control,Prototip,Tip,Validator,Validation,$proc,$value,IframeShim,dynamicOptionListCount,dynamicOptionListObjects,send=XMLHttpRequest.prototype.send;XMLHttpRequest.prototype.send=function(){return this.setRequestHeader("X-CSRF-Token",window.HS.HS_CSRF_TOKEN),send.apply(this,arguments)},$jq.ajaxSetup({cache:!1}),$jq.strPad=function(n,s,t){var e=n.toString();for(t||(t="0");e.length<s;)e=e+t;return e};function IsNumeric(t){var n,s="0123456789.",e=!0;for(i=0;i<t.length&&e==!0;i++)n=t.charAt(i),s.indexOf(n)==-1&&(e=!1);return e}function hs_msg(n,t,e){t=t||!1,e=e||!1,e&&$jq("#hs_msg").addClass(e),$jq("#hs_msg_inner").html(n),new Effect.Parallel([new Effect.Appear("hs_msg",{sync:!0})],{duration:.2}),t||setTimeout(function(){Effect.Fade("hs_msg",{duration:.2,onComplete:function(){e&&$jq("#hs_msg").removeClassName(e)}})},3e3)}function updateCsrfTokens(e){typeof window.HS!="undefined"&&(window.HS.HS_CSRF_TOKEN=e),typeof HS_CSRF_TOKEN!="undefined"&&(HS_CSRF_TOKEN=e),$jq('input[name="_token"]').val(e)}function login_form_sub(e){return e.preventDefault(),$jq.ajax({url:"login",data:new FormData(e.currentTarget),processData:!1,contentType:!1,type:"POST",headers:{accept:"application/json"},success:function(e){Tips.hideAll(),closeAllModals(),did_re_login=!0,updateCsrfTokens(e.csrf)}}),!1}function hs_isdefined(e){return!(typeof window[e]=="undefined")}function preloadImages(){for(var e,n=[],s=arguments.length,t=s;t--;)e=document.createElement("img"),e.src=arguments[t],n.push(e)}function is_wysiwyg(){return!(typeof tinyMCE=="undefined")&&tinyMCE.activeEditor!==null}function focus_note_body(e){is_wysiwyg()||Field.focus(e)}function get_note_body(e){return is_wysiwyg()?$jq("<div/>").html(tinyMCE.activeEditor.getContent()).text().trim()!=""?tinyMCE.activeEditor.getContent():"":$F(e)?$F(e):""}function set_note_body(t,e){return is_wysiwyg()?tinyMCE.activeEditor.setContent(e):$(t).value=e}function append_wysiwyg_note_body(e){return tinyMCE.activeEditor.insertContent(e)}function goPage(e){window.location.href=e}function hs_PeriodicalExecuter(s,e,n){var t="helpspot_pe_"+s;window[t]=setInterval(e,n*1e3),$jq(document).bind("idle.idleTimer",function(){clearInterval(window[t])}),$jq(document).bind("active.idleTimer",function(){e(),window[t]=setInterval(e,n*1e3)}),$jq.idleTimer(cHD_IDLE_TIMEOUT*1e3)}function closeAllModals(){modalIsOpen()&&mobiscroll.activeInstance.hide()}function modalIsClosed(){return typeof mobiscroll.activeInstance=="undefined"}function modalIsOpen(){return!modalIsClosed()}function hs_alert(s,n){var e,t,o,n=Object.extend({errorlist:!0,title:lg_js_notification},arguments[1]||{});if(n.errorlist){t=s.split(/(\r\n|[\r\n])/),e='<ul class="alert-error-list">';for(i=0;i<t.length;i++)trim(t[i])!=""&&(e=e+"<li>"+t[i]+"</li>");e=e+"</ul>"}else e=s.replace(/(\r\n|[\r\n])/g,"<br />");return o='<div class="alert-title">'+n.title+'</div><div class="alert-body">'+e+"</div>",hs_overlay({html:o})}function hs_confirm_submit(n,o,e){n.preventDefault();var t,s,e=Object.extend({title:lg_js_confirmation},arguments[2]||{});return closeAllModals(),t=$jq(".popup-holder"),t.html('<div class="alert-title">'+e.title+'</div><div class="alert-body">'+o+"</div>"),s=t.mobiscroll().popup({display:"center",scrollLock:!1,layout:"fixed",cssClass:"mbsc-no-padding md-content-scroll",onInit:function(){$jq("body").addClass("mobi-open")},onBeforeShow:e.beforeOpen,onShow:e.onOpen,onClose:function(){$jq("body").removeClass("mobi-open")},buttons:[{text:button_ok,handler:function(){n.target.submit()},icon:'',cssClass:"mobi-close accent"},{text:button_close,handler:"cancel",icon:'',cssClass:"mobi-close"}]}).mobiscroll("getInst"),s.show(),!1}function hs_confirm(i,t,e){var n,s,o,e=Object.extend({title:lg_js_confirmation,showCancelButton:!0},e||{});return closeAllModals(),n=$jq(".popup-holder"),n.html('<div class="alert-title">'+e.title+'</div><div class="alert-body">'+i+"</div>"),s=[{text:button_ok,handler:function(){$jq.type(t)==="function"?(t(),closeAllModals()):$jq.type(t)==="string"&&goPage(t)},icon:'',cssClass:"mobi-close accent"}],e.showCancelButton&&s.push({text:button_cancel,handler:"cancel",icon:'',cssClass:"mobi-close"}),o=n.mobiscroll().popup({display:"center",scrollLock:!1,layout:"fixed",cssClass:"mbsc-no-padding md-content-scroll",onInit:function(){$jq("body").addClass("mobi-open")},onBeforeShow:e.beforeOpen,onShow:e.onOpen,onClose:function(){$jq("body").removeClass("mobi-open")},buttons:s}).mobiscroll("getInst"),o.show(),!1}function kbui(e){var t="admin?pg=ajax_gateway&action=kbui&xBook="+e+"&rand="+ajaxRandomString();$jq.get(t,function(t){$jq("#kbui_box").html(t),hs_overlay("kbui_box",{display:"top",onOpen:function(){kbui_gettoc(e),$jq("#new_group").focus()}})})}function kbui_showpage(e){$jq("#kbui-page").html(ajaxLoading()),$jq("#kbui-page").load("admin?pg=ajax_gateway&action=kbui-page&xPage="+e+"&xPortal="+$jq("#xPortal").val())}function kbui_gettoc(e){$jq("#kbui-toc").load("admin?pg=ajax_gateway&action=kbui-toc&xBook="+e,function(){folderUI("kbui-toc"),$jq("#kbui-toc .kbui:first").click()})}function aKBL(e,s){if($jq("#tPost").length)var t="tPost",n=document.postform.tPost;else t="tBody",n=document.requestform.tBody;return t=="tBody"&&editor_type=="wysiwyg"&&is_wysiwyg("tBody")?append_wysiwyg_note_body('<a href="'+e+'">'+s+"</a>"):t=="tBody"&&editor_type=="markdown"?insertAtCursor(n,"["+s+"]("+e+")"):insertAtCursor(n,e),closeAllModals(),!1}function folderUI(t){var e=getCookie("sidebarOpenFolders")?getCookie("sidebarOpenFolders").split(","):[];$$("#"+t+" .folder").each(function(t){$(t).observe("click",function(s){Event.stop(s),$$("."+this.id).each(function(e){e.toggle()}),$$("#"+this.id+" span.arrow").each(function(e){e.toggleClassName("arrow-open")});var t,n=indexInArray(e,this.id);n==-1?e.push(this.id):e.splice(n,1),t=new Date,t.setFullYear(t.getFullYear()+10),setCookie("sidebarOpenFolders",e.join(","),t)})})}function toggleSidebarState(){var t,e=getCookie("sidebarState");e=="closed"?(e="open",$jq(".main-layout").removeClass("sidebar-closed")):(e="closed",$jq(".main-layout").addClass("sidebar-closed")),t=new Date,t.setFullYear(t.getFullYear()+10),setCookie("sidebarState",e,t)}function getHash(){var e=window.location.hash;return e.substring(1)}function targetopener(e,t,n){return!window.focus||!window.opener||(window.opener.focus(),n||(window.opener.location.href=e.href),t&&window.close(),!1)}function openWin(e,n,t){window.open(e,"",t)}function showPopWin(e,t,n,s){openWin(e,s,"height="+t+",width="+n+",scrollbars=yes,toolbar=no,location=no,status=no,resizable=yes")}function hsCloseWin(e){e||(e=2),setTimeout("window.close()",e*1e3)}function qtSet(t,s,o){if(times=Form.Element.getValue(t),times!=""){v=Form.Element.getValue(t).split("|");var n,e=$jq("#"+s).mobiscroll("getInst"),i=calendar_clean_date(v[0]);e.setVal(i,!0),e=$jq("#"+o).mobiscroll("getInst"),n=calendar_clean_date(v[2]),e.setVal(n,!0),v[4]&&$jq("#graph_grouping").val().search("date_")!=-1&&$jq("#graph_grouping").val().search("date_agg")==-1&&setSelectToValue("graph_grouping",v[4])}}function sidebarSearchAction(s){if(s=="request"){var t,n,e=trim($jq("#sidebar-q").val());IsNumeric(e)?goPage("admin?pg=request&reqid="+e):(n='<table class="tablebody no_borders" id="rsgroup_1" width="'+window.innerWidth+'" height="'+window.innerHeight+'" cellspacing="0" cellpadding="0" border="0"><tbody><tr><td class="js-request">'+ajaxLoading()+"</td></tr></tbody></table>",hs_overlay({html:n}),e.search(/@/)>0?(t="admin?pg=ajax_gateway&action=sidebarsearch&search_type=2&q="+eq(e)+"&anyall=all&sEmail="+eq(e)+"&rand="+ajaxRandomString()):(t="admin?pg=ajax_gateway&action=sidebarsearch&search_type=9&q="+eq(e)+"&anyall=any&rand="+ajaxRandomString()),$jq.get(t,function(e){$jq(".popup-holder").html(e)}))}}shortcutsON=!0;function hs_shortcutsOff(){shortcutsON=!1}function hs_shortcutsOn(){shortcutsON=!0}function setCookie(s,o,e,t,n,i){var a=s+"="+escape(o)+(e?";expires="+e.toUTCString():"")+(t?";path="+t:"")+(n?";domain="+n:"")+(i?";secure":"");document.cookie=a}function getCookie(s){for(var e,o=document.cookie,t=o.split(";"),n=t.length;n--;)if(e=t[n].split("="),e[0].substring(0,1)==" "&&(e[0]=e[0].substring(1,e[0].length)),unescape(e[0])==s)return unescape(e[1]);return''}function trim(e){return $jq.trim(e)}function setFieldFocus(e){e&&e.focus()}function clearFocusFill(e,n,t){e.value==n&&(e.value=""),t&&Element.addClassName(e.id,t)}function checkRowHighlight(e){document.getElementById(e+"_box")&&(document.getElementById(e+"_box").checked?document.getElementById(e).className="boldcheckboxrow":document.getElementById(e).className="")}function onloadRowHighlight(t,n){for(var e=0;e<n;e++)checkRowHighlight(t+e)}function checkRows(){$jq.each($jq(".checkbox-menu").find("input:checked"),function(){rowChecked(this.value)})}function rowChecked(e){$jq("#batch_action_buttons").show(),row=$("tr-"+e),box=$(e+"_checkbox"),box.checked?(Element.addClassName(box,"checkedfilterrow"),Element.addClassName(row,"checkedfilterrow")):(Element.removeClassName(box,"checkedfilterrow"),Element.removeClassName(row,"checkedfilterrow")),$("batch_action_buttons").removeClassName("thin-disabled")}function reorder_call(e,t){var n="admin",s="pg=ajax_gateway&action="+t+"&"+Sortable.serialize(e,{name:"sortorder"})+"&rand="+ajaxRandomString(),o=new Ajax.Request(n,{method:"get",parameters:s,onComplete:function(){$$("#tablesort .sortable").each(function(e){new Effect.Highlight(e,{duration:1,startcolor:"#DCE8D8",keepBackgroundImage:!0})})}})}function findIDbyValue(e,t){for(i=0;i<e.length;i++)if(e[i].value==t)return e[i].id}function getCheckedRadioButton(t){for(var e=0;e<t.length;e++)if(t[e].checked)return e}function getSelectVal(e){return $(e).options[$(e).selectedIndex].value}function hs_indexOf(t,n){for(var e=0;e<t.length;e++)if(t[e]==n)return e;return-1}function hs_inArray(t){var e;for(e=0;e<this.length;e++)if(this[e]===t)return!0;return!1}function indexInArray(t,n){for(var s=t.length,e=0;e<s;e++)if(t[e]==n)return e;return-1}function hs_pad(e,t,s,o){for(var n=e.length;n<s;n++)e=o?t+e:e+t;return e}function noenter(e,t){if(!e)var e=window.event;return e.keyCode?code=e.keyCode:e.which&&(code=e.which),t?code!=13||($(t).onclick(),!1):!(code==13)}function setSelectToValue(t,e,n){if(list=$(t).options,typeof n!="undefined"&&(list.selectedIndex=0),list&&e)for(i=0;i<list.length;i++)if(list[i].value==e){list.selectedIndex=i;break}}function setSelectByText(t,e){if(list=$(t).options,list&&e)for(i=0;i<list.length;i++)if(list[i].text==e){list.selectedIndex=i;break}}function help_toggle(t,e){var e=e||"";new Ajax.Updater("help_toggle_body","admin?pg=ajax_gateway&action=help&page="+t+"&from="+e+"&rand="+ajaxRandomString())}function checkUncheckRsAll(){return $jq("#batch_action_buttons").show(),$$("input[name^=checktable]").each(function(e){$(e).checked=!$(e).checked,$(e).onclick()}),!0}function checkUncheckRequestGroup(e){$jq("#batch_action_buttons").show();var t=$$("input."+e);$("groupcheckbox_"+e).checked?t.each(function(e){e.checked==!1&&e.click()}):t.each(function(e){e.checked==!0&&e.click()})}function streamViewPrev(n){if($jq("[id^=link-]").length)var t,e="link-";else e="takeit-";t=$jq("[id^="+e+"]").first().attr("id").replace(e,""),$jq("[id^="+e+"]").each(function(){if(t==currentReqidPopup)return hs_alert(n),!1;if($jq(this).attr("id")==e+currentReqidPopup)return showOverflow(t.replace(e,"")),!1;t=$jq(this).attr("id")})}function streamViewNext(n){if($jq("[id^=link-]").length)var t,e="link-";else e="takeit-";t=$jq("[id^="+e+"]").last().attr("id").replace(e,""),$jq($jq("[id^="+e+"]").get().reverse()).each(function(){if(t==currentReqidPopup)return hs_alert(n);if($jq(this).attr("id")==e+currentReqidPopup)return showOverflow(t.replace(e,""));t=$jq(this).attr("id")})}function showNoteItemMenu(){$jq(".note-stream-item-menubtn").length&&$jq(".note-stream-item-menubtn").each(function(){new Tip(this.id,$jq("#"+this.id+"-content").html(),{title:"",border:0,radius:0,delay:0,className:"hstinytip autoclose",stem:"topMiddle",showOn:"click",hideOn:!1,hideAfter:1,width:"auto",hook:{target:"bottomMiddle",tip:"topMiddle"},offset:{x:0,y:0}})})}function ms_select(e,n,s){var t=e+"-hidden";$(e).toggleClassName("select-multiple-selected"),$(t)?$(t).remove():$(e).insert({after:'<input type="hidden" id="'+t+'" name="'+s+'[]" value="'+n+'" />'})}function ms_select_all(e){$$("."+e+"-select-multiple a").each(function(e){$(e.id+"-hidden")||e.onclick()})}function ms_expand(e){$$("."+e+"-select-multiple")[0].setStyle({height:"500px"})}function yes_no_btn(t,e,n){t=="yes"?($(e+"-yes").addClassName("btn-selected"),$(e+"-yes").removeClassName("btn-yes-no"),$(e+"-no").addClassName("btn-yes-no"),$(e+"-no").removeClassName("btn-selected")):($(e+"-no").addClassName("btn-selected"),$(e+"-no").removeClassName("btn-yes-no"),$(e+"-yes").addClassName("btn-yes-no"),$(e+"-yes").removeClassName("btn-selected")),$(e).setValue(n)}function addFolder(e,t){var n=initModal({footer:!0,width:"600px",closeMethods:["overlay","button","escape"],href:"admin?pg=ajax_gateway&action=addfolder&default="+eq(e),buttons:[{text:button_save,handler:function(){add_folder_action($jq("#new_folder").val(),t)},icon:'',cssClass:"mobi-close accent"},{text:button_close,handler:"cancel",icon:'',cssClass:"mobi-close"}]})}function add_folder_action(t,e){foldername=t,folders=$(e).options,folderlen=folders.length,newoption=folderlen,$(e).options[newoption]=new Option(foldername,foldername),$(e).selectedIndex=newoption,closeAllModals()}function ttm_tip(e,t){new Tip(e,t,{title:"",className:"hstinytip autoclose",stem:"topMiddle",border:0,radius:0,showOn:"mouseover",hideOn:"mouseout",width:"auto",hook:{target:"bottomMiddle",tip:"topMiddle"}})}function ttm_tip_fat(e,t){new Tip(e,t,{title:"",className:"hstinytipfat autoclose",stem:"topMiddle",border:0,radius:0,showOn:"mouseover",hideOn:"mouseout",width:"160px",hook:{target:"bottomMiddle",tip:"topMiddle"}})}function initModal(e){return hs_overlay(e)}function callerCheckbox(){$jq(".popup-holder").on("click",".js-select-request",function(){$jq("#"+parseInt($jq(this).attr("rel"))+"_checkbox").trigger("click")})}currentReqidPopup=0;function showOverflow(e,s=!0){currentReqidPopup=e;var n,o="admin?pg=request.static&from_streamview=1&reqid="+e+"&rand="+ajaxRandomString(),t='';s==!0&&(t='			<button class="btn inline-action" onclick="streamViewPrev(\''+lg_streamview_end+'\')" style="margin-left:10px;">'+lg_prev+'</button> 			<button class="btn inline-action" onclick="streamViewNext(\''+lg_streamview_end+"')\">"+lg_next+"</button> 		"),n='		<div style="display:flex;flex-grow:1;justify-content:space-between;">			<div style=""> 				<input type="checkbox" value="1" rel="'+e+'" class="form-checkbox js-select-request" style="width: 30px;height: 30px;" /> 				<a href="admin?pg=request&reqid='+e+'" class="btn inline-action js-overflow-id" style="margin-left:10px;font-wight:bold;">'+e+"</a> 				"+t+' 			</div>			<div style="">  				<a href="" class="btn accent inline-action tingle-btn-right" onclick="closeAllModals();return false;">'+button_close+"</a>  			</div>		</div>	",$jq.get(o,function(s){modalIsOpen()?($jq(".popup-holder").html(s+" "+n),$jq(".prevNext").html(t),$jq("#"+parseInt(e)+"_checkbox").is(":checked")?$jq(".js-select-request").prop("checked",!0):$jq(".js-select-request").prop("checked",!1)):(closeAllModals(),overflowOpen=!0,modal=hs_overlay({html:s,footer:!1,footerHtml:n,stickyFooter:!0,width:window.innerWidth-100,close:!1,buttons:[],onOpen:function(){callerCheckbox(e),$jq("#"+parseInt(e)+"_checkbox").is(":checked")?$jq(".js-select-request").prop("checked",!0):$jq(".js-select-request").prop("checked",!1),$jq(".prevNext").html(t)}}))})}function showHistoryEmailAndHeaders(e,n,s){if(n=="emailheaders")var t="admin?pg=ajax_gateway&action=emailheaders&reqhisid="+e+"&rand="+ajaxRandomString();else t="admin?pg=ajax_gateway&action=emailsource&reqhisid="+e+"&rand="+ajaxRandomString();hs_overlay({href:t,title:s})}function simplemenu_action(n,e,t){t=typeof t!="undefined"?t:"menu",t!="triage"&&document.fire("hs_overlay:closed"),n=="unread"?($("replied_img_"+e).onclick(),new Effect.Highlight("tr-"+e,{duration:2,startcolor:"#ffffff",endcolor:"#ffff99"})):n=="trash"?($("tr-"+e).addClassName("tablerow-trash"),new Ajax.Request("admin?pg=ajax_gateway&action=simplemenu_trash",{method:"post",parameters:{reqid:e},onComplete:function(n){n.responseText!=""?hs_alert(n.responseText):(new Effect.Highlight("tr-"+e,{duration:1,startcolor:"#FDDFD7",endcolor:"#D8B3AE",afterFinish:function(){$("tr-"+e).remove()}}),t=="triage"&&triage_next())}})):n=="spam"&&($("tr-"+e).addClassName("tablerow-spam"),new Ajax.Request("admin?pg=ajax_gateway&action=simplemenu_spam",{method:"post",parameters:{reqid:e},onComplete:function(n){n.responseText!=""?(hs_alert(n.responseText),$("tr-"+e).removeClassName("tablerow-spam")):(new Effect.Highlight("tr-"+e,{duration:1,startcolor:"#FDDFD7",endcolor:"#D8B3AE",afterFinish:function(){$("tr-"+e).remove()}}),t=="triage"&&triage_next())}}))}person_status_update_details_flag=!1;function person_status_update_details(e,t,n,s){if(!person_status_update_details_flag){var o="xPersonStatus="+eq(e)+"&sPage="+eq(t)+"&fType="+eq(n)+"&sDetails="+eq(s)+"&rand="+ajaxRandomString(),i=new Ajax.Request("admin?pg=ajax_gateway&action=person_status_details",{method:"post",parameters:o});person_status_update_details_flag=!0}}function showPersonStatusWorkspace(e,t){var n=new Ajax.Request("admin?pg=ajax_gateway&action=person_status_workspace",{method:"get",parameters:{rand:ajaxRandomString(),reqid:t},onComplete:function(){new Tip(e,arguments[0].responseText,{title:t,border:0,radius:0,className:"hstinytip autoclose",stem:"leftMiddle",showOn:"click",hideOn:{element:"closeButton",event:"click"},hideOthers:!0,width:250,hook:{target:"rightMiddle",tip:"leftMiddle"},offset:{x:-5,y:0}}),$(e).prototip.show()}})}function hs_overlay(){if($jq.type(arguments[0])=="string")var t=$jq("#"+arguments[0]),e=arguments[1];else t=$jq(".popup-holder"),e=arguments[0];return e=Object.extend({href:!1,html:!1,footer:!1,footerHtml:'',stickyFooter:!1,onOpen:!1,onClose:!1,beforeOpen:!1,beforeClose:!1,cssClass:[],width:window.innerWidth-100,height:!1,display:"center",close:button_close,cssClass:"mbsc-no-padding md-content-scroll",buttons:[{text:button_close,handler:"cancel",icon:'',cssClass:"mobi-close"}]},e||{}),e.width&&t.css("width",e.width),scrollable=t.mobiscroll().popup({display:e.display,scrollLock:!1,layout:"fixed",cssClass:e.cssClass,onInit:function(){$jq("body").addClass("mobi-open")},onBeforeShow:e.beforeOpen,onShow:e.onOpen,onClose:function(){$jq("body").removeClass("mobi-open")},buttons:e.buttons}).mobiscroll("getInst"),e.href?$jq.get(e.href,function(n){t.html(n+" "+e.footerHtml),scrollable.show()}):e.html?(t.html(e.html+" "+e.footerHtml),scrollable.show()):scrollable.show(),!1}function insertTemplates(e){$("ta_"+e)&&($(e).value=$F("ta_"+e)),$("ta_"+e+"_html")&&($(e+"_html").value=$F("ta_"+e+"_html")),$("ta_"+e+"_HTML")&&($(e+"_HTML").value=$F("ta_"+e+"_HTML")),$("ta_"+e+"_subject")&&($(e+"_subject").value=$F("ta_"+e+"_subject")),$(e+"_savemsg").show(),closeAllModals()}function hs_hover(e,t){Element.hasClassName(e,t)?Element.removeClassName(e,t):Element.addClassName(e,t)}function getElementPosition(s){for(var e=document.getElementById(s),t=0,n=0;e;)t+=e.offsetLeft,n+=e.offsetTop,e=e.offsetParent;return navigator.userAgent.indexOf("Mac")!=-1&&typeof document.body.leftMargin!="undefined"&&(t+=document.body.leftMargin,n+=document.body.topMargin),{left:t,top:n}}function innerWinSize(){var e=0,t=0;return typeof window.innerWidth=="number"?(e=window.innerWidth,t=window.innerHeight):document.documentElement&&(document.documentElement.clientWidth||document.documentElement.clientHeight)?(e=document.documentElement.clientWidth,t=document.documentElement.clientHeight):document.body&&(document.body.clientWidth||document.body.clientHeight)&&(e=document.body.clientWidth,t=document.body.clientHeight),{width:e,height:t}}function custom_ajax_field_lookup(e,t,n,s){query=getRequestFields(),query.set("callingField",n);var i,o="&url="+eq(t)+"&"+query.toQueryString()+"&rand="+ajaxRandomString();$(e).innerHTML=s,$(e).show(),i=new Ajax.Request("admin?pg=ajax_gateway&action=ajax_field_lookup",{method:"post",parameters:o,onComplete:function(){$(e).innerHTML=arguments[0].responseText}})}function ll_popup(e){closeAllModals(),hs_overlay("ll_popup_content_"+e,{buttons:[]})}function ll_popup_move(e,t){$jq("#ll_popup_"+e).length?$jq("#ll_popup_"+e).click():hs_alert(t)}function getRequestFields(){var t,s,n=$H();if($("sUserId")){n=$H({sUserId:eq($F("sUserId")),sFirstName:eq($F("sFirstName")),sLastName:eq($F("sLastName")),sEmail:eq($F("sEmail")),sPhone:eq($F("sPhone")),xStatus:eq($F("xStatus")),xCategory:eq($F("xCategory")),xPersonAssignedTo:eq($F("xPersonAssignedTo"))}),s=document.getElementsByTagName("form");for(i=0;i<s.length;i++){t=Form.getElements(document.forms[i]);for(e=0;e<t.length;e++)t[e].id.indexOf("Custom")!==-1&&t[e].id.indexOf("_")===-1&&n.set(t[e].id,eq($F(t[e].id)))}}return n}HS_Effects={RsSetOrder:function(e,t){Element.hide(e),Element.show(t)},RsReturnSetOrder:function(e){goPage(e)}};function ajaxError(){}function ajaxLoadingImg(){return'<span class="spinner spin"></span>'}function ajaxLoading(){return'<div style="display: flex;justify-content: center;"><div class="inline_loading">'+ajaxLoadingImg()+"</div></div>"}function ajaxRandomString(){for(var e,t="0123456789ABCDEFGHIJKLMNOPQRSTUVWXTZabcdefghiklmnopqrstuvwxyz",o=8,n='',s=0;s<o;s++)e=Math.floor(Math.random()*t.length),n+=t.substring(e,e+1);return n.toString()}function addSortableColumn(o,r,h,m,l){var t,n,s,i,a,c,d,u,f,e=$(o);e.options?e.selectedIndex!=0&&(i=e.options[e.selectedIndex].text,e.options[e.selectedIndex].value.indexOf("@@@")!==-1?(s=e.options[e.selectedIndex].value.split("@@@"),t=s[0],n=s[1]?s[1]:"",d=s[2]=="hideflow"?"overflowMsg":"setColumnWidth"):(t=$F(o)),u=e.selectedIndex,e.selectedIndex=0,e.options[u]=null):(i=e.value,t=eq(e.value)),c=Math.floor(Math.random()*2e9),a=Builder.node("div",{id:o+"_"+t,className:"sortable_filter",style:"display:none;"},[Builder.node("img",{src:static_path+"/img5/grip-lines-regular.svg",className:"drag_handle",style:"vertical-align: middle;cursor:move;width:16px;height:16px;margin-right:6px;"})," ",m?Builder.node("img",{src:static_path+"/img/space.gif",width:"16px",height:"16px",style:"vertical-align: middle;cursor:pointer;"}):""," ",Builder.node("span",{id:o+"_"+t+"_text"},i),typeof n!="undefined"?Builder.node("span",{id:"column_width_"+t,className:"hand filter_width_text",onclick:d+"('column_width_"+t+"');"},n!==""?n:Builder.node("img",{src:static_path+"/img5/arrows-h-solid.svg"})):"",typeof n!="undefined"?Builder.node("input",{type:"hidden",name:"column_width_"+t+"_value",id:"column_width_"+t+"_value",value:n}):"",Builder.node("input",{type:"hidden",name:h,value:t}),typeof l!="undefined"?Builder.node("input",{class:"jscolor jscolor-small","data-jscolor":"{required:false,hash:true}",name:"sListItemsColors[]",value:"",id:c}):"",Builder.node("img",{src:static_path+"/img5/remove.svg",onClick:"return confirmRemove('"+o+"_"+t+"', confirmListDelete);",style:"vertical-align: middle;cursor:pointer;width:16px;height:16px;"})]),$(r).appendChild(a),Effect.Appear(a.id),typeof l!="undefined"&&(f=new jscolor(document.getElementById(c),{required:!1,hash:!0,value:null})),Sortable.destroy(r),Sortable.create(r,{tag:"div",constraint:"vertical"})}function insertColumnWidth(e){var t=trim($F(e+"_textbox"));$(e).update(t==""?'<img src="'+static_path+'/img5/arrows-h-solid.svg" />':t),$(e+"_value").value=t,$(e).prototip.remove()}function safari_order_fix(t,n){if(navigator.appVersion.match(/Konqueror|Safari|KHTML/)){var e=Form.getInputs(n,"hidden",t);for(i=0;i<e.length;i++)new_fields=Builder.node("input",{type:"hidden",name:t,value:e[i].value}),$(n).appendChild(new_fields);for(i=0;i<e.length;i++)Element.remove(e[i])}return!1}function stopFormEnter(e){return $F(e)==''||(document.getElementById(e).focus(),!1)}function eq(e){return encodeURIComponent(e)}function confirmRemove(e,t){return hs_confirm(t,function(){Element.remove(e)})}function insertAtCursor(e,t){if(document.selection)e.focus(),sel=document.selection.createRange(),sel.text=t;else if(e.selectionStart||e.selectionStart=="0"){var n=e.selectionStart,s=e.selectionEnd;e.value=e.value.substring(0,n)+t+e.value.substring(s,e.value.length)}else e.value+=t}function resize_all_textareas(){if(!RegExp("iPhone").test(navigator.userAgent)&&!RegExp("iPod").test(navigator.userAgent))for(var t=document.getElementsByTagName("textarea"),e=0;e<t.length;e++)t[e].id!="tBody"&&new ResizeableTextarea(t[e])}ResizeableTextarea=Class.create(),ResizeableTextarea.prototype={initialize:function(e,t){this.element=$(e),this.size=parseFloat(this.element.getStyle("height")||"100"),this.options=Object.extend({inScreen:!0,resizeStep:10,minHeight:this.size},t||{}),Event.observe(this.element,"keyup",this.resize.bindAsEventListener(this)),this.options.inScreen||(this.element.style.overflow="hidden"),this.element.setAttribute("wrap","virtual"),this.resize()},resize:function(){this.shrink(),this.grow()},shrink:function(){if(this.size<=this.options.minHeight)return;this.element.scrollHeight<=this.element.clientHeight&&(this.size-=this.options.resizeStep,this.element.style.height=this.size+"px",this.shrink())},grow:function(){if(this.element.scrollHeight>this.element.clientHeight){if(this.options.inScreen&&20+this.element.offsetTop+this.element.clientHeight>document.body.clientHeight)return;this.size+=this.element.scrollHeight-this.element.clientHeight+this.options.resizeStep,this.element.style.height=this.size+"px",this.grow()}}};function calendar_clean_date(t){var e=new Date(t*1e3);return new Date(e.getFullYear(),e.getMonth(),e.getDate(),12,0,0)}function showhide_datefield(e){$F(e+"_2")=="is"||$F(e+"_2")=="less_than"||$F(e+"_2")=="greater_than"?$(e+"_3")&&($(e+"_3").show(),$(e+"_3_show_calendar").show()):$(e+"_3")&&($(e+"_3").hide(),$(e+"_3_show_calendar").hide())}function showhide_thermostatfield(e){var s=$jq("#"+e+"_2"),t=$jq("#"+e+"_3_tf"),n=$jq("#"+e+"_3_sf");s.length&&(s.val()=="is"||s.val()=="less_than"||s.val()=="greater_than"?(n.length&&(n.hide(),n.attr("name",e+"_3_off")),t.length&&(t.attr("name",e+"_3"),t.show())):(t.length&&(t.hide(),t.attr("name",e+"_3_off")),n.length&&(n.attr("name",e+"_3"),n.show())))}function do_min_calc(){var e=$("calc_days").value*60*24,e=e+$("calc_hours").value*60;$(calc_row).value=e}note_option_string='<div id="@tabid"><table cellpadding="0" cellspacing="0" border="0" class="noteoptiontable"><tr class="noteoptionrow"><td class="noteoptiontab" width="145">@tabtext</td><td class="noteoptiontabexp" align="right">@tabexp</td></tr></table><table cellpadding="0" cellspacing="0" border="0" style="margin-bottom:0px;width: 100%;"><tr><td colspan="2" class="noteoptioninner" id="noteoptioninner_@tabid">@bodytext</td></tr></table></div>';function validate_email(e){var t=/^((([a-z]|\d|[!#$%&'*+\-/=?^_`{|}~]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])+(\.([a-z]|\d|[!#$%&'*+\-/=?^_`{|}~]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])+)*)|((\x22)((((\x20|\x09)*(\x0d\x0a))?(\x20|\x09)+)?(([\x01-\x08\x0b\x0c\x0e-\x1f\x7f]|\x21|[\x23-\x5b]|[\x5d-\x7e]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(\\([\x01-\x09\x0b\x0c\x0d-\x7f]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF]))))*(((\x20|\x09)*(\x0d\x0a))?(\x20|\x09)+)?(\x22)))@((([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])*([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])))\.)+(([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])*([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])))$/i;return!!e&&t.test(e)}function showViewers(){$jq.get("admin?pg=ajax_gateway&action=get_request_viewers",function(e){var t=[];$jq.each(e,function(e){t.push("viewing-"+e)}),$jq.each(e,function(e,t){$jq("#viewing-"+e).addClass("viewing-"+t.fType).attr("onclick","showPersonStatusWorkspace('viewing-"+e+"',"+e+");")}),$jq(".viewing").each(function(){$jq.inArray(this.id,t)==-1&&$jq(this).removeClass("viewing-2 viewing-1").removeAttr("onclick")})},"json")}$jq(document).ready(function(){var e=$jq("input.canCheck:checked").length==$jq("input.canCheck").length;$jq(".check-all").prop("checked",e),$jq(".check-all").on("change",function(){$jq(".canCheck").prop("checked",$jq(this).prop("checked"))}),$jq(".canCheck").on("change",function(){var e=$jq(".canCheck:checked").length==$jq(".canCheck").length;$jq(".check-all").prop("checked",e)}),$jq(".js-check-all").on("click",function(){var e=!($jq(".canCheck:checked").length==$jq(".canCheck").length);return $jq(".canCheck").prop("checked",e),!1}),$jq(".color-label").contrastColor()}),$jq(document).on("click","a.note-stream-item-pin",function(){var e=$jq(this).data("id");$jq.ajax({method:"GET",url:"admin?pg=ajax_gateway&action=request_history_pin",data:{xRequestHistory:e}}).done(function(){initRequestHistory()})}),$jq.fn.contrastColor=function(){return this.each(function(){var t,n,e=$jq(this).css("background-color");if((e=="transparent"||e=="rgba(0, 0, 0, 0)")&&($jq(this).parents().each(function(){if(e=$jq(this).css("background-color"),e!="transparent"&&e!="rgba(0, 0, 0, 0)")return!1}),e=="transparent"||e=="rgba(0, 0, 0, 0)"))return!1;t=e.replace(/^(rgb|rgba)\(/,'').replace(/\)$/,'').replace(/\s/g,'').split(","),n=(t[0]*299+t[1]*587+t[2]*114)/1e3,n>=128?$jq(this).removeClass("light-color"):$jq(this).addClass("light-color")})};function addSlashes(e){return e.replace(/'/g,"\\'")}function removeSlashes(e){return e.replace(/'/g,"'")}function dismissNotification(t){t.preventDefault();var e=t.currentTarget.dataset.notification;return typeof window.HS.user.notifications[e]!="undefined"&&delete window.HS.user.notifications[e],$jq("#notification-"+e).remove(),$jq.isEmptyObject(window.HS.user.notifications)&&$jq("#hs_notification_window").remove(),$jq.post(window.chost+"/notifications/"+e,{_method:"DELETE"}),console.log($jq("#hs_notification_window").length),$jq("#hs_notification_window").length==0&&$jq(".hdsystembox").remove(),!1}function dismissAllNotifications(e){return e.preventDefault(),$jq.post("/notifications/all",{_method:"DELETE"}),$jq(".hdsystembox").remove(),!1}if(Builder={NODEMAP:{AREA:"map",CAPTION:"table",COL:"table",COLGROUP:"table",LEGEND:"fieldset",OPTGROUP:"select",OPTION:"select",PARAM:"object",TBODY:"table",TD:"table",TFOOT:"table",TH:"table",THEAD:"table",TR:"table"},node:function(t){t=t.toUpperCase();var e,s,o=this.NODEMAP[t]||"div",n=document.createElement(o);try{n.innerHTML="<"+t+"></"+t+">"}catch(e){}if(e=n.firstChild||null,e&&e.tagName.toUpperCase()!=t&&(e=e.getElementsByTagName(t)[0]),e||(e=document.createElement(t)),!e)return;if(arguments[1])if(this._isStringOrNumber(arguments[1])||arguments[1]instanceof Array||arguments[1].tagName)this._children(e,arguments[1]);else if(s=this._attributes(arguments[1]),s.length){try{n.innerHTML="<"+t+" "+s+"></"+t+">"}catch(e){}if(e=n.firstChild||null,!e){e=document.createElement(t);for(attr in arguments[1])e[attr=="class"?"className":attr]=arguments[1][attr]}e.tagName.toUpperCase()!=t&&(e=n.getElementsByTagName(t)[0])}return arguments[2]&&this._children(e,arguments[2]),$(e)},_text:function(e){return document.createTextNode(e)},ATTR_MAP:{className:"class",htmlFor:"for"},_attributes:function(e){var t=[];for(attribute in e)t.push((attribute in this.ATTR_MAP?this.ATTR_MAP[attribute]:attribute)+'="'+e[attribute].toString().escapeHTML().gsub(/"/,"&quot;")+'"');return t.join(" ")},_children:function(t,e){if(e.tagName){t.appendChild(e);return}typeof e=="object"?e.flatten().each(function(e){typeof e=="object"?t.appendChild(e):Builder._isStringOrNumber(e)&&t.appendChild(Builder._text(e))}):Builder._isStringOrNumber(e)&&t.appendChild(Builder._text(e))},_isStringOrNumber:function(e){return typeof e=="string"||typeof e=="number"},build:function(t){var e=this.node("div");return $(e).update(t.strip()),e.down()},dump:function(e){typeof e!="object"&&typeof e!="function"&&(e=window);var t=("A ABBR ACRONYM ADDRESS APPLET AREA B BASE BASEFONT BDO BIG BLOCKQUOTE BODY BR BUTTON CAPTION CENTER CITE CODE COL COLGROUP DD DEL DFN DIR DIV DL DT EM FIELDSET FONT FORM FRAME FRAMESET H1 H2 H3 H4 H5 H6 HEAD HR HTML I IFRAME IMG INPUT INS ISINDEX KBD LABEL LEGEND LI LINK MAP MENU META NOFRAMES NOSCRIPT OBJECT OL OPTGROUP OPTION P PARAM PRE Q S SAMP SCRIPT SELECT SMALL SPAN STRIKE STRONG STYLE SUB SUP TABLE TBODY TD TEXTAREA TFOOT TH THEAD TITLE TR TT U UL VAR").split(/\s+/);t.each(function(t){e[t]=function(){return Builder.node.apply(Builder,[t].concat($A(arguments)))}})}},String.prototype.parseColor=function(){var e,n,t="#";if(this.slice(0,4)=="rgb("){n=this.slice(4,this.length-1).split(","),e=0;do t+=parseInt(n[e]).toColorPart();while(++e<3)}else if(this.slice(0,1)=="#"){if(this.length==4)for(e=1;e<4;e++)t+=(this.charAt(e)+this.charAt(e)).toLowerCase();this.length==7&&(t=this.toLowerCase())}return t.length==7?t:arguments[0]||this},Element.collectTextNodes=function(e){return $A($(e).childNodes).collect(function(e){return e.nodeType==3?e.nodeValue:e.hasChildNodes()?Element.collectTextNodes(e):''}).flatten().join('')},Element.collectTextNodesIgnoreClass=function(t,e){return $A($(t).childNodes).collect(function(t){return t.nodeType==3?t.nodeValue:t.hasChildNodes()&&!Element.hasClassName(t,e)?Element.collectTextNodesIgnoreClass(t,e):''}).flatten().join('')},Element.setContentZoom=function(e,t){return e=$(e),e.setStyle({fontSize:t/100+"em"}),Prototype.Browser.WebKit&&window.scrollBy(0,0),e},Element.getInlineOpacity=function(e){return $(e).style.opacity||''},Element.forceRerendering=function(e){try{e=$(e);var t=document.createTextNode(" ");e.appendChild(t),e.removeChild(t)}catch(e){}},Effect={_elementDoesNotExistError:{name:"ElementDoesNotExistError",message:"The specified DOM element does not exist, but is required for this effect to operate"},Transitions:{linear:Prototype.K,sinoidal:function(e){return-Math.cos(e*Math.PI)/2+.5},reverse:function(e){return 1-e},flicker:function(e){var e=-Math.cos(e*Math.PI)/4+.75+Math.random()/4;return e>1?1:e},wobble:function(e){return-Math.cos(e*Math.PI*(9*e))/2+.5},pulse:function(e,t){return-Math.cos(e*((t||5)-.5)*2*Math.PI)/2+.5},spring:function(e){return 1-Math.cos(e*4.5*Math.PI)*Math.exp(-e*6)},none:function(){return 0},full:function(){return 1}},DefaultOptions:{duration:1,fps:100,sync:!1,from:0,to:1,delay:0,queue:"parallel"},tagifyText:function(e){var t="position:relative";Prototype.Browser.IE&&(t+=";zoom:1"),e=$(e),$A(e.childNodes).each(function(n){n.nodeType==3&&(n.nodeValue.toArray().each(function(s){e.insertBefore(new Element("span",{style:t}).update(s==" "?String.fromCharCode(160):s),n)}),Element.remove(n))})},multiple:function(e,s){(typeof e=="object"||Object.isFunction(e))&&e.length?t=e:t=$(e).childNodes;var t,n=Object.extend({speed:.1,delay:0},arguments[2]||{}),o=n.delay;$A(t).each(function(e,t){new s(e,Object.extend(n,{delay:t*n.speed+o}))})},PAIRS:{slide:["SlideDown","SlideUp"],blind:["BlindDown","BlindUp"],appear:["Appear","Fade"]},toggle:function(e,t,n){return e=$(e),t=(t||"appear").toLowerCase(),Effect[Effect.PAIRS[t][e.visible()?1:0]](e,Object.extend({queue:{position:"end",scope:e.id||"global",limit:1}},n||{}))}},Effect.DefaultOptions.transition=Effect.Transitions.sinoidal,Effect.ScopedQueue=Class.create(Enumerable,{initialize:function(){this.effects=[],this.interval=null},_each:function(e){this.effects._each(e)},add:function(e){var t=(new Date).getTime(),n=Object.isString(e.options.queue)?e.options.queue:e.options.queue.position;switch(n){case"front":this.effects.findAll(function(e){return e.state=="idle"}).each(function(t){t.startOn+=e.finishOn,t.finishOn+=e.finishOn});break;case"with-last":t=this.effects.pluck("startOn").max()||t;break;case"end":t=this.effects.pluck("finishOn").max()||t;break}e.startOn+=t,e.finishOn+=t,(!e.options.queue.limit||this.effects.length<e.options.queue.limit)&&this.effects.push(e),this.interval||(this.interval=setInterval(this.loop.bind(this),15))},remove:function(e){this.effects=this.effects.reject(function(t){return t==e}),this.effects.length==0&&(clearInterval(this.interval),this.interval=null)},loop:function(){t=(new Date).getTime();for(var t,e=0,n=this.effects.length;e<n;e++)this.effects[e]&&this.effects[e].loop(t)}}),Effect.Queues={instances:$H(),get:function(e){return Object.isString(e)?this.instances.get(e)||this.instances.set(e,new Effect.ScopedQueue):e}},Effect.Queue=Effect.Queues.get("global"),Effect.Base=Class.create({position:null,start:function(e){e&&e.transition===!1&&(e.transition=Effect.Transitions.linear),this.options=Object.extend(Object.extend({},Effect.DefaultOptions),e||{}),this.currentFrame=0,this.state="idle",this.startOn=this.options.delay*1e3,this.finishOn=this.startOn+this.options.duration*1e3,this.fromToDelta=this.options.to-this.options.from,this.totalTime=this.finishOn-this.startOn,this.totalFrames=this.options.fps*this.options.duration,this.render=function(){function e(e,t){e.options[t+"Internal"]&&e.options[t+"Internal"](e),e.options[t]&&e.options[t](e)}return function(t){this.state==="idle"&&(this.state="running",e(this,"beforeSetup"),this.setup&&this.setup(),e(this,"afterSetup")),this.state==="running"&&(t=this.options.transition(t)*this.fromToDelta+this.options.from,this.position=t,e(this,"beforeUpdate"),this.update&&this.update(t),e(this,"afterUpdate"))}}(),this.event("beforeStart"),this.options.sync||Effect.Queues.get(Object.isString(this.options.queue)?"global":this.options.queue.scope).add(this)},loop:function(e){if(e>=this.startOn){if(e>=this.finishOn){this.render(1),this.cancel(),this.event("beforeFinish"),this.finish&&this.finish(),this.event("afterFinish");return}var t=(e-this.startOn)/this.totalTime,n=(t*this.totalFrames).round();n>this.currentFrame&&(this.render(t),this.currentFrame=n)}},cancel:function(){this.options.sync||Effect.Queues.get(Object.isString(this.options.queue)?"global":this.options.queue.scope).remove(this),this.state="finished"},event:function(e){this.options[e+"Internal"]&&this.options[e+"Internal"](this),this.options[e]&&this.options[e](this)},inspect:function(){var e=$H();for(property in this)Object.isFunction(this[property])||e.set(property,this[property]);return"#<Effect:"+e.inspect()+",options:"+$H(this.options).inspect()+">"}}),Effect.Parallel=Class.create(Effect.Base,{initialize:function(e){this.effects=e||[],this.start(arguments[1])},update:function(e){this.effects.invoke("render",e)},finish:function(e){this.effects.each(function(t){t.render(1),t.cancel(),t.event("beforeFinish"),t.finish&&t.finish(e),t.event("afterFinish")})}}),Effect.Tween=Class.create(Effect.Base,{initialize:function(e,s,o){e=Object.isString(e)?$(e):e;var n=$A(arguments),t=n.last(),i=n.length==5?n[3]:null;this.method=Object.isFunction(t)?t.bind(e):Object.isFunction(e[t])?e[t].bind(e):function(n){e[t]=n},this.start(Object.extend({from:s,to:o},i||{}))},update:function(e){this.method(e)}}),Effect.Event=Class.create(Effect.Base,{initialize:function(){this.start(Object.extend({duration:0},arguments[0]||{}))},update:Prototype.emptyFunction}),Effect.Opacity=Class.create(Effect.Base,{initialize:function(e){if(this.element=$(e),!this.element)throw Effect._elementDoesNotExistError;Prototype.Browser.IE&&!this.element.currentStyle.hasLayout&&this.element.setStyle({zoom:1});var t=Object.extend({from:this.element.getOpacity()||0,to:1},arguments[1]||{});this.start(t)},update:function(e){this.element.setOpacity(e)}}),Effect.Move=Class.create(Effect.Base,{initialize:function(e){if(this.element=$(e),!this.element)throw Effect._elementDoesNotExistError;var t=Object.extend({x:0,y:0,mode:"relative"},arguments[1]||{});this.start(t)},setup:function(){this.element.makePositioned(),this.originalLeft=parseFloat(this.element.getStyle("left")||"0"),this.originalTop=parseFloat(this.element.getStyle("top")||"0"),this.options.mode=="absolute"&&(this.options.x=this.options.x-this.originalLeft,this.options.y=this.options.y-this.originalTop)},update:function(e){this.element.setStyle({left:(this.options.x*e+this.originalLeft).round()+"px",top:(this.options.y*e+this.originalTop).round()+"px"})}}),Effect.MoveBy=function(e,t,n){return new Effect.Move(e,Object.extend({x:n,y:t},arguments[3]||{}))},Effect.Scale=Class.create(Effect.Base,{initialize:function(e,t){if(this.element=$(e),!this.element)throw Effect._elementDoesNotExistError;var n=Object.extend({scaleX:!0,scaleY:!0,scaleContent:!0,scaleFromCenter:!1,scaleMode:"box",scaleFrom:100,scaleTo:t},arguments[2]||{});this.start(n)},setup:function(){this.restoreAfterFinish=this.options.restoreAfterFinish||!1,this.elementPositioning=this.element.getStyle("position"),this.originalStyle={},["top","left","width","height","fontSize"].each(function(e){this.originalStyle[e]=this.element.style[e]}.bind(this)),this.originalTop=this.element.offsetTop,this.originalLeft=this.element.offsetLeft;var e=this.element.getStyle("font-size")||"100%";["em","px","%","pt"].each(function(t){e.indexOf(t)>0&&(this.fontSize=parseFloat(e),this.fontSizeType=t)}.bind(this)),this.factor=(this.options.scaleTo-this.options.scaleFrom)/100,this.dims=null,this.options.scaleMode=="box"&&(this.dims=[this.element.offsetHeight,this.element.offsetWidth]),/^content/.test(this.options.scaleMode)&&(this.dims=[this.element.scrollHeight,this.element.scrollWidth]),this.dims||(this.dims=[this.options.scaleMode.originalHeight,this.options.scaleMode.originalWidth])},update:function(t){var e=this.options.scaleFrom/100+this.factor*t;this.options.scaleContent&&this.fontSize&&this.element.setStyle({fontSize:this.fontSize*e+this.fontSizeType}),this.setDimensions(this.dims[0]*e,this.dims[1]*e)},finish:function(){this.restoreAfterFinish&&this.element.setStyle(this.originalStyle)},setDimensions:function(s,o){var t,n,e={};this.options.scaleX&&(e.width=o.round()+"px"),this.options.scaleY&&(e.height=s.round()+"px"),this.options.scaleFromCenter&&(t=(s-this.dims[0])/2,n=(o-this.dims[1])/2,this.elementPositioning=="absolute"?(this.options.scaleY&&(e.top=this.originalTop-t+"px"),this.options.scaleX&&(e.left=this.originalLeft-n+"px")):(this.options.scaleY&&(e.top=-t+"px"),this.options.scaleX&&(e.left=-n+"px"))),this.element.setStyle(e)}}),Effect.Highlight=Class.create(Effect.Base,{initialize:function(e){if(this.element=$(e),!this.element)throw Effect._elementDoesNotExistError;var t=Object.extend({startcolor:"#ffff99"},arguments[1]||{});this.start(t)},setup:function(){if(this.element.getStyle("display")=="none"){this.cancel();return}this.oldStyle={},this.options.keepBackgroundImage||(this.oldStyle.backgroundImage=this.element.getStyle("background-image"),this.element.setStyle({backgroundImage:"none"})),this.options.endcolor||(this.options.endcolor=this.element.getStyle("background-color").parseColor("#ffffff")),this.options.restorecolor||(this.options.restorecolor=this.element.getStyle("background-color")),this._base=$R(0,2).map(function(e){return parseInt(this.options.startcolor.slice(e*2+1,e*2+3),16)}.bind(this)),this._delta=$R(0,2).map(function(e){return parseInt(this.options.endcolor.slice(e*2+1,e*2+3),16)-this._base[e]}.bind(this))},update:function(e){this.element.setStyle({backgroundColor:$R(0,2).inject("#",function(n,s,t){return n+(this._base[t]+this._delta[t]*e).round().toColorPart()}.bind(this))})},finish:function(){this.element.setStyle(Object.extend(this.oldStyle,{backgroundColor:this.options.restorecolor}))}}),Effect.ScrollTo=function(s){var e=arguments[1]||{},t=document.viewport.getScrollOffsets(),n=$(s).cumulativeOffset();return e.offset&&(n[1]+=e.offset),new Effect.Tween(null,t.top,n[1],e,function(e){scrollTo(t.left,e.round())})},Effect.Fade=function(e){e=$(e);var t=e.getInlineOpacity(),n=Object.extend({from:e.getOpacity()||1,to:0,afterFinishInternal:function(e){if(e.options.to!=0)return;e.element.hide().setStyle({opacity:t})}},arguments[1]||{});return new Effect.Opacity(e,n)},Effect.Appear=function(e){e=$(e);var t=Object.extend({from:e.getStyle("display")=="none"?0:e.getOpacity()||0,to:1,afterFinishInternal:function(e){e.element.forceRerendering()},beforeSetup:function(e){e.element.setOpacity(e.options.from).show()}},arguments[1]||{});return new Effect.Opacity(e,t)},Effect.Puff=function(e){e=$(e);var t={opacity:e.getInlineOpacity(),position:e.getStyle("position"),top:e.style.top,left:e.style.left,width:e.style.width,height:e.style.height};return new Effect.Parallel([new Effect.Scale(e,200,{sync:!0,scaleFromCenter:!0,scaleContent:!0,restoreAfterFinish:!0}),new Effect.Opacity(e,{sync:!0,to:0})],Object.extend({duration:1,beforeSetupInternal:function(e){Position.absolutize(e.effects[0].element)},afterFinishInternal:function(e){e.effects[0].element.hide().setStyle(t)}},arguments[1]||{}))},Effect.BlindUp=function(e){return e=$(e),e.makeClipping(),new Effect.Scale(e,0,Object.extend({scaleContent:!1,scaleX:!1,restoreAfterFinish:!0,afterFinishInternal:function(e){e.element.hide().undoClipping()}},arguments[1]||{}))},Effect.BlindDown=function(e){e=$(e);var t=e.getDimensions();return new Effect.Scale(e,100,Object.extend({scaleContent:!1,scaleX:!1,scaleFrom:0,scaleMode:{originalHeight:t.height,originalWidth:t.width},restoreAfterFinish:!0,afterSetup:function(e){e.element.makeClipping().setStyle({height:"0px"}).show()},afterFinishInternal:function(e){e.element.undoClipping()}},arguments[1]||{}))},Effect.SwitchOff=function(e){e=$(e);var t=e.getInlineOpacity();return new Effect.Appear(e,Object.extend({duration:.4,from:0,transition:Effect.Transitions.flicker,afterFinishInternal:function(e){new Effect.Scale(e.element,1,{duration:.3,scaleFromCenter:!0,scaleX:!1,scaleContent:!1,restoreAfterFinish:!0,beforeSetup:function(e){e.element.makePositioned().makeClipping()},afterFinishInternal:function(e){e.element.hide().undoClipping().undoPositioned().setStyle({opacity:t})}})}},arguments[1]||{}))},Effect.DropOut=function(e){e=$(e);var t={top:e.getStyle("top"),left:e.getStyle("left"),opacity:e.getInlineOpacity()};return new Effect.Parallel([new Effect.Move(e,{x:0,y:100,sync:!0}),new Effect.Opacity(e,{sync:!0,to:0})],Object.extend({duration:.5,beforeSetup:function(e){e.effects[0].element.makePositioned()},afterFinishInternal:function(e){e.effects[0].element.hide().undoPositioned().setStyle(t)}},arguments[1]||{}))},Effect.Shake=function(n){n=$(n);var s=Object.extend({distance:20,duration:.5},arguments[1]||{}),e=parseFloat(s.distance),t=parseFloat(s.duration)/10,o={top:n.getStyle("top"),left:n.getStyle("left")};return new Effect.Move(n,{x:e,y:0,duration:t,afterFinishInternal:function(n){new Effect.Move(n.element,{x:-e*2,y:0,duration:t*2,afterFinishInternal:function(n){new Effect.Move(n.element,{x:e*2,y:0,duration:t*2,afterFinishInternal:function(n){new Effect.Move(n.element,{x:-e*2,y:0,duration:t*2,afterFinishInternal:function(n){new Effect.Move(n.element,{x:e*2,y:0,duration:t*2,afterFinishInternal:function(n){new Effect.Move(n.element,{x:-e,y:0,duration:t,afterFinishInternal:function(e){e.element.undoPositioned().setStyle(o)}})}})}})}})}})}})},Effect.SlideDown=function(e){e=$(e).cleanWhitespace();var n=e.down().getStyle("bottom"),t=e.getDimensions();return new Effect.Scale(e,100,Object.extend({scaleContent:!1,scaleX:!1,scaleFrom:window.opera?0:1,scaleMode:{originalHeight:t.height,originalWidth:t.width},restoreAfterFinish:!0,afterSetup:function(e){e.element.makePositioned(),e.element.down().makePositioned(),window.opera&&e.element.setStyle({top:''}),e.element.makeClipping().setStyle({height:"0px"}).show()},afterUpdateInternal:function(e){e.element.down().setStyle({bottom:e.dims[0]-e.element.clientHeight+"px"})},afterFinishInternal:function(e){e.element.undoClipping().undoPositioned(),e.element.down().undoPositioned().setStyle({bottom:n})}},arguments[1]||{}))},Effect.SlideUp=function(e){e=$(e).cleanWhitespace();var n=e.down().getStyle("bottom"),t=e.getDimensions();return new Effect.Scale(e,window.opera?0:1,Object.extend({scaleContent:!1,scaleX:!1,scaleMode:"box",scaleFrom:100,scaleMode:{originalHeight:t.height,originalWidth:t.width},restoreAfterFinish:!0,afterSetup:function(e){e.element.makePositioned(),e.element.down().makePositioned(),window.opera&&e.element.setStyle({top:''}),e.element.makeClipping().show()},afterUpdateInternal:function(e){e.element.down().setStyle({bottom:e.dims[0]-e.element.clientHeight+"px"})},afterFinishInternal:function(e){e.element.hide().undoClipping().undoPositioned(),e.element.down().undoPositioned().setStyle({bottom:n})}},arguments[1]||{}))},Effect.Squish=function(e){return new Effect.Scale(e,window.opera?1:0,{restoreAfterFinish:!0,beforeSetup:function(e){e.element.makeClipping()},afterFinishInternal:function(e){e.element.hide().undoClipping()}})},Effect.Grow=function(t){t=$(t);var n,s,o,i,a=Object.extend({direction:"center",moveTransition:Effect.Transitions.sinoidal,scaleTransition:Effect.Transitions.sinoidal,opacityTransition:Effect.Transitions.full},arguments[1]||{}),r={top:t.style.top,left:t.style.left,height:t.style.height,width:t.style.width,opacity:t.getInlineOpacity()},e=t.getDimensions();switch(a.direction){case"top-left":s=o=i=n=0;break;case"top-right":s=e.width,o=n=0,i=-e.width;break;case"bottom-left":s=i=0,o=e.height,n=-e.height;break;case"bottom-right":s=e.width,o=e.height,i=-e.width,n=-e.height;break;case"center":s=e.width/2,o=e.height/2,i=-e.width/2,n=-e.height/2;break}return new Effect.Move(t,{x:s,y:o,duration:.01,beforeSetup:function(e){e.element.hide().makeClipping().makePositioned()},afterFinishInternal:function(t){new Effect.Parallel([new Effect.Opacity(t.element,{sync:!0,to:1,from:0,transition:a.opacityTransition}),new Effect.Move(t.element,{x:i,y:n,sync:!0,transition:a.moveTransition}),new Effect.Scale(t.element,100,{scaleMode:{originalHeight:e.height,originalWidth:e.width},sync:!0,scaleFrom:window.opera?1:0,transition:a.scaleTransition,restoreAfterFinish:!0})],Object.extend({beforeSetup:function(e){e.effects[0].element.setStyle({height:"0px"}).show()},afterFinishInternal:function(e){e.effects[0].element.undoClipping().undoPositioned().setStyle(r)}},a))}})},Effect.Shrink=function(e){e=$(e);var n,s,o=Object.extend({direction:"center",moveTransition:Effect.Transitions.sinoidal,scaleTransition:Effect.Transitions.sinoidal,opacityTransition:Effect.Transitions.none},arguments[1]||{}),i={top:e.style.top,left:e.style.left,height:e.style.height,width:e.style.width,opacity:e.getInlineOpacity()},t=e.getDimensions();switch(o.direction){case"top-left":n=s=0;break;case"top-right":n=t.width,s=0;break;case"bottom-left":n=0,s=t.height;break;case"bottom-right":n=t.width,s=t.height;break;case"center":n=t.width/2,s=t.height/2;break}return new Effect.Parallel([new Effect.Opacity(e,{sync:!0,to:0,from:1,transition:o.opacityTransition}),new Effect.Scale(e,window.opera?1:0,{sync:!0,transition:o.scaleTransition,restoreAfterFinish:!0}),new Effect.Move(e,{x:n,y:s,sync:!0,transition:o.moveTransition})],Object.extend({beforeStartInternal:function(e){e.effects[0].element.makePositioned().makeClipping()},afterFinishInternal:function(e){e.effects[0].element.hide().undoClipping().undoPositioned().setStyle(i)}},o))},Effect.Pulsate=function(e){e=$(e);var t=arguments[1]||{},n=e.getInlineOpacity(),s=t.transition||Effect.Transitions.linear,o=function(e){return 1-s(-Math.cos(e*(t.pulses||5)*2*Math.PI)/2+.5)};return new Effect.Opacity(e,Object.extend(Object.extend({duration:2,from:0,afterFinishInternal:function(e){e.element.setStyle({opacity:n})}},t),{transition:o}))},Effect.Fold=function(e){e=$(e);var t={top:e.style.top,left:e.style.left,width:e.style.width,height:e.style.height};return e.makeClipping(),new Effect.Scale(e,5,Object.extend({scaleContent:!1,scaleX:!1,afterFinishInternal:function(){new Effect.Scale(e,1,{scaleContent:!1,scaleY:!1,afterFinishInternal:function(e){e.element.hide().undoClipping().setStyle(t)}})}},arguments[1]||{}))},Effect.Morph=Class.create(Effect.Base,{initialize:function(n){if(this.element=$(n),!this.element)throw Effect._elementDoesNotExistError;var t,e=Object.extend({style:{}},arguments[1]||{});Object.isString(e.style)?e.style.include(":")?this.style=e.style.parseStyle():(this.element.addClassName(e.style),this.style=$H(this.element.getStyles()),this.element.removeClassName(e.style),t=this.element.getStyles(),this.style=this.style.reject(function(e){return e.value==t[e.key]}),e.afterFinishInternal=function(e){e.element.addClassName(e.options.style),e.transforms.each(function(t){e.element.style[t.style]=''})}):this.style=$H(e.style),this.start(e)},setup:function(){function e(e){return(!e||["rgba(0, 0, 0, 0)","transparent"].include(e))&&(e="#ffffff"),e=e.parseColor(),$R(0,2).map(function(t){return parseInt(e.slice(t*2+1,t*2+3),16)})}this.transforms=this.style.map(function(a){var s,i,o=a[0],t=a[1],n=null;return t.parseColor("#zzzzzz")!="#zzzzzz"?(t=t.parseColor(),n="color"):o=="opacity"?(t=parseFloat(t),Prototype.Browser.IE&&!this.element.currentStyle.hasLayout&&this.element.setStyle({zoom:1})):Element.CSS_LENGTH.test(t)&&(s=t.match(/^([+-]?[0-9.]+)(.*)$/),t=parseFloat(s[1]),n=s.length==3?s[2]:null),i=this.element.getStyle(o),{style:o.camelize(),originalValue:n=="color"?e(i):parseFloat(i||0),targetValue:n=="color"?e(t):t,unit:n}}.bind(this)).reject(function(e){return e.originalValue==e.targetValue||e.unit!="color"&&(isNaN(e.originalValue)||isNaN(e.targetValue))})},update:function(t){for(var e,n={},s=this.transforms.length;s--;)n[(e=this.transforms[s]).style]=e.unit=="color"?"#"+Math.round(e.originalValue[0]+(e.targetValue[0]-e.originalValue[0])*t).toColorPart()+Math.round(e.originalValue[1]+(e.targetValue[1]-e.originalValue[1])*t).toColorPart()+Math.round(e.originalValue[2]+(e.targetValue[2]-e.originalValue[2])*t).toColorPart():(e.originalValue+(e.targetValue-e.originalValue)*t).toFixed(3)+(e.unit===null?'':e.unit);this.element.setStyle(n,!0)}}),Effect.Transform=Class.create({initialize:function(e){this.tracks=[],this.options=arguments[1]||{},this.addTracks(e)},addTracks:function(e){return e.each(function(e){e=$H(e);var t=e.values().first();this.tracks.push($H({ids:e.keys().first(),effect:Effect.Morph,options:{style:t}}))}.bind(this)),this},play:function(){return new Effect.Parallel(this.tracks.map(function(e){var t=e.get("ids"),n=e.get("effect"),s=e.get("options"),o=[$(t)||$$(t)].flatten();return o.map(function(e){return new n(e,Object.extend({sync:!0},s))})}).flatten(),this.options)}}),Element.CSS_PROPERTIES=$w("backgroundColor backgroundPosition borderBottomColor borderBottomStyle borderBottomWidth borderLeftColor borderLeftStyle borderLeftWidth borderRightColor borderRightStyle borderRightWidth borderSpacing borderTopColor borderTopStyle borderTopWidth bottom clip color fontSize fontWeight height left letterSpacing lineHeight marginBottom marginLeft marginRight marginTop markerOffset maxHeight maxWidth minHeight minWidth opacity outlineColor outlineOffset outlineWidth paddingBottom paddingLeft paddingRight paddingTop right textIndent top width wordSpacing zIndex"),Element.CSS_LENGTH=/^(([+-]?[0-9.]+)(em|ex|px|in|cm|mm|pt|pc|%))|0$/,String.__parseStyleElement=document.createElement("div"),String.prototype.parseStyle=function(){var e,t=$H();return Prototype.Browser.WebKit?e=new Element("div",{style:this}).style:(String.__parseStyleElement.innerHTML='<div style="'+this+'"></div>',e=String.__parseStyleElement.childNodes[0].style),Element.CSS_PROPERTIES.each(function(n){e[n]&&t.set(n,e[n])}),Prototype.Browser.IE&&this.include("opacity")&&t.set("opacity",this.match(/opacity:\s*((?:0|1)?(?:\.\d*)?)/)[1]),t},document.defaultView&&document.defaultView.getComputedStyle?Element.getStyles=function(e){var t=document.defaultView.getComputedStyle($(e),null);return Element.CSS_PROPERTIES.inject({},function(e,n){return e[n]=t[n],e})}:Element.getStyles=function(e){e=$(e);var n=e.currentStyle,t=Element.CSS_PROPERTIES.inject({},function(e,t){return e[t]=n[t],e});return t.opacity||(t.opacity=e.getOpacity()),t},Effect.Methods={morph:function(e,t){return e=$(e),new Effect.Morph(e,Object.extend({style:t},arguments[2]||{})),e},visualEffect:function(e,n,s){e=$(e);var t=n.dasherize().camelize(),o=t.charAt(0).toUpperCase()+t.substring(1);return new Effect[o](e,s),e},highlight:function(e,t){return e=$(e),new Effect.Highlight(e,t),e}},$w("fade appear grow shrink fold blindUp blindDown slideUp slideDown pulsate shake puff squish switchOff dropOut").each(function(e){Effect.Methods[e]=function(t,n){return t=$(t),Effect[e.charAt(0).toUpperCase()+e.substring(1)](t,n),t}}),$w("getInlineOpacity forceRerendering setContentZoom collectTextNodes collectTextNodesIgnoreClass getStyles").each(function(e){Effect.Methods[e]=Element[e]}),Element.addMethods(Effect.Methods),Object.isUndefined(Effect))throw"dragdrop.js requires including script.aculo.us' effects.js library";if(Droppables={drops:[],remove:function(e){this.drops=this.drops.reject(function(t){return t.element==$(e)})},add:function(t){t=$(t);var n,e=Object.extend({greedy:!0,hoverclass:null,tree:!1},arguments[1]||{});e.containment&&(e._containers=[],n=e.containment,Object.isArray(n)?n.each(function(t){e._containers.push($(t))}):e._containers.push($(n))),e.accept&&(e.accept=[e.accept].flatten()),Element.makePositioned(t),e.element=t,this.drops.push(e)},findDeepestChild:function(e){deepest=e[0];for(i=1;i<e.length;++i)Element.isParent(e[i].element,deepest.element)&&(deepest=e[i]);return deepest},isContained:function(t,n){var e;return n.tree?e=t.treeNode:e=t.parentNode,n._containers.detect(function(t){return e==t})},isAffected:function(n,t,e){return e.element!=t&&(!e._containers||this.isContained(t,e))&&(!e.accept||Element.classNames(t).detect(function(t){return e.accept.include(t)}))&&Position.within(e.element,n[0],n[1])},deactivate:function(e){e.hoverclass&&Element.removeClassName(e.element,e.hoverclass),this.last_active=null},activate:function(e){e.hoverclass&&Element.addClassName(e.element,e.hoverclass),this.last_active=e},show:function(t,s){if(!this.drops.length)return;var e,n=[];this.drops.each(function(e){Droppables.isAffected(t,s,e)&&n.push(e)}),n.length>0&&(e=Droppables.findDeepestChild(n)),this.last_active&&this.last_active!=e&&this.deactivate(this.last_active),e&&(Position.within(e.element,t[0],t[1]),e.onHover&&e.onHover(s,e.element,Position.overlap(e.overlap,e.element)),e!=this.last_active&&Droppables.activate(e))},fire:function(e,t){if(!this.last_active)return;if(Position.prepare(),this.isAffected([Event.pointerX(e),Event.pointerY(e)],t,this.last_active)&&this.last_active.onDrop)return this.last_active.onDrop(t,this.last_active.element,e),!0},reset:function(){this.last_active&&this.deactivate(this.last_active)}},Draggables={drags:[],observers:[],register:function(e){this.drags.length==0&&(this.eventMouseUp=this.endDrag.bindAsEventListener(this),this.eventMouseMove=this.updateDrag.bindAsEventListener(this),this.eventKeypress=this.keyPress.bindAsEventListener(this),Event.observe(document,"mouseup",this.eventMouseUp),Event.observe(document,"mousemove",this.eventMouseMove),Event.observe(document,"keypress",this.eventKeypress)),this.drags.push(e)},unregister:function(e){this.drags=this.drags.reject(function(t){return t==e}),this.drags.length==0&&(Event.stopObserving(document,"mouseup",this.eventMouseUp),Event.stopObserving(document,"mousemove",this.eventMouseMove),Event.stopObserving(document,"keypress",this.eventKeypress))},activate:function(e){e.options.delay?this._timeout=setTimeout(function(){Draggables._timeout=null,window.focus(),Draggables.activeDraggable=e}.bind(this),e.options.delay):(window.focus(),this.activeDraggable=e)},deactivate:function(){this.activeDraggable=null},updateDrag:function(e){if(!this.activeDraggable)return;var t=[Event.pointerX(e),Event.pointerY(e)];if(this._lastPointer&&this._lastPointer.inspect()==t.inspect())return;this._lastPointer=t,this.activeDraggable.updateDrag(e,t)},endDrag:function(e){if(this._timeout&&(clearTimeout(this._timeout),this._timeout=null),!this.activeDraggable)return;this._lastPointer=null,this.activeDraggable.endDrag(e),this.activeDraggable=null},keyPress:function(e){this.activeDraggable&&this.activeDraggable.keyPress(e)},addObserver:function(e){this.observers.push(e),this._cacheObserverCallbacks()},removeObserver:function(e){this.observers=this.observers.reject(function(t){return t.element==e}),this._cacheObserverCallbacks()},notify:function(e,t,n){this[e+"Count"]>0&&this.observers.each(function(s){s[e]&&s[e](e,t,n)}),t.options[e]&&t.options[e](t,n)},_cacheObserverCallbacks:function(){["onStart","onEnd","onDrag"].each(function(e){Draggables[e+"Count"]=Draggables.observers.select(function(t){return t[e]}).length})}},Draggable=Class.create({initialize:function(n){var e,t={handle:!1,reverteffect:function(n,e,t){var s=Math.sqrt(Math.abs(e^2)+Math.abs(t^2))*.02;new Effect.Move(n,{x:-t,y:-e,duration:s,queue:{scope:"_draggable",position:"end"}})},endeffect:function(e){var t=Object.isNumber(e._opacity)?e._opacity:1;new Effect.Opacity(e,{duration:.2,from:.7,to:t,queue:{scope:"_draggable",position:"end"},afterFinish:function(){Draggable._dragging[e]=!1}})},zindex:1e3,revert:!1,quiet:!1,scroll:!1,scrollSensitivity:20,scrollSpeed:15,snap:!1,delay:0};(!arguments[1]||Object.isUndefined(arguments[1].endeffect))&&Object.extend(t,{starteffect:function(e){e._opacity=Element.getOpacity(e),Draggable._dragging[e]=!0,new Effect.Opacity(e,{duration:.2,from:e._opacity,to:.7})}}),e=Object.extend(t,arguments[1]||{}),this.element=$(n),e.handle&&Object.isString(e.handle)&&(this.handle=this.element.down("."+e.handle,0)),this.handle||(this.handle=$(e.handle)),this.handle||(this.handle=this.element),e.scroll&&!e.scroll.scrollTo&&!e.scroll.outerHTML&&(e.scroll=$(e.scroll),this._isScrollChild=Element.childOf(this.element,e.scroll)),Element.makePositioned(this.element),this.options=e,this.dragging=!1,this.eventMouseDown=this.initDrag.bindAsEventListener(this),Event.observe(this.handle,"mousedown",this.eventMouseDown),Draggables.register(this)},destroy:function(){Event.stopObserving(this.handle,"mousedown",this.eventMouseDown),Draggables.unregister(this)},currentDelta:function(){return[parseInt(Element.getStyle(this.element,"left")||"0"),parseInt(Element.getStyle(this.element,"top")||"0")]},initDrag:function(e){if(!Object.isUndefined(Draggable._dragging[this.element])&&Draggable._dragging[this.element])return;if(Event.isLeftClick(e)){var t,n,s=Event.element(e);if((tag_name=s.tagName.toUpperCase())&&(tag_name=="INPUT"||tag_name=="SELECT"||tag_name=="OPTION"||tag_name=="BUTTON"||tag_name=="TEXTAREA"))return;t=[Event.pointerX(e),Event.pointerY(e)],n=this.element.cumulativeOffset(),this.offset=[0,1].map(function(e){return t[e]-n[e]}),Draggables.activate(this),Event.stop(e)}},startDrag:function(t){if(this.dragging=!0,this.delta||(this.delta=this.currentDelta()),this.options.zindex&&(this.originalZ=parseInt(Element.getStyle(this.element,"z-index")||0),this.element.style.zIndex=this.options.zindex),this.options.ghosting&&(this._clone=this.element.cloneNode(!0),this._originallyAbsolute=this.element.getStyle("position")=="absolute",this._originallyAbsolute||Position.absolutize(this.element),this.element.parentNode.insertBefore(this._clone,this.element)),this.options.scroll)if(this.options.scroll==window){var e=this._getWindowScroll(this.options.scroll);this.originalScrollLeft=e.left,this.originalScrollTop=e.top}else this.originalScrollLeft=this.options.scroll.scrollLeft,this.originalScrollTop=this.options.scroll.scrollTop;Draggables.notify("onStart",this,t),this.options.starteffect&&this.options.starteffect(this.element)},updateDrag:function(event,pointer){if(this.dragging||this.startDrag(event),this.options.quiet||(Position.prepare(),Droppables.show(pointer,this.element)),Draggables.notify("onDrag",this,event),this.draw(pointer),this.options.change&&this.options.change(this),this.options.scroll){if(this.stopScrolling(),this.options.scroll==window)with(this._getWindowScroll(this.options.scroll))p=[left,top,left+width,top+height];else p=Position.page(this.options.scroll).toArray(),p[0]+=this.options.scroll.scrollLeft+Position.deltaX,p[1]+=this.options.scroll.scrollTop+Position.deltaY,p.push(p[0]+this.options.scroll.offsetWidth),p.push(p[1]+this.options.scroll.offsetHeight);var p,speed=[0,0];pointer[0]<p[0]+this.options.scrollSensitivity&&(speed[0]=pointer[0]-(p[0]+this.options.scrollSensitivity)),pointer[1]<p[1]+this.options.scrollSensitivity&&(speed[1]=pointer[1]-(p[1]+this.options.scrollSensitivity)),pointer[0]>p[2]-this.options.scrollSensitivity&&(speed[0]=pointer[0]-(p[2]-this.options.scrollSensitivity)),pointer[1]>p[3]-this.options.scrollSensitivity&&(speed[1]=pointer[1]-(p[3]-this.options.scrollSensitivity)),this.startScrolling(speed)}Prototype.Browser.WebKit&&window.scrollBy(0,0),Event.stop(event)},finishDrag:function(n,o){if(this.dragging=!1,this.options.quiet){Position.prepare();var e,t,s,i=[Event.pointerX(n),Event.pointerY(n)];Droppables.show(i,this.element)}this.options.ghosting&&(this._originallyAbsolute||Position.relativize(this.element),delete this._originallyAbsolute,Element.remove(this._clone),this._clone=null),t=!1,o&&(t=Droppables.fire(n,this.element),t||(t=!1)),t&&this.options.onDropped&&this.options.onDropped(this.element),Draggables.notify("onEnd",this,n),e=this.options.revert,e&&Object.isFunction(e)&&(e=e(this.element)),s=this.currentDelta(),e&&this.options.reverteffect?(t==0||e!="failure")&&this.options.reverteffect(this.element,s[1]-this.delta[1],s[0]-this.delta[0]):this.delta=s,this.options.zindex&&(this.element.style.zIndex=this.originalZ),this.options.endeffect&&this.options.endeffect(this.element),Draggables.deactivate(this),Droppables.reset()},keyPress:function(e){if(e.keyCode!=Event.KEY_ESC)return;this.finishDrag(e,!1),Event.stop(e)},endDrag:function(e){if(!this.dragging)return;this.stopScrolling(),this.finishDrag(e,!0),Event.stop(e)},draw:function(i){var e,n,s,o,t=this.element.cumulativeOffset();this.options.ghosting&&(s=Position.realOffset(this.element),t[0]+=s[0]-Position.deltaX,t[1]+=s[1]-Position.deltaY),o=this.currentDelta(),t[0]-=o[0],t[1]-=o[1],this.options.scroll&&this.options.scroll!=window&&this._isScrollChild&&(t[0]-=this.options.scroll.scrollLeft-this.originalScrollLeft,t[1]-=this.options.scroll.scrollTop-this.originalScrollTop),e=[0,1].map(function(e){return i[e]-t[e]-this.offset[e]}.bind(this)),this.options.snap&&(Object.isFunction(this.options.snap)?e=this.options.snap(e[0],e[1],this):Object.isArray(this.options.snap)?e=e.map(function(t,e){return(t/this.options.snap[e]).round()*this.options.snap[e]}.bind(this)):e=e.map(function(e){return(e/this.options.snap).round()*this.options.snap}.bind(this))),n=this.element.style,(!this.options.constraint||this.options.constraint=="horizontal")&&(n.left=e[0]+"px"),(!this.options.constraint||this.options.constraint=="vertical")&&(n.top=e[1]+"px"),n.visibility=="hidden"&&(n.visibility="")},stopScrolling:function(){this.scrollInterval&&(clearInterval(this.scrollInterval),this.scrollInterval=null,Draggables._lastScrollPointer=null)},startScrolling:function(e){if(!e[0]&&!e[1])return;this.scrollSpeed=[e[0]*this.options.scrollSpeed,e[1]*this.options.scrollSpeed],this.lastScrolled=new Date,this.scrollInterval=setInterval(this.scroll.bind(this),10)},scroll:function(){var d,current=new Date,delta=current-this.lastScrolled;if(this.lastScrolled=current,this.options.scroll==window)with(this._getWindowScroll(this.options.scroll))(this.scrollSpeed[0]||this.scrollSpeed[1])&&(d=delta/1e3,this.options.scroll.scrollTo(left+d*this.scrollSpeed[0],top+d*this.scrollSpeed[1]));else this.options.scroll.scrollLeft+=this.scrollSpeed[0]*delta/1e3,this.options.scroll.scrollTop+=this.scrollSpeed[1]*delta/1e3;Position.prepare(),Droppables.show(Draggables._lastPointer,this.element),Draggables.notify("onDrag",this),this._isScrollChild&&(Draggables._lastScrollPointer=Draggables._lastScrollPointer||$A(Draggables._lastPointer),Draggables._lastScrollPointer[0]+=this.scrollSpeed[0]*delta/1e3,Draggables._lastScrollPointer[1]+=this.scrollSpeed[1]*delta/1e3,Draggables._lastScrollPointer[0]<0&&(Draggables._lastScrollPointer[0]=0),Draggables._lastScrollPointer[1]<0&&(Draggables._lastScrollPointer[1]=0),this.draw(Draggables._lastScrollPointer)),this.options.change&&this.options.change(this)},_getWindowScroll:function(w){var T,L,H,W;with(w.document)w.document.documentElement&&documentElement.scrollTop?(T=documentElement.scrollTop,L=documentElement.scrollLeft):w.document.body&&(T=body.scrollTop,L=body.scrollLeft),w.innerWidth?(W=w.innerWidth,H=w.innerHeight):w.document.documentElement&&documentElement.clientWidth?(W=documentElement.clientWidth,H=documentElement.clientHeight):(W=body.offsetWidth,H=body.offsetHeight);return{top:T,left:L,width:W,height:H}}}),Draggable._dragging={},SortableObserver=Class.create({initialize:function(e,t){this.element=$(e),this.observer=t,this.lastValue=Sortable.serialize(this.element)},onStart:function(){this.lastValue=Sortable.serialize(this.element)},onEnd:function(){Sortable.unmark(),this.lastValue!=Sortable.serialize(this.element)&&this.observer(this.element)}}),Sortable={SERIALIZE_RULE:/^[^_-](?:[A-Za-z0-9-_]*)[_](.*)$/,sortables:{},_findRootElement:function(e){for(;e.tagName.toUpperCase()!="BODY";){if(e.id&&Sortable.sortables[e.id])return e;e=e.parentNode}},options:function(e){if(e=Sortable._findRootElement($(e)),!e)return;return Sortable.sortables[e.id]},destroy:function(t){t=$(t);var e=Sortable.sortables[t.id];e&&(Draggables.removeObserver(e.element),e.droppables.each(function(e){Droppables.remove(e)}),e.draggables.invoke("destroy"),delete Sortable.sortables[e.element.id])},create:function(t){t=$(t);var n,s,o,e=Object.extend({element:t,tag:"li",dropOnEmpty:!1,tree:!1,treeTag:"ul",overlap:"vertical",constraint:"vertical",containment:t,handle:!1,only:!1,delay:0,hoverclass:null,ghosting:!1,quiet:!1,scroll:!1,scrollSensitivity:20,scrollSpeed:15,format:this.SERIALIZE_RULE,elements:!1,handles:!1,onChange:Prototype.emptyFunction,onUpdate:Prototype.emptyFunction},arguments[1]||{});this.destroy(t),n={revert:!0,quiet:e.quiet,scroll:e.scroll,scrollSpeed:e.scrollSpeed,scrollSensitivity:e.scrollSensitivity,delay:e.delay,ghosting:e.ghosting,constraint:e.constraint,handle:e.handle},e.starteffect&&(n.starteffect=e.starteffect),e.reverteffect?n.reverteffect=e.reverteffect:e.ghosting&&(n.reverteffect=function(e){e.style.top=0,e.style.left=0}),e.endeffect&&(n.endeffect=e.endeffect),e.zindex&&(n.zindex=e.zindex),o={overlap:e.overlap,containment:e.containment,tree:e.tree,hoverclass:e.hoverclass,onHover:Sortable.onHover},s={onHover:Sortable.onEmptyHover,overlap:e.overlap,containment:e.containment,hoverclass:e.hoverclass},Element.cleanWhitespace(t),e.draggables=[],e.droppables=[],(e.dropOnEmpty||e.tree)&&(Droppables.add(t,s),e.droppables.push(t)),(e.elements||this.findElements(t,e)||[]).each(function(s,i){var a=e.handles?$(e.handles[i]):e.handle?$(s).select("."+e.handle)[0]:s;e.draggables.push(new Draggable(s,Object.extend(n,{handle:a}))),Droppables.add(s,o),e.tree&&(s.treeNode=t),e.droppables.push(s)}),e.tree&&(Sortable.findTreeElements(t,e)||[]).each(function(n){Droppables.add(n,s),n.treeNode=t,e.droppables.push(n)}),this.sortables[t.identify()]=e,Draggables.addObserver(new SortableObserver(t,e.onUpdate))},findElements:function(t,e){return Element.findChildren(t,e.only,!!e.tree,e.tag)},findTreeElements:function(t,e){return Element.findChildren(t,e.only,!!e.tree,e.treeTag)},onHover:function(e,t,s){if(Element.isParent(t,e))return;if(s>.33&&s<.66&&Sortable.options(t).tree)return;if(s>.5){if(Sortable.mark(t,"before"),t.previousSibling!=e){var o,n=e.parentNode;e.style.visibility="hidden",t.parentNode.insertBefore(e,t),t.parentNode!=n&&Sortable.options(n).onChange(e),Sortable.options(t.parentNode).onChange(e)}}else Sortable.mark(t,"after"),o=t.nextSibling||null,o!=e&&(n=e.parentNode,e.style.visibility="hidden",t.parentNode.insertBefore(e,o),t.parentNode!=n&&Sortable.options(n).onChange(e),Sortable.options(t.parentNode).onChange(e))},onEmptyHover:function(o,s,c){var e,t,i,a,r=o.parentNode,n=Sortable.options(s);if(!Element.isParent(s,o)){if(e=Sortable.findElements(s,{tag:n.tag,only:n.only}),a=null,e){i=Element.offsetSize(s,n.overlap)*(1-c);for(t=0;t<e.length;t+=1)if(i-Element.offsetSize(e[t],n.overlap)>=0)i-=Element.offsetSize(e[t],n.overlap);else if(i-Element.offsetSize(e[t],n.overlap)/2>=0){a=t+1<e.length?e[t+1]:null;break}else{a=e[t];break}}s.insertBefore(o,a),Sortable.options(r).onChange(o),n.onChange(o)}},unmark:function(){Sortable._marker&&Sortable._marker.hide()},mark:function(t,s){var e,n=Sortable.options(t.parentNode);if(n&&!n.ghosting)return;Sortable._marker||(Sortable._marker=($("dropmarker")||Element.extend(document.createElement("DIV"))).hide().addClassName("dropmarker").setStyle({position:"absolute"}),document.getElementsByTagName("body").item(0).appendChild(Sortable._marker)),e=t.cumulativeOffset(),Sortable._marker.setStyle({left:e[0]+"px",top:e[1]+"px"}),s=="after"&&(n.overlap=="horizontal"?Sortable._marker.setStyle({left:e[0]+t.clientWidth+"px"}):Sortable._marker.setStyle({top:e[1]+t.clientHeight+"px"})),Sortable._marker.show()},_tree:function(a,t,n){for(var e,o,i=Sortable.findElements(a,t)||[],s=0;s<i.length;++s){if(o=i[s].id.match(t.format),!o)continue;e={id:encodeURIComponent(o?o[1]:null),element:a,parent:n,children:[],position:n.children.length,container:$(i[s]).down(t.treeTag)},e.container&&this._tree(e.container,t,e),n.children.push(e)}return n},tree:function(e){e=$(e);var t=this.options(e),n=Object.extend({tag:t.tag,treeTag:t.treeTag,only:t.only,name:e.id,format:t.format},arguments[1]||{}),s={id:null,parent:null,children:[],container:e,position:0};return Sortable._tree(e,n,s)},_constructIndex:function(e){var t='';do e.id&&(t="["+e.position+"]"+t);while((e=e.parent)!=null)return t},sequence:function(e){e=$(e);var t=Object.extend(this.options(e),arguments[1]||{});return $(this.findElements(e,t)||[]).map(function(e){return e.id.match(t.format)?e.id.match(t.format)[1]:''})},setSequence:function(e,s){e=$(e);var t=Object.extend(this.options(e),arguments[2]||{}),n={};this.findElements(e,t).each(function(e){e.id.match(t.format)&&(n[e.id.match(t.format)[1]]=[e,e.parentNode]),e.parentNode.removeChild(e)}),s.each(function(t){var e=n[t];e&&(e[1].appendChild(e[0]),delete n[t])})},serialize:function(e){e=$(e);var n=Object.extend(Sortable.options(e),arguments[1]||{}),t=encodeURIComponent(arguments[1]&&arguments[1].name?arguments[1].name:e.id);return n.tree?Sortable.tree(e,arguments[1]).children.map(function(e){return[t+Sortable._constructIndex(e)+"[id]="+encodeURIComponent(e.id)].concat(e.children.map(arguments.callee))}).flatten().join("&"):Sortable.sequence(e,arguments[1]).map(function(e){return t+"[]="+encodeURIComponent(e)}).join("&")}},Element.isParent=function(e,t){return!!e.parentNode&&e!=t&&(e.parentNode==t||Element.isParent(e.parentNode,t))},Element.findChildren=function(s,e,o,t){if(!s.hasChildNodes())return null;t=t.toUpperCase(),e&&(e=[e].flatten());var n=[];return $A(s.childNodes).each(function(s){if(s.tagName&&s.tagName.toUpperCase()==t&&(!e||Element.classNames(s).detect(function(t){return e.include(t)}))&&n.push(s),o){var i=Element.findChildren(s,e,o,t);i&&n.push(i)}}),n.length>0?n.flatten():[]},Element.offsetSize=function(t,e){return t["offset"+(e=="vertical"||e=="height"?"Height":"Width")]},typeof Effect=="undefined")throw"controls.js requires including script.aculo.us' effects.js library";if(Autocompleter={},Autocompleter.Base=Class.create({baseInitialize:function(e,n,t){e=$(e),this.element=e,this.update=$(n),this.hasFocus=!1,this.changed=!1,this.active=!1,this.index=0,this.entryCount=0,this.oldElementValue=this.element.value,this.setOptions?this.setOptions(t):this.options=t||{},this.options.paramName=this.options.paramName||this.element.name,this.options.tokens=this.options.tokens||[],this.options.frequency=this.options.frequency||.4,this.options.minChars=this.options.minChars||1,this.options.onShow=this.options.onShow||function(t,e){(!e.style.position||e.style.position=="absolute")&&(e.style.position="absolute",Position.clone(t,e,{setHeight:!1,offsetTop:t.offsetHeight})),Effect.Appear(e,{duration:.15})},this.options.onHide=this.options.onHide||function(t,e){new Effect.Fade(e,{duration:.15})},typeof this.options.tokens=="string"&&(this.options.tokens=new Array(this.options.tokens)),this.options.tokens.include("\n")||this.options.tokens.push("\n"),this.observer=null,this.element.setAttribute("autocomplete","off"),Element.hide(this.update),Event.observe(this.element,"blur",this.onBlur.bindAsEventListener(this)),Event.observe(this.element,"keydown",this.onKeyPress.bindAsEventListener(this))},show:function(){Element.getStyle(this.update,"display")=="none"&&this.options.onShow(this.element,this.update),!this.iefix&&Prototype.Browser.IE&&Element.getStyle(this.update,"position")=="absolute"&&(new Insertion.After(this.update,'<iframe id="'+this.update.id+'_iefix" '+'style="display:none;position:absolute;filter:progid:DXImageTransform.Microsoft.Alpha(opacity=0);" '+'src="javascript:false;" frameborder="0" scrolling="no"></iframe>'),this.iefix=$(this.update.id+"_iefix")),this.iefix&&setTimeout(this.fixIEOverlapping.bind(this),50)},fixIEOverlapping:function(){Position.clone(this.update,this.iefix,{setTop:!this.update.style.height}),this.iefix.style.zIndex=1,this.update.style.zIndex=2,Element.show(this.iefix)},hide:function(){this.stopIndicator(),Element.getStyle(this.update,"display")!="none"&&this.options.onHide(this.element,this.update),this.iefix&&Element.hide(this.iefix)},startIndicator:function(){this.options.indicator&&Element.show(this.options.indicator)},stopIndicator:function(){this.options.indicator&&Element.hide(this.options.indicator)},onKeyPress:function(e){if(this.active)switch(e.keyCode){case Event.KEY_TAB:case Event.KEY_RETURN:this.selectEntry(),Event.stop(e);case Event.KEY_ESC:this.hide(),this.active=!1,Event.stop(e);return;case Event.KEY_LEFT:case Event.KEY_RIGHT:return;case Event.KEY_UP:this.markPrevious(),this.render(),Event.stop(e);return;case Event.KEY_DOWN:this.markNext(),this.render(),Event.stop(e);return}else if(e.keyCode==Event.KEY_TAB||e.keyCode==Event.KEY_RETURN||Prototype.Browser.WebKit>0&&e.keyCode==0)return;this.changed=!0,this.hasFocus=!0,this.observer&&clearTimeout(this.observer),this.observer=setTimeout(this.onObserverEvent.bind(this),this.options.frequency*1e3)},activate:function(){this.changed=!1,this.hasFocus=!0,this.getUpdatedChoices()},onHover:function(e){var t=Event.findElement(e,"LI");this.index!=t.autocompleteIndex&&(this.index=t.autocompleteIndex,this.render()),Event.stop(e)},onClick:function(e){var t=Event.findElement(e,"LI");this.index=t.autocompleteIndex,this.selectEntry(),this.hide()},onBlur:function(){setTimeout(this.hide.bind(this),250),this.hasFocus=!1,this.active=!1},render:function(){if(this.entryCount>0){for(var e=0;e<this.entryCount;e++)this.index==e?Element.addClassName(this.getEntry(e),"selected"):Element.removeClassName(this.getEntry(e),"selected");this.hasFocus&&(this.show(),this.active=!0)}else this.active=!1,this.hide()},markPrevious:function(){this.index>0?this.index--:this.index=this.entryCount-1,this.getEntry(this.index).scrollIntoView(!0)},markNext:function(){this.index<this.entryCount-1?this.index++:this.index=0,this.getEntry(this.index).scrollIntoView(!1)},getEntry:function(e){return this.update.firstChild.childNodes[e]},getCurrentEntry:function(){return this.getEntry(this.index)},selectEntry:function(){this.active=!1,this.updateElement(this.getCurrentEntry())},updateElement:function(t){if(this.options.updateElement){this.options.updateElement(t);return}var e,s,o,i,n='';this.options.select?(s=$(t).select("."+this.options.select)||[],s.length>0&&(n=Element.collectTextNodes(s[0],this.options.select))):n=Element.collectTextNodesIgnoreClass(t,"informal"),e=this.getTokenBounds(),e[0]!=-1?(o=this.element.value.substr(0,e[0]),i=this.element.value.substr(e[0]).match(/^\s+/),i&&(o+=i[0]),this.element.value=o+n+this.element.value.substr(e[1])):this.element.value=n,this.oldElementValue=this.element.value,this.element.focus(),this.options.afterUpdateElement&&this.options.afterUpdateElement(this.element,t)},updateChoices:function(n){if(!this.changed&&this.hasFocus){if(this.update.innerHTML=n,Element.cleanWhitespace(this.update),Element.cleanWhitespace(this.update.down()),this.update.firstChild&&this.update.down().childNodes){this.entryCount=this.update.down().childNodes.length;for(var t,e=0;e<this.entryCount;e++)t=this.getEntry(e),t.autocompleteIndex=e,this.addObservers(t)}else this.entryCount=0;this.stopIndicator(),this.index=0,this.entryCount==1&&this.options.autoSelect?(this.selectEntry(),this.hide()):this.render()}},addObservers:function(e){Event.observe(e,"mouseover",this.onHover.bindAsEventListener(this)),Event.observe(e,"click",this.onClick.bindAsEventListener(this))},onObserverEvent:function(){this.changed=!1,this.tokenBounds=null,this.getToken().length>=this.options.minChars?this.getUpdatedChoices():(this.active=!1,this.hide()),this.oldElementValue=this.element.value},getToken:function(){var e=this.getTokenBounds();return this.element.value.substring(e[0],e[1]).strip()},getTokenBounds:function(){if(null!=this.tokenBounds)return this.tokenBounds;if(t=this.element.value,t.strip().empty())return[-1,0];for(var e,t,s=arguments.callee.getFirstDifferencePos(t,this.oldElementValue),a=s==this.oldElementValue.length?1:0,o=-1,i=t.length,n=0,r=this.options.tokens.length;n<r;++n)e=t.lastIndexOf(this.options.tokens[n],s+a-1),e>o&&(o=e),e=t.indexOf(this.options.tokens[n],s+a),-1!=e&&e<i&&(i=e);return this.tokenBounds=[o+1,i]}}),Autocompleter.Base.prototype.getTokenBounds.getFirstDifferencePos=function(t,n){for(var s=Math.min(t.length,n.length),e=0;e<s;++e)if(t[e]!=n[e])return e;return s},Ajax.Autocompleter=Class.create(Autocompleter.Base,{initialize:function(e,t,n,s){this.baseInitialize(e,t,s),this.options.asynchronous=!0,this.options.onComplete=this.onComplete.bind(this),this.options.defaultParams=this.options.parameters||null,this.url=n},getUpdatedChoices:function(){this.startIndicator();var e=encodeURIComponent(this.options.paramName)+"="+encodeURIComponent(this.getToken());this.options.parameters=this.options.callback?this.options.callback(this.element,e):e,this.options.defaultParams&&(this.options.parameters+="&"+this.options.defaultParams),new Ajax.Request(this.url,this.options)},onComplete:function(e){this.updateChoices(e.responseText)}}),Autocompleter.Local=Class.create(Autocompleter.Base,{initialize:function(e,t,n,s){this.baseInitialize(e,t,s),this.options.array=n},getUpdatedChoices:function(){this.updateChoices(this.options.selector(this))},setOptions:function(e){this.options=Object.extend({choices:10,partialSearch:!0,partialChars:2,ignoreCase:!0,fullSearch:!1,selector:function(s){for(var e,t,o=[],i=[],n=s.getToken(),r=0,a=0;a<s.options.array.length&&o.length<s.options.choices;a++)for(e=s.options.array[a],t=s.options.ignoreCase?e.toLowerCase().indexOf(n.toLowerCase()):e.indexOf(n);t!=-1;){if(t==0&&e.length!=n.length){o.push("<li><strong>"+e.substr(0,n.length)+"</strong>"+e.substr(n.length)+"</li>");break}if(n.length>=s.options.partialChars&&s.options.partialSearch&&t!=-1&&(s.options.fullSearch||/\s/.test(e.substr(t-1,1)))){i.push("<li>"+e.substr(0,t)+"<strong>"+e.substr(t,n.length)+"</strong>"+e.substr(t+n.length)+"</li>");break}t=s.options.ignoreCase?e.toLowerCase().indexOf(n.toLowerCase(),t+1):e.indexOf(n,t+1)}return i.length&&(o=o.concat(i.slice(0,s.options.choices-o.length))),"<ul>"+o.join('')+"</ul>"}},e||{})}}),Field.scrollFreeActivate=function(e){setTimeout(function(){Field.activate(e)},1)},Ajax.InPlaceEditor=Class.create({initialize:function(e,n,t){this.url=n,this.element=e=$(e),this.prepareOptions(),this._controls={},arguments.callee.dealWithDeprecatedOptions(t),Object.extend(this.options,t||{}),!this.options.formId&&this.element.id&&(this.options.formId=this.element.id+"-inplaceeditor",$(this.options.formId)&&(this.options.formId='')),this.options.externalControl&&(this.options.externalControl=$(this.options.externalControl)),this.options.externalControl||(this.options.externalControlOnly=!1),this._originalBackground=this.element.getStyle("background-color")||"transparent",this.element.title=this.options.clickToEditText,this._boundCancelHandler=this.handleFormCancellation.bind(this),this._boundComplete=(this.options.onComplete||Prototype.emptyFunction).bind(this),this._boundFailureHandler=this.handleAJAXFailure.bind(this),this._boundSubmitHandler=this.handleFormSubmission.bind(this),this._boundWrapperHandler=this.wrapUp.bind(this),this.registerListeners()},checkForEscapeOrReturn:function(e){if(!this._editing||e.ctrlKey||e.altKey||e.shiftKey)return;Event.KEY_ESC==e.keyCode?this.handleFormCancellation(e):Event.KEY_RETURN==e.keyCode&&this.handleFormSubmission(e)},createControl:function(e,a,s){var t,n,o=this.options[e+"Control"],i=this.options[e+"Text"];"button"==o?(n=document.createElement("input"),n.type="submit",n.value=i,n.className="editor_"+e+"_button","cancel"==e&&(n.onclick=this._boundCancelHandler),this._form.appendChild(n),this._controls[e]=n):"link"==o&&(t=document.createElement("a"),t.href="#",t.appendChild(document.createTextNode(i)),t.onclick="cancel"==e?this._boundCancelHandler:this._boundSubmitHandler,t.className="editor_"+e+"_link",s&&(t.className+=" "+s),this._form.appendChild(t),this._controls[e]=t)},createEditField:function(){var e,t,n=this.options.loadTextURL?this.options.loadingText:this.getText();1>=this.options.rows&&!/\r|\n/.test(this.getText())?(e=document.createElement("input"),e.type="text",t=this.options.size||this.options.cols||0,0<t&&(e.size=t)):(e=document.createElement("textarea"),e.rows=1>=this.options.rows?this.options.autoRows:this.options.rows,e.cols=this.options.cols||40),e.name=this.options.paramName,e.value=n,e.className="editor_field",this.options.submitOnBlur&&(e.onblur=this._boundSubmitHandler),this._controls.editor=e,this.options.loadTextURL&&this.loadExternalText(),this._form.appendChild(this._controls.editor)},createForm:function(){var t=this;function e(n,s){var e=t.options["text"+n+"Controls"];if(!e||s===!1)return;t._form.appendChild(document.createTextNode(e))}this._form=$(document.createElement("form")),this._form.id=this.options.formId,this._form.addClassName(this.options.formClassName),this._form.onsubmit=this._boundSubmitHandler,this.createEditField(),"textarea"==this._controls.editor.tagName.toLowerCase()&&this._form.appendChild(document.createElement("br")),this.options.onFormCustomization&&this.options.onFormCustomization(this,this._form),e("Before",this.options.okControl||this.options.cancelControl),this.createControl("ok",this._boundSubmitHandler),e("Between",this.options.okControl&&this.options.cancelControl),this.createControl("cancel",this._boundCancelHandler,"editor_cancel"),e("After",this.options.okControl||this.options.cancelControl)},destroy:function(){this._oldInnerHTML&&(this.element.innerHTML=this._oldInnerHTML),this.leaveEditMode(),this.unregisterListeners()},enterEditMode:function(e){if(this._saving||this._editing)return;this._editing=!0,this.triggerCallback("onEnterEditMode"),this.options.externalControl&&this.options.externalControl.hide(),this.element.hide(),this.createForm(),this.element.parentNode.insertBefore(this._form,this.element),this.options.loadTextURL||this.postProcessEditField(),e&&Event.stop(e)},enterHover:function(){if(this.options.hoverClassName&&this.element.addClassName(this.options.hoverClassName),this._saving)return;this.triggerCallback("onEnterHover")},getText:function(){return this.element.innerHTML.unescapeHTML()},handleAJAXFailure:function(e){this.triggerCallback("onFailure",e),this._oldInnerHTML&&(this.element.innerHTML=this._oldInnerHTML,this._oldInnerHTML=null)},handleFormCancellation:function(e){this.wrapUp(),e&&Event.stop(e)},handleFormSubmission:function(n){var e,t,s=this._form,o=$F(this._controls.editor);this.prepareSubmission(),e=this.options.callback(s,o)||'',Object.isString(e)&&(e=e.toQueryParams()),e.editorId=this.element.id,this.options.htmlResponse?(t=Object.extend({evalScripts:!0},this.options.ajaxOptions),Object.extend(t,{parameters:e,onComplete:this._boundWrapperHandler,onFailure:this._boundFailureHandler}),new Ajax.Updater({success:this.element},this.url,t)):(t=Object.extend({method:"get"},this.options.ajaxOptions),Object.extend(t,{parameters:e,onComplete:this._boundWrapperHandler,onFailure:this._boundFailureHandler}),new Ajax.Request(this.url,t)),n&&Event.stop(n)},leaveEditMode:function(){this.element.removeClassName(this.options.savingClassName),this.removeForm(),this.leaveHover(),this.element.style.backgroundColor=this._originalBackground,this.element.show(),this.options.externalControl&&this.options.externalControl.show(),this._saving=!1,this._editing=!1,this._oldInnerHTML=null,this.triggerCallback("onLeaveEditMode")},leaveHover:function(){if(this.options.hoverClassName&&this.element.removeClassName(this.options.hoverClassName),this._saving)return;this.triggerCallback("onLeaveHover")},loadExternalText:function(){this._form.addClassName(this.options.loadingClassName),this._controls.editor.disabled=!0;var e=Object.extend({method:"get"},this.options.ajaxOptions);Object.extend(e,{parameters:"editorId="+encodeURIComponent(this.element.id),onComplete:Prototype.emptyFunction,onSuccess:function(t){this._form.removeClassName(this.options.loadingClassName);var e=t.responseText;this.options.stripLoadedTextTags&&(e=e.stripTags()),this._controls.editor.value=e,this._controls.editor.disabled=!1,this.postProcessEditField()}.bind(this),onFailure:this._boundFailureHandler}),new Ajax.Request(this.options.loadTextURL,e)},postProcessEditField:function(){var e=this.options.fieldPostCreation;e&&$(this._controls.editor)["focus"==e?"focus":"activate"]()},prepareOptions:function(){this.options=Object.clone(Ajax.InPlaceEditor.DefaultOptions),Object.extend(this.options,Ajax.InPlaceEditor.DefaultCallbacks),[this._extraDefaultOptions].flatten().compact().each(function(e){Object.extend(this.options,e)}.bind(this))},prepareSubmission:function(){this._saving=!0,this.removeForm(),this.leaveHover(),this.showSaving()},registerListeners:function(){this._listeners={};var e;$H(Ajax.InPlaceEditor.Listeners).each(function(t){e=this[t.value].bind(this),this._listeners[t.key]=e,this.options.externalControlOnly||this.element.observe(t.key,e),this.options.externalControl&&this.options.externalControl.observe(t.key,e)}.bind(this))},removeForm:function(){if(!this._form)return;this._form.remove(),this._form=null,this._controls={}},showSaving:function(){this._oldInnerHTML=this.element.innerHTML,this.element.innerHTML=this.options.savingText,this.element.addClassName(this.options.savingClassName),this.element.style.backgroundColor=this._originalBackground,this.element.show()},triggerCallback:function(e,t){"function"==typeof this.options[e]&&this.options[e](this,t)},unregisterListeners:function(){$H(this._listeners).each(function(e){this.options.externalControlOnly||this.element.stopObserving(e.key,e.value),this.options.externalControl&&this.options.externalControl.stopObserving(e.key,e.value)}.bind(this))},wrapUp:function(e){this.leaveEditMode(),this._boundComplete(e,this.element)}}),Object.extend(Ajax.InPlaceEditor.prototype,{dispose:Ajax.InPlaceEditor.prototype.destroy}),Ajax.InPlaceCollectionEditor=Class.create(Ajax.InPlaceEditor,{initialize:function(e,t,n,s){this._extraDefaultOptions=Ajax.InPlaceCollectionEditor.DefaultOptions,e(t,n,s)},createEditField:function(){var e=document.createElement("select");e.name=this.options.paramName,e.size=1,this._controls.editor=e,this._collection=this.options.collection||[],this.options.loadCollectionURL?this.loadCollection():this.checkForExternalText(),this._form.appendChild(this._controls.editor)},loadCollection:function(){this._form.addClassName(this.options.loadingClassName),this.showLoadingText(this.options.loadingCollectionText);var e=Object.extend({method:"get"},this.options.ajaxOptions);Object.extend(e,{parameters:"editorId="+encodeURIComponent(this.element.id),onComplete:Prototype.emptyFunction,onSuccess:function(t){var e=t.responseText.strip();if(!/^\[.*\]$/.test(e))throw"Server returned an invalid collection representation.";this._collection=eval(e),this.checkForExternalText()}.bind(this),onFailure:this.onFailure}),new Ajax.Request(this.options.loadCollectionURL,e)},showLoadingText:function(t){this._controls.editor.disabled=!0;var e=this._controls.editor.firstChild;e||(e=document.createElement("option"),e.value='',this._controls.editor.appendChild(e),e.selected=!0),e.update((t||'').stripScripts().stripTags())},checkForExternalText:function(){this._text=this.getText(),this.options.loadTextURL?this.loadExternalText():this.buildOptionList()},loadExternalText:function(){this.showLoadingText(this.options.loadingText);var e=Object.extend({method:"get"},this.options.ajaxOptions);Object.extend(e,{parameters:"editorId="+encodeURIComponent(this.element.id),onComplete:Prototype.emptyFunction,onSuccess:function(e){this._text=e.responseText.strip(),this.buildOptionList()}.bind(this),onFailure:this.onFailure}),new Ajax.Request(this.options.loadTextURL,e)},buildOptionList:function(){this._form.removeClassName(this.options.loadingClassName),this._collection=this._collection.map(function(e){return 2===e.length?e:[e,e].flatten()});var e,t="value"in this.options?this.options.value:this._text,n=this._collection.any(function(e){return e[0]==t}.bind(this));this._controls.editor.update(''),this._collection.each(function(s,o){e=document.createElement("option"),e.value=s[0],e.selected=n?s[0]==t:0==o,e.appendChild(document.createTextNode(s[1])),this._controls.editor.appendChild(e)}.bind(this)),this._controls.editor.disabled=!1,Field.scrollFreeActivate(this._controls.editor)}}),Ajax.InPlaceEditor.prototype.initialize.dealWithDeprecatedOptions=function(e){if(!e)return;function t(t,n){if(t in e||n===void 0)return;e[t]=n}t("cancelControl",e.cancelLink?"link":e.cancelButton?"button":e.cancelLink==e.cancelButton!=!1&&void 0),t("okControl",e.okLink?"link":e.okButton?"button":e.okLink==e.okButton!=!1&&void 0),t("highlightColor",e.highlightcolor),t("highlightEndColor",e.highlightendcolor)},Object.extend(Ajax.InPlaceEditor,{DefaultOptions:{ajaxOptions:{},autoRows:3,cancelControl:"link",cancelText:"cancel",clickToEditText:"Click to edit",externalControl:null,externalControlOnly:!1,fieldPostCreation:"activate",formClassName:"inplaceeditor-form",formId:null,highlightColor:"#ffff99",highlightEndColor:"#ffffff",hoverClassName:'',htmlResponse:!0,loadingClassName:"inplaceeditor-loading",loadingText:"Loading...",okControl:"button",okText:"ok",paramName:"value",rows:1,savingClassName:"inplaceeditor-saving",savingText:"Saving...",size:0,stripLoadedTextTags:!1,submitOnBlur:!1,textAfterControls:'',textBeforeControls:'',textBetweenControls:''},DefaultCallbacks:{callback:function(e){return Form.serialize(e)},onComplete:function(t,e){new Effect.Highlight(e,{startcolor:this.options.highlightColor,keepBackgroundImage:!0})},onEnterEditMode:null,onEnterHover:function(e){e.element.style.backgroundColor=e.options.highlightColor,e._effect&&e._effect.cancel()},onFailure:function(e){alert("Error communication with the server: "+e.responseText.stripTags())},onFormCustomization:null,onLeaveEditMode:null,onLeaveHover:function(e){e._effect=new Effect.Highlight(e.element,{startcolor:e.options.highlightColor,endcolor:e.options.highlightEndColor,restorecolor:e._originalBackground,keepBackgroundImage:!0})}},Listeners:{click:"enterEditMode",keydown:"checkForEscapeOrReturn",mouseover:"enterHover",mouseout:"leaveHover"}}),Ajax.InPlaceCollectionEditor.DefaultOptions={loadingCollectionText:"Loading options..."},Form.Element.DelayedObserver=Class.create({initialize:function(e,t,n){this.delay=t||.5,this.element=$(e),this.callback=n,this.timer=null,this.lastValue=$F(this.element),Event.observe(this.element,"keyup",this.delayedListener.bindAsEventListener(this))},delayedListener:function(){if(this.lastValue==$F(this.element))return;this.timer&&clearTimeout(this.timer),this.timer=setTimeout(this.onTimerEvent.bind(this),this.delay*1e3),this.lastValue=$F(this.element)},onTimerEvent:function(){this.timer=null,this.callback(this.element,$F(this.element))}}),Control||(Control={}),Control.Slider=Class.create({initialize:function(t,n,s){var e=this;Object.isArray(t)?this.handles=t.collect(function(e){return $(e)}):this.handles=[$(t)],this.track=$(n),this.options=s||{},this.axis=this.options.axis||"horizontal",this.increment=this.options.increment||1,this.step=parseInt(this.options.step||"1"),this.range=this.options.range||$R(0,1),this.value=0,this.values=this.handles.map(function(){return 0}),this.spans=!!this.options.spans&&this.options.spans.map(function(e){return $(e)}),this.options.startSpan=$(this.options.startSpan||null),this.options.endSpan=$(this.options.endSpan||null),this.restricted=this.options.restricted||!1,this.maximum=this.options.maximum||this.range.end,this.minimum=this.options.minimum||this.range.start,this.alignX=parseInt(this.options.alignX||"0"),this.alignY=parseInt(this.options.alignY||"0"),this.trackLength=this.maximumOffset()-this.minimumOffset(),this.handleLength=this.isVertical()?this.handles[0].offsetHeight!=0?this.handles[0].offsetHeight:this.handles[0].style.height.replace(/px$/,""):this.handles[0].offsetWidth!=0?this.handles[0].offsetWidth:this.handles[0].style.width.replace(/px$/,""),this.active=!1,this.dragging=!1,this.disabled=!1,this.options.disabled&&this.setDisabled(),this.allowedValues=!!this.options.values&&this.options.values.sortBy(Prototype.K),this.allowedValues&&(this.minimum=this.allowedValues.min(),this.maximum=this.allowedValues.max()),this.eventMouseDown=this.startDrag.bindAsEventListener(this),this.eventMouseUp=this.endDrag.bindAsEventListener(this),this.eventMouseMove=this.update.bindAsEventListener(this),this.handles.each(function(n,t){t=e.handles.length-1-t,e.setValue(parseFloat((Object.isArray(e.options.sliderValue)?e.options.sliderValue[t]:e.options.sliderValue)||e.range.start),t),n.makePositioned().observe("mousedown",e.eventMouseDown)}),this.track.observe("mousedown",this.eventMouseDown),document.observe("mouseup",this.eventMouseUp),document.observe("mousemove",this.eventMouseMove),this.initialized=!0},dispose:function(){var e=this;Event.stopObserving(this.track,"mousedown",this.eventMouseDown),Event.stopObserving(document,"mouseup",this.eventMouseUp),Event.stopObserving(document,"mousemove",this.eventMouseMove),this.handles.each(function(t){Event.stopObserving(t,"mousedown",e.eventMouseDown)})},setDisabled:function(){this.disabled=!0},setEnabled:function(){this.disabled=!1},getNearestValue:function(e){if(this.allowedValues){if(e>=this.allowedValues.max())return this.allowedValues.max();if(e<=this.allowedValues.min())return this.allowedValues.min();var t=Math.abs(this.allowedValues[0]-e),n=this.allowedValues[0];return this.allowedValues.each(function(s){var o=Math.abs(s-e);o<=t&&(n=s,t=o)}),n}return e>this.range.end?this.range.end:e<this.range.start?this.range.start:e},setValue:function(t,e){this.active||(this.activeHandleIdx=e||0,this.activeHandle=this.handles[this.activeHandleIdx],this.updateStyles()),e=e||this.activeHandleIdx||0,this.initialized&&this.restricted&&(e>0&&t<this.values[e-1]&&(t=this.values[e-1]),e<this.handles.length-1&&t>this.values[e+1]&&(t=this.values[e+1])),t=this.getNearestValue(t),this.values[e]=t,this.value=this.values[0],this.handles[e].style[this.isVertical()?"top":"left"]=this.translateToPx(t),this.drawSpans(),(!this.dragging||!this.event)&&this.updateFinished()},setValueBy:function(t,e){this.setValue(this.values[e||this.activeHandleIdx||0]+t,e||this.activeHandleIdx||0)},translateToPx:function(e){return Math.round((this.trackLength-this.handleLength)/(this.range.end-this.range.start)*(e-this.range.start))+"px"},translateToValue:function(e){return e/(this.trackLength-this.handleLength)*(this.range.end-this.range.start)+this.range.start},getRange:function(e){var t=this.values.sortBy(Prototype.K);return e=e||0,$R(t[e],t[e+1])},minimumOffset:function(){return this.isVertical()?this.alignY:this.alignX},maximumOffset:function(){return this.isVertical()?(this.track.offsetHeight!=0?this.track.offsetHeight:this.track.style.height.replace(/px$/,""))-this.alignY:(this.track.offsetWidth!=0?this.track.offsetWidth:this.track.style.width.replace(/px$/,""))-this.alignX},isVertical:function(){return this.axis=="vertical"},drawSpans:function(){var e=this;this.spans&&$R(0,this.spans.length-1).each(function(t){e.setSpan(e.spans[t],e.getRange(t))}),this.options.startSpan&&this.setSpan(this.options.startSpan,$R(0,this.values.length>1?this.getRange(0).min():this.value)),this.options.endSpan&&this.setSpan(this.options.endSpan,$R(this.values.length>1?this.getRange(this.spans.length-1).max():this.value,this.maximum))},setSpan:function(t,e){this.isVertical()?(t.style.top=this.translateToPx(e.start),t.style.height=this.translateToPx(e.end-e.start+this.range.start)):(t.style.left=this.translateToPx(e.start),t.style.width=this.translateToPx(e.end-e.start+this.range.start))},updateStyles:function(){this.handles.each(function(e){Element.removeClassName(e,"selected")}),Element.addClassName(this.activeHandle,"selected")},startDrag:function(n){if(Event.isLeftClick(n)){if(!this.disabled){this.active=!0;var e,t=Event.element(n),s=[Event.pointerX(n),Event.pointerY(n)],o=t;if(o==this.track)e=this.track.cumulativeOffset(),this.event=n,this.setValue(this.translateToValue((this.isVertical()?s[1]-e[1]:s[0]-e[0])-this.handleLength/2)),e=this.activeHandle.cumulativeOffset(),this.offsetX=s[0]-e[0],this.offsetY=s[1]-e[1];else{for(;this.handles.indexOf(t)==-1&&t.parentNode;)t=t.parentNode;this.handles.indexOf(t)!=-1&&(this.activeHandle=t,this.activeHandleIdx=this.handles.indexOf(this.activeHandle),this.updateStyles(),e=this.activeHandle.cumulativeOffset(),this.offsetX=s[0]-e[0],this.offsetY=s[1]-e[1])}}Event.stop(n)}},update:function(e){this.active&&(this.dragging||(this.dragging=!0),this.draw(e),Prototype.Browser.WebKit&&window.scrollBy(0,0),Event.stop(e))},draw:function(t){var e=[Event.pointerX(t),Event.pointerY(t)],n=this.track.cumulativeOffset();e[0]-=this.offsetX+n[0],e[1]-=this.offsetY+n[1],this.event=t,this.setValue(this.translateToValue(this.isVertical()?e[1]:e[0])),this.initialized&&this.options.onSlide&&this.options.onSlide(this.values.length>1?this.values:this.value,this)},endDrag:function(e){this.active&&this.dragging&&(this.finishDrag(e,!0),Event.stop(e)),this.active=!1,this.dragging=!1},finishDrag:function(){this.active=!1,this.dragging=!1,this.updateFinished()},updateFinished:function(){this.initialized&&this.options.onChange&&this.options.onChange(this.values.length>1?this.values:this.value,this),this.event=null}}),Prototip={Version:"2.2.2"},Object.extend(Prototip,{REQUIRED_Prototype:"1.7",support:{canvas:!!document.createElement("canvas").getContext},insertScript:function(e){try{document.write("<script type='text/javascript' src='"+e+"'><\/script>")}catch(t){$$("head")[0].insert(new Element("script",{src:e,type:"text/javascript"}))}},start:function(){this.require("Prototype");var e=/prototip([\w\d-_.]+)?\.js(.*)/;this.path=(($$("script[src]").find(function(t){return t.src.match(e)})||{}).src||"").replace(e,""),Tips.paths=function(e){return{images:/^(https?:\/\/|\/)/.test(e.images)?e.images:this.path+e.images,javascript:/^(https?:\/\/|\/)/.test(e.javascript)?e.javascript:this.path+e.javascript}}.bind(this)(Tips.options.paths),Prototip.Styles||this.insertScript(Tips.paths.javascript+"styles.js"),this.support.canvas||(document.documentMode>=8&&!document.namespaces.ns_vml?document.namespaces.add("ns_vml","urn:schemas-microsoft-com:vml","#default#VML"):document.observe("dom:loaded",function(){var e=document.createStyleSheet();e.cssText="ns_vml\\:*{behavior:url(#default#VML)}"})),Tips.initialize(),Element.observe(window,"unload",this.unload)},require:function(e){if(typeof window[e]=="undefined"||this.convertVersionString(window[e].Version)<this.convertVersionString(this["REQUIRED_"+e]))throw"Prototip requires "+e+" >= "+this["REQUIRED_"+e]},convertVersionString:function(t){var e=t.replace(/_.*|\./g,""),e=parseInt(e+"0".times(4-e.length));return t.indexOf("_")>-1?e-1:e},toggleInt:function(e){return e>0?-1*e:e.abs()},unload:function(){Tips.removeAll()}}),Object.extend(Tips,function(){function e(e){if(!e)return;e.deactivate(),e.tooltip&&(e.wrapper.remove(),Tips.fixIE&&e.iframeShim.remove()),Tips.tips=Tips.tips.without(e)}return{tips:[],visible:[],initialize:function(){this.zIndexTop=this.zIndex},_inverse:{left:"right",right:"left",top:"bottom",bottom:"top",middle:"middle",horizontal:"vertical",vertical:"horizontal"},_stemTranslation:{width:"horizontal",height:"vertical"},inverseStem:function(e){return!!arguments[1]?this._inverse[e]:e},fixIE:function(t){var e=new RegExp("MSIE ([\\d.]+)").exec(t);return!!e&&parseFloat(e[1])<7}(navigator.userAgent),WebKit419:Prototype.Browser.WebKit&&!document.evaluate,add:function(e){this.tips.push(e)},remove:function(a){s=[];for(var n,s,o,t=0,i=this.tips.length;t<i;t++)n=this.tips[t],!o&&n.element==$(a)?o=n:n.element.parentNode||s.push(n);e(o);for(t=0,i=s.length;t<i;t++)n=s[t],e(n);a.prototip=null},removeAll:function(){for(var t=0,n=this.tips.length;t<n;t++)e(this.tips[t])},raise:function(e){if(e==this._highest)return;if(this.visible.length===0){this.zIndexTop=this.options.zIndex;for(var t=0,n=this.tips.length;t<n;t++)this.tips[t].wrapper.setStyle({zIndex:this.options.zIndex})}e.wrapper.setStyle({zIndex:this.zIndexTop++}),e.loader&&e.loader.setStyle({zIndex:this.zIndexTop}),this._highest=e},addVisibile:function(e){this.removeVisible(e),this.visible.push(e)},removeVisible:function(e){this.visible=this.visible.without(e)},hideAll:function(){Tips.visible.invoke("hide")},isChrome:function(e){return e||(e=32),navigator.userAgent.toLowerCase().indexOf("chrome/"+e)>-1},hook:function(a,i){a=$(a),i=$(i),o=Object.extend({offset:{x:0,y:0},position:!1},arguments[2]||{}),s=o.mouse||i.cumulativeOffset(),s.left+=o.offset.x,s.top+=o.offset.y;var e,t,n,s,o,c,d=o.mouse?[0,0]:i.cumulativeScrollOffset(),l=document.viewport.getScrollOffsets(),r=o.mouse?"mouseHook":"target";s.left+=-1*(d[0]-l[0]),this.isChrome("32")||(s.top+=-1*(d[1]-l[1])),o.mouse&&(c=[0,0],c.width=0,c.height=0),n={element:a.getDimensions()},t={element:Object.clone(s)},n[r]=o.mouse?c:i.getDimensions(),t[r]=Object.clone(s);for(e in t)switch(o[e]){case"topRight":case"rightTop":t[e].left+=n[e].width;break;case"topMiddle":t[e].left+=n[e].width/2;break;case"rightMiddle":t[e].left+=n[e].width,t[e].top+=n[e].height/2;break;case"bottomLeft":case"leftBottom":t[e].top+=n[e].height;break;case"bottomRight":case"rightBottom":t[e].left+=n[e].width,t[e].top+=n[e].height;break;case"bottomMiddle":t[e].left+=n[e].width/2,t[e].top+=n[e].height;break;case"leftMiddle":t[e].top+=n[e].height/2;break}return s.left+=-1*(t.element.left-t[r].left),s.top+=-1*(t.element.top-t[r].top),o.position&&a.setStyle({left:s.left+"px",top:s.top+"px"}),s}}}()),Tips.initialize(),Tip=Class.create({initialize:function(o,t){if(this.element=$(o),!this.element){throw"Prototip: Element not available, cannot create a tooltip.";return}Tips.remove(this.element);var s,n=Object.isString(t)||Object.isElement(t),e=n?arguments[2]||[]:t;this.content=n?t:null,e.style&&(e=Object.extend(Object.clone(Prototip.Styles[e.style]),e)),this.options=Object.extend(Object.extend({ajax:!1,border:0,borderColor:"#000000",radius:0,className:Tips.options.className,closeButton:Tips.options.closeButtons,delay:!(e.showOn&&e.showOn=="click")&&.14,hideAfter:!1,hideOn:"mouseleave",hideOthers:!1,hook:e.hook,offset:e.hook?{x:0,y:0}:{x:16,y:16},fixed:!!(e.hook&&!e.hook.mouse),showOn:"mousemove",stem:!1,style:"default",target:this.element,title:!1,viewport:!e.hook||!!e.hook.mouse,width:!1},Prototip.Styles.default),e),this.target=$(this.options.target),this.radius=this.options.radius,this.border=this.radius>this.options.border?this.radius:this.options.border,this.options.images?this.images=this.options.images.include("://")?this.options.images:Tips.paths.images+this.options.images:this.images=Tips.paths.images+"styles/"+(this.options.style||"")+"/",this.images.endsWith("/")||(this.images+="/"),Object.isString(this.options.stem)&&(this.options.stem={position:this.options.stem}),this.options.stem.position&&(this.options.stem=Object.extend(Object.clone(Prototip.Styles[this.options.style].stem)||{},this.options.stem),this.options.stem.position=[this.options.stem.position.match(/[a-z]+/)[0].toLowerCase(),this.options.stem.position.match(/[A-Z][a-z]+/)[0].toLowerCase()],this.options.stem.orientation=["left","right"].member(this.options.stem.position[0])?"horizontal":"vertical",this.stemInverse={horizontal:!1,vertical:!1}),this.options.ajax&&(this.options.ajax.options=Object.extend({onComplete:Prototype.emptyFunction},this.options.ajax.options||{})),this.options.hook.mouse&&(s=this.options.hook.tip.match(/[a-z]+/)[0].toLowerCase(),this.mouseHook=Tips._inverse[s]+Tips._inverse[this.options.hook.tip.match(/[A-Z][a-z]+/)[0].toLowerCase()].capitalize()),this.fixSafari2=Tips.WebKit419&&this.radius,this.setup(),Tips.add(this),this.activate(),Prototip.extend(this)},setup:function(){this.wrapper=new Element("div",{className:"prototip"}).setStyle({zIndex:Tips.options.zIndex}),this.fixSafari2&&(this.wrapper.hide=function(){return this.setStyle("left:-9500px;top:-9500px;visibility:hidden;"),this},this.wrapper.show=function(){return this.setStyle("visibility:visible"),this},this.wrapper.visible=function(){return this.getStyle("visibility")=="visible"&&parseFloat(this.getStyle("top").replace("px",""))>-9500}),this.wrapper.hide(),Tips.fixIE&&(this.iframeShim=new Element("iframe",{className:"iframeShim",src:"javascript:false;",frameBorder:0}).setStyle({display:"none",zIndex:Tips.options.zIndex-1,opacity:0})),this.options.ajax&&(this.showDelayed=this.showDelayed.wrap(this.ajaxShow)),this.tip=new Element("div",{className:"content"}),this.title=new Element("div",{className:"title"}).hide(),(this.options.closeButton||this.options.hideOn.element&&this.options.hideOn.element=="closeButton")&&(this.closeButton=new Element("div",{className:"close"}).setPngBackground(this.images+"close.png"))},build:function(){if(document.loaded)return this._build(),this._isBuilding=!0,!0;if(!this._isBuilding)return document.observe("dom:loaded",this._build),!1},_build:function(){$(document.body).insert(this.wrapper),Tips.fixIE&&$(document.body).insert(this.iframeShim),this.options.ajax&&$(document.body).insert(this.loader=new Element("div",{className:"prototipLoader"}).setPngBackground(this.images+"loader.gif").hide());var e,n,s,o,i,a,t="wrapper";this.options.stem.position&&(this.stem=new Element("div",{className:"prototip_Stem"}).setStyle({height:this.options.stem[this.options.stem.orientation=="vertical"?"height":"width"]+"px"}),o=this.options.stem.orientation=="horizontal",this[t].insert(this.stemWrapper=new Element("div",{className:"prototip_StemWrapper clearfix"}).insert(this.stemBox=new Element("div",{className:"prototip_StemBox clearfix"}))),this.stem.insert(this.stemImage=new Element("div",{className:"prototip_StemImage"}).setStyle({height:this.options.stem[o?"width":"height"]+"px",width:this.options.stem[o?"height":"width"]+"px"})),Tips.fixIE&&!this.options.stem.position[1].toUpperCase().include("MIDDLE")&&this.stemImage.setStyle({display:"inline"}),t="stemBox"),this.border&&(e=this.border,this[t].insert(this.borderFrame=new Element("ul",{className:"borderFrame"}).insert(this.borderTop=new Element("li",{className:"borderTop borderRow"}).setStyle("height: "+e+"px").insert(new Element("div",{className:"prototip_CornerWrapper prototip_CornerWrapperTopLeft"}).insert(new Element("div",{className:"prototip_Corner"}))).insert(a=new Element("div",{className:"prototip_BetweenCorners"}).setStyle({height:e+"px"}).insert(new Element("div",{className:"prototip_Between"}).setStyle({margin:"0 "+e+"px",height:e+"px"}))).insert(new Element("div",{className:"prototip_CornerWrapper prototip_CornerWrapperTopRight"}).insert(new Element("div",{className:"prototip_Corner"})))).insert(this.borderMiddle=new Element("li",{className:"borderMiddle borderRow"}).insert(this.borderCenter=new Element("div",{className:"borderCenter"}).setStyle("padding: 0 "+e+"px"))).insert(this.borderBottom=new Element("li",{className:"borderBottom borderRow"}).setStyle("height: "+e+"px").insert(new Element("div",{className:"prototip_CornerWrapper prototip_CornerWrapperBottomLeft"}).insert(new Element("div",{className:"prototip_Corner"}))).insert(a.cloneNode(!0)).insert(new Element("div",{className:"prototip_CornerWrapper prototip_CornerWrapperBottomRight"}).insert(new Element("div",{className:"prototip_Corner"}))))),t="borderCenter",n=this.borderFrame.select(".prototip_Corner"),$w("tl tr bl br").each(function(s,t){this.radius>0?Prototip.createCorner(n[t],s,{backgroundColor:this.options.borderColor,border:e,radius:this.options.radius}):n[t].addClassName("prototip_Fill"),n[t].setStyle({width:e+"px",height:e+"px"}).addClassName("prototip_Corner"+s.capitalize())}.bind(this)),this.borderFrame.select(".prototip_Between",".borderMiddle",".prototip_Fill").invoke("setStyle",{backgroundColor:this.options.borderColor})),this[t].insert(this.tooltip=new Element("div",{className:"tooltip "+this.options.className}).insert(this.toolbar=new Element("div",{className:"toolbar"}).insert(this.title))),this.options.width&&(s=this.options.width,Object.isNumber(s)&&(s+="px"),this.tooltip.setStyle("width:"+s)),this.stem&&(i={},i[this.options.stem.orientation=="horizontal"?"top":"bottom"]=this.stem,this.wrapper.insert(i),this.positionStem()),this.tooltip.insert(this.tip),this.options.ajax||this._update({title:this.options.title,content:this.content})},_update:function(e){s=this.wrapper.getStyle("visibility"),this.wrapper.setStyle("height:auto;width:auto;visibility:hidden").show(),this.border&&(this.borderTop.setStyle("height:0"),this.borderTop.setStyle("height:0")),e.title?(this.title.show().update(e.title),this.toolbar.show()):this.closeButton||(this.title.hide(),this.toolbar.hide()),Object.isElement(e.content)&&e.content.show(),(Object.isString(e.content)||Object.isElement(e.content))&&this.tip.update(e.content),this.tooltip.setStyle({width:this.tooltip.getWidth()+"px"}),this.wrapper.setStyle("visibility:visible").show(),this.tooltip.show();var s,o=this.tooltip.getDimensions(),t={width:o.width+"px"},n=[this.wrapper];Tips.fixIE&&n.push(this.iframeShim),this.closeButton&&(this.title.show().insert({top:this.closeButton}),this.toolbar.show()),(e.title||this.closeButton)&&this.toolbar.setStyle("width: 100%"),t.height=null,this.wrapper.setStyle({visibility:s}),this.tip.addClassName("clearfix"),(e.title||this.closeButton)&&this.title.addClassName("clearfix"),this.border&&(this.borderTop.setStyle("height:"+this.border+"px"),this.borderTop.setStyle("height:"+this.border+"px"),t="width: "+(o.width+2*this.border)+"px",n.push(this.borderFrame)),n.invoke("setStyle",t),this.stem&&(this.positionStem(),this.options.stem.orientation=="horizontal"&&this.wrapper.setStyle({width:this.wrapper.getWidth()+this.options.stem.height+"px"})),this.wrapper.hide()},activate:function(){this.eventShow=this.showDelayed.bindAsEventListener(this),this.eventHide=this.hide.bindAsEventListener(this),this.options.fixed&&this.options.showOn=="mousemove"&&(this.options.showOn="mouseover"),this.options.showOn&&this.options.showOn==this.options.hideOn&&(this.eventToggle=this.toggle.bindAsEventListener(this),this.element.observe(this.options.showOn,this.eventToggle)),this.closeButton&&this.closeButton.observe("mouseover",function(e){e.setPngBackground(this.images+"close_hover.png")}.bind(this,this.closeButton)).observe("mouseout",function(e){e.setPngBackground(this.images+"close.png")}.bind(this,this.closeButton));var t,n={element:this.eventToggle?[]:[this.element],target:this.eventToggle?[]:[this.target],tip:this.eventToggle?[]:[this.wrapper],closeButton:[],none:[]},e=this.options.hideOn.element;this.hideElement=e||(this.options.hideOn?"element":"none"),this.hideTargets=n[this.hideElement],!this.hideTargets&&e&&Object.isString(e)&&(this.hideTargets=this.tip.select(e)),$w("show hide").each(function(t){var n=t.capitalize(),e=this.options[t+"On"].event||this.options[t+"On"];e=="mouseover"?e=="mouseenter":e=="mouseout"&&e=="mouseleave",this[t+"Action"]=e}.bind(this)),!this.eventToggle&&this.options.showOn&&this.element.observe(this.options.showOn,this.eventShow),this.hideTargets&&this.options.hideOn&&this.hideTargets.invoke("observe",this.hideAction,this.eventHide),!this.options.fixed&&this.options.showOn=="click"&&(this.eventPosition=this.position.bindAsEventListener(this),this.element.observe("mousemove",this.eventPosition)),this.buttonEvent=this.hide.wrap(function(n,e){var t=e.findElement(".close");t&&(t.blur(),e.stop(),n(e))}).bindAsEventListener(this),(this.closeButton||this.options.hideOn&&this.options.hideOn.element==".close")&&this.wrapper.observe("click",this.buttonEvent),this.options.showOn!="click"&&this.hideElement!="element"&&(this.eventCheckDelay=function(){this.clearTimer("show")}.bindAsEventListener(this),this.element.observe("mouseleave",this.eventCheckDelay)),(this.options.hideOn||this.options.hideAfter)&&(t=[this.element,this.wrapper],this.activityEnter=function(){Tips.raise(this),this.cancelHideAfter()}.bindAsEventListener(this),this.activityLeave=this.hideAfter.bindAsEventListener(this),t.invoke("observe","mouseenter",this.activityEnter).invoke("observe","mouseleave",this.activityLeave)),this.options.ajax&&this.options.showOn!="click"&&(this.ajaxHideEvent=this.ajaxHide.bindAsEventListener(this),this.element.observe("mouseleave",this.ajaxHideEvent))},deactivate:function(){this.options.showOn&&this.options.showOn==this.options.hideOn?this.element.stopObserving(this.options.showOn,this.eventToggle):(this.options.showOn&&this.element.stopObserving(this.options.showOn,this.eventShow),this.hideTargets&&this.options.hideOn&&this.hideTargets.invoke("stopObserving")),this.eventPosition&&this.element.stopObserving("mousemove",this.eventPosition),this.eventCheckDelay&&this.element.stopObserving("mouseout",this.eventCheckDelay),this.wrapper.stopObserving(),(this.options.hideOn||this.options.hideAfter)&&this.element.stopObserving("mouseenter",this.activityEnter).stopObserving("mouseleave",this.activityLeave),this.ajaxHideEvent&&this.element.stopObserving("mouseleave",this.ajaxHideEvent)},ajaxShow:function(o,e){if(!this.tooltip&&!this.build())return;if(this.position(e),this.ajaxContentLoading)return;if(this.ajaxContentLoaded){o(e);return}if(this.ajaxContentLoading=!0,n={fakePointer:{pointerX:0,pointerY:0}},e.pointer)var t,s=e.pointer(),n={fakePointer:{pointerX:s.x,pointerY:s.y}};else e.fakePointer&&(n.fakePointer=e.fakePointer);return t=Object.clone(this.options.ajax.options),t.onComplete=t.onComplete.wrap(function(t,e){this._update({title:this.options.title,content:e.responseText}),this.position(n),function(){t(e);var n=this.loader&&this.loader.visible();this.loader&&(this.clearTimer("loader"),this.loader.remove(),this.loader=null),n&&this.show(),this.ajaxContentLoaded=!0,this.ajaxContentLoading=null}.bind(this).delay(.6)}.bind(this)),this.loaderTimer=Element.show.delay(this.options.delay,this.loader),this.wrapper.hide(),this.ajaxContentLoading=!0,this.loader.show(),this.ajaxTimer=function(){new Ajax.Request(this.options.ajax.url,t)}.bind(this).delay(this.options.delay),!1},ajaxHide:function(){this.clearTimer("loader")},showDelayed:function(e){if(!this.tooltip&&!this.build())return;if(this.position(e),this.wrapper.visible())return;this.clearTimer("show"),this.showTimer=this.show.bind(this).delay(this.options.delay)},clearTimer:function(e){this[e+"Timer"]&&clearTimeout(this[e+"Timer"])},show:function(){if(this.wrapper.visible())return;Tips.fixIE&&this.iframeShim.show(),this.options.hideOthers&&Tips.hideAll(),Tips.addVisibile(this),this.tooltip.show(),this.wrapper.show(),this.stem&&this.stem.show(),this.element.fire("prototip:shown")},hideAfter:function(){if(this.options.ajax&&this.loader&&this.options.showOn!="click"&&this.loader.hide(),!this.options.hideAfter)return;this.cancelHideAfter(),this.hideAfterTimer=this.hide.bind(this).delay(this.options.hideAfter)},cancelHideAfter:function(){this.options.hideAfter&&this.clearTimer("hideAfter")},hide:function(){if(this.clearTimer("show"),this.clearTimer("loader"),!this.wrapper.visible())return;this.afterHide()},afterHide:function(){Tips.fixIE&&this.iframeShim.hide(),this.loader&&this.loader.hide(),this.wrapper.hide(),(this.borderFrame||this.tooltip).show(),Tips.removeVisible(this),this.element.fire("prototip:hidden")},toggle:function(e){this.wrapper&&this.wrapper.visible()?this.hide(e):this.showDelayed(e)},positionStem:function(){var i,e=this.options.stem,s=arguments[0]||this.stemInverse,n=Tips.inverseStem(e.position[0],s[e.orientation]),t=Tips.inverseStem(e.position[1],s[Tips._inverse[e.orientation]]),o=this.radius||0;this.stemImage.setPngBackground(this.images+n+t+".png"),e.orientation=="horizontal"?(i=n=="left"?e.height:0,this.stemWrapper.setStyle("left: "+i+"px;"),this.stemImage.setStyle({float:n}),this.stem.setStyle({left:0,top:t=="bottom"?"100%":t=="middle"?"50%":0,marginTop:(t=="bottom"?-1*e.width:t=="middle"?-.5*e.width:0)+(t=="bottom"?-1*o:t=="top"?o:0)+"px"})):(this.stemWrapper.setStyle(n=="top"?"margin: 0; padding: "+e.height+"px 0 0 0;":"padding: 0; margin: 0 0 "+e.height+"px 0;"),this.stem.setStyle(n=="top"?"top: 0; bottom: auto;":"top: auto; bottom: 0;"),this.stemImage.setStyle({margin:0,float:t!="middle"?t:"none"}),t=="middle"?this.stemImage.setStyle("margin: 0 auto;"):this.stemImage.setStyle("margin-"+t+": "+o+"px;"),Tips.WebKit419&&(n=="bottom"?(this.stem.setStyle({position:"relative",clear:"both",top:"auto",bottom:"auto",float:"left",width:"100%",margin:-1*e.height+"px 0 0 0"}),this.stem.style.display="block"):this.stem.setStyle({position:"absolute",float:"none",margin:0}))),this.stemInverse=s},position:function(a){if(!this.tooltip&&!this.build())return;if(Tips.raise(this),Tips.fixIE&&(o=this.wrapper.getDimensions(),(!this.iframeShimDimensions||this.iframeShimDimensions.height!=o.height||this.iframeShimDimensions.width!=o.width)&&this.iframeShim.setStyle({width:o.width+"px",height:o.height+"px"}),this.iframeShimDimensions=o),this.options.hook){if(this.mouseHook){switch(d=document.viewport.getScrollOffsets(),r=a.fakePointer||{},n=2,this.mouseHook.toUpperCase()){case"LEFTTOP":case"TOPLEFT":t={x:0-n,y:0-n};break;case"TOPMIDDLE":t={x:0,y:0-n};break;case"TOPRIGHT":case"RIGHTTOP":t={x:n,y:0-n};break;case"RIGHTMIDDLE":t={x:n,y:0};break;case"RIGHTBOTTOM":case"BOTTOMRIGHT":t={x:n,y:n};break;case"BOTTOMMIDDLE":t={x:0,y:n};break;case"BOTTOMLEFT":case"LEFTBOTTOM":t={x:0-n,y:n};break;case"LEFTMIDDLE":t={x:0-n,y:0};break}t.x+=this.options.offset.x,t.y+=this.options.offset.y,c=Object.extend({offset:t},{element:this.options.hook.tip,mouseHook:this.mouseHook,mouse:{top:r.pointerY||Event.pointerY(a)-d.top,left:r.pointerX||Event.pointerX(a)-d.left}}),e=Tips.hook(this.wrapper,this.target,c),this.options.viewport&&(i=this.getPositionWithinViewport(e),s=i.stemInverse,e=i.position,e.left+=s.vertical?2*Prototip.toggleInt(t.x-this.options.offset.x):0,e.top+=s.vertical?2*Prototip.toggleInt(t.y-this.options.offset.y):0,this.stem&&(this.stemInverse.horizontal!=s.horizontal||this.stemInverse.vertical!=s.vertical)&&this.positionStem(s)),e={left:e.left+"px",top:e.top+"px"},this.wrapper.setStyle(e)}else c=Object.extend({offset:this.options.offset},{element:this.options.hook.tip,target:this.options.hook.target}),e=Tips.hook(this.wrapper,this.target,Object.extend({position:!0},c)),e={left:e.left+"px",top:e.top+"px"};this.loader&&(h=Tips.hook(this.loader,this.target,Object.extend({position:!0},c))),Tips.fixIE&&this.iframeShim.setStyle(e)}else{var t,n,s,o,i,c,d,u,h,l=this.target.cumulativeOffset(),r=a.fakePointer||{},e={left:(this.options.fixed?l[0]:r.pointerX||Event.pointerX(a))+this.options.offset.x,top:(this.options.fixed?l[1]:r.pointerY||Event.pointerY(a))+this.options.offset.y};!this.options.fixed&&this.element!==this.target&&(u=this.element.cumulativeOffset(),e.left+=-1*(u[0]-l[0]),e.top+=-1*(u[1]-l[1])),!this.options.fixed&&this.options.viewport&&(i=this.getPositionWithinViewport(e),s=i.stemInverse,e=i.position,this.stem&&(this.stemInverse.horizontal!=s.horizontal||this.stemInverse.vertical!=s.vertical)&&this.positionStem(s)),e={left:e.left+"px",top:e.top+"px"},this.wrapper.setStyle(e),this.loader&&this.loader.setStyle(e),Tips.fixIE&&this.iframeShim.setStyle(e)}},getPositionWithinViewport:function(n){var e,s={horizontal:!1,vertical:!1},o=this.wrapper.getDimensions(),i=document.viewport.getScrollOffsets(),a=document.viewport.getDimensions(),t={left:"width",top:"height"};for(e in t)n[e]+o[t[e]]-i[e]>a[t[e]]&&(n[e]=n[e]-(o[t[e]]+2*this.options.offset[e=="left"?"x":"y"]),this.stem&&(s[Tips._stemTranslation[t[e]]]=!0));return{position:n,stemInverse:s}}}),Object.extend(Prototip,{createCorner:function(c,r){var n,i,a,l,o=arguments[2]||this.options,t=o.radius,e=o.border,s={top:r.charAt(0)=="t",left:r.charAt(1)=="l"};this.support.canvas?(a=new Element("canvas",{className:"cornerCanvas"+r.capitalize(),width:e+"px",height:e+"px"}),c.insert(a),n=a.getContext("2d"),n.fillStyle=o.backgroundColor,n.arc(s.left?t:e-t,s.top?t:e-t,t,0,Math.PI*2,!0),n.fill(),n.fillRect(s.left?t:0,0,e-t,e),n.fillRect(0,s.top?t:0,e,e-t)):(c.insert(l=new Element("div").setStyle({width:e+"px",height:e+"px",margin:0,padding:0,display:"block",position:"relative",overflow:"hidden"})),i=new Element("ns_vml:roundrect",{fillcolor:o.backgroundColor,strokeWeight:"1px",strokeColor:o.backgroundColor,arcSize:(t/e*.5).toFixed(2)}).setStyle({width:2*e-1+"px",height:2*e-1+"px",position:"absolute",left:(s.left?0:-1*e)+"px",top:(s.top?0:-1*e)+"px"}),l.insert(i),i.outerHTML=i.outerHTML)}}),Element.addMethods({setPngBackground:function(e,n){e=$(e);var t=Object.extend({align:"top left",repeat:"no-repeat",sizingMethod:"scale",backgroundColor:""},arguments[2]||{});return e.setStyle(Tips.fixIE?{filter:"progid:DXImageTransform.Microsoft.AlphaImageLoader(src='"+n+"'', sizingMethod='"+t.sizingMethod+"')"}:{background:t.backgroundColor+" url("+n+") "+t.align+" "+t.repeat}),e}}),Prototip.Methods={hold:function(e){return!!(e.element&&!e.element.parentNode)},show:function(){if(Prototip.Methods.hold(this))return;if(Tips.raise(this),this.cancelHideAfter(),e={},this.options.hook&&!this.options.hook.mouse)e.fakePointer={pointerX:0,pointerY:0};else{var e,t=this.target.cumulativeOffset(),n=this.target.cumulativeScrollOffset(),s=document.viewport.getScrollOffsets();t.left+=-1*(n[0]-s[0]),t.top+=-1*(n[1]-s[1]),e.fakePointer={pointerX:t.left,pointerY:t.top}}this.options.ajax&&!this.ajaxContentLoaded?this.ajaxShow(this.showDelayed,e):this.showDelayed(e),this.hideAfter()}},Prototip.extend=function(e){e.element.prototip={},Object.extend(e.element.prototip,{show:Prototip.Methods.show.bind(e),hide:e.hide.bind(e),remove:Tips.remove.bind(Tips,e.element)})},Prototip.start(),Validator=Class.create(),Validator.prototype={initialize:function(t,n,e,s){typeof e=="function"?(this.options=$H(s),this._test=e):(this.options=$H(e),this._test=function(){return!0}),this.error=n||"Validation failed.",this.className=t},test:function(e,t){return this._test(e,t)&&this.options.all(function(n){return!Validator.methods[n.key]||Validator.methods[n.key](e,t,n.value)})}},Validator.methods={pattern:function(e,n,t){return Validation.get("IsEmpty").test(e)||t.test(e)},minLength:function(e,n,t){return e.length>=t},maxLength:function(e,n,t){return e.length<=t},min:function(e,n,t){return e>=parseFloat(t)},max:function(e,n,t){return e<=parseFloat(t)},notOneOf:function(e,n,t){return $A(t).all(function(t){return e!=t})},oneOf:function(e,n,t){return $A(t).any(function(t){return e==t})},is:function(e,n,t){return e==t},isNot:function(e,n,t){return e!=t},equalToField:function(e,n,t){return e==$F(t)},notEqualToField:function(e,n,t){return e!=$F(t)},include:function(e,t,n){return $A(n).all(function(n){return Validation.get(n).test(e,t)})}},Validation=Class.create(),Validation.prototype={initialize:function(e,t){if(this.options=Object.extend({onSubmit:!0,stopOnFirst:!1,immediate:!1,focusOnError:!0,useTitles:!1,onFormValidate:function(){},onElementValidate:function(){}},t||{}),this.form=$(e),this.options.onSubmit&&Event.observe(this.form,"submit",this.onSubmit.bind(this),!1),this.options.immediate){var n=this.options.useTitles,s=this.options.onElementValidate;Form.getElements(this.form).each(function(e){Event.observe(e,"blur",function(e){Validation.validate(Event.element(e),{useTitle:n,onElementValidate:s})})})}},onSubmit:function(e){this.validate()||Event.stop(e)},validate:function(){var e=!1,t=this.options.useTitles,n=this.options.onElementValidate;return this.options.stopOnFirst?e=Form.getElements(this.form).all(function(e){return Validation.validate(e,{useTitle:t,onElementValidate:n})}):e=Form.getElements(this.form).collect(function(e){return Validation.validate(e,{useTitle:t,onElementValidate:n})}).all(),!e&&this.options.focusOnError&&Form.getElements(this.form).findAll(function(e){return $(e).hasClassName("validation-failed")}).first().focus(),this.options.onFormValidate(e,this.form),e},reset:function(){Form.getElements(this.form).each(Validation.reset)}},Object.extend(Validation,{validate:function(e,t){t=Object.extend({useTitle:!1,onElementValidate:function(){}},t||{}),e=$(e);var n=e.classNames();return result=n.all(function(s){var n=Validation.test(s,e,t.useTitle);return t.onElementValidate(n,e),n})},test:function(n,e,r){var t,i,a,s=Validation.get(n),o="__advice"+n.camelize();try{if(Validation.isVisible(e)&&!s.test($F(e),e)){if(!e[o]){if(t=Validation.getAdvice(n,e),t==null){switch(a=r?e&&e.title?e.title:s.error:s.error,t='<div class="validation-advice" id="advice-'+n+"-"+Validation.getElmID(e)+'" style="display:none">'+a+"</div>",e.type.toLowerCase()){case"checkbox":case"radio":i=e.parentNode,i?new Insertion.Bottom(i,t):new Insertion.After(e,t);break;default:new Insertion.After(e,t)}t=Validation.getAdvice(n,e)}typeof Effect=="undefined"?t.style.display="block":new Effect.Appear(t,{duration:1})}return e[o]=!0,e.removeClassName("validation-passed"),e.addClassName("validation-failed"),!1}return t=Validation.getAdvice(n,e),t!=null&&t.hide(),e[o]='',e.removeClassName("validation-failed"),e.addClassName("validation-passed"),!0}catch(e){throw e}},isVisible:function(e){for(;e.tagName!="BODY";){if(!$(e).visible())return!1;e=e.parentNode}return!0},getAdvice:function(t,e){return $("advice-"+t+"-"+Validation.getElmID(e))||$("advice-"+Validation.getElmID(e))},getElmID:function(e){return e.id?e.id:e.name},reset:function(e){e=$(e);var t=e.classNames();t.each(function(t){var s,n="__advice"+t.camelize();e[n]&&(s=Validation.getAdvice(t,e),s.hide(),e[n]=''),e.removeClassName("validation-failed"),e.removeClassName("validation-passed")})},add:function(e,n,s,o){var t={};t[e]=new Validator(e,n,s,o),Object.extend(Validation.methods,t)},addAllThese:function(t){var e={};$A(t).each(function(t){e[t[0]]=new Validator(t[0],t[1],t[2],t.length>3?t[3]:{})}),Object.extend(Validation.methods,e)},get:function(e){return Validation.methods[e]?Validation.methods[e]:Validation.methods._LikeNoIDIEverSaw_},methods:{_LikeNoIDIEverSaw_:new Validator("_LikeNoIDIEverSaw_",'',{})}}),Validation.add("IsEmpty",'',function(e){return e==null||e.length==0}),Validation.addAllThese([["required","This is a required field.",function(e){return!Validation.get("IsEmpty").test(e)}],["validate-number","Please enter a valid number in this field.",function(e){return Validation.get("IsEmpty").test(e)||!isNaN(e)&&!/^\s+$/.test(e)}],["validate-digits","Please use numbers only in this field. please avoid spaces or other characters such as dots or commas.",function(e){return Validation.get("IsEmpty").test(e)||!/[^\d]/.test(e)}],["validate-alpha","Please use letters only (a-z) in this field.",function(e){return Validation.get("IsEmpty").test(e)||/^[a-zA-Z]+$/.test(e)}],["validate-alphanum","Please use only letters (a-z) or numbers (0-9) only in this field. No spaces or other characters are allowed.",function(e){return Validation.get("IsEmpty").test(e)||!/\W/.test(e)}],["validate-date","Please enter a valid date.",function(e){var t=new Date(e);return Validation.get("IsEmpty").test(e)||!isNaN(t)}],["validate-email","Please enter a valid email address. For example fred@domain.com .",function(e){return Validation.get("IsEmpty").test(e)||/\w{1,}[@][\w-]{1,}([.]([\w-]{1,})){1,3}$/.test(e)}],["validate-url","Please enter a valid URL.",function(e){return Validation.get("IsEmpty").test(e)||/^(http|https|ftp):\/\/(([A-Z0-9][A-Z0-9_-]*)(\.[A-Z0-9][A-Z0-9_-]*)+)(:(\d+))?\/?/i.test(e)}],["validate-date-au","Please use this date format: dd/mm/yyyy. For example 17/03/2006 for the 17th of March, 2006.",function(t){if(Validation.get("IsEmpty").test(t))return!0;var e,n=/^(\d{2})\/(\d{2})\/(\d{4})$/;return!!n.test(t)&&(e=new Date(t.replace(n,"$2/$1/$3")),parseInt(RegExp.$2,10)==1+e.getMonth()&&parseInt(RegExp.$1,10)==e.getDate()&&parseInt(RegExp.$3,10)==e.getFullYear())}],["validate-currency-dollar","Please enter a valid $ amount. For example $100.00 .",function(e){return Validation.get("IsEmpty").test(e)||/^\$?-?([1-9]{1}[0-9]{0,2}(,[0-9]{3})*(\.[0-9]{0,2})?|[1-9]{1}\d*(\.[0-9]{0,2})?|0(\.[0-9]{0,2})?|(\.[0-9]{1,2})?)$/.test(e)}],["validate-selection","Please make a selection",function(t,e){return e.options?e.selectedIndex>0:!Validation.get("IsEmpty").test(t)}],["validate-one-required","Please select one of the above options.",function(s,e){var t=e.parentNode,n=t.getElementsByTagName("INPUT");return $A(n).any(function(e){return $F(e)})}]]),typeof Control=="undefined"&&(Control={}),$proc=function(e){return typeof e=="function"?e:function(){return e}},$value=function(e){return typeof e=="function"?e():e},Object.Event={extend:function(e){e._objectEventSetup=function(e){this._observers=this._observers||{},this._observers[e]=this._observers[e]||[]},e.observe=function(e,t){if(typeof e=="string"&&typeof t!="undefined")this._objectEventSetup(e),this._observers[e].include(t)||this._observers[e].push(t);else for(var n in e)this.observe(n,e[n])},e.stopObserving=function(e,t){this._objectEventSetup(e),e&&t?this._observers[e]=this._observers[e].without(t):e?this._observers[e]=[]:this._observers={}},e.observeOnce=function(e,n){var t=function(){n.apply(this,arguments),this.stopObserving(e,t)}.bind(this);this._objectEventSetup(e),this._observers[e].push(t)},e.notify=function(t){this._objectEventSetup(t);var e,n=[],s=$A(arguments).slice(1);try{for(e=0;e<this._observers[t].length;++e)n.push(this._observers[t][e].apply(this._observers[t][e],s)||null)}catch(e){if(e==$break)return!1;throw e}return n},e.prototype&&(e.prototype._objectEventSetup=e._objectEventSetup,e.prototype.observe=e.observe,e.prototype.stopObserving=e.stopObserving,e.prototype.observeOnce=e.observeOnce,e.prototype.notify=function(t){if(e.notify){var s,o,i,n=$A(arguments).slice(1);n.unshift(this),n.unshift(t),e.notify.apply(e,n)}this._objectEventSetup(t),n=$A(arguments).slice(1),o=[];try{this.options&&this.options[t]&&typeof this.options[t]=="function"&&o.push(this.options[t].apply(this,n)||null),i=this._observers[t];for(s=0;s<i.length;++s)o.push(i[s].apply(i[s],n)||null)}catch(e){if(e==$break)return!1;throw e}return o})}},Element.addMethods({observeOnce:function(e,t,s){var n=function(){s.apply(this,arguments),Element.stopObserving(e,t,n)};Element.observe(e,t,n)}}),function(){function e(e){var t,n,s;if(e.wheelDelta?n=e.wheelDelta/120:e.detail&&(n=-e.detail/3),!n)return;if(t=Event.extend(e).target,t=Element.extend(t.nodeType===Node.TEXT_NODE?t.parentNode:t),s=t.fire("mouse:wheel",{delta:n}),s.stopped)return Event.stop(e),!1}document.observe("mousewheel",e),document.observe("DOMMouseScroll",e)}(),IframeShim=Class.create({initialize:function(){this.element=new Element("iframe",{style:"position:absolute;filter:progid:DXImageTransform.Microsoft.Alpha(opacity=0);display:none",src:"javascript:void(0);",frameborder:0}),$(document.body).insert(this.element)},hide:function(){return this.element.hide(),this},show:function(){return this.element.show(),this},positionUnder:function(e){var e=$(e),t=e.cumulativeOffset(),n=e.getDimensions();return this.element.setStyle({left:t[0]+"px",top:t[1]+"px",width:n.width+"px",height:n.height+"px",zIndex:e.getStyle("zIndex")-1}).show(),this},setBounds:function(e){for(prop in e)e[prop]+="px";return this.element.setStyle(e),this},destroy:function(){return this.element&&this.element.remove(),this}}),typeof Prototype=="undefined")throw"Control.Tabs requires Prototype to be loaded.";if(typeof Object.Event=="undefined")throw"Control.Tabs requires Object.Event to be loaded.";Control.Tabs=Class.create({initialize:function(e,n){if(!$(e))throw"Control.Tabs could not find the element: "+e;this.activeContainer=!1,this.activeLink=!1,this.containers=$H({}),this.links=[],Control.Tabs.instances.push(this),this.options={beforeChange:Prototype.emptyFunction,afterChange:Prototype.emptyFunction,hover:!1,linkSelector:"li a",setClassOnContainer:!1,activeClassName:"active",defaultTab:"first",autoLinkExternal:!0,targetRegExp:/#(.+)$/,showFunction:Element.show,hideFunction:Element.hide},Object.extend(this.options,n||{}),(typeof this.options.linkSelector=="string"?$(e).select(this.options.linkSelector):this.options.linkSelector($(e))).findAll(function(e){return/^#/.exec(e.href.replace(window.location.href.split("#")[0],''))}).each(function(e){this.addTab(e)}.bind(this)),this.containers.values().each(Element.hide),this.options.defaultTab=="first"?this.setActiveTab(this.links.first()):this.options.defaultTab=="last"?this.setActiveTab(this.links.last()):this.setActiveTab(this.options.defaultTab);var t=this.options.targetRegExp.exec(window.location);t&&t[1]&&t[1].split(",").each(function(e){this.setActiveTab(this.links.find(function(t){return t.key==e}))}.bind(this)),this.options.autoLinkExternal&&$A(document.getElementsByTagName("a")).each(function(e){if(!this.links.include(e)){var t=e.href.replace(window.location.href.split("#")[0],'');t.substring(0,1)=="#"&&this.containers.keys().include(t.substring(1))&&$(e).observe("click",function(t,e){this.setActiveTab(e.substring(1))}.bindAsEventListener(this,t))}}.bind(this))},addTab:function(e){this.links.push(e),e.key=e.getAttribute("href").replace(window.location.href.split("#")[0],'').split("#").last().replace(/#/,'');var t=$(e.key);if(!t)throw"Control.Tabs: #"+e.key+" was not found on the page.";this.containers.set(e.key,t),e[this.options.hover?"onmouseover":"onclick"]=function(e){return window.event&&Event.stop(window.event),this.setActiveTab(e),!1}.bind(this,e)},setActiveTab:function(e){if(!e&&typeof e=="undefined")return;if(typeof e=="string")this.setActiveTab(this.links.find(function(t){return t.key==e}));else if(typeof e=="number")this.setActiveTab(this.links[e]);else{if(this.notify("beforeChange",this.activeContainer,this.containers.get(e.key))===!1)return;this.activeContainer&&this.options.hideFunction(this.activeContainer),this.links.each(function(e){(this.options.setClassOnContainer?$(e.parentNode):e).removeClassName(this.options.activeClassName)}.bind(this)),(this.options.setClassOnContainer?$(e.parentNode):e).addClassName(this.options.activeClassName),this.activeContainer=this.containers.get(e.key),this.activeLink=e,this.options.showFunction(this.containers.get(e.key)),this.notify("afterChange",this.containers.get(e.key))}},next:function(){this.links.each(function(t,e){if(this.activeLink==t&&this.links[e+1])throw this.setActiveTab(this.links[e+1]),$break}.bind(this))},previous:function(){this.links.each(function(t,e){if(this.activeLink==t&&this.links[e-1])throw this.setActiveTab(this.links[e-1]),$break}.bind(this))},first:function(){this.setActiveTab(this.links.first())},last:function(){this.setActiveTab(this.links.last())}}),Object.extend(Control.Tabs,{instances:[],findByTabId:function(e){return Control.Tabs.instances.find(function(t){return t.links.find(function(t){return t.key==e})})}}),Object.Event.extend(Control.Tabs),dynamicOptionListCount=0,dynamicOptionListObjects=new Array;function initDynamicOptionLists(){for(var e,t,n,s,o,i,a,r,d,u,c=0;c<dynamicOptionListObjects.length;c++){if(e=dynamicOptionListObjects[c],e.formName!=null)e.form=document.forms[e.formName];else if(e.formIndex!=null)e.form=document.forms[e.formIndex];else{r=e.fieldNames[0][0];for(i=0;i<document.forms.length;i++)if(typeof document.forms[i][r]!="undefined"){e.form=document.forms[i];break}if(e.form==null){alert("ERROR: Couldn't find form element "+r+" in any form on the page! Init aborted");return}}for(s=0;s<e.fieldNames.length;s++)for(o=0;o<e.fieldNames[s].length-1;o++){if(n=e.form[e.fieldNames[s][o]],typeof n=="undefined"){alert("Select box named "+e.fieldNames[s][o]+" could not be found in the form. Init aborted");return}if(o==0&&n.options!=null)for(l=0;l<n.options.length;l++)t=n.options[l],d=e.findMatchingOptionInArray(e.options,t.text,t.value,!1),d!=null&&(u=t.selected,a=new Option(t.text,t.value,t.defaultSelected,t.selected),a.selected=t.selected,a.defaultSelected=t.defaultSelected,a.DOLOption=d,n.options[l]=a,n.options[l].selected=u);n.onchange==null&&(n.onchange=new Function("dynamicOptionListObjects["+e.index+"].change(this)"))}}resetDynamicOptionLists()}function resetDynamicOptionLists(n){for(var e,t,s=0;s<dynamicOptionListObjects.length;s++)if(e=dynamicOptionListObjects[s],typeof n=="undefined"||n==null||n==e.form)for(t=0;t<e.fieldNames.length;t++)e.change(e.form[e.fieldNames[t][0]],!0)}function DOLOption(e,t,n,s){return this.text=e,this.value=t,this.defaultSelected=n,this.selected=s,this.options=new Array,this}function DynamicOptionList(){if(this.form=null,this.options=new Array,this.longestString=new Array,this.numberOfOptions=new Array,this.currentNode=null,this.currentField=null,this.currentNodeDepth=0,this.fieldNames=new Array,this.formIndex=null,this.formName=null,this.fieldListIndexes=new Object,this.fieldIndexes=new Object,this.selectFirstOption=!0,this.numberOfOptions=new Array,this.longestString=new Array,this.values=new Object,this.forValue=DOL_forValue,this.forText=DOL_forText,this.forField=DOL_forField,this.forX=DOL_forX,this.addOptions=DOL_addOptions,this.addOptionsTextValue=DOL_addOptionsTextValue,this.setDefaultOptions=DOL_setDefaultOptions,this.setValues=DOL_setValues,this.setValue=DOL_setValues,this.setFormIndex=DOL_setFormIndex,this.setFormName=DOL_setFormName,this.printOptions=DOL_printOptions,this.addDependentFields=DOL_addDependentFields,this.change=DOL_change,this.child=DOL_child,this.selectChildOptions=DOL_selectChildOptions,this.populateChild=DOL_populateChild,this.change=DOL_change,this.addNewOptionToList=DOL_addNewOptionToList,this.findMatchingOptionInArray=DOL_findMatchingOptionInArray,arguments.length>0){for(var e=0;e<arguments.length;e++)this.fieldListIndexes[arguments[e].toString()]=this.fieldNames.length,this.fieldIndexes[arguments[e].toString()]=e;this.fieldNames[this.fieldNames.length]=arguments}this.index=window.dynamicOptionListCount++,window.dynamicOptionListObjects[this.index]=this}function DOL_findMatchingOptionInArray(t,a,s,r){if(t==null||typeof t=="undefined")return null;for(var e,n=null,o=null,i=0;i<t.length;i++){if(e=t[i],e.value==s&&e.text==a)return e;r||(n==null&&s!=null&&e.value==s&&(n=e),o==null&&a!=null&&e.text==a&&(o=e))}return n!=null?n:o}function DOL_forX(t,n){this.currentNode==null&&(this.currentNodeDepth=0);var s=this.currentNode==null?this:this.currentNode,e=this.findMatchingOptionInArray(s.options,n=="text"?t:null,n=="value"?t:null,!1);return e==null&&(e=new DOLOption(null,null,!1,!1),e[n]=t,s.options[s.options.length]=e),this.currentNode=e,this.currentNodeDepth++,this}function DOL_forValue(e){return this.forX(e,"value")}function DOL_forText(e){return this.forX(e,"text")}function DOL_forField(e){return this.currentField=e,this}function DOL_addNewOptionToList(e,s,o,i){var t,n=new DOLOption(s,o,i,!1);e==null&&(e=new Array);for(t=0;t<e.length;t++)if(e[t].text==n.text&&e[t].value==n.value)return n.selected&&(e[t].selected=!0),n.defaultSelected&&(e[t].defaultSelected=!0),e;e[e.length]=n}function DOL_addOptions(){this.currentNode==null&&(this.currentNode=this),this.currentNode.options==null&&(this.currentNode.options=new Array);for(var e,t=0;t<arguments.length;t++)e=arguments[t],this.addNewOptionToList(this.currentNode.options,e,e,!1),typeof this.numberOfOptions[this.currentNodeDepth]=="undefined"&&(this.numberOfOptions[this.currentNodeDepth]=0),this.currentNode.options.length>this.numberOfOptions[this.currentNodeDepth]&&(this.numberOfOptions[this.currentNodeDepth]=this.currentNode.options.length),(typeof this.longestString[this.currentNodeDepth]=="undefined"||e.length>this.longestString[this.currentNodeDepth].length)&&(this.longestString[this.currentNodeDepth]=e);this.currentNode=null,this.currentNodeDepth=0}function DOL_addOptionsTextValue(){this.currentNode==null&&(this.currentNode=this),this.currentNode.options==null&&(this.currentNode.options=new Array);for(var t,n,e=0;e<arguments.length;e++)t=arguments[e++],n=arguments[e],this.addNewOptionToList(this.currentNode.options,t,n,!1),typeof this.numberOfOptions[this.currentNodeDepth]=="undefined"&&(this.numberOfOptions[this.currentNodeDepth]=0),this.currentNode.options.length>this.numberOfOptions[this.currentNodeDepth]&&(this.numberOfOptions[this.currentNodeDepth]=this.currentNode.options.length),(typeof this.longestString[this.currentNodeDepth]=="undefined"||t.length>this.longestString[this.currentNodeDepth].length)&&(this.longestString[this.currentNodeDepth]=t);this.currentNode=null,this.currentNodeDepth=0}function DOL_child(e){var t=this.fieldListIndexes[e.name],n=this.fieldIndexes[e.name];return n<this.fieldNames[t].length-1?this.form[this.fieldNames[t][n+1]]:null}function DOL_setDefaultOptions(){this.currentNode==null&&(this.currentNode=this);for(var t,e=0;e<arguments.length;e++)t=this.findMatchingOptionInArray(this.currentNode.options,null,arguments[e],!1),t!=null&&(t.defaultSelected=!0);this.currentNode=null}function DOL_setValues(){if(this.currentField==null){alert("Can't call setValues() without using forField() first!");return}typeof this.values[this.currentField]=="undefined"&&(this.values[this.currentField]=new Object);for(var e=0;e<arguments.length;e++)this.values[this.currentField][arguments[e]]=!0;this.currentField=null}function DOL_setFormIndex(e){this.formIndex=e}function DOL_setFormName(e){this.formName=e}function DOL_printOptions(s){if(navigator.appName=="Netscape"&&parseInt(navigator.appVersion)<=4){var e,t=this.fieldIndexes[s],n="";if(typeof this.numberOfOptions[t]!="undefined")for(e=0;e<this.numberOfOptions[t];e++)n+="<OPTION>";if(n+="<OPTION>",typeof this.longestString[t]!="undefined")for(e=0;e<this.longestString[t].length;e++)n+="_";document.writeln(n)}}function DOL_addDependentFields(){for(var e=0;e<arguments.length;e++)this.fieldListIndexes[arguments[e].toString()]=this.fieldNames.length,this.fieldIndexes[arguments[e].toString()]=e;this.fieldNames[this.fieldNames.length]=arguments}function DOL_change(n,s){(s==null||typeof s=="undefined")&&(s=!1);var e,o,i,a,r,c,l,d=this.fieldListIndexes[n.name],u=this.fieldIndexes[n.name],t=this.child(n);if(t==null)return;if(n.type=="select-one")t.options!=null&&(t.options.length=0),n.options!=null&&n.options.length>0&&n.selectedIndex>=0&&(c=n.options[n.selectedIndex],this.populateChild(c.DOLOption,t,s),this.selectChildOptions(t,s));else if(n.type=="select-multiple"){if(a=new Array,!s)for(e=0;e<t.options.length;e++)o=t.options[e],o.selected&&this.addNewOptionToList(a,o.text,o.value,o.defaultSelected);if(t.options.length=0,n.options!=null){i=n.options;for(e=0;e<i.length;e++)i[e].selected&&this.populateChild(i[e].DOLOption,t,s);if(r=!1,!s)for(e=0;e<t.options.length;e++)l=this.findMatchingOptionInArray(a,t.options[e].text,t.options[e].value,!0),l!=null&&(t.options[e].selected=!0,r=!0);r||this.selectChildOptions(t,s)}}this.change(t,s)}function DOL_populateChild(s,e){if(s!=null&&s.options!=null)for(var t,n,o,i,a,c,r=0;r<s.options.length;r++){t=s.options[r],e.options==null&&(e.options=new Array),a=!1,c=!1;for(o=0;o<e.options.length;o++)if(i=e.options[o],i.text==t.text&&i.value==t.value){a=!0;break}a||(n=new Option(t.text,t.value,!1,!1),n.selected=!1,n.defaultSelected=!1,n.DOLOption=t,e.options[e.options.length]=n)}}function DOL_selectChildOptions(e,r){var t,n,o,i,s=this.values[e.name],a=!1;if(r&&s!=null&&typeof s!="undefined")for(n=0;n<e.options.length;n++)if(i=e.options[n].value,i!=null&&s[i]!=null&&typeof s[i]!="undefined"){a=!0;break}o=!1;for(n=0;n<e.options.length;n++)t=e.options[n],a&&t.value!=null&&s[t.value]!=null&&typeof s[t.value]!="undefined"?(t.selected=!0,o=!0):!a&&t.DOLOption!=null&&t.DOLOption.defaultSelected&&(t.selected=!0,o=!0);this.selectFirstOption&&!o&&e.options.length>0?e.options[0].selected=!0:!o&&e.type=="select-one"&&(e.selectedIndex=-1)}"use strict";window.jscolor||(window.jscolor=function(){var e={register:function(){e.attachDOMReadyEvent(e.init),e.attachEvent(document,"mousedown",e.onDocumentMouseDown),e.attachEvent(document,"touchstart",e.onDocumentTouchStart),e.attachEvent(window,"resize",e.onWindowResize)},init:function(){e.jscolor.lookupClass&&e.jscolor.installByClassName(e.jscolor.lookupClass)},tryInstallOnElements:function(n,l){c=new RegExp("(^|\\s)("+l+")(\\s*(\\{[^}]*\\})|\\s|$)","i");for(t=0;t<n.length;t+=1){if(n[t].type!==void 0&&n[t].type.toLowerCase()=="color"&&e.isColorAttrSupported)continue;if(!n[t].jscolor&&n[t].className&&(i=n[t].className.match(c))){var t,o,i,c,a=n[t],s=null,r=e.getDataAttr(a,"jscolor");if(r!==null?s=r:i[4]&&(s=i[4]),o={},s)try{o=new Function("return ("+s+")")()}catch(t){e.warn("Error parsing jscolor options: "+t+":\n"+s)}a.jscolor=new e.jscolor(a,o)}}},isColorAttrSupported:function(){var e=document.createElement("input");return!!(e.setAttribute&&(e.setAttribute("type","color"),e.type.toLowerCase()=="color"))}(),isCanvasSupported:function(){var e=document.createElement("canvas");return!!e.getContext&&!!e.getContext("2d")}(),fetchElement:function(e){return typeof e=="string"?document.getElementById(e):e},isElementType:function(e,t){return e.nodeName.toLowerCase()===t.toLowerCase()},getDataAttr:function(t,n){var s="data-"+n,e=t.getAttribute(s);return e!==null?e:null},attachEvent:function(e,t,n){e.addEventListener?e.addEventListener(t,n,!1):e.attachEvent&&e.attachEvent("on"+t,n)},detachEvent:function(e,t,n){e.removeEventListener?e.removeEventListener(t,n,!1):e.detachEvent&&e.detachEvent("on"+t,n)},_attachedGroupEvents:{},attachGroupEvent:function(t,n,s,o){e._attachedGroupEvents.hasOwnProperty(t)||(e._attachedGroupEvents[t]=[]),e._attachedGroupEvents[t].push([n,s,o]),e.attachEvent(n,s,o)},detachGroupEvents:function(t){if(e._attachedGroupEvents.hasOwnProperty(t)){for(var n,s=0;s<e._attachedGroupEvents[t].length;s+=1)n=e._attachedGroupEvents[t][s],e.detachEvent(n[0],n[1],n[2]);delete e._attachedGroupEvents[t]}},attachDOMReadyEvent:function(s){var t,n=!1,e=function(){n||(n=!0,s())};if(document.readyState==="complete"){setTimeout(e,1);return}document.addEventListener?(document.addEventListener("DOMContentLoaded",e,!1),window.addEventListener("load",e,!1)):document.attachEvent&&(document.attachEvent("onreadystatechange",function(){document.readyState==="complete"&&(document.detachEvent("onreadystatechange",arguments.callee),e())}),window.attachEvent("onload",e),document.documentElement.doScroll&&window==window.top&&(t=function(){if(!document.body)return;try{document.documentElement.doScroll("left"),e()}catch(e){setTimeout(t,1)}},t()))},warn:function(e){window.console&&window.console.warn&&window.console.warn(e)},preventDefault:function(e){e.preventDefault&&e.preventDefault(),e.returnValue=!1},captureTarget:function(t){t.setCapture&&(e._capturedTarget=t,e._capturedTarget.setCapture())},releaseTarget:function(){e._capturedTarget&&(e._capturedTarget.releaseCapture(),e._capturedTarget=null)},fireEvent:function(e,t){if(!e)return;if(document.createEvent){var n=document.createEvent("HTMLEvents");n.initEvent(t,!0,!0),e.dispatchEvent(n)}else document.createEventObject?(n=document.createEventObject(),e.fireEvent("on"+t,n)):e["on"+t]&&e["on"+t]()},classNameToList:function(e){return e.replace(/^\s+|\s+$/g,"").split(/\s+/)},hasClass:function(t,e){return!!e&&-1!=(" "+t.className.replace(/\s+/g," ")+" ").indexOf(" "+e+" ")},setClass:function(n,o){for(var s=e.classNameToList(o),t=0;t<s.length;t+=1)e.hasClass(n,s[t])||(n.className+=(n.className?" ":"")+s[t])},unsetClass:function(s,i){for(var o,n=e.classNameToList(i),t=0;t<n.length;t+=1)o=new RegExp("^\\s*"+n[t]+"\\s*|"+"\\s*"+n[t]+"\\s*$|"+"\\s+"+n[t]+"(\\s+)","g"),s.className=s.className.replace(o,"$1")},getStyle:function(e){return window.getComputedStyle?window.getComputedStyle(e):e.currentStyle},setStyle:function(){var t=document.createElement("div"),e=function(n){for(var e=0;e<n.length;e+=1)if(n[e]in t.style)return n[e]},n={borderRadius:e(["borderRadius","MozBorderRadius","webkitBorderRadius"]),boxShadow:e(["boxShadow","MozBoxShadow","webkitBoxShadow"])};return function(e,s,t){switch(s.toLowerCase()){case"opacity":var o=Math.round(parseFloat(t)*100);e.style.opacity=t,e.style.filter="alpha(opacity="+o+")";break;default:e.style[n[s]]=t}}}(),setBorderRadius:function(t,n){e.setStyle(t,"borderRadius",n||"0")},setBoxShadow:function(t,n){e.setStyle(t,"boxShadow",n||"none")},getElementPos:function(i,a){var s,t=0,n=0,o=i.getBoundingClientRect(),t=o.left,n=o.top;return a||(s=e.getViewPos(),t+=s[0],n+=s[1]),[t,n]},getElementSize:function(e){return[e.offsetWidth,e.offsetHeight]},getAbsPointerPos:function(e){e||(e=window.event);var t=0,n=0;return typeof e.changedTouches!="undefined"&&e.changedTouches.length?(t=e.changedTouches[0].clientX,n=e.changedTouches[0].clientY):typeof e.clientX=="number"&&(t=e.clientX,n=e.clientY),{x:t,y:n}},getRelPointerPos:function(e){e||(e=window.event);var a=e.target||e.srcElement,s=a.getBoundingClientRect(),o=0,i=0,t=0,n=0;return typeof e.changedTouches!="undefined"&&e.changedTouches.length?(t=e.changedTouches[0].clientX,n=e.changedTouches[0].clientY):typeof e.clientX=="number"&&(t=e.clientX,n=e.clientY),o=t-s.left,i=n-s.top,{x:o,y:i}},getViewPos:function(){var e=document.documentElement;return[(window.pageXOffset||e.scrollLeft)-(e.clientLeft||0),(window.pageYOffset||e.scrollTop)-(e.clientTop||0)]},getViewSize:function(){var e=document.documentElement;return[window.innerWidth||e.clientWidth,window.innerHeight||e.clientHeight]},redrawPosition:function(){if(e.picker&&e.picker.owner){r=e.picker.owner,r.fixed?(s=e.getElementPos(r.targetElement,!0),d=[0,0]):(s=e.getElementPos(r.targetElement),d=e.getViewPos());var t,n,a,i=e.getElementSize(r.targetElement),u=e.getViewSize(),c=e.getPickerOuterDims(r);switch(r.position.toLowerCase()){case"left":n=1,t=0,a=-1;break;case"right":n=1,t=0,a=1;break;case"top":n=0,t=1,a=-1;break;default:n=0,t=1,a=1}o=(i[t]+c[t])/2,r.smartPosition?(l=[-d[n]+s[n]+c[n]>u[n]?-d[n]+s[n]+i[n]/2>u[n]/2&&s[n]+i[n]-c[n]>=0?s[n]+i[n]-c[n]:s[n]:s[n],-d[t]+s[t]+i[t]+c[t]-o+o*a>u[t]?-d[t]+s[t]+i[t]/2>u[t]/2&&s[t]+i[t]-o-o*a>=0?s[t]+i[t]-o-o*a:s[t]+i[t]-o+o*a:s[t]+i[t]-o+o*a>=0?s[t]+i[t]-o+o*a:s[t]+i[t]-o-o*a]):(l=[s[n],s[t]+i[t]-o+o*a]);var s,o,r,l,d,h=l[n],m=l[t],f=r.fixed?"fixed":"absolute",p=(l[0]+c[0]>s[0]||l[0]<s[0]+i[0])&&l[1]+c[1]<s[1]+i[1];e._drawPosition(r,h,m,f,p)}},_drawPosition:function(t,n,s,o,i){var a=i?0:t.shadowBlur;e.picker.wrap.style.position=o,e.picker.wrap.style.left=n+"px",e.picker.wrap.style.top=s+"px",e.setBoxShadow(e.picker.boxS,t.shadow?new e.BoxShadow(0,a,t.shadowBlur,0,t.shadowColor):null)},getPickerDims:function(t){var n=!!e.getSliderComponent(t),s=[2*t.insetWidth+2*t.padding+t.width+(n?2*t.insetWidth+e.getPadToSliderPadding(t)+t.sliderSize:0),2*t.insetWidth+2*t.padding+t.height+(t.closable?2*t.insetWidth+t.padding+t.buttonHeight:0)];return s},getPickerOuterDims:function(t){var n=e.getPickerDims(t);return[n[0]+2*t.borderWidth,n[1]+2*t.borderWidth]},getPadToSliderPadding:function(e){return Math.max(e.padding,1.5*(2*e.pointerBorderWidth+e.pointerThickness))},getPadYComponent:function(e){switch(e.mode.charAt(1).toLowerCase()){case"v":return"v"}return"s"},getSliderComponent:function(e){if(e.mode.length>2)switch(e.mode.charAt(2).toLowerCase()){case"s":return"s";case"v":return"v"}return null},onDocumentMouseDown:function(n){n||(n=window.event);var t=n.target||n.srcElement;t._jscLinkedInstance?t._jscLinkedInstance.showOnClick&&t._jscLinkedInstance.show():t._jscControlName?e.onControlPointerStart(n,t,t._jscControlName,"mouse"):e.picker&&e.picker.owner&&e.picker.owner.hide()},onDocumentTouchStart:function(n){n||(n=window.event);var t=n.target||n.srcElement;t._jscLinkedInstance?t._jscLinkedInstance.showOnClick&&t._jscLinkedInstance.show():t._jscControlName?e.onControlPointerStart(n,t,t._jscControlName,"touch"):e.picker&&e.picker.owner&&e.picker.owner.hide()},onWindowResize:function(){e.redrawPosition()},onParentScroll:function(){e.picker&&e.picker.owner&&e.picker.owner.hide()},_pointerMoveEvent:{mouse:"mousemove",touch:"touchmove"},_pointerEndEvent:{mouse:"mouseup",touch:"touchend"},_pointerOrigin:null,_capturedTarget:null,onControlPointerStart:function(n,s,i,o){if(t=s._jscInstance,e.preventDefault(n),e.captureTarget(s),a=function(t,a){e.attachGroupEvent("drag",t,e._pointerMoveEvent[o],e.onDocumentPointerMove(n,s,i,o,a)),e.attachGroupEvent("drag",t,e._pointerEndEvent[o],e.onDocumentPointerEnd(n,s,i,o))},a(document,[0,0]),window.parent&&window.frameElement){var t,a,r,c,l=window.frameElement.getBoundingClientRect(),d=[-l.left,-l.top];a(window.parent.window.document,d)}switch(r=e.getAbsPointerPos(n),c=e.getRelPointerPos(n),e._pointerOrigin={x:r.x-c.x,y:r.y-c.y},i){case"pad":switch(e.getSliderComponent(t)){case"s":t.hsv[1]===0&&t.fromHSV(null,100,null);break;case"v":t.hsv[2]===0&&t.fromHSV(null,null,100)}e.setPad(t,n,0,0);break;case"sld":e.setSld(t,n,0)}e.dispatchFineChange(t)},onDocumentPointerMove:function(o,n,s,i,t){return function(o){var i=n._jscInstance;switch(s){case"pad":o||(o=window.event),e.setPad(i,o,t[0],t[1]),e.dispatchFineChange(i);break;case"sld":o||(o=window.event),e.setSld(i,o,t[1]),e.dispatchFineChange(i)}}},onDocumentPointerEnd:function(n,t){return function(){var n=t._jscInstance;e.detachGroupEvents("drag"),e.releaseTarget(),e.dispatchChange(n)}},dispatchChange:function(t){t.valueElement&&e.isElementType(t.valueElement,"input")&&e.fireEvent(t.valueElement,"change")},dispatchFineChange:function(e){if(e.onFineChange){var t;typeof e.onFineChange=="string"?t=new Function(e.onFineChange):t=e.onFineChange,t.call(e)}},setPad:function(t,c,l,i){var o=e.getAbsPointerPos(c),a=l+o.x-e._pointerOrigin.x-t.padding-t.insetWidth,r=i+o.y-e._pointerOrigin.y-t.padding-t.insetWidth,n=a*(360/(t.width-1)),s=100-r*(100/(t.height-1));switch(e.getPadYComponent(t)){case"s":t.fromHSV(n,s,null,e.leaveSld);break;case"v":t.fromHSV(n,null,s,e.leaveSld)}},setSld:function(t,s,o){var i=e.getAbsPointerPos(s),a=o+i.y-e._pointerOrigin.y-t.padding-t.insetWidth,n=100-a*(100/(t.height-1));switch(e.getSliderComponent(t)){case"s":t.fromHSV(null,n,null,e.leavePad);break;case"v":t.fromHSV(null,null,n,e.leavePad)}},_vmlNS:"jsc_vml_",_vmlCSS:"jsc_vml_css_",_vmlReady:!1,initVML:function(){if(!e._vmlReady){if(t=document,t.namespaces[e._vmlNS]||t.namespaces.add(e._vmlNS,"urn:schemas-microsoft-com:vml"),!t.styleSheets[e._vmlCSS]){var t,n,s=["shape","shapetype","group","background","path","formulas","handles","fill","stroke","shadow","textbox","textpath","imagedata","line","polyline","curve","rect","roundrect","oval","arc","image"],o=t.createStyleSheet();o.owningElement.id=e._vmlCSS;for(n=0;n<s.length;n+=1)o.addRule(e._vmlNS+"\\:"+s[n],"behavior:url(#default#VML);")}e._vmlReady=!0}},createPalette:function(){if(c={elm:null,draw:null},e.isCanvasSupported){var n,s,o,i,a,c,t=document.createElement("canvas"),r=t.getContext("2d"),l=function(s,o,i){t.width=s,t.height=o,r.clearRect(0,0,t.width,t.height);var n,e=r.createLinearGradient(0,0,t.width,0);switch(e.addColorStop(0,"#F00"),e.addColorStop(1/6,"#FF0"),e.addColorStop(2/6,"#0F0"),e.addColorStop(.5,"#0FF"),e.addColorStop(4/6,"#00F"),e.addColorStop(5/6,"#F0F"),e.addColorStop(1,"#F00"),r.fillStyle=e,r.fillRect(0,0,t.width,t.height),n=r.createLinearGradient(0,0,0,t.height),i.toLowerCase()){case"s":n.addColorStop(0,"rgba(255,255,255,0)"),n.addColorStop(1,"rgba(255,255,255,1)");break;case"v":n.addColorStop(0,"rgba(0,0,0,0)"),n.addColorStop(1,"rgba(0,0,0,1)")}r.fillStyle=n,r.fillRect(0,0,t.width,t.height)};c.elm=t,c.draw=l}else e.initVML(),i=document.createElement("div"),i.style.position="relative",i.style.overflow="hidden",a=document.createElement(e._vmlNS+":fill"),a.type="gradient",a.method="linear",a.angle="90",a.colors="16.67% #F0F, 33.33% #00F, 50% #0FF, 66.67% #0F0, 83.33% #FF0",s=document.createElement(e._vmlNS+":rect"),s.style.position="absolute",s.style.left="-1px",s.style.top="-1px",s.stroked=!1,s.appendChild(a),i.appendChild(s),n=document.createElement(e._vmlNS+":fill"),n.type="gradient",n.method="linear",n.angle="180",n.opacity="0",o=document.createElement(e._vmlNS+":rect"),o.style.position="absolute",o.style.left="-1px",o.style.top="-1px",o.stroked=!1,o.appendChild(n),i.appendChild(o),l=function(e,t,r){switch(i.style.width=e+"px",i.style.height=t+"px",s.style.width=o.style.width=e+1+"px",s.style.height=o.style.height=t+1+"px",a.color="#F00",a.color2="#F00",r.toLowerCase()){case"s":n.color=n.color2="#FFF";break;case"v":n.color=n.color2="#000"}},c.elm=i,c.draw=l;return c},createSliderGradient:function(){if(i={elm:null,draw:null},e.isCanvasSupported){var t,s,o,i,n=document.createElement("canvas"),a=n.getContext("2d"),r=function(t,s,o,i){n.width=t,n.height=s,a.clearRect(0,0,n.width,n.height);var e=a.createLinearGradient(0,0,0,n.height);e.addColorStop(0,o),e.addColorStop(1,i),a.fillStyle=e,a.fillRect(0,0,n.width,n.height)};i.elm=n,i.draw=r}else e.initVML(),s=document.createElement("div"),s.style.position="relative",s.style.overflow="hidden",o=document.createElement(e._vmlNS+":fill"),o.type="gradient",o.method="linear",o.angle="180",t=document.createElement(e._vmlNS+":rect"),t.style.position="absolute",t.style.left="-1px",t.style.top="-1px",t.stroked=!1,t.appendChild(o),s.appendChild(t),r=function(e,n,i,a){s.style.width=e+"px",s.style.height=n+"px",t.style.width=e+1+"px",t.style.height=n+1+"px",o.color=i,o.color2=a},i.elm=s,i.draw=r;return i},leaveValue:1,leaveStyle:2,leavePad:4,leaveSld:8,BoxShadow:function(){var e=function(e,t,n,s,o,i){this.hShadow=e,this.vShadow=t,this.blur=n,this.spread=s,this.color=o,this.inset=!!i};return e.prototype.toString=function(){var e=[Math.round(this.hShadow)+"px",Math.round(this.vShadow)+"px",Math.round(this.blur)+"px",Math.round(this.spread)+"px",this.color];return this.inset&&e.push("inset"),e.join(" ")},e}(),jscolor:function(n,i){function v(e,t,n){e/=255,t/=255,n/=255;var a,i=Math.min(Math.min(e,t),n),o=Math.max(Math.max(e,t),n),s=o-i;return s===0?[null,0,100*o]:(a=e===i?3+(n-t)/s:t===i?5+(e-n)/s:1+(t-e)/s,[60*(a===6?0:a),100*(s/o),100*o])}function s(s,i,a){if(e=255*(a/100),s===null)return[e,e,e];s/=60,i/=100;var e,o=Math.floor(s),r=o%2?s-o:1-(s-o),t=e*(1-i),n=e*(1-i*r);switch(o){case 6:case 0:return[e,n,t];case 1:return[n,e,t];case 2:return[t,e,n];case 3:return[t,n,e];case 4:return[n,t,e];case 5:return[e,t,n]}}function g(){e.unsetClass(t.targetElement,t.activeClass),e.picker.wrap.parentNode.removeChild(e.picker.wrap),delete e.picker.owner}function m(){function c(){var e=t.insetColor.split(/\s+/),s=e.length<2?e[0]:e[1]+" "+e[0]+" "+e[0]+" "+e[1];n.btn.style.borderColor=s}t._processParentElementsInDOM(),e.picker||(e.picker={owner:null,wrap:document.createElement("div"),box:document.createElement("div"),boxS:document.createElement("div"),boxB:document.createElement("div"),pad:document.createElement("div"),padB:document.createElement("div"),padM:document.createElement("div"),padPal:e.createPalette(),cross:document.createElement("div"),crossBY:document.createElement("div"),crossBX:document.createElement("div"),crossLY:document.createElement("div"),crossLX:document.createElement("div"),sld:document.createElement("div"),sldB:document.createElement("div"),sldM:document.createElement("div"),sldGrad:e.createSliderGradient(),sldPtrS:document.createElement("div"),sldPtrIB:document.createElement("div"),sldPtrMB:document.createElement("div"),sldPtrOB:document.createElement("div"),btn:document.createElement("div"),btnT:document.createElement("span")},e.picker.pad.appendChild(e.picker.padPal.elm),e.picker.padB.appendChild(e.picker.pad),e.picker.cross.appendChild(e.picker.crossBY),e.picker.cross.appendChild(e.picker.crossBX),e.picker.cross.appendChild(e.picker.crossLY),e.picker.cross.appendChild(e.picker.crossLX),e.picker.padB.appendChild(e.picker.cross),e.picker.box.appendChild(e.picker.padB),e.picker.box.appendChild(e.picker.padM),e.picker.sld.appendChild(e.picker.sldGrad.elm),e.picker.sldB.appendChild(e.picker.sld),e.picker.sldB.appendChild(e.picker.sldPtrOB),e.picker.sldPtrOB.appendChild(e.picker.sldPtrMB),e.picker.sldPtrMB.appendChild(e.picker.sldPtrIB),e.picker.sldPtrIB.appendChild(e.picker.sldPtrS),e.picker.box.appendChild(e.picker.sldB),e.picker.box.appendChild(e.picker.sldM),e.picker.btn.appendChild(e.picker.btnT),e.picker.box.appendChild(e.picker.btn),e.picker.boxB.appendChild(e.picker.box),e.picker.wrap.appendChild(e.picker.boxS),e.picker.wrap.appendChild(e.picker.boxB));var n=e.picker,a=!!e.getSliderComponent(t),s=e.getPickerDims(t),o=2*t.pointerBorderWidth+t.pointerThickness+2*t.crossSize,r=e.getPadToSliderPadding(t),i=Math.min(t.borderRadius,Math.round(t.padding*Math.PI)),d="crosshair";n.wrap.style.clear="both",n.wrap.style.width=s[0]+2*t.borderWidth+"px",n.wrap.style.height=s[1]+2*t.borderWidth+"px",n.wrap.style.zIndex=t.zIndex,n.box.style.width=s[0]+"px",n.box.style.height=s[1]+"px",n.boxS.style.position="absolute",n.boxS.style.left="0",n.boxS.style.top="0",n.boxS.style.width="100%",n.boxS.style.height="100%",e.setBorderRadius(n.boxS,i+"px"),n.boxB.style.position="relative",n.boxB.style.border=t.borderWidth+"px solid",n.boxB.style.borderColor=t.borderColor,n.boxB.style.background=t.backgroundColor,e.setBorderRadius(n.boxB,i+"px"),n.padM.style.background=n.sldM.style.background="#FFF",e.setStyle(n.padM,"opacity","0"),e.setStyle(n.sldM,"opacity","0"),n.pad.style.position="relative",n.pad.style.width=t.width+"px",n.pad.style.height=t.height+"px",n.padPal.draw(t.width,t.height,e.getPadYComponent(t)),n.padB.style.position="absolute",n.padB.style.left=t.padding+"px",n.padB.style.top=t.padding+"px",n.padB.style.border=t.insetWidth+"px solid",n.padB.style.borderColor=t.insetColor,n.padM._jscInstance=t,n.padM._jscControlName="pad",n.padM.style.position="absolute",n.padM.style.left="0",n.padM.style.top="0",n.padM.style.width=t.padding+2*t.insetWidth+t.width+r/2+"px",n.padM.style.height=s[1]+"px",n.padM.style.cursor=d,n.cross.style.position="absolute",n.cross.style.left=n.cross.style.top="0",n.cross.style.width=n.cross.style.height=o+"px",n.crossBY.style.position=n.crossBX.style.position="absolute",n.crossBY.style.background=n.crossBX.style.background=t.pointerBorderColor,n.crossBY.style.width=n.crossBX.style.height=2*t.pointerBorderWidth+t.pointerThickness+"px",n.crossBY.style.height=n.crossBX.style.width=o+"px",n.crossBY.style.left=n.crossBX.style.top=Math.floor(o/2)-Math.floor(t.pointerThickness/2)-t.pointerBorderWidth+"px",n.crossBY.style.top=n.crossBX.style.left="0",n.crossLY.style.position=n.crossLX.style.position="absolute",n.crossLY.style.background=n.crossLX.style.background=t.pointerColor,n.crossLY.style.height=n.crossLX.style.width=o-2*t.pointerBorderWidth+"px",n.crossLY.style.width=n.crossLX.style.height=t.pointerThickness+"px",n.crossLY.style.left=n.crossLX.style.top=Math.floor(o/2)-Math.floor(t.pointerThickness/2)+"px",n.crossLY.style.top=n.crossLX.style.left=t.pointerBorderWidth+"px",n.sld.style.overflow="hidden",n.sld.style.width=t.sliderSize+"px",n.sld.style.height=t.height+"px",n.sldGrad.draw(t.sliderSize,t.height,"#000","#000"),n.sldB.style.display=a?"block":"none",n.sldB.style.position="absolute",n.sldB.style.right=t.padding+"px",n.sldB.style.top=t.padding+"px",n.sldB.style.border=t.insetWidth+"px solid",n.sldB.style.borderColor=t.insetColor,n.sldM._jscInstance=t,n.sldM._jscControlName="sld",n.sldM.style.display=a?"block":"none",n.sldM.style.position="absolute",n.sldM.style.right="0",n.sldM.style.top="0",n.sldM.style.width=t.sliderSize+r/2+t.padding+2*t.insetWidth+"px",n.sldM.style.height=s[1]+"px",n.sldM.style.cursor="default",n.sldPtrIB.style.border=n.sldPtrOB.style.border=t.pointerBorderWidth+"px solid "+t.pointerBorderColor,n.sldPtrOB.style.position="absolute",n.sldPtrOB.style.left=-(2*t.pointerBorderWidth+t.pointerThickness)+"px",n.sldPtrOB.style.top="0",n.sldPtrMB.style.border=t.pointerThickness+"px solid "+t.pointerColor,n.sldPtrS.style.width=t.sliderSize+"px",n.sldPtrS.style.height=f+"px",n.btn.style.display=t.closable?"block":"none",n.btn.style.position="absolute",n.btn.style.left=t.padding+"px",n.btn.style.bottom=t.padding+"px",n.btn.style.padding="0 15px",n.btn.style.height=t.buttonHeight+"px",n.btn.style.border=t.insetWidth+"px solid",c(),n.btn.style.color=t.buttonColor,n.btn.style.font="12px sans-serif",n.btn.style.textAlign="center";try{n.btn.style.cursor="pointer"}catch(e){n.btn.style.cursor="hand"}n.btn.onmousedown=function(){t.hide()},n.btnT.style.lineHeight=t.buttonHeight+"px",n.btnT.innerHTML="",n.btnT.appendChild(document.createTextNode(t.closeText)),h(),u(),e.picker.owner&&e.picker.owner!==t&&e.unsetClass(e.picker.owner.targetElement,t.activeClass),e.picker.owner=t,e.isElementType(l,"body")?e.redrawPosition():e._drawPosition(t,0,0,"relative",!1),n.wrap.parentNode!=l&&l.appendChild(n.wrap),e.setClass(t.targetElement,t.activeClass)}function h(){switch(e.getPadYComponent(t)){case"s":n=1;break;case"v":n=2}var n,d=Math.round(t.hsv[0]/360*(t.width-1)),u=Math.round((1-t.hsv[n]/100)*(t.height-1)),h=2*t.pointerBorderWidth+t.pointerThickness+2*t.crossSize,l=-Math.floor(h/2);switch(e.picker.cross.style.left=d+l+"px",e.picker.cross.style.top=u+l+"px",e.getSliderComponent(t)){case"s":var r=s(t.hsv[0],100,t.hsv[2]),c=s(t.hsv[0],0,t.hsv[2]),o="rgb("+Math.round(r[0])+","+Math.round(r[1])+","+Math.round(r[2])+")",i="rgb("+Math.round(c[0])+","+Math.round(c[1])+","+Math.round(c[2])+")";e.picker.sldGrad.draw(t.sliderSize,t.height,o,i);break;case"v":var a=s(t.hsv[0],t.hsv[1],100),o="rgb("+Math.round(a[0])+","+Math.round(a[1])+","+Math.round(a[2])+")",i="#000";e.picker.sldGrad.draw(t.sliderSize,t.height,o,i)}}function u(){var n,o,s=e.getSliderComponent(t);if(s){switch(s){case"s":n=1;break;case"v":n=2}o=Math.round((1-t.hsv[n]/100)*(t.height-1)),e.picker.sldPtrOB.style.top=o-(2*t.pointerBorderWidth+t.pointerThickness)-Math.floor(f/2)+"px"}}function o(){return e.picker&&e.picker.owner===t}function b(){t.importColor()}this.value=null,this.valueElement=n,this.styleElement=n,this.required=!0,this.refine=!0,this.hash=!1,this.uppercase=!0,this.onFineChange=null,this.activeClass="jscolor-active",this.minS=0,this.maxS=100,this.minV=0,this.maxV=100,this.hsv=[0,0,100],this.rgb=[255,255,255],this.width=181,this.height=101,this.showOnClick=!0,this.mode="HSV",this.position="bottom",this.smartPosition=!0,this.sliderSize=16,this.crossSize=8,this.closable=!1,this.closeText="Close",this.buttonColor="#000000",this.buttonHeight=18,this.padding=12,this.backgroundColor="#FFFFFF",this.borderWidth=1,this.borderColor="#BBBBBB",this.borderRadius=8,this.insetWidth=1,this.insetColor="#BBBBBB",this.shadow=!0,this.shadowBlur=15,this.shadowColor="rgba(0,0,0,0.2)",this.pointerColor="#4C4C4C",this.pointerBorderColor="#FFFFFF",this.pointerBorderWidth=1,this.pointerThickness=2,this.zIndex=1e3,this.container=null;for(c in i)i.hasOwnProperty(c)&&(this[c]=i[c]);if(this.hide=function(){o()&&g()},this.show=function(){m()},this.redraw=function(){o()&&m()},this.importColor=function(){this.valueElement?e.isElementType(this.valueElement,"input")?this.refine?!this.required&&/^\s*$/.test(this.valueElement.value)?(this.valueElement.value="",this.styleElement&&(this.styleElement.style.backgroundImage=this.styleElement._jscOrigStyle.backgroundImage,this.styleElement.style.backgroundColor=this.styleElement._jscOrigStyle.backgroundColor,this.styleElement.style.color=this.styleElement._jscOrigStyle.color),this.exportColor(e.leaveValue|e.leaveStyle)):this.fromString(this.valueElement.value)||this.exportColor():this.fromString(this.valueElement.value,e.leaveValue)||(this.styleElement&&(this.styleElement.style.backgroundImage=this.styleElement._jscOrigStyle.backgroundImage,this.styleElement.style.backgroundColor=this.styleElement._jscOrigStyle.backgroundColor,this.styleElement.style.color=this.styleElement._jscOrigStyle.color),this.exportColor(e.leaveValue|e.leaveStyle)):this.exportColor():this.exportColor()},this.exportColor=function(n){if(!(n&e.leaveValue)&&this.valueElement){var t=this.toString();this.uppercase&&(t=t.toUpperCase()),this.hash&&(t="#"+t),e.isElementType(this.valueElement,"input")?this.valueElement.value=t:this.valueElement.innerHTML=t}n&e.leaveStyle||this.styleElement&&(this.styleElement.style.backgroundImage="none",this.styleElement.style.backgroundColor="#"+this.toString(),this.styleElement.style.color=this.isLight()?"#000":"#FFF"),!(n&e.leavePad)&&o()&&h(),!(n&e.leaveSld)&&o()&&u()},this.fromHSV=function(e,t,n,o){if(e!==null){if(isNaN(e))return!1;e=Math.max(0,Math.min(360,e))}if(t!==null){if(isNaN(t))return!1;t=Math.max(0,Math.min(100,this.maxS,t),this.minS)}if(n!==null){if(isNaN(n))return!1;n=Math.max(0,Math.min(100,this.maxV,n),this.minV)}this.rgb=s(e===null?this.hsv[0]:this.hsv[0]=e,t===null?this.hsv[1]:this.hsv[1]=t,n===null?this.hsv[2]:this.hsv[2]=n),this.exportColor(o)},this.fromRGB=function(t,n,o,a){if(t!==null){if(isNaN(t))return!1;t=Math.max(0,Math.min(255,t))}if(n!==null){if(isNaN(n))return!1;n=Math.max(0,Math.min(255,n))}if(o!==null){if(isNaN(o))return!1;o=Math.max(0,Math.min(255,o))}var i,e=v(t===null?this.rgb[0]:t,n===null?this.rgb[1]:n,o===null?this.rgb[2]:o);e[0]!==null&&(this.hsv[0]=Math.max(0,Math.min(360,e[0]))),e[2]!==0&&(this.hsv[1]=e[1]===null?null:Math.max(0,this.minS,Math.min(100,this.maxS,e[1]))),this.hsv[2]=e[2]===null?null:Math.max(0,this.minV,Math.min(100,this.maxV,e[2])),i=s(this.hsv[0],this.hsv[1],this.hsv[2]),this.rgb[0]=i[0],this.rgb[1]=i[1],this.rgb[2]=i[2],this.exportColor(a)},this.fromString=function(r,o){if(e=r.match(/^\W*([0-9A-F]{3}([0-9A-F]{3})?)\W*$/i))return e[1].length===6?this.fromRGB(parseInt(e[1].substr(0,2),16),parseInt(e[1].substr(2,2),16),parseInt(e[1].substr(4,2),16),o):this.fromRGB(parseInt(e[1].charAt(0)+e[1].charAt(0),16),parseInt(e[1].charAt(1)+e[1].charAt(1),16),parseInt(e[1].charAt(2)+e[1].charAt(2),16),o),!0;if((e=r.match(/^\W*rgba?\(([^)]*)\)\W*$/i))&&(t=e[1].split(","),n=/^\s*(\d*)(\.\d+)?\s*$/,t.length>=3&&(i=t[0].match(n))&&(s=t[1].match(n))&&(a=t[2].match(n)))){var e,t,n,s,i,a,c=parseFloat((i[1]||"0")+(i[2]||"")),l=parseFloat((s[1]||"0")+(s[2]||"")),d=parseFloat((a[1]||"0")+(a[2]||""));return this.fromRGB(c,l,d,o),!0}return!1},this.toString=function(){return(256|Math.round(this.rgb[0])).toString(16).substr(1)+(256|Math.round(this.rgb[1])).toString(16).substr(1)+(256|Math.round(this.rgb[2])).toString(16).substr(1)},this.toHEXString=function(){return"#"+this.toString().toUpperCase()},this.toRGBString=function(){return"rgb("+Math.round(this.rgb[0])+","+Math.round(this.rgb[1])+","+Math.round(this.rgb[2])+")"},this.isLight=function(){return.213*this.rgb[0]+.715*this.rgb[1]+.072*this.rgb[2]>127.5},this._processParentElementsInDOM=function(){if(this._linkedElementsProcessed)return;this._linkedElementsProcessed=!0;var n,t=this.targetElement;do n=e.getStyle(t),n&&n.position.toLowerCase()==="fixed"&&(this.fixed=!0),t!==this.targetElement&&(t._jscEventsAttached||(e.attachEvent(t,"scroll",e.onParentScroll),t._jscEventsAttached=!0));while((t=t.parentNode)&&!e.isElementType(t,"body"))},typeof n=="string"?(r=n,a=document.getElementById(r),a?this.targetElement=a:e.warn("Could not find target element with ID '"+r+"'")):n?this.targetElement=n:e.warn("Invalid target element: '"+n+"'"),this.targetElement._jscLinkedInstance){e.warn("Cannot link jscolor twice to the same element. Skipping.");return}this.targetElement._jscLinkedInstance=this,this.valueElement=e.fetchElement(this.valueElement),this.styleElement=e.fetchElement(this.styleElement);var a,r,c,d,p,t=this,l=this.container?e.fetchElement(this.container):document.getElementsByTagName("body")[0],f=3;e.isElementType(this.targetElement,"button")&&(this.targetElement.onclick?(p=this.targetElement.onclick,this.targetElement.onclick=function(e){return p.call(this,e),!1}):this.targetElement.onclick=function(){return!1}),this.valueElement&&e.isElementType(this.valueElement,"input")&&(d=function(){t.fromString(t.valueElement.value,e.leaveValue),e.dispatchFineChange(t)},e.attachEvent(this.valueElement,"keyup",d),e.attachEvent(this.valueElement,"input",d),e.attachEvent(this.valueElement,"blur",b),this.valueElement.setAttribute("autocomplete","off")),this.styleElement&&(this.styleElement._jscOrigStyle={backgroundImage:this.styleElement.style.backgroundImage,backgroundColor:this.styleElement.style.backgroundColor,color:this.styleElement.style.color}),this.value?this.fromString(this.value)||this.exportColor():this.importColor()}};return e.jscolor.lookupClass="jscolor",e.jscolor.installByClassName=function(t){var n=document.getElementsByTagName("input"),s=document.getElementsByTagName("button");e.tryInstallOnElements(n,t),e.tryInstallOnElements(s,t)},e.register(),e.jscolor}()),!function(t,e){"function"==typeof define&&define.amd?define(e):"object"==typeof exports?module.exports=e():t.tingle=e()}(this,function(){function e(e){var t={onClose:null,onOpen:null,beforeOpen:null,beforeClose:null,stickyFooter:!1,footer:!1,cssClass:[],closeLabel:"Close",closeMethods:["overlay","button","escape"]};this.opts=d({},t,e),this.init()}function s(){return'<svg viewBox="0 0 10 10" xmlns="http://www.w3.org/2000/svg"><path d="M.3 9.7c.2.2.4.3.7.3.3 0 .5-.1.7-.3L5 6.4l3.3 3.3c.2.2.5.3.7.3.2 0 .5-.1.7-.3.4-.4.4-1 0-1.4L6.4 5l3.3-3.3c.4-.4.4-1 0-1.4-.4-.4-1-.4-1.4 0L5 3.6 1.7.3C1.3-.1.7-.1.3.3c-.4.4-.4 1 0 1.4L3.6 5 .3 8.3c-.4.4-.4 1 0 1.4z" fill="#000" fill-rule="nonzero"/></svg>'}function t(){this.modalBoxFooter&&(this.modalBoxFooter.style.width=this.modalBox.clientWidth+"px",this.modalBoxFooter.style.left=this.modalBox.offsetLeft+"px")}function o(){this.modal=document.createElement("div"),this.modal.classList.add("tingle-modal"),0!==this.opts.closeMethods.length&&-1!==this.opts.closeMethods.indexOf("overlay")||this.modal.classList.add("tingle-modal--noOverlayClose"),this.modal.style.display="none",this.opts.cssClass.forEach(function(e){"string"==typeof e&&this.modal.classList.add(e)},this),-1!==this.opts.closeMethods.indexOf("button")&&(this.modalCloseBtn=document.createElement("button"),this.modalCloseBtn.type="button",this.modalCloseBtn.classList.add("tingle-modal__close"),this.modalCloseBtnIcon=document.createElement("span"),this.modalCloseBtnIcon.classList.add("tingle-modal__closeIcon"),this.modalCloseBtnIcon.innerHTML=s(),this.modalCloseBtnLabel=document.createElement("span"),this.modalCloseBtnLabel.classList.add("tingle-modal__closeLabel"),this.modalCloseBtnLabel.innerHTML=this.opts.closeLabel,this.modalCloseBtn.appendChild(this.modalCloseBtnIcon),this.modalCloseBtn.appendChild(this.modalCloseBtnLabel)),this.modalBox=document.createElement("div"),this.modalBox.classList.add("tingle-modal-box"),this.modalBoxContent=document.createElement("div"),this.modalBoxContent.classList.add("tingle-modal-box__content"),this.modalBox.appendChild(this.modalBoxContent),-1!==this.opts.closeMethods.indexOf("button")&&this.modal.appendChild(this.modalCloseBtn),this.modal.appendChild(this.modalBox)}function i(){this.modalBoxFooter=document.createElement("div"),this.modalBoxFooter.classList.add("tingle-modal-box__footer"),this.modalBox.appendChild(this.modalBoxFooter)}function u(){this._events={clickCloseBtn:this.close.bind(this),clickOverlay:r.bind(this),resize:this.checkOverflow.bind(this),keyboardNav:a.bind(this)},-1!==this.opts.closeMethods.indexOf("button")&&this.modalCloseBtn.addEventListener("click",this._events.clickCloseBtn),this.modal.addEventListener("mousedown",this._events.clickOverlay),window.addEventListener("resize",this._events.resize),document.addEventListener("keydown",this._events.keyboardNav)}function a(e){-1!==this.opts.closeMethods.indexOf("escape")&&27===e.which&&this.isOpen()&&this.close()}function r(e){-1!==this.opts.closeMethods.indexOf("overlay")&&!c(e.target,"tingle-modal")&&e.clientX<this.modal.clientWidth&&this.close()}function c(e,t){for(;(e=e.parentElement)&&!e.classList.contains(t););return e}function l(){-1!==this.opts.closeMethods.indexOf("button")&&this.modalCloseBtn.removeEventListener("click",this._events.clickCloseBtn),this.modal.removeEventListener("mousedown",this._events.clickOverlay),window.removeEventListener("resize",this._events.resize),document.removeEventListener("keydown",this._events.keyboardNav)}function d(){for(var t,e=1;e<arguments.length;e++)for(t in arguments[e])arguments[e].hasOwnProperty(t)&&(arguments[0][t]=arguments[e][t]);return arguments[0]}var n=!1;return e.prototype.init=function(){if(!this.modal)return o.call(this),u.call(this),document.body.insertBefore(this.modal,document.body.firstChild),this.opts.footer&&this.addFooter(),this},e.prototype._busy=function(e){n=e},e.prototype._isBusy=function(){return n},e.prototype.destroy=function(){null!==this.modal&&(this.isOpen()&&this.close(!0),l.call(this),this.modal.parentNode.removeChild(this.modal),this.modal=null)},e.prototype.isOpen=function(){return!!this.modal.classList.contains("tingle-modal--visible")},e.prototype.open=function(){if(!this._isBusy()){this._busy(!0);var e=this;return"function"==typeof e.opts.beforeOpen&&e.opts.beforeOpen(),this.modal.style.removeProperty?this.modal.style.removeProperty("display"):this.modal.style.removeAttribute("display"),this._scrollPosition=window.pageYOffset,document.body.classList.add("tingle-enabled"),document.body.style.top=-this._scrollPosition+"px",this.setStickyFooter(this.opts.stickyFooter),this.modal.classList.add("tingle-modal--visible"),"function"==typeof e.opts.onOpen&&e.opts.onOpen.call(e),e._busy(!1),this.checkOverflow(),this}},e.prototype.close=function(t){if(!this._isBusy()){if(this._busy(!0),t=t||!1,"function"==typeof this.opts.beforeClose&&!this.opts.beforeClose.call(this))return void this._busy(!1);document.body.classList.remove("tingle-enabled"),window.scrollTo(0,this._scrollPosition),document.body.style.top=null,this.modal.classList.remove("tingle-modal--visible");var e=this;e.modal.style.display="none","function"==typeof e.opts.onClose&&e.opts.onClose.call(this),e._busy(!1)}},e.prototype.setContent=function(e){return"string"==typeof e?this.modalBoxContent.innerHTML=e:(this.modalBoxContent.innerHTML="",this.modalBoxContent.appendChild(e)),this.isOpen()&&this.checkOverflow(),this},e.prototype.getContent=function(){return this.modalBoxContent},e.prototype.addFooter=function(){return i.call(this),this},e.prototype.setFooterContent=function(e){return this.modalBoxFooter.innerHTML=e,this},e.prototype.getFooterContent=function(){return this.modalBoxFooter},e.prototype.setStickyFooter=function(e){return this.isOverflow()||(e=!1),e?this.modalBox.contains(this.modalBoxFooter)&&(this.modalBox.removeChild(this.modalBoxFooter),this.modal.appendChild(this.modalBoxFooter),this.modalBoxFooter.classList.add("tingle-modal-box__footer--sticky"),t.call(this),this.modalBoxContent.style["padding-bottom"]=this.modalBoxFooter.clientHeight+20+"px"):this.modalBoxFooter&&(this.modalBox.contains(this.modalBoxFooter)||(this.modal.removeChild(this.modalBoxFooter),this.modalBox.appendChild(this.modalBoxFooter),this.modalBoxFooter.style.width="auto",this.modalBoxFooter.style.left="",this.modalBoxContent.style["padding-bottom"]="",this.modalBoxFooter.classList.remove("tingle-modal-box__footer--sticky"))),this},e.prototype.addFooterBtn=function(n,t,s){var e=document.createElement("button");return e.innerHTML=n,e.addEventListener("click",s),"string"==typeof t&&t.length&&t.split(" ").forEach(function(t){e.classList.add(t)}),this.modalBoxFooter.appendChild(e),e},e.prototype.resize=function(){console.warn("Resize is deprecated and will be removed in version 1.0")},e.prototype.isOverflow=function(){var e=window.innerHeight;return this.modalBox.clientHeight>=e},e.prototype.checkOverflow=function(){this.modal.classList.contains("tingle-modal--visible")&&(this.isOverflow()?this.modal.classList.add("tingle-modal--overflow"):this.modal.classList.remove("tingle-modal--overflow"),!this.isOverflow()&&this.opts.stickyFooter?this.setStickyFooter(!1):this.isOverflow()&&this.opts.stickyFooter&&(t.call(this),this.setStickyFooter(!0)))},{modal:e}}),!function(t,e){"object"==typeof exports&&"undefined"!=typeof module?module.exports=e(require("jquery")):"function"==typeof define&&define.amd?define(["jquery"],e):t.mobiscroll=e(t.jQuery)}(this,function(l){"use strict";function el(t,e,o){var n=t;return"object"===("undefined"==typeof e?"undefined":q(e))?t.each(function(){new e.component(this,e)}):("string"==typeof e&&t.each(function(){var t,i=s.instances[this.id];if(i&&i[e]&&(t=i[e].apply(this,Array.prototype.slice.call(o,1)),void 0!==t))return n=t,!1}),n)}function eu(e,t,n){eh[e]=function(s){return el(this,e3(s,{component:t,preset:n===!1?void 0:e}),arguments)}}function n(){}function N(e){var t,n=[];for(t in e)n.push(e[t]);return n}function f(t){var e,n={};if(t)for(e=0;e<t.length;e++)n[t[e]]=t[e];return n}function h(e){return e-parseFloat(e)>=0}function d(e){return"string"==typeof e}function c(e,t,n){return Math.max(t,Math.min(e,n))}function a(e,t){for(e+="",t=t||2;e.length<t;)e="0"+e;return e}function ew(e){"vibrate"in navigator&&navigator.vibrate(e||50)}function k(){C++,setTimeout(function(){C--},500)}function eE(s,e){if(!e.mbscClick){var t=(s.originalEvent||s).changedTouches[0],n=document.createEvent("MouseEvents");n.initMouseEvent("click",!0,!0,window,1,t.screenX,t.screenY,t.clientX,t.clientY,!1,!1,!1,!1,0,null),n.isMbscTap=!0,n.isIonicTap=!0,W=!0,e.mbscChange=!0,e.mbscClick=!0,e.dispatchEvent(n),W=!1,k(),setTimeout(function(){delete e.mbscClick})}}function o(t,s,o){var e=t.originalEvent||t,n=(o?"page":"client")+s;return e.targetTouches&&e.targetTouches[0]?e.targetTouches[0][n]:e.changedTouches&&e.changedTouches[0]?e.changedTouches[0][n]:t[n]}function eD(i,g,p,r,n,f){function m(n){e||(r&&n.preventDefault(),e=this,l=o(n,"X"),c=o(n,"Y"),t=!1,a=new Date)}function h(s){e&&!t&&(Math.abs(o(s,"X")-l)>n||Math.abs(o(s,"Y")-c)>n)&&(t=!0)}function b(n){e&&(f&&new Date-a<100||!t?eE(n,n.target):k(),e=!1)}function u(){e=!1}var e,t,a,c,l,v=s.$,d=v(g);n=n||9,i.settings.tap&&d.on("touchstart.mbsc",m).on("touchcancel.mbsc",u).on("touchmove.mbsc",h).on("touchend.mbsc",b),d.on("click.mbsc",function(e){r&&e.preventDefault(),p.call(this,e,i)})}function eT(e){if(C&&!W&&!e.isMbscTap&&("TEXTAREA"!=e.target.nodeName||"mousedown"!=e.type))return e.stopPropagation(),e.preventDefault(),!1}function e$(n,s){var e=document.createElement("script"),t="mbscjsonp"+ ++e1;window[t]=function(n){e.parentNode.removeChild(e),delete window[t],n&&s(n)},e.src=n+(n.indexOf("?")>=0?"&":"?")+"callback="+t,document.body.appendChild(e)}function e5(t,n){var e=new XMLHttpRequest;e.open("GET",t,!0),e.onload=function(){this.status>=200&&this.status<400&&n(JSON.parse(this.response))},e.onerror=function(){},e.send()}function en(e,t,n){"jsonp"==n?e$(e,t):e5(e,t)}function eR(e){var t;for(t in e)if(void 0!==E[e[t]])return!0;return!1}function ej(){var e,t=["Webkit","Moz","O","ms"];for(e in t)if(eR([t[e]+"Transform"]))return"-"+t[e].toLowerCase()+"-";return""}function em(n,t){if("touchstart"==n.type)e(t).attr("data-touch","1");else if(e(t).attr("data-touch"))return e(t).removeAttr("data-touch"),!1;return!0}function ef(s,o){var t,i,n=getComputedStyle(s[0]);return e.each(["t","webkitT","MozT","OT","msT"],function(s,e){if(void 0!==n[e+"ransform"])return t=n[e+"ransform"],!1}),t=t.split(")")[0].split(", "),i=o?t[13]||t[5]:t[12]||t[4]}function e6(t){if(t){if(P[t])return P[t];var s=e('<div style="background-color:'+t+';"></div>').appendTo("body"),i=getComputedStyle(s[0]),n=i.backgroundColor.replace(/rgb|rgba|\(|\)|\s/g,"").split(","),a=.299*n[0]+.587*n[1]+.114*n[2],o=a<130?"#fff":"#000";return s.remove(),P[t]=o,o}}function ep(f,y,j,x,O,d){function b(n){var l,t=e(this),i=+t.attr("data-step"),a=+t.attr("data-index"),r=!0;O&&n.stopPropagation(),"touchstart"==n.type&&t.closest(".mbsc-no-touch").removeClass("mbsc-no-touch"),"mousedown"==n.type&&n.preventDefault(),"keydown"!=n.type?(_=o(n,"X"),p=o(n,"Y"),l=em(n,this)):l=32===n.keyCode,s||!l||t.hasClass("mbsc-disabled")||(v(a,i,n)&&(t.addClass("mbsc-active"),d&&d.addRipple(t.find(".mbsc-segmented-content"),n)),"mousedown"==n.type&&e(document).on("mousemove",c).on("mouseup",u))}function c(e){(Math.abs(_-o(e,"X"))>7||Math.abs(p-o(e,"Y"))>7)&&(r=!0,l())}function u(t){"touchend"==t.type&&t.preventDefault(),l(),"mouseup"==t.type&&e(document).off("mousemove",c).off("mouseup",u)}function l(){s=!1,clearInterval(m),t&&(t.removeClass("mbsc-active"),d&&setTimeout(function(){d.removeRipple()},100))}function v(e,t,n){return s||C(e)||(a=e,i=t,h=n,s=!0,r=!1,setTimeout(g,100)),s}function g(){return t&&t.hasClass("mbsc-disabled")?void l():(!s&&r||(r=!0,y(a,i,h,g)),void(s&&j&&(clearInterval(m),m=setInterval(function(){y(a,i,h)},j))))}function w(){f.off("touchstart mousedown keydown",b).off("touchmove",c).off("touchend touchcancel keyup",u)}var t,s,i,a,r,h,m,p,_,C=x||n;return f.on("touchstart mousedown keydown",b).on("touchmove",c).on("touchend touchcancel keyup",u),{start:v,stop:l,destroy:w}}function ek(h,m,r){function u(){s.style.width="100000px",s.style.height="100000px",e.scrollLeft=1e5,e.scrollTop=1e5,n.scrollLeft=1e5,n.scrollTop=1e5}function o(){var t=new Date,a=0;c||(t-l>200&&!e.scrollTop&&!e.scrollLeft&&(l=t,u()),a||(a=j(o)))}function d(){i||(i=j(f))}function f(){i=0,u(),m()}var e=void 0,s=void 0,a=void 0,i=void 0,n=void 0,c=void 0,l=0,t=document.createElement("div");return t.innerHTML=e9,t.dir="ltr",n=t.childNodes[1],e=t.childNodes[0],s=e.childNodes[0],h.appendChild(t),e.addEventListener("scroll",d),n.addEventListener("scroll",d),r?r.runOutsideAngular(function(){j(o)}):j(o),{detach:function(){h.removeChild(t),c=!0}}}function H(e){return(e+"").replace('"',"___")}function m(n,r,s,t,o,i,a){var e=new Date(n,r,s,t||0,o||0,i||0,a||0);return 23==e.getHours()&&0===(t||0)&&e.setHours(e.getHours()+2),e}function v(i,n,h){if(!n)return null;var o,c,l,s=t({},J,h),r=function(t){for(var e=0;o+1<i.length&&i.charAt(o+1)==t;)e++,o++;return e},a=function(t,n,s){var e=""+n;if(r(t))for(;e.length<s;)e="0"+e;return e},u=function(t,e,n,s){return r(t)?s[e]:n[e]},e="",d=!1;for(o=0;o<i.length;o++)if(d)"'"!=i.charAt(o)||r("'")?e+=i.charAt(o):d=!1;else switch(i.charAt(o)){case"d":e+=a("d",s.getDay(n),2);break;case"D":e+=u("D",n.getDay(),s.dayNamesShort,s.dayNames);break;case"o":e+=a("o",(n.getTime()-new Date(n.getFullYear(),0,0).getTime())/864e5,3);break;case"m":e+=a("m",s.getMonth(n)+1,2);break;case"M":e+=u("M",s.getMonth(n),s.monthNamesShort,s.monthNames);break;case"y":l=s.getYear(n),e+=r("y")?l:(l%100<10?"0":"")+l%100;break;case"h":c=n.getHours(),e+=a("h",c>12?c-12:0===c?12:c,2);break;case"H":e+=a("H",n.getHours(),2);break;case"i":e+=a("i",n.getMinutes(),2);break;case"s":e+=a("s",n.getSeconds(),2);break;case"a":e+=n.getHours()>11?s.pmText:s.amText;break;case"A":e+=n.getHours()>11?s.pmText.toUpperCase():s.amText.toUpperCase();break;case"'":r("'")?e+="'":d=!0;break;default:e+=i.charAt(o)}return e}function L(u,n,x){if(e=t({},J,x),a=i(e.defaultValue||new Date),!u||!n)return a;if(n.getTime)return n;n="object"==("undefined"==typeof n?"undefined":q(n))?n.toString():n+"";var e,a,r,h,p,y=e.shortYearCutoff,l=e.getYear(a),d=e.getMonth(a)+1,c=e.getDay(a),_=-1,s=a.getHours(),O=a.getMinutes(),w=0,f=-1,b=!1,v=function(t){var e=r+1<u.length&&u.charAt(r+1)==t;return e&&r++,e},o=function(e){v(e);var s="@"==e?14:"!"==e?20:"y"==e?4:"o"==e?3:2,o=new RegExp("^\\d{1,"+s+"}"),t=n.substr(m).match(o);return t?(m+=t[0].length,parseInt(t[0],10)):0},g=function(s,o,i){var e,t=v(s)?i:o;for(e=0;e<t.length;e++)if(n.substr(m,t[e].length).toLowerCase()==t[e].toLowerCase())return m+=t[e].length,e+1;return 0},j=function(){m++},m=0;for(r=0;r<u.length;r++)if(b)"'"!=u.charAt(r)||v("'")?j():b=!1;else switch(u.charAt(r)){case"d":c=o("d");break;case"D":g("D",e.dayNamesShort,e.dayNames);break;case"o":_=o("o");break;case"m":d=o("m");break;case"M":d=g("M",e.monthNamesShort,e.monthNames);break;case"y":l=o("y");break;case"H":s=o("H");break;case"h":s=o("h");break;case"i":O=o("i");break;case"s":w=o("s");break;case"a":f=g("a",[e.amText,e.pmText],[e.amText,e.pmText])-1;break;case"A":f=g("A",[e.amText,e.pmText],[e.amText,e.pmText])-1;break;case"'":v("'")?j():b=!0;break;default:j()}if(l<100&&(l+=(new Date).getFullYear()-(new Date).getFullYear()%100+(l<=("string"!=typeof y?y:(new Date).getFullYear()%100+parseInt(y,10))?0:-100)),_>-1){d=1,c=_;do p=32-new Date(l,d-1,32,12).getDate(),c>p&&(d++,c-=p);while(c>p)}return s=f==-1?s:f&&s<12?s+12:f||12!=s?s:0,h=e.getDate(l,d-1,c,s,O,w),e.getYear(h)!=l||e.getMonth(h)+1!=d||e.getDay(h)!=c?a:h}function e8(e,t){return Math.round((t-e)/864e5)}function w(e){return m(e.getFullYear(),e.getMonth(),e.getDate())}function ec(e){return e.getFullYear()+"-"+(e.getMonth()+1)+"-"+e.getDate()}function eN(t,e){var n="",s="";return t&&(e.h&&(s+=a(t.getHours())+":"+a(t.getMinutes()),e.s&&(s+=":"+a(t.getSeconds())),e.u&&(s+="."+a(t.getMilliseconds(),3)),e.tz&&(s+=e.tz)),e.y?(n+=t.getFullYear(),e.m&&(n+="-"+a(t.getMonth()+1),e.d&&(n+="-"+a(t.getDate())),e.h&&(n+="T"+s))):e.h&&(n=s)),n}function e4(o,i,n){var e,t,s={y:1,m:2,d:3,h:4,i:5,s:6,u:7,tz:8};if(n)for(e in s)t=o[s[e]-i],t&&(n[e]="tz"==e?t:1)}function X(e,t,o){var s=window.moment||t.moment,n=t.returnFormat;if(e){if("moment"==n&&s)return s(e);if("locale"==n)return v(o,e,t);if("iso8601"==n)return eN(e,t.isoParts)}return e}function i(t,s,o,n){var e;return t?t.getTime?t:t.toDate?t.toDate():("string"==typeof t&&(t=t.trim()),(e=ed.exec(t))?(e4(e,2,n),new Date(1970,0,1,e[2]?+e[2]:0,e[3]?+e[3]:0,e[4]?+e[4]:0,e[5]?+e[5]:0)):(e||(e=ez.exec(t)),e?(e4(e,0,n),new Date(e[1]?+e[1]:1970,e[2]?e[2]-1:0,e[3]?+e[3]:1,e[4]?+e[4]:0,e[5]?+e[5]:0,e[6]?+e[6]:0,e[7]?+e[7]:0)):L(s,t,o))):null}function R(e,t){return e.getFullYear()==t.getFullYear()&&e.getMonth()==t.getMonth()&&e.getDate()==t.getDate()}function eA(e){return e[0].innerWidth||e.innerWidth()}l=l&&l.hasOwnProperty("default")?l.default:l;var s=s||{},I={},eS={},q="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e},e3=l.extend,eh={};s.$=l,l.mobiscroll=s,l.fn.mobiscroll=function(e){return e3(this,eh),el(this,e,arguments)};var r,u,O,Y,D=[],b="undefined"!=typeof window,x=b?navigator.userAgent:"",eF=/Safari/.test(x),T=x.match(/Android|iPhone|iPad|iPod|Windows Phone|Windows|MSIE/i),j=b&&window.requestAnimationFrame||function(e){return setTimeout(e,20)},er=b&&window.cancelAnimationFrame||function(e){clearTimeout(e)};/Android/i.test(T)?(r="android",O=x.match(/Android\s+([\d.]+)/i),O&&(D=O[0].replace("Android ","").split("."))):/iPhone|iPad|iPod/i.test(T)?(r="ios",O=x.match(/OS\s+([\d_]+)/i),O&&(D=O[0].replace(/_/g,".").replace("OS ","").split("."))):/Windows Phone/i.test(T)?r="wp":/Windows|MSIE/i.test(T)&&(r="windows"),u=D[0],Y=D[1],C=0,W=void 0,b&&(["mouseover","mousedown","mouseup","click"].forEach(function(e){document.addEventListener(e,eT,!0)}),"android"==r&&u<5&&document.addEventListener("change",function(e){C&&"checkbox"==e.target.type&&!e.target.mbscChange&&(e.stopPropagation(),e.preventDefault()),delete e.target.mbscChange},!0)),s.uid="c5d09426";var y,e=s.$,eP=+new Date,S={},ee={},eL={xsmall:0,small:576,medium:768,large:992,xlarge:1200},t=e.extend;t(I,{getCoord:o,preventClick:k,vibrate:ew}),y=t(s,{$:e,version:"4.8.2",autoTheme:"mobiscroll",themes:{form:{},page:{},frame:{},scroller:{},listview:{},navigation:{},progress:{},card:{}},platform:{name:r,majorVersion:u,minorVersion:Y},i18n:{},instances:S,classes:ee,util:I,settings:{},setDefaults:function(e){t(this.settings,e)},customTheme:function(a,o){var e,i=s.themes,n=["frame","scroller","listview","navigation","form","page","progress","card"];for(e=0;e<n.length;e++)i[n[e]][a]=t({},i[n[e]][o],{baseTheme:o})}}),Z=function(a,i){function m(s){var t,n;return o.responsive&&(n=s||f.offsetWidth,e.each(o.responsive,function(s,e){n>=(e.breakpoint||eL[s])&&(t=e)})),t}function g(){e(a).addClass("mbsc-comp"),a.id?S[a.id]&&S[a.id].destroy():a.id="mobiscroll"+ ++eP,S[a.id]=s,s.__ready=!0}var o,r,c,l,d,u,h,f,p,s=this;s.settings={},s.element=a,s._init=n,s._destroy=n,s._processSettings=n,s._checkResp=function(t){if(s&&s._responsive){var e=m(t);if(c!==e)return c=e,s.init({}),!0}},s.init=function(n,g){var v,b;n&&s.getVal&&(b=s.getVal());for(v in s.settings)delete s.settings[v];o=s.settings,t(i,n),s._hasDef&&(d=y.settings),t(o,s._defaults,d,i),s._hasTheme&&(r=o.theme,"auto"!=r&&r||(r=y.autoTheme),"default"==r&&(r="mobiscroll"),i.theme=r,h=y.themes[s._class]?y.themes[s._class][r]:{}),s._hasLang&&(p=y.i18n[o.lang]),t(o,h,p,d,i),f=e(o.context)[0],s._responsive&&(c||(c=m()),t(o,c)),s._processSettings(c||{}),s._presets&&(l=s._presets[o.preset],l&&(l=l.call(a,s,i),t(o,l,i,c))),s._init(n),n&&s.setVal&&s.setVal(void 0===g?b:g,!0),u("onInit")},s.destroy=function(){s&&(s._destroy(),u("onDestroy"),delete S[a.id],s=null)},s.tap=function(e,t,n,o,i){eD(s,e,t,n,o,i)},s.trigger=function(n,r){var e,t,o,c=[d,h,l,i];for(e=0;e<4;e++)t=c[e],t&&t[n]&&(o=t[n].call(a,r||{},s));return o},s.option=function(e,a,r){var t={},n=["data","invalid","valid","readonly"];/calendar|eventcalendar|range/.test(o.preset)&&n.push("marked","labels","colors"),"object"===("undefined"==typeof e?"undefined":q(e))?t=e:t[e]=a,n.forEach(function(e){i[e]=o[e]}),s.init(t,r)},s.getInst=function(){return s},i=i||{},u=s.trigger,s.__ready||g()},e1=0,I.getJson=en,P={},b&&(E=document.createElement("modernizr").style,_=ej(),p=_.replace(/^-/,"").replace(/-$/,"").replace("moz","Moz"),M=void 0!==E.animation?"animationend":"webkitAnimationEnd",eb=void 0!==E.transition,e7=void 0===E.touchAction||"ios"==r&&!eF&&(u<12||12==u&&Y<2));var p,g,_,C,E,M,F,P,B,V,W,Z,e1,e7,eb,K="position:absolute;left:0;top:0;",eg=K+"right:0;bottom:0;overflow:hidden;z-index:-1;",e9='<div style="'+eg+'"><div style="'+K+'"></div></div><div style="'+eg+'"><div style="'+K+'width:200%;height:200%;"></div></div>',et=s.themes,ev=/(iphone|ipod)/i.test(x)&&u>=7,eo="android"==r,U="ios"==r,eC=U&&8==u,ey=U&&u>7,e_=function(e){e.preventDefault()},e2="input,select,textarea,button",eO='textarea,button,input[type="button"],input[type="submit"]',ex=e2+',[tabindex="0"]',$=function($,S,er){function ec(t){j&&j.removeClass("mbsc-active"),j=e(this),j.hasClass("mbsc-disabled")||j.hasClass("mbsc-fr-btn-nhl")||j.addClass("mbsc-active"),"mousedown"===t.type?e(document).on("mouseup",A):"pointerdown"===t.type&&e(document).on("pointerup",A)}function A(t){j&&(j.removeClass("mbsc-active"),j=null),"mouseup"===t.type?e(document).off("mouseup",A):"pointerup"===t.type&&e(document).off("pointerup",A)}function G(n){s.activeInstance==t&&(13!=n.keyCode||e(n.target).is(eO)&&!n.shiftKey?27==n.keyCode&&t.cancel():t.select())}function ea(e){e||eo||!t._activeElm||(H=new Date,t._activeElm.focus())}function Q(o){var s=V,n=i.focusOnClose;t._markupRemove(),a.remove(),r&&(l.mbscModals--,i.scrollLock&&l.mbscLock--,l.mbscLock||C.removeClass("mbsc-fr-lock"),O&&(l.mbscIOSLock--,l.mbscIOSLock||(C.removeClass("mbsc-fr-lock-ios"),y.css({top:"",left:""}),v.scrollLeft(l.mbscScrollLeft),v.scrollTop(l.mbscScrollTop))),l.mbscModals||C.removeClass("mbsc-fr-lock-ctx"),l.mbscModals&&!P||o||(s||(s=h),setTimeout(function(){void 0===n||n===!0?(F=!0,s[0].focus()):n&&e(n)[0].focus()},200))),P=void 0,W=!1,m("onHide")}function el(){clearTimeout(X),X=setTimeout(function(){t.position(!0)&&(w.style.visibility="hidden",w.offsetHeight,w.style.visibility="")},200)}function J(e){s.activeInstance==t&&e.target.nodeType&&!Y.contains(e.target)&&new Date-H>100&&(H=new Date,t._activeElm.focus())}function ee(c,s){function o(){a.off(M,o).removeClass("mbsc-anim-in mbsc-anim-trans mbsc-anim-trans-"+u).find(".mbsc-fr-popup").removeClass("mbsc-anim-"+u),ea(s)}if(r)a.appendTo(y);else if(h.is("div")&&!t._hasContent)h.empty().append(a);else if(h.hasClass("mbsc-control")){var n=h.closest(".mbsc-control-w");a.insertAfter(n),n.hasClass("mbsc-select")&&n.addClass("mbsc-select-inline")}else a.insertAfter(h);W=!0,t._markupInserted(a),m("onMarkupInserted",{target:g}),a.on("mousedown",".mbsc-btn-e,.mbsc-fr-btn-e",e_).on("touchstart mousedown",function(e){i.stopProp&&e.stopPropagation()}).on("keydown",".mbsc-fr-btn-e",function(e){32==e.keyCode&&(e.preventDefault(),e.stopPropagation(),this.click())}).on("keydown",function(t){if(32!=t.keyCode||e(t.target).is(e2)){if(9==t.keyCode&&r&&i.focusTrap){var n=a.find(ex).filter(function(){return this.offsetWidth>0||this.offsetHeight>0}),c=n.index(e(":focus",a)),s=n.length-1,o=0;t.shiftKey&&(s=0,o=-1),c===s&&(n.eq(o)[0].focus(),t.preventDefault())}}else t.preventDefault()}).on("touchstart mousedown pointerdown",".mbsc-fr-btn-e",ec).on("touchend",".mbsc-fr-btn-e",A),g.addEventListener("touchstart",function(){z||(z=!0,y.find(".mbsc-no-touch").removeClass("mbsc-no-touch"))},!0),e.each(p,function(s,n){t.tap(e(".mbsc-fr-btn"+s,a),function(e){n=d(n)?t.buttons[n]:n,(d(n.handler)?t.handlers[n.handler]:n.handler).call(this,e,t)},!0)}),t._attachEvents(a),t.position()!==!1&&((r||t._checkSize)&&(L=ek(g,el,i.zone)),r&&(a.removeClass("mbsc-fr-pos"),u&&!c?a.addClass("mbsc-anim-in mbsc-anim-trans mbsc-anim-trans-"+u).on(M,o).find(".mbsc-fr-popup").addClass("mbsc-anim-"+u):ea(s)),m("onShow",{target:g,valueText:t._tempValue}))}function D(e,n){t._isVisible||(e&&e(),t.show()!==!1&&(V=n))}function et(){t._fillValue(),m("onSet",{valueText:t._value})}function en(){m("onCancel",{valueText:t._value})}function es(){t.setVal(null,!0)}var i,a,r,l,u,m,f,p,g,v,b,j,y,_,w,O,x,C,E,T,z,N,L,P,I,B,W,U,K,q,Y,X,ei,t=this,h=e($),R=[],H=new Date;Z.call(this,$,S,!0),t.position=function(Y){var n,s,d,p,j,O,C,k,A,F,z,D,L,R,P,H,V,$,q,I={},u=0,o=0,S=0,M=0;if(!W)return!1;if(V=E,$=T,n=g.offsetHeight,s=g.offsetWidth,s&&n&&(E!==s||T!==n||!Y)){if(t._checkResp(s))return!1;if(E=s,T=n,t._isFullScreen||/top|bottom/.test(i.display)?x.width(s):r&&K.width(""),t._position(a),!t._isFullScreen&&/center|bubble/.test(i.display)&&(e(".mbsc-w-p",a).each(function(){A=this.getBoundingClientRect().width,M+=A,S=A>S?A:S}),O=M>s-16||i.tabs===!0,K.css({width:t._isLiquid?Math.min(i.maxPopupWidth,s-16):Math.ceil(O?S:M),"white-space":O?"":"nowrap"})),m("onPosition",{target:g,popup:w,hasTabs:O,oldWidth:V,oldHeight:$,windowWidth:s,windowHeight:n})!==!1&&r)return N&&(u=v.scrollLeft(),o=v.scrollTop(),E&&B.css({width:"",height:""})),_=w.offsetWidth,f=w.offsetHeight,U=f<=n&&_<=s,"center"==i.display?(p=Math.max(0,u+(s-_)/2),d=Math.max(0,o+(n-f)/2)):"bubble"==i.display?(C=void 0===i.anchor?h:e(i.anchor),D=e(".mbsc-fr-arr-i",a)[0],R=C.offset(),j=R.top+(b?o-y.offset().top:0),L=R.left+(b?u-y.offset().left:0),H=C[0].offsetWidth,P=C[0].offsetHeight,z=D.offsetWidth,k=D.offsetHeight,p=c(L-(_-H)/2,u+3,u+s-_-3),d=j+P+k/2,d+f+8>o+n&&j-f-k/2>o?(x.removeClass("mbsc-fr-bubble-bottom").addClass("mbsc-fr-bubble-top"),d=j-f-k/2):x.removeClass("mbsc-fr-bubble-top").addClass("mbsc-fr-bubble-bottom"),e(".mbsc-fr-arr",a).css({left:c(L+H/2-(p+(_-z)/2),0,z)}),U=d>o&&p>u&&d+f<=o+n&&p+_<=u+s):(p=u,d="top"==i.display?o:Math.max(0,o+n-f)),N&&(F=Math.max(d+f,b?l.scrollHeight:e(document).height()),q=Math.max(p+_,b?l.scrollWidth:e(document).width()),B.css({width:q,height:F}),i.scroll&&"bubble"==i.display&&(d+f+8>o+n||j>o+n||j+P<o)&&v.scrollTop(Math.min(j,d+f-n+8,F-n))),I.top=Math.floor(d),I.left=Math.floor(p),x.css(I),!0}},t.attachShow=function(a,o){var s,n=e(a).off(".mbsc"),r=n.prop("readonly");"inline"!==i.display&&((i.showOnFocus||i.showOnTap)&&n.is("input,select")&&(n.prop("readonly",!0).on("mousedown.mbsc",function(e){e.preventDefault()}).on("focus.mbsc",function(){t._isVisible&&this.blur()}),s=e('label[for="'+n.attr("id")+'"]'),s.length||(s=n.closest("label"))),n.is("select")||(i.showOnFocus&&n.on("focus.mbsc",function(){F?F=!1:D(o,n)}),i.showOnTap&&(n.on("keydown.mbsc",function(e){32!=e.keyCode&&13!=e.keyCode||(e.preventDefault(),e.stopPropagation(),D(o,n))}),t.tap(n,function(e){e.isMbscTap&&(z=!0),D(o,n)}),s&&s.length&&t.tap(s,function(e){e.preventDefault(),e.target!==n[0]&&D(o,n)}))),R.push({readOnly:r,el:n,lbl:s}))},t.select=function(){r?t.hide(!1,"set",!1,et):et()},t.cancel=function(){r?t.hide(!1,"cancel",!1,en):en()},t.clear=function(){t._clearValue(),m("onClear"),r&&t._isVisible&&!t.live?t.hide(!1,"clear",!1,es):es()},t.enable=function(){i.disabled=!1,e.each(R,function(t,e){e.el.is("input,select")&&(e.el[0].disabled=!1)})},t.disable=function(){i.disabled=!0,e.each(R,function(t,e){e.el.is("input,select")&&(e.el[0].disabled=!0)})},t.show=function(S,L){if(!i.disabled&&!t._isVisible){if(t._readValue(),m("onBeforeShow")===!1)return!1;if(V=null,u=i.animate,p=i.buttons||[],N=b||"bubble"==i.display,O=ev&&!N&&i.scrollLock,_=p.length>0,u!==!1&&("top"==i.display?u=u||"slidedown":"bottom"==i.display?u=u||"slideup":"center"!=i.display&&"bubble"!=i.display||(u=u||"pop")),r&&(E=0,T=0,O&&!C.hasClass("mbsc-fr-lock-ios")&&(l.mbscScrollTop=M=Math.max(0,v.scrollTop()),l.mbscScrollLeft=D=Math.max(0,v.scrollLeft()),y.css({top:-M+"px",left:-D+"px"})),C.addClass((i.scrollLock?"mbsc-fr-lock":"")+(O?" mbsc-fr-lock-ios":"")+(b?" mbsc-fr-lock-ctx":"")),e(document.activeElement).is("input,textarea")&&document.activeElement.blur(),P=s.activeInstance,s.activeInstance=t,l.mbscModals=(l.mbscModals||0)+1,O&&(l.mbscIOSLock=(l.mbscIOSLock||0)+1),i.scrollLock&&(l.mbscLock=(l.mbscLock||0)+1)),c='<div lang="'+i.lang+'" class="mbsc-fr mbsc-'+i.theme+(i.baseTheme?" mbsc-"+i.baseTheme:"")+" mbsc-fr-"+i.display+" "+(i.cssClass||"")+" "+(i.compClass||"")+(t._isLiquid?" mbsc-fr-liq":"")+(r?" mbsc-fr-pos"+(i.showOverlay?"":" mbsc-fr-no-overlay"):"")+(I?" mbsc-fr-pointer":"")+(ey?" mbsc-fr-hb":"")+(z?"":" mbsc-no-touch")+(O?" mbsc-platform-ios":"")+(_?p.length>=3?" mbsc-fr-btn-block ":"":" mbsc-fr-nobtn")+'">'+(r?'<div class="mbsc-fr-persp">'+(i.showOverlay?'<div class="mbsc-fr-overlay"></div>':"")+'<div role="dialog" class="mbsc-fr-scroll">':"")+'<div class="mbsc-fr-popup'+(i.rtl?" mbsc-rtl":" mbsc-ltr")+(i.headerText?" mbsc-fr-has-hdr":"")+'">'+("bubble"===i.display?'<div class="mbsc-fr-arr-w"><div class="mbsc-fr-arr-i"><div class="mbsc-fr-arr"></div></div></div>':"")+(r?'<div class="mbsc-fr-focus" tabindex="-1"></div>':"")+'<div class="mbsc-fr-w">'+(i.headerText?'<div class="mbsc-fr-hdr">'+(d(i.headerText)?i.headerText:"")+"</div>":"")+'<div class="mbsc-fr-c">',c+=t._generateContent(),c+="</div>",_){var n,c,h,f,j,_,A,M,F,D,R,H=p.length;for(c+='<div class="mbsc-fr-btn-cont">',f=0;f<p.length;f++)A=i.btnReverse?H-f-1:f,n=p[A],n=d(n)?t.buttons[n]:n,"set"===n.handler&&(n.parentClass="mbsc-fr-btn-s"),"cancel"===n.handler&&(n.parentClass="mbsc-fr-btn-c"),c+="<div"+(i.btnWidth?' style="width:'+100/p.length+'%"':"")+' class="mbsc-fr-btn-w '+(n.parentClass||"")+'"><div tabindex="0" role="button" class="mbsc-fr-btn'+A+" mbsc-fr-btn-e "+(void 0===n.cssClass?i.btnClass:n.cssClass)+(n.icon?" mbsc-ic mbsc-ic-"+n.icon:"")+'">'+(n.text||"")+"</div></div>";c+="</div>"}(c+="</div></div></div></div>"+(r?"</div></div>":""),a=e(c),B=e(".mbsc-fr-persp",a),q=e(".mbsc-fr-scroll",a),K=e(".mbsc-fr-w",a),x=e(".mbsc-fr-popup",a),ei=e(".mbsc-fr-hdr",a),g=a[0],Y=q[0],w=x[0],t._activeElm=e(".mbsc-fr-focus",a)[0],t._markup=a,t._isVisible=!0,t.markup=g,t._markupReady(a),m("onMarkupReady",{target:g}),r&&(e(window).on("keydown",G),i.scrollLock&&a.on("touchmove mousewheel wheel",function(e){U&&e.preventDefault()}),i.focusTrap&&v.on("focusin",J),i.closeOnOverlayTap))&&q.on("touchstart mousedown",function(e){h||e.target!=Y||(h=!0,j=!1,F=o(e,"X"),R=o(e,"Y"))}).on("touchmove mousemove",function(e){h&&!j&&(Math.abs(o(e,"X")-F)>9||Math.abs(o(e,"Y")-R)>9)&&(j=!0)}).on("touchcancel",function(){h=!1}).on("touchend click",function(e){h&&!j&&(t.cancel(),"touchend"==e.type&&k()),h=!1}),r&&O?setTimeout(function(){ee(S,L)},100):ee(S,L)}},t.hide=function(n,o,i,c){function l(){a.off(M,l),Q(n)}return!(!t._isVisible||!i&&!t._isValid&&"set"==o||!i&&m("onBeforeClose",{valueText:t._tempValue,button:o})===!1)&&(t._isVisible=!1,L&&(L.detach(),L=null),r&&(e(document.activeElement).is("input,textarea")&&w.contains(document.activeElement)&&document.activeElement.blur(),s.activeInstance==t&&(s.activeInstance=P),e(window).off("keydown",G),v.off("focusin",J)),a&&(r&&u&&!n?a.addClass("mbsc-anim-out mbsc-anim-trans mbsc-anim-trans-"+u).on(M,l).find(".mbsc-fr-popup").addClass("mbsc-anim-"+u):Q(n),t._detachEvents(a)),c&&c(),h.trigger("blur"),void m("onClose",{valueText:t._value}))},t.isVisible=function(){return t._isVisible},t.setVal=n,t.getVal=n,t._generateContent=n,t._attachEvents=n,t._detachEvents=n,t._readValue=n,t._clearValue=n,t._fillValue=n,t._markupReady=n,t._markupInserted=n,t._markupRemove=n,t._position=n,t.__processSettings=n,t.__init=n,t.__destroy=n,t._destroy=function(){t.hide(!0,!1,!0),h.off(".mbsc"),e.each(R,function(t,e){e.el.off(".mbsc").prop("readonly",e.readOnly),e.lbl&&e.lbl.off(".mbsc")}),t.__destroy()},t._updateHeader=function(){var e=i.headerText,n=e?"function"==typeof e?e.call($,t._tempValue):e.replace(/\{value\}/i,t._tempValue):"";ei.html(n||"&nbsp;")},t._processSettings=function(n){var s,o;for(t.__processSettings(n),I=!i.touchUi,I&&(i.display=n.display||S.display||"bubble",i.buttons=n.buttons||S.buttons||[],i.showOverlay=n.showOverlay||S.showOverlay||!1),i.buttons=i.buttons||("inline"!==i.display?["cancel","set"]:[]),i.headerText=void 0===i.headerText?"inline"!==i.display&&"{value}":i.headerText,p=i.buttons||[],r="inline"!==i.display,b="body"!=i.context,y=e(i.context),C=b?y:e("body,html"),l=y[0],t._$window=v=e(b?i.context:window),t.live=!0,o=0;o<p.length;o++)s=p[o],"ok"!=s&&"set"!=s&&"set"!=s.handler||(t.live=!1);t.buttons.set={text:i.setText,icon:i.setIcon,handler:"set"},t.buttons.cancel={text:i.cancelText,icon:i.cancelIcon,handler:"cancel"},t.buttons.close={text:i.closeText,icon:i.closeIcon,handler:"cancel"},t.buttons.clear={text:i.clearText,icon:i.clearIcon,handler:"clear"},t._isInput=h.is("input")},t._init=function(n){var e=t._isVisible,s=e&&!a.hasClass("mbsc-fr-pos");e&&t.hide(!0,!1,!0),h.off(".mbsc"),t.__init(n),t._isLiquid="liquid"==i.layout,r?(t._readValue(),t._hasContent||i.skipShow||t.attachShow(h),e&&t.show(s)):t.show(),h.removeClass("mbsc-cloak").filter("input, select, textarea").on("change.mbsc",function(){t._preventChange||t.setVal(h.val(),!0,!1),t._preventChange=!1})},t.buttons={},t.handlers={set:t.select,cancel:t.cancel,clear:t.clear},t._value=null,t._isValid=!0,t._isVisible=!1,i=t.settings,m=t.trigger,er||t.init()},e0,ea,ei;$.prototype._defaults={lang:"en",setText:"Set",selectedText:"{count} selected",closeText:"Close",cancelText:"Cancel",clearText:"Clear",context:"body",maxPopupWidth:600,disabled:!1,closeOnOverlayTap:!0,showOnFocus:eo||U,showOnTap:!0,display:"center",scroll:!0,scrollLock:!0,showOverlay:!0,tap:!0,touchUi:!0,btnClass:"mbsc-fr-btn",btnWidth:!0,focusTrap:!0,focusOnClose:!eC},ee.Frame=$,et.frame.mobiscroll={headerText:!1,btnWidth:!1},et.scroller.mobiscroll=t({},et.frame.mobiscroll,{rows:5,showLabel:!1,selectedLineBorder:1,weekDays:"min",checkIcon:"ion-ios7-checkmark-empty",btnPlusClass:"mbsc-ic mbsc-ic-arrow-down5",btnMinusClass:"mbsc-ic mbsc-ic-arrow-up5",btnCalPrevClass:"mbsc-ic mbsc-ic-arrow-left5",btnCalNextClass:"mbsc-ic mbsc-ic-arrow-right5"}),b&&e(window).on("focus",function(){V&&(F=!0)}),e0="ios"==r,B=function(D,e8,eg){function eh(i){b("onStart",{domEvent:i}),t.stopProp&&i.stopPropagation(),t.prevDef&&i.preventDefault(),t.readonly||t.lock&&A||em(i,this)&&!T&&(w&&w.removeClass("mbsc-active"),R=!1,A||(w=e(i.target).closest(".mbsc-btn-e",this),w.length&&!w.hasClass("mbsc-disabled")&&(R=!0,et=setTimeout(function(){w.addClass("mbsc-active")},100))),T=!0,O=!1,H=!1,r.scrolled=A,q=o(i,"X"),P=o(i,"Y"),G=q,W=0,L=0,s=0,ep=new Date,m=+ef(C,n)||0,A&&V(m,e0?0:1),"mousedown"===i.type&&e(document).on("mousemove",K).on("mouseup",N))}function K(e){T&&(t.stopProp&&e.stopPropagation(),G=o(e,"X"),ea=o(e,"Y"),W=G-q,L=ea-P,s=n?L:W,R&&(Math.abs(L)>t.thresholdY||Math.abs(W)>t.thresholdX)&&(clearTimeout(et),w.removeClass("mbsc-active"),R=!1),(r.scrolled||!H&&Math.abs(s)>ec)&&(O||b("onGestureStart",a),r.scrolled=O=!0,k||(k=!0,U=j(e2))),n||t.scrollLock?e.preventDefault():r.scrolled?e.preventDefault():Math.abs(L)>7&&(H=!0,r.scrolled=!0,N()))}function e2(){y&&(s=c(s,-i*y,i*y)),V(c(m+s,f-z,u+z)),k=!1}function N(n){if(T){var o,i=new Date-ep;t.stopProp&&n&&n.stopPropagation(),er(U),k=!1,!H&&r.scrolled&&(t.momentum&&i<300&&(o=s/i,s=Math.max(Math.abs(s),o*o/t.speedUnit)*(s<0?-1:1)),eo(s)),R&&(clearTimeout(et),w.addClass("mbsc-active"),setTimeout(function(){w.removeClass("mbsc-active")},100),H||r.scrolled||b("onBtnTap",{target:w[0],domEvent:n})),n&&"mouseup"==n.type&&e(document).off("mousemove",K).off("mouseup",N),T=!1}}function e1(e){if(e=e.originalEvent||e,s=n?void 0==e.deltaY?e.wheelDelta||e.detail:e.deltaY:e.deltaX,b("onStart",{domEvent:e}),t.stopProp&&e.stopPropagation(),s){if(e.preventDefault(),e.deltaMode&&1==e.deltaMode&&(s*=15),s=c(-s,-E,E),m=l,t.readonly)return;if(O||ed(),m+s<f&&(m=f,s=0),m+s>u&&(m=u,s=0),k||(k=!0,U=j(e2)),!s&&O)return;O=!0,clearTimeout(e4),e4=setTimeout(function(){er(U),k=!1,O=!1,eo(s)},200)}}function e6(n){b("onStart",{domEvent:n}),t.readonly||(n.stopPropagation(),m=l,O=!1,n.target==x?(P=o(n,"Y",!0),e(document).on("mousemove",ei).on("mouseup",en)):(P=Q.offset().top,ei(n),en()))}function ei(t){var e=(o(t,"Y",!0)-P)/g;es?(s=-(y*i*2+g)*e,s=c(s,-i*y,i*y)):s=(f-u-g)*e,O||ed(),O=!0,V(c(m+s,f-z,u+z))}function en(){m=l,eo(0),e(document).off("mousemove",ei).off("mouseup",en)}function el(e){e.stopPropagation()}function ed(){a={posX:n?0:l,posY:n?l:0,originX:n?0:m,originY:n?m:0,direction:s>0?n?270:360:n?90:180},b("onGestureStart",a)}function eo(o){var e,s,d;if(y&&(o=c(o,-i*y,i*y)),s=c(Math.round((m+o)/i)*i,f,u),v){if(o<0){for(e=v.length-1;e>=0;e--)if(Math.abs(s)+g>=v[e].breakpoint){M=e,Y=2,s=v[e].snap2;break}}else if(o>=0)for(e=0;e<v.length;e++)if(Math.abs(s)<=v[e].breakpoint){M=e,Y=1,s=v[e].snap1;break}s=c(s,f,u)}d=t.time||(l<f||l>u?1e3:Math.max(1e3,Math.abs(s-l)*t.timeUnit)),a.destinationX=n?0:s,a.destinationY=n?s:0,a.duration=d,a.transitionTiming=I,b("onGestureEnd",a),r.scroll(s,d)}function V(e,s,j,r){var c,d=e!=l,h=s>1,v=s?_+"transform "+Math.round(s)+"ms "+I:"",o=function(){clearInterval($),clearTimeout(X),A=!1,l=e,a.posX=n?0:e,a.posY=n?e:0,d&&b("onMove",a),h&&b("onAnimationEnd",a),r&&r()};a={posX:n?0:l,posY:n?l:0,originX:n?0:m,originY:n?m:0,direction:e-l>0?n?270:360:n?90:180},l=e,h&&(a.destinationX=n?0:e,a.destinationY=n?e:0,a.duration=s,a.transitionTiming=I,b("onAnimationStart",a)),ee[p+"Transition"]=v,ee[p+"Transform"]="translate3d("+(n?"0,"+e+"px,":e+"px,0,")+"0)",x&&S&&(c=es?(e3-e)/(y*i*2):(e-u)/(f-u),x.style[p+"Transition"]=v,x.style[p+"Transform"]="translate3d(0,"+Math.max(0,Math.min((g-S)*c,g-S))+"px,0)"),!d&&!A||!s||s<=1?o():s&&(A=!j,clearInterval($),$=setInterval(function(){var t=+ef(C,n)||0;a.posX=n?0:t,a.posY=n?t:0,b("onMove",a),Math.abs(t-e)<2&&o()},100),clearTimeout(X),X=setTimeout(function(){o()},s)),t.sync&&t.sync(e,s,I)}var n,s,i,a,l,u,m,f,g,v,b,y,w,O,x,C,E,k,A,S,T,z,L,R,P,H,I,B,$,W,U,q,G,X,Q,J,et,ea,es,e3,eu,e4,ep,ee,ec,r=this,M=0,Y=1,t=e8,F=e(D);Z.call(this,D,e8,!0),r.scrolled=!1,r.scroll=function(t,n,s,o){t=h(t)?Math.round(t/i)*i:Math.ceil((e(t,D).length?Math.round(C.offset()[J]-e(t,D).offset()[J]):l)/i)*i,t=c(t,f,u),M=Math.round(t/i),m=l,e3=y*i+t,V(t,n,s,o)},r.refresh=function(s){var e;for(g=(void 0===t.contSize?n?F.height():F.width():t.contSize)||0,u=(void 0===t.maxScroll?0:t.maxScroll)||0,f=Math.min(u,void 0===t.minScroll?Math.min(0,n?g-C.height():g-C.width()):t.minScroll)||0,v=null,!n&&t.rtl&&(e=u,u=-f,f=-e),d(t.snap)&&(v=[],C.find(t.snap).each(function(){var e=n?this.offsetTop:this.offsetLeft,t=n?this.offsetHeight:this.offsetWidth;v.push({breakpoint:e+t/2,snap1:-e,snap2:g-e-t})})),i=h(t.snap)?t.snap:1,y=t.snap?t.maxSnapScroll:0,I=t.easing,z=t.elastic?h(t.snap)?i:h(t.elastic)?t.elastic:0:0,E=i;E>44;)E/=2;E=Math.round(44/E)*E,x&&(es=f==-(1/0)||u==1/0,S=f<u?Math.max(20,g*g/(u-f+g)):0,x.style.height=S+"px",eu.style.height=S?"":0),void 0===l&&(l=t.initialPos,M=Math.round(l/i)),s||r.scroll(t.snap?v?v[M]["snap"+Y]:M*i:l)},r._processSettings=function(){n="Y"==t.axis,J=n?"top":"left",C=t.moveElement||F.children().eq(0),ee=C[0].style,ec=n?t.thresholdY:t.thresholdX,t.scrollbar&&(B=t.scrollbar,Q=B.find(".mbsc-sc-bar"),x=Q[0],eu=B[0])},r._init=function(){r.refresh(),F.on("touchstart mousedown",eh).on("touchmove",K).on("touchend touchcancel",N),t.mousewheel&&F.on("wheel mousewheel",e1),x&&B.on("mousedown",e6).on("click",el),D.addEventListener("click",function(e){r.scrolled&&(r.scrolled=!1,e.stopPropagation(),e.preventDefault())},!0)},r._destroy=function(){clearInterval($),F.off("touchstart mousedown",eh).off("touchmove",K).off("touchend touchcancel",N).off("wheel mousewheel",e1),x&&B.off("mousedown",e6).off("click",el)},t=r.settings,b=r.trigger,eg||r.init()},B.prototype={_defaults:{speedUnit:.0022,timeUnit:3,initialPos:0,axis:"Y",thresholdX:10,thresholdY:5,easing:"cubic-bezier(0.190, 1.000, 0.220, 1.000)",stopProp:!0,momentum:!0,mousewheel:!0,elastic:!0}};var G={},Q=b?window.CSS:null,eM=Q&&Q.supports&&Q.supports("(transform-style: preserve-3d)"),z=function(f,L,G){function q(t){var n,s,o=+e(this).attr("data-index");38==t.keyCode?(n=!0,s=-1):40==t.keyCode?(n=!0,s=1):32==t.keyCode&&(n=!0,U(o,e(t.target))),n&&(t.stopPropagation(),t.preventDefault(),s&&y.start(o,s,t))}function Y(){y.stop()}function U(t,i){var e=r[t],c=+i.attr("data-index"),o=v(e,c),a=n._tempSelected[t],d=h(e.multiple)?e.multiple:1/0;l("onItemTap",{target:i[0],index:t,value:o,selected:i.hasClass("mbsc-sc-itm-sel")})!==!1&&(e.multiple&&!e._disabled[o]&&(void 0!==a[o]?(i.removeClass(m).removeAttr("aria-selected"),delete a[o]):(1==d&&(n._tempSelected[t]=a={},e._$markup.find(".mbsc-sc-itm-sel").removeClass(m).removeAttr("aria-selected")),N(a).length<d&&(i.addClass(m).attr("aria-selected","true"),a[o]=o))),z(e,t,c,I,e._index<c?1:2,!0,e.multiple),n.live&&(!e.multiple||1===e.multiple&&s.tapSelect)&&(s.setOnTap===!0||s.setOnTap[t])&&setTimeout(function(){n.select()},s.tapSelect?0:200))}function K(t,n){var e=r[t];return e&&(!e.multiple||1!==e.multiple&&n&&(s.setOnTap===!0||s.setOnTap[t]))}function W(e){return-(e.max-e._offset-(e.multiple&&!a?Math.floor(s.rows/2):0))*o}function V(e){return-(e.min-e._offset+(e.multiple&&!a?Math.floor(s.rows/2):0))*o}function k(e,t){return(e._array?e._map[t]:+e.getIndex(t,n))||0}function R(t,s){var o=t.data;if(s>=t.min&&s<=t.max)return t._array?t.circular?e(o).get(s%t._length):o[s]:e.isFunction(o)?o(s,n):""}function O(t){return e.isPlainObject(t)?void 0!==t.value?t.value:t.display:t}function Q(t){var n=e.isPlainObject(t)?t.display:t;return void 0===n?"":n}function v(e,t){return O(R(e,t))}function Z(e,t,o){var n=r[e];z(n,e,n._index+t,s.delay+100,1==t?1:2,!1,!1,"keydown"==o.type)}function P(t){return e.isArray(s.readonly)?s.readonly[t]:s.readonly}function T(n,a,l){var r=n._index-n._batch;return n.data=n.data||[],n.key=void 0!==n.key?n.key:a,n.label=void 0!==n.label?n.label:a,n._map={},n._array=e.isArray(n.data),n._array&&(n._length=n.data.length,e.each(n.data,function(e,t){n._map[O(t)]=e})),n.circular=void 0===s.circular?void 0===n.circular?n._array&&n._length>s.rows:n.circular:e.isArray(s.circular)?s.circular[a]:s.circular,n.min=n._array?n.circular?-(1/0):0:void 0===n.min?-(1/0):n.min,n.max=n._array?n.circular?1/0:n._length-1:void 0===n.max?1/0:n.max,n._nr=a,n._index=k(n,i[a]),n._disabled={},n._batch=0,n._current=n._index,n._first=n._index-c,n._last=n._index+c,n._offset=n._first,l?(n._offset-=n._margin/o+(n._index-r),n._margin+=(n._index-r)*o):n._margin=0,n._refresh=function(e){t(n._scroller.settings,{minScroll:W(n),maxScroll:V(n)}),n._scroller.refresh(e)},S[n.key]=n,n}function u(r,h,y,b,p){var e,t,a,c,l,d,f,g,u="",v=n._tempSelected[h],w=r._disabled||{};for(a=y;a<=b;a++)e=R(r,a),f=Q(e),t=O(e),g=e&&void 0!==e.cssClass?e.cssClass:"",c=e&&void 0!==e.label?e.label:"",d=e&&e.invalid,l=void 0!==t&&t==i[h]&&!r.multiple,u+='<div role="option" tabindex="-1" aria-selected="'+!!v[t]+'" class="mbsc-sc-itm '+(p?"mbsc-sc-itm-3d ":"")+g+" "+(l?"mbsc-sc-itm-sel ":"")+(v[t]?m:"")+(void 0===t?" mbsc-sc-itm-ph":" mbsc-btn-e")+(d?" mbsc-sc-itm-inv-h mbsc-disabled":"")+(w[t]?" mbsc-sc-itm-inv mbsc-disabled":"")+'" data-index="'+a+'" data-val="'+H(t)+'"'+(c?' aria-label="'+c+'"':"")+(l?' aria-selected="true"':"")+' style="height:'+o+"px;line-height:"+o+"px;"+(p?_+"transform:rotateX("+(r._offset-a)*M%360+"deg) translateZ("+o*s.rows/2+"px);":"")+'">'+(j>1?'<div class="mbsc-sc-itm-ml" style="line-height:'+Math.round(o/j)+"px;font-size:"+Math.round(o/j*.8)+'px;">':"")+f+(j>1?"</div>":"")+"</div>";return u}function X(t,h,f){var m=Math.round(-f/o)+t._offset,n=m-t._current,s=t._first,i=t._last,r=s+c-d+1,l=i-c+d;n&&(t._first+=n,t._last+=n,t._current=m,n>0?(t._$scroller.append(u(t,h,Math.max(i+1,s+n),i+n)),e(".mbsc-sc-itm",t._$scroller).slice(0,Math.min(n,i-s+1)).remove(),a&&(t._$3d.append(u(t,h,Math.max(l+1,r+n),l+n,!0)),e(".mbsc-sc-itm",t._$3d).slice(0,Math.min(n,l-r+1)).attr("class","mbsc-sc-itm-del"))):n<0&&(t._$scroller.prepend(u(t,h,s+n,Math.min(s-1,i+n))),e(".mbsc-sc-itm",t._$scroller).slice(Math.max(n,s-i-1)).remove(),a&&(t._$3d.prepend(u(t,h,r+n,Math.min(r-1,l+n),!0)),e(".mbsc-sc-itm",t._$3d).slice(Math.max(n,r-l-1)).attr("class","mbsc-sc-itm-del"))),t._margin+=n*o,t._$scroller.css("margin-top",t._margin+"px"))}function D(m,h,d,u){var s,e=r[m],c=u||e._disabled,t=k(e,h),o=v(e,t),l=o,a=o,n=0,i=0;if(c[o]===!0){for(s=0;t-n>=e.min&&c[l]&&s<100;)s++,n++,l=v(e,t-n);for(s=0;t+i<e.max&&c[a]&&s<100;)s++,i++,a=v(e,t+i);o=(i<n&&i&&2!==d||!n||t-n<0||1==d)&&!c[a]?a:l}return o}function C(j,t,y,d,x,_,O){var h,v,b,p=n._isVisible,g=!0,u=s.validate.call(f,{values:i.slice(0),index:t,direction:y},n)||{},g=!1;u.valid&&(n._tempWheelArray=i=u.valid.slice(0)),_||e.each(r,function(r,s){if(p&&s._$markup.find(".mbsc-sc-itm-inv").removeClass("mbsc-sc-itm-inv mbsc-disabled"),s._disabled={},u.disabled&&u.disabled[r]&&e.each(u.disabled[r],function(t,e){s._disabled[e]=!0,p&&s._$markup.find('.mbsc-sc-itm[data-val="'+H(e)+'"]').addClass("mbsc-sc-itm-inv mbsc-disabled")}),i[r]=s.multiple?i[r]:D(r,i[r],y),p){if(s.multiple&&void 0!==t||s._$markup.find(".mbsc-sc-itm-sel").removeClass(m).removeAttr("aria-selected"),h=k(s,i[r]),v=h-s._index+s._batch,Math.abs(v)>2*c+1&&(b=v+(2*c+1)*(v>0?-1:1),s._offset+=b,s._margin-=b*o,s._refresh()),s._index=h+s._batch,s.multiple){if(void 0===t)for(var l in n._tempSelected[r])s._$markup.find('.mbsc-sc-itm[data-val="'+H(l)+'"]').addClass(m).attr("aria-selected","true")}else s._$markup.find('.mbsc-sc-itm[data-val="'+H(i[r])+'"]').addClass("mbsc-sc-itm-sel").attr("aria-selected","true");s._$active&&s._$active.attr("tabindex",-1),s._$active=s._$markup.find('.mbsc-sc-itm[data-index="'+s._index+'"]').eq(a&&s.multiple?1:0).attr("tabindex",0),O&&t===r&&s._$active.length&&(s._$active[0].focus(),s._$scroller.parent().scrollTop(0)),s._scroller.scroll(-(h-s._offset+s._batch)*o,t===r||void 0===t?j:I,x)}}),l("onValidated",{index:t,time:j}),n._tempValue=s.formatValue.call(f,i,n),p&&n._updateHeader(),n.live&&K(t,_)&&(n._hasValue=d||n._hasValue,w(d,d,0,!0),d&&l("onSet",{valueText:n._value})),d&&l("onChange",{index:t,valueText:n._tempValue})}function z(e,n,t,o,a,r,c,l){var s=v(e,t);void 0!==s&&(i[n]=s,e._batch=e._array?Math.floor(t/e._length)*e._length:0,e._index=t,setTimeout(function(){C(o,n,a,!0,r,c,l)},10))}function w(a,o,c,d,u){if(d?n._tempValue=s.formatValue.call(f,n._tempWheelArray,n):C(c),!u){n._wheelArray=[];for(var e=0;e<i.length;e++)n._wheelArray[e]=r[e]&&r[e].multiple?Object.keys(n._tempSelected[e]||{})[0]:i[e];n._value=n._hasValue?n._tempValue:null,n._selected=t(!0,{},n._tempSelected)}a&&(n._isInput&&F.val(n._hasValue?n._tempValue:""),l("onFill",{valueText:n._hasValue?n._tempValue:"",change:o}),o&&(n._preventChange=!0,F.trigger("change")))}var s,o,i,a,r,l,d,m,g,b,j,y,x,E,A,S,M,c=40,I=1e3,n=this,F=e(f);$.call(this,f,L,!0),n.setVal=n._setVal=function(t,o,a,r,c){n._hasValue=null!==t&&void 0!==t,n._tempWheelArray=i=e.isArray(t)?t.slice(0):s.parseValue.call(f,t,n)||[],w(o,void 0===a?o:a,c,!1,r)},n.getVal=n._getVal=function(t){var e=n._hasValue||t?n[t?"_tempValue":"_value"]:null;return h(e)?+e:e},n.setArrayVal=n.setVal,n.getArrayVal=function(e){return e?n._tempWheelArray:n._wheelArray},n.changeWheel=function(i,r,l){var s,o;e.each(i,function(e,i){s=S[e],s&&(o=s._nr,t(s,i),T(s,o,!0),n._isVisible&&(a&&s._$3d.html(u(s,o,s._first+c-d+1,s._last-c+d,!0)),s._$scroller.html(u(s,o,s._first,s._last)).css("margin-top",s._margin+"px"),s._refresh(g)))}),!n._isVisible||n._isLiquid||g||n.position(),g||C(r,void 0,void 0,l)},n.getValidValue=D,n._generateContent=function(){var h,m=0,l="",f=a?_+"transform: translateZ("+(o*s.rows/2+3)+"px);":"",p='<div class="mbsc-sc-whl-l" style="'+f+"height:"+o+"px;margin-top:-"+(o/2+(s.selectedLineBorder||0))+'px;"></div>',i=0;return e.each(s.wheels,function(v,g){l+='<div class="mbsc-w-p mbsc-sc-whl-gr-c'+(a?" mbsc-sc-whl-gr-3d-c":"")+(s.showLabel?" mbsc-sc-lbl-v":"")+'">'+p+'<div class="mbsc-sc-whl-gr'+(a?" mbsc-sc-whl-gr-3d":"")+(b?" mbsc-sc-cp":"")+(s.width||s.maxWidth?'"':'" style="max-width:'+s.maxPopupWidth+'px;"')+">",e.each(g,function(g,e){n._tempSelected[i]=t({},n._selected[i]),r[i]=T(e,i),m+=s.maxWidth?s.maxWidth[i]||s.maxWidth:s.width?s.width[i]||s.width:0,h=void 0!==e.label?e.label:g,l+='<div class="mbsc-sc-whl-w '+(e.cssClass||"")+(e.multiple?" mbsc-sc-whl-multi":"")+'" style="'+(s.width?"width:"+(s.width[i]||s.width)+"px;":(s.minWidth?"min-width:"+(s.minWidth[i]||s.minWidth)+"px;":"")+(s.maxWidth?"max-width:"+(s.maxWidth[i]||s.maxWidth)+"px;":""))+'">'+(E?'<div class="mbsc-sc-bar-c"><div class="mbsc-sc-bar"></div></div>':"")+'<div class="mbsc-sc-whl-o" style="'+f+'"></div>'+p+'<div aria-live="off" aria-label="'+h+'"'+(e.multiple?' aria-multiselectable="true"':"")+' role="listbox" data-index="'+i+'" class="mbsc-sc-whl" style="height:'+s.rows*o*(a?1.1:1)+'px;">'+(b?'<div data-index="'+i+'" data-step="1" class="mbsc-sc-btn mbsc-sc-btn-plus '+(s.btnPlusClass||"")+'"></div><div data-index="'+i+'" data-step="-1" class="mbsc-sc-btn mbsc-sc-btn-minus '+(s.btnMinusClass||"")+'"></div>':"")+'<div class="mbsc-sc-lbl">'+h+'</div><div class="mbsc-sc-whl-c" style="height:'+x+"px;margin-top:-"+(x/2+1)+"px;"+f+'"><div class="mbsc-sc-whl-sc" style="top:'+(x-o)/2+'px;">',l+=u(e,i,e._first,e._last)+"</div></div>",a&&(l+='<div class="mbsc-sc-whl-3d" style="height:'+o+"px;margin-top:-"+o/2+'px;">',l+=u(e,i,e._first+c-d+1,e._last-c+d,!0),l+="</div>"),l+="</div></div>",i++}),l+="</div></div>"}),m&&(s.maxPopupWidth=m),l},n._attachEvents=function(t){y=ep(e(".mbsc-sc-btn",t),Z,s.delay,P,!0),e(".mbsc-sc-whl",t).on("keydown",q).on("keyup",Y)},n._detachEvents=function(){y.stop();for(var e=0;e<r.length;e++)r[e]._scroller.destroy()},n._markupReady=function(t){A=t,e(".mbsc-sc-whl-w",A).each(function(n){var d,i=e(this),t=r[n];t._$markup=i,t._$scroller=e(".mbsc-sc-whl-sc",this),t._$3d=e(".mbsc-sc-whl-3d",this),t._scroller=new B(this,{mousewheel:s.mousewheel,moveElement:t._$scroller,scrollbar:e(".mbsc-sc-bar-c",this),initialPos:(t._first-t._index)*o,contSize:s.rows*o,snap:o,minScroll:W(t),maxScroll:V(t),maxSnapScroll:c,prevDef:!0,stopProp:!0,timeUnit:3,easing:"cubic-bezier(0.190, 1.000, 0.220, 1.000)",sync:function(n,e,s){var i=e?_+"transform "+Math.round(e)+"ms "+s:"";a&&(t._$3d[0].style[p+"Transition"]=i,t._$3d[0].style[p+"Transform"]="rotateX("+-n/o*M+"deg)")},onStart:function(t,e){e.settings.readonly=P(n)},onGestureStart:function(){i.addClass("mbsc-sc-whl-a mbsc-sc-whl-anim"),l("onWheelGestureStart",{index:n})},onGestureEnd:function(e){var s=90==e.direction?1:2,i=e.duration,a=e.destinationY;d=Math.round(-a/o)+t._offset,z(t,n,d,i,s)},onAnimationStart:function(){i.addClass("mbsc-sc-whl-anim")},onAnimationEnd:function(){i.removeClass("mbsc-sc-whl-a mbsc-sc-whl-anim"),l("onWheelAnimationEnd",{index:n}),t._$3d.find(".mbsc-sc-itm-del").remove()},onMove:function(e){X(t,n,e.posY)},onBtnTap:function(t){U(n,e(t.target))}})}),C()},n._fillValue=function(){n._hasValue=!0,w(!0,!0,0,!0)},n._clearValue=function(){e(".mbsc-sc-whl-multi .mbsc-sc-itm-sel",A).removeClass(m).removeAttr("aria-selected")},n._readValue=function(){var a=F.val()||"",o=0;""!==a&&(n._hasValue=!0),n._tempWheelArray=i=n._hasValue&&n._wheelArray?n._wheelArray.slice(0):s.parseValue.call(f,a,n)||[],n._tempSelected=t(!0,{},n._selected),e.each(s.wheels,function(n,t){e.each(t,function(t,e){r[o]=T(e,o),o++})}),w(!1,!1,0,!0),l("onRead")},n.__processSettings=function(e){s=n.settings,l=n.trigger,j=s.multiline,m="mbsc-sc-itm-sel mbsc-ic mbsc-ic-"+s.checkIcon,E=!s.touchUi,E&&(s.tapSelect=!0,s.circular=!1,s.rows=e.rows||L.rows||7)},n.__init=function(e){e&&(n._wheelArray=null),r=[],S={},b=s.showScrollArrows,a=s.scroll3d&&eM&&!b&&!E&&("ios"==s.theme||"ios"==s.baseTheme),o=s.height,x=a?2*Math.round((o-.03*(o*s.rows/2+3))/2):o,d=Math.round(1.8*s.rows),M=360/(2*d),b&&(s.rows=Math.max(3,s.rows))},n._getItemValue=O,n._tempSelected={},n._selected={},G||n.init()};z.prototype={_hasDef:!0,_hasTheme:!0,_hasLang:!0,_responsive:!0,_class:"scroller",_presets:G,_defaults:t({},$.prototype._defaults,{minWidth:80,height:40,rows:3,multiline:1,delay:200,readonly:!1,showLabel:!0,setOnTap:!1,wheels:[],preset:"",speedUnit:.0012,timeUnit:.08,checkIcon:"checkmark",compClass:"mbsc-sc",validate:function(){},formatValue:function(e){return e.join(" ")},parseValue:function(n,t){var s,o,i=[],a=[],r=0;return null!==n&&void 0!==n&&(i=(n+"").split(" ")),e.each(t.settings.wheels,function(c,n){e.each(n,function(c,n){o=n.data,s=t._getItemValue(o[0]),e.each(o,function(n,e){if(i[r]==t._getItemValue(e))return s=t._getItemValue(e),!1}),a.push(s),r++})}),a}})},ee.Scroller=z,ea={inputClass:"",rtl:!1,showInput:!0,groupLabel:"Groups",dataHtml:"html",dataText:"text",dataValue:"value",dataGroup:"group",dataDisabled:"disabled",filterPlaceholderText:"Type to filter",filterEmptyText:"No results",filterClearIcon:"material-close"},G.select=function(o,ec){function H(r){var t,n,o,i,m,f,p,d=0,l=0,h={},c={},b={},u=[],S=[];C.length=0,R?e.each(E,function(a,e){f=e[s.dataText]+"",p=e[s.dataHtml],n=e[s.dataValue],i=e[s.dataGroup],t={value:n,html:p,text:f,index:a,cssClass:B?"mbsc-sel-gr-itm":""},c[n]=t,r&&!V(f,r)||(u.push(t),x&&(void 0===h[i]?(o={text:i,value:l,options:[],index:l},b[l]=o,h[i]=l,S.push(o),l++):o=b[h[i]],w&&(t.index=o.options.length),t.group=h[i],o.options.push(t)),e[s.dataDisabled]&&C.push(n))}):x?(m=!0,e("optgroup",a).each(function(n){b[n]={text:this.label,value:n,options:[],index:n},m=!0,e("option",this).each(function(e){t={value:this.value,text:this.text,index:w?e:d++,group:n,cssClass:B?"mbsc-sel-gr-itm":""},c[this.value]=t,r&&!V(this.text,r)||(m&&(S.push(b[n]),m=!1),u.push(t),b[n].options.push(t),this.disabled&&C.push(this.value))})})):e("option",a).each(function(e){t={value:this.value,text:this.text,index:e},c[this.value]=t,r&&!V(this.text,r)||(u.push(t),this.disabled&&C.push(this.value))}),et=s.defaultValue?s.defaultValue:u.length?u[0].value:"",B&&(u=[],d=0,e.each(b,function(o,s){s.options.length&&(n="__group"+o,t={text:s.text,value:n,group:o,index:d++,cssClass:"mbsc-sel-gr"},c[n]=t,u.push(t),C.push(t.value),e.each(s.options,function(t,e){e.index=d++,u.push(e)}))})),T&&(u.length?T.removeClass("mbsc-sel-empty-v"):T.addClass("mbsc-sel-empty-v"))}function es(t,s,i,n,a){var e,o=[];for(e=0;e<t.length;e++)o.push({value:t[e].value,display:t[e].html||t[e].text,cssClass:t[e].cssClass});return{circular:!1,multiple:s&&!n?1:n,cssClass:(s&&!n?"mbsc-sel-one":"")+" "+a,data:o,label:i}}function ei(){return es(S,m,s.groupLabel,!1,"mbsc-sel-gr-whl")}function Z(){return es(w&&b[g]?b[g].options:u,m,er,v,"")}function Q(){var e=[[]];return j&&(M=ei(),X?e[0][r]=M:e[r]=[M]),L=Z(),X?e[0][i]=L:e[i]=[L],e}function z(t){v&&(t&&d(t)&&(t=t.split(",")),e.isArray(t)&&(t=t[0])),l=void 0===t||null===t||""===t?et:t,!c[l]&&u&&u.length&&(l=u[0].value),j&&(g=c[l]?c[l].group:null)}function I(e){return Y[e]||(c[e]?c[e].text:"")}function eu(o,e,a){var t,n,s=[],r=a?e._selected:e._tempSelected;if(m){for(t in r[i])s.push(I(t));return s.join(", ")}return n=o[i],I(n)}function P(){var t,n="",e=o.getVal(),i=s.formatValue.call(D,o.getArrayVal(),o,!0);if(s.filter&&"inline"==s.display||p.val(i),a.is("select")&&R){if(v)for(t=0;t<e.length;t++)n+='<option value="'+e[t]+'">'+I(e[t])+"</option>";else n='<option value="'+(null===e?"":e)+'">'+i+"</option>";a.html(n)}D!==p[0]&&a.val(e)}function $(){var e={};e[i]=Z(),A=!0,o.changeWheel(e)}function V(t,e){return e=e.replace(/[-/\\^$*+?.()|[\]{}]/g,"\\$&"),t.match(new RegExp(e,"ig"))}function W(e){return s.data.dataField?e[s.data.dataField]:s.data.processResponse?s.data.processResponse(e):e}function U(t){var e={};H(t),z(l),s.wheels=Q(),e[i]=L,o._tempWheelArray[i]=l,j&&(e[r]=M,o._tempWheelArray[r]=g),o.changeWheel(e,0,!0),P()}function K(e){return o.trigger("onFilter",{filterText:e})}function J(e){e[r]!=g&&(g=e[r],l=b[g].options[0].value,e[i]=l,w?$():o.setArrayVal(e,!1,!1,!0,q))}var i,r,l,u,p,g,b,y,_,E,A,S,M,L,eo,et="",Y={},q=1e3,D=this,a=e(D),ed=t({},o.settings),s=t(o.settings,ea,ed),T=e('<div class="mbsc-sel-empty">'+s.filterEmptyText+"</div>"),el=s.readonly,c={},G=s.layout||(/top|bottom|inline/.test(s.display)||s.filter?"liquid":""),X="liquid"==G||!s.touchUi,v=h(s.select)?s.select:"multiple"==s.select||a.prop("multiple"),m=v||!(!s.filter&&!s.tapSelect)&&1,F=this.id+"_dummy",ee=e('label[for="'+this.id+'"]').attr("for",F),er=void 0!==s.label?s.label:ee.length?ee.text():a.attr("name"),O=s.group,R=!!s.data,x=R?!!s.group:e("optgroup",a).length,j=x&&O&&O.groupWheel!==!1,w=x&&O&&j&&O.clustered===!0,B=x&&(!O||O.header!==!1&&!w),k=a.val()||(v?[]:[""]),C=[];return o.setVal=function(e,a,l,t,u){if(m&&(null===e||void 0===e||v||(e=[e]),e&&d(e)&&(e=e.split(",")),o._tempSelected[i]=f(e),t||(o._selected[i]=f(e)),e=e?e[0]:null,j)){var n=c[e],s=n&&n.group;o._tempSelected[r]=f([s]),t||(o._selected[r]=f([s]))}o._setVal(e,a,l,t,u)},o.getVal=function(t,n){var e;return m?(e=N(t?o._tempSelected[i]:o._selected[i]),e=v?e:e.length?e[0]:null):(e=t?o._tempWheelArray:o._hasValue?o._wheelArray:null,e=e?e[i]:null),v?e:void 0!==e?x&&n?[c[e]?c[e].group:null,e]:e:null},o.refresh=function(t,i,o){o=o||n,t?(E=t,_||(s.data=t)):e.isArray(s.data)&&(E=s.data),!t&&_&&void 0===i?en(s.data.url,function(e){E=W(e),U(),o()},s.data.dataType):(U(i),o())},ec.invalid||(s.invalid=C),j?(r=0,i=1):(r=-1,i=0),m&&(v&&a.prop("multiple",!0),k&&d(k)&&(k=k.split(",")),o._selected[i]=f(k)),o._$input&&o._$input.remove(),a.next().is(".mbsc-select-input")?p=a.next().removeAttr("tabindex"):s.input?p=e(s.input):(s.filter&&"inline"==s.display?o._$input=e('<div class="mbsc-sel-input-wrap"><input type="text" id="'+F+'" class="mbsc-select-input mbsc-control '+s.inputClass+'" readonly /></div>'):(p=e('<input type="text" id="'+F+'" class="mbsc-select-input mbsc-control '+s.inputClass+'" readonly />'),o._$input=p),s.showInput&&(o._$input.insertAfter(a),p||(p=o._$input.find("#"+F)))),o.attachShow(p.attr("placeholder",s.placeholder||"")),p[0]!==D&&a.addClass("mbsc-sel-hdn").attr("tabindex",-1).attr("data-enhance",!1),!m||s.rows%2||(s.rows=s.rows-1),s.filter&&(eo=s.filter.minLength||0),_=s.data&&s.data.url,_?o.refresh():(R&&(E=s.data),H(),z(a.val())),{layout:G,headerText:!1,anchor:p,compClass:"mbsc-sc mbsc-sel"+(m?" mbsc-sel-multi":""),setOnTap:!j||[!1,!0],formatValue:eu,tapSelect:m,parseValue:function(e){return z(void 0===e?a.val():e),j?[g,l]:[l]},validate:function(t){var n=t.index,e=[];return e[i]=s.invalid,w&&!A&&void 0===n&&$(),A=!1,{disabled:e}},onRead:P,onFill:P,onMarkupReady:function(c,o){if(s.filter){var t,i,r,a=e(".mbsc-fr-w",c.target),n=e('<span class="mbsc-sel-filter-clear mbsc-ic mbsc-ic-'+s.filterClearIcon+'"></span>');"inline"==s.display?(i=p,p.parent().find(".mbsc-sel-filter-clear").remove()):(a.find(".mbsc-fr-c").before('<div class="mbsc-input mbsc-sel-filter-cont mbsc-control-w mbsc-'+s.theme+(s.baseTheme?" mbsc-"+s.baseTheme:"")+'"><span class="mbsc-input-wrap"><input tabindex="0" type="text" class="mbsc-sel-filter-input mbsc-control"/></span></div>'),i=a.find(".mbsc-sel-filter-input")),y=null,t=i[0],i.prop("readonly",!1).attr("placeholder",s.filterPlaceholderText).parent().append(n),a.find(".mbsc-fr-c").prepend(T),o._activeElm=t,o.tap(n,function(){y=null,t.value="",o.refresh(),n.removeClass("mbsc-sel-filter-show-clear"),K("")}),i.on("keydown",function(e){13!=e.keyCode&&27!=e.keyCode||(e.stopPropagation(),t.blur())}).on("input",function(){clearTimeout(r),t.value.length?n.addClass("mbsc-sel-filter-show-clear"):n.removeClass("mbsc-sel-filter-show-clear"),r=setTimeout(function(){y!==t.value&&K(t.value)!==!1&&(y=t.value,(y.length>=eo||!y.length)&&(_&&s.data.remoteFilter?en(s.data.url+encodeURIComponent(y),function(e){o.refresh(W(e))},s.data.dataType):o.refresh(void 0,y)))},500)})}},onBeforeShow:function(){v&&s.counter&&(s.headerText=function(){var t=0;return e.each(o._tempSelected[i],function(){t++}),(t>1?s.selectedPluralText||s.selectedText:s.selectedText).replace(/{count}/,t)}),z(a.val()),m&&j&&(o._selected[r]=f([g])),s.filter&&H(void 0),o.settings.wheels=Q(),A=!0},onWheelGestureStart:function(e){e.index==r&&(s.readonly=[!1,!0])},onWheelAnimationEnd:function(t){var e=o.getArrayVal(!0);t.index==r?(s.readonly=el,m||J(e)):t.index==i&&e[i]!=l&&(l=e[i],j&&c[l]&&c[l].group!=g&&(g=c[l].group,e[r]=g,o._tempSelected[r]=f([g]),o.setArrayVal(e,!1,!1,!0,q)))},onItemTap:function(e){var t;if(e.index==i&&(Y[e.value]=c[e.value].text,m&&!v&&e.selected))return!1;if(e.index==r&&m){if(e.selected)return!1;t=o.getArrayVal(!0),t[r]=e.value,J(t)}},onClose:function(){_&&s.data.remoteFilter&&y&&o.refresh()},onDestroy:function(){o._$input&&o._$input.remove(),a.removeClass("mbsc-sel-hdn").removeAttr("tabindex")}}},eu("select",z);var ez=/^(\d{4}|[+-]\d{6})(?:-(\d{2})(?:-(\d{2}))?)?(?:T(\d{2}):(\d{2})(?::(\d{2})(?:\.(\d{3}))?)?((Z)|([+-])(\d{2})(?::(\d{2}))?)?)?$/,ed=/^((\d{2}):(\d{2})(?::(\d{2})(?:\.(\d{3}))?)?(?:(Z)|([+-])(\d{2})(?::(\d{2}))?)?)?$/,es=/^\d{1,2}(\/\d{1,2})?$/,A=/^w\d$/i,J={shortYearCutoff:"+10",monthNames:["January","February","March","April","May","June","July","August","September","October","November","December"],monthNamesShort:["Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"],dayNames:["Sunday","Monday","Tuesday","Wednesday","Thursday","Friday","Saturday"],dayNamesShort:["Sun","Mon","Tue","Wed","Thu","Fri","Sat"],dayNamesMin:["S","M","T","W","T","F","S"],amText:"am",pmText:"pm",getYear:function(e){return e.getFullYear()},getMonth:function(e){return e.getMonth()},getDay:function(e){return e.getDate()},getDate:m,getMaxDayOfMonth:function(e,t){return 32-new Date(e,t,32,12).getDate()},getWeekNumber:function(e){e=new Date(e),e.setHours(0,0,0),e.setDate(e.getDate()+4-(e.getDay()||7));var t=new Date(e.getFullYear(),0,1);return Math.ceil(((e-t)/864e5+1)/7)}};I.datetime={formatDate:v,parseDate:L};var eW={separator:" ",dateFormat:"mm/dd/yy",dateDisplay:"MMddyy",timeFormat:"h:ii A",dayText:"Day",monthText:"Month",yearText:"Year",hourText:"Hours",minuteText:"Minutes",ampmText:"&nbsp;",secText:"Seconds",nowText:"Now",todayText:"Today"},eH=function(o){function q(s){var e,t,o,r,a=[];if(s){for(o=0;o<s.length;o++)if(e=s[o],e.start&&e.end&&!ed.test(e.start))for(t=new Date(i(e.start,h,n)),r=new Date(i(e.end,h,n));t<=r;)a.push(m(t.getFullYear(),t.getMonth(),t.getDate())),t.setDate(t.getDate()+1);else a.push(e);return a}return s}function u(t,e,n,s){return Math.min(s,Math.floor(t/e)*e+n)}function $(e,t,n){return Math.floor((n-t)/e)*e+t}function ev(e){return n.getYear(e)}function e7(e){return n.getMonth(e)}function e6(e){return n.getDay(e)}function ef(t){var e=t.getHours();return e=w&&e>=12?e-12:e,u(e,x,F,er)}function e8(e){return u(e.getMinutes(),j,M,ea)}function ej(e){return u(e.getSeconds(),C,B,ei)}function em(e){return e.getMilliseconds()}function eh(e){return e.getHours()>11?1:0}function N(e){return e.getFullYear()+"-"+a(e.getMonth()+1)+"-"+a(e.getDate())}function e3(e){return u(Math.round((e.getTime()-new Date(e).setHours(0,0,0,0))/1e3),D||1,0,86400)}function g(o,e,n,i){var t;return void 0===s[e]||(t=+o[s[e]],isNaN(t))?n?d[e](n):void 0!==z[e]?z[e]:d[e](i):t}function _(t){if(o=new Date((new Date).setHours(0,0,0,0)),null===t)return t;void 0!==s.dd&&(e=t[s.dd].split("-"),e=new Date(e[0],e[1]-1,e[2])),void 0!==s.tt&&(e=e||o,e=new Date(e.getTime()+t[s.tt]%86400*1e3));var e,o,i=g(t,"y",e,o),a=g(t,"m",e,o),c=Math.min(g(t,"d",e,o),n.getMaxDayOfMonth(i,a)),r=g(t,"h",e,o);return n.getDate(i,a,c,w&&g(t,"a",e,o)?r+12:r,g(t,"i",e,o),g(t,"s",e,o),g(t,"u",e,o))}function k(t,a){var e,n,o=["y","m","d","a","h","i","s","u","dd","tt"],i=[];if(null===t||void 0===t)return t;for(n=0;n<o.length;n++)e=o[n],void 0!==s[e]&&(i[s[e]]=d[e](t)),a&&(z[e]=d[e](t));return i}function I(e,t){return t?Math.floor(new Date(e)/864e5):e.getMonth()+12*(e.getFullYear()-1970)}function e2(e){return{value:e,display:(/yy/i.test(S)?e:(e+"").substr(2,2))+(n.yearSuffix||"")}}function e0(e){return e}function e1(t){var e=/d/i.test(t);return{label:"",cssClass:"mbsc-dt-whl-date",min:r?I(N(r),e):void 0,max:l?I(N(l),e):void 0,data:function(o){var i=new Date((new Date).setHours(0,0,0,0)),s=e?new Date(864e5*o):new Date(1970,o,1);return e&&(s=new Date(s.getUTCFullYear(),s.getUTCMonth(),s.getUTCDate())),{invalid:e&&!O(s,!0),value:N(s),display:i.getTime()==s.getTime()?n.todayText:v(t,s,n)}},getIndex:function(t){return I(t,e)}}}function eu(s){var e,t,o,i=[];for(/s/i.test(s)?t=C:/i/i.test(s)?t=60*j:/h/i.test(s)&&(t=3600*x),D=f.tt=t,e=0;e<86400;e+=t)o=new Date((new Date).setHours(0,0,0,0)+1e3*e),i.push({value:e,display:v(s,o,n)});return{label:"",cssClass:"mbsc-dt-whl-time",data:i}}function e4(){var e,t,i,d,u,f,p,g,o=0,v=[],m=[],h=[];if(/date/i.test(E)){for(p=ee.split(/\|/.test(ee)?"|":""),u=0;u<p.length;u++)if(t=p[u],d=0,t.length)if(/y/i.test(t)&&(c.y=1,d++),/m/i.test(t)&&(c.y=1,c.m=1,d++),/d/i.test(t)&&(c.y=1,c.m=1,c.d=1,d++),d>1&&void 0===s.dd)s.dd=o,o++,m.push(e1(t)),h=m,Q=!0;else if(/y/i.test(t)&&void 0===s.y)s.y=o,o++,m.push({cssClass:"mbsc-dt-whl-y",label:n.yearText,min:r?n.getYear(r):void 0,max:l?n.getYear(l):void 0,data:e2,getIndex:e0});else if(/m/i.test(t)&&void 0===s.m){for(s.m=o,i=[],o++,e=0;e<12;e++)f=S.replace(/[dy|]/gi,"").replace(/mm/,a(e+1)+(n.monthSuffix||"")).replace(/m/,e+1+(n.monthSuffix||"")),i.push({value:e,display:/MM/.test(f)?f.replace(/MM/,'<span class="mbsc-dt-month">'+n.monthNames[e]+"</span>"):f.replace(/M/,'<span class="mbsc-dt-month">'+n.monthNamesShort[e]+"</span>")});m.push({cssClass:"mbsc-dt-whl-m",label:n.monthText,data:i})}else if(/d/i.test(t)&&void 0===s.d){for(s.d=o,i=[],o++,e=1;e<32;e++)i.push({value:e,display:(/dd/i.test(S)?a(e):e)+(n.daySuffix||"")});m.push({cssClass:"mbsc-dt-whl-d",label:n.dayText,data:i})}v.push(m)}if(/time/i.test(E)){for(g=U.split(/\|/.test(U)?"|":""),u=0;u<g.length;u++)if(t=g[u],d=0,t.length&&(/h/i.test(t)&&(c.h=1,d++),/i/i.test(t)&&(c.i=1,d++),/s/i.test(t)&&(c.s=1,d++),/a/i.test(t)&&d++),d>1&&void 0===s.tt)s.tt=o,o++,h.push(eu(t));else if(/h/i.test(t)&&void 0===s.h){for(i=[],s.h=o,c.h=1,o++,e=F;e<(w?12:24);e+=x)i.push({value:e,display:w&&0===e?12:/hh/i.test(T)?a(e):e});h.push({cssClass:"mbsc-dt-whl-h",label:n.hourText,data:i})}else if(/i/i.test(t)&&void 0===s.i){for(i=[],s.i=o,c.i=1,o++,e=M;e<60;e+=j)i.push({value:e,display:/ii/i.test(T)?a(e):e});h.push({cssClass:"mbsc-dt-whl-i",label:n.minuteText,data:i})}else if(/s/i.test(t)&&void 0===s.s){for(i=[],s.s=o,c.s=1,o++,e=B;e<60;e+=C)i.push({value:e,display:/ss/i.test(T)?a(e):e});h.push({cssClass:"mbsc-dt-whl-s",label:n.secText,data:i})}else/a/i.test(t)&&void 0===s.a&&(s.a=o,o++,h.push({cssClass:"mbsc-dt-whl-a",label:n.ampmText,data:/A/.test(t)?[{value:0,display:n.amText.toUpperCase()},{value:1,display:n.pmText.toUpperCase()}]:[{value:0,display:n.amText},{value:1,display:n.pmText}]}));h!=m&&v.push(h)}return v}function ep(n){var e,s,o,t={};if(n.is("input")){switch(n.attr("type")){case"date":e="yy-mm-dd";break;case"datetime":e="yy-mm-ddTHH:ii:ssZ";break;case"datetime-local":e="yy-mm-ddTHH:ii:ss";break;case"month":e="yy-mm",t.dateOrder="mmyy";break;case"time":e="HH:ii:ss"}t.format=e,s=n.attr("min"),o=n.attr("max"),s&&"undefined"!=s&&(t.min=L(e,s)),o&&"undefined"!=o&&(t.max=L(e,o))}return t}function eg(e,o){var t,n,i=!1,s=!1,a=0,c=0,d=r?_(k(r)):-(1/0),u=l?_(k(l)):1/0;if(O(e))return e;if(e<d&&(e=d),e>u&&(e=u),t=e,n=e,2!==o)for(i=O(t,!0);!i&&t<u&&a<100;)t=new Date(t.getTime()+864e5),i=O(t,!0),a++;if(1!==o)for(s=O(n,!0);!s&&n>d&&c<100;)n=new Date(n.getTime()-864e5),s=O(n,!0),c++;return 1===o&&i?t:2===o&&s?n:eo(e,t)?t:eo(e,n)?n:c<=a&&s?n:t}function eo(e,t){return n.getYear(e)===n.getYear(t)&&n.getMonth(e)===n.getMonth(t)}function O(e,t){return!(!t&&e<r)&&!(!t&&e>l)&&(!!et(e,V)||!et(e,W))}function et(s,a){var e,t,o;if(a)for(o=0;o<a.length;o++)if(t=a[o],e=t+"",!t.start)if(A.test(e)){if(e=+e.replace("w",""),e==s.getDay())return!0}else if(es.test(e)){if(e=e.split("/"),e[1]){if(e[0]-1==s.getMonth()&&e[1]==s.getDate())return!0}else if(e[0]==s.getDate())return!0}else if(t=i(t,h,n),s.getFullYear()==t.getFullYear()&&s.getMonth()==t.getMonth()&&s.getDate()==t.getDate())return!0;return!1}function en(c,d,l,u,m,r,o){var e,t,s,a;if(c)for(a=0;a<c.length;a++)if(t=c[a],e=t+"",!t.start)if(A.test(e))for(e=+e.replace("w",""),s=e-u;s<m;s+=7)s>=0&&(r[s+1]=o);else es.test(e)?(e=e.split("/"),e[1]?e[0]-1==l&&(r[e[1]]=o):r[e[0]]=o):(t=i(t,h,n),n.getYear(t)==d&&n.getMonth(t)==l&&(r[n.getDay(t)]=o))}function Z(_,C,o,R,z,E,H,L){var e,t,s,i,r,l,d,h,m,g,v,j,O,x,k,S,M,F,T,N,P={},c=n.getDate(R,z,E),a=["a","h","i","s"];if(_){for(r=0;r<_.length;r++)i=_[r],i.start&&(i.apply=!1,l=i.d,j=l+"",h=j.split("/"),l&&(l.getTime&&R==n.getYear(l)&&z==n.getMonth(l)&&E==n.getDay(l)||!A.test(j)&&(h[1]&&E==h[1]&&z==h[0]-1||!h[1]&&E==h[0])||A.test(j)&&c.getDay()==+j.replace("w",""))&&(i.apply=!0,P[c]=!0));for(r=0;r<_.length;r++)if(i=_[r],M=0,F=0,S=y[o],O=p[o],d=!0,v=!0,k=!1,i.start&&(i.apply||!i.d&&!P[c])){for(t=i.start.split(":"),s=i.end.split(":"),e=0;e<3;e++)void 0===t[e]&&(t[e]=0),void 0===s[e]&&(s[e]=59),t[e]=+t[e],s[e]=+s[e];if("tt"==o)S=u(Math.round((new Date(c).setHours(t[0],t[1],t[2])-new Date(c).setHours(0,0,0,0))/1e3),D,0,86400),O=u(Math.round((new Date(c).setHours(s[0],s[1],s[2])-new Date(c).setHours(0,0,0,0))/1e3),D,0,86400);else{for(t.unshift(t[0]>11?1:0),s.unshift(s[0]>11?1:0),w&&(t[1]>=12&&(t[1]=t[1]-12),s[1]>=12&&(s[1]=s[1]-12)),e=0;e<C;e++)void 0!==b[e]&&(m=u(t[e],f[a[e]],y[a[e]],p[a[e]]),g=u(s[e],f[a[e]],y[a[e]],p[a[e]]),T=0,N=0,x=0,w&&1==e&&(T=t[0]?12:0,N=s[0]?12:0,x=b[0]?12:0),d||(m=0),v||(g=p[a[e]]),(d||v)&&m+T<b[e]+x&&b[e]+x<g+N&&(k=!0),b[e]!=m&&(d=!1),b[e]!=g&&(v=!1));if(!L)for(e=C+1;e<4;e++)t[e]>0&&(M=f[o]),s[e]<p[a[e]]&&(F=f[o]);k||(m=u(t[C],f[o],y[o],p[o])+M,g=u(s[C],f[o],y[o],p[o])-F,d&&(S=m),v&&(O=g))}if(d||v||k)for(e=S;e<=O;e+=f[o])H[e]=!L}}}var D,G,Q,s={},z={},c={},b=[],Y=ep(e(this)),ec=t({},o.settings),e9=eS[ec.calendarSystem],n=t(o.settings,J,e9,eW,Y,ec),E=n.preset,H="datetime"==E?n.dateFormat+n.separator+n.timeFormat:"time"==E?n.timeFormat:n.dateFormat,h=Y.format||H,ee=n.dateWheels||n.dateFormat,U=n.timeWheels||n.timeFormat,S=n.dateWheels||n.dateDisplay,T=U,e5=n.baseTheme||n.theme,W=q(n.invalid),V=q(n.valid),r=i(n.min,h,n),l=i(n.max,h,n),K=/time/i.test(E),w=/h/.test(T),el=/D/.test(S),R=n.steps||{},x=R.hour||n.stepHour||1,j=R.minute||n.stepMinute||1,C=R.second||n.stepSecond||1,P=R.zeroBased,F=P||!r?0:r.getHours()%x,M=P||!r?0:r.getMinutes()%j,B=P||!r?0:r.getSeconds()%C,er=$(x,F,w?11:23),ea=$(j,M,59),ei=$(j,M,59),y={y:r?r.getFullYear():-(1/0),m:0,d:1,h:F,i:M,s:B,a:0,tt:0},p={y:l?l.getFullYear():1/0,m:11,d:31,h:er,i:ea,s:ei,a:1,tt:86400},f={y:1,m:1,d:1,h:x,i:j,s:C,a:1,tt:1},eb={bootstrap:46,ios:50,material:46,mobiscroll:46,windows:50},d={y:ev,m:e7,d:e6,h:ef,i:e8,s:ej,u:em,a:eh,dd:N,tt:e3};return o.getVal=function(e){return o._hasValue||e?X(_(o.getArrayVal(e)),n,h):null},o.getDate=function(e){return o._hasValue||e?_(o.getArrayVal(e)):null},o.setDate=function(e,t,n,s,i){o.setArrayVal(k(e,!0),t,i,s,n)},G=e4(),n.isoParts=c,o._format=H,o._order=s,o.handlers.now=function(){o.setDate(new Date,o.live,1e3,!0,!0)},o.buttons.now={text:n.nowText,icon:n.nowIcon,handler:"now"},{minWidth:Q&&K?eb[e5]:void 0,compClass:"mbsc-dt mbsc-sc",wheels:G,headerText:!!n.headerText&&function(){return v(H,_(o.getArrayVal(!0)),n)},formatValue:function(e){return v(h,_(e),n)},parseValue:function(e){return e||(z={},o._hasValue=!1),k(i(e||n.defaultValue||new Date,h,n,c),!!e)},validate:function(x){var t,c,v,w,F=x.values,j=x.index,M=x.direction,m=n.wheels[0][s.d],u=eg(_(F),M),A=k(u),h=[],E={},i=d.y(u),a=d.m(u),g=n.getMaxDayOfMonth(i,a),O=!0,C=!0;if(e.each(["dd","y","m","d","tt","a","h","i","s"],function(_,t){var b,j,m=y[t],v=p[t],o=d[t](u);if(h[s[t]]=[],O&&r&&(m=d[t](r)),C&&l&&(v=d[t](l)),o<m&&(o=m),o>v&&(o=v),"dd"!==t&&"tt"!==t&&(O&&(O=o==m),C&&(C=o==v)),void 0!==s[t]){if("y"!=t&&"dd"!=t)for(c=y[t];c<=p[t];c+=f[t])(c<m||c>v)&&h[s[t]].push(c);"d"==t&&(j=n.getDate(i,a,1).getDay(),b={},en(W,i,a,j,g,b,1),en(V,i,a,j,g,b,0),e.each(b,function(e,n){n&&h[s[t]].push(e)}))}}),K&&e.each(["a","h","i","s","tt"],function(r,t){var l=d[t](u),c=d.d(u),n={};void 0!==s[t]&&(Z(W,r,t,i,a,c,n,0),Z(V,r,t,i,a,c,n,1),e.each(n,function(e,n){n&&h[s[t]].push(e)}),b[r]=o.getValidValue(s[t],l,M,n))}),m&&(m._length!==g||el&&(void 0===j||j===s.y||j===s.m))){for(E[s.d]=m,m.data=[],t=1;t<=g;t++)w=n.getDate(i,a,t).getDay(),v=S.replace(/[my|]/gi,"").replace(/dd/,(t<10?"0"+t:t)+(n.daySuffix||"")).replace(/d/,t+(n.daySuffix||"")),m.data.push({value:t,display:/DD/.test(v)?v.replace(/DD/,'<span class="mbsc-dt-day">'+n.dayNames[w]+"</span>"):v.replace(/D/,'<span class="mbsc-dt-day">'+n.dayNamesShort[w]+"</span>")});o._tempWheelArray[s.d]=A[s.d],o.changeWheel(E)}return{disabled:h,valid:A}}}},eI={controls:["calendar"],firstDay:0,weekDays:"short",maxMonthWidth:170,breakPointMd:768,months:1,pageBuffer:1,weeks:6,highlight:!0,outerMonthChange:!0,quickNav:!0,yearChange:!0,tabs:"auto",todayClass:"mbsc-cal-today",btnCalPrevClass:"mbsc-ic mbsc-ic-arrow-left6",btnCalNextClass:"mbsc-ic mbsc-ic-arrow-right6",dateText:"Date",timeText:"Time",todayText:"Today",fromText:"Start",toText:"End",moreEventsText:"{count} more",prevMonthText:"Previous Month",nextMonthText:"Next Month",prevYearText:"Previous Year",nextYearText:"Next Year"},eB=function(o){function td(e){e.hasClass("mbsc-cal-h")&&(e.removeClass("mbsc-cal-h"),o._onSelectShow())}function T(e){e.hasClass("mbsc-cal-h")||e.addClass("mbsc-cal-h")}function eW(e){e.hasClass("mbsc-cal-h")?td(e):T(e)}function tc(){var u={},V=[],$={},f=o.trigger,ee=e(eZ),a=t({},o.settings),s=t(o.settings,eI,a),n=s.controls.join(","),j=s.firstDay,K=s.rtl,E=s.pageBuffer,Y=s.weekCounter,r=s.weeks,g=6==r,p="vertical"==s.calendarScroll,eG="inline"==s.display?ee.is("div")?ee:ee.parent():o._$window,eT="full"==s.weekDays?"":"min"==s.weekDays?"Min":"Short",i=s.layout||("inline"==s.display||/top|bottom/.test(s.display)&&s.touchUi?"liquid":""),S="liquid"==i,e1=S?null:s.calendarWidth,O=K&&!p?-1:1,eO="mbsc-disabled "+(s.disabledClass||""),e4="mbsc-selected "+(s.selectedTabClass||""),z="mbsc-selected "+(s.selectedClass||""),e2=Math.max(1,Math.floor(((s.calendarHeight||0)/r-45)/18));n.match(/calendar/)&&(u.calendar=1,h=!0),n.match(/date/)&&!h&&(u.date=1),n.match(/time/)&&(u.time=1),s.controls.forEach(function(e){u[e]&&V.push(e)}),F=s.quickNav&&h&&g,N=s.yearChange&&g,S&&h&&"center"==s.display&&(o._isFullScreen=!0),s.layout=i,s.preset=(u.date||h?"date":"")+(u.time?"time":"")}function tn(){eg=N?s.monthNamesShort:s.monthNames,e7=s.yearSuffix||"",ef=(s.dateWheels||s.dateFormat).search(/m/i),ed=(s.dateWheels||s.dateFormat).search(/y/i),L=o._format,s.min&&(v=w(i(s.min,L,s)),em=s.getYear(v),eV=s.getMonth(v),eB=s.getDate(12*Math.floor(em/12),0,1)),s.max&&(_=w(i(s.max,L,s)),e3=s.getYear(_),eL=s.getMonth(_),eD=s.getDate(12*Math.floor(e3/12),0,1))}function J(e,t,n){e[t]=e[t]||[],e[t].push(n)}function Z(d,a,r){var t,n,c,l,u=s.getYear(a),h=s.getMonth(a),o={};return d&&e.each(d,function(d,e){if(c=e.d||e.start||e,n=c+"",e.start&&e.end)for(t=w(i(e.start,L,s)),l=w(i(e.end,L,s));t<=l;)J(o,t,e),t=s.getDate(s.getYear(t),s.getMonth(t),s.getDay(t)+1);else if(A.test(n))for(t=G(a,!1,+n.replace("w",""));t<=r;)J(o,t,e),t=s.getDate(s.getYear(t),s.getMonth(t),s.getDay(t)+7);else if(es.test(n))if(n=n.split("/"),n[1])for(t=s.getDate(u,n[0]-1,n[1]);t<=r;)J(o,t,e),t=s.getDate(s.getYear(t)+1,s.getMonth(t),n[1]);else for(t=s.getDate(u,h,n[0]);t<=r;)J(o,t,e),t=s.getDate(s.getYear(t),s.getMonth(t)+1,n[0]);else J(o,w(i(c,L,s)),e)}),o}function eJ(e){return!(e<v)&&!(e>_)&&(void 0===eQ[e]||void 0!==e$[e])}function to(n){var e,a,r,c,s=!!el[n]&&el[n],i=!!e0[n]&&e0[n],d=i&&i[0].background?i[0].background:s&&s[0].background,l="";if(i)for(e=0;e<i.length;e++)l+=(i[e].cssClass||"")+" ";if(s){for(r='<div class="mbsc-cal-marks">',e=0;e<s.length;e++)a=s[e],l+=(a.cssClass||"")+" ",r+='<div class="mbsc-cal-mark"'+(a.color?' style="background:'+a.color+';"':"")+"></div>";r+="</div>"}return c={marked:s,background:d,cssClass:l,markup:$[n]?$[n].join(""):ey?r:""},t(c,o._getDayProps(n,c))}function ek(e){return' style="'+(p?"transform: translateY("+100*e+"%)":"left:"+100*e*O+"%")+'"'}function tt(){c="auto"==s.months?Math.max(1,Math.min(3,Math.floor((e1||eA(eG))/280))):+s.months,ej=c+2*E,y=0,p=p&&c<2,eo=void 0===s.showOuterDays?c<2&&!p:s.showOuterDays}function eY(e){return C(e,c-1)>_&&(e=C(_,1-c)),e<v&&(e=v),e}function er(t,i,s){var n=t.color,o=t.text;return'<div data-id="'+t._id+'" data-index="'+i+'" class="mbsc-cal-txt" title="'+e("<div>"+o+"</div>").text()+'"'+(n?' style="background:'+n+(s?";color:"+e6(n):"")+';"':"")+">"+(s?o:"")+"</div>"}function eN(n){var t=G(C(n,-y-E),!1),e=G(C(n,-y+c+E-1),!1),e=s.getDate(s.getYear(e),s.getMonth(e),s.getDay(e)+7*r);o._onGenMonth(t,e),eQ=Z(s.invalid,t,e),e$=Z(s.valid,t,e),el=Z(s.labels||s.events||s.marked,t,e),e0=Z(s.colors,t,e),eb=o._labels||el||e0,I=s.labels||o._labels,I&&!function(){$={};for(var o={},n=t,a=function(){n.getDay()==j&&(o={});for(var u,y,_=e2,g=eb[n]||[],c=g.length,l=[],a=void 0,f=void 0,e=0,d=0,r=0,b=void 0;e<_;)if(a=null,g.forEach(function(t,n){o[e]==t&&(a=t,f=n)}),e==_-1&&(d<c-1||c&&r==c&&!a))u=c-d,y=(u>1?s.moreEventsPluralText||s.moreEventsText:s.moreEventsText).replace(/{count}/,u),u&&l.push('<div class="mbsc-cal-txt-more">'+y+"</div>"),a&&(o[e]=null,a._days.forEach(function(t){$[t][e]='<div class="mbsc-cal-txt-more">'+s.moreEventsText.replace(/{count}/,1)+"</div>"})),d++,e++;else if(a)f==r&&r++,R(n,i(a.end))&&(o[e]=null),l.push(er(a,f)),e++,d++,a._days.push(n);else if(r<c){var t=g[r],v=t.start&&i(t.start),h=t.end&&i(t.end),p=n.getDay(),O=j-p>0?7:0,m=h&&!R(v,h);v&&!R(n,v)&&p!=j||(void 0===t._id&&(t._id=tu++),m&&(o[e]=t),t._days=[n],b=m?100*Math.min(e8(n,w(h))+1,7+j-p-O):100,l.push(m?'<div class="mbsc-cal-txt-w" style="width:'+b+'%">'+er(t,r,!0)+"</div>"+er(t,r):er(t,r,!0)),e++,d++),r++}else l.push('<div class="mbsc-cal-txt-ph"></div>'),e++;$[n]=l,n=s.getDate(s.getYear(n),s.getMonth(n),s.getDay(n)+1)};n<e;)a()}()}function ti(e){var t=s.getYear(e),n=s.getMonth(e),d=e,x=e;eC(e),f("onMonthChange",{year:t,month:n}),f("onMonthLoading",{year:t,month:n}),f("onPageChange",{firstDay:e}),f("onPageLoading",{firstDay:e}),eN(e)}function eS(e){var t=s.getYear(e),n=s.getMonth(e);void 0===eh?eX(e,t,n):e_(e,eh,!0),ex(x,a.focus),a.focus=!1}function eX(n,s,o){var t=a.$scroller;e(".mbsc-cal-slide",t).removeClass("mbsc-cal-slide-a"),e(".mbsc-cal-slide",t).slice(E,E+c).addClass("mbsc-cal-slide-a"),I&&e(".mbsc-cal-slide-a .mbsc-cal-txt",t).on("mouseenter",function(){var n=e(this).attr("data-id");e('.mbsc-cal-txt[data-id="'+n+'"]',t).addClass("mbsc-hover")}).on("mouseleave",function(){e(".mbsc-cal-txt.mbsc-hover",t).removeClass("mbsc-hover")}),f("onMonthLoaded",{year:s,month:o}),f("onPageLoaded",{firstDay:n})}function t1(){var e,t;return e='<div class="mbsc-cal-tabs-c"><div class="mbsc-cal-tabs" role="tablist">',V.forEach(function(n,o){t=s[("calendar"==n?"date":n)+"Text"],e+='<div role="tab" aria-controls="'+(eZ.id+"-mbsc-pnl-"+o)+'" class="mbsc-cal-tab mbsc-fr-btn-e '+(o?"":e4)+'" data-control="'+n+'"'+(s.tabLink?'><a href="#">'+t+"</a>":' tabindex="0">'+t)+"</div>"}),e+="</div></div>"}function te(){var e,t,n,o,i,l,a="",u=K?s.btnCalNextClass:s.btnCalPrevClass,h=K?s.btnCalPrevClass:s.btnCalNextClass;for(o='<div class="mbsc-cal-btn-w"><div data-step="-1" role="button" tabindex="0" aria-label="'+s.prevMonthText+'" class="'+u+' mbsc-cal-prev mbsc-cal-prev-m mbsc-cal-btn mbsc-fr-btn mbsc-fr-btn-e"></div>',e=0;e<(r?c:1);e++)o+='<div role="button" class="mbsc-cal-month"></div>';if(o+='<div data-step="1" role="button" tabindex="0" aria-label="'+s.nextMonthText+'" class="'+h+' mbsc-cal-next mbsc-cal-next-m mbsc-cal-btn mbsc-fr-btn mbsc-fr-btn-e"></div></div>',N&&(a='<div class="mbsc-cal-btn-w"><div data-step="-12" role="button" tabindex="0" aria-label="'+s.prevYearText+'" class="'+u+' mbsc-cal-prev mbsc-cal-prev-y mbsc-cal-btn mbsc-fr-btn mbsc-fr-btn-e"></div><div role="button" class="mbsc-cal-year"></div><div data-step="12" role="button" tabindex="0" aria-label="'+s.nextYearText+'" class="'+h+' mbsc-cal-next mbsc-cal-next-y mbsc-cal-btn mbsc-fr-btn mbsc-fr-btn-e"></div></div>'),r&&(l=e_(d)),t='<div class="mbsc-w-p mbsc-cal-c"><div class="mbsc-cal '+(g?"":" mbsc-cal-week-view")+(c>1?" mbsc-cal-multi ":"")+(Y?" mbsc-cal-weeks ":"")+(p?" mbsc-cal-vertical":"")+(ey?" mbsc-cal-has-marks":"")+(I?" mbsc-cal-has-labels":"")+(eo?"":" mbsc-cal-hide-diff ")+(s.calendarClass||"")+'"'+(S?"":' style="width:'+(e1||280*c)+'px;"')+'><div class="mbsc-cal-hdr">'+(ed<ef||c>1?a+o:o+a)+"</div>",r){for(t+='<div class="mbsc-cal-body"><div class="mbsc-cal-day-picker"><div class="mbsc-cal-days-c">',n=0;n<c;n++){for(t+='<div class="mbsc-cal-days">',e=0;e<7;e++)i=(e+j)%7,t+='<div class="mbsc-cal-week-day'+i+'" aria-label="'+s.dayNames[i]+'">'+s["dayNames"+eT][i]+"</div>";t+="</div>"}t+='</div><div class="mbsc-cal-scroll-c mbsc-cal-day-scroll-c '+(s.calendarClass||"")+'"'+(s.calendarHeight?' style="height:'+s.calendarHeight+'px"':"")+'><div class="mbsc-cal-scroll" style="width:'+100/c+'%">'+l+"</div></div>"}if(t+="</div>",F){for(t+='<div class="mbsc-cal-month-picker mbsc-cal-picker mbsc-cal-h"><div class="mbsc-cal-scroll-c '+(s.calendarClass||"")+'"><div class="mbsc-cal-scroll">',e=0;e<3;e++){for(t+='<div class="mbsc-cal-slide"'+ek(e-1)+'><div role="grid" class="mbsc-cal-table"><div class="mbsc-cal-row">',n=0;n<12;n++)n&&n%3===0&&(t+='</div><div class="mbsc-cal-row">'),t+='<div role="gridcell"'+(1==e?' tabindex="-1" aria-label="'+s.monthNames[n]+'" data-val="'+n+'"':"")+' class="mbsc-cal-cell'+(1==e?" mbsc-btn-e":"")+'"><div class="mbsc-cal-cell-i mbsc-cal-cell-txt">'+(1==e?s.monthNamesShort[n]:"&nbsp;")+"</div></div>";t+="</div></div></div>"}for(t+="</div></div></div>",t+='<div class="mbsc-cal-year-picker mbsc-cal-picker mbsc-cal-h"><div class="mbsc-cal-scroll-c '+(s.calendarClass||"")+'"><div class="mbsc-cal-scroll">',e=-1;e<2;e++)t+=eq(eM(d,e),e);t+="</div></div></div>"}return t+="</div></div></div>"}function eq(o,i){var t,e=s.getYear(o),n='<div class="mbsc-cal-slide"'+ek(i)+'><div role="grid" class="mbsc-cal-table"><div class="mbsc-cal-row">';for(t=0;t<12;t++)t&&t%3===0&&(n+='</div><div class="mbsc-cal-row">'),n+='<div role="gridcell" tabindex="-1" aria-label="'+e+'" data-val="'+e+'" class="mbsc-cal-cell mbsc-btn-e '+(e<em||e>e3?" mbsc-disabled ":"")+(e==s.getYear(d)?z:"")+'"><div class="mbsc-cal-cell-i mbsc-cal-cell-txt">'+e+e7+"</div></div>",e++;return n+="</div></div></div>"}function eK(O,S){var e,n,i,c,l,d,h,m,f,p,g,v,b,_,w,x,a=1,A=s.getYear(O),y=s.getMonth(O),C=s.getDay(O),E=null!==s.defaultValue||o._hasValue?o.getDate(!0):null,k=s.getDate(A,y,C).getDay(),M=j-k>0?7:0,u='<div class="mbsc-cal-slide"'+ek(S)+'><div role="grid" class="mbsc-cal-table"><div class="mbsc-cal-row">';for(i=0;i<7*r;i++)_=i+j-M,n=s.getDate(A,y,_-k+C),h=n.getFullYear(),l=n.getMonth(),d=n.getDate(),b=s.getMonth(n),c=s.getDay(n),x=s.getMaxDayOfMonth(h,l),f=h+"-"+(l+1)+"-"+d,e=t({valid:eJ(n),selected:E&&R(E,n)},to(n)),g=e.valid,v=e.selected,p=e.cssClass,w=new Date(n).setHours(12,0,0,0)===(new Date).setHours(12,0,0,0),m=b!==y,e5[f]=e,i&&i%7===0&&(u+='</div><div class="mbsc-cal-row">'),Y&&i%7===0&&("month"==Y&&m&&a>1?a=1==d?1:2:"year"==Y&&(a=s.getWeekNumber(s.getDate(h,l,d+(7-j+1)%7))),u+='<div role="gridcell" class="mbsc-cal-cell mbsc-cal-week-nr">'+a+"</div>",a++),u+='<div role="gridcell" aria-label="'+(w?s.todayText+", ":"")+s.dayNames[n.getDay()]+", "+s.monthNames[b]+" "+c+" "+(e.ariaLabel?", "+e.ariaLabel:"")+'"'+(m&&!eo?' aria-hidden="true"':' data-full="'+f+'"')+(v?' aria-selected="true"':"")+(g?' tabindex="-1"':' aria-disabled="true"')+' class="mbsc-cal-cell mbsc-cal-day mbsc-cal-day'+_%7+" "+(s.dayClass||"")+" "+(v?z:"")+(w?" "+s.todayClass:"")+(p?" "+p:"")+(1==c?" mbsc-cal-day-first":"")+(c==x?" mbsc-cal-day-last":"")+(m?" mbsc-cal-day-diff":"")+(g?" mbsc-btn-e":" mbsc-disabled")+(e.marked?" mbsc-cal-day-marked":"")+(e.background?" mbsc-cal-day-colored":"")+'"><div class="mbsc-cal-cell-i mbsc-cal-day-i"><div class="mbsc-cal-day-date mbsc-cal-cell-txt"'+(e.background?' style="background:'+e.background+";color:"+e6(e.background)+'"':"")+">"+c+"</div>"+(e.markup||"")+"</div></div>";return u+="</div></div></div>"}function e_(e,d,c){var t,o=s.getYear(e),i=s.getMonth(e),l=a?a.pos:0,n="";if(e5={},r)for(d||(f("onMonthLoading",{year:o,month:i}),f("onPageLoading",{firstDay:e})),eN(e),t=0;t<ej;t++)n+=eK(C(e,t-y-E),l*O+t-E);return eh=void 0,c&&a&&(a.$active=null,a.$scroller.html(n),eX(e,o,i)),n}function ex(n,s){if(a){var t=a.$active;t&&t.length&&(t[0].blur(),t.hasClass("mbsc-disabled")?t.removeAttr("tabindex"):t.attr("tabindex","-1")),a.$active=e('.mbsc-cal-slide-a .mbsc-cal-day[data-full="'+ec(n)+'"]',a.$scroller).attr("tabindex","0"),s&&a.$active.length&&a.$active[0].focus()}}function ts(n){var t=a&&a.$scroller;s.highlight&&a&&(e(".mbsc-selected",t).removeClass(z).removeAttr("aria-selected"),(null!==s.defaultValue||o._hasValue)&&e('.mbsc-cal-day[data-full="'+ec(n)+'"]',t).addClass(z).attr("aria-selected","true"))}function eP(n,t){e(".mbsc-selected",t).removeClass(z).removeAttr("aria-selected"),e('.mbsc-cal-cell[data-val="'+n+'"]',t).addClass(z).attr("aria-selected","true")}function H(e,i,l,u){var t,n;b&&(e<v&&(e=v),e>_&&(e=_),"calendar"!==b&&U&&!i||(o._isSetDate=!i,h&&r&&(n=G(eY(e),g),ev&&(e<C(d,-y)||e>=C(d,c-y))&&(t=g?s.getMonth(n)-s.getMonth(d)+12*(s.getYear(n)-s.getYear(d)):Math.floor(e8(d,n)/(7*r)),t&&(a.queue=[],a.focus=u&&l,ea(a,t,l))),t&&l||ex(e,u),i||ts(e),g||eC(e,!0),x=e,ev=!0),o._onSetDate(e,t),o._isSetDate=!1))}function eC(o,u){var t,r,d,n=s.getYear(o),a=s.getMonth(o),i=n+e7;if(F){if(eP(a,M.$scroller),eP(n,P.$scroller),ea(P,Math.floor(n/12)-Math.floor(s.getYear(P.first)/12),!0),e(".mbsc-cal-cell",M.$scroller).removeClass("mbsc-disabled"),n===em)for(t=0;t<eV;t++)e('.mbsc-cal-cell[data-val="'+t+'"]',M.$scroller).addClass("mbsc-disabled");if(n===e3)for(t=eL+1;t<=12;t++)e('.mbsc-cal-cell[data-val="'+t+'"]',M.$scroller).addClass("mbsc-disabled")}for(u||(e9(e(".mbsc-cal-prev-m",l),C(o,-y)<=v),e9(e(".mbsc-cal-next-m",l),C(o,c-y)>_),e9(e(".mbsc-cal-prev-y",l),s.getDate(n-1,a+1,1)<=v),e9(e(".mbsc-cal-next-y",l),s.getDate(n+1,a,1)>_)),ew.attr("aria-label",n).html(i),t=0;t<c;t++)o=s.getDate(n,a-y+t,1),d=s.getYear(o),r=s.getMonth(o),i=d+e7,Q.eq(t).attr("aria-label",s.monthNames[r]+(N?"":" "+n)).html((!N&&ed<ef?i+" ":"")+eg[r]+(!N&&ed>ef?" "+i:""))}function e9(e,t){t?e.addClass(eO).attr("aria-disabled","true"):e.removeClass(eO).removeAttr("aria-disabled")}function ez(i,v){var p,b,d=o.getDate(!0),r=i[0],c=i.attr("data-full"),l=c?c.split("-"):[],a=m(l[0],l[1]-1,l[2]),u=m(a.getFullYear(),a.getMonth(),a.getDate(),d.getHours(),d.getMinutes(),d.getSeconds()),g=i.hasClass("mbsc-selected"),n=e(v.target),h=n[0];if(eo||!i.hasClass("mbsc-cal-day-diff")){if(I&&r.contains(h))for(;h!=r;){if(n.hasClass("mbsc-cal-txt")||n.hasClass("mbsc-cal-txt-more")){if(b=n.attr("data-index"),p=eb[a],f("onLabelTap",{date:u,domEvent:v,target:n[0],labels:p,label:p[b]})===!1)return;break}n=n.parent(),h=n[0]}f("onDayChange",t(e5[c],{date:u,target:r,selected:g}))===!1||s.readonly||i.hasClass("mbsc-disabled")||o._selectDay(i,a,u,g)}}function ta(e){T(X),H(s.getDate(s.getYear(a.first),e.attr("data-val"),1),!0,!0)}function tr(e){T(q),H(s.getDate(e.attr("data-val"),s.getMonth(a.first),1),!0,!0)}function G(e,a,t){var n=s.getYear(e),o=s.getMonth(e),i=e.getDay(),r=j-i>0?7:0;return a?s.getDate(n,o,1):s.getDate(n,o,(void 0===t?j:t)-r-i+s.getDay(e))}function C(e,t){var n=s.getYear(e),o=s.getMonth(e),i=s.getDay(e);return g?s.getDate(n,o+t,1):s.getDate(n,o,i+t*r*7)}function eM(e,t){var n=12*Math.floor(s.getYear(e)/12);return s.getDate(n+12*t,0,1)}function ea(e,t,n,s){t&&o._isVisible&&(e.queue.push(arguments),1==e.queue.length&&eF(e,t,n,s))}function eF(t,n,b,g){var s,i,m="",o=t.$scroller,r=t.buffer,d=t.offset,u=t.pages,v=t.total,l=t.first,f=t.genPage,c=t.getFirst,a=n>0?Math.min(n,r):Math.max(n,-r),p=t.pos*O+a-n+d,h=Math.abs(n)>r;t.callback&&(t.load(),t.callback(!0)),t.first=c(l,n),t.pos+=a*O,t.changing=!0,t.load=function(){if(h){for(s=0;s<u;s++)i=n+s-d,m+=f(c(l,i),p+i);n>0?(e(".mbsc-cal-slide",o).slice(-u).remove(),o.append(m)):n<0&&(e(".mbsc-cal-slide",o).slice(0,u).remove(),o.prepend(m))}},t.callback=function(b){var m=Math.abs(a),u="";for(s=0;s<m;s++)i=n+s-d-r+(n>0?v-m:0),u+=f(c(l,i),p+i);if(n>0?(o.append(u),e(".mbsc-cal-slide",o).slice(0,a).remove()):n<0&&(o.prepend(u),e(".mbsc-cal-slide",o).slice(a).remove()),h){for(u="",s=0;s<m;s++)i=n+s-d-r+(n>0?0:v-m),u+=f(c(l,i),p+i);n>0?(e(".mbsc-cal-slide",o).slice(0,a).remove(),o.prepend(u)):n<0&&(e(".mbsc-cal-slide",o).slice(a).remove(),o.append(u))}ei(t),g&&!b&&g(),t.callback=null,t.load=null,t.queue.shift(),h=!1,t.queue.length?eF.apply(this,t.queue[0]):(t.changing=!1,t.onAfterChange(t.first))},t.onBeforeChange(t.first),t.load&&(t.load(),t.scroller.scroll(-t.pos*t.size,b?200:0,!1,t.callback))}function eE(r,v,f,a,d,c,l,b,u,h,m,i,g){var n=p?"Y":"X",t={$scroller:e(".mbsc-cal-scroll",r),queue:[],buffer:a,offset:d,pages:c,first:b,total:l,pos:0,min:v,max:f,genPage:i,getFirst:g,onBeforeChange:h,onAfterChange:m};return t.scroller=new B(r,{axis:n,easing:"",contSize:0,maxSnapScroll:a,mousewheel:void 0===s.mousewheel?p:s.mousewheel,time:200,lock:!0,rtl:K,stopProp:!1,minScroll:0,maxScroll:0,onBtnTap:function(t){"touchend"==t.domEvent.type&&k(),u(e(t.target),t.domEvent)},onAnimationStart:function(){t.changing=!0},onAnimationEnd:function(e){i&&ea(t,Math.round((-t.pos*t.size-e["pos"+n])/t.size)*O)}}),o._scrollers.push(t.scroller),t}function ei(e,a){var n,s=0,o=0,i=e.first;if(!e.changing||!a){if(e.getFirst){for(s=e.buffer,o=e.buffer;o&&e.getFirst(i,o+e.pages-e.offset-1)>e.max;)o--;for(;s&&e.getFirst(i,1-s-e.offset)<=e.min;)s--}n=Math.round(eu/e.pages),S&&n&&e.size!=n&&e.$scroller[p?"height":"width"](n),t(e.scroller.settings,{snap:n,minScroll:(-e.pos*O-o)*n,maxScroll:(-e.pos*O+s)*n}),e.size=n,e.scroller.refresh()}}function eR(e){o._onRefresh(e),o._isVisible&&h&&r&&(a&&a.changing?eh=e:(e_(d,e,!0),ex(x)))}function tl(){if(h&&r){var t=e(".mbsc-cal-scroll-c",l);a=eE(t[0],v,_,E,y,c,ej,d,ez,ti,eS,eK,C),F&&(M=eE(t[1],null,null,1,0,1,3,d,ta),P=eE(t[2],eB,eD,1,0,1,3,d,tr,n,n,eq,eM),o.tap(Q,function(){eW(X),T(q)}),o.tap(ew,function(){eW(q),T(X)})),ep(e(".mbsc-cal-btn",l),function(n,e,s,t){ea(a,e,!0,t)}),eS(d),null===s.defaultValue&&!o._hasValue||o._multiple||(o._activeElm=a.$active[0]),D.on("keydown",function(i){var n,e=s.getYear(x),t=s.getMonth(x),o=s.getDay(x);switch(i.keyCode){case 32:ez(a.$active,i);break;case 37:n=s.getDate(e,t,o-1*O);break;case 39:n=s.getDate(e,t,o+1*O);break;case 38:n=s.getDate(e,t,o-7);break;case 40:n=s.getDate(e,t,o+7);break;case 36:n=s.getDate(e,t,1);break;case 35:n=s.getDate(e,t+1,0);break;case 33:n=i.altKey?s.getDate(e-1,t,o):g?s.getDate(e,t-1,o):s.getDate(e,t,o-7*r);break;case 34:n=i.altKey?s.getDate(e+1,t,o):g?s.getDate(e,t+1,o):s.getDate(e,t,o+7*r)}n&&(i.preventDefault(),H(n,!0,!1,!0))})}o.tap(e(".mbsc-cal-tab",l),function(){o.changeTab(e(this).attr("data-control"))})}var s,a,r,c,l,d,u,h,f,p,g,v,b,j,y,_,O,x,E,S,M,F,z,D,N,L,P,I,V,$,W,U,K,q,Y,X,Q,ee,en,ew,eU,eu,e1,eG,eO,e4,e5,ey,eQ,et,eb,el,e0,eD,e2,eL,e3,eB,eV,em,ef,eg,ev,eh,ej,eo,e$,eT,ed,e7,tu=1,eZ=this;return tc(),eU=eH.call(this,o),tn(),o.refresh=function(){eR(!1)},o.redraw=function(){eR(!0)},o.navigate=function(e,t){H(i(e,L,s),!0,t)},o.changeTab=function(t){o._isVisible&&u[t]&&b!=t&&(b=t,e(".mbsc-cal-tab",l).removeClass(e4).removeAttr("aria-selected"),e('.mbsc-cal-tab[data-control="'+t+'"]',l).addClass(e4).attr("aria-selected","true"),U&&(W.addClass("mbsc-cal-h"),u[b].removeClass("mbsc-cal-h")),"calendar"==b&&H(o.getDate(!0),!1,!0),o._showDayPicker(),o.trigger("onTabChange",{tab:b}))},o._checkSize=!0,o._onGenMonth=n,o._onSelectShow=n,o._onSetDate=n,o._onRefresh=n,o._getDayProps=n,o._prepareObj=Z,o._showDayPicker=function(){F&&(T(q,!0),T(X,!0))},o._selectDay=o.__selectDay=function(n,i,t){var e=o.live;ev=s.outerMonthChange,et=!0,o.setDate(t,e,1e3,!e,!0),e&&f("onSet",{valueText:o._value})},t(eU,{labels:null,compClass:"mbsc-calendar mbsc-dt mbsc-sc",onMarkupReady:function(n){var t=0;l=e(n.target),en=e(".mbsc-fr-c",l),x=o.getDate(!0),eu=0,h&&(ey=!(!s.marked&&!s.data||s.labels||s.multiLabel||s.showEventCount),ev=!0,b="calendar",tt(),d=G(eY(x),g),en.append(te()),Q=e(".mbsc-cal-month",l),ew=e(".mbsc-cal-year",l),D=e(".mbsc-cal-day-scroll-c",l)),F&&(q=e(".mbsc-cal-year-picker",l),X=e(".mbsc-cal-month-picker",l)),W=e(".mbsc-w-p",l),V.length>1&&en.before(t1()),["date","time","calendar"].forEach(function(e){u[e]?(u[e]=W.eq(t),t++):"date"==e&&!u.date&&h&&(W.eq(t).remove(),t++)}),V.forEach(function(e){en.append(u[e])}),!h&&u.date&&u.date.css("position","relative"),o._scrollers=[],tl()},onShow:function(){h&&r&&eC(g?d:x)},onHide:function(){o._scrollers.forEach(function(e){e.destroy()}),e5=null,a=null,M=null,P=null,b=null},onValidated:function(s){var t,e=s.index,n=o._order,i=o.getDate(!0);et?t="calendar":void 0!==e&&(t=n.dd==e||n.d==e||n.m==e||n.y==e?"date":"time"),f("onSetDate",{date:i,control:t}),"time"!==t&&H(i,!1,!!s.time,et&&!o._multiple),et=!1},onPosition:function(t){if(_=t.oldHeight,i=t.windowHeight,U=(t.hasTabs||s.tabs===!0||s.tabs!==!1&&S)&&V.length>1,S&&(t.windowWidth>=s.breakPointMd?e(t.target).addClass("mbsc-fr-md"):e(t.target).removeClass("mbsc-fr-md")),U?(l.addClass("mbsc-cal-tabbed"),b=e(".mbsc-cal-tab.mbsc-selected",l).attr("data-control"),W.addClass("mbsc-cal-h"),u[b].removeClass("mbsc-cal-h")):(l.removeClass("mbsc-cal-tabbed"),W.removeClass("mbsc-cal-h")),o._isFullScreen&&(D.height(""),g=t.popup.offsetHeight,v=i-g+D[0].offsetHeight,i>=g&&D.height(v)),I&&r&&i!=_){var n,i,m,f,g,v,j,_,w,C=v||D[0].offsetHeight,O=e(".mbsc-cal-txt-ph")[0],E=O.offsetTop,k=O.offsetHeight,x=Math.max(1,Math.floor((C/r-E)/(k+2)));e2!=x&&(e2=x,o.redraw())}if(h&&r){if(f=S||p||U?D[0][p?"offsetHeight":"offsetWidth"]:e1||280*c,m=f!=eu,eu=f,S&&m&&N)for(eg=s.maxMonthWidth>Q[0].offsetWidth?s.monthNamesShort:s.monthNames,w=s.getYear(d),j=s.getMonth(d),n=0;n<c;n++)Q.eq(n).text(eg[s.getMonth(s.getDate(w,j-y+n,1))]);m&&ei(a,!0)}F&&m&&(ei(M,!0),ei(P,!0))}})},eV={};return G.calendar=function(o){function y(e){return m(e.getFullYear(),e.getMonth(),e.getDate())}function u(e){var t,o,a=null;if(s={},e&&e.length)for(o=0;o<e.length;o++)t=i(e[o],c,n,n.isoParts),a=a||t,s[y(t)]=t;return a}function f(){o.redraw()}var r,c,d,p,j,_=t({},o.settings),n=t(o.settings,eV,_),g="mbsc-selected "+(n.selectedClass||""),l=n.defaultValue,a="multiple"==n.select||n.select>1||"week"==n.selectType,b=h(n.select)?n.select:1/0,s={};return r=eB.call(this,o),j=void 0===n.firstSelectDay?n.firstDay:n.firstSelectDay,c=o._format,a&&u(l),o._multiple=a,o._getDayProps=function(e){return{selected:a?void 0!==s[e]:void 0}},o._selectDay=function(p,t,u,h){if(n.setOnDayTap&&"multiple"!=n.select&&"inline"!=n.display)return o.setDate(u),void o.select();if(a)if("week"==n.selectType){var r,c,l,i=t.getDay()-j;for(i=i<0?7+i:i,"multiple"!=n.select&&(s={}),r=0;r<7;r++)c=m(t.getFullYear(),t.getMonth(),t.getDate()-i+r),h?delete s[c]:N(s).length/7<b&&(s[c]=c);f()}else l=e('.mbsc-cal-day[data-full="'+p.attr("data-full")+'"]',d),h?(l.removeClass(g).removeAttr("aria-selected"),delete s[t]):N(s).length<b&&(l.addClass(g).attr("aria-selected","true"),s[t]=t);o.__selectDay(p,t,u)},o.setVal=function(e,t,n,s,i){a&&(e=u(e)),o._setVal(e,t,n,s,i),a&&f()},o.getVal=function(i){var e,t=[];if(a){for(e in s)t.push(X(s[e],n,c));return t}return X(o.getDate(i),n,c)},t({},r,{highlight:!a,outerMonthChange:!a,parseValue:function(e){return a&&e&&"string"==typeof e&&(e=u(e.split(","))),a&&l&&l.length&&(n.defaultValue=l[0]),r.parseValue.call(this,e)},formatValue:function(i){var e,t=[];if(a){for(e in s)t.push(v(c,s[e],n));return t.join(", ")}return r.formatValue.call(this,i,o)},onClear:function(){a&&(s={},f())},onBeforeShow:function(){void 0!==n.setOnDayTap||n.buttons&&n.buttons.length||1!=n.controls.length||(n.setOnDayTap=!0),n.setOnDayTap&&"inline"!=n.display&&(n.outerMonthChange=!1),n.counter&&a&&(n.headerText=function(){var t=0,o="week"==n.selectType?7:1;return e.each(s,function(){t++}),t=Math.round(t/o),(t>1?n.selectedPluralText||n.selectedText:n.selectedText).replace(/{count}/,t)})},onMarkupReady:function(n){r.onMarkupReady.call(this,n),d=e(n.target),a&&(e(".mbsc-fr-hdr",d).attr("aria-live","off"),p=t({},s))},onCancel:function(){!o.live&&a&&(s=t({},p))}})},eu("calendar",z),s.customTheme("mobiscroll-dark","mobiscroll"),ei=s.themes,g="mobiscroll","android"==r?g="material":"ios"==r?g="ios":"wp"==r&&(g="windows"),e.each(ei.frame,function(e,t){return g&&t.baseTheme==g&&"mobiscroll-dark"!=e&&"material-dark"!=e&&"windows-dark"!=e&&"ios-dark"!=e?(s.autoTheme=e,!1):void(e==g&&(s.autoTheme=e))}),s}),!function(e,t){"object"==typeof exports&&"undefined"!=typeof module?module.exports=t(require("jquery")):"function"==typeof define&&define.amd?define(["jquery"],t):(e=e||self).mobiscroll=t(e.jQuery)}(this,function(z){"use strict";function U(e){return(U="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function E(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function n3(s,n){for(var e,t=0;t<n.length;t++)e=n[t],e.enumerable=e.enumerable||!1,e.configurable=!0,"value"in e&&(e.writable=!0),Object.defineProperty(s,e.key,e)}function Y(e,t,n){return t&&n3(e.prototype,t),n&&n3(e,n),e}function G(t,e){if("function"!=typeof e&&null!==e)throw new TypeError("Super expression must either be null or a function");t.prototype=Object.create(e&&e.prototype,{constructor:{value:t,writable:!0,configurable:!0}}),e&&nl(t,e)}function g(e){return(g=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)})(e)}function nl(e,t){return(nl=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e})(e,t)}function ea(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function q(t,e){return!e||"object"!=typeof e&&"function"!=typeof e?ea(t):e}function er(e,t,n){return(er="undefined"!=typeof Reflect&&Reflect.get?Reflect.get:function(s,t,o){var e,n=function(e,t){for(;!Object.prototype.hasOwnProperty.call(e,t)&&null!==(e=g(e)););return e}(s,t);if(n)return e=Object.getOwnPropertyDescriptor(n,t),e.get?e.get.call(o):e.value})(e,t,n||e)}z=z&&z.hasOwnProperty("default")?z.default:z;var t=t||{},eg={},eG={},no=z.extend,nt={};function tG(n,e,o){var s=n;return"object"===U(e)?n.each(function(){new e.component(this,e)}):("string"==typeof e&&n.each(function(){var i,n=t.instances[this.id];if(n&&n[e]&&void 0!==(i=n[e].apply(this,Array.prototype.slice.call(o,1))))return s=i,!1}),s)}function r(e,t,n){nt[e]=function(s){return tG(this,no(s,{component:t,preset:!1===n?void 0:e}),arguments)}}(t.$=z).mobiscroll=t,z.fn.mobiscroll=function(e){return no(this,nt),tG(this,e,arguments)};var h,m,w,x,k,N,R,W,X,Q,eP,t$,ez=[],y="undefined"!=typeof window,n_=y&&window.matchMedia&&window.matchMedia("(prefers-color-scheme:dark)").matches,e1=y?navigator.userAgent:"",n9=y?navigator.platform:"",nw=y?navigator.maxTouchPoints:0,nx=/Safari/.test(e1),ek=e1.match(/Android|iPhone|iPad|iPod|Windows Phone|Windows|MSIE/i),F=y&&window.requestAnimationFrame||function(e){return setTimeout(e,20)},eY=y&&window.cancelAnimationFrame||function(e){clearTimeout(e)},tv,e6,ef,et,ts,nd,tw,eA,tY,t1,e0,t0,t2,t3,th,tC,t8,tp,tg,t_,e7,tf,tE,tP,tH,tB,tV,ns,na,n0,n2,tl,nh,tu,n8,t6,n6,t5,ng,n5,nv,es,n7,te,ne,tZ,nj,tX,tW,tz,tR,eC,eu,en,ec,ee,em,nf;function s(){}function eH(e){var t,n=[];for(t in e)n.push(e[t]);return n}function $(t){var e,n={};if(t)for(e=0;e<t.length;e++)n[t[e]]=t[e];return n}function v(e){return 0<=e-parseFloat(e)}function L(e){return"string"==typeof e}function f(e,t,n){return Math.max(t,Math.min(e,n))}function j(e,t){for(e+="",t=t||2;e.length<t;)e="0"+e;return e}function eq(n,e){var t,s;return e=e||100,function(){var i=this,o=+new Date,a=arguments;t&&o<t+e?(clearTimeout(s),s=setTimeout(function(){t=o,n.apply(i,a)},e)):(t=o,n.apply(i,a))}}function eX(e){"vibrate"in navigator&&navigator.vibrate(e||50)}function _(t,e,n){return 100*(t-e)/(n-e)}function ti(t,n,s){var e=s.attr(t);return void 0===e||""===e?n:"true"===e}/Android/i.test(ek)?(h="android",(eP=e1.match(/Android\s+([\d.]+)/i))&&(ez=eP[0].replace("Android ","").split("."))):/iPhone|iPad|iPod/i.test(ek)||/iPhone|iPad|iPod/i.test(n9)||"MacIntel"===n9&&1<nw?(h="ios",(eP=e1.match(/OS\s+([\d_]+)/i))&&(ez=eP[0].replace(/_/g,".").replace("OS ","").split("."))):/Windows Phone/i.test(ek)?h="wp":/Windows|MSIE/i.test(ek)&&(h="windows"),x=ez[0],t$=ez[1],e6=0;function P(){e6++,setTimeout(function(){e6--},500)}function tb(s,e){if(!e.mbscClick){var t=(s.originalEvent||s).changedTouches[0],n=document.createEvent("MouseEvents");n.initMouseEvent("click",!0,!0,window,1,t.screenX,t.screenY,t.clientX,t.clientY,!1,!1,!1,!1,0,null),n.isMbscTap=!0,n.isIonicTap=!0,tv=!0,e.mbscChange=!0,e.mbscClick=!0,e.dispatchEvent(n),tv=!1,P(),setTimeout(function(){delete e.mbscClick})}}function o(t,s,o){var e=t.originalEvent||t,n=(o?"page":"client")+s;return e.targetTouches&&e.targetTouches[0]?e.targetTouches[0][n]:e.changedTouches&&e.changedTouches[0]?e.changedTouches[0][n]:t[n]}function ej(t){var e,n=["switch","range","rating","segmented","stepper"],i=t[0],o=t.attr("data-role"),s=t.attr("type")||i.nodeName.toLowerCase();if(/(switch|range|rating|segmented|stepper|select)/.test(o))s=o;else for(e=0;e<n.length;e++)t.is("[mbsc-"+n[e]+"]")&&(s=n[e]);return s}function t4(e,t,n){e.focus(),/(button|submit|checkbox|switch|radio)/.test(t)&&n.preventDefault(),/select/.test(t)||tb(n,e)}function eI(i,u,h,l,s,m){var e,n,a,c,d,r=(0,t.$)(u);s=s||9,i.settings.tap&&r.on("touchstart.mbsc",function(t){e||(l&&t.preventDefault(),e=this,c=o(t,"X"),a=o(t,"Y"),n=!1,d=new Date)}).on("touchcancel.mbsc",function(){e=!1}).on("touchmove.mbsc",function(t){e&&!n&&(Math.abs(o(t,"X")-c)>s||Math.abs(o(t,"Y")-a)>s)&&(n=!0)}).on("touchend.mbsc",function(t){e&&(m&&new Date-d<100||!n?tb(t,t.target):P(),e=!1)}),r.on("click.mbsc",function(e){l&&e.preventDefault(),h.call(this,e,i)})}function nT(e){if(e6&&!tv&&!e.isMbscTap&&("TEXTAREA"!=e.target.nodeName||"mousedown"!=e.type))return e.stopPropagation(),e.preventDefault(),!1}function tU(e){return e[0].innerWidth||e.innerWidth()}function tq(t){var e=t.theme,n=t.themeVariant;return"auto"!=e&&e||(e=I.autoTheme),"default"==e&&(e="mobiscroll"),("dark"===n||n_&&"auto"===n)&&I.themes.form[e+"-dark"]&&(e+="-dark"),e}function p(t,n,s){y&&e(function(){e(t).each(function(){new n(this,{})}),e(document).on("mbsc-enhance",function(s,o){e(s.target).is(t)?new n(s.target,o||{}):e(t,s.target).each(function(){new n(this,o||{})})}),s&&e(document).on("mbsc-refresh",function(s){var n;e(s.target).is(t)?(n=A[s.target.id])&&n.refresh():e(t,s.target).each(function(){(n=A[this.id])&&n.refresh()})})})}y&&(["mouseover","mousedown","mouseup","click"].forEach(function(e){document.addEventListener(e,nT,!0)}),"android"==h&&x<5&&document.addEventListener("change",function(e){e6&&"checkbox"==e.target.type&&!e.target.mbscChange&&(e.stopPropagation(),e.preventDefault()),delete e.target.mbscChange},!0));var I,e=t.$,nR=+new Date,A={},u={},nB={},nW={xsmall:0,small:576,medium:768,large:992,xlarge:1200},n=e.extend;n(eg,{getCoord:o,preventClick:P,vibrate:eX}),I=n(t,{$:e,version:"4.9.1",autoTheme:"mobiscroll",themes:{form:{},page:{},frame:{},scroller:{},listview:{},navigation:{},progress:{},card:{}},platform:{name:h,majorVersion:x,minorVersion:t$},uid:"c5d09426",i18n:{},instances:A,classes:u,util:eg,settings:{},setDefaults:function(e){n(this.settings,e)},customTheme:function(a,o){var e,i=t.themes,s=["frame","scroller","listview","navigation","form","page","progress","card"];for(e=0;e<s.length;e++)i[s[e]][a]=n({},i[s[e]][o],{baseTheme:o})}});function S(a,i){var o,r,c,l,d,u,h,m,f,t=this;function p(s){var t,n=nB;return o.responsive&&(t=s||tU(f),e.each(o.responsive,function(s,e){t>=(e.breakpoint||nW[s])&&(n=e)})),n}t.settings={},t.element=a,t._init=s,t._destroy=s,t._processSettings=s,t._checkResp=function(n){if(t&&t._responsive){var e=p(n);if(r!==e)return r=e,t.init({}),!0}},t._getRespCont=function(){return e(o.context)[0]},t.init=function(e,s){var g,v;for(g in e&&t.getVal&&(v=t.getVal()),t.settings)delete t.settings[g];o=t.settings,n(i,e),t._hasDef&&(l=I.settings),n(o,t._defaults,l,i),t._hasTheme&&(u=tq(o),i.theme=u,d=I.themes[t._class]?I.themes[t._class][u]:{}),t._hasLang&&(m=I.i18n[o.lang]),n(o,d,m,l,i),f=t._getRespCont(),t._responsive&&(r=r||p(),n(o,r)),t._processSettings(r||{}),t._presets&&(c=t._presets[o.preset])&&(c=c.call(a,t,i),n(o,c,i,r)),t._init(e),e&&t.setVal&&t.setVal(void 0===s?v:s,!0),h("onInit")},t.destroy=function(){t&&(t._destroy(),h("onDestroy"),delete A[a.id],t=null)},t.tap=function(e,n,s,o,i){eI(t,e,n,s,o,i)},t.trigger=function(s,r){var e,n,o,u=[l,d,c,i];for(e=0;e<4;e++)(n=u[e])&&n[s]&&(o=n[s].call(a,r||{},t));return o},t.option=function(e,a,r){var n={},s=["data","invalid","valid","readonly"];/calendar|eventcalendar|range/.test(o.preset)&&s.push("marked","labels","colors"),"object"===U(e)?n=e:n[e]=a,s.forEach(function(e){i[e]=o[e]}),t.init(n,r)},t.getInst=function(){return t},i=i||{},h=t.trigger,t.__ready||(e(a).addClass("mbsc-comp"),a.id?A[a.id]&&A[a.id].destroy():a.id="mobiscroll"+ ++nR,(A[a.id]=t).__ready=!0)}function T(n,r,s,t,o,i,a){var e=new Date(n,r,s,t||0,o||0,i||0,a||0);return 23==e.getHours()&&0===(t||0)&&e.setHours(e.getHours()+2),e}function l(a,t,h){if(!t)return null;function c(t){for(var e=0;o+1<a.length&&a.charAt(o+1)==t;)e++,o++;return e}function i(t,n,s){var e=""+n;if(c(t))for(;e.length<s;)e="0"+e;return e}function u(t,e,n,s){return c(t)?s[e]:n[e]}var o,r,l,s=n({},ev,h),e="",d=!1;for(o=0;o<a.length;o++)if(d)"'"!=a.charAt(o)||c("'")?e+=a.charAt(o):d=!1;else switch(a.charAt(o)){case"d":e+=i("d",s.getDay(t),2);break;case"D":e+=u("D",t.getDay(),s.dayNamesShort,s.dayNames);break;case"o":e+=i("o",(t.getTime()-new Date(t.getFullYear(),0,0).getTime())/864e5,3);break;case"m":e+=i("m",s.getMonth(t)+1,2);break;case"M":e+=u("M",s.getMonth(t),s.monthNamesShort,s.monthNames);break;case"y":l=s.getYear(t),e+=c("y")?l:(l%100<10?"0":"")+l%100;break;case"h":r=t.getHours(),e+=i("h",12<r?r-12:0===r?12:r,2);break;case"H":e+=i("H",t.getHours(),2);break;case"i":e+=i("i",t.getMinutes(),2);break;case"s":e+=i("s",t.getSeconds(),2);break;case"a":e+=11<t.getHours()?s.pmText:s.amText;break;case"A":e+=11<t.getHours()?s.pmText.toUpperCase():s.amText.toUpperCase();break;case"'":c("'")?e+="'":d=!0;break;default:e+=a.charAt(o)}return e}function Z(u,t,x){if(e=n({},ev,x),i=a(e.defaultValue||new Date),!u||!t)return i;if(t.getTime)return t;t="object"==U(t)?t.toString():t+"";function g(t){var e=o+1<u.length&&u.charAt(o+1)==t;return e&&o++,e}function r(e){g(e);var s=new RegExp("^\\d{1,"+("@"==e?14:"!"==e?20:"y"==e?4:"o"==e?3:2)+"}"),n=t.substr(f).match(s);return n?(f+=n[0].length,parseInt(n[0],10)):0}function p(s,o,i){var e,n=g(s)?i:o;for(e=0;e<n.length;e++)if(t.substr(f,n[e].length).toLowerCase()==n[e].toLowerCase())return f+=n[e].length,e+1;return 0}function y(){f++}var e,o,i,h,v,b=e.shortYearCutoff,d=e.getYear(i),l=e.getMonth(i)+1,c=e.getDay(i),j=-1,s=i.getHours(),O=i.getMinutes(),w=0,m=-1,_=!1,f=0;for(o=0;o<u.length;o++)if(_)"'"!=u.charAt(o)||g("'")?y():_=!1;else switch(u.charAt(o)){case"d":c=r("d");break;case"D":p("D",e.dayNamesShort,e.dayNames);break;case"o":j=r("o");break;case"m":l=r("m");break;case"M":l=p("M",e.monthNamesShort,e.monthNames);break;case"y":d=r("y");break;case"H":s=r("H");break;case"h":s=r("h");break;case"i":O=r("i");break;case"s":w=r("s");break;case"a":m=p("a",[e.amText,e.pmText],[e.amText,e.pmText])-1;break;case"A":m=p("A",[e.amText,e.pmText],[e.amText,e.pmText])-1;break;case"'":g("'")?y():_=!0;break;default:y()}if(d<100&&(d+=(new Date).getFullYear()-(new Date).getFullYear()%100+(d<=("string"!=typeof b?b:(new Date).getFullYear()%100+parseInt(b,10))?0:-100)),-1<j){l=1,c=j;do v=32-new Date(d,l-1,32,12).getDate(),v<c&&(l++,c-=v);while(v<c)}return s=-1==m?s:m&&s<12?s+12:m||12!=s?s:0,h=e.getDate(d,l-1,c,s,O,w),e.getYear(h)!=d||e.getMonth(h)+1!=l||e.getDay(h)!=c?i:h}function nu(e,t){return Math.round((t-e)/864e5)}function M(e){return T(e.getFullYear(),e.getMonth(),e.getDate())}function e2(e){return e.getFullYear()+"-"+(e.getMonth()+1)+"-"+e.getDate()}function n1(o,i,t){var e,n,s={y:1,m:2,d:3,h:4,i:5,s:6,u:7,tz:8};if(t)for(e in s)(n=o[s[e]-i])&&(t[e]="tz"==e?n:1)}function K(e,t,o){var s=window.moment||t.moment,n=t.returnFormat;if(e){if("moment"==n&&s)return s(e);if("locale"==n)return l(o,e,t);if("iso8601"==n)return function(t,e){var n="",s="";return t&&(e.h&&(s+=j(t.getHours())+":"+j(t.getMinutes()),e.s&&(s+=":"+j(t.getSeconds())),e.u&&(s+="."+j(t.getMilliseconds(),3)),e.tz&&(s+=e.tz)),e.y?(n+=t.getFullYear(),e.m&&(n+="-"+j(t.getMonth()+1),e.d&&(n+="-"+j(t.getDate())),e.h&&(n+="T"+s))):e.h&&(n=s)),n}(e,t.isoParts)}return e}function a(t,s,o,n){var e;return t?t.getTime?t:t.toDate?t.toDate():("string"==typeof t&&(t=t.trim()),(e=nm.exec(t))?(n1(e,2,n),new Date(1970,0,1,e[2]?+e[2]:0,e[3]?+e[3]:0,e[4]?+e[4]:0,e[5]?+e[5]:0)):(e=e||nU.exec(t))?(n1(e,0,n),new Date(e[1]?+e[1]:1970,e[2]?e[2]-1:0,e[3]?+e[3]:1,e[4]?+e[4]:0,e[5]?+e[5]:0,e[6]?+e[6]:0,e[7]?+e[7]:0)):Z(s,t,o)):null}function b(e,t){return e.getFullYear()==t.getFullYear()&&e.getMonth()==t.getMonth()&&e.getDate()==t.getDate()}var nU=/^(\d{4}|[+-]\d{6})(?:-(\d{2})(?:-(\d{2}))?)?(?:T(\d{2}):(\d{2})(?::(\d{2})(?:\.(\d{3}))?)?((Z)|([+-])(\d{2})(?::(\d{2}))?)?)?$/,nm=/^((\d{2}):(\d{2})(?::(\d{2})(?:\.(\d{3}))?)?(?:(Z)|([+-])(\d{2})(?::(\d{2}))?)?)?$/,B=/^\d{1,2}(\/\d{1,2})?$/,C=/^w\d$/i,ev={shortYearCutoff:"+10",monthNames:["January","February","March","April","May","June","July","August","September","October","November","December"],monthNamesShort:["Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"],dayNames:["Sunday","Monday","Tuesday","Wednesday","Thursday","Friday","Saturday"],dayNamesShort:["Sun","Mon","Tue","Wed","Thu","Fri","Sat"],dayNamesMin:["S","M","T","W","T","F","S"],amText:"am",pmText:"pm",getYear:function(e){return e.getFullYear()},getMonth:function(e){return e.getMonth()},getDay:function(e){return e.getDate()},getDate:T,getMaxDayOfMonth:function(e,t){return 32-new Date(e,t,32,12).getDate()},getWeekNumber:function(e){(e=new Date(e)).setHours(0,0,0),e.setDate(e.getDate()+4-(e.getDay()||7));var t=new Date(e.getFullYear(),0,1);return Math.ceil(((e-t)/864e5+1)/7)}};function nq(e){var t;for(t in e)if(void 0!==ef[e[t]])return!0;return!1}function H(n,t){if("touchstart"==n.type)e(t).attr("data-touch","1");else if(e(t).attr("data-touch"))return e(t).removeAttr("data-touch"),!1;return!0}function t7(s,o){var t,n=getComputedStyle(s[0]);return e.each(["t","webkitT","MozT","OT","msT"],function(s,e){if(void 0!==n[e+"ransform"])return t=n[e+"ransform"],!1}),t=t.split(")")[0].split(", "),o?t[13]||t[5]:t[12]||t[4]}function np(t){if(t){if(eA[t])return eA[t];var s=e('<div style="background-color:'+t+';"></div>').appendTo("body"),n=getComputedStyle(s[0]).backgroundColor.replace(/rgb|rgba|\(|\)|\s/g,"").split(","),o=.299*n[0]+.587*n[1]+.114*n[2]<130?"#fff":"#000";return s.remove(),eA[t]=o}}function nK(o,s,i,t){var n=e(o);i?(n.scrollTop(s),t&&t()):function a(s,o,e,t,n){var r=Math.min(1,(new Date-o)/468),c=.5*(1-Math.cos(Math.PI*r)),i=e+(t-e)*c;s.scrollTop(i),i!==t?F(function(){a(s,o,e,t,n)}):n&&n()}(n,new Date,n.scrollTop(),s,t)}eg.datetime={formatDate:l,parseDate:Z},eA={};function eS(f,y,b,x,O,l){var t,n,i,a,r,h,m,p,g,w=x||s;function j(s){var c;t=e(this),r=+t.attr("data-step"),a=+t.attr("data-index"),i=!0,O&&s.stopPropagation(),"touchstart"==s.type&&t.closest(".mbsc-no-touch").removeClass("mbsc-no-touch"),"mousedown"==s.type&&s.preventDefault(),c="keydown"!=s.type?(g=o(s,"X"),p=o(s,"Y"),H(s,this)):32===s.keyCode,n||!c||t.hasClass("mbsc-disabled")||(v(a,r,s)&&(t.addClass("mbsc-active"),l&&l.addRipple(t.find(".mbsc-segmented-content"),s)),"mousedown"==s.type&&e(document).on("mousemove",d).on("mouseup",u))}function d(e){(7<Math.abs(g-o(e,"X"))||7<Math.abs(p-o(e,"Y")))&&(i=!0,c())}function u(t){"touchend"==t.type&&t.preventDefault(),c(),"mouseup"==t.type&&e(document).off("mousemove",d).off("mouseup",u)}function c(){n=!1,clearInterval(h),t&&(t.removeClass("mbsc-active"),l&&setTimeout(function(){l.removeRipple()},100))}function v(e,t,s){return n||w(e)||(a=e,r=t,m=s,i=!(n=!0),setTimeout(_,100)),n}function _(){t&&t.hasClass("mbsc-disabled")?c():(!n&&i||(i=!0,y(a,r,m,_)),n&&b&&(clearInterval(h),h=setInterval(function(){y(a,r,m)},b)))}return f.on("touchstart mousedown keydown",j).on("touchmove",d).on("touchend touchcancel keyup",u),{start:v,stop:c,destroy:function(){f.off("touchstart mousedown keydown",j).off("touchmove",d).off("touchend touchcancel keyup",u)}}}y&&(ef=document.createElement("modernizr").style,k=function(){var e,t=["Webkit","Moz","O","ms"];for(e in t)if(nq([t[e]+"Transform"]))return"-"+t[e].toLowerCase()+"-";return""}(),m=k.replace(/^-/,"").replace(/-$/,"").replace("moz","Moz"),W=void 0!==ef.animation?"animationend":"webkitAnimationEnd",ts=void 0!==ef.transition,tw=(nd="ios"===h&&!nx)&&window.webkit&&window.webkit.messageHandlers,et=void 0===ef.touchAction||nd&&!tw);var eJ="position:absolute;left:0;top:0;",nb=eJ+"right:0;bottom:0;overflow:hidden;z-index:-1;",n$='<div style="'+nb+'"><div style="'+eJ+'"></div></div><div style="'+nb+'"><div style="'+eJ+'width:200%;height:200%;"></div></div>';function nc(h,m,r){function u(){s.style.width="100000px",s.style.height="100000px",e.scrollLeft=1e5,e.scrollTop=1e5,n.scrollLeft=1e5,n.scrollTop=1e5}function o(){var t=new Date,a=0;c||(200<t-l&&!e.scrollTop&&!e.scrollLeft&&(l=t,u()),a=a||F(o))}function d(){i=i||F(f)}function f(){i=0,u(),m()}var e,n,s,i,a,c,l=0,t=document.createElement("div");return t.innerHTML=n$,t.dir="ltr",n=t.childNodes[1],e=t.childNodes[0],s=e.childNodes[0],h.appendChild(t),e.addEventListener("scroll",d),n.addEventListener("scroll",d),r?r.runOutsideAngular(function(){F(o)}):F(o),{detach:function(){h.removeChild(t),c=!0}}}function nV(e){e.preventDefault()}function O($,k,eo){var i,a,r,c,d,u,h,m,p,g,v,b,j,y,_,w,O,x,C,A,F,T,D,N,R,H,I,V,U,K,q,X,ee,n=this,l=e($),z=[],B=new Date;function ei(t){y&&y.removeClass("mbsc-active"),(y=e(this)).hasClass("mbsc-disabled")||y.hasClass("mbsc-fr-btn-nhl")||y.addClass("mbsc-active"),"mousedown"===t.type?e(document).on("mouseup",E):"pointerdown"===t.type&&e(document).on("pointerup",E)}function E(t){y&&(y.removeClass("mbsc-active"),y=null),"mouseup"===t.type?e(document).off("mouseup",E):"pointerup"===t.type&&e(document).off("pointerup",E)}function Q(s){t.activeInstance==n&&(13!=s.keyCode||e(s.target).is('textarea,button,input[type="button"],input[type="submit"]')&&!s.shiftKey?27==s.keyCode&&n.cancel():n.select())}function Z(e){e||tQ||!n._activeElm||(B=new Date,n._activeElm.focus())}function J(o){var s=eL,t=i.focusOnClose;n._markupRemove(),a.remove(),r&&(c.mbscModals--,i.scrollLock&&c.mbscLock--,c.mbscLock||O.removeClass("mbsc-fr-lock"),w&&(c.mbscIOSLock--,c.mbscIOSLock||(O.removeClass("mbsc-fr-lock-ios"),v.css({top:"",left:""}),p.scrollLeft(c.mbscScrollLeft),p.scrollTop(c.mbscScrollTop))),c.mbscModals||O.removeClass("mbsc-fr-lock-ctx"),c.mbscModals&&!D||o||(s=s||l,setTimeout(function(){void 0===t||!0===t?(eR=!0,s[0].focus()):t&&e(t)[0].focus()},200))),D=void 0,V=!1,u("onHide")}function ea(){clearTimeout(q),q=setTimeout(function(){n.position(!0)&&(j.style.visibility="hidden",j.offsetHeight,j.style.visibility="")},200)}function et(e){t.activeInstance==n&&e.target.nodeType&&!I.contains(e.target)&&100<new Date-B&&(B=new Date,n._activeElm.focus())}function en(b,f){if(r)a.appendTo(v);else if(l.is("div")&&!n._hasContent)l.empty().append(a);else if(l.hasClass("mbsc-control")){var t,s,m,p,c=l.closest(".mbsc-control-w");a.insertAfter(c),c.hasClass("mbsc-select")&&c.addClass("mbsc-select-inline")}else a.insertAfter(l);V=!0,n._markupInserted(a),u("onMarkupInserted",{target:g}),r&&i.closeOnOverlayTap&&R.on("touchstart mousedown",function(e){t||e.target!=I||(s=!(t=!0),p=o(e,"X"),m=o(e,"Y"))}).on("touchmove mousemove",function(e){t&&!s&&(9<Math.abs(o(e,"X")-p)||9<Math.abs(o(e,"Y")-m))&&(s=!0)}).on("touchcancel",function(){t=!1}).on("touchend click",function(e){t&&!s&&(n.cancel(),"touchend"==e.type&&P()),t=!1}),a.on("mousedown",".mbsc-btn-e,.mbsc-fr-btn-e",nV).on("touchstart mousedown",function(e){i.stopProp&&e.stopPropagation()}).on("keydown",".mbsc-fr-btn-e",function(e){32==e.keyCode&&(e.preventDefault(),e.stopPropagation(),this.click())}).on("keydown",function(t){if(32!=t.keyCode||e(t.target).is(nP)){if(9==t.keyCode&&r&&i.focusTrap){var n=a.find('input,select,textarea,button,[tabindex="0"]').filter(function(){return 0<this.offsetWidth||0<this.offsetHeight}),c=n.index(e(":focus",a)),s=n.length-1,o=0;t.shiftKey&&(s=0,o=-1),c===s&&(n.eq(o)[0].focus(),t.preventDefault())}}else t.preventDefault()}).on("touchstart mousedown pointerdown",".mbsc-fr-btn-e",ei).on("touchend",".mbsc-fr-btn-e",E),g.addEventListener("touchstart",function(){N||(N=!0,v.find(".mbsc-no-touch").removeClass("mbsc-no-touch"))},!0),e.each(h,function(s,t){n.tap(e(".mbsc-fr-btn"+s,a),function(e){t=L(t)?n.buttons[t]:t,(L(t.handler)?n.handlers[t.handler]:t.handler).call(this,e,n)},!0)}),n._attachEvents(a),!1!==n.position()&&((r||n._checkSize)&&(F=nc(g,ea,i.zone)),r&&(a.removeClass("mbsc-fr-pos"),d&&!b?a.addClass("mbsc-anim-in mbsc-anim-trans mbsc-anim-trans-"+d).on(W,function e(){a.off(W,e).removeClass("mbsc-anim-in mbsc-anim-trans mbsc-anim-trans-"+d).find(".mbsc-fr-popup").removeClass("mbsc-anim-"+d),Z(f)}).find(".mbsc-fr-popup").addClass("mbsc-anim-"+d):Z(f)),u("onShow",{target:g,valueText:n._tempValue}))}function M(e,t){n._isVisible||(e&&e(),!1!==n.show()&&(eL=t))}function G(){n._fillValue(),u("onSet",{valueText:n._value})}function Y(){u("onCancel",{valueText:n._value})}function es(){n.setVal(null,!0)}S.call(this,$,k,!0),n.position=function(Y){var t,s,d,y,w,E,k,S,M,F,z,D,N,R,P,B,$,W,q,L={},h=0,o=0,O=0,I=0;if(!V)return!1;if(B=C,$=T,t=g.offsetHeight,(s=g.offsetWidth)&&t&&(C!==s||T!==t||!Y)){if(n._checkResp(s))return!1;if(C=s,T=t,n._isFullScreen||/top|bottom/.test(i.display)?x.width(s):r&&U.width(""),n._position(a),!n._isFullScreen&&/center|bubble/.test(i.display)&&(e(".mbsc-w-p",a).each(function(){E=this.getBoundingClientRect().width,I+=E,O=O<E?E:O}),S=s-16<I||!0===i.tabs,U.css({width:n._isLiquid?Math.min(i.maxPopupWidth,s-16):Math.ceil(S?O:I),"white-space":S?"":"nowrap"})),!1!==u("onPosition",{target:g,popup:j,hasTabs:S,oldWidth:B,oldHeight:$,windowWidth:s,windowHeight:t})&&r)return A&&(h=p.scrollLeft(),o=p.scrollTop(),C&&K.css({width:"",height:""})),_=j.offsetWidth,m=j.offsetHeight,H=m<=t&&_<=s,"center"==i.display?(y=Math.max(0,h+(s-_)/2),d=Math.max(0,o+(t-m)/2)):"bubble"==i.display?(k=void 0===i.anchor?l:e(i.anchor),R=e(".mbsc-fr-arr-i",a)[0],w=(q=k.offset()).top+(b?o-v.offset().top:0),P=q.left+(b?h-v.offset().left:0),D=k[0].offsetWidth,z=k[0].offsetHeight,N=R.offsetWidth,M=R.offsetHeight,y=f(P-(_-D)/2,h+3,h+s-_-3),o+t<(d=w+z+M/2)+m+8&&o<w-m-M/2?(x.removeClass("mbsc-fr-bubble-bottom").addClass("mbsc-fr-bubble-top"),d=w-m-M/2):x.removeClass("mbsc-fr-bubble-top").addClass("mbsc-fr-bubble-bottom"),e(".mbsc-fr-arr",a).css({left:f(P+D/2-(y+(_-N)/2),0,N)}),H=o<d&&h<y&&d+m<=o+t&&y+_<=h+s):(y=h,d="top"==i.display?o:Math.max(0,o+t-m)),A&&(F=Math.max(d+m,b?c.scrollHeight:e(document).height()),W=Math.max(y+_,b?c.scrollWidth:e(document).width()),K.css({width:W,height:F}),i.scroll&&"bubble"==i.display&&(o+t<d+m+8||o+t<w||w+z<o)&&p.scrollTop(Math.min(w,d+m-t+8,F-t))),L.top=Math.floor(d),L.left=Math.floor(y),x.css(L),!0}},n.attachShow=function(a,o){var s,t=e(a).off(".mbsc"),r=t.prop("readonly");"inline"!==i.display&&((i.showOnFocus||i.showOnTap)&&t.is("input,select")&&(t.prop("readonly",!0).on("mousedown.mbsc",function(e){e.preventDefault()}).on("focus.mbsc",function(){n._isVisible&&this.blur()}),(s=e('label[for="'+t.attr("id")+'"]')).length||(s=t.closest("label"))),t.is("select")||(i.showOnFocus&&t.on("focus.mbsc",function(){eR?eR=!1:M(o,t)}),i.showOnTap&&(t.on("keydown.mbsc",function(e){32!=e.keyCode&&13!=e.keyCode||(e.preventDefault(),e.stopPropagation(),M(o,t))}),n.tap(t,function(e){e.isMbscTap&&(N=!0),M(o,t)}),s&&s.length&&n.tap(s,function(e){e.preventDefault(),e.target!==t[0]&&M(o,t)}))),z.push({readOnly:r,el:t,lbl:s}))},n.select=function(){r?n.hide(!1,"set",!1,G):G()},n.cancel=function(){r?n.hide(!1,"cancel",!1,Y):Y()},n.clear=function(){n._clearValue(),u("onClear"),r&&n._isVisible&&!n.live?n.hide(!1,"clear",!1,es):es()},n.enable=function(){i.disabled=!1,e.each(z,function(t,e){e.el.is("input,select")&&(e.el[0].disabled=!1)})},n.disable=function(){i.disabled=!0,e.each(z,function(t,e){e.el.is("input,select")&&(e.el[0].disabled=!0)})},n.show=function(E,k){if(!i.disabled&&!n._isVisible){if(n._readValue(),!1===u("onBeforeShow"))return!1;if(eL=null,d=i.animate,h=i.buttons||[],A=b||"bubble"==i.display,w=nI&&!A&&i.scrollLock,m=0<h.length,!1!==d&&("top"==i.display?d=d||"slidedown":"bottom"==i.display?d=d||"slideup":"center"!=i.display&&"bubble"!=i.display||(d=d||"pop")),r&&(T=C=0,w&&!O.hasClass("mbsc-fr-lock-ios")&&(c.mbscScrollTop=_=Math.max(0,p.scrollTop()),c.mbscScrollLeft=y=Math.max(0,p.scrollLeft()),v.css({top:-_+"px",left:-y+"px"})),O.addClass((i.scrollLock?"mbsc-fr-lock":"")+(w?" mbsc-fr-lock-ios":"")+(b?" mbsc-fr-lock-ctx":"")),e(document.activeElement).is("input,textarea")&&document.activeElement.blur(),D=t.activeInstance,t.activeInstance=n,c.mbscModals=(c.mbscModals||0)+1,w&&(c.mbscIOSLock=(c.mbscIOSLock||0)+1),i.scrollLock&&(c.mbscLock=(c.mbscLock||0)+1)),o='<div lang="'+i.lang+'" class="mbsc-fr mbsc-'+i.theme+(i.baseTheme?" mbsc-"+i.baseTheme:"")+" mbsc-fr-"+i.display+" "+(i.cssClass||"")+" "+(i.compClass||"")+(n._isLiquid?" mbsc-fr-liq":"")+(r?" mbsc-fr-pos"+(i.showOverlay?"":" mbsc-fr-no-overlay"):"")+(ee?" mbsc-fr-pointer":"")+(nH?" mbsc-fr-hb":"")+(N?"":" mbsc-no-touch")+(w?" mbsc-platform-ios":"")+(m?3<=h.length?" mbsc-fr-btn-block ":"":" mbsc-fr-nobtn")+'">'+(r?'<div class="mbsc-fr-persp">'+(i.showOverlay?'<div class="mbsc-fr-overlay"></div>':"")+'<div role="dialog" class="mbsc-fr-scroll">':"")+'<div class="mbsc-fr-popup'+(i.rtl?" mbsc-rtl":" mbsc-ltr")+(i.headerText?" mbsc-fr-has-hdr":"")+'">'+("bubble"===i.display?'<div class="mbsc-fr-arr-w"><div class="mbsc-fr-arr-i"><div class="mbsc-fr-arr"></div></div></div>':"")+(r?'<div class="mbsc-fr-focus" tabindex="-1"></div>':"")+'<div class="mbsc-fr-w">'+(i.headerText?'<div class="mbsc-fr-hdr">'+(L(i.headerText)?i.headerText:"")+"</div>":"")+'<div class="mbsc-fr-c">',o+=n._generateContent(),o+="</div>",m){var s,o,l,m,f,y,_,S=h.length;for(o+='<div class="mbsc-fr-btn-cont">',l=0;l<h.length;l++)f=i.btnReverse?S-l-1:l,"set"===(s=L(s=h[f])?n.buttons[s]:s).handler&&(s.parentClass="mbsc-fr-btn-s"),"cancel"===s.handler&&(s.parentClass="mbsc-fr-btn-c"),o+="<div"+(i.btnWidth?' style="width:'+100/h.length+'%"':"")+' class="mbsc-fr-btn-w '+(s.parentClass||"")+'"><div tabindex="0" role="button" class="mbsc-fr-btn'+f+" mbsc-fr-btn-e "+(void 0===s.cssClass?i.btnClass:s.cssClass)+(s.icon?" mbsc-ic mbsc-ic-"+s.icon:"")+'">'+(s.text||"")+"</div></div>";o+="</div>"}a=e(o+="</div></div></div></div>"+(r?"</div></div>":"")),K=e(".mbsc-fr-persp",a),R=e(".mbsc-fr-scroll",a),U=e(".mbsc-fr-w",a),x=e(".mbsc-fr-popup",a),X=e(".mbsc-fr-hdr",a),g=a[0],I=R[0],j=x[0],n._activeElm=e(".mbsc-fr-focus",a)[0],n._markup=a,n._isVisible=!0,n.markup=g,n._markupReady(a),u("onMarkupReady",{target:g}),r&&(e(window).on("keydown",Q),i.scrollLock&&a.on("touchmove mousewheel wheel",function(e){H&&e.preventDefault()}),i.focusTrap&&p.on("focusin",et)),r?setTimeout(function(){en(E,k)},w?100:0):en(E,k)}},n.hide=function(s,o,i,c){if(!n._isVisible||!i&&!n._isValid&&"set"==o||!i&&!1===u("onBeforeClose",{valueText:n._tempValue,button:o}))return!1;n._isVisible=!1,F&&(F.detach(),F=null),r&&(e(document.activeElement).is("input,textarea")&&j.contains(document.activeElement)&&document.activeElement.blur(),t.activeInstance==n&&(t.activeInstance=D),e(window).off("keydown",Q),p.off("focusin",et)),a&&(r&&d&&!s?a.addClass("mbsc-anim-out mbsc-anim-trans mbsc-anim-trans-"+d).on(W,function e(){a.off(W,e),J(s)}).find(".mbsc-fr-popup").addClass("mbsc-anim-"+d):J(s),n._detachEvents(a)),c&&c(),l.trigger("blur"),u("onClose",{valueText:n._value})},n.isVisible=function(){return n._isVisible},n.setVal=s,n.getVal=s,n._generateContent=s,n._attachEvents=s,n._detachEvents=s,n._readValue=s,n._clearValue=s,n._fillValue=s,n._markupReady=s,n._markupInserted=s,n._markupRemove=s,n._position=s,n.__processSettings=s,n.__init=s,n.__destroy=s,n._destroy=function(){n.hide(!0,!1,!0),l.off(".mbsc"),e.each(z,function(t,e){e.el.off(".mbsc").prop("readonly",e.readOnly),e.lbl&&e.lbl.off(".mbsc")}),n.__destroy()},n._updateHeader=function(){var e=i.headerText,t=e?"function"==typeof e?e.call($,n._tempValue):e.replace(/\{value\}/i,n._tempValue):"";X.html(t||"&nbsp;")},n._getRespCont=function(){return b="body"!=i.context,p=e(b?i.context:window),"inline"==i.display?l.is("div")?l:l.parent():p},n._processSettings=function(t){var s,o;for(n.__processSettings(t),(ee=!i.touchUi)&&(i.display=t.display||k.display||"bubble",i.buttons=t.buttons||k.buttons||[],i.showOverlay=t.showOverlay||k.showOverlay||!1),i.buttons=i.buttons||("inline"!==i.display?["cancel","set"]:[]),i.headerText=void 0===i.headerText?"inline"!==i.display&&"{value}":i.headerText,h=i.buttons||[],r="inline"!==i.display,v=e(i.context),O=b?v:e("body,html"),c=v[0],n.live=!0,s=0;s<h.length;s++)"ok"!=(o=h[s])&&"set"!=o&&"set"!=o.handler||(n.live=!1);n.buttons.set={text:i.setText,icon:i.setIcon,handler:"set"},n.buttons.cancel={text:i.cancelText,icon:i.cancelIcon,handler:"cancel"},n.buttons.close={text:i.closeText,icon:i.closeIcon,handler:"cancel"},n.buttons.clear={text:i.clearText,icon:i.clearIcon,handler:"clear"},n._isInput=l.is("input")},n._init=function(t){var e=n._isVisible,s=e&&!a.hasClass("mbsc-fr-pos");e&&n.hide(!0,!1,!0),l.off(".mbsc"),n.__init(t),n._isLiquid="liquid"==i.layout,r?(n._readValue(),n._hasContent||i.skipShow||n.attachShow(l),e&&n.show(s)):n.show(),l.removeClass("mbsc-cloak").filter("input, select, textarea").on("change.mbsc",function(){n._preventChange||n.setVal(l.val(),!0,!1),n._preventChange=!1})},n.buttons={},n.handlers={set:n.select,cancel:n.cancel,clear:n.clear},n._value=null,n._isValid=!0,n._isVisible=!1,i=n.settings,u=n.trigger,eo||n.init()}var eL,eR,eW=t.themes,nI=/(iphone|ipod)/i.test(e1)&&7<=x,tQ="android"==h,eB="ios"==h,nH=eB&&7<x,nP="input,select,textarea,button";O.prototype._defaults={lang:"en",setText:"Set",selectedText:"{count} selected",closeText:"Close",cancelText:"Cancel",clearText:"Clear",context:"body",maxPopupWidth:600,disabled:!1,closeOnOverlayTap:!0,showOnFocus:tQ||eB,showOnTap:!0,display:"center",scroll:!0,scrollLock:!0,showOverlay:!0,tap:!0,touchUi:!0,btnClass:"mbsc-fr-btn",btnWidth:!0,focusTrap:!0,focusOnClose:!eB||8!=x},u.Frame=O,eW.frame.mobiscroll={headerText:!1,btnWidth:!1},eW.scroller.mobiscroll=n({},eW.frame.mobiscroll,{rows:5,showLabel:!1,selectedLineBorder:1,weekDays:"min",checkIcon:"ion-ios7-checkmark-empty",btnPlusClass:"mbsc-ic mbsc-ic-arrow-down5",btnMinusClass:"mbsc-ic mbsc-ic-arrow-up5",btnCalPrevClass:"mbsc-ic mbsc-ic-arrow-left5",btnCalNextClass:"mbsc-ic mbsc-ic-arrow-right5"}),y&&e(window).on("focus",function(){eL&&(eR=!0)});function J(V,e1,ef){var n,s,a,r,c,l,d,u,h,p,g,b,j,y,_,w,O,x,C,A,T,z,D,N,R,P,I,B,W,K,Y,G,Q,J,et,ea,es,en,er,ee,ec,e8,e2,e3,ed,i=this,M=0,ei=1,t=e1,E=e(V);function el(a){b("onStart",{domEvent:a}),t.stopProp&&a.stopPropagation(),t.prevDef&&a.preventDefault(),t.readonly||t.lock&&C||H(a,this)&&!N&&(_&&_.removeClass("mbsc-active"),T=!1,C||(_=e(a.target).closest(".mbsc-btn-e",this)).length&&!_.hasClass("mbsc-disabled")&&(T=!0,ea=setTimeout(function(){_.addClass("mbsc-active")},100)),D=j=!(N=!0),i.scrolled=C,Q=o(a,"X"),B=o(a,"Y"),en=Q,s=z=Y=0,e3=new Date,d=+t7(O,n)||0,C&&U(d,tY?0:1),"mousedown"===a.type&&e(document).on("mousemove",q).on("mouseup",$))}function q(e){N&&(t.stopProp&&e.stopPropagation(),en=o(e,"X"),er=o(e,"Y"),Y=en-Q,z=er-B,s=n?z:Y,T&&(Math.abs(z)>t.thresholdY||Math.abs(Y)>t.thresholdX)&&(clearTimeout(ea),_.removeClass("mbsc-active"),T=!1),(i.scrolled||!D&&Math.abs(s)>ed)&&(j||b("onGestureStart",c),i.scrolled=j=!0,x||(x=!0,K=F(eu))),n||t.scrollLock?e.preventDefault():i.scrolled?e.preventDefault():7<Math.abs(z)&&(D=!0,i.scrolled=!0,$()))}function eu(){g&&(s=f(s,-a*g,a*g)),U(f(d+s,u-I,l+I)),x=!1}function $(n){if(N){var o,a=new Date-e3;t.stopProp&&n&&n.stopPropagation(),eY(K),x=!1,!D&&i.scrolled&&(t.momentum&&a<300&&(o=s/a,s=Math.max(Math.abs(s),o*o/t.speedUnit)*(s<0?-1:1)),eo(s)),T&&(clearTimeout(ea),_.addClass("mbsc-active"),setTimeout(function(){_.removeClass("mbsc-active")},100),D||i.scrolled||b("onBtnTap",{target:_[0],domEvent:n})),n&&"mouseup"==n.type&&e(document).off("mousemove",q).off("mouseup",$),N=!1}}function e4(e){if(e=e.originalEvent||e,s=n?null==e.deltaY?e.wheelDelta||e.detail:e.deltaY:e.deltaX,b("onStart",{domEvent:e}),t.stopProp&&e.stopPropagation(),s){if(e.preventDefault(),e.deltaMode&&1==e.deltaMode&&(s*=15),s=f(-s,-w,w),d=r,t.readonly)return;if(j||em(),d+s<u&&(d=u,s=0),l<d+s&&(d=l,s=0),x||(x=!0,K=F(eu)),!s&&j)return;j=!0,clearTimeout(e2),e2=setTimeout(function(){eY(K),j=x=!1,eo(s)},200)}}function e0(n){b("onStart",{domEvent:n}),t.readonly||(n.stopPropagation(),d=r,j=!1,n.target==y?(B=o(n,"Y",!0),e(document).on("mousemove",X).on("mouseup",Z)):(B=et.offset().top,X(n),Z()))}function X(t){var e=(o(t,"Y",!0)-B)/h;s=ee?f(s=-(g*a*2+h)*e,-a*g,a*g):(u-l-h)*e,j||em(),j=!0,U(f(d+s,u-I,l+I))}function Z(){d=r,eo(0),e(document).off("mousemove",X).off("mouseup",Z)}function eh(e){e.stopPropagation()}function em(){b("onGestureStart",c={posX:n?0:r,posY:n?r:0,originX:n?0:d,originY:n?d:0,direction:0<s?n?270:360:n?90:180})}function eo(o){var e,s,m;if(g&&(o=f(o,-a*g,a*g)),e=f(Math.round((d+o)/a)*a,u,l),p){if(o<0){for(s=p.length-1;0<=s;s--)if(Math.abs(e)+h>=p[s].breakpoint){ei=2,e=p[M=s].snap2;break}}else if(0<=o)for(s=0;s<p.length;s++)if(Math.abs(e)<=p[s].breakpoint){ei=1,e=p[M=s].snap1;break}e=f(e,u,l)}m=t.time||(r<u||l<r?1e3:Math.max(1e3,Math.abs(e-r)*t.timeUnit)),c.destinationX=n?0:e,c.destinationY=n?e:0,c.duration=m,c.transitionTiming=R,b("onGestureEnd",c),i.scroll(e,m)}function U(e,s,_,f){function o(){clearInterval(W),clearTimeout(J),C=!1,r=e,c.posX=n?0:e,c.posY=n?e:0,v&&b("onMove",c),j&&b("onAnimationEnd",c),f&&f()}var p,v=e!=r,j=1<s,i=s?k+"transform "+Math.round(s)+"ms "+R:"";c={posX:n?0:r,posY:n?r:0,originX:n?0:d,originY:n?d:0,direction:0<e-r?n?270:360:n?90:180},r=e,j&&(c.destinationX=n?0:e,c.destinationY=n?e:0,c.duration=s,c.transitionTiming=R,b("onAnimationStart",c)),G[m+"Transition"]=i,G[m+"Transform"]="translate3d("+(n?"0,"+e+"px,":e+"px,0,")+"0)",y&&A&&(p=ee?(ec-e)/(g*a*2):(e-l)/(u-l),y.style[m+"Transition"]=i,y.style[m+"Transform"]="translate3d(0,"+Math.max(0,Math.min((h-A)*p,h-A))+"px,0)"),!v&&!C||!s||s<=1?o():s&&(C=!_,clearInterval(W),W=setInterval(function(){var t=+t7(O,n)||0;c.posX=n?0:t,c.posY=n?t:0,b("onMove",c),Math.abs(t-e)<2&&o()},100),clearTimeout(J),J=setTimeout(function(){o()},s)),t.sync&&t.sync(e,s,R)}S.call(this,V,e1,!0),i.scrolled=!1,i.scroll=function(t,n,s,o){t=f(t=v(t)?Math.round(t/a)*a:Math.ceil((e(t,V).length?Math.round(O.offset()[es]-e(t,V).offset()[es]):r)/a)*a,u,l),M=Math.round(t/a),d=r,ec=g*a+t,U(t,n,s,o)},i.refresh=function(s){var e;for(h=(void 0===t.contSize?n?E.height():E.width():t.contSize)||0,l=(void 0===t.maxScroll?0:t.maxScroll)||0,u=Math.min(l,void 0===t.minScroll?Math.min(0,n?h-O.height():h-O.width()):t.minScroll)||0,p=null,!n&&t.rtl&&(e=l,l=-u,u=-e),L(t.snap)&&(p=[],O.find(t.snap).each(function(){var e=n?this.offsetTop:this.offsetLeft,t=n?this.offsetHeight:this.offsetWidth;p.push({breakpoint:e+t/2,snap1:-e,snap2:h-e-t})})),a=v(t.snap)?t.snap:1,g=t.snap?t.maxSnapScroll:0,R=t.easing,I=t.elastic?v(t.snap)?a:v(t.elastic)?t.elastic:0:0,w=a;44<w;)w/=2;w=Math.round(44/w)*w,y&&(ee=u==-1/0||l==1/0,A=u<l?Math.max(20,h*h/(l-u+h)):0,y.style.height=A+"px",e8.style.height=A?"":0),void 0===r&&(r=t.initialPos,M=Math.round(r/a)),s||i.scroll(t.snap?p?p[M]["snap"+ei]:M*a:r)},i._processSettings=function(){n="Y"==t.axis,es=n?"top":"left",O=t.moveElement||E.children().eq(0),G=O[0].style,ed=n?t.thresholdY:t.thresholdX,t.scrollbar&&(P=t.scrollbar,et=P.find(".mbsc-sc-bar"),y=et[0],e8=P[0])},i._init=function(){i.refresh(),E.on("touchstart mousedown",el).on("touchmove",q).on("touchend touchcancel",$),t.mousewheel&&E.on("wheel mousewheel",e4),y&&P.on("mousedown",e0).on("click",eh),V.addEventListener("click",function(e){i.scrolled&&(i.scrolled=!1,e.stopPropagation(),e.preventDefault())},!0)},i._destroy=function(){clearInterval(W),E.off("touchstart mousedown",el).off("touchmove",q).off("touchend touchcancel",$).off("wheel mousewheel",e4),y&&P.off("mousedown",e0).off("click",eh)},t=i.settings,b=i.trigger,ef||i.init()}tY="ios"==h,J.prototype={_defaults:{speedUnit:.0022,timeUnit:3,initialPos:0,axis:"Y",thresholdX:10,thresholdY:5,easing:"cubic-bezier(0.190, 1.000, 0.220, 1.000)",stopProp:!0,momentum:!0,mousewheel:!0,elastic:!0}};var c={},eQ=y?window.CSS:null,nL=eQ&&eQ.supports&&eQ.supports("(transform-style: preserve-3d)");function eT(e){return(e+"").replace('"',"___")}function d(f,D,$){var s,o,i,a,r,l,u,h,p,g,j,_,x,E,M,F,R,c=40,P=1e3,t=this,S=e(f);function U(t){var n,s,o=+e(this).attr("data-index");38==t.keyCode?(n=!0,s=-1):40==t.keyCode?(n=!0,s=1):32==t.keyCode&&(n=!0,V(o,e(t.target))),n&&(t.stopPropagation(),t.preventDefault(),s&&_.start(o,s,t))}function K(){_.stop()}function V(n,i){var e=r[n],c=+i.attr("data-index"),o=b(e,c),a=t._tempSelected[n],d=v(e.multiple)?e.multiple:1/0;!1!==l("onItemTap",{target:i[0],index:n,value:o,selected:i.hasClass("mbsc-sc-itm-sel")})&&(e.multiple&&!e._disabled[o]&&(void 0!==a[o]?(i.removeClass(u).removeAttr("aria-selected"),delete a[o]):(1==d&&(t._tempSelected[n]=a={},e._$markup.find(".mbsc-sc-itm-sel").removeClass(u).removeAttr("aria-selected")),eH(a).length<d&&(i.addClass(u).attr("aria-selected","true"),a[o]=o))),A(e,n,c,P,e._index<c?1:2,!0,e.multiple),t.live&&(!e.multiple||1===e.multiple&&s.tapSelect)&&(!0===s.setOnTap||s.setOnTap[n])&&setTimeout(function(){t.select()},s.tapSelect?0:200))}function N(e){return-(e.max-e._offset-(e.multiple&&!a?Math.floor(s.rows/2):0))*o}function B(e){return-(e.min-e._offset+(e.multiple&&!a?Math.floor(s.rows/2):0))*o}function T(e,n){return(e._array?e._map[n]:+e.getIndex(n,t))||0}function L(n,s){var o=n.data;if(s>=n.min&&s<=n.max)return n._array?n.circular?e(o).get(s%n._length):o[s]:e.isFunction(o)?o(s,t):""}function y(t){return e.isPlainObject(t)?void 0!==t.value?t.value:t.display:t}function b(e,t){return y(L(e,t))}function W(e,t,o){var n=r[e];A(n,e,n._index+t,s.delay+100,1==t?1:2,!1,!1,"keydown"==o.type)}function I(t){return e.isArray(s.readonly)?s.readonly[t]:s.readonly}function z(t,a,l){var r=t._index-t._batch;return t.data=t.data||[],t.key=void 0!==t.key?t.key:a,t.label=void 0!==t.label?t.label:a,t._map={},t._array=e.isArray(t.data),t._array&&(t._length=t.data.length,e.each(t.data,function(e,n){t._map[y(n)]=e})),t.circular=void 0===s.circular?void 0===t.circular?t._array&&t._length>s.rows:t.circular:e.isArray(s.circular)?s.circular[a]:s.circular,t.min=t._array?t.circular?-1/0:0:void 0===t.min?-1/0:t.min,t.max=t._array?t.circular?1/0:t._length-1:void 0===t.max?1/0:t.max,t._nr=a,t._index=T(t,i[a]),t._disabled={},t._batch=0,t._current=t._index,t._first=t._index-c,t._last=t._index+c,t._offset=t._first,l?(t._offset-=t._margin/o+(t._index-r),t._margin+=(t._index-r)*o):t._margin=0,t._refresh=function(e){n(t._scroller.settings,{minScroll:N(t),maxScroll:B(t)}),t._scroller.refresh(e)},E[t.key]=t}function d(l,_,x,O,p){var n,a,r,c,d,h,m,f,v,w,g="",b=t._tempSelected[_],C=l._disabled||{};for(r=x;r<=O;r++)n=L(l,r),c=n,m=void 0===(w=e.isPlainObject(c)?c.display:c)?"":w,a=y(n),f=n&&void 0!==n.cssClass?n.cssClass:"",h=n&&void 0!==n.label?n.label:"",v=n&&n.invalid,d=void 0!==a&&a==i[_]&&!l.multiple,g+='<div role="option" tabindex="-1" aria-selected="'+!!b[a]+'" class="mbsc-sc-itm '+(p?"mbsc-sc-itm-3d ":"")+f+" "+(d?"mbsc-sc-itm-sel ":"")+(b[a]?u:"")+(void 0===a?" mbsc-sc-itm-ph":" mbsc-btn-e")+(v?" mbsc-sc-itm-inv-h mbsc-disabled":"")+(C[a]?" mbsc-sc-itm-inv mbsc-disabled":"")+'" data-index="'+r+'" data-val="'+eT(a)+'"'+(h?' aria-label="'+h+'"':"")+(d?' aria-selected="true"':"")+' style="height:'+o+"px;line-height:"+o+"px;"+(p?k+"transform:rotateX("+(l._offset-r)*M%360+"deg) translateZ("+o*s.rows/2+"px);":"")+'">'+(1<j?'<div class="mbsc-sc-itm-ml" style="line-height:'+Math.round(o/j)+"px;font-size:"+Math.round(o/j*.8)+'px;">':"")+m+(1<j?"</div>":"")+"</div>";return g}function H(m,h,d,u){var n,e=r[m],a=u||e._disabled,t=T(e,h),o=b(e,t),l=o,i=o,s=0,c=0;if(!0===a[o]){for(n=0;t-s>=e.min&&a[l]&&n<100;)n++,l=b(e,t-++s);for(n=0;t+c<e.max&&a[i]&&n<100;)n++,i=b(e,t+ ++c);o=(c<s&&c&&2!==d||!s||t-s<0||1==d)&&!a[i]?i:l}return o}function w(j,n,y,h,O,_,w){var d,m,v,b,p=t._isVisible;g=!0,d=s.validate.call(f,{values:i.slice(0),index:n,direction:y},t)||{},g=!1,d.valid&&(t._tempWheelArray=i=d.valid.slice(0)),_||e.each(r,function(r,s){if(p&&s._$markup.find(".mbsc-sc-itm-inv").removeClass("mbsc-sc-itm-inv mbsc-disabled"),s._disabled={},d.disabled&&d.disabled[r]&&e.each(d.disabled[r],function(t,e){s._disabled[e]=!0,p&&s._$markup.find('.mbsc-sc-itm[data-val="'+eT(e)+'"]').addClass("mbsc-sc-itm-inv mbsc-disabled")}),i[r]=s.multiple?i[r]:H(r,i[r],y),p){if(s.multiple&&void 0!==n||s._$markup.find(".mbsc-sc-itm-sel").removeClass(u).removeAttr("aria-selected"),m=T(s,i[r]),v=m-s._index+s._batch,Math.abs(v)>2*c+1&&(b=v+(2*c+1)*(0<v?-1:1),s._offset+=b,s._margin-=b*o,s._refresh()),s._index=m+s._batch,s.multiple){if(void 0===n)for(var l in t._tempSelected[r])s._$markup.find('.mbsc-sc-itm[data-val="'+eT(l)+'"]').addClass(u).attr("aria-selected","true")}else s._$markup.find('.mbsc-sc-itm[data-val="'+eT(i[r])+'"]').addClass("mbsc-sc-itm-sel").attr("aria-selected","true");s._$active&&s._$active.attr("tabindex",-1),s._$active=s._$markup.find('.mbsc-sc-itm[data-index="'+s._index+'"]').eq(a&&s.multiple?1:0).attr("tabindex",0),w&&n===r&&s._$active.length&&(s._$active[0].focus(),s._$scroller.parent().scrollTop(0)),s._scroller.scroll(-(m-s._offset+s._batch)*o,n===r||void 0===n?j:P,O)}}),l("onValidated",{index:n,time:j}),t._tempValue=s.formatValue.call(f,i,t),p&&t._updateHeader(),t.live&&function(t,n){var e=r[t];return e&&(!e.multiple||1!==e.multiple&&n&&(!0===s.setOnTap||s.setOnTap[t]))}(n,_)&&(t._hasValue=h||t._hasValue,C(h,h,0,!0),h&&l("onSet",{valueText:t._value})),h&&l("onChange",{index:n,valueText:t._tempValue})}function A(e,n,t,o,a,r,c,l){var s=b(e,t);void 0!==s&&(i[n]=s,e._batch=e._array?Math.floor(t/e._length)*e._length:0,e._index=t,setTimeout(function(){w(o,n,a,!0,r,c,l)},10))}function C(a,o,c,d,u){if(d?t._tempValue=s.formatValue.call(f,t._tempWheelArray,t):w(c),!u){t._wheelArray=[];for(var e=0;e<i.length;e++)t._wheelArray[e]=r[e]&&r[e].multiple?Object.keys(t._tempSelected[e]||{})[0]:i[e];t._value=t._hasValue?t._tempValue:null,t._selected=n(!0,{},t._tempSelected)}a&&(t._isInput&&S.val(t._hasValue?t._tempValue:""),l("onFill",{valueText:t._hasValue?t._tempValue:"",change:o}),o&&(t._preventChange=!0,S.trigger("change")))}O.call(this,f,D,!0),t.setVal=t._setVal=function(n,o,a,r,c){t._hasValue=null!=n,t._tempWheelArray=i=e.isArray(n)?n.slice(0):s.parseValue.call(f,n,t)||[],C(o,void 0===a?o:a,c,!1,r)},t.getVal=t._getVal=function(n){var e=t._hasValue||n?t[n?"_tempValue":"_value"]:null;return v(e)?+e:e},t.setArrayVal=t.setVal,t.getArrayVal=function(e){return e?t._tempWheelArray:t._wheelArray},t.changeWheel=function(i,r,l){var s,o;e.each(i,function(e,i){(s=E[e])&&(o=s._nr,n(s,i),z(s,o,!0),t._isVisible&&(a&&s._$3d.html(d(s,o,s._first+c-h+1,s._last-c+h,!0)),s._$scroller.html(d(s,o,s._first,s._last)).css("margin-top",s._margin+"px"),s._refresh(g)))}),!t._isVisible||t._isLiquid||g||t.position(),g||w(r,void 0,void 0,l)},t.getValidValue=H,t._generateContent=function(){var u,m=0,l="",f=a?k+"transform: translateZ("+(o*s.rows/2+3)+"px);":"",g='<div class="mbsc-sc-whl-l" style="'+f+"height:"+o+"px;margin-top:-"+(o/2+(s.selectedLineBorder||0))+'px;"></div>',i=0;return e.each(s.wheels,function(b,v){l+='<div class="mbsc-w-p mbsc-sc-whl-gr-c'+(a?" mbsc-sc-whl-gr-3d-c":"")+(s.showLabel?" mbsc-sc-lbl-v":"")+'">'+g+'<div class="mbsc-sc-whl-gr'+(a?" mbsc-sc-whl-gr-3d":"")+(p?" mbsc-sc-cp":"")+(s.width||s.maxWidth?'"':'" style="max-width:'+s.maxPopupWidth+'px;"')+">",e.each(v,function(v,e){t._tempSelected[i]=n({},t._selected[i]),r[i]=z(e,i),m+=s.maxWidth?s.maxWidth[i]||s.maxWidth:s.width?s.width[i]||s.width:0,u=void 0!==e.label?e.label:v,l+='<div class="mbsc-sc-whl-w '+(e.cssClass||"")+(e.multiple?" mbsc-sc-whl-multi":"")+'" style="'+(s.width?"width:"+(s.width[i]||s.width)+"px;":(s.minWidth?"min-width:"+(s.minWidth[i]||s.minWidth)+"px;":"")+(s.maxWidth?"max-width:"+(s.maxWidth[i]||s.maxWidth)+"px;":""))+'">'+(F?'<div class="mbsc-sc-bar-c"><div class="mbsc-sc-bar"></div></div>':"")+'<div class="mbsc-sc-whl-o" style="'+f+'"></div>'+g+'<div aria-live="off" aria-label="'+u+'"'+(e.multiple?' aria-multiselectable="true"':"")+' role="listbox" data-index="'+i+'" class="mbsc-sc-whl" style="height:'+s.rows*o*(a?1.1:1)+'px;">'+(p?'<div data-index="'+i+'" data-step="1" class="mbsc-sc-btn mbsc-sc-btn-plus '+(s.btnPlusClass||"")+'"></div><div data-index="'+i+'" data-step="-1" class="mbsc-sc-btn mbsc-sc-btn-minus '+(s.btnMinusClass||"")+'"></div>':"")+'<div class="mbsc-sc-lbl">'+u+'</div><div class="mbsc-sc-whl-c" style="height:'+x+"px;margin-top:-"+(x/2+1)+"px;"+f+'"><div class="mbsc-sc-whl-sc" style="top:'+(x-o)/2+'px;">',l+=d(e,i,e._first,e._last)+"</div></div>",a&&(l+='<div class="mbsc-sc-whl-3d" style="height:'+o+"px;margin-top:-"+o/2+'px;">',l+=d(e,i,e._first+c-h+1,e._last-c+h,!0),l+="</div>"),l+="</div></div>",i++}),l+="</div></div>"}),m&&(s.maxPopupWidth=m),l},t._attachEvents=function(t){_=eS(e(".mbsc-sc-btn",t),W,s.delay,I,!0),e(".mbsc-sc-whl",t).on("keydown",U).on("keyup",K)},t._detachEvents=function(){_.stop();for(var e=0;e<r.length;e++)r[e]._scroller.destroy()},t._markupReady=function(t){e(".mbsc-sc-whl-w",R=t).each(function(n){var u,i=e(this),t=r[n];t._$markup=i,t._$scroller=e(".mbsc-sc-whl-sc",this),t._$3d=e(".mbsc-sc-whl-3d",this),t._scroller=new J(this,{mousewheel:s.mousewheel,moveElement:t._$scroller,scrollbar:e(".mbsc-sc-bar-c",this),initialPos:(t._first-t._index)*o,contSize:s.rows*o,snap:o,minScroll:N(t),maxScroll:B(t),maxSnapScroll:c,prevDef:!0,stopProp:!0,timeUnit:3,easing:"cubic-bezier(0.190, 1.000, 0.220, 1.000)",sync:function(n,e,s){var i=e?k+"transform "+Math.round(e)+"ms "+s:"";a&&(t._$3d[0].style[m+"Transition"]=i,t._$3d[0].style[m+"Transform"]="rotateX("+-n/o*M+"deg)")},onStart:function(t,e){e.settings.readonly=I(n)},onGestureStart:function(){i.addClass("mbsc-sc-whl-a mbsc-sc-whl-anim"),l("onWheelGestureStart",{index:n})},onGestureEnd:function(e){var s=90==e.direction?1:2,i=e.duration,a=e.destinationY;u=Math.round(-a/o)+t._offset,A(t,n,u,i,s)},onAnimationStart:function(){i.addClass("mbsc-sc-whl-anim")},onAnimationEnd:function(){i.removeClass("mbsc-sc-whl-a mbsc-sc-whl-anim"),l("onWheelAnimationEnd",{index:n}),t._$3d.find(".mbsc-sc-itm-del").remove()},onMove:function(s){!function(t,u,f){var m=Math.round(-f/o)+t._offset,n=m-t._current,s=t._first,i=t._last,r=s+c-h+1,l=i-c+h;n&&(t._first+=n,t._last+=n,t._current=m,0<n?(t._$scroller.append(d(t,u,Math.max(i+1,s+n),i+n)),e(".mbsc-sc-itm",t._$scroller).slice(0,Math.min(n,i-s+1)).remove(),a&&(t._$3d.append(d(t,u,Math.max(l+1,r+n),l+n,!0)),e(".mbsc-sc-itm",t._$3d).slice(0,Math.min(n,l-r+1)).attr("class","mbsc-sc-itm-del"))):n<0&&(t._$scroller.prepend(d(t,u,s+n,Math.min(s-1,i+n))),e(".mbsc-sc-itm",t._$scroller).slice(Math.max(n,s-i-1)).remove(),a&&(t._$3d.prepend(d(t,u,r+n,Math.min(r-1,l+n),!0)),e(".mbsc-sc-itm",t._$3d).slice(Math.max(n,r-l-1)).attr("class","mbsc-sc-itm-del"))),t._margin+=n*o,t._$scroller.css("margin-top",t._margin+"px"))}(t,n,s.posY)},onBtnTap:function(t){V(n,e(t.target))}})}),w()},t._fillValue=function(){C(t._hasValue=!0,!0,0,!0)},t._clearValue=function(){e(".mbsc-sc-whl-multi .mbsc-sc-itm-sel",R).removeClass(u).removeAttr("aria-selected")},t._readValue=function(){var a=S.val()||"",o=0;""!==a&&(t._hasValue=!0),t._tempWheelArray=i=t._hasValue&&t._wheelArray?t._wheelArray.slice(0):s.parseValue.call(f,a,t)||[],t._tempSelected=n(!0,{},t._selected),e.each(s.wheels,function(n,t){e.each(t,function(t,e){r[o]=z(e,o),o++})}),C(!1,!1,0,!0),l("onRead")},t.__processSettings=function(e){s=t.settings,l=t.trigger,j=s.multiline,u="mbsc-sc-itm-sel mbsc-ic mbsc-ic-"+s.checkIcon,(F=!s.touchUi)&&(s.tapSelect=!0,s.circular=!1,s.rows=e.rows||D.rows||7)},t.__init=function(e){e&&(t._wheelArray=null),r=[],E={},p=s.showScrollArrows,a=s.scroll3d&&nL&&!p&&!F&&("ios"==s.theme||"ios"==s.baseTheme),o=s.height,x=a?2*Math.round((o-.03*(o*s.rows/2+3))/2):o,h=Math.round(1.8*s.rows),M=360/(2*h),p&&(s.rows=Math.max(3,s.rows))},t._getItemValue=y,t._tempSelected={},t._selected={},$||t.init()}d.prototype={_hasDef:!0,_hasTheme:!0,_hasLang:!0,_responsive:!0,_class:"scroller",_presets:c,_defaults:n({},O.prototype._defaults,{minWidth:80,height:40,rows:3,multiline:1,delay:200,readonly:!1,showLabel:!0,setOnTap:!1,wheels:[],preset:"",speedUnit:.0012,timeUnit:.08,checkIcon:"checkmark",compClass:"mbsc-sc",validate:function(){},formatValue:function(e){return e.join(" ")},parseValue:function(o,t){var n,s,i=[],a=[],r=0;return null!=o&&(i=(o+"").split(" ")),e.each(t.settings.wheels,function(c,o){e.each(o,function(c,o){s=o.data,n=t._getItemValue(s[0]),e.each(s,function(s,e){if(i[r]==t._getItemValue(e))return n=t._getItemValue(e),!1}),a.push(n),r++})}),a}})},u.Scroller=d;function eF(o){function J(s){var e,n,o,r,i=[];if(s){for(o=0;o<s.length;o++)if((n=s[o]).start&&n.end&&!nm.test(n.start))for(e=new Date(a(n.start,m,t)),r=new Date(a(n.end,m,t));e<=r;)i.push(T(e.getFullYear(),e.getMonth(),e.getDate())),e.setDate(e.getDate()+1);else i.push(n);return i}return s}function u(t,e,n,s){return Math.min(s,Math.floor(t/e)*e+n)}function P(e,t,n){return Math.floor((n-t)/e)*e+t}function D(e){return e.getFullYear()+"-"+j(e.getMonth()+1)+"-"+j(e.getDate())}function p(o,e,n,i){var t;return void 0===s[e]||(t=+o[s[e]],isNaN(t))?n?c[e](n):void 0!==N[e]?N[e]:c[e](i):t}function v(n){if(o=new Date((new Date).setHours(0,0,0,0)),null===n)return n;void 0!==s.dd&&(e=n[s.dd].split("-"),e=new Date(e[0],e[1]-1,e[2])),void 0!==s.tt&&(e=e||o,e=new Date(e.getTime()+n[s.tt]%86400*1e3));var e,o,i=p(n,"y",e,o),a=p(n,"m",e,o),c=Math.min(p(n,"d",e,o),t.getMaxDayOfMonth(i,a)),r=p(n,"h",e,o);return t.getDate(i,a,c,_&&p(n,"a",e,o)?r+12:r,p(n,"i",e,o),p(n,"s",e,o),p(n,"u",e,o))}function S(t,a){var e,n,o=["y","m","d","a","h","i","s","u","dd","tt"],i=[];if(null==t)return t;for(n=0;n<o.length;n++)void 0!==s[e=o[n]]&&(i[s[e]]=c[e](t)),a&&(N[e]=c[e](t));return i}function H(e,t){return t?Math.floor(new Date(e)/864e5):e.getMonth()+12*(e.getFullYear()-1970)}function el(e){return{value:e,display:(/yy/i.test(x)?e:(e+"").substr(2,2))+(t.yearSuffix||"")}}function ed(e){return e}function eu(n){var e=/d/i.test(n);return{label:"",cssClass:"mbsc-dt-whl-date",min:i?H(D(i),e):void 0,max:d?H(D(d),e):void 0,data:function(o){var i=new Date((new Date).setHours(0,0,0,0)),s=e?new Date(864e5*o):new Date(1970,o,1);return e&&(s=new Date(s.getUTCFullYear(),s.getUTCMonth(),s.getUTCDate())),{invalid:e&&!O(s,!0),value:D(s),display:i.getTime()==s.getTime()?t.todayText:l(n,s,t)}},getIndex:function(t){return H(t,e)}}}function e1(s){var e,n,o,i=[];for(/s/i.test(s)?n=E:/i/i.test(s)?n=60*b:/h/i.test(s)&&(n=3600*w),z=h.tt=n,e=0;e<86400;e+=n)o=new Date((new Date).setHours(0,0,0,0)+1e3*e),i.push({value:e,display:l(s,o,t)});return{label:"",cssClass:"mbsc-dt-whl-time",data:i}}function en(e,n){return t.getYear(e)===t.getYear(n)&&t.getMonth(e)===t.getMonth(n)}function O(e,t){return!(!t&&e<i)&&!(!t&&d<e)&&(!!et(e,W)||!et(e,$))}function et(n,i){var e,s,o;if(i)for(o=0;o<i.length;o++)if(e=(s=i[o])+"",!s.start)if(C.test(e)){if((e=+e.replace("w",""))==n.getDay())return!0}else if(B.test(e)){if((e=e.split("/"))[1]){if(e[0]-1==n.getMonth()&&e[1]==n.getDate())return!0}else if(e[0]==n.getDate())return!0}else if(s=a(s,m,t),n.getFullYear()==s.getFullYear()&&n.getMonth()==s.getMonth()&&n.getDate()==s.getDate())return!0;return!1}function Q(c,d,l,u,h,r,o){var e,n,s,i;if(c)for(i=0;i<c.length;i++)if(e=(n=c[i])+"",!n.start)if(C.test(e))for(s=(e=+e.replace("w",""))-u;s<h;s+=7)0<=s&&(r[s+1]=o);else B.test(e)?(e=e.split("/"))[1]?e[0]-1==l&&(r[e[1]]=o):r[e[0]]=o:(n=a(n,m,t),t.getYear(n)==d&&t.getMonth(n)==l&&(r[t.getDay(n)]=o))}function G(j,x,o,R,D,E,H,P){var e,n,s,i,a,l,d,m,p,v,b,w,O,k,A,S,M,F,T,N,L={},c=t.getDate(R,D,E),r=["a","h","i","s"];if(j){for(a=0;a<j.length;a++)(i=j[a]).start&&(i.apply=!1,d=(A=(p=i.d)+"").split("/"),p&&(p.getTime&&R==t.getYear(p)&&D==t.getMonth(p)&&E==t.getDay(p)||!C.test(A)&&(d[1]&&E==d[1]&&D==d[0]-1||!d[1]&&E==d[0])||C.test(A)&&c.getDay()==+A.replace("w",""))&&(i.apply=!0,L[c]=!0));for(a=0;a<j.length;a++)if(i=j[a],N=F=0,S=y[o],O=f[o],k=!(l=b=!0),i.start&&(i.apply||!i.d&&!L[c])){for(n=i.start.split(":"),s=i.end.split(":"),e=0;e<3;e++)void 0===n[e]&&(n[e]=0),void 0===s[e]&&(s[e]=59),n[e]=+n[e],s[e]=+s[e];if("tt"==o)S=u(Math.round((new Date(c).setHours(n[0],n[1],n[2])-new Date(c).setHours(0,0,0,0))/1e3),z,0,86400),O=u(Math.round((new Date(c).setHours(s[0],s[1],s[2])-new Date(c).setHours(0,0,0,0))/1e3),z,0,86400);else{for(n.unshift(11<n[0]?1:0),s.unshift(11<s[0]?1:0),_&&(12<=n[1]&&(n[1]=n[1]-12),12<=s[1]&&(s[1]=s[1]-12)),e=0;e<x;e++)void 0!==g[e]&&(m=u(n[e],h[r[e]],y[r[e]],f[r[e]]),v=u(s[e],h[r[e]],y[r[e]],f[r[e]]),w=M=T=0,_&&1==e&&(T=n[0]?12:0,M=s[0]?12:0,w=g[0]?12:0),b||(m=0),l||(v=f[r[e]]),(b||l)&&m+T<g[e]+w&&g[e]+w<v+M&&(k=!0),g[e]!=m&&(b=!1),g[e]!=v&&(l=!1));if(!P)for(e=x+1;e<4;e++)0<n[e]&&(F=h[o]),s[e]<f[r[e]]&&(N=h[o]);k||(m=u(n[x],h[o],y[o],f[o])+F,v=u(s[x],h[o],y[o],f[o])-N,b&&(S=m),l&&(O=v))}if(b||l||k)for(e=S;e<=O;e+=h[o])H[e]=!P}}}var z,Y,ee,s={},N={},r={},g=[],eo=function(n){var e,s,o,t={};if(n.is("input")){switch(n.attr("type")){case"date":e="yy-mm-dd";break;case"datetime":e="yy-mm-ddTHH:ii:ssZ";break;case"datetime-local":e="yy-mm-ddTHH:ii:ss";break;case"month":e="yy-mm",t.dateOrder="mmyy";break;case"time":e="HH:ii:ss"}t.format=e,s=n.attr("min"),o=n.attr("max"),s&&"undefined"!=s&&(t.min=Z(e,s)),o&&"undefined"!=o&&(t.max=Z(e,o))}return t}(e(this)),es=n({},o.settings),e0=eG[es.calendarSystem],t=n(o.settings,ev,e0,nN,eo,es),k=t.preset,R="datetime"==k?t.dateFormat+t.separator+t.timeFormat:"time"==k?t.timeFormat:t.dateFormat,m=eo.format||R,ei=t.dateWheels||t.dateFormat,I=t.timeWheels||t.timeFormat,x=t.dateWheels||t.dateDisplay,M=I,ec=t.baseTheme||t.theme,$=J(t.invalid),W=J(t.valid),i=a(t.min,m,t),d=a(t.max,m,t),X=/time/i.test(k),_=/h/.test(M),e4=/D/.test(x),L=t.steps||{},w=L.hour||t.stepHour||1,b=L.minute||t.stepMinute||1,E=L.second||t.stepSecond||1,V=L.zeroBased,F=V||!i?0:i.getHours()%w,A=V||!i?0:i.getMinutes()%b,U=V||!i?0:i.getSeconds()%E,q=P(w,F,_?11:23),er=P(b,A,59),ea=P(b,A,59),y={y:i?i.getFullYear():-1/0,m:0,d:1,h:F,i:A,s:U,a:0,tt:0},f={y:d?d.getFullYear():1/0,m:11,d:31,h:q,i:er,s:ea,a:1,tt:86400},h={y:1,m:1,d:1,h:w,i:b,s:E,a:1,tt:1},c={y:function(e){return t.getYear(e)},m:function(e){return t.getMonth(e)},d:function(e){return t.getDay(e)},h:function(t){var e=t.getHours();return u(e=_&&12<=e?e-12:e,w,F,q)},i:function(e){return u(e.getMinutes(),b,A,er)},s:function(e){return u(e.getSeconds(),E,U,ea)},u:function(e){return e.getMilliseconds()},a:function(e){return 11<e.getHours()?1:0},dd:D,tt:function(e){return u(Math.round((e.getTime()-new Date(e).setHours(0,0,0,0))/1e3),z||1,0,86400)}};return o.getVal=function(e){return o._hasValue||e?K(v(o.getArrayVal(e)),t,m):null},o.getDate=function(e){return o._hasValue||e?v(o.getArrayVal(e)):null},o.setDate=function(e,t,n,s,i){o.setArrayVal(S(e,!0),t,i,s,n)},ee=function(){var e,n,a,c,l,m,f,p,o=0,g=[],h=[],u=[];if(/date/i.test(k)){for(f=ei.split(/\|/.test(ei)?"|":""),l=0;l<f.length;l++)if(c=0,(n=f[l]).length)if(/y/i.test(n)&&(r.y=1,c++),/m/i.test(n)&&(r.y=1,r.m=1,c++),/d/i.test(n)&&(r.y=1,r.m=1,r.d=1,c++),1<c&&void 0===s.dd)s.dd=o,o++,h.push(eu(n)),u=h,Y=!0;else if(/y/i.test(n)&&void 0===s.y)s.y=o,o++,h.push({cssClass:"mbsc-dt-whl-y",label:t.yearText,min:i?t.getYear(i):void 0,max:d?t.getYear(d):void 0,data:el,getIndex:ed});else if(/m/i.test(n)&&void 0===s.m){for(s.m=o,a=[],o++,e=0;e<12;e++)m=x.replace(/[dy|]/gi,"").replace(/mm/,j(e+1)+(t.monthSuffix||"")).replace(/m/,e+1+(t.monthSuffix||"")),a.push({value:e,display:/MM/.test(m)?m.replace(/MM/,'<span class="mbsc-dt-month">'+t.monthNames[e]+"</span>"):m.replace(/M/,'<span class="mbsc-dt-month">'+t.monthNamesShort[e]+"</span>")});h.push({cssClass:"mbsc-dt-whl-m",label:t.monthText,data:a})}else if(/d/i.test(n)&&void 0===s.d){for(s.d=o,a=[],o++,e=1;e<32;e++)a.push({value:e,display:(/dd/i.test(x)?j(e):e)+(t.daySuffix||"")});h.push({cssClass:"mbsc-dt-whl-d",label:t.dayText,data:a})}g.push(h)}if(/time/i.test(k)){for(p=I.split(/\|/.test(I)?"|":""),l=0;l<p.length;l++)if(c=0,(n=p[l]).length&&(/h/i.test(n)&&(r.h=1,c++),/i/i.test(n)&&(r.i=1,c++),/s/i.test(n)&&(r.s=1,c++),/a/i.test(n)&&c++),1<c&&void 0===s.tt)s.tt=o,o++,u.push(e1(n));else if(/h/i.test(n)&&void 0===s.h){for(a=[],s.h=o,r.h=1,o++,e=F;e<(_?12:24);e+=w)a.push({value:e,display:_&&0===e?12:/hh/i.test(M)?j(e):e});u.push({cssClass:"mbsc-dt-whl-h",label:t.hourText,data:a})}else if(/i/i.test(n)&&void 0===s.i){for(a=[],s.i=o,r.i=1,o++,e=A;e<60;e+=b)a.push({value:e,display:/ii/i.test(M)?j(e):e});u.push({cssClass:"mbsc-dt-whl-i",label:t.minuteText,data:a})}else if(/s/i.test(n)&&void 0===s.s){for(a=[],s.s=o,r.s=1,o++,e=U;e<60;e+=E)a.push({value:e,display:/ss/i.test(M)?j(e):e});u.push({cssClass:"mbsc-dt-whl-s",label:t.secText,data:a})}else/a/i.test(n)&&void 0===s.a&&(s.a=o,o++,u.push({cssClass:"mbsc-dt-whl-a",label:t.ampmText,data:/A/.test(n)?[{value:0,display:t.amText.toUpperCase()},{value:1,display:t.pmText.toUpperCase()}]:[{value:0,display:t.amText},{value:1,display:t.pmText}]}));u!=h&&g.push(u)}return g}(),t.isoParts=r,o._format=R,o._order=s,o.handlers.now=function(){o.setDate(new Date,o.live,1e3,!0,!0)},o.buttons.now={text:t.nowText,icon:t.nowIcon,handler:"now"},{minWidth:Y&&X?{bootstrap:46,ios:50,material:46,mobiscroll:46,windows:50}[ec]:void 0,compClass:"mbsc-dt mbsc-sc",wheels:ee,headerText:!!t.headerText&&function(){return l(R,v(o.getArrayVal(!0)),t)},formatValue:function(e){return l(m,v(e),t)},parseValue:function(e){return e||(N={},o._hasValue=!1),S(a(e||t.defaultValue||new Date,m,t,r),!!e)},validate:function(E){var n,l,j,w,T=E.values,_=E.index,F=E.direction,b=t.wheels[0][s.d],u=function(e,o){var t,n,a=!1,s=!1,r=0,c=0,l=i?v(S(i)):-1/0,u=d?v(S(d)):1/0;if(O(e))return e;if(e<l&&(e=l),u<e&&(e=u),n=t=e,2!==o)for(a=O(t,!0);!a&&t<u&&r<100;)a=O(t=new Date(t.getTime()+864e5),!0),r++;if(1!==o)for(s=O(n,!0);!s&&l<n&&c<100;)s=O(n=new Date(n.getTime()-864e5),!0),c++;return 1===o&&a?t:2===o&&s?n:en(e,t)?t:en(e,n)?n:c<=r&&s?n:t}(v(T),F),M=S(u),m=[],A={},a=c.y(u),r=c.m(u),p=t.getMaxDayOfMonth(a,r),C=!0,k=!0;if(e.each(["dd","y","m","d","tt","a","h","i","s"],function(_,n){var b,j,g=y[n],v=f[n],o=c[n](u);if(m[s[n]]=[],C&&i&&(g=c[n](i)),k&&d&&(v=c[n](d)),o<g&&(o=g),v<o&&(o=v),"dd"!==n&&"tt"!==n&&(C=C&&o==g,k=k&&o==v),void 0!==s[n]){if("y"!=n&&"dd"!=n)for(l=y[n];l<=f[n];l+=h[n])(l<g||v<l)&&m[s[n]].push(l);"d"==n&&(j=t.getDate(a,r,1).getDay(),b={},Q($,a,r,j,p,b,1),Q(W,a,r,j,p,b,0),e.each(b,function(e,t){t&&m[s[n]].push(e)}))}}),X&&e.each(["a","h","i","s","tt"],function(i,t){var d=c[t](u),l=c.d(u),n={};void 0!==s[t]&&(G($,i,t,a,r,l,n,0),G(W,i,t,a,r,l,n,1),e.each(n,function(e,n){n&&m[s[t]].push(e)}),g[i]=o.getValidValue(s[t],d,F,n))}),b&&(b._length!==p||e4&&(void 0===_||_===s.y||_===s.m))){for((A[s.d]=b).data=[],n=1;n<=p;n++)w=t.getDate(a,r,n).getDay(),j=x.replace(/[my|]/gi,"").replace(/dd/,(n<10?"0"+n:n)+(t.daySuffix||"")).replace(/d/,n+(t.daySuffix||"")),b.data.push({value:n,display:/DD/.test(j)?j.replace(/DD/,'<span class="mbsc-dt-day">'+t.dayNames[w]+"</span>"):j.replace(/D/,'<span class="mbsc-dt-day">'+t.dayNamesShort[w]+"</span>")});o._tempWheelArray[s.d]=M[s.d],o.changeWheel(A)}return{disabled:m,valid:M}}}}function eZ(o){var t,i,r,c,l,d,u,h,m,f,p,g,v,j,y,_,w,x,E,k,A,S,F,D,N,L,R,H,I,$,W,U,K,Y,G,X,Q,e8,e_,ek,e4,es,eK,eC,el,ec,ev,eV,et,ey,ep,e6,eT,eu,eL,en,eI,e$,ei,ea,er,e5,ed,e9,em,eW,eE,eg,e0,eo,ej,eB,eJ=1,eZ=this;function z(e){e.hasClass("mbsc-cal-h")||e.addClass("mbsc-cal-h")}function eH(e){e.hasClass("mbsc-cal-h")?function(e){e.hasClass("mbsc-cal-h")&&(e.removeClass("mbsc-cal-h"),o._onSelectShow())}(e):z(e)}function ee(e,t,n){e[t]=e[t]||[],e[t].push(n)}function q(d,i,r){var n,s,c,l,u=t.getYear(i),h=t.getMonth(i),o={};return d&&e.each(d,function(d,e){if(c=e.d||e.start||e,s=c+"",e.start&&e.end)for(n=M(a(e.start,F,t)),l=M(a(e.end,F,t));n<=l;)ee(o,n,e),n=t.getDate(t.getYear(n),t.getMonth(n),t.getDay(n)+1);else if(C.test(s))for(n=Z(i,!1,+s.replace("w",""));n<=r;)ee(o,n,e),n=t.getDate(t.getYear(n),t.getMonth(n),t.getDay(n)+7);else if(B.test(s))if((s=s.split("/"))[1])for(n=t.getDate(u,s[0]-1,s[1]);n<=r;)ee(o,n,e),n=t.getDate(t.getYear(n)+1,t.getMonth(n),s[1]);else for(n=t.getDate(u,h,s[0]);n<=r;)ee(o,n,e),n=t.getDate(t.getYear(n),t.getMonth(n)+1,s[0]);else ee(o,M(a(c,F,t)),e)}),o}function eQ(t){var e,a,r,c,s=!!ep[t]&&ep[t],i=!!e6[t]&&e6[t],d=i&&i[0].background?i[0].background:s&&s[0].background,l="";if(i)for(e=0;e<i.length;e++)l+=(i[e].cssClass||"")+" ";if(s){for(a='<div class="mbsc-cal-marks">',e=0;e<s.length;e++)l+=((r=s[e]).cssClass||"")+" ",a+='<div class="mbsc-cal-mark"'+(r.color?' style="background:'+r.color+';"':"")+"></div>";a+="</div>"}return c={marked:s,background:d,cssClass:l,markup:I[t]?I[t].join(""):ev?a:""},n(c,o._getDayProps(t,c))}function e7(e){return' style="'+(f?"transform: translateY("+100*e+"%)":"left:"+100*e*w+"%")+'"'}function eM(e){return O(e,l-1)>_&&(e=O(_,1-l)),e<v&&(e=v),e}function e1(t,i,s){var n=t.color,o=t.text;return'<div data-id="'+t._id+'" data-index="'+i+'" class="mbsc-cal-txt" title="'+e("<div>"+o+"</div>").text()+'"'+(n?' style="background:'+n+(s?";color:"+np(n):"")+';"':"")+">"+(s?o:"")+"</div>"}function ez(s){var n=Z(O(s,-g-E),!1),e=Z(O(s,-g+l+E-1),!1),e=t.getDate(t.getYear(e),t.getMonth(e),t.getDay(e)+7*r);o._onGenMonth(n,e),eV=q(t.invalid,n,e),eW=q(t.valid,n,e),ep=q(t.labels||t.events||t.marked,n,e),e6=q(t.colors,n,e),ey=o._labels||ep||e6,(W=t.labels||o._labels)&&function(){I={};for(var o={},s=n,i=function(){s.getDay()==j&&(o={});for(var u,_,w=eu,g=ey[s]||[],c=g.length,l=[],i=void 0,f=void 0,e=0,d=0,r=0,y=void 0;e<w;)if(i=null,g.forEach(function(t,n){o[e]==t&&(i=t,f=n)}),e==w-1&&(d<c-1||c&&r==c&&!i))u=c-d,_=(1<u&&t.moreEventsPluralText||t.moreEventsText).replace(/{count}/,u),u&&l.push('<div class="mbsc-cal-txt-more">'+_+"</div>"),i&&(o[e]=null,i._days.forEach(function(n){I[n][e]='<div class="mbsc-cal-txt-more">'+t.moreEventsText.replace(/{count}/,1)+"</div>"})),d++,e++;else if(i)f==r&&r++,b(s,a(i.end))&&(o[e]=null),l.push(e1(i,f)),e++,d++,i._days.push(s);else if(r<c){var n=g[r],v=n.start&&a(n.start),h=n.end&&a(n.end),p=s.getDay(),O=0<j-p?7:0,m=h&&!b(v,h);v&&!b(s,v)&&p!=j||(void 0===n._id&&(n._id=eJ++),m&&(o[e]=n),n._days=[s],y=m?100*Math.min(nu(s,M(h))+1,7+j-p-O):100,l.push(m?'<div class="mbsc-cal-txt-w" style="width:'+y+'%">'+e1(n,r,!0)+"</div>"+e1(n,r):e1(n,r,!0)),e++,d++),r++}else l.push('<div class="mbsc-cal-txt-ph"></div>'),e++;I[s]=l,s=t.getDate(t.getYear(s),t.getMonth(s),t.getDay(s)+1)};s<e;)i()}()}function eX(e){var n=t.getYear(e),s=t.getMonth(e);eb(x=d=e),h("onMonthChange",{year:n,month:s}),h("onMonthLoading",{year:n,month:s}),h("onPageChange",{firstDay:e}),h("onPageLoading",{firstDay:e}),ez(e)}function eN(e){var n=t.getYear(e),s=t.getMonth(e);void 0===ed?eq(e,n,s):ex(e,ed,!0),eO(x,i.focus),i.focus=!1}function eq(n,s,o){var t=i.$scroller;e(".mbsc-cal-slide",t).removeClass("mbsc-cal-slide-a"),e(".mbsc-cal-slide",t).slice(E,E+l).addClass("mbsc-cal-slide-a"),W&&e(".mbsc-cal-slide-a .mbsc-cal-txt",t).on("mouseenter",function(){var n=e(this).attr("data-id");e('.mbsc-cal-txt[data-id="'+n+'"]',t).addClass("mbsc-hover")}).on("mouseleave",function(){e(".mbsc-cal-txt.mbsc-hover",t).removeClass("mbsc-hover")}),h("onMonthLoaded",{year:s,month:o}),h("onPageLoaded",{firstDay:n})}function eR(o,i){var n,e=t.getYear(o),s='<div class="mbsc-cal-slide"'+e7(i)+'><div role="grid" class="mbsc-cal-table"><div class="mbsc-cal-row">';for(n=0;n<12;n++)n&&n%3==0&&(s+='</div><div class="mbsc-cal-row">'),s+='<div role="gridcell" tabindex="-1" aria-label="'+e+'" data-val="'+e+'" class="mbsc-cal-cell mbsc-btn-e '+(e<ei||en<e?" mbsc-disabled ":"")+(e==t.getYear(d)?D:"")+'"><div class="mbsc-cal-cell-i mbsc-cal-cell-txt">'+e+e0+"</div></div>",e++;return s+="</div></div></div>"}function eP(k,z){var e,s,i,a,l,d,h,m,f,p,g,y,w,x,C,E,A,c=1,T=t.getYear(k),O=t.getMonth(k),S=t.getDay(k),M=null!==t.defaultValue||o._hasValue?o.getDate(!0):null,F=t.getDate(T,O,S).getDay(),N=0<j-F?7:0,u='<div class="mbsc-cal-slide"'+e7(z)+'><div role="grid" class="mbsc-cal-table"><div class="mbsc-cal-row">';for(i=0;i<7*r;i++)x=i+j-N,m=(s=t.getDate(T,O,x-F+S)).getFullYear(),h=s.getMonth(),d=s.getDate(),w=t.getMonth(s),a=t.getDay(s),A=t.getMaxDayOfMonth(m,h),C=m+"-"+(h+1)+"-"+d,g=(e=n({valid:(l=s,!(l<v||_<l||void 0!==eV[l]&&void 0===eW[l])),selected:M&&b(M,s)},eQ(s))).valid,y=e.selected,p=e.cssClass,E=new Date(s).setHours(12,0,0,0)===(new Date).setHours(12,0,0,0),f=w!==O,ec[C]=e,i&&i%7==0&&(u+='</div><div class="mbsc-cal-row">'),Y&&i%7==0&&("month"==Y&&f&&1<c?c=1==d?1:2:"year"==Y&&(c=t.getWeekNumber(t.getDate(m,h,d+(7-j+1)%7))),u+='<div role="gridcell" class="mbsc-cal-cell mbsc-cal-week-nr">'+c+"</div>",c++),u+='<div role="gridcell" aria-label="'+(E?t.todayText+", ":"")+t.dayNames[s.getDay()]+", "+t.monthNames[w]+" "+a+" "+(e.ariaLabel?", "+e.ariaLabel:"")+'"'+(f&&!em?' aria-hidden="true"':' data-full="'+C+'"')+(y?' aria-selected="true"':"")+(g?' tabindex="-1"':' aria-disabled="true"')+' class="mbsc-cal-cell mbsc-cal-day mbsc-cal-day'+x%7+" "+(t.dayClass||"")+" "+(y?D:"")+(E?" "+t.todayClass:"")+(p?" "+p:"")+(1==a?" mbsc-cal-day-first":"")+(a==A?" mbsc-cal-day-last":"")+(f?" mbsc-cal-day-diff":"")+(g?" mbsc-btn-e":" mbsc-disabled")+(e.marked?" mbsc-cal-day-marked":"")+(e.background?" mbsc-cal-day-colored":"")+'"><div class="mbsc-cal-cell-i mbsc-cal-day-i"><div class="mbsc-cal-day-date mbsc-cal-cell-txt"'+(e.background?' style="background:'+e.background+";color:"+np(e.background)+'"':"")+">"+a+"</div>"+(e.markup?"<div>"+e.markup+"</div>":"")+"</div></div>";return u+="</div></div></div>"}function ex(e,d,c){var n,o=t.getYear(e),a=t.getMonth(e),l=i?i.pos:0,s="";if(ec={},r)for(d||(h("onMonthLoading",{year:o,month:a}),h("onPageLoading",{firstDay:e})),ez(e),n=0;n<e9;n++)s+=eP(O(e,n-g-E),l*w+n-E);return ed=void 0,c&&i&&(i.$active=null,i.$scroller.html(s),eq(e,o,a)),s}function eO(n,s){if(i){var t=i.$active;t&&t.length&&(t[0].blur(),t.hasClass("mbsc-disabled")?t.removeAttr("tabindex"):t.attr("tabindex","-1")),i.$active=e('.mbsc-cal-slide-a .mbsc-cal-day[data-full="'+e2(n)+'"]',i.$scroller).attr("tabindex","0"),s&&i.$active.length&&i.$active[0].focus()}}function eY(n,t){e(".mbsc-selected",t).removeClass(D).removeAttr("aria-selected"),e('.mbsc-cal-cell[data-val="'+n+'"]',t).addClass(D).attr("aria-selected","true")}function V(n,c,u,h){var s,a;y&&(n<v&&(n=v),_<n&&(n=_),"calendar"!==y&&K&&!c||(o._isSetDate=!c,m&&r&&(a=Z(eM(n),p),e5&&(n<O(d,-g)||n>=O(d,l-g))&&(s=p?t.getMonth(a)-t.getMonth(d)+12*(t.getYear(a)-t.getYear(d)):Math.floor(nu(d,a)/(7*r)))&&(i.queue=[],i.focus=h&&u,e3(i,s,u)),s&&u||eO(n,h),c||function(s){var n=i&&i.$scroller;t.highlight&&i&&(e(".mbsc-selected",n).removeClass(D).removeAttr("aria-selected"),null===t.defaultValue&&!o._hasValue||e('.mbsc-cal-day[data-full="'+e2(s)+'"]',n).addClass(D).attr("aria-selected","true"))}(n),p||eb(n,!0),x=n,e5=!0),o._onSetDate(n,s),o._isSetDate=!1))}function eb(o,u){var n,r,d,s=t.getYear(o),a=t.getMonth(o),i=s+e0;if(N){if(eY(a,L.$scroller),eY(s,R.$scroller),e3(R,Math.floor(s/12)-Math.floor(t.getYear(R.first)/12),!0),e(".mbsc-cal-cell",L.$scroller).removeClass("mbsc-disabled"),s===ei)for(n=0;n<e$;n++)e('.mbsc-cal-cell[data-val="'+n+'"]',L.$scroller).addClass("mbsc-disabled");if(s===en)for(n=eL+1;n<=12;n++)e('.mbsc-cal-cell[data-val="'+n+'"]',L.$scroller).addClass("mbsc-disabled")}for(u||(ef(e(".mbsc-cal-prev-m",c),O(o,-g)<=v),ef(e(".mbsc-cal-next-m",c),O(o,l-g)>_),ef(e(".mbsc-cal-prev-y",c),t.getDate(s-1,a+1,1)<=v),ef(e(".mbsc-cal-next-y",c),t.getDate(s+1,a,1)>_)),e_.attr("aria-label",s).html(i),n=0;n<l;n++)o=t.getDate(s,a-g+n,1),d=t.getYear(o),r=t.getMonth(o),i=d+e0,G.eq(n).attr("aria-label",t.monthNames[r]+(S?"":" "+s)).html((!S&&eg<ea?i+" ":"")+er[r]+(!S&&ea<eg?" "+i:""))}function ef(e,t){t?e.addClass(eC).attr("aria-disabled","true"):e.removeClass(eC).removeAttr("aria-disabled")}function eU(i,g){var f,v,d=o.getDate(!0),r=i[0],c=i.attr("data-full"),l=c?c.split("-"):[],a=T(l[0],l[1]-1,l[2]),u=T(a.getFullYear(),a.getMonth(),a.getDate(),d.getHours(),d.getMinutes(),d.getSeconds()),p=i.hasClass("mbsc-selected"),s=e(g.target),m=s[0];if(em||!i.hasClass("mbsc-cal-day-diff")){if(W&&r.contains(m))for(;m!=r;){if(s.hasClass("mbsc-cal-txt")||s.hasClass("mbsc-cal-txt-more")){if(v=s.attr("data-index"),f=ey[a],!1===h("onLabelTap",{date:u,domEvent:g,target:s[0],labels:f,label:f[v]}))return;break}m=(s=s.parent())[0]}!1===h("onDayChange",n(ec[c],{date:u,target:r,selected:p}))||t.readonly||i.hasClass("mbsc-disabled")||o._selectDay(i,a,u,p)}}function eG(e){z(X),V(t.getDate(t.getYear(i.first),e.attr("data-val"),1),!0,!0)}function te(e){z(Q),V(t.getDate(e.attr("data-val"),t.getMonth(i.first),1),!0,!0)}function Z(e,a,n){var s=t.getYear(e),o=t.getMonth(e),i=e.getDay(),r=0<j-i?7:0;return a?t.getDate(s,o,1):t.getDate(s,o,(void 0===n?j:n)-r-i+t.getDay(e))}function O(e,n){var s=t.getYear(e),o=t.getMonth(e),i=t.getDay(e);return p?t.getDate(s,o+n,1):t.getDate(s,o,i+n*r*7)}function eD(e,n){var s=12*Math.floor(t.getYear(e)/12);return t.getDate(s+12*n,0,1)}function e3(t,n,s,i){n&&o._isVisible&&(t.queue.push(arguments),1==t.queue.length&&function y(t,n,j,b){var s,r,f="",i=t.$scroller,c=t.buffer,l=t.offset,m=t.pages,v=t.total,u=t.first,g=t.genPage,d=t.getFirst,a=0<n?Math.min(n,c):Math.max(n,-c),p=t.pos*w+a-n+l,h=Math.abs(n)>c;t.callback&&(t.load(),t.callback(!0)),t.first=d(u,n),t.pos+=a*w,t.changing=!0,t.load=function(){if(h){for(s=0;s<m;s++)f+=g(d(u,r=n+s-l),p+r);0<n?(e(".mbsc-cal-slide",i).slice(-m).remove(),i.append(f)):n<0&&(e(".mbsc-cal-slide",i).slice(0,m).remove(),i.prepend(f))}},t.callback=function(j){var f=Math.abs(a),m="";if(o._isVisible){for(s=0;s<f;s++)m+=g(d(u,r=n+s-l-c+(0<n?v-f:0)),p+r);if(0<n?(i.append(m),e(".mbsc-cal-slide",i).slice(0,a).remove()):n<0&&(i.prepend(m),e(".mbsc-cal-slide",i).slice(a).remove()),h){for(m="",s=0;s<f;s++)m+=g(d(u,r=n+s-l-c+(0<n?0:v-f)),p+r);0<n?(e(".mbsc-cal-slide",i).slice(0,a).remove(),i.prepend(m)):n<0&&(e(".mbsc-cal-slide",i).slice(a).remove(),i.append(m))}eh(t),b&&!j&&b(),t.callback=null,t.load=null,t.queue.shift(),h=!1,t.queue.length?y.apply(this,t.queue[0]):(t.changing=!1,t.onAfterChange(t.first))}},t.onBeforeChange(t.first),t.load&&(t.load(),t.scroller.scroll(-t.pos*t.size,j?200:0,!1,t.callback))}(t,n,s,i))}function ew(r,v,p,a,d,c,l,b,u,h,m,i,g){var s=f?"Y":"X",n={$scroller:e(".mbsc-cal-scroll",r),queue:[],buffer:a,offset:d,pages:c,first:b,total:l,pos:0,min:v,max:p,genPage:i,getFirst:g,onBeforeChange:h,onAfterChange:m};return n.scroller=new J(r,{axis:s,easing:"",contSize:0,maxSnapScroll:a,mousewheel:void 0===t.mousewheel?f:t.mousewheel,time:200,lock:!0,rtl:U,stopProp:!1,minScroll:0,maxScroll:0,onBtnTap:function(t){"touchend"==t.domEvent.type&&P(),u(e(t.target),t.domEvent)},onAnimationStart:function(){n.changing=!0},onAnimationEnd:function(e){i&&e3(n,Math.round((-n.pos*n.size-e["pos"+s])/n.size)*w)}}),o._scrollers.push(n.scroller),n}function eh(e,a){var t,s=0,o=0,i=e.first;if(!e.changing||!a){if(e.getFirst){for(s=e.buffer,o=e.buffer;o&&e.getFirst(i,o+e.pages-e.offset-1)>e.max;)o--;for(;s&&e.getFirst(i,1-s-e.offset)<=e.min;)s--}t=Math.round(e4/e.pages),A&&t&&e.size!=t&&e.$scroller[f?"height":"width"](t),n(e.scroller.settings,{snap:t,minScroll:(-e.pos*w-o)*t,maxScroll:(-e.pos*w+s)*t}),e.size=t,e.scroller.refresh()}}function eA(e){o._onRefresh(e),o._isVisible&&m&&r&&(i&&i.changing?ed=e:(ex(d,e,!0),eO(x)))}return u={},$=[],I={},h=o.trigger,eB=n({},o.settings),eo=(t=n(o.settings,nD,eB)).controls.join(","),j=t.firstDay,U=t.rtl,E=t.pageBuffer,Y=t.weekCounter,r=t.weeks,p=6==r,f="vertical"==t.calendarScroll,eK=o._getRespCont(),eE="full"==t.weekDays?"":"min"==t.weekDays?"Min":"Short",ej=t.layout||("inline"==t.display||/top|bottom/.test(t.display)&&t.touchUi?"liquid":""),es=(A="liquid"==ej)?null:t.calendarWidth,w=U&&!f?-1:1,eC="mbsc-disabled "+(t.disabledClass||""),el="mbsc-selected "+(t.selectedTabClass||""),D="mbsc-selected "+(t.selectedClass||""),eu=Math.max(1,Math.floor(((t.calendarHeight||0)/r-45)/18)),eo.match(/calendar/)&&(u.calendar=1,m=!0),eo.match(/date/)&&!m&&(u.date=1),eo.match(/time/)&&(u.time=1),t.controls.forEach(function(e){u[e]&&$.push(e)}),N=t.quickNav&&m&&p,S=t.yearChange&&p,A&&m&&"center"==t.display&&(o._isFullScreen=!0),t.layout=ej,t.preset=(u.date||m?"date":"")+(u.time?"time":""),ek=eF.call(this,o),er=S?t.monthNamesShort:t.monthNames,e0=t.yearSuffix||"",ea=(t.dateWheels||t.dateFormat).search(/m/i),eg=(t.dateWheels||t.dateFormat).search(/y/i),F=o._format,t.min&&(v=M(a(t.min,F,t)),ei=t.getYear(v),e$=t.getMonth(v),eI=t.getDate(12*Math.floor(ei/12),0,1)),t.max&&(_=M(a(t.max,F,t)),en=t.getYear(_),eL=t.getMonth(_),eT=t.getDate(12*Math.floor(en/12),0,1)),o.refresh=function(){eA(!1)},o.redraw=function(){eA(!0)},o.navigate=function(e,n){V(a(e,F,t),!0,n)},o.changeTab=function(t){o._isVisible&&u[t]&&y!=t&&(y=t,e(".mbsc-cal-tab",c).removeClass(el).removeAttr("aria-selected"),e('.mbsc-cal-tab[data-control="'+t+'"]',c).addClass(el).attr("aria-selected","true"),K&&(H.addClass("mbsc-cal-h"),u[y].removeClass("mbsc-cal-h")),"calendar"==y&&V(o.getDate(!0),!1,!0),o._showDayPicker(),o.trigger("onTabChange",{tab:y}))},o._checkSize=!0,o._onGenMonth=s,o._onSelectShow=s,o._onSetDate=s,o._onRefresh=s,o._getDayProps=s,o._prepareObj=q,o._showDayPicker=function(){N&&(z(Q),z(X))},o._selectDay=o.__selectDay=function(s,i,n){var e=o.live;e5=t.outerMonthChange,et=!0,o.setDate(n,e,1e3,!e,!0),e&&h("onSet",{valueText:o._value})},n(ek,{labels:null,compClass:"mbsc-calendar mbsc-dt mbsc-sc",onMarkupReady:function(a){var n=0;c=e(a.target),e8=e(".mbsc-fr-c",c),x=o.getDate(!0),e4=0,m&&(ev=!(!t.marked&&!t.data||t.labels||t.multiLabel||t.showEventCount),e5=!0,y="calendar",l="auto"==t.months?Math.max(1,Math.min(3,Math.floor((es||tU(eK))/280))):+t.months,e9=l+2*E,f=f&&l<2,em=void(g=0)===t.showOuterDays?l<2&&!f:t.showOuterDays,d=Z(eM(x),p),e8.append(function(){var e,n,s,o,i,c,a="",u=U?t.btnCalNextClass:t.btnCalPrevClass,h=U?t.btnCalPrevClass:t.btnCalNextClass;for(o='<div class="mbsc-cal-btn-w"><div data-step="-1" role="button" tabindex="0" aria-label="'+t.prevMonthText+'" class="'+u+' mbsc-cal-prev mbsc-cal-prev-m mbsc-cal-btn mbsc-fr-btn mbsc-fr-btn-e"></div>',e=0;e<(r?l:1);e++)o+='<div role="button" class="mbsc-cal-month"></div>';if(o+='<div data-step="1" role="button" tabindex="0" aria-label="'+t.nextMonthText+'" class="'+h+' mbsc-cal-next mbsc-cal-next-m mbsc-cal-btn mbsc-fr-btn mbsc-fr-btn-e"></div></div>',S&&(a='<div class="mbsc-cal-btn-w"><div data-step="-12" role="button" tabindex="0" aria-label="'+t.prevYearText+'" class="'+u+' mbsc-cal-prev mbsc-cal-prev-y mbsc-cal-btn mbsc-fr-btn mbsc-fr-btn-e"></div><div role="button" class="mbsc-cal-year"></div><div data-step="12" role="button" tabindex="0" aria-label="'+t.nextYearText+'" class="'+h+' mbsc-cal-next mbsc-cal-next-y mbsc-cal-btn mbsc-fr-btn mbsc-fr-btn-e"></div></div>'),r&&(c=ex(d)),n='<div class="mbsc-w-p mbsc-cal-c"><div class="mbsc-cal '+(p?"":" mbsc-cal-week-view")+(1<l?" mbsc-cal-multi ":"")+(Y?" mbsc-cal-weeks ":"")+(f?" mbsc-cal-vertical":"")+(ev?" mbsc-cal-has-marks":"")+(W?" mbsc-cal-has-labels":"")+(em?"":" mbsc-cal-hide-diff ")+(t.calendarClass||"")+'"'+(A?"":' style="width:'+(es||280*l)+'px;"')+'><div class="mbsc-cal-hdr">'+(eg<ea||1<l?a+o:o+a)+"</div>",r){for(n+='<div class="mbsc-cal-body"><div class="mbsc-cal-day-picker"><div class="mbsc-cal-days-c">',s=0;s<l;s++){for(n+='<div class="mbsc-cal-days">',e=0;e<7;e++)n+='<div class="mbsc-cal-week-day'+(i=(e+j)%7)+'" aria-label="'+t.dayNames[i]+'">'+t["dayNames"+eE][i]+"</div>";n+="</div>"}n+='</div><div class="mbsc-cal-scroll-c mbsc-cal-day-scroll-c '+(t.calendarClass||"")+'"'+(t.calendarHeight?' style="height:'+t.calendarHeight+'px"':"")+'><div class="mbsc-cal-scroll" style="width:'+100/l+'%">'+c+"</div></div>"}if(n+="</div>",N){for(n+='<div class="mbsc-cal-month-picker mbsc-cal-picker mbsc-cal-h"><div class="mbsc-cal-scroll-c '+(t.calendarClass||"")+'"><div class="mbsc-cal-scroll">',e=0;e<3;e++){for(n+='<div class="mbsc-cal-slide"'+e7(e-1)+'><div role="grid" class="mbsc-cal-table"><div class="mbsc-cal-row">',s=0;s<12;s++)s&&s%3==0&&(n+='</div><div class="mbsc-cal-row">'),n+='<div role="gridcell"'+(1==e?' tabindex="-1" aria-label="'+t.monthNames[s]+'" data-val="'+s+'"':"")+' class="mbsc-cal-cell'+(1==e?" mbsc-btn-e":"")+'"><div class="mbsc-cal-cell-i mbsc-cal-cell-txt">'+(1==e?t.monthNamesShort[s]:"&nbsp;")+"</div></div>";n+="</div></div></div>"}for(n+="</div></div></div>",n+='<div class="mbsc-cal-year-picker mbsc-cal-picker mbsc-cal-h"><div class="mbsc-cal-scroll-c '+(t.calendarClass||"")+'"><div class="mbsc-cal-scroll">',e=-1;e<2;e++)n+=eR(eD(d,e),e);n+="</div></div></div>"}return n+="</div></div></div>"}()),G=e(".mbsc-cal-month",c),e_=e(".mbsc-cal-year",c),k=e(".mbsc-cal-day-scroll-c",c)),N&&(Q=e(".mbsc-cal-year-picker",c),X=e(".mbsc-cal-month-picker",c)),H=e(".mbsc-w-p",c),1<$.length&&e8.before(function(){var e,n;return e='<div class="mbsc-cal-tabs-c"><div class="mbsc-cal-tabs" role="tablist">',$.forEach(function(s,o){n=t[("calendar"==s?"date":s)+"Text"],e+='<div role="tab" aria-controls="'+eZ.id+"-mbsc-pnl-"+o+'" class="mbsc-cal-tab mbsc-fr-btn-e '+(o?"":el)+'" data-control="'+s+'"'+(t.tabLink?'><a href="#">'+n+"</a>":' tabindex="0">'+n)+"</div>"}),e+="</div></div>"}()),["date","time","calendar"].forEach(function(e){u[e]?(u[e]=H.eq(n),n++):"date"==e&&!u.date&&m&&(H.eq(n).remove(),n++)}),$.forEach(function(e){e8.append(u[e])}),!m&&u.date&&u.date.css("position","relative"),o._scrollers=[],function(){if(m&&r){var n=e(".mbsc-cal-scroll-c",c);i=ew(n[0],v,_,E,g,l,e9,d,eU,eX,eN,eP,O),N&&(L=ew(n[1],null,null,1,0,1,3,d,eG),R=ew(n[2],eI,eT,1,0,1,3,d,te,s,s,eR,eD),o.tap(G,function(){eH(X),z(Q)}),o.tap(e_,function(){eH(Q),z(X)})),eS(e(".mbsc-cal-btn",c),function(n,e,s,t){e3(i,e,!0,t)}),eN(d),null===t.defaultValue&&!o._hasValue||o._multiple||(o._activeElm=i.$active[0]),k.on("keydown",function(a){var s,e=t.getYear(x),n=t.getMonth(x),o=t.getDay(x);switch(a.keyCode){case 32:eU(i.$active,a);break;case 37:s=t.getDate(e,n,o-1*w);break;case 39:s=t.getDate(e,n,o+1*w);break;case 38:s=t.getDate(e,n,o-7);break;case 40:s=t.getDate(e,n,o+7);break;case 36:s=t.getDate(e,n,1);break;case 35:s=t.getDate(e,n+1,0);break;case 33:s=a.altKey?t.getDate(e-1,n,o):p?t.getDate(e,n-1,o):t.getDate(e,n,o-7*r);break;case 34:s=a.altKey?t.getDate(e+1,n,o):p?t.getDate(e,n+1,o):t.getDate(e,n,o+7*r)}s&&(a.preventDefault(),V(s,!0,!1,!0))})}o.tap(e(".mbsc-cal-tab",c),function(){o.changeTab(e(this).attr("data-control"))})}()},onShow:function(){m&&r&&eb(p?d:x)},onHide:function(){o._scrollers.forEach(function(e){e.destroy()}),y=R=L=i=ec=null},onValidated:function(s){var t,e=s.index,n=o._order,i=o.getDate(!0);et?t="calendar":void 0!==e&&(t=n.dd==e||n.d==e||n.m==e||n.y==e?"date":"time"),h("onSetDate",{date:i,control:t}),"time"!==t&&V(i,!1,!!s.time,et&&!o._multiple),et=!1},onPosition:function(n){if(_=n.oldHeight,h=n.windowHeight,K=(n.hasTabs||!0===t.tabs||!1!==t.tabs&&A)&&1<$.length,A&&(n.windowWidth>=t.breakPointMd?e(n.target).addClass("mbsc-fr-md"):e(n.target).removeClass("mbsc-fr-md")),K?(c.addClass("mbsc-cal-tabbed"),y=e(".mbsc-cal-tab.mbsc-selected",c).attr("data-control"),H.addClass("mbsc-cal-h"),u[y].removeClass("mbsc-cal-h")):(c.removeClass("mbsc-cal-tabbed"),H.removeClass("mbsc-cal-h")),o._isFullScreen&&(k.height(""),p=h-(v=n.popup.offsetHeight)+k[0].offsetHeight,v<=h&&k.height(p)),W&&r&&h!=_){var s,a,h,p,v,b,j,_,w,C=p||k[0].offsetHeight,O=k.find(".mbsc-cal-txt-ph")[0],E=O.offsetTop,M=O.offsetHeight,x=Math.max(1,Math.floor((C/r-E)/(M+2)));eu!=x&&(eu=x,o.redraw())}if(m&&r){if(a=(b=A||f||K?k[0][f?"offsetHeight":"offsetWidth"]:es||280*l)!=e4,e4=b,A&&a&&S)for(er=t.maxMonthWidth>G[0].offsetWidth?t.monthNamesShort:t.monthNames,w=t.getYear(d),j=t.getMonth(d),s=0;s<l;s++)G.eq(s).text(er[t.getMonth(t.getDate(w,j-g+s,1))]);a&&eh(i,!0)}N&&a&&(eh(L,!0),eh(R,!0))}})}var nN={separator:" ",dateFormat:"mm/dd/yy",dateDisplay:"MMddyy",timeFormat:"h:ii A",dayText:"Day",monthText:"Month",yearText:"Year",hourText:"Hours",minuteText:"Minutes",ampmText:"&nbsp;",secText:"Seconds",nowText:"Now",todayText:"Today"},nD={controls:["calendar"],firstDay:0,weekDays:"short",maxMonthWidth:170,breakPointMd:768,months:1,pageBuffer:1,weeks:6,highlight:!0,outerMonthChange:!0,quickNav:!0,yearChange:!0,tabs:"auto",todayClass:"mbsc-cal-today",btnCalPrevClass:"mbsc-ic mbsc-ic-arrow-left6",btnCalNextClass:"mbsc-ic mbsc-ic-arrow-right6",dateText:"Date",timeText:"Time",todayText:"Today",fromText:"Start",toText:"End",moreEventsText:"{count} more",prevMonthText:"Previous Month",nextMonthText:"Next Month",prevYearText:"Previous Year",nextYearText:"Next Year"},nz={};c.calendar=function(o){function m(e){var n,o,i,r=null;if(s={},e&&e.length)for(o=0;o<e.length;o++)n=a(e[o],c,t,t.isoParts),r=r||n,s[T((i=n).getFullYear(),i.getMonth(),i.getDate())]=n;return r}function h(){o.redraw()}var r,c,u,f,g,j=n({},o.settings),t=n(o.settings,nz,j),p="mbsc-selected "+(t.selectedClass||""),d=t.defaultValue,i="multiple"==t.select||1<t.select||"week"==t.selectType,b=v(t.select)?t.select:1/0,s={};return r=eZ.call(this,o),g=void 0===t.firstSelectDay?t.firstDay:t.firstSelectDay,c=o._format,i&&m(d),o._multiple=i,o._getDayProps=function(e){return{selected:i?void 0!==s[e]:void 0}},o._selectDay=function(f,n,d,m){if(t.setOnDayTap&&"multiple"!=t.select&&"inline"!=t.display)return o.setDate(d),void o.select();if(i)if("week"==t.selectType){var r,c,l,a=n.getDay()-g;for(a=a<0?7+a:a,"multiple"!=t.select&&(s={}),r=0;r<7;r++)c=T(n.getFullYear(),n.getMonth(),n.getDate()-a+r),m?delete s[c]:eH(s).length/7<b&&(s[c]=c);h()}else l=e('.mbsc-cal-day[data-full="'+f.attr("data-full")+'"]',u),m?(l.removeClass(p).removeAttr("aria-selected"),delete s[n]):eH(s).length<b&&(l.addClass(p).attr("aria-selected","true"),s[n]=n);o.__selectDay(f,n,d)},o.setVal=function(e,t,n,s,a){i&&(e=m(e)),o._setVal(e,t,n,s,a),i&&h()},o.getVal=function(a){var e,n=[];if(i){for(e in s)n.push(K(s[e],t,c));return n}return K(o.getDate(a),t,c)},n({},r,{highlight:!i,outerMonthChange:!i,parseValue:function(e){return i&&e&&"string"==typeof e&&(e=m(e.split(","))),i&&d&&d.length&&(t.defaultValue=d[0]),r.parseValue.call(this,e)},formatValue:function(a){var e,n=[];if(i){for(e in s)n.push(l(c,s[e],t));return n.join(", ")}return r.formatValue.call(this,a,o)},onClear:function(){i&&(s={},h())},onBeforeShow:function(){void 0!==t.setOnDayTap||t.buttons&&t.buttons.length||1!=t.controls.length||(t.setOnDayTap=!0),t.setOnDayTap&&"inline"!=t.display&&(t.outerMonthChange=!1),t.counter&&i&&(t.headerText=function(){var n=0,o="week"==t.selectType?7:1;return e.each(s,function(){n++}),(1<(n=Math.round(n/o))&&t.selectedPluralText||t.selectedText).replace(/{count}/,n)})},onMarkupReady:function(t){r.onMarkupReady.call(this,t),u=e(t.target),i&&(e(".mbsc-fr-hdr",u).attr("aria-live","off"),f=n({},s))},onCancel:function(){!o.live&&i&&(s=n({},f))}})},r("calendar",d);var eE,ex="mbsc-input-wrap",tI=["touchstart","touchmove","touchend","touchcancel","mousedown","mousemove","mouseup","mouseleave"],nF={tap:et};function tL(s,r){var t={},c=s[0],o=s.parent(),a=o.find(".mbsc-err-msg"),l=s.attr("data-icon-align")||"left",i=s.attr("data-icon");o.hasClass(ex)?o=o.parent():e('<span class="'+ex+'"></span>').insertAfter(s).append(s),a&&o.find("."+ex).append(a),i&&(-1!==i.indexOf("{")?t=JSON.parse(i):t[l]=i),"file"==c.type&&(t.right=s.attr("data-icon-upload")||"upload"),(i||r)&&(n(t,r),o.addClass((t.right?"mbsc-ic-right ":"")+(t.left?" mbsc-ic-left":"")).find("."+ex).append('<span class="mbsc-input-fill"></span>').append(t.left?'<span class="mbsc-input-ic mbsc-left-ic mbsc-ic mbsc-ic-'+t.left+'"></span>':"").append(t.right?'<span class="mbsc-input-ic mbsc-right-ic mbsc-ic mbsc-ic-'+t.right+'"></span>':""))}function tF(s,o,t,n,i){"segmented"==o?s.closest(".mbsc-segmented").addClass("box"==t?"mbsc-input-box":"").addClass("outline"==t?"mbsc-input-outline":""):"button"!=o&&"submit"!=o&&(s.addClass("mbsc-control-w").addClass("box"==t?"mbsc-input-box":"").addClass("outline"==t?"mbsc-input-outline":"").addClass("inline"==n?"mbsc-label-inline":"").addClass("stacked"==n?"mbsc-label-stacked":"").addClass("floating"==n?"mbsc-label-floating":"").addClass("floating"==n&&i.value?"mbsc-label-floating-active":"").find("label").addClass("mbsc-label").each(function(n,t){e(t).attr("title",e(t).text())}),s.contents().filter(function(){return 3==this.nodeType&&this.nodeValue&&/\S/.test(this.nodeValue)}).each(function(){e('<span class="mbsc-label" title="'+this.textContent.trim()+'"></span>').insertAfter(this).append(this)}))}function tA(n){var e=t.themes.form[n];return e&&e.addRipple?e:null}function tk(t,n,s){var e=t.attr(n);return void 0===e||""===e?s:e}function td(e){var n=t.themes.form[e.theme].baseTheme;return"mbsc-"+e.theme+(n?" mbsc-"+n:"")+(e.rtl?" mbsc-rtl":" mbsc-ltr")}X=function(){function s(a,m){d=this,E(this,s);var d,o=n({},nF,t.settings,m),i=e(a),l=i.parent(),r=l.hasClass("mbsc-input-wrap")?l.parent():l,c=i.next().hasClass("mbsc-fr")?i.next():null,u=ej(i),f=tk(i,"data-input-style",o.inputStyle),h=tk(i,"data-label-style",o.labelStyle);a.mbscInst&&a.mbscInst.destroy(),c&&c.insertAfter(r),o.theme=tq(o),void 0===o.rtl&&o.lang&&t.i18n[o.lang]&&(o.rtl=t.i18n[o.lang].rtl),tF(r,u,f,h,a),i.addClass("mbsc-control"),this._handle=this._handle.bind(this),tI.forEach(function(e){i.on(e,d._handle)}),this.settings=o,this._type=u,this._elm=a,this._$elm=i,this._$parent=r,this._$frame=c,this._ripple=tA(o.theme),this._isFloating="floating"==h||r.hasClass("mbsc-label-floating"),this.cssClass=td(o),this.getClassElm().addClass(this.cssClass),a.mbscInst=this}return Y(s,[{key:"getClassElm",value:function(){return this._$parent}},{key:"destroy",value:function(){var e=this;this._$elm.removeClass("mbsc-control"),this.getClassElm().removeClass(this.cssClass),tI.forEach(function(t){e._$elm.off(t,e._handle)}),delete this._elm.mbscInst}},{key:"option",value:function(t){n(this.settings,t);var e=this.getClassElm();this.cssClass&&e.removeClass(this.cssClass),this.cssClass=td(this.settings),e.addClass(this.cssClass),this._ripple=tA(this.settings.theme)}},{key:"_handle",value:function(e){switch(e.type){case"touchstart":case"mousedown":this._onStart(e);break;case"touchmove":case"mousemove":this._onMove(e);break;case"touchend":case"touchcancel":case"mouseup":case"mouseleave":this._onEnd(e)}}},{key:"_addRipple",value:function(e){this._ripple&&this._$rippleElm&&this._ripple.addRipple(this._$rippleElm,e)}},{key:"_removeRipple",value:function(){this._ripple&&this._$rippleElm&&this._ripple.removeRipple()}},{key:"_onStart",value:function(e){var t=this._elm;H(e,t)&&(this._startX=o(e,"X"),this._startY=o(e,"Y"),eE&&eE.removeClass("mbsc-active"),t.disabled||(this._isActive=!0,(eE=this._$elm).addClass("mbsc-active"),this._addRipple(e))),"touchstart"==e.type&&this._$elm.closest(".mbsc-no-touch").removeClass("mbsc-no-touch")}},{key:"_onMove",value:function(e){(this._isActive&&9<Math.abs(o(e,"X")-this._startX)||9<Math.abs(o(e,"Y")-this._startY))&&(this._$elm.removeClass("mbsc-active"),this._removeRipple(),this._isActive=!1)}},{key:"_onEnd",value:function(e){var t=this,n=this._elm,s=this._type;this._isActive&&this.settings.tap&&"touchend"==e.type&&!n.readOnly&&t4(n,s,e),this._isActive&&setTimeout(function(){t._$elm.removeClass("mbsc-active"),t._removeRipple()},100),this._isActive=!1,eE=null}}]),s}(),t.themes.form.mobiscroll={},t1=["focus","change","blur","animationstart"],e0=function(){function t(r,c){E(this,t);var n,a,s=(n=q(this,g(t).call(this,r,c)))._$elm,o=n._$parent,i=o.find(".mbsc-select-input, .mbsc-color-input");return(!function(r,c,t){var i={},n=t[0],a=t.attr("data-password-toggle"),s=t.attr("data-icon-show")||"eye",o=t.attr("data-icon-hide")||"eye-blocked";a&&(i.right="password"==n.type?s:o),tL(t,i),a&&eI(r,c.find(".mbsc-right-ic").addClass("mbsc-input-toggle"),function(){"text"==n.type?(n.type="password",e(this).addClass("mbsc-ic-"+s).removeClass("mbsc-ic-"+o)):(n.type="text",e(this).removeClass("mbsc-ic-"+s).addClass("mbsc-ic-"+o))})}(ea(n),o,s),n._checkLabel=n._checkLabel.bind(ea(n)),n._mouseDown=n._mouseDown.bind(ea(n)),n._setText=n._setText.bind(ea(n)),"file"==r.type)&&(a=o.find(".mbsc-file-input"),n._$input=a.length?a:e('<input type="text" class="'+(s.attr("class")||"")+' mbsc-file-input" placeholder="'+(s.attr("placeholder")||"")+'"/>').insertAfter(s),s.on("change",n._setText)),o.addClass("mbsc-input").on("mousedown",n._mouseDown),t1.forEach(function(e){s.on(e,n._checkLabel)}),i.length&&(s.after(i),i.hasClass("mbsc-select-input")&&(n._delm=i[0],n.refresh())),n}return G(t,X),Y(t,[{key:"_setText",value:function(s){for(var t=s.target.files,n=[],e=0;e<t.length;++e)n.push(t[e].name);this._$input.val(n)}},{key:"_checkLabel",value:function(e){if(this._isFloating){var t=this._delm||this._elm;t.value||document.activeElement===t||e&&("focus"==e.type||"animationstart"==e.type&&this._$elm.is("*:-webkit-autofill"))?this._$parent.addClass("mbsc-label-floating-active"):this._$parent.removeClass("mbsc-label-floating-active")}}},{key:"_mouseDown",value:function(e){document.activeElement===this._elm&&e.target!==this._elm&&e.preventDefault()}},{key:"refresh",value:function(){this._checkLabel()}},{key:"destroy",value:function(){var e=this;er(g(t.prototype),"destroy",this).call(this),this._$parent.off("mousedown",this._mouseDown).removeClass("mbsc-ic-left mbsc-ic-right").find(".mbsc-input-ic").remove(),this._$parent.find(".mbsc-input-fill").remove(),t1.forEach(function(t){e._$elm.off(t,e._checkLabel)}),this._$elm.off("change",this._setText)}}]),t}(),p("[mbsc-input]",e0),t0=function(){function e(o,i){E(this,e);var n,t=(n=q(this,g(e).call(this,o,i)))._$elm,s=t.attr("data-icon");return t.addClass("mbsc-btn mbsc-no-touch").find(".mbsc-btn-ic").remove(),s&&(t.prepend('<span class="mbsc-btn-ic mbsc-ic mbsc-ic-'+s+'"></span>'),""===t.text()&&t.addClass("mbsc-btn-icon-only")),n._$rippleElm=t,n}return G(e,X),Y(e,[{key:"getClassElm",value:function(){return this._$elm}}]),e}(),p("[mbsc-button]",t0),t2=function(){function e(n,s){var t;return E(this,e),(t=q(this,g(e).call(this,n,s)))._$parent.prepend(t._$elm).addClass("mbsc-checkbox mbsc-control-w").find(".mbsc-checkbox-box").remove(),t._$elm.after('<span class="mbsc-checkbox-box"></span>'),t}return G(e,X),e}(),p("[mbsc-checkbox]",t2),t3=function(){function e(n,s){var t;return E(this,e),(t=q(this,g(e).call(this,n,s)))._$parent.addClass("mbsc-radio mbsc-control-w").find(".mbsc-radio-box").remove(),t._$elm.after('<span class="mbsc-radio-box"><span></span></span>'),t}return G(e,X),e}(),p("[mbsc-radio]",t3),th=function(){function t(c,r){E(this,t);var n,o=(n=q(this,g(t).call(this,c,r)))._$elm,i=n._$parent,a=i.find(".mbsc-select-input"),s=a.length?a:e('<input tabindex="-1" class="mbsc-select-input mbsc-control" readonly>');return n._$input=s,n._delm=s[0],n._setText=n._setText.bind(ea(n)),i.addClass("mbsc-select"+(n._$frame?" mbsc-select-inline":"")),o.after(s),s.after('<span class="mbsc-select-ic mbsc-ic mbsc-ic-arrow-down5"></span>'),o.on("change",n._setText),n._setText(),n}return G(t,e0),Y(t,[{key:"destroy",value:function(){er(g(t.prototype),"destroy",this).call(this),this._$parent.find(".mbsc-select-ic").remove(),this._$elm.off("change",this._setText)}},{key:"_setText",value:function(){var t=this._elm,n=e(t);n.is("select")&&!n.hasClass("mbsc-comp")&&this._$input.val(-1!=t.selectedIndex?t.options[t.selectedIndex].text:""),this.refresh()}}]),t}(),p("[mbsc-dropdown]",th),t8=["change","keydown","input","scroll"];function tO(){clearTimeout(tC),tC=setTimeout(function(){e("textarea.mbsc-control").each(function(){ed(this)})},100)}function ed(t){var n,s,o,i=e(t).attr("rows")||6;t.offsetHeight&&(t.style.height="",s=t.scrollHeight-t.offsetHeight,n=t.offsetHeight+(0<s?s:0),i<(o=Math.round(n/24))?(n=24*i+(n-24*o),e(t).addClass("mbsc-textarea-scroll")):e(t).removeClass("mbsc-textarea-scroll"),n&&(t.style.height=n+"px"))}y&&e(window).on("resize orientationchange",tO),tp=function(){function t(n,s){var e;return E(this,t),(e=q(this,g(t).call(this,n,s)))._$parent.addClass("mbsc-textarea"),t8.forEach(function(t){e._$elm.on(t,e._handle)}),ed(n),e}return G(t,e0),Y(t,[{key:"destroy",value:function(){var e=this;er(g(t.prototype),"destroy",this).call(this),t8.forEach(function(t){e._$elm.off(t,e._handle)})}},{key:"refresh",value:function(){er(g(t.prototype),"refresh",this).call(this),clearTimeout(this._debounce),ed(this._elm)}},{key:"_handle",value:function(n){switch(er(g(t.prototype),"_handle",this).call(this,n),n.type){case"change":ed(this._elm);break;case"keydown":case"input":this._onInput(n);break;case"scroll":!function(t){if(n=e(t),!n.hasClass("mbsc-textarea-scroll")){var n,o=t.scrollHeight-t.offsetHeight,s=t.offsetHeight+o;Math.round(s/24)<=(n.attr("rows")||6)&&(t.scrollTop=0,t.style.height=s+"px")}}(this._elm)}}},{key:"_onInput",value:function(){var e=this;clearTimeout(this._debounce),this._debounce=setTimeout(function(){ed(e._elm)},100)}}]),t}(),p("[mbsc-textarea]",tp),tg=function(){function t(c,r){E(this,t);var n,s,i,a=(s=q(this,g(t).call(this,c,r)))._$elm,o=s._$parent;return o.hasClass("mbsc-segmented-item-ready")||(i=e('<div class="mbsc-segmented mbsc-no-touch"></div>'),o.after(i),o.parent().find('input[name="'+a.attr("name")+'"]').each(function(){var t=e(this);n=t.parent().addClass("mbsc-segmented-item mbsc-segmented-item-ready"),e('<span class="mbsc-segmented-content">'+(t.attr("data-icon")?'<span class="mbsc-ic mbsc-ic-'+t.attr("data-icon")+'"></span>':"")+"</span>").append(n.contents()).appendTo(n),n.prepend(t),i.append(n)})),s._$rippleElm=a.next(),s}return G(t,X),Y(t,[{key:"getClassElm",value:function(){return this._$elm.closest(".mbsc-segmented")}}]),t}(),p("[mbsc-segmented]",tg);function eb(l,i){var n,o,a,c,u,h,m,f,g,v,j,y,_,w,E,k,d="",r=this,s=e(l),C=n;function x(){var t;l.disabled||(t=parseFloat(e(this).val()),b(isNaN(t)?n:t))}function A(){return l.disabled}function M(t,e){b(n+e*c)}function b(o,e,t){C=n,void 0===e&&(e=!0),void 0===t&&(t=e),n=O(o),y.removeClass("mbsc-disabled"),e&&s.val(n),n==h?_.addClass("mbsc-disabled"):n==m&&w.addClass("mbsc-disabled"),n!==C&&t&&s.trigger("change")}function p(t,n,o){var e=s.attr(t);return void 0===e||""===e?n:o?e:+e}function O(e){return+Math.min(m,Math.max(Math.round(e/c)*c,h)).toFixed(k)}S.call(this,l,i,!0),r.getVal=function(){var e=parseFloat(s.val());return O(e=isNaN(e)?n:e)},r.setVal=function(e,t,s){e=parseFloat(e),b(isNaN(e)?n:e,t,s)},r._init=function(){f=s.parent().hasClass("mbsc-stepper"),o=f?s.closest(".mbsc-stepper-cont"):s.parent(),a=r.settings,h=void 0===i.min?p("min",a.min):i.min,m=void 0===i.max?p("max",a.max):i.max,c=void 0===i.step?p("step",a.step):i.step,k=Math.abs(c)<1?(c+"").split(".")[1].length:0,v=void 0===i.inputStyle?p("data-input-style",a.inputStyle,!0):i.inputStyle,j=s.attr("data-val")||a.val,n=O(+l.value||0),u=t.themes.form[a.theme],E=u&&u.addRipple?u:null,f||o.addClass("mbsc-stepper-cont mbsc-no-touch mbsc-control-w").addClass("box"==v?"mbsc-input-box":"").addClass("outline"==v?"mbsc-input-outline":"").append('<span class="mbsc-segmented mbsc-stepper"></span>').find(".mbsc-stepper").append('<span class="mbsc-segmented-item mbsc-stepper-control mbsc-stepper-minus '+(n==h?"mbsc-disabled":"")+'" data-step="-1" tabindex="0"><span class="mbsc-segmented-content"><span class="mbsc-ic mbsc-ic-minus"></span></span></span>').append('<span class="mbsc-segmented-item mbsc-stepper-control mbsc-stepper-plus '+(n==m?"mbsc-disabled":"")+'"  data-step="1" tabindex="0"><span class="mbsc-segmented-content"> <span class="mbsc-ic mbsc-ic-plus"></span></span></span>').prepend(s),d&&(o.removeClass(d),o.find(".mbsc-segmented").removeClass(d)),d="mbsc-"+a.theme+(u.baseTheme?" mbsc-"+u.baseTheme:"")+(a.rtl?" mbsc-rtl":" mbsc-ltr"),o.addClass(d),o.find(".mbsc-segmented").addClass(d),_=e(".mbsc-stepper-minus",o),w=e(".mbsc-stepper-plus",o),y=e(".mbsc-stepper-control",o),f||("left"==j?(o.addClass("mbsc-stepper-val-left"),s.after('<span class="mbsc-segmented-item"><span class="mbsc-segmented-content"></span></span>')):"right"==j?(o.addClass("mbsc-stepper-val-right"),w.after('<span class="mbsc-segmented-item"><span class="mbsc-segmented-content"></span></span>')):_.after('<span class="mbsc-segmented-item"><span class="mbsc-segmented-content mbsc-stepper-val"></span></span>')),g||(s.on("change",x),g=eS(y,M,150,A,!1,E)),s.val(n).attr("data-role","stepper").attr("min",h).attr("max",m).attr("step",c).addClass("mbsc-control"),l.mbscInst=r},r._destroy=function(){s.removeClass("mbsc-control").off("change",x),g.destroy(),delete l.mbscInst},r.init()}eb.prototype={_class:"stepper",_hasDef:!0,_hasTheme:!0,_hasLang:!0,_defaults:{min:0,max:100,step:1}},p("[mbsc-stepper]",u.Stepper=eb);function ty(i,l,c){var n,o,a,r,t=this;S.call(this,i,l,!0),t.__init=s,t.__destroy=s,t._init=function(){var s;r=t.settings,a=e(i),s=!!n,n=(n=a.parent()).hasClass("mbsc-input-wrap")?n.parent():n,t._$parent=n,o&&n.removeClass(o),o=t._css+" mbsc-progress-w mbsc-control-w "+td(r),n.addClass(o),a.addClass("mbsc-control"),t.__init(),s||t._attachChange(),t.refresh(),i.mbscInst=t},t._destroy=function(){t.__destroy(),n.removeClass(o),a.removeClass("mbsc-control"),delete i.mbscInst},c||t.init()}function tj(M,et,en){var n,i,a,r,c,d,u,h,m,f,p,g,v,b,j,w,O,x,C,E,S,F,z,D,N,L,R,I,B,V,$,K,q,J,t=this,W=new Date;function U(t){"mousedown"===t.type&&t.preventDefault(),!H(t,this)||p&&!u||M.disabled||M.readOnly||(c.stopProp&&t.stopPropagation(),F=w=!(p=!0),N=o(t,"X"),q=o(t,"Y"),b=N,j.removeClass("mbsc-progress-anim"),L=g?e(".mbsc-slider-handle",this):h,S&&S.removeClass("mbsc-handle-curr"),S=L.parent().addClass("mbsc-active mbsc-handle-curr"),a.addClass("mbsc-active"),i=+L.attr("data-index"),B=j[0].offsetWidth,z=j[0].getBoundingClientRect().left,"mousedown"===t.type&&(v=!0,e(document).on("mousemove",y).on("mouseup",k)),"mouseenter"===t.type&&(u=!0,e(document).on("mousemove",y)))}function y(e){p&&(b=o(e,"X"),J=o(e,"Y"),$=b-N,K=J-q,5<Math.abs($)&&(w=!0),(w||v||u)&&50<Math.abs(W-new Date)&&(W=new Date,X(b,c.round,V&&(!u||v))),w?e.preventDefault():7<Math.abs(K)&&"touchmove"==e.type&&T())}function k(e){p&&(e.preventDefault(),g||j.addClass("mbsc-progress-anim"),u&&!v?l(n[i],i,!1,!1,!0):X(b,!0,!0),w||F||("touchend"==e.type&&P(),t._onTap(n[i])),"mouseup"==e.type&&(v=!1),"mouseleave"==e.type&&(u=!1),u||T())}function Y(){p&&T()}function G(){var o=t._readValue(e(this)),s=+e(this).attr("data-index");o!==n[s]&&(n[s]=o,l(E[s]=o,s))}function ee(e){e.stopPropagation()}function Q(e){e.preventDefault()}function Z(s){var t;if(!M.disabled){switch(s.keyCode){case 38:case 39:t=1;break;case 40:case 37:t=-1}t&&(s.preventDefault(),x||(i=+e(this).attr("data-index"),l(n[i]+d*t,i,!0),x=setInterval(function(){l(n[i]+d*t,i,!0)},200)))}}function A(e){e.preventDefault(),clearInterval(x),x=null}function T(){p=!1,S.removeClass("mbsc-active"),a.removeClass("mbsc-active"),e(document).off("mousemove",y).off("mouseup",k)}function X(n,s,o){var e=s?Math.min(Math[t._rounding||"round"](Math.max(100*(n-z)/B,0)/D/d)*d*100/(m-r+O),100):Math.max(0,Math.min(100*(n-z)/B,100));C&&(e=100-e),l(Math.round((r-O+e/D)*R)/R,i,o,e)}function l(e,s,i,a,l,c){var d=h.eq(s),o=d.parent();e=Math.min(m,Math.max(e,r)),void 0===c&&(c=i),t._update?e=t._update(e,n,s,a,g,l,o):o.css({left:C?"auto":(a||_(e,r,m))+"%",right:C?(a||_(e,r,m))+"%":"auto"}),r<e?o.removeClass("mbsc-slider-start"):(n[s]>r||l)&&o.addClass("mbsc-slider-start"),i&&(n[s]=e),i&&E[s]!=e&&(F=!0,E[s]=e,t._fillValue(e,s,c)),d.attr("aria-valuenow",e)}ty.call(this,M,et,!0),t._onTap=s,t.___init=s,t.___destroy=s,t._attachChange=function(){a.on(c.changeEvent,G)},t.__init=function(){var s;h&&(s=!0,h.parent().remove()),t.___init(),f=t._$parent,j=t._$track,a=f.find("input"),c=t.settings,r=t._min,m=t._max,O=t._base||0,d=t._step,V=t._live,R=d%1!=0?100/(100*(d%1).toFixed(2)):1,D=100/(m-r+O)||100,g=1<a.length,C=c.rtl,n=[],E=[],a.each(function(s){n[s]=t._readValue(e(this)),e(this).attr("data-index",s)}),h=f.find(".mbsc-slider-handle"),I=f.find(g?".mbsc-slider-handle-cont":".mbsc-progress-cont"),h.on("keydown",Z).on("keyup",A).on("blur",A),I.on("touchstart mousedown"+(c.hover?" mouseenter":""),U).on("touchmove",y).on("touchend touchcancel"+(c.hover?" mouseleave":""),k).on("pointercancel",Y),s||(a.on("click",ee),f.on("click",Q))},t.__destroy=function(){f.off("click",Q),a.off(c.changeEvent,G).off("click",ee),h.off("keydown",Z).off("keyup",A).off("blur",A),I.off("touchstart mousedown mouseenter",U).off("touchmove",y).off("touchend touchcancel mouseleave",k).off("pointercancel",Y),t.___destroy()},t.refresh=function(){a.each(function(n){l(t._readValue(e(this)),n,!0,!1,!0,!1)})},t.getVal=function(){return g?n.slice(0):n[0]},t.setVal=t._setVal=function(t,o,s){e.isArray(t)||(t=[t]),e.each(t,function(e,t){n[e]=t}),e.each(t,function(e,t){l(t,e,!0,!1,!0,s)})},en||t.init()}function ey(o,r){var s,i,a,c,t=this;n(r=r||{},{changeEvent:"click",round:!1}),tj.call(this,o,r,!0),t._readValue=function(){return o.checked?1:0},t._fillValue=function(e,n,t){s.prop("checked",!!e),t&&s.trigger("change")},t._onTap=function(e){t._setVal(e?0:1)},t.___init=function(){i=t.settings,s=e(o),(c=s.parent()).find(".mbsc-switch-track").remove(),c.prepend(s),s.attr("data-role","switch").after('<span class="mbsc-progress-cont mbsc-switch-track"><span class="mbsc-progress-track mbsc-progress-anim"><span class="mbsc-slider-handle-cont"><span class="mbsc-slider-handle mbsc-switch-handle" data-index="0"><span class="mbsc-switch-txt-off">'+i.offText+'</span><span class="mbsc-switch-txt-on">'+i.onText+"</span></span></span></span></span>"),a&&a.destroy(),a=new X(o,i),t._$track=c.find(".mbsc-progress-track"),t._min=0,t._max=1,t._step=1},t.___destroy=function(){a.destroy()},t.getVal=function(){return o.checked},t.setVal=function(e,n,s){t._setVal(e?1:0,n,s)},t.init()}ey.prototype={_class:"switch",_css:"mbsc-switch",_hasTheme:!0,_hasLang:!0,_hasDef:!0,_defaults:{stopProp:!0,offText:"Off",onText:"On"}},p("[mbsc-switch]",u.Switch=ey);function eh(p,a,_){var n,s,o,i,c,l,d,u,h,m,f,v,b,j,t=this;function y(){var e=r("value",o);e!==l&&g(e)}function r(t,n,o){var e=s.attr(t);return void 0===e||""===e?n:o?e:+e}function g(e,a,n,i){e=Math.min(c,Math.max(e,o)),j.css("width",100*(e-o)/(c-o)+"%"),void 0===n&&(n=!0),void 0===i&&(i=n),e===l&&!a||t._display(e),e!==l&&(l=e,n&&s.attr("value",l),i&&s.trigger("change"))}ty.call(this,p,a,!0),t._display=function(e){h=f&&n.returnAffix?f.replace(/\{value\}/,e).replace(/\{max\}/,c):e,u&&u.html(h),d&&d.html(h)},t._attachChange=function(){s.on("change",y)},t.__init=function(){var h,g,y,_;if(n=t.settings,s=e(p),_=!!i,i=t._$parent,o=t._min=void 0===a.min?r("min",n.min):a.min,c=t._max=void 0===a.max?r("max",n.max):a.max,b=void 0===a.inputStyle?r("data-input-style",n.inputStyle,!0):a.inputStyle,v=void 0===a.labelStyle?r("data-label-style",n.labelStyle,!0):a.labelStyle,l=r("value",o),g=s.attr("data-val")||n.val,h=(h=s.attr("data-step-labels"))?JSON.parse(h):n.stepLabels,f=s.attr("data-template")||(100!=c||n.template?n.template:"{value}%"),_?(g&&(d.remove(),i.removeClass("mbsc-progress-value-"+("right"==g?"right":"left"))),h&&e(".mbsc-progress-step-label",m).remove()):(tF(i,null,b,v,p),tL(s),i.find(".mbsc-input-wrap").append('<span class="mbsc-progress-cont"><span class="mbsc-progress-track mbsc-progress-anim"><span class="mbsc-progress-bar"></span></span></span>'),j=t._$progress=i.find(".mbsc-progress-bar"),m=t._$track=i.find(".mbsc-progress-track")),s.attr("min",o).attr("max",c),g&&(d=e('<span class="mbsc-progress-value"></span>'),i.addClass("mbsc-progress-value-"+("right"==g?"right":"left")).find(".mbsc-input-wrap").append(d)),h)for(y=0;y<h.length;++y)m.append('<span class="mbsc-progress-step-label" style="'+(n.rtl?"right":"left")+": "+100*(h[y]-o)/(c-o)+'%" >'+h[y]+"</span>");u=e(s.attr("data-target")||n.target)},t.__destroy=function(){i.removeClass("mbsc-ic-left mbsc-ic-right").find(".mbsc-progress-cont").remove(),i.find(".mbsc-input-ic").remove(),s.off("change",y)},t.refresh=function(){g(r("value",o),!0,!1)},t.getVal=function(){return l},t.setVal=function(e,t,n){g(e,!0,t,n)},_||t.init()}eh.prototype={_class:"progress",_css:"mbsc-progress",_hasTheme:!0,_hasLang:!0,_hasDef:!0,_defaults:{min:0,max:100,returnAffix:!0}},p("[mbsc-progress]",u.Progress=eh);function ei(p,h,w){t=this,eh.call(this,p,h,!0);var t,n,s,o,i,a,r,c,l,d,u,m,f,g,v,b,j=t.__init,y=t.__destroy;tj.call(this,p,h,!0),v=t.__init,b=t.__destroy,t.__init=function(){j(),v()},t.__destroy=function(){y(),b()},t._update=function(e,o,a,r,l,h,m){return f?0===a?(e=Math.min(e,o[1]),c.css({width:_(o[1],s,n)-_(e,s,n)+"%",left:i?"auto":_(e,s,n)+"%",right:i?_(e,s,n)+"%":"auto"})):(e=Math.max(e,o[0]),c.css({width:_(e,s,n)-_(o[0],s,n)+"%"})):l||!u?m.css({left:i?"auto":(r||_(e,s,n))+"%",right:i?(r||_(e,s,n))+"%":"auto"}):c.css("width",(r||_(e,s,n))+"%"),d&&g.eq(a).html(e),l||o[a]==e&&!h||t._display(e),e},t._readValue=function(e){return+e.val()},t._fillValue=function(t,e,n){o.eq(e).val(t),n&&o.eq(e).trigger("change")},t._markupReady=function(){var t,r;if(d&&a.addClass("mbsc-slider-has-tooltip"),1!=l)for(r=(n-s)/l,t=0;t<=r;++t)m.append('<span class="mbsc-slider-step" style="'+(i?"right":"left")+":"+100/r*t+'%"></span>');o.each(function(t){"range"==this.type&&e(this).attr("min",s).attr("max",n).attr("step",l),(u?c:m).append('<span class="mbsc-slider-handle-cont'+(f&&!t?" mbsc-slider-handle-left":"")+'"><span tabindex="0" class="mbsc-slider-handle" aria-valuemin="'+s+'" aria-valuemax="'+n+'" data-index="'+t+'"></span>'+(d?'<span class="mbsc-slider-tooltip"></span>':"")+"</span>")}),g=a.find(".mbsc-slider-tooltip")},t.___init=function(){a&&(a.removeClass("mbsc-slider-has-tooltip"),1!=l&&e(".mbsc-slider-step",m).remove()),a=t._$parent,m=t._$track,c=t._$progress,o=a.find("input"),r=t.settings,s=t._min,n=t._max,t._step=l=void 0===h.step?+o.attr("step")||r.step:h.step,t._live=ti("data-live",r.live,o),d=ti("data-tooltip",r.tooltip,o),u=ti("data-highlight",r.highlight,o)&&o.length<3,f=u&&2==o.length,i=r.rtl,t._markupReady()},w||t.init()}ei.prototype={_class:"progress",_css:"mbsc-progress mbsc-slider",_hasTheme:!0,_hasLang:!0,_hasDef:!0,_defaults:{changeEvent:"change",stopProp:!0,min:0,max:100,step:1,live:!0,highlight:!0,round:!0,returnAffix:!0}},p("[mbsc-slider]",u.Slider=ei);function e9(l,u,h){var n,s,o,i,a,r,c,t=this,d=e(l);ei.call(this,l,u,!0),t._update=function(e,o,i,c,a,r){return n.css("width",_(e,0,s)+"%"),a||o[i]==e&&!r||t._display(e),e},t._markupReady=function(){var e,l="",u="";for(i=t._$track,n=t._$progress,o=t.settings,a=t._min,s=t._max,t._base=a,t._rounding=o.rtl?"floor":"ceil",r=d.attr("data-empty")||o.empty,c=d.attr("data-filled")||o.filled,e=0;e<s;++e)l+='<span class="mbsc-ic mbsc-ic-'+r+'"></span>',u+='<span class="mbsc-ic mbsc-ic-'+c+'"></span>';i.html(l),i.append(n),n.html(u),i.append('<span class="mbsc-rating-handle-cont"><span tabindex="0" class="mbsc-slider-handle" aria-valuemin="'+a+'" aria-valuemax="'+s+'" data-index="0"></span></span>')},h||t.init()}e9.prototype={_class:"progress",_css:"mbsc-progress mbsc-rating",_hasTheme:!0,_hasLang:!0,_hasDef:!0,_defaults:{changeEvent:"change",stopProp:!0,min:1,max:5,step:1,live:!0,round:!0,hover:!0,highlight:!0,returnAffix:!0,empty:"star",filled:"star3"}},p("[mbsc-rating]",u.Rating=e9),t_=1,e7=function(){function t(c,i){var n,s,o,a,l,r=this;E(this,t),n=e(c),(this.settings=i,this._isOpen=i.isOpen||!1,n.addClass("mbsc-collapsible "+(this._isOpen?"mbsc-collapsible-open":"")),(o=(a=n.hasClass("mbsc-card")?(s=n.find(".mbsc-card-header").eq(0).addClass("mbsc-collapsible-header"),n.find(".mbsc-card-content").eq(0).addClass("mbsc-collapsible-content")):n.hasClass("mbsc-form-group")||n.hasClass("mbsc-form-group-inset")?(s=n.find(".mbsc-form-group-title").eq(0).addClass("mbsc-collapsible-header"),n.find(".mbsc-form-group-content").eq(0).addClass("mbsc-collapsible-content")):(s=n.find(".mbsc-collapsible-header").eq(0),n.find(".mbsc-collapsible-content").eq(0)))[0])&&!o.id&&(o.id="mbsc-collapsible-"+t_++),s.length&&o)&&(l=e('<span class="mbsc-collapsible-icon mbsc-ic mbsc-ic-arrow-down5"></span>'),eI(this,s,function(){r.collapse()}),s.attr("role","button").attr("aria-expanded",this._isOpen).attr("aria-controls",o.id).attr("tabindex","0").on("mousedown",function(e){e.preventDefault()}).on("keydown",function(e){32!==e.which&&13!=e.keyCode||(e.preventDefault(),r.collapse())}).append(l)),(c.mbscInst=this)._$header=s,this._$content=a,this._$elm=n,this._$accordionParent=n.parent("[mbsc-accordion], mbsc-accordion, .mbsc-accordion"),this.show=this.show.bind(this),this.hide=this.hide.bind(this),this.toggle=this.toggle.bind(this)}return Y(t,[{key:"collapse",value:function(e){var n=this._$elm,t=this._$content;void 0===e&&(e=!this._isOpen),e&&this._isOpen||!e&&!this._isOpen||!t.length||(e?(ts&&t.on("transitionend",function e(){t.off("transitionend",e).css("height","")}).css("height",t[0].scrollHeight),n.addClass("mbsc-collapsible-open")):(ts&&t.css("height",getComputedStyle(t[0]).height),setTimeout(function(){t.css("height",0),n.removeClass("mbsc-collapsible-open")},50)),e&&this._$accordionParent&&this._$accordionParent.find(".mbsc-collapsible-open").each(function(){this!==n[0]&&this.mbscInst.hide()}),this._isOpen=e,this._$header.attr("aria-expanded",this._isOpen))}},{key:"show",value:function(){this.collapse(!0)}},{key:"hide",value:function(){this.collapse(!1)}},{key:"toggle",value:function(){this.collapse()}},{key:"destroy",value:function(){this._$elm.removeClass("mbsc-collapsible mbsc-collapsible-open"),this._$content.removeClass("mbsc-collapsible-content"),this._$header.removeClass("mbsc-collapsible-header").find(".mbsc-collapsible-icon").remove()}}]),t}(),u.CollapsibleBase=e7,tf=0;function tx(s,n,t,o){e("input,select,textarea,progress,button",s).each(function(){var s=this,o=e(s),i=ej(o);if("false"!=o.attr("data-enhance"))if(o.hasClass("mbsc-control"))s.mbscInst&&s.mbscInst.option({theme:t.theme,lang:t.lang,rtl:t.rtl,onText:t.onText,offText:t.offText,stopProp:t.stopProp});else switch(s.id||(s.id="mbsc-form-control-"+ ++tf),i){case"button":case"submit":n[s.id]=new t0(s,{theme:t.theme,rtl:t.rtl,tap:t.tap});break;case"switch":n[s.id]=new ey(s,{theme:t.theme,lang:t.lang,rtl:t.rtl,tap:t.tap,onText:t.onText,offText:t.offText,stopProp:t.stopProp});break;case"checkbox":n[s.id]=new t2(s,{tap:t.tap,theme:t.theme,rtl:t.rtl});break;case"range":e(s).parent().hasClass("mbsc-slider")||(n[s.id]=new ei(s,{theme:t.theme,lang:t.lang,rtl:t.rtl,stopProp:t.stopProp,labelStyle:t.labelStyle}));break;case"rating":n[s.id]=new e9(s,{theme:t.theme,lang:t.lang,rtl:t.rtl,stopProp:t.stopProp});break;case"progress":n[s.id]=new eh(s,{theme:t.theme,lang:t.lang,rtl:t.rtl,labelStyle:t.labelStyle});break;case"radio":n[s.id]=new t3(s,{tap:t.tap,theme:t.theme,rtl:t.rtl});break;case"select":case"select-one":case"select-multiple":n[s.id]=new th(s,{tap:t.tap,inputStyle:t.inputStyle,labelStyle:t.labelStyle,theme:t.theme,rtl:t.rtl});break;case"textarea":n[s.id]=new tp(s,{tap:t.tap,inputStyle:t.inputStyle,labelStyle:t.labelStyle,theme:t.theme,rtl:t.rtl});break;case"segmented":n[s.id]=new tg(s,{theme:t.theme,rtl:t.rtl,tap:t.tap,inputStyle:t.inputStyle});break;case"stepper":n[s.id]=new eb(s,{theme:t.theme,rtl:t.rtl});break;case"hidden":return;default:n[s.id]=new e0(s,{tap:t.tap,inputStyle:t.inputStyle,labelStyle:t.labelStyle,theme:t.theme,rtl:t.rtl})}}),e("[data-collapsible]:not(.mbsc-collapsible)",s).each(function(){var t=this,s=e(t).attr("data-open");t.id||(t.id="mbsc-form-control-"+ ++tf),n[t.id]=new e7(t,{isOpen:void 0!==s&&"false"!=s}),A[t.id]=n[t.id]}),o||tO()}function tm(a,l){var t,o,i="",s=e(a),r={},n=this;function c(){s.removeClass("mbsc-no-touch")}S.call(this,a,l,!0),n.refresh=function(e){tx(s,r,o,e)},n._init=function(){var e,r=void 0!==o.collapsible||void 0!==s.attr("data-collapsible");(s.hasClass("mbsc-card")||s.on("touchstart",c).show(),i&&s.removeClass(i),i="mbsc-card mbsc-form mbsc-no-touch mbsc-"+o.theme+(tE?" mbsc-form-hb":"")+(o.baseTheme?" mbsc-"+o.baseTheme:"")+(o.rtl?" mbsc-rtl":" mbsc-ltr"),s.addClass(i).removeClass("mbsc-cloak"),r&&!t)&&(e=s.attr("data-open"),t=new e7(a,{isOpen:void 0!==e&&"false"!=e||!0===o.collapsible})),n.refresh()},n._destroy=function(){for(var e in s.removeClass(i).off("touchstart",c),r)r[e].destroy();t&&t.destroy()},n.toggle=function(){t&&t.toggle()},n.hide=function(){t&&t.hide()},n.show=function(){t&&t.show()},o=n.settings,n.init()}tE="ios"==h&&7<x;function e_(t){var n=[Math.round(t.r).toString(16),Math.round(t.g).toString(16),Math.round(t.b).toString(16)];return e.each(n,function(t,e){1==e.length&&(n[t]="0"+e)}),"#"+n.join("")}function eO(e){return{r:(e=parseInt(-1<e.indexOf("#")?e.substring(1):e,16))>>16,g:(65280&e)>>8,b:255&e,toString:function(){return"rgb("+this.r+","+this.g+","+this.b+")"}}}function tS(r){var t,n,s,i,a,o=r.h,c=255*r.s/100,e=255*r.v/100;return 0==c?t=n=s=e:(i=(255-c)*e/255,a=o%60*(e-i)/60,360==o&&(o=0),o<60?(t=e,n=(s=i)+a):o<120?(s=i,t=(n=e)-a):o<180?(n=e,s=(t=i)+a):o<240?(t=i,n=(s=e)-a):o<300?(s=e,t=(n=i)+a):o<360?(n=i,s=(t=e)-a):t=n=s=0),{r:t,g:n,b:s,toString:function(){return"rgb("+this.r+","+this.g+","+this.b+")"}}}function tM(e){var o,i,n=0,a=Math.min(e.r,e.g,e.b),t=Math.max(e.r,e.g,e.b),s=t-a;return n=(o=(i=t)?255*s/t:0)?e.r==t?(e.g-e.b)/s:e.g==t?2+(e.b-e.r)/s:4+(e.r-e.g)/s:-1,(n*=60)<0&&(n+=360),{h:n,s:o*=100/255,v:i*=100/255,toString:function(){return"hsv("+Math.round(this.h)+","+Math.round(this.s)+"%,"+Math.round(this.v)+"%)"}}}function ny(c){var t,n,r,a=c.r/255,s=c.g/255,o=c.b/255,e=Math.max(a,s,o),i=Math.min(a,s,o),l=(e+i)/2;if(e==i)t=r=0;else{switch(n=e-i,r=.5<l?n/(2-e-i):n/(e+i),e){case a:t=(s-o)/n+(s<o?6:0);break;case s:t=(o-a)/n+2;break;case o:t=(a-s)/n+4}t/=6}return{h:Math.round(360*t),s:Math.round(100*r),l:Math.round(100*l),toString:function(){return"hsl("+this.h+","+this.s+"%,"+this.l+"%)"}}}function tT(e){return ny(eO(e))}function nM(e){return e_(function(c){var t,n,s,o,r,l,e=c.h,a=c.s,i=c.l;return isFinite(e)||(e=0),isFinite(a)||(a=0),isFinite(i)||(i=0),(e/=60)<0&&(e=6- -e%6),e%=6,a=Math.max(0,Math.min(1,a/100)),i=Math.max(0,Math.min(1,i/100)),n=(t=(1-Math.abs(2*i-1))*a)*(1-Math.abs(e%2-1)),l=e<1?(o=t,s=n,0):e<2?(o=n,s=t,0):e<3?(o=0,s=t,n):e<4?(o=0,s=n,t):e<5?(o=n,s=0,t):(o=t,s=0,n),r=i-t/2,{r:Math.round(255*(o+r)),g:Math.round(255*(s+r)),b:Math.round(255*(l+r)),toString:function(){return"rgb("+this.r+","+this.g+","+this.b+")"}}}(e))}function tD(e){return e_(tS(e))}function tN(e){return tM(eO(e))}tm.prototype={_hasDef:!0,_hasTheme:!0,_hasLang:!0,_class:"card",_defaults:{tap:et,stopProp:!0,rtl:!1}},p("[mbsc-card]",u.Card=tm,!0),r("card",tm,!1);function ta(C,K,q){var s,o,i,c,l,d,h,m,f,p,g,b,j,y,_,w,x,E,S,D,L,I,B,V,U,t=this,v=e(C),r=0,u={},a={};function N(i,o,r){if(!r){t._value=t._hasValue?t._tempValue.slice(0):null;for(var e=0;e<s.length;++e)s[e].tempChangedColor&&t._value&&-1!=t._value.indexOf(s[e].tempChangedColor)&&(s[e].changedColor=s[e].tempChangedColor),delete s[e].tempChangedColor}i&&(t._isInput&&v.val(t._hasValue?t._tempValue:""),w("onFill",{valueText:t._hasValue?t._tempValue:"",change:o}),o&&(u=n(!0,{},a),t._preventChange=!0,v.trigger("change")),$(t._value,!0))}function P(t,e){return'<div class="mbsc-color-input-item" data-color="'+(void 0!==(e=void 0!==e?e:z(t))?e:t)+'" style="background: '+t+';">'+(l?"":'<div class="mbsc-color-input-item-close mbsc-ic mbsc-ic-material-close"></div>')+"</div>"}function H(e){L[0].style.background=e?k+"linear-gradient(left, "+(i.rtl?"#000000":"#FFFFFF")+" 0%, "+e+" 50%, "+(i.rtl?"#FFFFFF":"#000000")+" 100%)":""}function z(e){if(Object.keys(a).length&&!isNaN(e))return e;for(var t in s)if(e==s[t].color||e==s[t].changedColor)return t}function A(t,s){var n,e=t.match(/\d+/gim);switch(!0){case-1<t.indexOf("rgb"):n=e_({r:e[0],g:e[1],b:e[2]});break;case-1<t.indexOf("hsl"):n=nM({h:e[0],s:e[1],l:e[2]});break;case-1<t.indexOf("hsv"):n=tD({h:e[0],s:e[1],v:e[2]});break;case-1<t.indexOf("#"):n=t}return function(e,t){switch(t){case"rgb":return eO(e);case"hsl":return tT(e);case"hsv":return tN(e);default:return e}}(n,s||i.format)}function M(t,n){e(".mbsc-color-active",n).removeClass("mbsc-color-active"),m&&(t.parent().addClass("mbsc-color-active"),c&&t&&void 0!==r&&d.eq(r).parent().addClass("mbsc-color-active"))}function $(v,C){var h,b,_,x,f=[],O=0,w=e.map(s,function(e){return e.changedColor||e.color});if(l)v=e.isArray(v)?v[0]:v,-1<(_=w.indexOf(v))&&f.push(_),v&&!f.length&&m&&(b=+e(".mbsc-color-input-item",g).attr("data-color"),isNaN(b)?b=void 0:f.push(b),o=b);else if(v)if(c&&m)for(x in u)void 0!==u[x].colorIndex&&f.push(+u[x].colorIndex);else for(h=0;h<v.length;++h)-1<(_=w.indexOf(v[h]))&&(f.push(_),w[_]="temp"+h);for(h=0;h<f.length;++h)s[f[h]]&&F(!0,f[h],O++,s[f[h]].changedColor||s[f[h]].color,!0);for(h=0;h<s.length;++h)-1==f.indexOf(h)&&F(!1,h,void 0,s[h].changedColor||s[h].color,!1);if(c)for(h=O;h<i.select;++h)a[h]={},d&&d.eq(h).addClass("mbsc-color-preview-item-empty").css({background:"transparent"});u=n(!0,{},a),!1!==C&&function(){if(S){var n,h="";if(g.empty(),t._hasValue){if(l)h+=P(t._value,o);else for(n=0;n<t._value.length;++n)h+=P(t._value[n],Object.keys(a).length&&a[n].colorIndex?a[n].colorIndex:z(t._value[n]));g.append(h),t.tap(e(".mbsc-color-input-item",g),function(n){if(e(n.target).hasClass("mbsc-color-input-item-close")){var l=e(this).index();n.stopPropagation(),n.preventDefault(),void 0===o&&(o=e(n.target).parent().attr("data-color")),c&&s[o]&&(r=s[o].previewInd,d.eq(r).parent().removeClass("mbsc-color-active"),u[l]={},a[l]={}),t._value.splice(l,1),t.setVal(t._value,!0,!0)}else m&&"inline"!==i.display&&(o=e(n.target).attr("data-color"),isNaN(o)&&(o=z(o)),o&&s[o]&&(s[o].selected=!0,r=s[o].previewInd,setTimeout(function(){j.scroll(p.eq(o),400),c&&y.scroll(d.eq(r),400)},200)))})}}}()}function F(e,n,o,i,l,u){if(c&&l&&(a[o].colorIndex=e?n:void 0,a[o].color=e?i:void 0,d)){var r=d.eq(o);r.removeClass("mbsc-color-preview-item-empty").css({background:e?i:"transparent"}),e||r.addClass("mbsc-color-preview-item-empty").parent().removeClass("mbsc-color-active")}u&&(e?t._tempValue.splice(o,0,i):t._tempValue.splice(t._tempValue.indexOf(i),1)),p&&(e?p.eq(n).addClass("mbsc-color-selected"):p.eq(n).removeClass("mbsc-color-selected").parent().removeClass("mbsc-color-active")),s[n].previewInd=e?o:void 0,s[n].selected=e}function T(e,t){void 0!==e&&(l||s[e]&&s[e].selected)?s[o=e]&&(b=s[e].changedColor||s[e].color,h=p.eq(e),m&&(M(p.eq(e),t||""),(x=A(s[e].color,"hsl")).l=A(b,"hsl").l,H(s[e].color),E.setVal(100-x.l,!1,!1))):m&&H()}function W(n,s){var t=e(n.target).index();o=a[t].colorIndex,h=p.eq(o),r=t,T(o,s),j.scroll(h,250),w("onPreviewItemTap",{target:n.target,value:a[t].color,index:t})}function R(u,n){var f=!1,p=e(".mbsc-color-selected",n);if((h=e(u.target)).hasClass("mbsc-color-clear-item"))return b="",void t.clear();(l||B>+p.length||h.hasClass("mbsc-color-selected"))&&(o=h.attr("data-index"),c&&(r=void 0!==s[o].previewInd?s[o].previewInd:function(){var e;for(e=0;e<i.select;++e)if(void 0===a[e].colorIndex)return e}(),f=m&&h.hasClass("mbsc-color-selected")&&!h.parent().hasClass("mbsc-color-active"),6<d.length&&y.scroll(d.eq(r))),b=s[o].changedColor||s[o].color,l?(p.removeClass("mbsc-color-selected"),(t._tempValue=b)&&h.toggleClass("mbsc-color-selected"),M(h,n)):(M(h,n),f||F(!s[o].selected,o,r,b,!0,!0)),T(o,n),t.live&&(t._fillValue(),w("onSet",{value:t._value})),w("onItemTap",{target:u.target,value:b,selected:s[o].selected,index:o}),t._updateHeader())}O.call(this,C,K,!0),t.setVal=t._setVal=function(n,s,o,i){t._hasValue=null!=n,t._tempValue=l?e.isArray(n)?n[0]:n:e.isArray(n)?n:n?[n]:[],N(s,void 0===o?s:o,i)},t.getVal=t._getVal=function(e){return t._hasValue||e?U?function(){var e,t=[];for(e=0;e<s.length;++e)s[e].selected&&t.push(s[e]);return t}():t[e?"_tempValue":"_value"]:null},t._readValue=function(){var e=v.val()||"";t._hasValue=!1,0!==e.length&&""!==e&&(t._hasValue=!0),t._hasValue?(t._tempValue=l?e:"hex"==i.format?e.split(","):e.match(/[a-z]{3}\((\d+\.?\d{0,}?),\s*([\d.]+)%{0,},\s*([\d.]+)%{0,}\)/gim),N(!0)):t._tempValue=[],$(t._tempValue,t._hasValue)},t._fillValue=function(){N(t._hasValue=!0,!0)},t._generateContent=function(){var e,t,n,a,o=_?1:0;for(I=f?Math.ceil((s.length+o)/i.rows):i.rows,t='<div class="mbsc-color-scroll-cont mbsc-w-p '+(f?"":"mbsc-color-vertical")+'"><div class="mbsc-color-cont">'+(f?'<div class="mbsc-color-row">':""),e=0;e<s.length;++e)n=s[e].changedColor||s[e].color,_&&0===e&&(t+='<div class="mbsc-color-item-c"><div tabindex="0" class="mbsc-color-clear-item mbsc-btn-e mbsc-color-selected"><div class="mbsc-color-clear-cross"></div></div></div>'),0!==e&&(e+o)%I==0&&(t+=f?'</div><div class="mbsc-color-row">':""),t+='<div class="mbsc-color-item-c"><div tabindex="0" data-index="'+e+'" class="mbsc-color-item mbsc-btn-e mbsc-ic mbsc-ic-material-check mbsc-color-btn-e '+(s[e].selected?"mbsc-color-selected":"")+'"  style="background:'+n+'"></div></div>';if(t+="</div></div>"+(f?"</div>":""),m&&(t+='<div class="mbsc-color-slider-cont"><input class="mbsc-color-slider" type="range" data-highlight="false" value="50" min="0" max="100"/></div>'),c){for(a in t+='<div class="mbsc-color-preview-cont"><div class="mbsc-color-refine-preview">',u)t+='<div class="mbsc-color-preview-item-c mbsc-btn-e mbsc-color-btn-e" tabindex="0"><div class="mbsc-color-preview-item '+(u[a].color?"":"mbsc-color-preview-item-empty")+'" style="background: '+(u[a].color||"initial")+';"></div></div>';t+="</div></div>"}return t},t._position=function(n){var e,t;f||(e=n.find(".mbsc-color-cont"),t=Math.ceil(e.find(".mbsc-color-item-c")[0].offsetWidth),e.width(Math.min(Math.floor(n.find(".mbsc-fr-c").width()/t),Math.round(s.length/i.rows))*t+1)),j&&j.refresh(),y&&y.refresh()},t._markupInserted=function(e){f||e.find(".mbsc-color-scroll-cont").css("max-height",e.find(".mbsc-color-item-c")[0].offsetHeight*i.rows),j=new J(e.find(".mbsc-color-scroll-cont")[0],{axis:f?"X":"Y",rtl:i.rtl,elastic:60,stopProp:!1,mousewheel:i.mousewheel,onBtnTap:function(t){R(t,e)}})},t._attachEvents=function(n){var u;p=e(".mbsc-color-item",n),n.on("keydown",".mbsc-color-btn-e",function(e){e.stopPropagation(),32==e.keyCode&&(e.target.classList.contains("mbsc-color-item")?R(e,n):W(e,n))}),c&&(d=e(".mbsc-color-preview-item",n)),m&&(n.addClass("mbsc-color-refine"),D=e(".mbsc-color-slider",n),E=new ei(D[0],{theme:i.theme,rtl:i.rtl}),L=n.find(".mbsc-progress-track"),o&&t._value&&T(o,n),D.on("change",function(){void 0!==o&&(l||s[o]&&s[o].selected)&&(x.l=100-this.value,u=A(x.toString()).toString(),l?t._tempValue=u:t._tempValue[void 0!==r?r:t._tempValue.length]=u,s[o].tempChangedColor=u,p.eq(o).css("background",u),c&&(a[r].color=u,d.eq(r).removeClass("mbsc-color-preview-item-empty").css({background:u})),t.live&&eq(t._fillValue()))})),c&&(y=new J(n.find(".mbsc-color-preview-cont")[0],{axis:"X",rtl:i.rtl,stopProp:!1,mousewheel:i.mousewheel,onBtnTap:function(e){W(e,n)}})),t._updateHeader()},t._markupRemove=function(){j&&j.destroy(),E&&E.destroy(),y&&y.destroy()},t.__processSettings=function(){var n,o;if(i=t.settings,w=t.trigger,f="horizontal"==i.navigation,t._value=[],t._tempValue=[],l="single"==i.select,_=void 0!==i.clear?i.clear:l,!(o=i.data||[]).length)switch(i.format){case"rgb":o=["rgb(255,235,60)","rgb(255,153,0)","rgb(244,68,55)","rgb(234,30,99)","rgb(156,38,176)","rgb(104,58,183)","rgb(63,81,181)","rgb(33,150,243)","rgb(0,151,136)","rgb(75,175,79)","rgb(126,93,78)","rgb(158,158,158)"],_&&o.splice(10,0,"rgb(83, 71, 65)");break;case"hsl":o=["hsl(54,100%,62%)","hsl(36,100%,50%)","hsl(4,90%,59%)","hsl(340,83%,52%)","hsl(291,64%,42%)","hsl(262,52%,47%)","hsl(231,48%,48%)","hsl(207,90%,54%)","hsl(174,100%,30%)","hsl(122,40%,49%)","hsl(19,24%,40%)","hsl(0,0%,62%)"],_&&o.splice(10,0,"hsl(20, 12%, 29%)");break;default:o=["#ffeb3c","#ff9900","#f44437","#ea1e63","#9c26b0","#683ab7","#3f51b5","#2196f3","#009788","#4baf4f","#7e5d4e","#9e9e9e"],_&&o.splice(10,0,"#534741")}if(m="refine"==i.mode,c=!isNaN(i.select),B=isNaN(i.select)?l?2:o.length:i.select,U=e.isPlainObject(o[0]),c&&!Object.keys(u).length)for(n=0;n<i.select;++n)u[n]={},a[n]={};for(s=o.slice(0),n=0;n<s.length;++n)e.isPlainObject(o[n])?s[n].color=o[n].color:(o[n]=o[n].toLowerCase(),s[n]={key:n,name:o[n],color:o[n]});V=i.defaultValue||s[0].color,x=A(b=V,"hsl"),(S=i.enhance&&v.is("input"))&&(v.hasClass("mbsc-color-input-hdn")?g=v.prev():((g=e("<div "+(C.placeholder?'data-placeholder="'+C.placeholder+'"':"")+' class="mbsc-control mbsc-color-input '+(i.inputClass||"")+'" readonly ></div>')).insertBefore(v),v.addClass("mbsc-color-input-hdn").attr("tabindex",-1)),i.anchor=g,t.attachShow(g))},t.__destroy=function(){S&&(v.removeClass("mbsc-color-input-hdn"),g.remove())},t._checkSize=!0,q||t.init()}ta.prototype={_hasDef:!0,_hasTheme:!0,_hasLang:!0,_class:"color",_defaults:n({},O.prototype._defaults,{headerText:!1,validate:s,parseValue:s,enhance:!0,rows:2,select:"single",format:"hex",navigation:"horizontal",compClass:"mbsc-color"})},u.Color=ta,t.themes.color=t.themes.frame,r("color",ta,!(eg.color={hsv2hex:tD,hsv2rgb:tS,rgb2hsv:tM,rgb2hex:e_,rgb2hsl:ny,hex2rgb:eO,hex2hsv:tN,hex2hsl:tT})),c.date=eF,c.time=eF,c.datetime=eF,r("date",d),r("time",d),r("datetime",d),w=function(a,c,r){function i(t){e(".mbsc-fr-c",t).hasClass("mbsc-wdg-c")||(e(".mbsc-fr-c",t).addClass("mbsc-wdg-c").append(s.show()),e(".mbsc-w-p",t).length||e(".mbsc-fr-c",t).addClass("mbsc-w-p"))}var t,o,s=e(a),n=this;O.call(this,a,c,!0),n._generateContent=function(){return""},n._markupReady=function(e){"inline"!=t.display&&i(e)},n._markupInserted=function(e){"inline"==t.display&&i(e),e.trigger("mbsc-enhance",[{theme:t.theme,lang:t.lang}])},n._markupRemove=function(){s.hide(),o&&o.parent().length&&o.after(s)},n.__processSettings=function(){t=n.settings,n.buttons.ok={text:t.okText,icon:t.okIcon,handler:"set"},t.buttons=t.buttons||("inline"==t.display?[]:["ok"]),!o&&s.parent().length&&(o=e(document.createComment("popup")),s.before(o)),s.hide()},r||n.init()},w.prototype={_hasDef:!0,_hasTheme:!0,_hasContent:!0,_hasLang:!0,_responsive:!0,_class:"popup",_defaults:n({},O.prototype._defaults,{compClass:"mbsc-wdg",okText:"OK",headerText:!1})},u.Popup=w,tP=u.Widget=w,t.themes.popup=t.themes.frame,tH=0;function to(e,t,n){"jsonp"==n?function(n,s){var e=document.createElement("script"),t="mbscjsonp"+ ++tH;window[t]=function(n){e.parentNode.removeChild(e),delete window[t],n&&s(n)},e.src=n+(0<=n.indexOf("?")?"&":"?")+"callback="+t,document.body.appendChild(e)}(e,t):function(t,n){var e=new XMLHttpRequest;e.open("GET",t,!0),e.onload=function(){200<=this.status&&this.status<400&&n(JSON.parse(this.response))},e.onerror=function(){},e.send()}(e,t)}eg.getJson=to,tB={view:{calendar:{type:"month",popover:!0}},allDayText:"All-day",labelsShort:["Yrs","Mths","Days","Hrs","Mins","Secs"],eventText:"event",eventsText:"events",noEventsText:"No events"},tV={yearChange:!1,weekDays:"short"},c.eventcalendar=function(s,er){function X(w,F,_){var o,i,u,h,S=0,k=[],d="",v=[];for(_=_||s._prepareObj(c,w,F),i=M(w);i<=F;i.setDate(i.getDate()+1))(o=_[M(i)])&&o.length&&v.push({d:new Date(i),list:Q(o)});if(0<v.length)for(u=0;u<v.length;u++){for(d+='<div><div class="mbsc-lv-gr-title mbsc-event-day" data-full="'+e2((o=v[u]).d)+'">'+l(t.dateFormat,o.d,t)+"</div>",h=0;h<o.list.length;h++){var n=o.list[h],f=n.start?a(n.start):null,r=n.end?a(n.end):null,O=n.color,E=C.test(n.d)||B.test(n.d),p=f&&r&&!b(f,r),A=!p||b(f,o.d),j=!p||b(r,o.d),y=n.d?E?n.d:a(n.d):f,T=n.allDay||E||p&&!A&&!j;k.push({d:o.d,e:n}),d+='<div class="mbsc-lv-item mbsc-lv-item-actionable" data-index="'+S+'"><div class="mbsc-event-time">'+(T?t.allDayText:A&&y&&y.getTime?l(t.timeFormat,y):p&&j?t.toText:"")+(!T&&j&&r&&r.getTime?"<br/>"+l(t.timeFormat,r):"")+'</div><div class="mbsc-event-color"'+(O?' style="background:'+O+';"':"")+'></div><div class="mbsc-event-txt">'+n.text+"</div></div>",S++}d+="</div>"}else d+='<div class="mbsc-lv-gr-title mbsc-event-empty"><div class="mbsc-empty"><h3>'+t.noEventsText+"</h3></div></div>";x++,g.html('<div class="mbsc-lv mbsc-lv-v">'+d+"</div>").scrollTop(0),setTimeout(function(){x--},150),s.tap(e(".mbsc-lv-item",g),function(n){var t=k[e(this).attr("data-index")];m("onEventSelect",{domEvent:n,event:t.e,date:t.d})})}function en(){if(f){var n=M(f.d);!function(n,r,d){if(n){var i,c='<div class="mbsc-cal-event-list">',o=e('<div class="mbsc-cal-events '+(t.eventBubbleClass||"")+'"><div class="mbsc-cal-events-i"><div class="mbsc-cal-events-sc"></div><div class="mbsc-sc-bar-c"><div class="mbsc-sc-bar"></div></div></div></div>'),u=e(".mbsc-cal-events-i",o),f=e(".mbsc-cal-events-sc",o);s.tap(u,function(){i.scrolled||y()}),h=new w(o[0],{display:"bubble",theme:t.theme,lang:t.lang,context:t.context,buttons:[],anchor:d,showOverlay:!1,cssClass:"mbsc-no-padding mbsc-cal-events-popup",onShow:function(){i=new J(u[0],{scrollbar:e(".mbsc-sc-bar-c",o),stopProp:!1})},onClose:function(t,e){e.destroy(),i.destroy()}}),A=d,n=Q(n),e.each(n,function(y,n){var o=n.start?a(n.start):null,s=n.end?a(n.end):null,p=C.test(n.d)||B.test(n.d),d=n.d?p?n.d:a(n.d):o,i=o&&s&&!b(o,s),f=!i||b(o,r),j=!i||b(s,r),v=n.allDay||p||i&&!f&&!j,g=n.color,u="",h="",m=e("<div>"+n.text+"</div>").text();d.getTime&&(u=l((i?"MM d yy ":"")+t.timeFormat,d)),s&&(h=l((i?"MM d yy ":"")+t.timeFormat,s)),c+='<div role="button" title="'+m+'" aria-label="'+m+(u?", "+t.fromText+": "+u:"")+(h?", "+t.toText+": "+h:"")+'" class="mbsc-cal-event mbsc-lv-item mbsc-lv-item-actionable"><div class="mbsc-cal-event-color" style="'+(g?"background:"+g+";":"")+'"></div><div class="mbsc-cal-event-text"><div class="mbsc-cal-event-time">'+(v?t.allDayText:f&&d.getTime?l(t.timeFormat,d):"")+"</div>"+n.text+"</div>"+(o&&s&&!n.allDay?'<div class="mbsc-cal-event-dur">'+t.formatDuration(o,s,n)+"</div>":"")+"</div>"}),c+="</div>",f.html(c),h.show(),m("onEventBubbleShow",{target:A,eventList:o[0]}),s.tap(e(".mbsc-cal-event",f),function(t){i.scrolled||m("onEventSelect",{domEvent:t,event:n[e(this).index()],date:r})}),S=!0}}(f.events||_[n],n,f.cell||e('.mbsc-cal-slide-a .mbsc-cal-day[data-full="'+e2(n)+'"]',s._markup)[0]),f=null}}function Q(e){return e.slice(0).sort(function(e,t){var n=e.start?a(e.start):null,s=t.start?a(t.start):null,o=e.end?a(e.end):null,i=t.end?a(t.end):null,u=C.test(e.d)||B.test(e.d),h=C.test(t.d)||B.test(t.d),r=e.d?u?e.d:a(e.d):n,c=t.d?h?t.d:a(t.d):s,l=r.getTime?n&&o&&n.toDateString()!==o.toDateString()?1:e.allDay?2:r.getTime():0,d=c.getTime?s&&i&&s.toDateString()!==i.toDateString()?1:t.allDay?2:c.getTime():0;return l==d?e.text>t.text?1:-1:l-d})}function ei(){var t,n,o;x||e(".mbsc-event-day",this).each(function(){if(0<=(o=this.offsetTop-I.scrollTop)&&o<35)return t=e(this).attr("data-full").split("-"),b(n=T(t[0],t[1]-1,t[2]),p)||(H=!0,s.setVal(n)),!1})}function y(){h&&S&&h.hide(),A=null,S=!1}function eo(t){0==e(t.target).closest(".mbsc-cal-day").length&&y()}function D(){y(),s.redraw()}function es(s){var a,c,e=t.getYear(s),n=t.getMonth(s),l=t.getDay(s),r=s;"day"==i?d=t.getDate(e,n,l+o-1):"week"==i?(c=r.getDay(),a=l+t.firstDay-(0<t.firstDay-c?7:0)-c,r=t.getDate(e,n,a),d=t.getDate(e,n,a+7*o-1)):"month"==i?(r=t.getDate(e,n,1),d=t.getDate(e,n+o,0)):"year"==i&&(r=t.getDate(e,0,1),d=t.getDate(e+o,0,0))}function q(n,s){if(v&&!H){var t=e('.mbsc-event-day[data-full="'+e2(n)+'"]',g);t.length&&(x++,nK(I,t.parent()[0].offsetTop,s,function(){setTimeout(function(){x--},150)}))}}function P(e,t){e&&m("onPageChange",{firstDay:r,lastDay:d}),t||m("onPageLoading",{firstDay:r,lastDay:d}),m("onPageLoaded",{firstDay:r,lastDay:d})}var o,i,r,d,u,h,f,p,g,v,j,_,O,E,k,A,S,F,z,N,L,R,H,I,V,$,W,U,Y,Z,et,ee,ec=this,ea=n({},s.settings),t=n(s.settings,tB,ea,tV,er),x=0,G=0,c=n(!0,[],t.data),K=!0,m=s.trigger;return t.data=c,e.each(c,function(t,e){void 0===e._id&&(e._id=G++)}),V=t.view,u=V.calendar,j=V.eventList,Z=t.months,L=t.weeks,E=u?("week"==u.type?L=u.size||1:u.size&&(Z=u.size),!1):!(L=0),j&&(i=j.type,o=j.size||1),R=u&&u.labels,U=j&&j.scrollable,v=V.eventList,N=void 0===t.eventBubble?u&&u.popover:t.eventBubble,t.weeks=L,t.months=Z,O=eZ.call(this,s),s._onSelectShow=function(){y()},s._onGenMonth=function(e,t){_=s._prepareObj(c,e,t),s._labels=R?_:null},s._onRefresh=function(e){ee=!0,F=z=null,E&&P(!1,e)},s._onSetDate=function(e,t){p=e,E?H||(es(e),P(!0)):t||W||(v&&"day"==i&&X(e,e,_),!N&&!Y||$||en(),q(e)),$=Y=H=!1},s._getDayProps=function(s){var e=_[s],n={events:e};return t.marked||t.labels||R||(e?(n.background=e[0]&&e[0].background,n.marked=e,n.markup=t.showEventCount?'<div class="mbsc-cal-txt">'+e.length+" "+(1<e.length?t.eventsText:t.eventText)+"</div>":'<div class="mbsc-cal-marks"><div class="mbsc-cal-mark"></div></div>'):n.markup=t.showEventCount?'<div class="mbsc-cal-txt-ph"></div>':""),n},s.addEvent=function(t){var s=[];return t=n(!0,[],e.isArray(t)?t:[t]),e.each(t,function(t,e){void 0===e._id&&(e._id=G++),c.push(e),s.push(e._id)}),D(),s},s.updateEvent=function(t){e.each(c,function(e,n){if(n._id===t._id)return c.splice(e,1,t),!1}),D()},s.removeEvent=function(t){t=e.isArray(t)?t:[t],e.each(t,function(n,t){e.each(c,function(e,n){if(n._id===t)return c.splice(e,1),!1})}),D()},s.getEvents=function(e){var t;return e?(e.setHours(0,0,0,0),(t=s._prepareObj(c,e,e))[e]?Q(t[e]):[]):n(!0,[],c)},s.setEvents=function(o){var s=[];return t.data=c=n(!0,[],o),e.each(c,function(t,e){void 0===e._id&&(e._id=G++),s.push(e._id)}),D(),s},s.navigate=function(e,t,n){f=n?{d:e}:null,s.setVal(e,!0,!0,!1,t?200:0)},n({},O,{multiLabel:R,headerText:!1,buttons:"inline"!==t.display?["close"]:t.buttons,compClass:"mbsc-ev-cal mbsc-calendar mbsc-dt mbsc-sc",formatDuration:function(r,c){var e=t.labelsShort,a=c-r,o=Math.abs(a)/1e3,i=o/60,s=i/60,n=s/24,l=n/365;return o<45&&Math.round(o)+" "+e[5].toLowerCase()||i<45&&Math.round(i)+" "+e[4].toLowerCase()||s<24&&Math.round(s)+" "+e[3].toLowerCase()||n<30&&Math.round(n)+" "+e[2].toLowerCase()||n<365&&Math.round(n/30)+" "+e[1].toLowerCase()||Math.round(l)+" "+e[0].toLowerCase()},onMarkupReady:function(n,s){k=e(n.target),p=s.getDate(!0),v&&((g=e('<div class="mbsc-lv-cont mbsc-lv-'+t.theme+(t.baseTheme?" mbsc-lv-"+t.baseTheme:"")+(U?" mbsc-event-list-h":"")+' mbsc-event-list"></div>').appendTo(e(".mbsc-fr-w",k))).on("scroll",eq(ei)),I=g[0]),O.onMarkupReady.call(this,n),et=e(".mbsc-cal-month",k),S=!1,es(p),v&&E&&(P(),eS(e(".mbsc-cal-btn",k),function(c,e){var n=t.getYear(r),s=t.getMonth(r),a=t.getDay(r);"day"==i?(r=t.getDate(n,s,a+e*o),d=t.getDate(n,s,a+(e+1)*o-1)):"week"==i?(r=t.getDate(n,s,a+e*o*7),d=t.getDate(n,s,a+(e+1)*o*7-1)):"month"==i?(r=t.getDate(n,s+e*o,1),d=t.getDate(n,s+(e+1)*o,0)):"year"==i&&(r=t.getDate(n+e*o,0,1),d=t.getDate(n+(e+1)*o,0,0)),P(!0)},200)),e(document).on("click",eo)},onDayChange:function(s){var n=s.target,o=n!==A;y(),o&&(Y=!1!==N&&e(".mbsc-cal-txt-more",n).length,f={d:s.date,cell:t.outerMonthChange&&e(n).hasClass("mbsc-cal-day-diff")?null:n,events:s.events})},onLabelTap:function(e){e.label&&(m("onEventSelect",{domEvent:e.domEvent,event:e.label,date:e.date}),$=!0)},onPageChange:function(e){y(),W=!0,s._isSetDate||s.setVal(e.firstDay)},onPageLoaded:function(a){var e=a.firstDay,n=a.lastDay;v&&(E?z&&F&&b(z,e)&&b(F,n)||(X(z=e,F=n),function(s,a){var e,d=(t.dateWheels||t.dateFormat).search(/m/i),u=(t.dateWheels||t.dateFormat).search(/y/i),n=t.getYear(s),r=t.getMonth(s),c=t.getYear(a),h=t.getMonth(a);"day"==i?e=l(t.dateFormat,s,t)+(1<o?" - "+l(t.dateFormat,a,t):""):"week"==i?e=l(t.dateFormat,s,t)+" - "+l(t.dateFormat,a,t):"month"==i?e=1==o?u<d?n+" "+t.monthNames[r]:t.monthNames[r]+" "+n:u<d?n+" "+t.monthNamesShort[r]+" - "+c+" "+t.monthNamesShort[h]:t.monthNamesShort[r]+" "+n+" - "+t.monthNamesShort[h]+" "+c:"year"==i&&(e=n+(1<o?" - "+c:"")),et.html(e)}(e,n)):(n="month"==i?t.getDate(t.getYear(e),t.getMonth(e)+o,0):"week"==i?t.getDate(t.getYear(e),t.getMonth(e),t.getDay(e)+7*o-1):e=s.getVal(!0),X(e,n,_)),K||b(p,e)||(q(p,ee),ee=!1)),N&&en(),W=!1},onPosition:function(n){if(O.onPosition.call(this,n),h&&h.position(),v&&U){g.addClass("mbsc-event-list-h");var e=function(e){var t=getComputedStyle(e);return e.innerHeight||e.clientHeight-parseFloat(t.paddingTop)-parseFloat(t.paddingBottom)}("inline"==t.display?ec.parentNode:window)-n.popup.offsetHeight;I.style.height=200<e?e+"px":"",g.removeClass("mbsc-event-list-h"),K&&e&&(q(p,!0),K=!1)}},onHide:function(){O.onHide.call(this),h&&h.destroy(),e(document).off("click",eo)}})},r("eventcalendar",d);var eM,nS=y&&!!window.Promise,D=[],el=[];function eK(e){D.length||e.show(),D.push(e)}function eN(s,e,t,o){return n({display:e.display||"center",cssClass:"mbsc-alert",okText:e.okText,cancelText:e.cancelText,context:e.context,theme:e.theme,closeOnOverlayTap:!1,onBeforeClose:function(){s.shift()},onHide:function(s,n){t&&t(n._resolve),e.callback&&e.callback(n._resolve),n&&n.destroy(),D.length?D[0].show():el.length&&el[0].show(!1,!0)}},o)}function e$(e){return(e.title?"<h2>"+e.title+"</h2>":"")+"<p>"+(e.message||"")+"</p>"}function nA(e,t,n){eK(new w(e,eN(D,t,n)))}function nk(t,n,s){var e=new w(t,eN(D,n,s,{buttons:["cancel","ok"],onSet:function(){e._resolve=!0}}));e._resolve=!1,eK(e)}function nE(n,s,o){var e,t=new w(n,eN(D,s,o,{buttons:["cancel","ok"],onMarkupReady:function(s,t){var n=t.settings;t._markup.find("label").addClass("mbsc-"+n.theme+(n.baseTheme?" mbsc-"+n.baseTheme:"")),e=t._markup.find("input")[0],setTimeout(function(){e.focus(),e.setSelectionRange(0,e.value.length)},300)},onSet:function(){t._resolve=e.value}}));t._resolve=null,eK(t)}function tJ(s,t,o,i,a){var n;!function(e){var t=el.length;el.push(e),D.length||(t?el[0].hide():e.show(!1,!0))}(new w(s,eN(el,t,o,{display:t.display||"bottom",animate:a,cssClass:(i||"mbsc-snackbar")+(t.color?" mbsc-"+t.color:""),scrollLock:!1,focusTrap:!1,buttons:[],onMarkupReady:function(n,t){var e=t.settings;t._markup.find("button").addClass("mbsc-"+e.theme+(e.baseTheme?" mbsc-"+e.baseTheme:""))},onShow:function(o,s){eM=s,!1!==t.duration&&(n=setTimeout(function(){s&&s.hide()},t.duration||3e3)),t.button&&s.tap(e(".mbsc-snackbar-btn",o.target),function(){s.hide(),t.button.action&&t.button.action.call(this)})},onClose:function(){eM=null,clearTimeout(n)}})))}function nC(e,t,n){tJ(e,t,n,"mbsc-toast","fade")}function ep(e,t,n){var s;return nS?s=new Promise(function(s){e(t,n,s)}):e(t,n),s}t.alert=function(e){var t=document.createElement("div");return t.innerHTML=e$(e),ep(nA,t,e)},t.confirm=function(e){var t=document.createElement("div");return t.innerHTML=e$(e),ep(nk,t,e)},t.prompt=function(e){var t=document.createElement("div");return t.innerHTML=e$(e)+'<label class="mbsc-input">'+(e.label?'<span class="mbsc-label">'+e.label+"</span>":"")+'<input class="mbsc-control" tabindex="0" type="'+(e.inputType||"text")+'" placeholder="'+(e.placeholder||"")+'" value="'+(e.value||"")+'"></label>',ep(nE,t,e)},t.snackbar=function(t){var n=document.createElement("div"),e=t.button;return n.innerHTML='<div class="mbsc-snackbar-cont"><div class="mbsc-snackbar-msg">'+(t.message||"")+"</div>"+(e?'<button class="mbsc-snackbar-btn mbsc-btn mbsc-btn-flat">'+(e.icon?'<span class="mbsc-ic '+(e.text?"mbsc-btn-ic ":"")+"mbsc-ic-"+e.icon+'"></span>':"")+(e.text||"")+"</button>":"")+"</div>",ep(tJ,n,t)},t.toast=function(e){var t=document.createElement("div");return t.innerHTML='<div class="mbsc-toast-msg">'+(e.message||"")+"</div>",ep(nC,t,e)},t.notification={dismiss:function(){eM&&eM.hide()}};function eU(c,l){var n,i="",o=e(c),a={},s=this;function r(){o.removeClass("mbsc-no-touch")}S.call(this,c,l,!0),s.refresh=function(e){n.enhance&&tx(o,a,n,e)},s._init=function(){t.themes.form[n.theme]||(n.theme="mobiscroll"),o.hasClass("mbsc-form")||o.on("touchstart",r).show(),i&&o.removeClass(i),i="mbsc-form mbsc-no-touch mbsc-"+n.theme+(ns?" mbsc-form-hb":"")+(n.baseTheme?" mbsc-"+n.baseTheme:"")+(n.rtl?" mbsc-rtl":" mbsc-ltr")+("box"==n.inputStyle?" mbsc-form-box":"")+("outline"==n.inputStyle?" mbsc-form-outline":""),o.addClass(i).removeClass("mbsc-cloak"),s.refresh()},s._destroy=function(){for(var e in o.removeClass(i).off("touchstart",r),a)a[e].destroy()},s.controls=a,n=s.settings,s.init()}ns="ios"==h&&7<x,eU.prototype={_hasDef:!0,_hasTheme:!0,_hasLang:!0,_class:"form",_defaults:{tap:et,stopProp:!0,rtl:!1,enhance:!0}},p("[mbsc-enhance],[mbsc-form]",u.Form=eU,!0);function eD(i,a){var n="",o=e(i),s=this,t=s.settings;S.call(this,i,a,!0),s._init=function(){var a=t.context,s=e(a),r=s.find(".mbsc-ms-top .mbsc-ms"),c=s.find(".mbsc-ms-bottom .mbsc-ms"),i={};"body"==a?e("body,html").addClass("mbsc-page-ctx"):s.addClass("mbsc-page-ctx"),n&&o.removeClass(n),r.length&&(i.paddingTop=r[0].offsetHeight),c.length&&(i.paddingBottom=c[0].offsetHeight),n="mbsc-page mbsc-"+t.theme+(t.baseTheme?" mbsc-"+t.baseTheme:"")+(t.rtl?" mbsc-rtl":" mbsc-ltr"),o.addClass(n).removeClass("mbsc-cloak").css(i)},s._destroy=function(){o.removeClass(n)},t=s.settings,s.init()}eD.prototype={_hasDef:!0,_hasTheme:!0,_hasLang:!0,_class:"page",_defaults:{context:"body"}},u.Page=eD,t.themes.page.mobiscroll={},p("[mbsc-page]",eD),r("page",eD,!1),r("form",eU,!1),r("progress",eh,!1),r("slider",ei,!1),r("stepper",eb,!1),r("switch",ey,!1),r("rating",e9,!1);function ni(s){var o,a,l,_,w=n({},s.settings),t=n(s.settings,na,w),j=t.layout||(/top|bottom/.test(t.display)?"liquid":""),u="liquid"==j,O=t.readonly,c=e(this),p=this.id+"_dummy",i=0,f=[],r=t.wheelArray||function i(n){var o=[],a=1<n.length?n:n.children(t.itemSelector);return a.each(function(u){n=e(this),a=n.clone(),a.children("ul,ol").remove(),a.children(t.itemSelector).remove();var n,a,l=s._processMarkup?s._processMarkup(a):a.html().replace(/^\s\s*/,"").replace(/\s\s*$/,""),d=!!n.attr("data-invalid"),c={key:void 0===n.attr("data-val")||null===n.attr("data-val")?u:n.attr("data-val"),value:l,invalid:d,children:null},r="li"===t.itemSelector?n.children("ul,ol"):n.children(t.itemSelector);r.length&&(c.children=i(r)),o.push(c)}),o}(c),v=function(o){for(var e,n=[],s=o,t=!0,i=0;t;)e=m(s),n[i++]=e.key,(t=e.children)&&(s=t);return n}(r);function x(a,r,i){for(var n,o,e=0,t=i,s=[];e<r;){o=a[e];for(n in t)if(t[n].key==o){t=t[n].children;break}e++}for(e=0;e<t.length;)t[e].invalid&&s.push(t[e].key),e++;return s}function d(l,c,i){var e,n,s=0,a=!0,o=[[]],t=r;if(c)for(e=0;e<c;e++)u?o[0][e]={}:o[e]=[{}];for(;a;){for(u?o[0][s]=y(t,s):o[s]=[y(t,s)],e=0,n=null;e<t.length&&!n;)t[e].key==l[s]&&(void 0!==i&&s<=i||void 0===i)&&(n=t[e]),e++;(n=n||m(t))&&n.children?(t=n.children,s++):a=!1}return o}function m(e,s){if(!e)return!1;for(var n,t=0;t<e.length;)if(!(n=e[t++]).invalid)return s?t-1:n;return!1}function y(n,s){for(var o={data:[],label:t.labels&&t.labels[s]?t.labels[s]:s},e=0;e<n.length;)o.data.push({value:n[e].key,display:n[e].value}),e++;return o}function g(t){s._isVisible&&e(".mbsc-sc-whl-w",s._markup).css("display","").slice(t).hide()}function h(i,l){for(var t,s,o,n=0,e=r,a=!0,c=[];a;){if(void 0!==i[n]&&n<=l)for(s=0,o=i[n],t=void 0;s<e.length&&void 0===t;)e[s].key!=i[n]||e[s].invalid||(t=s),s++;else o=e[t=m(e,!0)]&&e[t].key;c[n]=o,n++,a=!!e[t]&&e[t].children,e=e[t]&&e[t].children}return{lvl:n,nVector:c}}function b(n,a,o){var e,i,t=(a||0)+1,r=[],c={};for(i=d(n,null,a),e=0;e<n.length;e++)s._tempWheelArray[e]=n[e]=o.nVector[e]||0;for(;t<o.lvl;)c[t]=u?i[0][t]:i[t][0],r.push(t++);g(o.lvl),f=n.slice(0),r.length&&(l=!0,s.changeWheel(c))}return function s(t,n){var e;for(i=i<n?n:i,e=0;e<t.length;e++)t[e].children&&s(t[e].children,n+1)}(r,1),_=d(v,i),e("#"+p).remove(),t.input?o=e(t.input):t.showInput&&(o=e('<input type="text" id="'+p+'" value="" class="'+t.inputClass+'" placeholder="'+(t.placeholder||"")+'" readonly />').insertBefore(c)),o&&s.attachShow(o),t.wheelArray||c.hide(),{wheels:_,anchor:o,layout:j,headerText:!1,setOnTap:1==i,formatValue:function(e){return void 0===a&&(a=h(e,e.length).lvl),e.slice(0,a).join(" ")},parseValue:function(e){return e?(e+"").split(" "):(t.defaultValue||v).slice(0)},onBeforeShow:function(){var e=s.getArrayVal(!0);f=e.slice(0),t.wheels=d(e,i,i),l=!0},onWheelGestureStart:function(e){t.readonly=function(t,n){for(var e=[];t;)e[--t]=!0;return e[n]=!1,e}(i,e.index)},onWheelAnimationEnd:function(i){var e=i.index,n=s.getArrayVal(!0),o=h(n,e);a=o.lvl,t.readonly=O,n[e]!=f[e]&&b(n,e,o)},onFill:function(e){a=void 0,o&&o.val(e.valueText)},validate:function(n){var e=n.values,s=n.index,t=h(e,e.length);return a=t.lvl,void 0===s&&(g(t.lvl),l||b(e,s,t)),l=!1,{disabled:function(n,s,o){for(var e=0,t=[];e<n;)t[e]=x(o,e,s),e++;return t}(a,r,e)}},onDestroy:function(){o&&e("#"+p).remove(),c.show()}}}na={invalid:[],showInput:!0,inputClass:"",itemSelector:"li"},c.image=function(t){return t.settings.enhance&&(t._processMarkup=function(t){var n=t.attr("data-icon");return t.children().each(function(n,t){(t=e(t)).is("img")?e('<div class="mbsc-img-c"></div>').insertAfter(t).append(t.addClass("mbsc-img")):t.is("p")&&t.addClass("mbsc-img-txt")}),n&&t.prepend('<div class="mbsc-ic mbsc-ic-'+n+'"></div'),t.html('<div class="mbsc-img-w">'+t.html()+"</div>"),t.html()}),ni.call(this,t)},r("image",d);function nr(no,nr){var t,i,r,c,l,d,u,h,f,p,g,y,O,x,C,E,M,T,z,D,N,L,R,I,B,K,q,Y,G,X,J,e9,e4,e$,tF,ns,eI,eH,nn,es,nt,ev,et,ne,tJ,tQ,tX,e7,eT,t3,t2,tG,tq,t0,eF,tU,tW,t1,tu,e6,to,eS,tl,e1,ee,tc,eb,ek,ex,eO,eQ,t_,eh,ta,ea,t$,eC,eE,tB,ti,el,ez,tH,eN,en,tr,ei,ts,tn,ec,tP,tL,eU,eK,eJ,tN,te,eW,eB,e0,eA,e_,em,ef,td,a=this,_=no,w=e(_),$=0,eu=0,j=0,eD={},t8={},b={};function tD(){eC=ez=!1,tP=l=0,tL=new Date,e1=O.width(),nn=eg(O),d=nn.index(i),g=i[0].offsetHeight,j=i[0].offsetTop,u=e0[i.attr("data-type")||"defaults"],ec=u.stages}function ni(n){var s;"touchstart"===n.type&&(r.removeClass("mbsc-no-touch"),eE=!0,clearTimeout(tB)),!H(n,this)||L||$||V||eV||(to=!(e$=L=!0),tl="touchstart"===n.type,eU=o(n,"X"),eK=o(n,"Y"),J=C=0,i=e(this),s=i,tD(),te=u.actionable||i.hasClass("mbsc-lv-parent")||i.hasClass("mbsc-lv-back"),tc=i.offset().top,te&&(e4=setTimeout(function(){s.addClass(tl?eo:""),p("onItemActivate",{target:s[0],domEvent:n})},120)),a.sortable&&!i.hasClass("mbsc-lv-back")&&(a.sortable.group||(ta=i.nextUntil(".mbsc-lv-gr-title").filter(".mbsc-lv-item"),ti=i.prevUntil(".mbsc-lv-gr-title").filter(".mbsc-lv-item")),eO=(a.sortable.group?O.children(D).eq(0):ti.length?ti.eq(-1):i)[0].offsetTop-j,ex=(a.sortable.group?O.children(D).eq(-1):ta.length?ta.eq(-1):i)[0].offsetTop-j,a.sortable.handle?e(n.target).hasClass("mbsc-lv-handle")&&(clearTimeout(e4),"Moz"===m?(n.preventDefault(),ty()):eB=setTimeout(function(){ty()},100)):eB=setTimeout(function(){e7.appendTo(i),e7[0].style[m+"Animation"]="mbsc-lv-fill "+(t.sortDelay-100)+"ms linear",clearTimeout(tJ),clearTimeout(e4),e$=!1,eB=setTimeout(function(){e7[0].style[m+"Animation"]="",ty()},t.sortDelay-80)},80)),"mousedown"==n.type&&e(document).on("mousemove",tg).on("mouseup",eL))}function tg(s){var t=!1,r=!0,m=l;if(L)if(tQ=o(s,"X"),tX=o(s,"Y"),C=tQ-eU,J=tX-eK,clearTimeout(tJ),et||K||en||i.hasClass("mbsc-lv-back")||(10<Math.abs(J)?(en=!0,eL(n({},s,{type:"mousemove"==s.type?"mouseup":"touchend"})),clearTimeout(e4)):7<Math.abs(C)&&tA()),K)s.preventDefault(),l=C/e1*100,tb(m);else if(et){s.preventDefault();var h,d=c.scrollTop(),e=Math.max(eO,Math.min(J+ef,ex)),u=z?tc-td+d-ef:tc;em+d<u+e+g?(c.scrollTop(u+e-em+g),h=!0):u+e<d&&(c.scrollTop(u+e),h=!0),h&&(ef+=c.scrollTop()-d),q&&(a.sortable.multiLevel&&M.hasClass("mbsc-lv-parent")?q<j+g/4+e?t=!0:q<j+g-g/4+e&&(T=M.addClass("mbsc-lv-item-hl"),r=!1):q<j+g/2+e&&(M.hasClass("mbsc-lv-back")?a.sortable.multiLevel&&(R=M.addClass("mbsc-lv-item-hl"),r=!1):t=!0),t&&(N.insertAfter(M),M=eM(y=M,"next"),Y=q,q=M.length&&M[0].offsetTop,E++)),!t&&Y&&(a.sortable.multiLevel&&y.hasClass("mbsc-lv-parent")?j+g-g/4+e<Y?t=!0:j+g/4+e<Y&&(T=y.addClass("mbsc-lv-item-hl"),r=!1):j+g/2+e<Y&&(y.hasClass("mbsc-lv-back")?a.sortable.multiLevel&&(R=y.addClass("mbsc-lv-item-hl"),r=!1):t=!0),t&&(N.insertBefore(y),y=eM(M=y,"prev"),q=Y,Y=y.length&&y[0].offsetTop+y[0].offsetHeight,E--)),r&&(T&&(T.removeClass("mbsc-lv-item-hl"),T=!1),R&&(R.removeClass("mbsc-lv-item-hl"),R=!1)),t&&p("onSortChange",{target:i[0],index:E}),tz(i,e),p("onSort",{target:i[0],index:E})}else(5<Math.abs(C)||5<Math.abs(J))&&t6()}function eL(o){var n,s,r,c=i;L&&(L=!1,t6(),"mouseup"==o.type&&e(document).off("mousemove",tg).off("mouseup",eL),en||(tB=setTimeout(function(){eE=!1},300)),(K||en||et)&&(eC=!0),K?tC():et?(s=O,T?(er(i.detach()),n=b[T.attr("data-ref")],E=eg(n.child).length,T.removeClass("mbsc-lv-item-hl"),t.navigateOnDrop?tm(T,function(){a.add(null,i,null,null,T,!0),e5(i),ep(i,d,s,!0)}):(a.add(null,i,null,null,T,!0),ep(i,d,s,!0))):R?(er(i.detach()),n=b[R.attr("data-back")],E=eg(n.parent).index(n.item)+1,R.removeClass("mbsc-lv-item-hl"),t.navigateOnDrop?tm(R,function(){a.add(null,i,E,null,O,!0),e5(i),ep(i,d,s,!0)}):(a.add(null,i,E,null,n.parent,!0),ep(i,d,s,!0))):(r=N[0].offsetTop-j,tz(i,r,6*Math.abs(r-Math.max(eO,Math.min(J+ef,ex))),function(){er(i),i.insertBefore(N),ep(i,d,s,E!==d)})),et=!1):!en&&Math.abs(C)<5&&Math.abs(J)<5&&(to=!0,"touchend"===o.type&&t.tap&&t4(o.target,ej(e(o.target)),o)),clearTimeout(e4),setTimeout(function(){c.removeClass(eo),p("onItemDeactivate",{target:c[0]})},100),en=!1,h=null)}function na(t){var n;to&&(n="true"==i.attr("data-selected"),u.tap&&u.tap.call(_,{target:i,index:d,domEvent:t},a),te&&!i.hasClass(eo)&&(i.addClass(tl?eo:""),p("onItemActivate",{target:i[0],domEvent:t})),tr&&(t_?n?eR(i):t5(i):(eR(e(D,r).filter("."+e8)),t5(i))),!1!==p("onItemTap",{target:i[0],index:d,domEvent:t,selected:n})&&tm(i))}function tA(){(K=e2(u.swipe,{target:i[0],index:d,direction:0<C?"right":"left"}))&&(t6(),clearTimeout(e4),u.actions?(e9=tY(u,C),eQ.html(u.icons).show().children().css("width",e9+"%"),x.hide(),e(".mbsc-lv-ic-m",f).removeClass("mbsc-lv-ic-disabled"),e(u.leftMenu).each(tk),e(u.rightMenu).each(tk)):(x.show(),eQ.hide(),es=u.start,h=ec[es],el=ec[es-1],eh=ec[es+1]),i.addClass("mbsc-lv-item-swiping").removeClass(eo),eW.css("line-height",g+"px"),f.css({top:j,height:g,backgroundColor:tK(C)}).addClass("mbsc-lv-stage-c-v").appendTo(O.parent()),t.iconSlide&&i.append(x),p("onSlideStart",{target:i[0],index:d}))}function tb(n){var e=!1;eN||(u.actions?f.attr("class","mbsc-lv-stage-c-v mbsc-lv-stage-c mbsc-lv-"+(l<0?"right":"left")):(el&&(l<0?l<=el.percent:l<h.percent)?(eh=h,h=el,el=ec[--es-1],e=!0):eh&&(l<0?l>h.percent:l>=eh.percent)&&(el=h,h=eh,eh=ec[++es+1],e=!0),h&&(!e&&0<l!=n<=0||eZ(h,t.iconSlide),e&&p("onStageChange",{target:i[0],index:d,stage:h}))),ei||(eN=!0,tH=F(tM)))}function tC(n){var o,a,s=!1;eY(tH),eN=!1,ei||tM(),u.actions?10<Math.abs(l)&&e9&&(e3(i,l<0?-e9:e9,200),V=s=!0,tF=i,ns=d,e(document).on("touchstart.mbsc-lv-conf mousedown.mbsc-lv-conf",function(e){e.preventDefault(),eG(i,!0,n)})):l&&(t.quickSwipe&&!ei&&(o=(a=new Date-tL)<300&&50<C,a<300&&C<-50?(ez=!0,eZ(h=u.left,t.iconSlide)):o&&(ez=!0,eZ(h=u.right,t.iconSlide))),h&&h.action&&(e2(h.disabled,{target:i[0],index:d})||(s=!0,(V=ei||e2(h.confirm,{target:i[0],index:d}))?(e3(i,(l<0?-1:1)*x[0].offsetWidth*100/e1,200,!0),tS(h,i,d,!1,n)):t9(h,i,d,n)))),s||eG(i,!0,n),K=!1}function ty(){R=T=!(et=!0),ef=0,E=d,t.vibrate&&eX(),M=eM(i,"next"),q=M.length&&M[0].offsetTop,y=eM(i,"prev"),Y=y.length&&y[0].offsetTop+y[0].offsetHeight,N.height(g).insertAfter(i),i.css({top:j}).addClass("mbsc-lv-item-dragging").removeClass(eo).appendTo(ne),p("onSortStart",{target:i[0],index:E})}function ep(e,n,s,o){e.removeClass("mbsc-lv-item-dragging"),N.remove(),p("onSortEnd",{target:e[0],index:E}),t.vibrate&&eX(),o&&(a.addUndoAction(function(t){a.move(e,n,null,t,s,!0)},!0),p("onSortUpdate",{target:e[0],index:E}))}function tw(){eE||(clearTimeout(tW),V&&e(document).trigger("touchstart"),t0&&(a.close(G,eF),t0=!1,G=null))}function ey(){clearTimeout(nt),nt=setTimeout(function(){em=c[0].innerHeight||c.innerHeight(),td=z?c.offset().top:0,L&&(j=i[0].offsetTop,g=i[0].offsetHeight,f.css({top:j,height:g}))},200)}function tx(e){eC&&(e.stopPropagation(),e.preventDefault(),eC=!1)}function tj(){eS||(clearTimeout(t$),t$=setTimeout(function(){var e=z?c[0].getBoundingClientRect().top+c.innerHeight():window.innerHeight,t=ek[0].getBoundingClientRect().top-3<e;!eS&&t&&p("onListEnd")},250))}function tE(){if(et||!L){var n,o=c.scrollTop(),s=w.offset().top,i=w[0].offsetHeight,t=z?c.offset().top:o;e(".mbsc-lv-gr-title",w).each(function(o,s){e(s).offset().top<t&&(n=s)}),s<t&&t<s+i?B.show().empty().append(e(n).clone()):B.hide()}}function tk(n,t){e2(t.disabled,{target:i[0],index:d})&&e(".mbsc-ic-"+t.icon,f).addClass("mbsc-lv-ic-disabled")}function t9(s,n,o,r){var i,c={icon:"undo2",text:t.undoText,action:function(){a.undo()}};s.undo&&(a.startActionTrack(),e.isFunction(s.undo)&&a.addUndoAction(function(){s.undo.call(_,{target:n[0],index:o},a)}),eA=n.attr("data-ref")),i=s.action.call(_,{target:n[0],index:o},a),s.undo?(a.endActionTrack(),!1!==i&&e3(n,+n.attr("data-pos")<0?-100:100,200),N.height(g).insertAfter(n),n.css("top",j).addClass("mbsc-lv-item-undo"),eQ.hide(),x.show(),f.append(x),eZ(c),tS(c,n,o,!0,r)):eG(n,i,r)}function tS(c,t,r,s,i){var n,a;V=!0,e(document).off(".mbsc-lv-conf").on("touchstart.mbsc-lv-conf mousedown.mbsc-lv-conf",function(e){e.preventDefault(),s&&tR(t),eG(t,!0,i)}),ev||x.off(".mbsc-lv-conf").on("touchstart.mbsc-lv-conf mousedown.mbsc-lv-conf",function(e){e.stopPropagation(),a=o(e,"X"),n=o(e,"Y")}).on("touchend.mbsc-lv-conf mouseup.mbsc-lv-conf",function(e){e.preventDefault(),"touchend"===e.type&&P(),Math.abs(o(e,"X")-a)<10&&Math.abs(o(e,"Y")-n)<10&&(t9(c,t,r,i),s&&(e_=null,tR(t)))})}function tM(){e3(i,tP+100*C/e1),eN=!1}function eG(t,s,n){e(document).off(".mbsc-lv-conf"),x.off(".mbsc-lv-conf"),!1!==s?e3(t,0,"0"!==t.attr("data-pos")?200:0,!1,function(){tt(t,n),er(t)}):tt(t,n),V=!1}function e3(t,e,n,o,s){e=Math.max("right"==K?0:-100,Math.min(e,"left"==K?0:100)),X=t[0].style,t.attr("data-pos",e),X[m+"Transform"]="translate3d("+(o?e1*e/100+"px":e+"%")+",0,0)",X[m+"Transition"]=k+"transform "+(n||0)+"ms",s&&($++,setTimeout(function(){s(),$--},n)),l=e}function tz(s,e,t,n){e=Math.max(eO,Math.min(e,ex)),(X=s[0].style)[m+"Transform"]="translate3d(0,"+e+"px,0)",X[m+"Transition"]=k+"transform "+(t||0)+"ms ease-out",n&&($++,setTimeout(function(){n(),$--},t))}function t6(){clearTimeout(eB),!e$&&a.sortable&&(e$=!0,e7.remove())}function eZ(e,s){var n=e2(e.text,{target:i[0],index:d})||"";e2(e.disabled,{target:i[0],index:d})?f.addClass("mbsc-lv-ic-disabled"):f.removeClass("mbsc-lv-ic-disabled"),f.css("background-color",e.color||(0===e.percent?tK(l):n4)),x.attr("class","mbsc-lv-ic-c mbsc-lv-ic-"+(s?"move-":"")+(l<0?"right":"left")),e6.attr("class"," mbsc-lv-ic-s mbsc-lv-ic mbsc-ic mbsc-ic-"+(e.icon||"none")),eW.attr("class","mbsc-lv-ic-text"+(e.icon?"":" mbsc-lv-ic-text-only")+(n?"":" mbsc-lv-ic-only")).html(n||"&nbsp;"),t.animateIcons&&(ez?e6.addClass("mbsc-lv-ic-v"):setTimeout(function(){e6.addClass("mbsc-lv-ic-a")},10))}function tt(e,t){L||(e6.attr("class","mbsc-lv-ic-s mbsc-lv-ic mbsc-ic mbsc-ic-none"),f.attr("style","").removeClass("mbsc-lv-stage-c-v"),eW.html("")),f.removeClass("mbsc-lv-left mbsc-lv-right"),e&&(p("onSlideEnd",{target:e[0],index:d}),t&&t())}function tR(e){e.css("top","").removeClass("mbsc-lv-item-undo"),e_?a.animate(N,"collapse",function(){N.remove()}):N.remove(),tt(),e_=eA=null}function er(e){(X=e[0].style)[m+"Transform"]="",X[m+"Transition"]="",X.top="",e.removeClass("mbsc-lv-item-swiping")}function e2(t,n){return e.isFunction(t)?t.call(this,n,a):t}function tI(e){return tr&&!e.hasClass("mbsc-lv-parent")&&!e.hasClass("mbsc-lv-back")}function tO(n){var o,i,s=n.attr("data-ref"),r=n.attr("data-role"),l=e0[n.attr("data-type")||"defaults"],c=tI(n)&&"true"==n.attr("data-selected");s||(s=ew++,n.attr("data-ref",s)),b[s]={item:n,child:n.children(I),parent:n.parent(),ref:n.parent()[0]===_?null:n.parent().parent().attr("data-ref")},n.addClass("list-divider"==r?"mbsc-lv-gr-title":"mbsc-lv-item"+(l.actionable?" mbsc-lv-item-actionable":"")+(c?" "+e8:"")),n.attr("aria-selected",c?"true":"false"),a.sortable.handle&&"list-divider"!=r&&!n.children(".mbsc-lv-handle-c").length&&n.append(tG),t.enhance&&!n.hasClass("mbsc-lv-item-enhanced")&&(o=n.attr("data-icon"),i=n.find("img").eq(0).addClass("mbsc-lv-img"),i.is(":first-child")?n.addClass("mbsc-lv-img-"+(t.rtl?"right":"left")):i.length&&n.addClass("mbsc-lv-img-"+(t.rtl?"left":"right")),n.addClass("mbsc-lv-item-enhanced").children().each(function(n,t){(t=e(t)).is("p, h1, h2, h3, h4, h5, h6")&&t.addClass("mbsc-lv-txt")}),o&&n.addClass("mbsc-lv-item-ic-"+(n.attr("data-icon-align")||(t.rtl?"right":"left"))).append('<div class="mbsc-lv-item-ic mbsc-ic mbsc-ic-'+o+'"></div>'))}function tV(t){e(D,t).not(".mbsc-lv-back").each(function(){tO(e(this))}),e(I,t).not(".mbsc-lv").addClass("mbsc-lv").prepend(t1).parent().addClass("mbsc-lv-parent mbsc-lv-item-actionable").prepend(tu),e(".mbsc-lv-back",t).each(function(){e(this).attr("data-back",e(this).parent().parent().attr("data-ref"))})}function eg(e){return e.children(D).not(".mbsc-lv-back").not(".mbsc-lv-removed").not(".mbsc-lv-ph")}function Z(t){return"object"!==U(t)&&(t=e(D,r).filter('[data-id="'+t+'"]')),e(t)}function eM(e,t){for(e=e[t]();e.length&&(!e.hasClass("mbsc-lv-item")||e.hasClass("mbsc-lv-ph")||e.hasClass("mbsc-lv-item-dragging"));){if(!a.sortable.group&&e.hasClass("mbsc-lv-gr-title"))return!1;e=e[t]()}return e}function tK(e){return(0<e?u.right:u.left).color||n4}function Q(e){return v(e)?e+"":0}function tY(e,n){return+(n<0?Q((e.actionsWidth||0).right)||Q(e.actionsWidth)||Q(t.actionsWidth.right)||Q(t.actionsWidth):Q((e.actionsWidth||0).left)||Q(e.actionsWidth)||Q(t.actionsWidth.left)||Q(t.actionsWidth))}function e5(n,o){if(n){var t=c.scrollTop(),s=n.is(".mbsc-lv-item")?n[0].offsetHeight:0,e=n.offset().top+(z?t-td:0);o?(e<t||t+em<e+s)&&c.scrollTop(e):e<t?c.scrollTop(e):t+em<e+s&&c.scrollTop(Math.min(e,e+s-em/2))}}function th(n,e,o,i,c){var l=e.parent(),a=e.prev();i=i||s,a[0]===x[0]&&(a=x.prev()),t.rtl&&(n="l"===n?"r":"l"),O[0]!==e[0]?(p("onNavStart",{level:eu,direction:n,list:e[0]}),ts.prepend(e.addClass("mbsc-lv-v mbsc-lv-sl-new")),e5(r),tZ(ts,"mbsc-lv-sl-"+n,function(){O.removeClass("mbsc-lv-sl-curr"),e.removeClass("mbsc-lv-sl-new").addClass("mbsc-lv-sl-curr"),eI&&eI.length?O.removeClass("mbsc-lv-v").insertAfter(eI):eH.append(O.removeClass("mbsc-lv-v")),eI=a,eH=l,O=e,e5(o,c),i.call(_,o),p("onNavEnd",{level:eu,direction:n,list:e[0]})})):(e5(o,c),i.call(_,o))}function tm(e,t){$||(e.hasClass("mbsc-lv-parent")?(eu++,th("r",b[e.attr("data-ref")].child,null,t)):e.hasClass("mbsc-lv-back")&&(eu--,th("l",b[e.attr("data-back")].parent,b[e.attr("data-back")].item,t)))}function tZ(e,o,n){var a;function i(){clearTimeout(a),$--,e.off(W,i).removeClass(o),n.call(_,e)}n=n||s,t.animation&&"mbsc-lv-item-none"!==o?($++,e.on(W,i).addClass(o),a=setTimeout(i,250)):n.call(_,e)}function tf(e,t){var s=e.attr("data-ref"),n=t8[s]=t8[s]||[];t&&n.push(t),e.attr("data-action")||(t=n.shift())&&(e.attr("data-action",1),t(function(){e.removeAttr("data-action"),n.length?tf(e):delete t8[s]}))}function tp(t,i,a){var s,o;t&&t.length&&(o=100/(t.length+2),e.each(t,function(r,e){void 0===e.key&&(e.key=tn++),void 0===e.percent&&(e.percent=i*o*(r+1),a&&((s=n({},e)).key=tn++,s.percent=-o*(r+1),t.push(s),eD[s.key]=s)),eD[e.key]=e}))}function t5(e){tI(e)&&e.addClass(e8).attr("data-selected","true").attr("aria-selected","true")}function eR(e){e.removeClass(e8).removeAttr("data-selected").removeAttr("aria-selected")}S.call(this,no,nr,!0),a.animate=function(e,t,n){tZ(e,"mbsc-lv-item-"+t,n)},a.add=function(A,j,l,v,x,d){var c,u,h,f,O,C,g="",o=void 0===x?w:Z(x),i=o,n="object"!==U(j)?e("<"+ee+' data-ref="'+ew+++'" data-id="'+A+'">'+j+"</"+ee+">"):e(j),y=n[0],E=y.style,k=n.attr("data-pos")<0?"left":"right",m=n.attr("data-ref");v=v||s,m||(m=ew++,n.attr("data-ref",m)),tO(n),d||a.addUndoAction(function(e){C?a.navigate(o,function(){i.remove(),o.removeClass("mbsc-lv-parent").children(".mbsc-lv-arr").remove(),c.child=o.children(I),a.remove(n,null,e,!0)}):a.remove(n,null,e,!0)},!0),tf(n,function(s){er(n.css("top","").removeClass("mbsc-lv-item-undo")),o.is(D)?(u=o.attr("data-ref"),o.children(I).length||(C=!0,o.append("<"+eb+"></"+eb+">"))):u=o.children(".mbsc-lv-back").attr("data-back"),(c=b[u])&&(c.child.length?i=c.child:(o.addClass("mbsc-lv-parent").prepend(tu),i=o.children(I).prepend(t1).addClass("mbsc-lv"),c.child=i,e(".mbsc-lv-back",o).attr("data-back",u))),b[m]={item:n,child:n.children(I),parent:i,ref:u},f=eg(i),h=f.length,null==l&&(l=h),d&&(g="mbsc-lv-item-new-"+(d?k:"")),tV(n.addClass(g)),!1!==l&&(h?l<h?n.insertBefore(f.eq(l)):n.insertAfter(f.eq(h-1)):(O=e(".mbsc-lv-back",i)).length?n.insertAfter(O):i.append(n)),r.trigger("mbsc-refresh"),t.animateAddRemove&&i.hasClass("mbsc-lv-v")?(E.height=y.offsetHeight+"px",a.animate(n,d&&eA===m?"none":"expand",function(e){a.animate(e,d?"add-"+k:"pop-in",function(e){E.height="",v.call(_,e.removeClass(g)),s()})})):(v.call(_,n.removeClass(g)),s()),p("onItemAdd",{target:y})})},a.swipe=function(e,s,t,o,a){var n;e=Z(e),i=e,ev=o,L=ei=!0,t=void 0===t?300:t,C=0<s?1:-1,tD(),tA(),e3(e,s,t),clearTimeout(tN),clearInterval(eJ),eJ=setInterval(function(){n=l,l=t7(e)/e1*100,tb(n)},10),tN=setTimeout(function(){clearInterval(eJ),n=l,l=s,tb(n),tC(a),L=ei=ev=!1},t)},a.openStage=function(t,e,n,s){eD[e]&&a.swipe(t,eD[e].percent,n,s)},a.openActions=function(e,t,s,o){e=Z(e);var n=tY(e0[e.attr("data-type")||"defaults"],"left"==t?-1:1);a.swipe(e,"left"==t?-n:n,s,o)},a.close=function(e,t){a.swipe(e,0,t)},a.remove=function(f,c,o,m){var n,i,r,l,d,u,h;o=o||s,r=(n=Z(f)).attr("data-ref"),n.length&&b[r]&&(i=n.parent(),h=eg(i).index(n),d=n[0].style,function n(t){t&&(l=l||t.hasClass("mbsc-lv-v"),t.children("[data-ref]").each(function(){var t=e(this).attr("data-ref");b[t]&&(n(b[t].child),delete b[t])}))}(b[r].child),l&&(u=t.animation,t.animation=!1,a.navigate(n),t.animation=u),delete b[r],m||(n.attr("data-ref")===eA&&(e_=!0),a.addUndoAction(function(e){a.add(null,n,h,e,i,!0)},!0)),tf(n,function(e){c=c||(n.attr("data-pos")<0?"left":"right"),t.animateAddRemove&&i.hasClass("mbsc-lv-v")?a.animate(n.addClass("mbsc-lv-removed"),m?"pop-out":"remove-"+c,function(t){d.height=t[0].offsetHeight+"px",a.animate(t,"collapse",function(t){d.height="",er(t.removeClass("mbsc-lv-removed")),!1!==o.call(_,t)&&t.remove(),e()})}):(!1!==o.call(_,n)&&n.remove(),e()),p("onItemRemove",{target:n[0]})}))},a.move=function(e,n,s,o,i,t){e=Z(e),t||a.startActionTrack(),f.append(x),a.remove(e,s,null,t),a.add(null,e,n,o,i,t),t||a.endActionTrack()},a.navigate=function(e,s){var t,n;e=Z(e),t=b[e.attr("data-ref")],n=function(n){for(var t=0,e=b[n.attr("data-ref")];e&&e.ref;)t++,e=b[e.ref];return t}(e),t&&(th(eu<=n?"r":"l",t.parent,e,s,!0),eu=n)},a.showLoading=function(){eS=!0,ek.addClass("mbsc-show-lv-loading"),c.scrollTop(z?c[0].scrollHeight:e(t.context)[0].scrollHeight)},a.hideLoading=function(){ek.removeClass("mbsc-show-lv-loading"),setTimeout(function(){eS=!1},100)},a.select=function(t){t_||eR(e(D,r).filter("."+e8)),t5(Z(t))},a.deselect=function(e){eR(Z(e))},a._processSettings=function(){w.is("[mbsc-enhance]")&&(t3=!0,w.removeAttr("mbsc-enhance"))},a._init=function(){var s,i,d,g=w.find(I).length?"left":"right",n=0,h="",m="",p="";eb=t.listNode,I=t.listSelector,ee=t.itemNode,D=t.itemSelector,t_="multiple"==t.select,tr="off"!=t.select,"group"===(s=t.sort||t.sortable||!1)&&(s={group:!1,multiLevel:!0}),!0===s&&(s={group:!0,multiLevel:!0,handle:t.sortHandle}),s&&void 0===s.handle&&(s.handle=t.sortHandle),s.handle&&(t2=!0===s.handle?g:s.handle,tG='<div class="mbsc-lv-handle-c mbsc-lv-item-h-'+t2+' mbsc-lv-handle"><div class="'+t.handleClass+' mbsc-lv-handle-bar-c mbsc-lv-handle">'+t.handleMarkup+"</div></div>"),t1="<"+ee+' class="mbsc-lv-item mbsc-lv-back mbsc-lv-item-actionable">'+t.backText+'<div class="mbsc-lv-arr mbsc-lv-ic mbsc-ic '+t.leftArrowClass+'"></div></'+ee+">",tu='<div class="mbsc-lv-arr mbsc-lv-ic mbsc-ic '+t.rightArrowClass+'"></div>',i="mbsc-no-touch mbsc-lv-cont mbsc-lv-"+t.theme+" mbsc-"+t.theme+(nO?" mbsc-lv-hb":"")+(t.rtl?" mbsc-lv-rtl mbsc-rtl":" mbsc-ltr")+(t.baseTheme?" mbsc-lv-"+t.baseTheme+" mbsc-"+t.baseTheme:"")+(t.animateIcons?" mbsc-lv-ic-anim":"")+(t.striped?" mbsc-lv-alt-row":"")+(t.fixedHeader?" mbsc-lv-has-fixed-header":"")+(s.handle?" mbsc-lv-handle-"+t2:""),a.sortable=s||!1,r?(r.attr("class",i),e(".mbsc-lv-handle-c",r).remove(),e(D,r).not(".mbsc-lv-back").removeClass("mbsc-lv-item"),c.off("orientationchange resize",ey),ea&&c.off("scroll touchmove",ea),c.off("scroll touchmove",tj)):(h+='<div class="mbsc-lv-multi-c"></div>',h+='<div class="mbsc-lv-ic-c"><div class="mbsc-lv-ic-s mbsc-lv-ic mbsc-ic mbsc-ic-none"></div><div class="mbsc-lv-ic-text"></div></div>',w.addClass("mbsc-lv mbsc-lv-v mbsc-lv-root").removeClass("mbsc-cloak").show(),f=e('<div class="mbsc-lv-stage-c">'+h+"</div>"),x=e(".mbsc-lv-ic-c",f),eQ=e(".mbsc-lv-multi-c",f),e6=e(".mbsc-lv-ic-s",f),eW=e(".mbsc-lv-ic-text",f),N=e("<"+ee+' class="mbsc-lv-item mbsc-lv-ph"></'+ee+">"),e7=e('<div class="mbsc-lv-fill-item"></div>'),r=e('<div class="'+i+'"><'+eb+' class="mbsc-lv mbsc-lv-dummy"></'+eb+'><div class="mbsc-lv-sl-c"></div><div class="mbsc-lv-loading"><span class="mbsc-ic mbsc-ic-'+(t.loadingIcon||"loop2")+'"></span></div></div>'),ne=e(".mbsc-lv-dummy",r),ek=e(".mbsc-lv-loading",r),r.insertAfter(w),ey(),r.on("touchstart mousedown",".mbsc-lv-item",ni).on("touchmove",".mbsc-lv-item",tg).on("touchend touchcancel",".mbsc-lv-item",eL).on("click",".mbsc-lv-item",na),_.addEventListener("click",tx,!0),r.on("touchstart mousedown",".mbsc-lv-ic-m",function(e){ev||(e.stopPropagation(),e.preventDefault()),eU=o(e,"X"),eK=o(e,"Y")}).on("touchend mouseup",".mbsc-lv-ic-m",function(t){ev||("touchend"===t.type&&P(),V&&!e(this).hasClass("mbsc-lv-ic-disabled")&&Math.abs(o(t,"X")-eU)<10&&Math.abs(o(t,"Y")-eK)<10&&t9((l<0?u.rightMenu:u.leftMenu)[e(this).index()],tF,ns))}),ts=e(".mbsc-lv-sl-c",r).append(w.addClass("mbsc-lv-sl-curr")).attr("data-ref",ew++),O=w,eH=r),z="body"!==t.context,(c=e(z?t.context:window)).on("orientationchange resize",ey),c.on("scroll touchmove",tj),tn=0,(e0=t.itemGroups||{}).defaults={swipeleft:t.swipeleft,swiperight:t.swiperight,stages:t.stages,actions:t.actions,actionsWidth:t.actionsWidth,actionable:t.actionable},tV(w),e.each(e0,function(o,s){if(s.swipe=void 0!==s.swipe?s.swipe:t.swipe,s.actionable=void 0!==s.actionable?s.actionable:t.actionable,s.stages=s.stages||[],tp(s.stages,1,!0),tp(s.stages.left,1),tp(s.stages.right,-1),(s.stages.left||s.stages.right)&&(s.stages=[].concat(s.stages.left||[],s.stages.right||[])),eT=!1,s.stages.length||(s.swipeleft&&s.stages.push({percent:-30,action:s.swipeleft}),s.swiperight&&s.stages.push({percent:30,action:s.swiperight})),e.each(s.stages,function(t,e){if(0===e.percent)return!(eT=!0)}),eT||s.stages.push({percent:0}),s.stages.sort(function(e,t){return e.percent-t.percent}),e.each(s.stages,function(e,t){if(0===t.percent)return s.start=e,!1}),eT?s.left=s.right=s.stages[s.start]:(s.left=s.stages[s.start-1]||{},s.right=s.stages[s.start+1]||{}),s.actions){for(s.leftMenu=s.actions.left||s.actions,s.rightMenu=s.actions.right||s.leftMenu,p=m="",n=0;n<s.leftMenu.length;n++)m+="<div "+(s.leftMenu[n].color?'style="background-color: '+s.leftMenu[n].color+'"':"")+' class="mbsc-lv-ic-m mbsc-lv-ic mbsc-ic mbsc-ic-'+s.leftMenu[n].icon+'">'+(s.leftMenu[n].text||"")+"</div>";for(n=0;n<s.rightMenu.length;++n)p+="<div "+(s.rightMenu[n].color?'style="background-color: '+s.rightMenu[n].color+'"':"")+' class="mbsc-lv-ic-m mbsc-lv-ic mbsc-ic mbsc-ic-'+s.rightMenu[n].icon+'">'+(s.rightMenu[n].text||"")+"</div>";s.actions.left&&(s.swipe=s.actions.right?s.swipe:"right"),s.actions.right&&(s.swipe=s.actions.left?s.swipe:"left"),s.icons='<div class="mbsc-lv-multi mbsc-lv-multi-ic-left">'+m+'</div><div class="mbsc-lv-multi mbsc-lv-multi-ic-right">'+p+"</div>"}}),t.fixedHeader&&(d="mbsc-lv-fixed-header"+(z?" mbsc-lv-fixed-header-ctx mbsc-lv-"+t.theme+" mbsc-"+t.theme+(t.baseTheme?" mbsc-lv-"+t.baseTheme+" mbsc-"+t.baseTheme:""):""),B?(B.attr("class",d),tE()):B=e('<div class="'+d+'"></div>'),z?c.before(B):r.prepend(B),ea=eq(tE,200),c.on("scroll touchmove",ea)),t.hover&&(eF||r.on("mouseover.mbsc-lv",".mbsc-lv-item",function(){G&&G[0]==this||(tw(),G=e(this),e0[G.attr("data-type")||"defaults"].actions&&(tW=setTimeout(function(){eE?G=null:(t0=!0,a.openActions(G,tq,eF,!1))},tU)))}).on("mouseleave.mbsc-lv",tw),eF=t.hover.time||200,tU=t.hover.timeout||200,tq=t.hover.direction||t.hover||"right"),t3&&r.attr("mbsc-enhance",""),r.trigger("mbsc-enhance",[{theme:t.theme,lang:t.lang}])},a._destroy=function(){var t;eH.append(O),z&&B&&B.remove(),t3&&(w.attr("mbsc-enhance",""),(t=A[r[0].id])&&t.destroy()),_.removeEventListener("click",tx,!0),r.find(".mbsc-lv-txt,.mbsc-lv-img").removeClass("mbsc-lv-txt mbsc-lv-img"),r.find(I).removeClass("mbsc-lv mbsc-lv-v mbsc-lv-root mbsc-lv-sl-curr").find(D).removeClass("mbsc-lv-gr-title mbsc-lv-item mbsc-lv-item-enhanced mbsc-lv-parent mbsc-lv-img-left mbsc-lv-img-right mbsc-lv-item-ic-left mbsc-lv-item-ic-right").removeAttr("data-ref"),e(".mbsc-lv-back,.mbsc-lv-handle-c,.mbsc-lv-arr,.mbsc-lv-item-ic",r).remove(),w.insertAfter(r),r.remove(),f.remove(),c.off("orientationchange resize",ey),c.off("scroll touchmove",tj),ea&&c.off("scroll touchmove",ea)};var eV,tT=[],ed=[],tv=[],eP=0;a.startActionTrack=function(){eP||(tv=[]),eP++},a.endActionTrack=function(){--eP||ed.push(tv)},a.addUndoAction=function(n,s){var e={action:n,async:s};eP?tv.push(e):(ed.push([e]),ed.length>t.undoLimit&&ed.shift())},a.undo=function(){var e,t,n;function s(){t<0?(eV=!1,o()):(e=n[t],t--,e.async?e.action(s):(e.action(),s()))}function o(){(n=tT.shift())&&(eV=!0,t=n.length-1,s())}ed.length&&tT.push(ed.pop()),eV||o()},t=a.settings,p=a.trigger,a.init()}var V,e8="mbsc-selected",eo="mbsc-lv-item-active",nO="ios"==h&&7<x,ew=1,n4="transparent";nr.prototype={_class:"listview",_hasDef:!0,_hasTheme:!0,_hasLang:!0,_defaults:{context:"body",actionsWidth:90,sortDelay:250,undoLimit:10,tap:et,swipe:!0,quickSwipe:!0,animateAddRemove:!0,animateIcons:!0,animation:!0,revert:!0,vibrate:!0,actionable:!0,handleClass:"",handleMarkup:'<div class="mbsc-lv-handle-bar mbsc-lv-handle"></div><div class="mbsc-lv-handle-bar mbsc-lv-handle"></div><div class="mbsc-lv-handle-bar mbsc-lv-handle"></div>',listNode:"ul",listSelector:"ul,ol",itemNode:"li",itemSelector:"li",leftArrowClass:"mbsc-ic-arrow-left4",rightArrowClass:"mbsc-ic-arrow-right4",backText:"Back",undoText:"Undo",stages:[],select:"off"}},r("listview",u.ListView=nr,!(t.themes.listview.mobiscroll={leftArrowClass:"mbsc-ic-arrow-left5",rightArrowClass:"mbsc-ic-arrow-right5"})),n0={batch:50,min:0,max:100,defaultUnit:"",units:null,unitNames:null,invalid:[],sign:!1,step:.05,scale:2,convert:function(e){return e},signText:"&nbsp;",wholeText:"Whole",fractionText:"Fraction",unitText:"Unit"},c.measurement=function(w){var s,a,r,c,d,u,h,g,C,E,F,N,P,W,K=n({},w.settings),t=n(w.settings,n0,K),T={},k=[[]],v={},x={},M={},H=[],o=t.sign,p=t.units&&t.units.length,y=p?t.defaultUnit||t.units[0]:"",S=[],i=t.step<1,l=1<t.step?t.step:1,z=i?Math.max(t.scale,(t.step+"").split(".")[1].length):1,O=Math.pow(10,z),b=Math.round(i?t.step*O:t.step),B=0,R=0,m=0;function U(e){return Math.max(r,Math.min(h,i?e<0?Math.ceil(e):Math.floor(e):L(Math.round(e-B),b)+B))}function I(e){return i?L((Math.abs(e)-Math.abs(U(e)))*O-R,b)+R:0}function A(e){var t=U(e),n=I(e);return O<=n&&(e<0?t--:t++,n=0),[e<0?"-":"+",t,n]}function V(e){var t=+e[c],n=i?e[g]/O*(t<0?-1:1):0;return(o&&"-"==e[0]?-1:1)*(t+n)}function L(t,e){return Math.round(t/e)*e}function _(e,n,s){return n!==s&&t.convert?t.convert.call(this,e,n,s):e}function D(s){var e,n;d=_(t.min,y,s),u=_(t.max,y,s),i?(r=d<0?Math.ceil(d):Math.floor(d),h=u<0?Math.ceil(u):Math.floor(u),F=I(d),P=I(u)):(r=Math.round(d),h=Math.round(u),h=r+Math.floor((h-r)/b)*b,B=r%b),e=r,n=h,o&&(n=Math.abs(e)>Math.abs(n)?Math.abs(e):Math.abs(n),e=e<0?0:e),x.min=e<0?Math.ceil(e/l):Math.floor(e/l),x.max=n<0?Math.ceil(n/l):Math.floor(n/l)}function $(e){return V(e).toFixed(i?z:0)+(p?" "+S[e[C]]:"")}if(w.setVal=function(t,n,s,o,i){w._setVal(e.isArray(t)?$(t):t,n,s,o,i)},t.units)for(s=0;s<t.units.length;++s)N=t.units[s],S.push(t.unitNames&&t.unitNames[N]||N);if(o)if(o=!1,p)for(s=0;s<t.units.length;s++)_(t.min,y,t.units[s])<0&&(o=!0);else o=t.min<0;if(o&&(k[0].push({data:["-","+"],label:t.signText}),m++),x={label:t.wholeText,data:function(e){return r%l+e*l},getIndex:function(e){return Math.round((e-r%l)/l)}},k[0].push(x),c=m++,D(y),i){for(k[0].push(M),M.data=[],M.label=t.fractionText,s=R;s<O;s+=b)H.push(s),M.data.push({value:s,display:"."+j(s,z)});g=m++,W=Math.ceil(100/b),t.invalid&&t.invalid.length&&(e.each(t.invalid,function(n,t){var e=0<t?Math.floor(t):Math.ceil(t);0===e&&(e=t<=0?-.001:.001),v[e]=(v[e]||0)+1,0===t&&(v[e=.001]=(v[e]||0)+1)}),e.each(v,function(e,t){t<W?delete v[e]:v[e]=e}))}if(p){for(T={data:[],label:t.unitText,cssClass:"mbsc-msr-whl-unit",circular:!1},s=0;s<t.units.length;s++)T.data.push({value:s,display:S[s]});k[0].push(T)}return C=m,{wheels:k,minWidth:o&&i?70:80,showLabel:!1,formatValue:$,compClass:"mbsc-msr mbsc-sc",parseValue:function(v){var n,b=((("number"==typeof v?v+"":v)||t.defaultValue)+"").split(" "),l=+b[0],m=[],s="";return p&&(s=-1==(s=-1==(s=e.inArray(b[1],S))?e.inArray(y,t.units):s)?0:s),D(E=p?t.units[s]:""),(n=A(l=f(l=isNaN(l)?0:l,d,u)))[1]=f(n[1],r,h),a=l,o&&(m[0]=n[0],n[1]=Math.abs(n[1])),m[c]=n[1],i&&(m[g]=n[2]),p&&(m[C]=s),m},onCancel:function(){a=void 0},validate:function(N){var s,S,M,T,z,n=N.values,O=N.index,$=N.direction,R={},j=[],I={},k=p?t.units[n[C]]:"";if(o&&0===O&&(a=Math.abs(a)*("-"==n[0]?-1:1)),(O===c||O===g&&i||void 0===a||void 0===O)&&(a=V(n),E=k),(p&&O===C&&E!==k||void 0===O)&&(D(k),a=_(a,E,k),E=k,s=A(a),void 0!==O&&(I[c]=x,w.changeWheel(I)),o&&(n[0]=s[0])),j[c]=[],o)for(j[0]=[],0<d&&(j[0].push("-"),n[0]="+"),u<0&&(j[0].push("+"),n[0]="-"),T=Math.abs("-"==n[0]?r:h),m=T+l;m<T+20*l;m+=l)j[c].push(m),R[m]=!0;if(a=f(a,d,u),s=A(a),z=o?Math.abs(s[1]):s[1],M=o?"-"==n[0]:a<0,n[c]=z,M&&(s[0]="-"),i&&(n[g]=s[2]),e.each(i?v:t.invalid,function(t,e){if(o&&M){if(!(e<=0))return;e=Math.abs(e)}e=L(_(e,y,k),i?1:b),R[e]=!0,j[c].push(e)}),n[c]=w.getValidValue(c,z,$,R),s[1]=n[c]*(o&&M?-1:1),i){j[g]=[];var B=o?n[0]+n[1]:(a<0?"-":"+")+Math.abs(s[1]),W=(d<0?"-":"+")+Math.abs(r),U=(u<0?"-":"+")+Math.abs(h);B===W&&e(H).each(function(t,e){(M?F<e:e<F)&&j[g].push(e)}),B===U&&e(H).each(function(t,e){(M?e<P:P<e)&&j[g].push(e)}),e.each(t.invalid,function(t,e){S=A(_(e,y,k)),(s[0]===S[0]||0===s[1]&&0===S[1]&&0===S[2])&&s[1]===S[1]&&j[g].push(S[2])})}return{disabled:j,valid:n}}}},n2={min:0,max:100,defaultUnit:"km",units:["m","km","in","ft","yd","mi"]},tl={mm:.001,cm:.01,dm:.1,m:1,dam:10,hm:100,km:1e3,in:.0254,ft:.3048,yd:.9144,ch:20.1168,fur:201.168,mi:1609.344,lea:4828.032},c.distance=function(e){var t=n({},n2,e.settings);return n(e.settings,t,{sign:!1,convert:function(e,t,n){return e*tl[t]/tl[n]}}),c.measurement.call(this,e)},nh={min:0,max:100,defaultUnit:"N",units:["N","kp","lbf","pdl"]},tu={N:1,kp:9.80665,lbf:4.448222,pdl:.138255},c.force=function(e){var t=n({},nh,e.settings);return n(e.settings,t,{sign:!1,convert:function(e,t,n){return e*tu[t]/tu[n]}}),c.measurement.call(this,e)},n8={min:0,max:1e3,defaultUnit:"kg",units:["g","kg","oz","lb"],unitNames:{tlong:"t (long)",tshort:"t (short)"}},t6={mg:.001,cg:.01,dg:.1,g:1,dag:10,hg:100,kg:1e3,t:1e6,drc:1.7718452,oz:28.3495,lb:453.59237,st:6350.29318,qtr:12700.58636,cwt:50802.34544,tlong:1016046.9088,tshort:907184.74},c.mass=function(e){var t=n({},n8,e.settings);return n(e.settings,t,{sign:!1,convert:function(e,t,n){return e*t6[t]/t6[n]}}),c.measurement.call(this,e)},n6={min:0,max:100,defaultUnit:"kph",units:["kph","mph","mps","fps","knot"],unitNames:{kph:"km/h",mph:"mi/h",mps:"m/s",fps:"ft/s",knot:"knot"}},t5={kph:1,mph:1.60934,mps:3.6,fps:1.09728,knot:1.852},c.speed=function(e){var t=n({},n6,e.settings);return n(e.settings,t,{sign:!1,convert:function(e,t,n){return e*t5[t]/t5[n]}}),c.measurement.call(this,e)},ng={min:-20,max:40,defaultUnit:"c",units:["c","k","f","r"],unitNames:{c:"°C",k:"K",f:"°F",r:"°R"}},n5={c2k:function(e){return e+273.15},c2f:function(e){return 9*e/5+32},c2r:function(e){return 9*(e+273.15)/5},k2c:function(e){return e-273.15},k2f:function(e){return 9*e/5-459.67},k2r:function(e){return 9*e/5},f2c:function(e){return 5*(e-32)/9},f2k:function(e){return 5*(e+459.67)/9},f2r:function(e){return e+459.67},r2c:function(e){return 5*(e-491.67)/9},r2k:function(e){return 5*e/9},r2f:function(e){return e-459.67}},c.temperature=function(e){var t=n({},ng,e.settings);return n(e.settings,t,{sign:!0,convert:function(e,t,n){return n5[t+"2"+n](e)}}),c.measurement.call(this,e)},r("measurement",d),r("distance",d),r("force",d),r("mass",d),r("speed",d),r("temperature",d);function e4(g,z,F){var o,i,a,r,c,d,u,h,m,b,j,O,x,C,E,k,M,_={},y=1e3,t=this,l=e(g);function f(e){clearTimeout(M),M=setTimeout(function(){w(!e||"load"!==e.type)},200)}function T(){b&&p(e(this),!0)}function p(e,l){if(e.length){if(l=t._onItemTap(e,l),(r=e).parent()[0]==g){var d=e.offset().left,n=e[0].offsetLeft,s=e[0].offsetWidth,f=a.offset().left;u&&(n=m-n-s),"a"==o.variant?d<f?h.scroll(u?n+s-i:-n,y,!0):f+i<d+s&&h.scroll(u?n:i-n-s,y,!0):h.scroll((i/2-n-s/2)*(u?-1:1),y,!0)}l&&c("onItemTap",{target:e[0]})}}function A(){var n;t._initMarkup(a),l.find(".mbsc-ripple").remove(),t._$items=l.children(),t._$items.each(function(a){var i,s=e(this),o=s.attr("data-ref"),o=o||nv++;0===a&&(n=s),r=r||t._getActiveItem(s),i="mbsc-scv-item mbsc-btn-e "+((t._getItemProps(s)||{}).cssClass||""),s.attr("data-ref",o).removeClass(_[o]).addClass(i),_[o]=i}),r=r||n,t._markupReady(a)}function w(r,c){var e=o.itemWidth,s=o.layout;if(t.contWidth=i=a.width(),t._checkResp())return!1;r&&C===i||!i||(C=i,v(s)&&(d=i?i/s:e)<e&&(s="liquid"),e&&("liquid"==s?d=i?i/Math.min(Math.floor(i/e),t._$items.length):e:"fixed"==s&&(d=e)),t._size(i,d),d&&l.children().css("width",d+"px"),t.totalWidth=m=g.offsetWidth,n(h.settings,{contSize:i,maxSnapScroll:!!o.paging&&1,maxScroll:0,minScroll:i<m?i-m:0,snap:o.paging?i:!!E&&(d||".mbsc-scv-item"),elastic:i<m&&(d||i)}),h.refresh(c))}S.call(this,g,z,!0),t.navigate=function(e,n){p(t._getItem(e),n)},t.next=function(t){if(r){var e=r.next();e.length&&p(r=e,t)}},t.prev=function(t){if(r){var e=r.prev();e.length&&p(r=e,t)}},t.refresh=t.position=function(e){A(),w(!1,e)},t._init=function(){var n;k=e(o.context),j=e("body"==o.context?window:o.context),t.__init(),u=o.rtl,E=!(!o.itemWidth||"fixed"==o.layout||void 0!==o.snap)||o.snap,n="mbsc-scv-c mbsc-no-touch mbsc-"+o.theme+" "+(o.cssClass||"")+" "+(o.wrapperClass||"")+(o.baseTheme?" mbsc-"+o.baseTheme:"")+(u?" mbsc-rtl":" mbsc-ltr")+(o.itemWidth?" mbsc-scv-hasw":"")+("body"==o.context?"":" mbsc-scv-ctx")+" "+(t._getContClass()||""),a?(a.attr("class",n),l.off(".mbsc-ripple")):((a=e('<div class="'+n+'"><div class="mbsc-scv-sc"></div></div>').on("click",".mbsc-scv-item",T).insertAfter(l)).find(".mbsc-scv-sc").append(l),a.find("img").on("load",f),j.on("orientationchange resize",f),x=nc(a[0],f,o.zone),h=new J(a[0],{axis:"X",contSize:0,maxScroll:0,maxSnapScroll:1,minScroll:0,snap:1,elastic:1,rtl:u,mousewheel:o.mousewheel,thresholdX:o.threshold,stopProp:o.stopProp,onStart:function(e){"touchstart"==e.domEvent.type&&(b=!1,O||(O=!0,k.find(".mbsc-no-touch").removeClass("mbsc-no-touch")))},onBtnTap:function(s){b=!0;var t=s.domEvent,n=t.target;"touchend"===t.type&&o.tap&&t4(n,ej(e(n)),t)},onGestureStart:function(e){c("onGestureStart",e)},onGestureEnd:function(e){c("onGestureEnd",e)},onMove:function(e){c("onMove",e)},onAnimationStart:function(e){c("onAnimationStart",e)},onAnimationEnd:function(e){c("onAnimationEnd",e)}})),l.css("display","").addClass("mbsc-scv").removeClass("mbsc-cloak"),A(),c("onMarkupReady",{target:a[0]}),w()},t._size=s,t._initMarkup=s,t._markupReady=s,t._getContClass=s,t._getItemProps=s,t._getActiveItem=s,t.__init=s,t.__destroy=s,t._destroy=function(){t.__destroy(),j.off("orientationchange resize",f),l.removeClass("mbsc-scv").insertAfter(a).find(".mbsc-scv-item").each(function(){var t=e(this);t.width("").removeClass(_[t.attr("data-ref")])}),a.remove(),h.destroy(),x.detach()},t._getItem=function(n){return"object"!==U(n)&&(n=t._$items.filter('[data-id="'+n+'"]')),e(n)},t._onItemTap=function(t,e){return void 0===e||e},o=t.settings,c=t.trigger,F||t.init()}nv=1,e4.prototype={_class:"scrollview",_hasDef:!0,_hasTheme:!0,_hasLang:!0,_responsive:!0,_defaults:{tap:et,stopProp:!1,context:"body",layout:"liquid"}},u.ScrollView=e4;function e5(u,p,f){var n,o,i,a,r,l,h=e(u),t=this;function m(){o&&"inline"!=o&&l.find(".mbsc-page").css("padding-"+o,"")}function d(e){e.addClass(r).attr("data-selected","true").attr("aria-selected","true")}function c(e){e.removeClass(r).removeAttr("data-selected").removeAttr("aria-selected")}e4.call(this,u,p,!0),t.select=function(e){i||c(t._$items.filter(".mbsc-ms-item-sel")),d(t._getItem(e))},t.deselect=function(e){c(t._getItem(e))},t.enable=function(e){t._getItem(e).removeClass("mbsc-disabled").removeAttr("data-disabled").removeAttr("aria-disabled")},t.disable=function(e){t._getItem(e).addClass("mbsc-disabled").attr("data-disabled","true").attr("aria-disabled","true")},t.setBadge=function(s,n){var o;s=t._getItem(s).attr("data-badge",n),(o=e(".mbsc-ms-badge",s)).length?n?o.html(n):o.remove():n&&s.append('<span class="mbsc-ms-badge">'+n+"</span>")},t._markupReady=function(e){t._hasIcons?e.addClass("mbsc-ms-icons"):e.removeClass("mbsc-ms-icons"),t._hasText?e.addClass("mbsc-ms-txt"):e.removeClass("mbsc-ms-txt"),t.__markupReady(e)},t._size=function(e,n){t.__size(e,n),"inline"!=o&&l.find(".mbsc-page").css("padding-"+o,u.offsetHeight+"px")},t._onItemTap=function(e,n){return!1!==t.__onItemTap(e,n)&&(void 0===n&&(n=!i),a&&n&&!e.hasClass("mbsc-disabled")&&(i?"true"==e.attr("data-selected")?c(e):d(e):(c(t._$items.filter(".mbsc-ms-item-sel")),d(e))),n)},t._getActiveItem=function(e){var t="true"==e.attr("data-selected");if(a&&!i&&t)return e},t._getItemProps=function(e){var o="true"==e.attr("data-selected"),i="true"==e.attr("data-disabled"),s=e.attr("data-icon"),a=e.attr("data-badge");return e.attr("data-role","button").attr("aria-selected",o?"true":"false").attr("aria-disabled",i?"true":"false").find(".mbsc-ms-badge").remove(),a&&e.append('<span class="mbsc-ms-badge">'+a+"</span>"),s&&(t._hasIcons=!0),e.text()&&(t._hasText=!0),{cssClass:"mbsc-ms-item "+(n.itemClass||"")+" "+(o?r:"")+(i?" mbsc-disabled "+(n.disabledClass||""):"")+(s?" mbsc-ms-ic mbsc-ic mbsc-ic-"+s:"")}},t._getContClass=function(){return" mbsc-ms-c mbsc-ms-"+n.variant+" mbsc-ms-"+o+(a?"":" mbsc-ms-nosel")+(t.__getContClass()||"")},t.__init=function(){t.___init(),l=e(n.context),m(),o=n.display,i="multiple"==n.select,a="off"!=n.select,r=" mbsc-ms-item-sel "+(n.activeClass||""),h.addClass("mbsc-ms mbsc-ms-base "+(n.groupClass||""))},t.__destroy=function(){h.removeClass("mbsc-ms mbsc-ms-base "+(n.groupClass||"")),m(),t.___destroy()},t.__onItemTap=s,t.__getContClass=s,t.__markupReady=s,t.__size=s,t.___init=s,t.___destroy=s,n=t.settings,f||t.init()}e5.prototype={_defaults:n({},e4.prototype._defaults)};function t9(e,t){e5.call(this,e,t,!0),this.___init=function(){},this.init()}t9.prototype={_class:"optionlist",_hasDef:!0,_hasTheme:!0,_hasLang:!0,_responsive:!0,_defaults:n({},e5.prototype._defaults,{select:"multiple",variant:"a",display:"inline"})},u.Optionlist=t9,t.themes.optionlist=t.themes.navigation,r("optionlist",t9,!1);function tK(c,d){var t,s,i,a,r,o=e(c),l=o.is("ul,ol"),n=this;e5.call(this,c,d,!0),n._initMarkup=function(){s&&s.remove(),a&&o.append(a.children())},n.__size=function(u,c){var l,h=c||72,m=n._$items.length,d=0;i.hide(),"bottom"==t.type&&(o.removeClass("mbsc-scv-liq"),s.remove(),n._$items.remove().each(function(n){var t=e(this);o.append(t),d+=c||this.offsetWidth||0,Math.round(d+(n<m-1?h:0))>u&&(l=!0,a.append(t.css("width","").addClass("mbsc-fr-btn-e")))}),s.attr("class",r+(t.moreIcon?" mbsc-menu-item-ic mbsc-ms-ic mbsc-ic mbsc-ic-"+t.moreIcon:"")).html(n._hasIcons&&n._hasText?t.moreText:""),l&&o.append(s)),"liquid"==t.layout&&o.addClass("mbsc-scv-liq")},n.__onItemTap=function(e){if(e.hasClass("mbsc-menu-item")&&!1!==n.trigger("onMenuShow",{target:e[0],menu:i}))return i.show(!1,!0),!1},n.__getContClass=function(){return"hamburger"==t.type?" mbsc-ms-hamburger":""},n.__markupReady=function(e){"hamburger"==t.type&&(a.append(n._$items.addClass("mbsc-fr-btn-e")),s.attr("class",r+(t.menuIcon?" mbsc-menu-item-ic mbsc-ms-ic mbsc-ic mbsc-ic-"+t.menuIcon:"")).html(t.menuText||""),o.append(s),t.menuText&&t.menuIcon||e.removeClass("mbsc-ms-icons"),t.menuText?e.addClass("mbsc-ms-txt"):e.removeClass("mbsc-ms-txt"))},n.___init=function(){var o;"tab"==t.type?(t.display=t.display||"top",t.variant=t.variant||"b"):"bottom"==t.type?(t.display=t.display||"bottom",t.variant=t.variant||"a"):"hamburger"==t.type&&(t.display=t.display||"inline",t.variant=t.variant||"a"),r="mbsc-scv-item mbsc-ms-item mbsc-btn-e mbsc-menu-item "+(t.itemClass||""),s||(s=e(l?"<li></li>":"<div></div>"),a=e(l?"<ul></ul>":"<div></div>").addClass("mbsc-scv mbsc-ms")),i=new w(a[0],{display:"bubble",theme:t.theme,lang:t.lang,context:t.context,buttons:[],anchor:s,onBeforeShow:function(t,e){o=null,e.settings.cssClass="mbsc-wdg mbsc-ms-a mbsc-ms-more"+(n._hasText?"":" mbsc-ms-more-icons")},onBeforeClose:function(){return n.trigger("onMenuHide",{target:o&&o[0],menu:i})},onMarkupReady:function(s,t){n.tap(t._markup.find(".mbsc-fr-c"),function(t){(o=e(t.target).closest(".mbsc-ms-item")).length&&!o.hasClass("mbsc-disabled")&&(n.navigate(o,!0),i.hide())})}})},n.___destroy=function(){i.destroy(),o.append(n._$items),s.remove()},t=n.settings,n.init()}tK.prototype={_class:"navigation",_hasDef:!0,_hasTheme:!0,_hasLang:!0,_responsive:!0,_defaults:n({},e5.prototype._defaults,{type:"bottom",moreText:"More",moreIcon:"material-more-horiz",menuIcon:"material-menu"})},r("nav",u.Navigation=tK,!1),c.number=c.measurement,r("number",d);function tr(u,V,W){var s,i,c,l,d,h,m,g,b,j,_,w,x,M,z,D,N,L,B,p=e(u),t=this,S=[],a=[],r={},T={},U={107:"+",109:"-"},k={48:0,49:1,50:2,51:3,52:4,53:5,54:6,55:7,56:8,57:9,96:0,97:1,98:2,99:3,100:4,101:5,102:6,103:7,104:8,105:9};function f(o){var n,a=s.validate.call(u,{values:i.slice(0),variables:r},t)||[],f=a&&a.disabled||[];if(t._isValid=!a.invalid,t._tempValue=s.formatValue.call(u,i.slice(0),r,t),c=i.length,w=a.length||d,t._isVisible){if(e(".mbsc-np-ph",l).each(function(t){e(this).html("ltr"==s.fill?c<=t?b:h||i[t]:d-w<=t?t+c<d?b:h||i[t+c-d]:"")}),e(".mbsc-np-cph",l).each(function(){e(this).html(r[e(this).attr("data-var")]||e(this).attr("data-ph"))}),c===d)for(n=0;n<=9;n++)f.push(n);for(e(".mbsc-np-btn",l).removeClass(m),n=0;n<f.length;n++)e('.mbsc-np-btn[data-val="'+f[n]+'"]',l).addClass(m);t._isValid?e(".mbsc-fr-btn-s .mbsc-fr-btn",l).removeClass(m):e(".mbsc-fr-btn-s .mbsc-fr-btn",l).addClass(m),t.live&&(t._hasValue=o||t._hasValue,y(o,!1,o),o&&x("onSet",{valueText:t._value}))}}function y(s,o,e,c){o&&f(),c||(g=i.slice(0),T=n({},r),S=a.slice(0),t._value=t._hasValue?t._tempValue:null),s&&(t._isInput&&p.val(t._hasValue&&t._isValid?t._value:""),x("onFill",{valueText:t._hasValue?t._tempValue:"",change:e}),e&&(t._preventChange=!0,p.trigger("change")))}function R(o){var e,t,n=o||[],s=[];for(a=[],r={},e=0;e<n.length;e++)/:/.test(n[e])?(t=n[e].split(":"),r[t[0]]=t[1],a.push(t[0])):(s.push(n[e]),a.push("digit"));return s}function P(e,t){!(c||t||s.allowLeadingZero)||e.hasClass("mbsc-disabled")||e.hasClass("mbsc-np-btn-empty")||c<d&&(a.push("digit"),i.push(t),f(!0))}function I(o){var e,t,n=o.attr("data-val"),d="false"!==o.attr("data-track"),l=o.attr("data-var");if(!o.hasClass("mbsc-disabled")){if(l&&(e=l.split(":"),d&&a.push(e[0]),r[e[0]]=void 0===e[2]?e[1]:r[e[0]]==e[1]?e[2]:e[1]),n.length+c<=w)for(t=0;t<n.length;++t)e=v(n[t])?+n[t]:n[t],(s.allowLeadingZero||c||e)&&(a.push("digit"),i.push(e),c=i.length);f(!0)}}function E(){var e,n,t=a.pop();if(c||"digit"!==t){if("digit"!==t&&r[t])for(delete r[t],n=a.slice(0),a=[],e=0;e<n.length;e++)n[e]!==t&&a.push(n[e]);else i.pop();f(!0)}}function F(){clearInterval(_),j=!1}function $(t){if(H(t,this)){if("keydown"==t.type&&32!=t.keyCode)return;!function(e){j=!0,B=o(e,"X"),L=o(e,"Y"),clearInterval(_),clearTimeout(_),E(),_=setInterval(function(){E()},150)}(t),"mousedown"==t.type&&e(document).on("mousemove",C).on("mouseup",A)}}function C(e){j&&(N=o(e,"X"),D=o(e,"Y"),z=N-B,M=D-L,(7<Math.abs(z)||7<Math.abs(M))&&F())}function A(t){j&&(t.preventDefault(),F(),"mouseup"==t.type&&e(document).off("mousemove",C).off("mouseup",A))}O.call(this,u,V,!0),t.setVal=t._setVal=function(n,o,a,r){t._hasValue=null!=n,i=R(e.isArray(n)?n.slice(0):s.parseValue.call(u,n,t)),y(o,!0,void 0===a?o:a,r)},t.getVal=t._getVal=function(e){return t._hasValue||e?t[e?"_tempValue":"_value"]:null},t.setArrayVal=t.setVal,t.getArrayVal=function(e){return e?i.slice(0):t._hasValue?g.slice(0):null},t._readValue=function(){var e=p.val()||"";""!==e&&(t._hasValue=!0),h?(r={},a=[],i=[]):(r=t._hasValue?T:{},a=t._hasValue?S:[],i=t._hasValue&&g?g.slice(0):R(s.parseValue.call(u,e,t)),y(!1,!0))},t._fillValue=function(){y(t._hasValue=!0,!1,!0)},t._generateContent=function(){var t,o,i,n=1,e="";for(e+='<div class="mbsc-np-hdr"><div role="button" tabindex="0" aria-label="'+s.deleteText+'" class="mbsc-np-del mbsc-fr-btn-e mbsc-ic mbsc-ic-'+s.deleteIcon+'"></div><div class="mbsc-np-dsp">',e+=s.template.replace(/d/g,'<span class="mbsc-np-ph">'+b+"</span>").replace(/&#100;/g,"d").replace(/{([a-zA-Z0-9]*):?([a-zA-Z0-9-_]*)}/g,'<span class="mbsc-np-cph" data-var="$1" data-ph="$2">$2</span>'),e+="</div></div>",e+='<div class="mbsc-np-tbl-c mbsc-w-p"><div class="mbsc-np-tbl">',o=0;o<4;o++){for(e+='<div class="mbsc-np-row">',i=0;i<3;i++)10==(t=n)||12==n?t="":11==n&&(t=0),""===t?10==n&&s.leftKey?e+='<div role="button" tabindex="0" class="mbsc-np-btn mbsc-np-btn-custom mbsc-fr-btn-e" '+(s.leftKey.variable?'data-var="'+s.leftKey.variable+'"':"")+' data-val="'+(s.leftKey.value||"")+'" '+(void 0!==s.leftKey.track?' data-track="'+s.leftKey.track+'"':"")+">"+s.leftKey.text+"</div>":12==n&&s.rightKey?e+='<div role="button" tabindex="0" class="mbsc-np-btn mbsc-np-btn-custom mbsc-fr-btn-e" '+(s.rightKey.variable?'data-var="'+s.rightKey.variable+'"':"")+' data-val="'+(s.rightKey.value||"")+'" '+(void 0!==s.rightKey.track?' data-track="'+s.rightKey.track+'"':"")+" >"+s.rightKey.text+"</div>":e+='<div class="mbsc-np-btn mbsc-np-btn-empty"></div>':e+='<div tabindex="0" role="button" class="mbsc-np-btn mbsc-fr-btn-e" data-val="'+t+'">'+t+"</div>",n++;e+="</div>"}return e+="</div></div>"},t._markupReady=function(){l=t._markup,f()},t._attachEvents=function(n){n.on("keydown",function(t){var s;void 0!==U[t.keyCode]?(s=e('.mbsc-np-btn[data-var="sign:-:"]',n)).length&&(r.sign=107==t.keyCode?"-":"",I(s)):void 0!==k[t.keyCode]?P(e('.mbsc-np-btn[data-val="'+k[t.keyCode]+'"]',n),k[t.keyCode]):8==t.keyCode&&(t.preventDefault(),E())}),t.tap(e(".mbsc-np-btn",n),function(){var t=e(this);t.hasClass("mbsc-np-btn-custom")?I(t):P(t,+t.attr("data-val"))},!1,30,!0),e(".mbsc-np-del",n).on("touchstart mousedown keydown",$).on("touchmove mousemove",C).on("touchend mouseup keyup",A)},t.__init=function(){(s=t.settings).template=s.template.replace(/\\d/,"&#100;"),b=s.placeholder,d=(s.template.match(/d/g)||[]).length,m="mbsc-disabled "+(s.disabledClass||""),h=s.mask,x=t.trigger,h&&p.is("input")&&p.attr("type","password")},t._indexOf=function(t,n){var e;for(e=0;e<t.length;++e)if(t[e].toString()===n.toString())return e;return-1},W||t.init()}es={},tr.prototype={_hasDef:!0,_hasTheme:!0,_hasLang:!0,_responsive:!0,_class:"numpad",_presets:es,_defaults:n({},O.prototype._defaults,{template:"dd.dd",placeholder:"0",deleteIcon:"backspace",allowLeadingZero:!1,headerText:!1,fill:"rtl",compClass:"mbsc-np",deleteText:"Delete",decimalSeparator:".",thousandsSeparator:",",validate:s,parseValue:s,formatValue:function(a,d,u){var n,i=1,c=u.settings,l=c.placeholder,s=c.template,r=a.length,o=s.length,t="";for(n=0;n<o;n++)"d"==s[o-n-1]?(t=i<=r?a[r-i]+t:l+t,i++):t=s[o-n-1]+t;return e.each(d,function(e,n){t=t.replace("{"+e+"}",n)}),e("<div>"+t+"</div>").text()}})},u.Numpad=tr,t.themes.numpad=t.themes.frame,n7={min:0,max:99.99,scale:2,prefix:"",suffix:"",returnAffix:!1};function tc(n){for(var e=0,t=1,s=0;n.length;)3<e?t=3600:1<e&&(t=60),s+=n.pop()*t*(e%2?10:1),e++;return s}es.decimal=function(s){function a(s,i){for(var t,n=s.slice(0),e=0;n.length;)e=10*e+n.shift();for(t=0;t<o;t++)e/=10;return i?-1*e:e}function c(n){var e=a(n).toFixed(o).split(".");return e[0].replace(/\B(?=(\d{3})+(?!\d))/g,t.thousandsSeparator)+(e[1]?t.decimalSeparator+e[1]:"")}var u=n({},s.settings),t=n(s.settings,n7,u),o=t.scale,i=+t.min.toFixed(o),r=+t.max.toFixed(o),d=i<0,l=new RegExp(t.thousandsSeparator,"g");return s.setVal=function(e,t,n,o){return s._setVal(f(e,i,r),t,n,o)},s.getVal=function(o){var e=s._getVal(o),n=(e+"").replace(l,"").replace(t.decimalSeparator,".");return v(n)?+n:e},{template:(d?"{sign}":"")+t.prefix.replace(/d/g,"\\d")+Array((Math.floor(Math.max(r,Math.abs(i)))+"").length+1).join("d")+(o?"."+Array(o+1).join("d"):"")+t.suffix.replace(/d/g,"\\d"),leftKey:d?{text:"-/+",variable:"sign:-:",track:!1}:void 0,parseValue:function(a){var e,n,i=a||t.defaultValue,s=[];if(i&&(e=(i=(i+"").replace(l,"").replace(t.decimalSeparator,".")).match(/\d+\.?\d*/g)))for(e=(+e[0]).toFixed(o),n=0;n<e.length;n++)"."!=e[n]&&(+e[n]?s.push(+e[n]):s.length&&s.push(0));return a<0&&s.push("sign:-"),s},formatValue:function(e,n){var s=c(e);return(a(e,n&&"-"==n.sign)<0?"-":"")+(t.returnAffix?t.prefix+s+t.suffix:s)},validate:function(n){var o=n.values,u=c(o),l=a(o,n.variables&&"-"==n.variables.sign),d=[];return o.length||t.allowLeadingZero||d.push(0),s.isVisible()&&e(".mbsc-np-dsp",s._markup).html((n.variables.sign||"")+t.prefix+u+t.suffix),{disabled:d,invalid:r<l||l<i||!!t.invalid&&-1!=s._indexOf(t.invalid,l)}}}},te=["h","m","s"],ne={min:0,max:362439,defaultValue:0,hourTextShort:"h",minuteTextShort:"m",secTextShort:"s"},es.timespan=function(s){var r=n({},s.settings),t=n(s.settings,ne,r),i={h:t.hourTextShort.replace(/d/g,"\\d"),m:t.minuteTextShort.replace(/d/g,"\\d"),s:t.secTextShort.replace(/d/g,"\\d")},o='d<span class="mbsc-np-sup mbsc-np-time">'+i.s+"</span>";function a(o){var n,t="",s=3600;return e(te).each(function(a,e){n=Math.floor(o/s),o-=n*s,s/=60,(0<n||"s"==e&&!t)&&(t=t+(t?" ":"")+n+i[e])}),t}return 9<t.max&&(o="d"+o),99<t.max&&(o='<span class="mbsc-np-ts-m">'+(639<t.max?"d":"")+'d</span><span class="mbsc-np-sup mbsc-np-time">'+i.m+"</span>"+o),6039<t.max&&(o='<span class="mbsc-np-ts-h">'+(38439<t.max?"d":"")+'d</span><span class="mbsc-np-sup mbsc-np-time">'+i.h+"</span>"+o),s.setVal=function(e,t,n,o){return v(e)&&(e=a(e)),s._setVal(e,t,n,o)},s.getVal=function(e){return s._hasValue||e?tc(s.getArrayVal(e)):null},{template:o,parseValue:function(r){var s,o=r||a(t.defaultValue),n=[];return o&&e(te).each(function(t,e){(s=new RegExp("(\\d+)"+i[e],"gi").exec(o))?9<(s=+s[1])?(n.push(Math.floor(s/10)),n.push(s%10)):(n.length&&n.push(0),(s||n.length)&&n.push(s)):n.length&&(n.push(0),n.push(0))}),n},formatValue:function(e){return a(tc(e))},validate:function(i){var n=i.values,e=tc(n.slice(0)),o=[];return n.length||o.push(0),{disabled:o,invalid:e>t.max||e<t.min||!!t.invalid&&-1!=s._indexOf(t.invalid,+e)}}}},tZ={timeFormat:"hh:ii A",amText:"am",pmText:"pm"},es.time=function(u){var g=n({},u.settings),t=n(u.settings,tZ,g),c=t.timeFormat.split(":"),s=t.timeFormat.match(/a/i),f=s?"a"==s[0]?t.amText:t.amText.toUpperCase():"",m=s?"a"==s[0]?t.pmText:t.pmText.toUpperCase():"",h=0,o=t.min?""+t.min.getHours():"",i=t.max?""+t.max.getHours():"",d=t.min?""+(t.min.getMinutes()<10?"0"+t.min.getMinutes():t.min.getMinutes()):"",a=t.max?""+(t.max.getMinutes()<10?"0"+t.max.getMinutes():t.max.getMinutes()):"",l=t.min?""+(t.min.getSeconds()<10?"0"+t.min.getSeconds():t.min.getSeconds()):"",r=t.max?""+(t.max.getSeconds()<10?"0"+t.max.getSeconds():t.max.getSeconds()):"";function p(n,o){var t,s="";for(t=0;t<n.length;++t)s+=n[t]+(t%2==(n.length%2==1?0:1)&&t!=n.length-1?":":"");return e.each(o,function(t,e){s+=" "+e}),s}return t.min&&t.min.setFullYear(2014,7,20),t.max&&t.max.setFullYear(2014,7,20),{placeholder:"-",allowLeadingZero:!0,template:(3==c.length?"dd:dd:dd":2==c.length?"dd:dd":"dd")+(s?'<span class="mbsc-np-sup">{ampm:--}</span>':""),leftKey:s?{text:f,variable:"ampm:"+f,value:"00"}:{text:":00",value:"00"},rightKey:s?{text:m,variable:"ampm:"+m,value:"00"}:{text:":30",value:"30"},parseValue:function(a){var e,n,o=a||t.defaultValue,i=[];if(o){if(n=(o+="").match(/\d/g))for(e=0;e<n.length;e++)i.push(+n[e]);s&&i.push("ampm:"+(o.match(new RegExp(t.pmText,"gi"))?m:f))}return i},formatValue:function(e,t){return p(e,t)},validate:function(m){var e=m.values,f=p(e,m.variables),n=3<=e.length?new Date(2014,7,20,""+e[0]+(e.length%2==0?e[1]:""),""+e[e.length%2==0?2:1]+e[e.length%2==0?3:2]):"";return{disabled:function(n){var e,m,f,g,v,b,j,y,_,w,u=[],p=2*c.length;if(h=p,n.length||(s&&(u.push(0),u.push(t.leftKey.value)),u.push(t.rightKey.value)),!s&&(p-n.length<2||1!=n[0]&&(2<n[0]||3<n[1])&&p-n.length<=2)&&(u.push("30"),u.push("00")),(s?1<n[0]||2<n[1]:1!=n[0]&&(2<n[0]||3<n[1]))&&n[0]&&(n.unshift(0),h=p-1),n.length==p)for(e=0;e<=9;++e)u.push(e);else if(1==n.length&&s&&1==n[0]||n.length&&n.length%2==0||!s&&2==n[0]&&3<n[1]&&n.length%2==1)for(e=6;e<=9;++e)u.push(e);if(y=void 0!==n[1]?""+n[0]+n[1]:"",_=+a==+(void 0!==n[3]?""+n[2]+n[3]:""),t.invalid)for(e=0;e<t.invalid.length;++e)if(w=t.invalid[e].getHours(),f=t.invalid[e].getMinutes(),m=t.invalid[e].getSeconds(),w==+y){if(2==c.length&&(f<10?0:+(""+f)[0])==+n[2]){u.push(f<10?f:+(""+f)[1]);break}if((m<10?0:+(""+m)[0])==+n[4]){u.push(m<10?m:+(""+m)[1]);break}}if(t.min||t.max){if(g=(b=+i==+y)&&_,v=(j=+o==+y)&&_,0===n.length){for(e=s?2:19<o?o[0]:3;e<=(1==o[0]?9:o[0]-1);++e)u.push(e);if(10<=o&&(u.push(0),2==o[0]))for(e=3;e<=9;++e)u.push(e);if(i&&i<10||o&&10<=o)for(e=i&&i<10?+i[0]+1:0;e<(o&&10<=o?o[0]:10);++e)u.push(e)}if(1==n.length){if(0===n[0])for(e=0;e<o[0];++e)u.push(e);if(o&&0!==n[0]&&(s?1==n[0]:2==n[0]))for(e=s?3:4;e<=9;++e)u.push(e);if(n[0]==o[0])for(e=0;e<o[1];++e)u.push(e);if(n[0]==i[0]&&!s)for(e=+i[1]+1;e<=9;++e)u.push(e)}if(2==n.length&&(j||b))for(e=b?+a[0]+1:0;e<(j?+d[0]:10);++e)u.push(e);if(3==n.length&&(b&&n[2]==a[0]||j&&n[2]==d[0]))for(e=b&&n[2]==a[0]?+a[1]+1:0;e<(j&&n[2]==d[0]?+d[1]:10);++e)u.push(e);if(4==n.length&&(v||g))for(e=g?+r[0]+1:0;e<(v?+l[0]:10);++e)u.push(e);if(5==n.length&&(v&&n[4]==l[0]||g&&n[4]==r[0]))for(e=g&&n[4]==r[0]?+r[1]+1:0;e<(v&&n[4]==l[0]?+l[1]:10);++e)u.push(e)}return u}(e),length:h,invalid:(s?!new RegExp("^(0?[1-9]|1[012])(:[0-5]\\d)?(:[0-5][0-9]) (?:"+t.amText+"|"+t.pmText+")$","i").test(f):!/^([0-1]?[0-9]|2[0-4]):([0-5][0-9])(:[0-5][0-9])?$/.test(f))||!!t.invalid&&-1!=u._indexOf(t.invalid,n)||!(!t.min||t.min<=n)||!(!t.max||n<=t.max)}}}},nj={dateOrder:"mdy",dateFormat:"mm/dd/yy",delimiter:"/"},r("numpad",tr,!(es.date=function(d){var s,o,i,h,u=[],b=n({},d.settings),t=n(d.settings,ev,nj,b),a=t.dateOrder,g=t.min?""+(t.getMonth(t.min)+1):0,m=t.max?""+(t.getMonth(t.max)+1):0,p=t.min?""+t.getDay(t.min):0,f=t.max?""+t.getDay(t.max):0,r=t.min?""+t.getYear(t.min):0,c=t.max?""+t.getYear(t.max):0;for(a=(a=(a=a.replace(/y+/gi,"yyyy")).replace(/m+/gi,"mm")).replace(/d+/gi,"dd"),s=a.toUpperCase().indexOf("Y"),o=a.toUpperCase().indexOf("M"),i=a.toUpperCase().indexOf("D"),a="",u.push({val:s,n:"yyyy"},{val:o,n:"mm"},{val:i,n:"dd"}),u.sort(function(e,t){return e.val-t.val}),e.each(u,function(t,e){a+=e.n}),s=a.indexOf("y"),o=a.indexOf("m"),i=a.indexOf("d"),a="",h=0;h<8;++h)a+="d",h+1!=s&&h+1!=o&&h+1!=i||(a+=t.delimiter);function v(e){return new Date(+(""+e[s]+e[s+1]+e[s+2]+e[s+3]),""+e[o]+e[o+1]-1,+(""+e[i]+e[i+1]))}return d.getVal=function(e){return d._hasValue||e?v(d.getArrayVal(e)):null},{placeholder:"-",fill:"ltr",allowLeadingZero:!0,template:a,parseValue:function(i){var e,n=[],o=i||t.defaultValue,s=Z(t.dateFormat,o,t);if(o)for(e=0;e<u.length;++e)n=/m/i.test(u[e].n)?n.concat(((t.getMonth(s)<9?"0":"")+(t.getMonth(s)+1)).split("")):/d/i.test(u[e].n)?n.concat(((t.getDay(s)<10?"0":"")+t.getDay(s)).split("")):n.concat((t.getYear(s)+"").split(""));return n},formatValue:function(e){return l(t.dateFormat,v(e),t)},validate:function(a){var n=a.values,e=v(n);return{disabled:function(n){var e,l,h,v,j,w,a=[],d=void 0!==n[s+3]?""+n[s]+n[s+1]+n[s+2]+n[s+3]:"",u=void 0!==n[o+1]?""+n[o]+n[o+1]:"",b=void 0!==n[i+1]?""+n[i]+n[i+1]:"",O=""+t.getMaxDayOfMonth(d||2012,u-1||0),y=r===d&&+g==+u,_=c===d&&+m==+u;if(t.invalid)for(e=0;e<t.invalid.length;++e){if(j=t.getYear(t.invalid[e]),h=t.getMonth(t.invalid[e]),v=t.getDay(t.invalid[e]),j==+d&&h+1==+u&&(v<10?0:+(""+v)[0])==+n[i]){a.push(v<10?v:+(""+v)[1]);break}if(h+1==+u&&v==+b&&(""+j).substring(0,3)==""+n[s]+n[s+1]+n[s+2]){a.push((""+j)[3]);break}if(j==+d&&v==+b&&(h<10?0:+(""+(h+1))[0])==+n[o]){a.push(h<10?h:+(""+(h+1))[1]);break}}if("31"!=b||n.length!=o&&n.length!=o+1||(1!=n[o]?a.push(2,4,6,9,11):a.push(1)),"30"==b&&0===n[o]&&n.length<=o+1&&a.push(2),n.length==o){for(e=c===d&&+m<10?1:2;e<=9;++e)a.push(e);r===d&&10<=+g&&a.push(0)}if(n.length==o+1){if(1==n[o]){for(e=c===d?+m[1]+1:3;e<=9;++e)a.push(e);if(r==d)for(e=0;e<+g[1];++e)a.push(e)}if(0===n[o]&&(a.push(0),c===d||r===d))for(e=c===d?+f<+b?+m:+m+1:0;e<=(r===d?g-1:9);++e)a.push(e)}if(n.length==i){for(e=_?1+(10<+f?+f[0]:0):+O[0]+1;e<=9;++e)a.push(e);if(y)for(e=0;e<(+p<10?0:p[0]);++e)a.push(e)}if(n.length==i+1){if(3<=n[i]||"02"==u)for(e=+O[1]+1;e<=9;++e)a.push(e);if(_&&+f[0]==n[i])for(e=+f[1]+1;e<=9;++e)a.push(e);if(y&&p[0]==n[i])for(e=0;e<+p[1];++e)a.push(e);if(0===n[i]&&(a.push(0),_||y))for(e=_?+f+1:1;e<=(y?p-1:9);++e)a.push(e)}if(void 0!==n[s+2]&&"02"==u&&"29"==b)for(l=+(""+n[s]+n[s+1]+n[s+2]+0);l<=+(""+n[s]+n[s+1]+n[s+2]+9);++l)a.push((w=l)%4==0&&w%100!=0||w%400==0?"":l%10);if(n.length==s){if(t.min)for(e=0;e<+r[0];++e)a.push(e);if(t.max)for(e=+c[0]+1;e<=9;++e)a.push(e);a.push(0)}if(t.min||t.max)for(l=1;l<4;++l)if(n.length==s+l){if(n[s+l-1]==+r[l-1]&&(3!=l||n[s+l-2]==+r[l-2]))for(e=0;e<+r[l]+(3==l&&n[o+1]&&+u<+g?1:0);++e)a.push(e);if(n[s+l-1]==+c[l-1]&&(3!=l||n[s+l-2]==+c[l-2]))for(e=+c[l]+(3==l&&+m<+u?0:1);e<=9;++e)a.push(e)}return a}(n),invalid:!("Invalid Date"!=e&&(!t.min||t.min<=e)&&(!t.max||e<=t.max))||!!t.invalid&&-1!=d._indexOf(t.invalid,e)}}}})),tX={autoCorrect:!0,showSelector:!0,minRange:1,rangeTap:!0},c.range=function(s){function $(e,t){e&&(e.setFullYear(t.getFullYear()),e.setMonth(t.getMonth()),e.setDate(t.getDate()))}function v(o,t){var n=s._order,e=new Date(o);return void 0===n.h&&e.setHours(t?23:0),void 0===n.i&&e.setMinutes(t?59:0),void 0===n.s&&e.setSeconds(t?59:0),e.setMilliseconds(t?999:0),e}function E(e,t){return new Date(e.getFullYear(),e.getMonth(),e.getDate()+t)}function P(e){R?(o-i>t.maxRange-1&&(e?i=new Date(Math.max(_,o-t.maxRange+1)):o=new Date(Math.min(w,+i+t.maxRange-1))),o-i<t.minRange-1&&(e?i=new Date(Math.max(_,o-t.minRange+1)):o=new Date(Math.min(w,+i+t.minRange-1)))):(Math.ceil((o-i)/C)>F&&(e?i=v(Math.max(_,E(o,1-F)),!1):o=v(Math.min(w,E(i,F-1)),!0)),Math.ceil((o-i)/C)<z&&(e?i=v(Math.max(_,E(o,1-z)),!1):o=v(Math.min(w,E(i,z-1)),!0)))}function T(t,n){var e=!0;return t&&i&&o&&(P(r),P(!r)),i&&o||(e=!1),n&&B(),e}function D(){L&&d&&(e(".mbsc-range-btn",d).removeClass(k).removeAttr("aria-checked"),function(e){e.addClass(k).attr("aria-checked","true")}(e(".mbsc-range-btn",d).eq(r)))}function B(){var n,a,u,m,f,p=0,g=h||!r?" mbsc-cal-day-hl mbsc-cal-sel-start":" mbsc-cal-sel-start",v=h||r?" mbsc-cal-day-hl mbsc-cal-sel-end":" mbsc-cal-sel-end";if(s.startVal=i?l(c,i,t):"",s.endVal=o?l(c,o,t):"",d&&(e(".mbsc-range-btn-v-start",d).html(s.startVal||"&nbsp;"),e(".mbsc-range-btn-v-end",d).html(s.endVal||"&nbsp;"),n=i?new Date(i):null,a=o?new Date(o):null,!n&&a&&(n=new Date(a)),!a&&n&&(a=new Date(n)),u=r?a:n,e(".mbsc-cal-day-picker .mbsc-cal-day-hl",d).removeClass(A),e(".mbsc-cal-day-picker .mbsc-selected",d).removeClass("mbsc-cal-sel-start mbsc-cal-sel-end "+k).removeAttr("aria-selected"),n&&a))for(m=n.setHours(0,0,0,0),f=a.setHours(0,0,0,0);n<=a&&p<126;)e('.mbsc-cal-day[data-full="'+u.getFullYear()+"-"+(u.getMonth()+1)+"-"+u.getDate()+'"]',d).addClass(k+" "+(u.getTime()===m?g:"")+(u.getTime()===f?v:"")).attr("aria-selected","true"),u.setDate(u.getDate()+(r?-1:1)),u.setHours(0,0,0,0),p++}function S(e,t){return{h:e?e.getHours():t?23:0,i:e?e.getMinutes():t?59:0,s:e?e.getSeconds():t?59:0}}function V(){i&&(p=!0,s.setDate(i,!1,0,!0),i=s.getDate(!0)),o&&(p=!0,s.setDate(o,!1,0,!0),o=s.getDate(!0))}var o,i,c,d,p,g,b,j,y,_,w,O,N,L,R,f=s._startDate,m=s._endDate,r=0,x=new Date,W=n({},s.settings),t=n(s.settings,tX,W),M=t.anchor,h=t.rangeTap,C=864e5,z=Math.max(1,Math.ceil(t.minRange/C)),F=Math.max(1,Math.ceil(t.maxRange/C)),I="mbsc-disabled "+(t.disabledClass||""),k="mbsc-selected "+(t.selectedClass||""),A="mbsc-cal-day-hl",u=null===t.defaultValue?[]:t.defaultValue||[new Date(x.setHours(0,0,0,0)),new Date(x.getFullYear(),x.getMonth(),x.getDate()+6,23,59,59,999)];return h&&(t.tabs=!0),b=eZ.call(this,s),c=s._format,R=/time/i.test(t.controls.join(",")),N="time"===t.controls.join(""),L=t.showSelector,w=t.max?v(a(t.max,c,t),!0):1/0,_=t.min?v(a(t.min,c,t),!1):-1/0,u[0]=a(u[0],c,t,t.isoParts),u[1]=a(u[1],c,t,t.isoParts),t.startInput&&s.attachShow(e(t.startInput),function(){r=0,t.anchor=M||e(t.startInput)}),t.endInput&&s.attachShow(e(t.endInput),function(){r=1,t.anchor=M||e(t.endInput)}),s._getDayProps=function(e,s){var t=i?new Date(i.getFullYear(),i.getMonth(),i.getDate()):null,n=o?new Date(o.getFullYear(),o.getMonth(),o.getDate()):null;return{selected:t&&n&&t<=e&&e<=o,cssClass:s.cssClass+" "+((h||!r)&&t&&t.getTime()===e.getTime()||(h||r)&&n&&n.getTime()===e.getTime()?A:"")+(t&&t.getTime()===e.getTime()?" mbsc-cal-sel-start":"")+(n&&n.getTime()===e.getTime()?" mbsc-cal-sel-end":"")}},s.setVal=function(g,u,h,n,p){var d,e=g||[];i=a(e[0],c,t,t.isoParts),o=a(e[1],c,t,t.isoParts),V(),s.startVal=i?l(c,i,t):"",s.endVal=o?l(c,o,t):"",d=b.parseValue(r?o:i,s),n||(s._startDate=f=i,s._endDate=m=o),y=!0,s._setVal(d,u,h,n,p)},s.getVal=function(e){return e?[K(i,t,c),K(o,t,c)]:s._hasValue?[K(f,t,c),K(m,t,c)]:null},s.setActiveDate=function(n){var t;r="start"==n?0:1,t="start"==n?i:o,s.isVisible()&&(D(),h||(e(".mbsc-cal-table .mbsc-cal-day-hl",d).removeClass(A),t&&e('.mbsc-cal-day[data-full="'+t.getFullYear()+"-"+(t.getMonth()+1)+"-"+t.getDate()+'"]',d).addClass(A)),t&&(p=!0,s.setDate(t,!1,1e3,!0)))},s.getValue=s.getVal,n({},b,{highlight:!1,outerMonthChange:!1,formatValue:function(){return s.startVal+(t.endInput?"":s.endVal?" - "+s.endVal:"")},parseValue:function(n){var o=n?n.split(" - "):[],i=t.startInput?e(t.startInput).val():o[0],a=t.endInput?e(t.endInput).val():o[1];return t.defaultValue=u[1],m=a?Z(c,a,t):u[1],t.defaultValue=u[0],f=i?Z(c,i,t):u[2],t.defaultValue=u[r],s.startVal=f?l(c,f,t):"",s.endVal=m?l(c,m,t):"",s._startDate=f,s._endDate=m,b.parseValue(r?m:f,s)},onFill:function(n){!function(n){s._startDate=f=i,s._endDate=m=o,t.startInput&&(e(t.startInput).val(s.startVal),n&&e(t.startInput).trigger("change")),t.endInput&&(e(t.endInput).val(s.endVal),n&&e(t.endInput).trigger("change"))}(n.change)},onBeforeClose:function(e){if("set"===e.button&&!T(!0,!0))return s.setActiveDate(r?"start":"end"),!1},onHide:function(){b.onHide.call(s),r=0,d=null,t.anchor=M},onClear:function(){h&&(r=0)},onBeforeShow:function(){i=f||u[0],o=m||u[1],O=S(i,0),j=S(o,1),t.counter&&(t.headerText=function(){var e=i&&o?Math.max(1,Math.round((new Date(o).setHours(0,0,0,0)-new Date(i).setHours(0,0,0,0))/864e5)+1):0;return(1<e&&t.selectedPluralText||t.selectedText).replace(/{count}/,e)}),y=!0},onMarkupReady:function(a){var n;V(),(r&&o||!r&&i)&&(p=!0,s.setDate(r?o:i,!1,0,!0)),B(),b.onMarkupReady.call(this,a),(d=e(a.target)).addClass("mbsc-range"),L&&(n='<div class="mbsc-range-btn-t" role="radiogroup"><div class="mbsc-range-btn-c mbsc-range-btn-start"><div role="radio" data-select="start" class="mbsc-fr-btn-e mbsc-fr-btn-nhl mbsc-range-btn">'+t.fromText+'<div class="mbsc-range-btn-v mbsc-range-btn-v-start">'+(s.startVal||"&nbsp;")+'</div></div></div><div class="mbsc-range-btn-c mbsc-range-btn-end"><div role="radio" data-select="end" class="mbsc-fr-btn-e mbsc-fr-btn-nhl mbsc-range-btn">'+t.toText+'<div class="mbsc-range-btn-v mbsc-range-btn-v-end">'+(s.endVal||"&nbsp;")+"</div></div></div></div>",t.headerText?e(".mbsc-fr-hdr",d).after(n):e(".mbsc-fr-w",d).prepend(n),D()),e(".mbsc-range-btn",d).on("touchstart click",function(t){H(t,this)&&(s._showDayPicker(),s.setActiveDate(e(this).attr("data-select")))})},onDayChange:function(e){e.active=r?"end":"start",g=!0},onSetDate:function(a){var n;p||(n=v(a.date,r),y&&!g||(h&&g&&(1==r&&n<i&&(r=0),r?n.setHours(j.h,j.i,j.s,999):n.setHours(O.h,O.i,O.s,0)),r?(o=new Date(n),j=S(o)):(i=new Date(n),O=S(i)),N&&t.autoCorrect&&($(i,n),$(o,n)),h&&g&&!r&&(o=null))),N&&!t.autoCorrect&&o<i&&(o=new Date(o.setDate(o.getDate()+1))),s._isValid=T(y||g||t.autoCorrect,!p),a.active=r?"end":"start",!p&&h&&(g&&(r=r?0:1),D()),s.isVisible()&&(s._isValid?e(".mbsc-fr-btn-s .mbsc-fr-btn",s._markup).removeClass(I):e(".mbsc-fr-btn-s .mbsc-fr-btn",s._markup).addClass(I)),p=y=g=!1},onTabChange:function(e){"calendar"!=e.tab&&s.setDate(r?o:i,!1,1e3,!0),T(!0,!0)}})},r("range",d),r("scroller",d,!1),r("scrollview",e4,!1),tW={inputClass:"",rtl:!1,showInput:!0,groupLabel:"Groups",dataHtml:"html",dataText:"text",dataValue:"value",dataGroup:"group",dataDisabled:"disabled",filterPlaceholderText:"Type to filter",filterEmptyText:"No results",filterClearIcon:"material-close"},c.select=function(o,es){var i,r,l,d,h,m,g,b,O,x,F,T,z,D,ee,J="",Z={},q=1e3,k=this,a=e(k),eo=n({},o.settings),t=n(o.settings,tW,eo),A=e('<div class="mbsc-sel-empty">'+t.filterEmptyText+"</div>"),ei=t.readonly,c={},B=t.layout||(/top|bottom|inline/.test(t.display)||t.filter?"liquid":""),W="liquid"==B||!t.touchUi,f=v(t.select)?t.select:"multiple"==t.select||a.prop("multiple"),u=f||!(!t.filter&&!t.tapSelect)&&1,S=this.id+"_dummy",V=e('label[for="'+this.id+'"]').attr("for",S),ea=void 0!==t.label?t.label:V.length?V.text():a.attr("name"),_=t.group,E=!!t.data,y=E?!!t.group:e("optgroup",a).length,p=y&&_&&!1!==_.groupWheel,j=y&&_&&p&&!0===_.clustered,N=y&&(!_||!1!==_.header&&!j),C=a.val()||(f?[]:[""]),w=[];function I(r){var n,s,o,i,m,f,p,u=0,l=0,h={};c={},g={},d=[],D=[],w.length=0,E?e.each(x,function(a,e){f=e[t.dataText]+"",p=e[t.dataHtml],s=e[t.dataValue],i=e[t.dataGroup],n={value:s,html:p,text:f,index:a,cssClass:N?"mbsc-sel-gr-itm":""},c[s]=n,r&&!P(f,r)||(d.push(n),y&&(void 0===h[i]?(o={text:i,value:l,options:[],index:l},g[l]=o,h[i]=l,D.push(o),l++):o=g[h[i]],j&&(n.index=o.options.length),n.group=h[i],o.options.push(n)),e[t.dataDisabled]&&w.push(s))}):y?(m=!0,e("optgroup",a).each(function(t){g[t]={text:this.label,value:t,options:[],index:t},m=!0,e("option",this).each(function(e){n={value:this.value,text:this.text,index:j?e:u++,group:t,cssClass:N?"mbsc-sel-gr-itm":""},c[this.value]=n,r&&!P(this.text,r)||(m&&(D.push(g[t]),m=!1),d.push(n),g[t].options.push(n),this.disabled&&w.push(this.value))})})):e("option",a).each(function(e){n={value:this.value,text:this.text,index:e},c[this.value]=n,r&&!P(this.text,r)||(d.push(n),this.disabled&&w.push(this.value))}),J=t.defaultValue?t.defaultValue:d.length?d[0].value:"",N&&(d=[],u=0,e.each(g,function(o,t){t.options.length&&(s="__group"+o,n={text:t.text,value:s,group:o,index:u++,cssClass:"mbsc-sel-gr"},c[s]=n,d.push(n),w.push(n.value),e.each(t.options,function(t,e){e.index=u++,d.push(e)}))})),A&&(d.length?A.removeClass("mbsc-sel-empty-v"):A.addClass("mbsc-sel-empty-v"))}function G(t,s,i,n,a){var e,o=[];for(e=0;e<t.length;e++)o.push({value:t[e].value,display:t[e].html||t[e].text,cssClass:t[e].cssClass});return{circular:!1,multiple:s&&!n?1:n,cssClass:(s&&!n?"mbsc-sel-one":"")+" "+a,data:o,label:i}}function X(){return G(j&&g[h]?g[h].options:d,u,ea,f,"")}function Q(){var e=[[]];return p&&(z=G(D,u,t.groupLabel,!1,"mbsc-sel-gr-whl"),W?e[0][r]=z:e[r]=[z]),T=X(),W?e[0][i]=T:e[i]=[T],e}function M(t){f&&(t&&L(t)&&(t=t.split(",")),e.isArray(t)&&(t=t[0])),!c[l=null==t||""===t?J:t]&&d&&d.length&&(l=d[0].value),p&&(h=c[l]?c[l].group:null)}function H(e){return Z[e]||(c[e]?c[e].text:"")}function R(){var n,s="",e=o.getVal(),i=t.formatValue.call(k,o.getArrayVal(),o,!0);if(t.filter&&"inline"==t.display||m.val(i),a.is("select")&&E){if(f)for(n=0;n<e.length;n++)s+='<option value="'+e[n]+'">'+H(e[n])+"</option>";else s='<option value="'+(null===e?"":e)+'">'+i+"</option>";a.html(s)}k!==m[0]&&a.val(e)}function et(){var e={};e[i]=X(),F=!0,o.changeWheel(e)}function P(t,e){return e=e.replace(/[-/\\^$*+?.()|[\]{}]/g,"\\$&"),t.match(new RegExp(e,"ig"))}function en(e){return t.data.dataField?e[t.data.dataField]:t.data.processResponse?t.data.processResponse(e):e}function K(n){var e={};I(n),M(l),t.wheels=Q(),e[i]=T,o._tempWheelArray[i]=l,p&&(e[r]=z,o._tempWheelArray[r]=h),o.changeWheel(e,0,!0),R()}function U(e){return o.trigger("onFilter",{filterText:e})}function Y(e){e[r]!=h&&(h=e[r],l=g[h].options[0].value,e[i]=l,j?et():o.setArrayVal(e,!1,!1,!0,q))}return o.setVal=function(e,a,l,t,d){if(u&&(null==e||f||(e=[e]),e&&L(e)&&(e=e.split(",")),o._tempSelected[i]=$(e),t||(o._selected[i]=$(e)),e=e?e[0]:null,p)){var n=c[e],s=n&&n.group;o._tempSelected[r]=$([s]),t||(o._selected[r]=$([s]))}o._setVal(e,a,l,t,d)},o.getVal=function(t,n){var e;return e=u?(e=eH(t?o._tempSelected[i]:o._selected[i]),f?e:e.length?e[0]:null):(e=t?o._tempWheelArray:o._hasValue?o._wheelArray:null)?e[i]:null,f?e:void 0!==e?y&&n?[c[e]?c[e].group:null,e]:e:null},o.refresh=function(n,i,o){o=o||s,n?(x=n,O||(t.data=n)):e.isArray(t.data)&&(x=t.data),!n&&O&&void 0===i?to(t.data.url,function(e){x=en(e),K(),o()},t.data.dataType):(K(i),o())},es.invalid||(t.invalid=w),i=p?(r=0,1):(r=-1,0),u&&(f&&a.prop("multiple",!0),C&&L(C)&&(C=C.split(",")),o._selected[i]=$(C)),o._$input&&o._$input.remove(),a.next().is(".mbsc-select-input")?m=a.next().removeAttr("tabindex"):t.input?m=e(t.input):(t.filter&&"inline"==t.display?o._$input=e('<div class="mbsc-sel-input-wrap"><input type="text" id="'+S+'" class="mbsc-select-input mbsc-control '+t.inputClass+'" readonly /></div>'):(m=e('<input type="text" id="'+S+'" class="mbsc-select-input mbsc-control '+t.inputClass+'" readonly />'),o._$input=m),t.showInput&&(o._$input.insertAfter(a),m=m||o._$input.find("#"+S))),o.attachShow(m.attr("placeholder",t.placeholder||"")),m[0]!==k&&(a.addClass("mbsc-sel-hdn").attr("tabindex",-1),t.showInput||a.attr("data-enhance",!1)),!u||t.rows%2||(t.rows=t.rows-1),t.filter&&(ee=t.filter.minLength||0),(O=t.data&&t.data.url)?o.refresh():(E&&(x=t.data),I(),M(a.val())),{layout:B,headerText:!1,anchor:m,compClass:"mbsc-sc mbsc-sel"+(u?" mbsc-sel-multi":""),setOnTap:!p||[!1,!0],formatValue:function(s,e,o){var t,n=[],a=o?e._selected:e._tempSelected;if(u){for(t in a[i])n.push(H(t));return n.join(", ")}return H(s[i])},tapSelect:u,parseValue:function(e){return M(void 0===e?a.val():e),p?[h,l]:[l]},validate:function(n){var s=n.index,e=[];return e[i]=t.invalid,j&&!F&&void 0===s&&et(),F=!1,{disabled:e}},onRead:R,onFill:R,onMarkupReady:function(c,o){if(t.filter){var n,i,r,a=e(".mbsc-fr-w",c.target),s=e('<span class="mbsc-sel-filter-clear mbsc-ic mbsc-ic-'+t.filterClearIcon+'"></span>');"inline"==t.display?(i=m).parent().find(".mbsc-sel-filter-clear").remove():(a.find(".mbsc-fr-c").before('<div class="mbsc-input mbsc-sel-filter-cont mbsc-control-w mbsc-'+t.theme+(t.baseTheme?" mbsc-"+t.baseTheme:"")+'"><span class="mbsc-input-wrap"><input tabindex="0" type="text" class="mbsc-sel-filter-input mbsc-control"/></span></div>'),i=a.find(".mbsc-sel-filter-input")),b=null,n=i[0],i.prop("readonly",!1).attr("placeholder",t.filterPlaceholderText).parent().append(s),a.find(".mbsc-fr-c").prepend(A),o._activeElm=n,o.tap(s,function(){b=null,n.value="",o.refresh(),s.removeClass("mbsc-sel-filter-show-clear"),U("")}),i.on("keydown",function(e){13!=e.keyCode&&27!=e.keyCode||(e.stopPropagation(),n.blur())}).on("input",function(){clearTimeout(r),n.value.length?s.addClass("mbsc-sel-filter-show-clear"):s.removeClass("mbsc-sel-filter-show-clear"),r=setTimeout(function(){b!==n.value&&!1!==U(n.value)&&((b=n.value).length>=ee||!b.length)&&(O&&t.data.remoteFilter?to(t.data.url+encodeURIComponent(b),function(e){o.refresh(en(e))},t.data.dataType):o.refresh(void 0,b))},500)})}},onBeforeShow:function(){f&&t.counter&&(t.headerText=function(){var n=0;return e.each(o._tempSelected[i],function(){n++}),(1<n&&t.selectedPluralText||t.selectedText).replace(/{count}/,n)}),M(a.val()),u&&p&&(o._selected[r]=$([h])),t.filter&&I(void 0),o.settings.wheels=Q(),F=!0},onWheelGestureStart:function(e){e.index==r&&(t.readonly=[!1,!0])},onWheelAnimationEnd:function(n){var e=o.getArrayVal(!0);n.index==r?(t.readonly=ei,u||Y(e)):n.index==i&&e[i]!=l&&(l=e[i],p&&c[l]&&c[l].group!=h&&(h=c[l].group,e[r]=h,o._tempSelected[r]=$([h]),o.setArrayVal(e,!1,!1,!0,q)))},onItemTap:function(e){var t;if(e.index==i&&(Z[e.value]=c[e.value].text,u&&!f&&e.selected))return!1;if(e.index==r&&u){if(e.selected)return!1;(t=o.getArrayVal(!0))[r]=e.value,Y(t)}},onClose:function(){O&&t.data.remoteFilter&&b&&o.refresh()},onDestroy:function(){o._$input&&o._$input.remove(),a.removeClass("mbsc-sel-hdn").removeAttr("tabindex")}}},r("select",d),tz={autostart:!1,step:1,useShortLabels:!1,labels:["Years","Months","Days","Hours","Minutes","Seconds",""],labelsShort:["Yrs","Mths","Days","Hrs","Mins","Secs",""],startText:"Start",stopText:"Stop",resetText:"Reset",lapText:"Lap",hideText:"Hide",mode:"countdown"},c.timer=function(t){function N(e){return new Date(e.getUTCFullYear(),e.getUTCMonth(),e.getUTCDate(),e.getUTCHours(),e.getUTCMinutes(),e.getUTCSeconds(),e.getUTCMilliseconds())}function D(e,t,n){return(t||"")+(e<10?"0":"")+e+'<span class="mbsc-timer-lbl">'+n+"</span>"}function b(s){var t,n=[],o=function(h){if(t={},M&&r[u].index>r.days.index){var t,o,i,a,m,f=new Date,n=c?f:d,s=c?d:f;for(s=N(s),n=N(n),t.years=n.getFullYear()-s.getFullYear(),t.months=n.getMonth()-s.getMonth(),t.days=n.getDate()-s.getDate(),t.hours=n.getHours()-s.getHours(),t.minutes=n.getMinutes()-s.getMinutes(),t.seconds=n.getSeconds()-s.getSeconds(),t.fract=(n.getMilliseconds()-s.getMilliseconds())/10,a=l.length;0<a;a--)o=l[a-1],m=r[o],i=l[e.inArray(o,l)-1],r[i]&&t[o]<0&&(t[i]--,t[o]+="months"==i?32-new Date(n.getFullYear(),n.getMonth(),32).getDate():m.until+1);"months"==u&&(t.months+=12*t.years,delete t.years)}else e(l).each(function(n,e){r[e].index<=r[u].index&&(t[e]=Math.floor(h/r[e].limit),h-=t[e]*r[e].limit)});return t}(s);return e(l).each(function(s,e){v[e]&&(t=Math.max(Math.round(C/r[e].limit),1),n.push(Math.round(o[e]/t)*t))}),n}function S(e){M?(i=d-new Date,c=i<0&&(i*=-1,!0),w=!(o=0)):(void 0!==d?(w=!1,i=1e3*d,c="countdown"!=s.mode):(i=0,c="countdown"!=s.mode,w=c),e&&(o=0))}function E(){h?(e(".mbsc-fr-w",a).addClass("mbsc-timer-running mbsc-timer-locked"),e(".mbsc-timer-btn-toggle-c > div",a).text(s.stopText),t.buttons.start.icon&&e(".mbsc-timer-btn-toggle-c > div",a).removeClass("mbsc-ic-"+t.buttons.start.icon),t.buttons.stop.icon&&e(".mbsc-timer-btn-toggle-c > div",a).addClass("mbsc-ic-"+t.buttons.stop.icon),"stopwatch"==s.mode&&(e(".mbsc-timer-btn-resetlap-c > div",a).text(s.lapText),t.buttons.reset.icon&&e(".mbsc-timer-btn-resetlap-c > div",a).removeClass("mbsc-ic-"+t.buttons.reset.icon),t.buttons.lap.icon&&e(".mbsc-timer-btn-resetlap-c > div",a).addClass("mbsc-ic-"+t.buttons.lap.icon))):(e(".mbsc-fr-w",a).removeClass("mbsc-timer-running"),e(".mbsc-timer-btn-toggle-c > div",a).text(s.startText),t.buttons.start.icon&&e(".mbsc-timer-btn-toggle-c > div",a).addClass("mbsc-ic-"+t.buttons.start.icon),t.buttons.stop.icon&&e(".mbsc-timer-btn-toggle-c > div",a).removeClass("mbsc-ic-"+t.buttons.stop.icon),"stopwatch"==s.mode&&(e(".mbsc-timer-btn-resetlap-c > div",a).text(s.resetText),t.buttons.reset.icon&&e(".mbsc-timer-btn-resetlap-c > div",a).addClass("mbsc-ic-"+t.buttons.reset.icon),t.buttons.lap.icon&&e(".mbsc-timer-btn-resetlap-c > div",a).removeClass("mbsc-ic-"+t.buttons.lap.icon)))}var o,i,a,c,p,g,j,y,_,x,z,L=n({},t.settings),s=n(t.settings,tz,L),m=s.useShortLabels?s.labelsShort:s.labels,T=["resetlap","toggle"],l=["years","months","days","hours","minutes","seconds","fract"],r={years:{index:6,until:10,limit:31536e6,label:m[0],wheel:{}},months:{index:5,until:11,limit:2592e6,label:m[1],wheel:{}},days:{index:4,until:31,limit:864e5,label:m[2],wheel:{}},hours:{index:3,until:23,limit:36e5,label:m[3],wheel:{}},minutes:{index:2,until:59,limit:6e4,label:m[4],wheel:{}},seconds:{index:1,until:59,limit:1e3,label:m[5],wheel:{}},fract:{index:0,until:99,limit:10,label:m[6],prefix:".",wheel:{}}},v={},k=[],A=0,h=!1,f=!0,w=!1,C=Math.max(10,1e3*s.step),u=s.maxWheel,O="stopwatch"==s.mode||M,d=s.targetTime,M=d&&void 0!==d.getTime,F=[[]];return t.start=function(){if(f&&t.reset(),!h){if(S(),!w&&i<=o)return;f=!(h=!0),j=new Date,p=o,s.readonly=!0,t.setVal(b(c?o:i-o),!0,!0,!1,100),x=setInterval(function(){o=new Date-j+p,t.setVal(b(c?o:i-o),!0,!0,!1,Math.min(100,g-10)),!w&&i<=o+g&&(clearInterval(x),setTimeout(function(){t.stop(),o=i,t.setVal(b(c?o:0),!0,!0,!1,100),t.trigger("onFinish",{time:i}),f=!0},i-o))},g),E(),t.trigger("onStart")}},t.stop=function(){h&&(h=!1,clearInterval(x),o=new Date-j+p,E(),t.trigger("onStop",{ellapsed:o}))},t.toggle=function(){h?t.stop():t.start()},t.reset=function(){t.stop(),k=[],A=o=0,t.setVal(b(c?0:i),!0,!0,!1,1e3),t.settings.readonly=O,f=!0,O||e(".mbsc-fr-w",a).removeClass("mbsc-timer-locked"),t.trigger("onReset")},t.lap=function(){h&&(_=new Date-j+p,z=_-A,A=_,k.push(_),t.trigger("onLap",{ellapsed:_,lap:z,laps:k}))},t.resetlap=function(){h&&"stopwatch"==s.mode?t.lap():t.reset()},t.getTime=function(){return i},t.setTime=function(e){d=e/1e3,i=e},t.getEllapsedTime=function(){return f?0:h?new Date-j+p:o},t.setEllapsedTime=function(e,n){f||(p=o=e,j=new Date,t.setVal(b(c?o:i-o),!0,n,!1,1e3))},S(!0),u||i||(u="minutes"),"inline"!==s.display&&T.unshift("hide"),u||e(l).each(function(t,e){if(!u&&i>=r[e].limit)return u=e,!1}),e(l).each(function(n,t){!function(s){var o=1,t=r[s],n=t.wheel,i=t.prefix,c=t.until,a=r[l[e.inArray(s,l)-1]];if(t.index<=r[u].index&&(!a||a.limit>C))if(v[s]||F[0].push(n),v[s]=1,n.data=[],n.label=t.label||"",n.cssClass="mbsc-timer-whl-"+s,C>=t.limit&&(o=Math.max(Math.round(C/t.limit),1),g=o*t.limit),s==u)n.min=0,n.data=function(e){return{value:e,display:D(e,i,t.label)}},n.getIndex=function(e){return e};else for(y=0;y<=c;y+=o)n.data.push({value:y,display:D(y,i,t.label)})}(t)}),g=Math.max(97,g),s.autostart&&setTimeout(function(){t.start()},0),t.handlers.toggle=t.toggle,t.handlers.start=t.start,t.handlers.stop=t.stop,t.handlers.resetlap=t.resetlap,t.handlers.reset=t.reset,t.handlers.lap=t.lap,t.buttons.toggle={parentClass:"mbsc-timer-btn-toggle-c",text:s.startText,icon:s.startIcon,handler:"toggle"},t.buttons.start={text:s.startText,icon:s.startIcon,handler:"start"},t.buttons.stop={text:s.stopText,icon:s.stopIcon,handler:"stop"},t.buttons.reset={text:s.resetText,icon:s.resetIcon,handler:"reset"},t.buttons.lap={text:s.lapText,icon:s.lapIcon,handler:"lap"},t.buttons.resetlap={parentClass:"mbsc-timer-btn-resetlap-c",text:s.resetText,icon:s.resetIcon,handler:"resetlap"},t.buttons.hide={parentClass:"mbsc-timer-btn-hide-c",text:s.hideText,icon:s.closeIcon,handler:"cancel"},{minWidth:100,wheels:F,headerText:!1,readonly:O,buttons:T,compClass:"mbsc-timer mbsc-sc",parseValue:function(){return b(c?0:i)},formatValue:function(n){var s="",t=0;return e(l).each(function(o,e){"fract"!=e&&v[e]&&(s+=n[t]+("seconds"==e&&v.fract?"."+n[t+1]:"")+" "+m[o]+" ",t++)}),s},validate:function(t){var s=t.values,o=t.index,n=0;f&&void 0!==o&&(d=0,e(l).each(function(t,e){v[e]&&(d+=r[e].limit*s[n],n++)}),d/=1e3,S(!0))},onBeforeShow:function(){s.showLabel=!0},onMarkupReady:function(t){a=e(t.target),E(),O&&e(".mbsc-fr-w",a).addClass("mbsc-timer-locked")},onPosition:function(t){e(".mbsc-fr-w",t.target).css("min-width",0).css("min-width",e(".mbsc-fr-btn-cont",t.target)[0].offsetWidth)},onDestroy:function(){clearInterval(x)}}},r("timer",d),tR={wheelOrder:"hhiiss",useShortLabels:!1,min:0,max:1/0,labels:["Years","Months","Days","Hours","Minutes","Seconds"],labelsShort:["Yrs","Mths","Days","Hrs","Mins","Secs"]};function i(e){return e<-1e-7?Math.ceil(e-1e-7):Math.floor(e+1e-7)}function tt(t,e,s){t=parseInt(t),e=parseInt(e),s=parseInt(s);var n,o,r,c,a=new Array(0,0,0);return c=1582<t||1582==t&&10<e||1582==t&&10==e&&14<s?i(1461*(t+4800+i((e-14)/12))/4)+i(367*(e-2-12*i((e-14)/12))/12)-i(3*i((t+4900+i((e-14)/12))/100)/4)+s-32075:367*t-i(7*(t+5001+i((e-9)/7))/4)+i(275*e/9)+s+1729777,r=i(((n=c-1948440+10632)-1)/10631),o=i((10985-(n=n-10631*r+354))/5316)*i(50*n/17719)+i(n/5670)*i(43*n/15238),n=n-i((30-o)/15)*i(17719*o/50)-i(o/16)*i(15238*o/43)+29,e=i(24*n/709),s=n-i(709*e/24),t=30*r+o-30,a[2]=s,a[1]=e,a[0]=t,a}c.timespan=function(i){function u(n){var t={};return e(m).each(function(o,e){t[e]=r[e]?Math.floor(n/s[e].limit):0,n-=t[e]*s[e].limit}),t}function f(e,t,n){return(e<10&&t?"0":"")+e+'<span class="mbsc-ts-lbl">'+n+"</span>"}function b(t){var n=0;return e.each(o,function(e,o){isNaN(+t[0])||(n+=s[o.v].limit*t[e])}),n}var c,l,h,p,y,w=n({},i.settings),t=n(i.settings,tR,w),g=t.wheelOrder,a=t.useShortLabels?t.labelsShort:t.labels,m=["years","months","days","hours","minutes","seconds"],s={years:{ord:0,index:6,until:10,limit:31536e6,label:a[0],re:"y",wheel:{}},months:{ord:1,index:5,until:11,limit:2592e6,label:a[1],re:"m",wheel:{}},days:{ord:2,index:4,until:31,limit:864e5,label:a[2],re:"d",wheel:{}},hours:{ord:3,index:3,until:23,limit:36e5,label:a[3],re:"h",wheel:{}},minutes:{ord:4,index:2,until:59,limit:6e4,label:a[4],re:"i",wheel:{}},seconds:{ord:5,index:1,until:59,limit:1e3,label:a[5],re:"s",wheel:{}}},o=[],j=t.steps||[],r={},d="seconds",O=t.defaultValue||Math.max(t.min,Math.min(0,t.max)),_=[[]];return e(m).each(function(t,e){-1<(y=g.search(new RegExp(s[e].re,"i")))&&(o.push({o:y,v:e}),s[e].index>s[d].index&&(d=e))}),o.sort(function(e,t){return e.o>t.o?1:-1}),e.each(o,function(t,e){r[e.v]=t+1,_[0].push(s[e.v].wheel)}),h=u(t.min),l=u(t.max),e.each(o,function(t,e){!function(n){var i=!1,o=j[r[n]-1]||1,t=s[n],a=t.label,e=t.wheel;if(e.data=[],e.label=t.label,g.match(new RegExp(t.re+t.re,"i"))&&(i=!0),n==d)e.min=h[n],e.max=l[n],e.data=function(e){return{value:e*o,display:f(e*o,i,a)}},e.getIndex=function(e){return Math.round(e/o)};else for(c=0;c<=t.until;c+=o)e.data.push({value:c,display:f(c,i,a)})}(e.v)}),i.getVal=function(e,t){return t?i._getVal(e):i._hasValue||e?b(i.getArrayVal(e)):null},{minWidth:100,showLabel:!0,wheels:_,compClass:"mbsc-ts mbsc-sc",parseValue:function(i){var a,n=[];return v(i)||!i?(p=u(i||O),e.each(o,function(t,e){n.push(p[e.v])})):e.each(o,function(o,e){a=new RegExp("(\\d+)\\s?("+t.labels[s[e.v].ord]+"|"+t.labelsShort[s[e.v].ord]+")","gi").exec(i),n.push(a?a[1]:0)}),e(n).each(function(e,t){n[e]=function(t,e){return Math.floor(t/e)*e}(t,j[e]||1)}),n},formatValue:function(n){var t="";return e.each(o,function(e,o){t+=+n[e]?n[e]+" "+s[o.v].label+" ":""}),t?t.replace(/\s+$/g,""):0},validate:function(v){var t,n,o,a,c=v.values,j=v.direction,f=[],p=!0,g=!0;return e(m).each(function(v,m){if(void 0!==r[m]){if(n=r[m]-1,f[n]=[],o={},m!=d){if(p)for(t=l[m]+1;t<=s[m].until;t++)o[t]=!0;if(g)for(t=0;t<h[m];t++)o[t]=!0}c[n]=i.getValidValue(n,c[n],j,o),a=u(b(c)),p=p&&a[m]==l[m],g=g&&a[m]==h[m],e.each(o,function(e){f[n].push(e)})}}),{disabled:f}}}},r("timespan",d),c.treelist=ni,r("treelist",d),r("popup",w,!1),r("widget",tP,!1),eG.hijri={getYear:function(e){return tt(e.getFullYear(),e.getMonth()+1,e.getDate())[0]},getMonth:function(e){return--tt(e.getFullYear(),e.getMonth()+1,e.getDate())[1]},getDay:function(e){return tt(e.getFullYear(),e.getMonth()+1,e.getDate())[2]},getDate:function(t,e,s,o,a,r,c){e<0&&(t+=Math.floor(e/12),e=12+e%12),11<e&&(t+=Math.floor(e/12),e%=12);var n=function(o,n,a){o=parseInt(o),n=parseInt(n),a=parseInt(a);var e,t,s,r,l,d,c=new Array(3);return o=2299160<(d=i((11*o+3)/30)+354*o+30*n-i((n-1)/2)+a+1948440-385)?(r=i(4*(e=68569+d)/146097),e-=i((146097*r+3)/4),s=i(4e3*(e+1)/1461001),e=e-i(1461*s/4)+31,t=i(80*e/2447),a=e-i(2447*t/80),n=t+2-12*(e=i(t/11)),100*(r-49)+s+e):(l=i(((t=1402+d)-1)/1461),r=i(((e=t-1461*l)-1)/365)-i(e/1461),t=i(80*(s=e-365*r+30)/2447),a=s-i(2447*t/80),n=t+2-12*(s=i(t/11)),4*l+r+s-4716),c[2]=a,c[1]=n,c[0]=o,c}(t,+e+1,s);return new Date(n[0],n[1]-1,n[2],o||0,a||0,r||0,c||0)},getMaxDayOfMonth:function(t,e){return[30,29,30,29,30,29,30,29,30,29,30,29][e]+(11===e&&(11*t+14)%30<11?1:0)}},t.i18n.ar={rtl:!0,setText:"تعيين",cancelText:"إلغاء",clearText:"مسح",selectedText:"{count} المحدد",dateFormat:"dd/mm/yy",dayNames:["الأحد","الاثنين","الثلاثاء","الأربعاء","الخميس","الجمعة","السبت"],dayNamesShort:["أحد","اثنين","ثلاثاء","أربعاء","خميس","جمعة","سبت"],dayNamesMin:["ح","ن","ث","ر","خ","ج","س"],dayText:"يوم",hourText:"ساعات",minuteText:"الدقائق",monthNames:["كانون الثاني","شهر فبراير","مارس","أبريل","قد","يونيو","يوليو","أغسطس","سبتمبر","شهر اكتوبر","شهر نوفمبر","ديسمبر"],monthNamesShort:["كانون الثاني","شهر فبراير","مارس","أبريل","قد","يونيو","يوليو","أغسطس","سبتمبر","شهر اكتوبر","شهر نوفمبر","ديسمبر"],monthText:"شهر",secText:"ثواني",amText:"ص",pmText:"م",timeFormat:"hh:ii A",yearText:"عام",nowText:"الآن",firstDay:0,dateText:"تاريخ",timeText:"وقت",closeText:"إغلاق",todayText:"اليوم",prevMonthText:"الشهر السابق",nextMonthText:"الشهر القادم",prevYearText:"السنه السابقة",nextYearText:"العام القادم",allDayText:"اليوم كله",noEventsText:"لا توجد احداث",eventText:"الحدث",eventsText:"أحداث",moreEventsText:"واحد آخر",moreEventsPluralText:"اثنان آخران {count}",fromText:"يبدا",toText:"ينتهي",wholeText:"كامل",fractionText:"جزء",unitText:"وحدة",delimiter:"/",decimalSeparator:".",thousandsSeparator:",",labels:["سنوات","أشهر","أيام","ساعة","دقائق","ثواني",""],labelsShort:["سنوات","أشهر","أيام","ساعة","دقائق","ثواني",""],startText:"بدء",stopText:"إيقاف",resetText:"إعادة ضبط",lapText:"الدورة",hideText:"إخفاء",offText:"إيقاف",onText:"تشغيل",backText:"رجوع",undoText:"تراجع"},t.i18n.bg={setText:"Задаване",cancelText:"Отмяна",clearText:"Изчистване",selectedText:"{count} подбран",dateFormat:"dd.mm.yy",dayNames:["Неделя","Понеделник","Вторник","Сряда","Четвъртък","Петък","Събота"],dayNamesShort:["Нед","Пон","Вто","Сря","Чет","Пет","Съб"],dayNamesMin:["Не","По","Вт","Ср","Че","Пе","Съ"],dayText:"ден",delimiter:".",hourText:"час",minuteText:"минута",monthNames:["Януари","Февруари","Март","Април","Май","Юни","Юли","Август","Септември","Октомври","Ноември","Декември"],monthNamesShort:["Яну","Фев","Мар","Апр","Май","Юни","Юли","Авг","Сеп","Окт","Нов","Дек"],monthText:"месец",secText:"секунди",timeFormat:"H:ii",yearText:"година",nowText:"Сега",pmText:"pm",amText:"am",firstDay:1,dateText:"Дата",timeText:"път",todayText:"днес",prevMonthText:"Предишния месец",nextMonthText:"Следващият месец",prevYearText:"Предходната година",nextYearText:"Следващата година",closeText:"затвори",eventText:"Събитие",eventsText:"Събития",allDayText:"Цял ден",noEventsText:"Няма събития",moreEventsText:"Още {count}",fromText:"ОТ",toText:"ДО",wholeText:"цяло",fractionText:"фракция",unitText:"единица",labels:["Години","месеца","дни","часа","минути","секунди",""],labelsShort:["Години","месеца","дни","часа","минути","секунди",""],startText:"Старт",stopText:"Стоп",resetText:"Нулиране",lapText:"Обиколка",hideText:"крия",backText:"връщане",undoText:"ОТМЯНА",offText:"ИЗКЛ",onText:"ВКЛ",decimalSeparator:",",thousandsSeparator:" "},t.i18n.ca={setText:"Acceptar",cancelText:"Cancel·lar",clearText:"Esborrar",selectedText:"{count} seleccionat",selectedPluralText:"{count} seleccionats",dateFormat:"dd/mm/yy",dayNames:["Diumenge","Dilluns","Dimarts","Dimecres","Dijous","Divendres","Dissabte"],dayNamesShort:["Dg","Dl","Dt","Dc","Dj","Dv","Ds"],dayNamesMin:["Dg","Dl","Dt","Dc","Dj","Dv","Ds"],dayText:"Dia",hourText:"Hores",minuteText:"Minuts",monthNames:["Gener","Febrer","Març","Abril","Maig","Juny","Juliol","Agost","Setembre","Octubre","Novembre","Desembre"],monthNamesShort:["Gen","Feb","Mar","Abr","Mai","Jun","Jul","Ago","Set","Oct","Nov","Des"],monthText:"Mes",secText:"Segons",timeFormat:"HH:ii",yearText:"Any",nowText:"Ara",pmText:"pm",amText:"am",todayText:"Avui",firstDay:1,dateText:"Data",timeText:"Temps",closeText:"Tancar",allDayText:"Tot el dia",noEventsText:"Cap esdeveniment",eventText:"Esdeveniments",eventsText:"Esdeveniments",moreEventsText:"{count} més",fromText:"Iniciar",toText:"Final",wholeText:"Sencer",fractionText:"Fracció",unitText:"Unitat",labels:["Anys","Mesos","Dies","Hores","Minuts","Segons",""],labelsShort:["Anys","Mesos","Dies","Hrs","Mins","Secs",""],startText:"Iniciar",stopText:"Aturar",resetText:"Reiniciar",lapText:"Volta",hideText:"Amagar",backText:"Enrere",undoText:"Desfés",offText:"No",onText:"Si"},t.i18n.cs={setText:"Zadej",cancelText:"Storno",clearText:"Vymazat",selectedText:"Označený: {count}",dateFormat:"dd.mm.yy",dayNames:["Neděle","Pondělí","Úterý","Středa","Čtvrtek","Pátek","Sobota"],dayNamesShort:["Ne","Po","Út","St","Čt","Pá","So"],dayNamesMin:["N","P","Ú","S","Č","P","S"],dayText:"Den",hourText:"Hodiny",minuteText:"Minuty",monthNames:["Leden","Únor","Březen","Duben","Květen","Červen","Červenec","Srpen","Září","Říjen","Listopad","Prosinec"],monthNamesShort:["Led","Úno","Bře","Dub","Kvě","Čer","Čvc","Spr","Zář","Říj","Lis","Pro"],monthText:"Měsíc",secText:"Sekundy",timeFormat:"HH:ii",yearText:"Rok",nowText:"Teď",amText:"am",pmText:"pm",todayText:"Dnes",firstDay:1,dateText:"Datum",timeText:"Čas",closeText:"Zavřít",allDayText:"Celý den",noEventsText:"Žádné události",eventText:"Událostí",eventsText:"Události",moreEventsText:"{count} další",fromText:"Začátek",toText:"Konec",wholeText:"Celý",fractionText:"Část",unitText:"Jednotka",labels:["Roky","Měsíce","Dny","Hodiny","Minuty","Sekundy",""],labelsShort:["Rok","Měs","Dny","Hod","Min","Sec",""],startText:"Start",stopText:"Stop",resetText:"Resetovat",lapText:"Etapa",hideText:"Schovat",backText:"Zpět",undoText:"Zpět",offText:"O",onText:"I",decimalSeparator:",",thousandsSeparator:" "},t.i18n.da={setText:"Sæt",cancelText:"Annuller",clearText:"Ryd",selectedText:"{count} valgt",selectedPluralText:"{count} valgt",dateFormat:"dd/mm/yy",dayNames:["Søndag","Mandag","Tirsdag","Onsdag","Torsdag","Fredag","Lørdag"],dayNamesShort:["Søn","Man","Tir","Ons","Tor","Fre","Lør"],dayNamesMin:["S","M","T","O","T","F","L"],dayText:"Dag",hourText:"Timer",minuteText:"Minutter",monthNames:["Januar","Februar","Marts","April","Maj","Juni","Juli","August","September","Oktober","November","December"],monthNamesShort:["Jan","Feb","Mar","Apr","Maj","Jun","Jul","Aug","Sep","Okt","Nov","Dec"],monthText:"Måned",secText:"Sekunder",amText:"am",pmText:"pm",timeFormat:"HH.ii",yearText:"År",nowText:"Nu",todayText:"I dag",firstDay:1,dateText:"Dato",timeText:"Tid",closeText:"Luk",allDayText:"Hele dagen",noEventsText:"Ingen begivenheder",eventText:"Begivenheder",eventsText:"Begivenheder",moreEventsText:"{count} mere",fromText:"Start",toText:"Slut",wholeText:"Hele",fractionText:"Dele",unitText:"Enhed",labels:["År","Måneder","Dage","Timer","Minutter","Sekunder",""],labelsShort:["År","Mdr","Dg","Timer","Min","Sek",""],startText:"Start",stopText:"Stop",resetText:"Nulstil",lapText:"Omgang",hideText:"Skjul",offText:"Fra",onText:"Til",backText:"Tilbage",undoText:"Fortryd"},t.i18n.de={setText:"OK",cancelText:"Abbrechen",clearText:"Löschen",selectedText:"{count} ausgewählt",dateFormat:"dd.mm.yy",dayNames:["Sonntag","Montag","Dienstag","Mittwoch","Donnerstag","Freitag","Samstag"],dayNamesShort:["So","Mo","Di","Mi","Do","Fr","Sa"],dayNamesMin:["S","M","D","M","D","F","S"],dayText:"Tag",delimiter:".",hourText:"Stunde",minuteText:"Minuten",monthNames:["Januar","Februar","März","April","Mai","Juni","Juli","August","September","Oktober","November","Dezember"],monthNamesShort:["Jan","Feb","Mär","Apr","Mai","Jun","Jul","Aug","Sep","Okt","Nov","Dez"],monthText:"Monat",secText:"Sekunden",timeFormat:"HH:ii",yearText:"Jahr",nowText:"Jetzt",pmText:"pm",amText:"am",todayText:"Heute",firstDay:1,dateText:"Datum",timeText:"Zeit",closeText:"Schließen",allDayText:"Ganztägig",noEventsText:"Keine Ereignisse",eventText:"Ereignis",eventsText:"Ereignisse",moreEventsText:"{count} weiteres Element",moreEventsPluralText:"{count} weitere Elemente",fromText:"Von",toText:"Bis",wholeText:"Ganze Zahl",fractionText:"Bruchzahl",unitText:"Maßeinheit",labels:["Jahre","Monate","Tage","Stunden","Minuten","Sekunden",""],labelsShort:["Jahr.","Mon.","Tag.","Std.","Min.","Sek.",""],startText:"Starten",stopText:"Stoppen",resetText:"Zurücksetzen",lapText:"Lap",hideText:"Ausblenden",backText:"Zurück",undoText:"Rückgängig machen",offText:"Aus",onText:"Ein",decimalSeparator:",",thousandsSeparator:" "},t.i18n.el={setText:"Ορισμος",cancelText:"Ακυρωση",clearText:"Διαγραφη",selectedText:"{count} επιλεγμένα",dateFormat:"dd/mm/yy",dayNames:["Κυριακή","Δευτέρα","Τρίτη","Τετάρτη","Πέμπτη","Παρασκευή","Σάββατο"],dayNamesShort:["Κυρ","Δευ","Τρι","Τετ","Πεμ","Παρ","Σαβ"],dayNamesMin:["Κυ","Δε","Τρ","Τε","Πε","Πα","Σα"],dayText:"ημέρα",delimiter:"/",hourText:"ώρα",minuteText:"λεπτό",monthNames:["Ιανουάριος","Φεβρουάριος","Μάρτιος","Απρίλιος","Μάιος","Ιούνιος","Ιούλιος","Αύγουστος","Σεπτέμβριος","Οκτώβριος","Νοέμβριος","Δεκέμβριος"],monthNamesShort:["Ιαν","Φεβ","Μαρ","Απρ","Μαι","Ιουν","Ιουλ","Αυγ","Σεπ","Οκτ","Νοε","Δεκ"],monthText:"Μήνας",secText:"δευτερόλεπτα",timeFormat:"H:ii",yearText:"έτος",nowText:"τώρα",pmText:"μμ",amText:"πμ",firstDay:1,dateText:"Ημερομηνία",timeText:"φορά",todayText:"Σήμερα",prevMonthText:"Προηγούμενο μήνα",nextMonthText:"Επόμενο μήνα",prevYearText:"Προηγούμενο έτος",nextYearText:"Επόμενο έτος",closeText:"Κλείσιμο",eventText:"Γεγονότα",eventsText:"Γεγονότα",allDayText:"Ολοήμερο",noEventsText:"Δεν υπάρχουν γεγονότα",moreEventsText:"{count} ακόμη",fromText:"Αρχή",toText:"Τέλος",wholeText:"Ολόκληρος",fractionText:"κλάσμα",unitText:"Μονάδα",labels:["Χρόνια","Μήνες","Ημέρες","Ωρες","Λεπτά","δευτερόλεπτα",""],labelsShort:["Χρόνια","Μήνες","Ημέρες","Ωρες","Λεπτά","δευτ",""],startText:"΄Εναρξη",stopText:"Διακοπή",resetText:"Επαναφορά",lapText:"Γύρος",hideText:"κρύβω",backText:"Πίσω",undoText:"Αναιρεση",offText:"Ανενεργό",onText:"Ενεργό",decimalSeparator:",",thousandsSeparator:" "},t.i18n["en-GB"]=t.i18n["en-UK"]={dateFormat:"dd/mm/yy",timeFormat:"HH:ii"},t.i18n.es={setText:"Aceptar",cancelText:"Cancelar",clearText:"Borrar",selectedText:"{count} seleccionado",selectedPluralText:"{count} seleccionados",dateFormat:"dd/mm/yy",dayNames:["Domingo","Lunes","Martes","Miércoles","Jueves","Viernes","Sábado"],dayNamesShort:["Do","Lu","Ma","Mi","Ju","Vi","Sá"],dayNamesMin:["D","L","M","M","J","V","S"],dayText:"Día",hourText:"Horas",minuteText:"Minutos",monthNames:["Enero","Febrero","Marzo","Abril","Mayo","Junio","Julio","Agosto","Septiembre","Octubre","Noviembre","Diciembre"],monthNamesShort:["Ene","Feb","Mar","Abr","May","Jun","Jul","Ago","Sep","Oct","Nov","Dic"],monthText:"Mes",secText:"Segundos",timeFormat:"HH:ii",yearText:"A&ntilde;o",nowText:"Ahora",pmText:"pm",amText:"am",todayText:"Hoy",firstDay:1,dateText:"Fecha",timeText:"Tiempo",closeText:"Cerrar",allDayText:"Todo el día",noEventsText:"No hay eventos",eventText:"Evento",eventsText:"Eventos",moreEventsText:"{count} más",fromText:"Iniciar",toText:"Final",wholeText:"Entero",fractionText:"Fracción",unitText:"Unidad",labels:["Años","Meses","Días","Horas","Minutos","Segundos",""],labelsShort:["Año","Mes","Día","Hora","Min","Seg",""],startText:"Iniciar",stopText:"Deténgase",resetText:"Reinicializar",lapText:"Lap",hideText:"Esconder",backText:"Atrás",undoText:"Deshacer",offText:"No",onText:"Sí",decimalSeparator:",",thousandsSeparator:" "},eC=[31,28,31,30,31,30,31,31,30,31,30,31],eu=[31,31,31,31,31,31,30,30,30,30,30,29];function tn(i,r,c){var e,t,o,l,n=(i=parseInt(i))-1600,a=(r=parseInt(r))-1,d=(c=parseInt(c))-1,s=365*n+parseInt((3+n)/4)-parseInt((99+n)/100)+parseInt((399+n)/400);for(t=0;t<a;++t)s+=eC[t];1<a&&(n%4==0&&n%100!=0||n%400==0)&&++s,e=(s+=d)-79,l=parseInt(e/12053),e%=12053,o=979+33*l+4*parseInt(e/1461);for(366<=(e%=1461)&&(o+=parseInt((e-1)/365),e=(e-1)%365),t=0;t<11&&e>=eu[t];++t)e-=eu[t];return[o,t+1,e+1]}eG.jalali={getYear:function(e){return tn(e.getFullYear(),e.getMonth()+1,e.getDate())[0]},getMonth:function(e){return--tn(e.getFullYear(),e.getMonth()+1,e.getDate())[1]},getDay:function(e){return tn(e.getFullYear(),e.getMonth()+1,e.getDate())[2]},getDate:function(t,e,s,o,i,a,r){e<0&&(t+=Math.floor(e/12),e=12+e%12),11<e&&(t+=Math.floor(e/12),e%=12);var n=function(i,a,c){var t,o=(i=parseInt(i))-979,l=(a=parseInt(a))-1,d=(c=parseInt(c))-1,r=365*o+8*parseInt(o/33)+parseInt((o%33+3)/4);for(t=0;t<l;++t)r+=eu[t];var e=(r+=d)+79,s=1600+400*parseInt(e/146097),n=!0;for(36525<=(e%=146097)&&(e--,s+=100*parseInt(e/36524),365<=(e%=36524)?e++:n=!1),s+=4*parseInt(e/1461),366<=(e%=1461)&&(n=!1,e--,s+=parseInt(e/365),e%=365),t=0;eC[t]+(1==t&&n)<=e;t++)e-=eC[t]+(1==t&&n);return[s,t+1,e+1]}(t,+e+1,s);return new Date(n[0],n[1]-1,n[2],o||0,i||0,a||0,r||0)},getMaxDayOfMonth:function(o,i){for(var e,t,n,s=31;!1==(e=i+1,n=s,!((t=o)<0||32767<t||e<1||12<e||n<1||n>eu[e-1]+(12==e&&(t-979)%33%4==0)));)s--;return s}},t.i18n.fa={setText:"تاييد",cancelText:"انصراف",clearText:"واضح ",selectedText:"{count} منتخب",calendarSystem:"jalali",dateFormat:"yy/mm/dd",dayNames:["يکشنبه","دوشنبه","سه‌شنبه","چهارشنبه","پنج‌شنبه","جمعه","شنبه"],dayNamesShort:["ی","د","س","چ","پ","ج","ش"],dayNamesMin:["ی","د","س","چ","پ","ج","ش"],dayText:"روز",hourText:"ساعت",minuteText:"دقيقه",monthNames:["فروردين","ارديبهشت","خرداد","تير","مرداد","شهريور","مهر","آبان","آذر","دی","بهمن","اسفند"],monthNamesShort:["فروردين","ارديبهشت","خرداد","تير","مرداد","شهريور","مهر","آبان","آذر","دی","بهمن","اسفند"],monthText:"ماه",secText:"ثانيه",timeFormat:"HH:ii",timeWheels:"iiHH",yearText:"سال",nowText:"اکنون",amText:"ب",pmText:"ص",todayText:"امروز",firstDay:6,rtl:!0,dateText:"تاریخ ",timeText:"زمان ",closeText:"نزدیک",allDayText:"تمام روز",noEventsText:"هیچ رویداد",eventText:"رویداد",eventsText:"رویدادها",moreEventsText:"{count} مورد دیگر",fromText:"شروع ",toText:"پایان",wholeText:"تمام",fractionText:"کسر",unitText:"واحد",labels:["سال","ماه","روز","ساعت","دقیقه","ثانیه",""],labelsShort:["سال","ماه","روز","ساعت","دقیقه","ثانیه",""],startText:"شروع",stopText:"پايان",resetText:"تنظیم مجدد",lapText:"Lap",hideText:"پنهان کردن",backText:"پشت",undoText:"واچیدن"},t.i18n.fi={setText:"Aseta",cancelText:"Peruuta",clearText:"Tyhjennä",selectedText:"{count} valita",dateFormat:"d. MM yy",dayNames:["Sunnuntai","Maanantai","Tiistai","Keskiviiko","Torstai","Perjantai","Lauantai"],dayNamesShort:["Su","Ma","Ti","Ke","To","Pe","La"],dayNamesMin:["S","M","T","K","T","P","L"],dayText:"Päivä",delimiter:".",hourText:"Tuntia",minuteText:"Minuutti",monthNames:["Tammikuu","Helmikuu","Maaliskuu","Huhtikuu","Toukokuu","Kesäkuu","Heinäkuu","Elokuu","Syyskuu","Lokakuu","Marraskuu","Joulukuu"],monthNamesShort:["Tam","Hel","Maa","Huh","Tou","Kes","Hei","Elo","Syy","Lok","Mar","Jou"],monthText:"Kuukausi",secText:"Sekunda",timeFormat:"H:ii",yearText:"Vuosi",nowText:"Nyt",pmText:"pm",amText:"am",firstDay:1,dateText:"Päiväys",timeText:"Aika",todayText:"Tänään",prevMonthText:"Edellinen kuukausi",nextMonthText:"Ensi kuussa",prevYearText:"Edellinen vuosi",nextYearText:"Ensi vuosi",closeText:"Sulje",eventText:"Tapahtumia",eventsText:"Tapahtumia",allDayText:"Koko päivä",noEventsText:"Ei tapahtumia",moreEventsText:"{count} muu",moreEventsPluralText:"{count} muuta",fromText:"Alkaa",toText:"Päättyy",wholeText:"Kokonainen",fractionText:"Murtoluku",unitText:"Yksikkö",labels:["Vuosi","Kuukausi","Päivä","Tunnin","Minuutti","sekuntia",""],labelsShort:["Vuo","Kuu","Päi","Tun","Min","Sek",""],startText:"Käynnistys",stopText:"Seis",resetText:"Aseta uudelleen",lapText:"Kierros",hideText:"Vuota",backText:"Edellinen",undoText:"Kumoa",offText:"Pois",onText:"Päällä",decimalSeparator:",",thousandsSeparator:" "},t.i18n.fr={setText:"Terminer",cancelText:"Annuler",clearText:"Effacer",selectedText:"{count} sélectionné",selectedPluralText:"{count} sélectionnés",dateFormat:"dd/mm/yy",dayNames:["Dimanche","Lundi","Mardi","Mercredi","Jeudi","Vendredi","Samedi"],dayNamesShort:["Dim.","Lun.","Mar.","Mer.","Jeu.","Ven.","Sam."],dayNamesMin:["D","L","M","M","J","V","S"],dayText:"Jour",monthText:"Mois",monthNames:["Janvier","Février","Mars","Avril","Mai","Juin","Juillet","Août","Septembre","Octobre","Novembre","Décembre"],monthNamesShort:["Janv.","Févr.","Mars","Avril","Mai","Juin","Juil.","Août","Sept.","Oct.","Nov.","Déc."],hourText:"Heures",minuteText:"Minutes",secText:"Secondes",timeFormat:"HH:ii",yearText:"Année",nowText:"Maintenant",pmText:"pm",amText:"am",todayText:"Aujourd'hui",firstDay:1,dateText:"Date",timeText:"Heure",closeText:"Fermer",allDayText:"Toute la journée",noEventsText:"Aucun événement",eventText:"Événement",eventsText:"Événements",moreEventsText:"{count} autre",moreEventsPluralText:"{count} autres",fromText:"Démarrer",toText:"Fin",wholeText:"Entier",fractionText:"Fraction",unitText:"Unité",labels:["Ans","Mois","Jours","Heures","Minutes","Secondes",""],labelsShort:["Ans","Mois","Jours","Hrs","Min","Sec",""],startText:"Démarrer",stopText:"Arrêter",resetText:"Réinitialiser",lapText:"Lap",hideText:"Cachez",backText:"Retour",undoText:"Annuler",offText:"Non",onText:"Oui",decimalSeparator:",",thousandsSeparator:" "},t.i18n.he={rtl:!0,setText:"שמירה",cancelText:"ביטול",clearText:"נקה",selectedText:"{count} נבחר",selectedPluralText:"{count} נבחרו",dateFormat:"dd/mm/yy",dayNames:["ראשון","שני","שלישי","רביעי","חמישי","שישי","שבת"],dayNamesShort:["א'","ב'","ג'","ד'","ה'","ו'","ש'"],dayNamesMin:["א","ב","ג","ד","ה","ו","ש"],dayText:"יום",hourText:"שעות",minuteText:"דקות",monthNames:["ינואר","פברואר","מרץ","אפריל","מאי","יוני","יולי","אוגוסט","ספטמבר","אוקטובר","נובמבר","דצמבר"],monthNamesShort:["ינו","פבר","מרץ","אפר","מאי","יונ","יול","אוג","ספט","אוק","נוב","דצמ"],monthText:"חודש",secText:"שניות",amText:"am",pmText:"pm",timeFormat:"HH:ii",timeWheels:"iiHH",yearText:"שנה",nowText:"עכשיו",firstDay:0,dateText:"תאריך",timeText:"זמן",closeText:"סגירה",todayText:"היום",allDayText:"כל היום",noEventsText:"אין אירועים",eventText:"מִקרֶה",eventsText:"מִקרֶה",moreEventsText:"אירוע אחד נוסף",moreEventsPluralText:"{count} אירועים נוספים",fromText:"התחלה",toText:"סיום",wholeText:"כֹּל",fractionText:"שבריר",unitText:"יחידה",labels:["שנים","חודשים","ימים","שעות","דקות","שניים",""],labelsShort:["שנים","חודשים","ימים","שעות","דקות","שניים",""],startText:"התחל",stopText:"עצור",resetText:"אתחול",lapText:"הקפה",hideText:"הסתר",offText:"כיבוי",onText:"הפעלה",backText:"חזור",undoText:"ביטול פעולה"},t.i18n.hi={setText:"सैट करें",cancelText:"रद्द करें",clearText:"साफ़ को",selectedText:"{count} चयनित",dateFormat:"dd/mm/yy",dayNames:["रविवार","सोमवार","मंगलवार","बुधवार","गुरुवार","शुक्रवार","शनिवार"],dayNamesShort:["रवि","सोम","मंगल","बुध","गुरु","शुक्र","शनि"],dayNamesMin:["रवि","सोम","मंगल","बुध","गुरु","शुक्र","शनि"],dayText:"दिन",delimiter:".",hourText:"घंटा",minuteText:"मिनट",monthNames:["जनवरी ","फरवरी","मार्च","अप्रेल","मई","जून","जूलाई","अगस्त ","सितम्बर","अक्टूबर","नवम्बर","दिसम्बर"],monthNamesShort:["जन","फर","मार्च","अप्रेल","मई","जून","जूलाई","अग","सित","अक्ट","नव","दि"],monthText:"महीना",secText:"सेकंड",timeFormat:"H:ii",yearText:"साल",nowText:"अब",pmText:"अपराह्न",amText:"पूर्वाह्न",firstDay:1,dateText:"तिथि",timeText:"समय",todayText:"आज",prevMonthText:"पिछ्ला महिना",nextMonthText:"अगले महीने",prevYearText:"पिछला साल",nextYearText:"अगले वर्ष",closeText:"बंद",eventText:"इवेट३",eventsText:"इवेट३",allDayText:"पूरे दिन",noEventsText:"Ei tapahtumia",moreEventsText:"{count} और",fromText:"से",toText:"तक",wholeText:"समूचा",fractionText:"अंश",unitText:"इकाई",labels:["साल","महीने","दिन","घंटे","मिनट","सेकंड",""],labelsShort:["साल","महीने","दिन","घंटे","मिनट","सेकंड",""],startText:"प्रारंभ",stopText:"रोकें",resetText:"रीसेट करें",lapText:"लैप",hideText:"छिपाना",backText:"वापस",undoText:"वापस लाएं",offText:"बंद",onText:"चालू",decimalSeparator:",",thousandsSeparator:" "},t.i18n.hr={setText:"Postavi",cancelText:"Izlaz",clearText:"Izbriši",selectedText:"{count} odabran",dateFormat:"dd.mm.yy",dayNames:["Nedjelja","Ponedjeljak","Utorak","Srijeda","Četvrtak","Petak","Subota"],dayNamesShort:["Ned","Pon","Uto","Sri","Čet","Pet","Sub"],dayNamesMin:["Ne","Po","Ut","Sr","Če","Pe","Su"],dayText:"Dan",delimiter:".",hourText:"Sat",minuteText:"Minuta",monthNames:["Siječanj","Veljača","Ožujak","Travanj","Svibanj","Lipanj","Srpanj","Kolovoz","Rujan","Listopad","Studeni","Prosinac"],monthNamesShort:["Sij","Velj","Ožu","Tra","Svi","Lip","Srp","Kol","Ruj","Lis","Stu","Pro"],monthText:"Mjesec",secText:"Sekunda",timeFormat:"H:ii",yearText:"Godina",nowText:"Sada",pmText:"pm",amText:"am",firstDay:1,dateText:"Datum",timeText:"Vrijeme",todayText:"Danas",prevMonthText:"Prethodni mjesec",nextMonthText:"Sljedeći mjesec",prevYearText:"Prethodni godina",nextYearText:"Slijedeće godine",closeText:"Zatvori",eventText:"Događaj",eventsText:"događaja",allDayText:"Cijeli dan",noEventsText:"Bez događaja",moreEventsText:"Još {count}",fromText:"Počinje",toText:"Završava",wholeText:"Cjelina",fractionText:"Frakcija",unitText:"Jedinica",labels:["godina","mjesec","dan","sat","minuta","sekunda",""],labelsShort:["god","mje","dan","sat","min","sec",""],startText:"Početak",stopText:"Prekid",resetText:"Resetiraj",lapText:"Ciklus",hideText:"Sakriti",backText:"Natrag",undoText:"Poništavanje",offText:"Uklj.",onText:"Isklj.",decimalSeparator:",",thousandsSeparator:" "},t.i18n.hu={setText:"OK",cancelText:"Mégse",clearText:"Törlés",selectedText:"{count} kiválasztva",dateFormat:"yy.mm.dd.",dayNames:["Vasárnap","Hétfő","Kedd","Szerda","Csütörtök","Péntek","Szombat"],dayNamesShort:["Va","Hé","Ke","Sze","Csü","Pé","Szo"],dayNamesMin:["V","H","K","Sz","Cs","P","Sz"],dayText:"Nap",delimiter:".",hourText:"Óra",minuteText:"Perc",monthNames:["Január","Február","Március","Április","Május","Június","Július","Augusztus","Szeptember","Október","November","December"],monthNamesShort:["Jan","Feb","Már","Ápr","Máj","Jún","Júl","Aug","Szep","Okt","Nov","Dec"],monthText:"Hónap",secText:"Másodperc",timeFormat:"H:ii",yearText:"Év",nowText:"Most",pmText:"pm",amText:"am",firstDay:1,dateText:"Dátum",timeText:"Idő",todayText:"Ma",prevMonthText:"Előző hónap",nextMonthText:"Következő hónap",prevYearText:"Előző év",nextYearText:"Következő év",closeText:"Bezár",eventText:"esemény",eventsText:"esemény",allDayText:"Egész nap",noEventsText:"Nincs esemény",moreEventsText:"{count} további",fromText:"Eleje",toText:"Vége",wholeText:"Egész",fractionText:"Tört",unitText:"Egység",labels:["Év","Hónap","Nap","Óra","Perc","Másodperc",""],labelsShort:["Év","Hó.","Nap","Óra","Perc","Mp.",""],startText:"Indít",stopText:"Megállít",resetText:"Visszaállít",lapText:"Lap",hideText:"Elrejt",backText:"Vissza",undoText:"Visszavon",offText:"Ki",onText:"Be",decimalSeparator:",",thousandsSeparator:" "},t.i18n.it={setText:"OK",cancelText:"Annulla",clearText:"Chiarire",selectedText:"{count} selezionato",selectedPluralText:"{count} selezionati",dateFormat:"dd/mm/yy",dayNames:["Domenica","Lunedì","Mertedì","Mercoledì","Giovedì","Venerdì","Sabato"],dayNamesShort:["Do","Lu","Ma","Me","Gi","Ve","Sa"],dayNamesMin:["D","L","M","M","G","V","S"],dayText:"Giorno",hourText:"Ore",minuteText:"Minuti",monthNames:["Gennaio","Febbraio","Marzo","Aprile","Maggio","Giugno","Luglio","Agosto","Settembre","Ottobre","Novembre","Dicembre"],monthNamesShort:["Gen","Feb","Mar","Apr","Mag","Giu","Lug","Ago","Set","Ott","Nov","Dic"],monthText:"Mese",secText:"Secondi",timeFormat:"HH:ii",yearText:"Anno",nowText:"Ora",pmText:"pm",amText:"am",todayText:"Oggi",firstDay:1,dateText:"Data",timeText:"Volta",closeText:"Chiudere",allDayText:"Tutto il giorno",noEventsText:"Nessun evento",eventText:"Evento",eventsText:"Eventi",moreEventsText:"{count} altro",moreEventsPluralText:"altri {count}",fromText:"Inizio",toText:"Fine",wholeText:"Intero",fractionText:"Frazione",unitText:"Unità",labels:["Anni","Mesi","Giorni","Ore","Minuti","Secondi",""],labelsShort:["Anni","Mesi","Gio","Ore","Min","Sec",""],startText:"Inizio",stopText:"Arresto",resetText:"Ripristina",lapText:"Lap",hideText:"Nascondi",backText:"Indietro",undoText:"Annulla",offText:"Via",onText:"Su",decimalSeparator:",",thousandsSeparator:" "},t.i18n.ja={setText:"セット",cancelText:"キャンセル",clearText:"クリア",selectedText:"{count} 選択",dateFormat:"yy年mm月dd日",dayNames:["日","月","火","水","木","金","土"],dayNamesShort:["日","月","火","水","木","金","土"],dayNamesMin:["日","月","火","水","木","金","土"],dayText:"日",hourText:"時",minuteText:"分",monthNames:["1月","2月","3月","4月","5月","6月","7月","8月","9月","10月","11月","12月"],monthNamesShort:["1月","2月","3月","4月","5月","6月","7月","8月","9月","10月","11月","12月"],monthText:"月",secText:"秒",timeFormat:"HH:ii",yearText:"年",nowText:"今",pmText:"午後",amText:"午前",yearSuffix:"年",monthSuffix:"月",daySuffix:"日",todayText:"今日",dateText:"日付",timeText:"時間",closeText:"クローズ",allDayText:"終日",noEventsText:"イベントはありません",eventText:"イベント",eventsText:"イベント",moreEventsText:"他 {count} 件",fromText:"開始",toText:"終わり",wholeText:"全数",fractionText:"分数",unitText:"単位",labels:["年間","月間","日間","時間","分","秒",""],labelsShort:["年間","月間","日間","時間","分","秒",""],startText:"開始",stopText:"停止",resetText:"リセット",lapText:"ラップ",hideText:"隠す",backText:"バック",undoText:"アンドゥ"},t.i18n.ko={setText:"설정",cancelText:"취소",clearText:"삭제",selectedText:"{count} 선택된",dateFormat:"yy년mm월dd일",dayNames:["일요일","월요일","화요일","수요일","목요일","금요일","토요일"],dayNamesShort:["일","월","화","수","목","금","토"],dayNamesMin:["일","월","화","수","목","금","토"],dayText:"일",delimiter:"-",hourText:"시간",minuteText:"분",monthNames:["1월","2월","3월","4월","5월","6월","7월","8월","9월","10월","11월","12월"],monthNamesShort:["1월","2월","3월","4월","5월","6월","7월","8월","9월","10월","11월","12월"],monthText:"달",secText:"초",timeFormat:"H:ii",yearText:"년",nowText:"지금",pmText:"오후",amText:"오전",yearSuffix:"년",monthSuffix:"월",daySuffix:"일",firstDay:0,dateText:"날짜",timeText:"시간",todayText:"오늘",prevMonthText:"이전 달",nextMonthText:"다음 달",prevYearText:"이전 년",nextYearText:"다음 년",closeText:"닫기",eventText:"이벤트",eventsText:"이벤트",allDayText:"종일",noEventsText:"이벤트 없음",moreEventsText:"{count}개 더보기",fromText:"시작",toText:"종료",wholeText:"정수",fractionText:"분수",unitText:"단위",labels:["년","달","일","시간","분","초",""],labelsShort:["년","달","일","시간","분","초",""],startText:"시작",stopText:"중지 ",resetText:"초기화",lapText:"기록",hideText:"숨는 장소",backText:"뒤로",undoText:"실행취소",offText:"끔",onText:"켬",decimalSeparator:",",thousandsSeparator:" "},t.i18n.lt={setText:"OK",cancelText:"Atšaukti",clearText:"Išvalyti",selectedText:"Pasirinktas {count}",selectedPluralText:"Pasirinkti {count}",dateFormat:"yy-mm-dd",dayNames:["Sekmadienis","Pirmadienis","Antradienis","Trečiadienis","Ketvirtadienis","Penktadienis","Šeštadienis"],dayNamesShort:["S","Pr","A","T","K","Pn","Š"],dayNamesMin:["S","Pr","A","T","K","Pn","Š"],dayText:"Diena",hourText:"Valanda",minuteText:"Minutes",monthNames:["Sausis","Vasaris","Kovas","Balandis","Gegužė","Birželis","Liepa","Rugpjūtis","Rugsėjis","Spalis","Lapkritis","Gruodis"],monthNamesShort:["Sau","Vas","Kov","Bal","Geg","Bir","Lie","Rugp","Rugs","Spa","Lap","Gruo"],monthText:"Mėnuo",secText:"Sekundes",amText:"am",pmText:"pm",timeFormat:"HH:ii",yearText:"Metai",nowText:"Dabar",todayText:"Šiandien",firstDay:1,dateText:"Data",timeText:"Laikas",closeText:"Uždaryti",allDayText:"Visą dieną",noEventsText:"Nėra įvykių",eventText:"Įvykių",eventsText:"Įvykiai",moreEventsText:"Dar {count}",fromText:"Nuo",toText:"Iki",wholeText:"Visas",fractionText:"Frakcija",unitText:"Vienetas",labels:["Metai","Mėnesiai","Dienos","Valandos","Minutes","Sekundes",""],labelsShort:["m","mėn.","d","h","min","s",""],startText:"Pradėti",stopText:"Sustabdyti",resetText:"Išnaujo",lapText:"Ratas",hideText:"Slėpti",backText:"Atgal",undoText:"Anuliuoti",offText:"Išj.",onText:"Įj.",decimalSeparator:",",thousandsSeparator:" "},t.i18n.nl={setText:"Instellen",cancelText:"Annuleren",clearText:"Leegmaken",selectedText:"{count} gekozen",dateFormat:"dd-mm-yy",dayNames:["Zondag","Maandag","Dinsdag","Woensdag","Donderdag","Vrijdag","Zaterdag"],dayNamesShort:["zo","ma","di","wo","do","vr","za"],dayNamesMin:["z","m","d","w","d","v","z"],dayText:"Dag",hourText:"Uur",minuteText:"Minuten",monthNames:["januari","februari","maart","april","mei","juni","juli","augustus","september","oktober","november","december"],monthNamesShort:["jan","feb","mrt","apr","mei","jun","jul","aug","sep","okt","nov","dec"],monthText:"Maand",secText:"Seconden",timeFormat:"HH:ii",yearText:"Jaar",nowText:"Nu",pmText:"pm",amText:"am",todayText:"Vandaag",firstDay:1,dateText:"Datum",timeText:"Tijd",closeText:"Sluiten",allDayText:"Hele dag",noEventsText:"Geen activiteiten",eventText:"Activiteit",eventsText:"Activiteiten",moreEventsText:"nog {count}",fromText:"Start",toText:"Einde",wholeText:"geheel",fractionText:"fractie",unitText:"eenheid",labels:["Jaren","Maanden","Dagen","Uren","Minuten","Seconden",""],labelsShort:["j","m","d","u","min","sec",""],startText:"Start",stopText:"Stop",resetText:"Reset",lapText:"Ronde",hideText:"Verbergen",backText:"Terug",undoText:"Onged. maken",offText:"Uit",onText:"Aan",decimalSeparator:",",thousandsSeparator:" "},t.i18n.no={setText:"OK",cancelText:"Avbryt",clearText:"Tømme",selectedText:"{count} valgt",dateFormat:"dd.mm.yy",dayNames:["Søndag","Mandag","Tirsdag","Onsdag","Torsdag","Fredag","Lørdag"],dayNamesShort:["Sø","Ma","Ti","On","To","Fr","Lø"],dayNamesMin:["S","M","T","O","T","F","L"],dayText:"Dag",delimiter:".",hourText:"Time",minuteText:"Minutt",monthNames:["Januar","Februar","Mars","April","Mai","Juni","Juli","August","September","Oktober","November","Desember"],monthNamesShort:["Jan","Feb","Mar","Apr","Mai","Jun","Jul","Aug","Sep","Okt","Nov","Des"],monthText:"Måned",secText:"Sekund",timeFormat:"HH:ii",yearText:"År",nowText:"Nå",pmText:"pm",amText:"am",todayText:"I dag",firstDay:1,dateText:"Dato",timeText:"Tid",closeText:"Lukk",allDayText:"Hele dagen",noEventsText:"Ingen hendelser",eventText:"Hendelse",eventsText:"Hendelser",moreEventsText:"{count} mere",fromText:"Start",toText:"End",wholeText:"Hele",fractionText:"Fraksjon",unitText:"Enhet",labels:["År","Måneder","Dager","Timer","Minutter","Sekunder",""],labelsShort:["År","Mån","Dag","Time","Min","Sek",""],startText:"Start",stopText:"Stopp",resetText:"Tilbakestille",lapText:"Runde",hideText:"Skjul",backText:"Tilbake",undoText:"Angre",offText:"Av",onText:"På",decimalSeparator:",",thousandsSeparator:" "},t.i18n.pl={setText:"Zestaw",cancelText:"Anuluj",clearText:"Oczyścić",selectedText:"Wybór: {count}",dateFormat:"yy-mm-dd",dayNames:["Niedziela","Poniedziałek","Wtorek","Środa","Czwartek","Piątek","Sobota"],dayNamesShort:["Niedz.","Pon.","Wt.","Śr.","Czw.","Pt.","Sob."],dayNamesMin:["N","P","W","Ś","C","P","S"],dayText:"Dzień",hourText:"Godziny",minuteText:"Minuty",monthNames:["Styczeń","Luty","Marzec","Kwiecień","Maj","Czerwiec","Lipiec","Sierpień","Wrzesień","Październik","Listopad","Grudzień"],monthNamesShort:["Sty","Lut","Mar","Kwi","Maj","Cze","Lip","Sie","Wrz","Paź","Lis","Gru"],monthText:"Miesiąc",secText:"Sekundy",timeFormat:"HH:ii",yearText:"Rok",nowText:"Teraz",amText:"am",pmText:"pm",todayText:"Dzisiaj",firstDay:1,dateText:"Data",timeText:"Czas",closeText:"Zakończenie",allDayText:"Cały dzień",noEventsText:"Brak wydarzeń",eventText:"Wydarzeń",eventsText:"Wydarzenia",moreEventsText:"Jeszcze {count}",fromText:"Rozpoczęcie",toText:"Koniec",wholeText:"Cały",fractionText:"Ułamek",unitText:"Jednostka",labels:["Lata","Miesiąc","Dni","Godziny","Minuty","Sekundy",""],labelsShort:["R","M","Dz","Godz","Min","Sek",""],startText:"Rozpoczęcie",stopText:"Zatrzymać",resetText:"Zresetować",lapText:"Zakładka",hideText:"Ukryć",backText:"Wróć",undoText:"Cofnij",offText:"Wył",onText:"Wł",decimalSeparator:",",thousandsSeparator:" "},t.i18n["pt-BR"]={setText:"Selecionar",cancelText:"Cancelar",clearText:"Claro",selectedText:"{count} selecionado",selectedPluralText:"{count} selecionados",dateFormat:"dd/mm/yy",dayNames:["Domingo","Segunda-feira","Terça-feira","Quarta-feira","Quinta-feira","Sexta-feira","Sábado"],dayNamesShort:["Dom","Seg","Ter","Qua","Qui","Sex","Sáb"],dayNamesMin:["D","S","T","Q","Q","S","S"],dayText:"Dia",hourText:"Hora",minuteText:"Minutos",monthNames:["Janeiro","Fevereiro","Março","Abril","Maio","Junho","Julho","Agosto","Setembro","Outubro","Novembro","Dezembro"],monthNamesShort:["Jan","Fev","Mar","Abr","Mai","Jun","Jul","Ago","Set","Out","Nov","Dez"],monthText:"Mês",secText:"Segundo",timeFormat:"HH:ii",yearText:"Ano",nowText:"Agora",pmText:"pm",amText:"am",todayText:"Hoje",dateText:"Data",timeText:"Tempo",closeText:"Fechar",allDayText:"Dia inteiro",noEventsText:"Nenhum evento",eventText:"Evento",eventsText:"Eventos",moreEventsText:"Mais {count}",fromText:"In&iacute;cio",toText:"Fim",wholeText:"Inteiro",fractionText:"Fração",unitText:"Unidade",labels:["Anos","Meses","Dias","Horas","Minutos","Segundos",""],labelsShort:["Ano","M&ecirc;s","Dia","Hora","Min","Seg",""],startText:"Começar",stopText:"Pare",resetText:"Reinicializar",lapText:"Lap",hideText:"Esconder",backText:"Anterior",undoText:"Desfazer",offText:"Desl",onText:"Lig",decimalSeparator:",",thousandsSeparator:" "},t.i18n["pt-PT"]={setText:"Seleccionar",cancelText:"Cancelar",clearText:"Claro",selectedText:"{count} selecionado",selectedPluralText:"{count} selecionados",dateFormat:"dd-mm-yy",dayNames:["Domingo","Segunda-feira","Terça-feira","Quarta-feira","Quinta-feira","Sexta-feira","Sábado"],dayNamesShort:["Dom","Seg","Ter","Qua","Qui","Sex","Sáb"],dayNamesMin:["D","S","T","Q","Q","S","S"],dayText:"Dia",hourText:"Horas",minuteText:"Minutos",monthNames:["Janeiro","Fevereiro","Março","Abril","Maio","Junho","Julho","Agosto","Setembro","Outubro","Novembro","Dezembro"],monthNamesShort:["Jan","Fev","Mar","Abr","Mai","Jun","Jul","Ago","Set","Out","Nov","Dez"],monthText:"Mês",secText:"Segundo",timeFormat:"HH:ii",yearText:"Ano",nowText:"Actualizar",pmText:"pm",amText:"am",todayText:"Hoy",firstDay:1,dateText:"Data",timeText:"Tempo",closeText:"Fechar",allDayText:"Todo o dia",noEventsText:"Nenhum evento",eventText:"Evento",eventsText:"Eventos",moreEventsText:"mais {count}",fromText:"Início",toText:"Fim",wholeText:"Inteiro",fractionText:"Fracção",unitText:"Unidade",labels:["Anos","Meses","Dias","Horas","Minutos","Segundos",""],labelsShort:["Ano","Mês","Dia","Hora","Min","Seg",""],startText:"Começar",stopText:"Parar",resetText:"Reinicializar",lapText:"Lap",hideText:"Esconder",backText:"Anterior",undoText:"Anular",offText:"Desl",onText:"Lig",decimalSeparator:",",thousandsSeparator:" "},t.i18n.ro={setText:"Setare",cancelText:"Anulare",clearText:"Ştergere",selectedText:"{count} selectat",selectedPluralText:"{count} selectate",dateFormat:"dd.mm.yy",dayNames:["Duminică","Luni","Marți","Miercuri","Joi","Vineri","Sâmbătă"],dayNamesShort:["Du","Lu","Ma","Mi","Jo","Vi","Sâ"],dayNamesMin:["D","L","M","M","J","V","S"],dayText:" Ziua",delimiter:".",hourText:" Ore ",minuteText:"Minute",monthNames:["Ianuarie","Februarie","Martie","Aprilie","Mai","Iunie","Iulie","August","Septembrie","Octombrie","Noiembrie","Decembrie"],monthNamesShort:["Ian.","Feb.","Mar.","Apr.","Mai","Iun.","Iul.","Aug.","Sept.","Oct.","Nov.","Dec."],monthText:"Luna",secText:"Secunde",timeFormat:"HH:ii",yearText:"Anul",nowText:"Acum",amText:"am",pmText:"pm",todayText:"Astăzi",prevMonthText:"Luna anterioară",nextMonthText:"Luna următoare",prevYearText:"Anul anterior",nextYearText:"Anul următor",eventText:"Eveniment",eventsText:"Evenimente",allDayText:"Toată ziua",noEventsText:"Niciun eveniment",moreEventsText:"Încă unul",moreEventsPluralText:"Încă {count}",firstDay:1,dateText:"Data",timeText:"Ora",closeText:"Închidere",fromText:"Start",toText:"Final",wholeText:"Complet",fractionText:"Parţial",unitText:"Unitate",labels:["Ani","Luni","Zile","Ore","Minute","Secunde",""],labelsShort:["Ani","Luni","Zile","Ore","Min.","Sec.",""],startText:"Start",stopText:"Stop",resetText:"Resetare",lapText:"Tură",hideText:"Ascundere",backText:"Înapoi",undoText:"Anulează",offText:"Nu",onText:"Da",decimalSeparator:",",thousandsSeparator:" "},t.i18n["ru-UA"]={setText:"Установить",cancelText:"Отменить",clearText:"Очиститьr",selectedText:"{count} Вібрать",dateFormat:"dd.mm.yy",dayNames:["воскресенье","понедельник","вторник","среда","четверг","пятница","суббота"],dayNamesShort:["вс","пн","вт","ср","чт","пт","сб"],dayNamesMin:["в","п","в","с","ч","п","с"],dayText:"День",delimiter:".",hourText:"Часы",minuteText:"Минуты",monthNames:["Январь","Февраль","Март","Апрель","Май","Июнь","Июль","Август","Сентябрь","Октябрь","Ноябрь","Декабрь"],monthNamesShort:["Янв.","Февр.","Март","Апр.","Май","Июнь","Июль","Авг.","Сент.","Окт.","Нояб.","Дек."],monthText:"Месяцы",secText:"Сикунды",timeFormat:"HH:ii",yearText:"Год",nowText:"Сейчас",amText:"am",pmText:"pm",todayText:"Cегодня",firstDay:1,dateText:"Дата",timeText:"Время",closeText:"Закрыть",allDayText:"Весь день",noEventsText:"Нет событий",eventText:"Мероприятия",eventsText:"Мероприятия",moreEventsText:"Ещё {count}",fromText:"Начало",toText:"Конец",wholeText:"Весь",fractionText:"Часть",unitText:"Единица",labels:["Годы"," Месяцы "," Дни "," Часы "," Минуты "," Секунды",""],labelsShort:["Год","Мес.","Дн.","Ч.","Мин.","Сек.",""],startText:"Старт",stopText:"Стоп",resetText:" Сброс ",lapText:" Этап ",hideText:" Скрыть ",backText:"назад",undoText:"ОтменитЬ",offText:"O",onText:"I",decimalSeparator:",",thousandsSeparator:" "},t.i18n["ru-RU"]=t.i18n.ru={setText:"Установить",cancelText:"Отмена",clearText:"Очистить",selectedText:"{count} Выбрать",dateFormat:"dd.mm.yy",dayNames:["воскресенье","понедельник","вторник","среда","четверг","пятница","суббота"],dayNamesShort:["вс","пн","вт","ср","чт","пт","сб"],dayNamesMin:["в","п","в","с","ч","п","с"],dayText:"День",delimiter:".",hourText:"Час",minuteText:"Минут",monthNames:["Январь","Февраль","Март","Апрель","Май","Июнь","Июль","Август","Сентябрь","Октябрь","Ноябрь","Декабрь"],monthNamesShort:["Янв","Фев","Мар","Апр","Май","Июн","Июл","Авг","Сен","Окт","Ноя","Дек"],monthText:"Месяц",secText:"Секунд",timeFormat:"HH:ii",yearText:"Год",nowText:"Сейчас",amText:"am",pmText:"pm",todayText:"Cегодня",firstDay:1,dateText:"Дата",timeText:"Время",closeText:"Закрыть",allDayText:"Весь день",noEventsText:"Нет событий",eventText:"Мероприятия",eventsText:"Мероприятия",moreEventsText:"Ещё {count}",fromText:"Начало",toText:"Конец",wholeText:"Целое",fractionText:"Дробное",unitText:"Единица",labels:["Лет","Месяцев","Дней","Часов","Минут","Секунд",""],labelsShort:["Лет","Мес","Дн","Час","Мин","Сек",""],startText:"Старт",stopText:"Стоп",resetText:"Сбросить",lapText:"Круг",hideText:"Скрыть",backText:"назад",undoText:"ОтменитЬ",offText:"O",onText:"I",decimalSeparator:",",thousandsSeparator:" "},t.i18n.sk={setText:"Zadaj",cancelText:"Zrušiť",clearText:"Vymazať",selectedText:"Označený: {count}",dateFormat:"d.m.yy",dayNames:["Nedeľa","Pondelok","Utorok","Streda","Štvrtok","Piatok","Sobota"],dayNamesShort:["Ne","Po","Ut","St","Št","Pi","So"],dayNamesMin:["N","P","U","S","Š","P","S"],dayText:"Ďeň",hourText:"Hodiny",minuteText:"Minúty",monthNames:["Január","Február","Marec","Apríl","Máj","Jún","Júl","August","September","Október","November","December"],monthNamesShort:["Jan","Feb","Mar","Apr","Máj","Jún","Júl","Aug","Sep","Okt","Nov","Dec"],monthText:"Mesiac",secText:"Sekundy",timeFormat:"H:ii",yearText:"Rok",nowText:"Teraz",amText:"am",pmText:"pm",todayText:"Dnes",firstDay:1,dateText:"Datum",timeText:"Čas",closeText:"Zavrieť",allDayText:"Celý deň",noEventsText:"Žiadne udalosti",eventText:"Udalostí",eventsText:"Udalosti",moreEventsText:"{count} ďalšia",moreEventsPluralText:"{count} ďalšie",fromText:"Začiatok",toText:"Koniec",wholeText:"Celý",fractionText:"Časť",unitText:"Jednotka",labels:["Roky","Mesiace","Dni","Hodiny","Minúty","Sekundy",""],labelsShort:["Rok","Mes","Dni","Hod","Min","Sec",""],startText:"Start",stopText:"Stop",resetText:"Resetovať",lapText:"Etapa",hideText:"Schovať",backText:"Späť",undoText:"Späť",offText:"O",onText:"I",decimalSeparator:",",thousandsSeparator:" "},t.i18n.sr={setText:"Постави",cancelText:"Откажи",clearText:"Обриши",selectedText:"{count} изабрана",dateFormat:"dd.mm.yy",dayNames:["Недеља","Понедељак","Уторак","Среда","Четвртак","Петак","Субота"],dayNamesShort:["Нед","Пон","Уто","Сре","Чет","Пет","Суб"],dayNamesMin:["Не","По","Ут","Ср","Че","Пе","Су"],dayText:"Дан",delimiter:".",hourText:"Час",minuteText:"Минут",monthNames:["Јануар","Фебруар","Март","Април","Мај","Јун","Јул","Август","Септембар","Октобар","Новембар","Децембар"],monthNamesShort:["Јан","Феб","Мар","Апр","Мај","Јун","Јул","Авг","Сеп","Окт","Нов","Дец"],monthText:"месец",secText:"Секунд",timeFormat:"H:ii",yearText:"година",nowText:"сада",pmText:"pm",amText:"am",firstDay:1,dateText:"Датум",timeText:"време",todayText:"Данас",prevMonthText:"Претходни мјесец",nextMonthText:"Следећег месеца",prevYearText:"Претходна године",nextYearText:"Следеће године",closeText:"Затвори",eventText:"Догађај",eventsText:"Догађаји",allDayText:"Цео дан",noEventsText:"Нема догађаја",moreEventsText:"Још {count}",fromText:"Од",toText:"До",wholeText:"цео",fractionText:"Фракција",unitText:"единица",labels:["Године","Месеци","Дана","Сати","Минута","Секунди",""],labelsShort:["Год","Мес","Дана","Сати","Мину","Секу",""],startText:"Започни",stopText:"Стоп",resetText:"Ресетуј",lapText:"Круг",hideText:"Сакрити",backText:"Повратак",undoText:"Опозови",offText:"нe",onText:"да",decimalSeparator:",",thousandsSeparator:" "},t.i18n.sv={setText:"OK",cancelText:"Avbryt",clearText:"Klara",selectedText:"{count} vald",dateFormat:"yy-mm-dd",dayNames:["Söndag","Måndag","Tisdag","Onsdag","Torsdag","Fredag","Lördag"],dayNamesShort:["Sö","Må","Ti","On","To","Fr","Lö"],dayNamesMin:["S","M","T","O","T","F","L"],dayText:"Dag",hourText:"Timme",minuteText:"Minut",monthNames:["Januari","Februari","Mars","April","Maj","Juni","Juli","Augusti","September","Oktober","November","December"],monthNamesShort:["Jan","Feb","Mar","Apr","Maj","Jun","Jul","Aug","Sep","Okt","Nov","Dec"],monthText:"Månad",secText:"Sekund",timeFormat:"HH:ii",yearText:"År",nowText:"Nu",pmText:"pm",amText:"am",todayText:"I dag",firstDay:1,dateText:"Datum",timeText:"Tid",closeText:"Stäng",allDayText:"Heldag",noEventsText:"Inga aktiviteter",eventText:"Händelse",eventsText:"Händelser",moreEventsText:"{count} till",fromText:"Start",toText:"Slut",wholeText:"Hela",fractionText:"Bråk",unitText:"Enhet",labels:["År","Månader","Dagar","Timmar","Minuter","Sekunder",""],labelsShort:["År","Mån","Dag","Tim","Min","Sek",""],startText:"Start",stopText:"Stopp",resetText:"Återställ",lapText:"Varv",hideText:"Dölj",backText:"Tillbaka",undoText:"Ångra",offText:"Av",onText:"På"},t.i18n.th={setText:"ตั้งค่า",cancelText:"ยกเลิก",clearText:"ล้าง",selectedText:"{count} เลือก",dateFormat:"dd/mm/yy",dayNames:["อาทิตย์","จันทร์","อังคาร","พุธ","พฤหัสบดี","ศุกร์","เสาร์"],dayNamesShort:["อา.","จ.","อ.","พ.","พฤ.","ศ.","ส."],dayNamesMin:["อา.","จ.","อ.","พ.","พฤ.","ศ.","ส."],dayText:"วัน",delimiter:".",hourText:"ชั่วโมง",minuteText:"นาที",monthNames:["มกราคม","กุมภาพันธ์","มีนาคม","เมษายน","พฤษภาคม","มิถุนายน","กรกฎาคม","สิงหาคม","กันยายน","ตุลาคม","พฤศจิกายน","ธันวาคม"],monthNamesShort:["ม.ค.","ก.พ.","มี.ค.","เม.ย.","พ.ค.","มิ.ย.","ก.ค.","ส.ค.","ก.ย.","ต.ค.","พ.ย.","ธ.ค."],monthText:"เดือน",secText:"วินาที",timeFormat:"HH:ii",yearText:"ปี",nowText:"ตอนนี้",pmText:"pm",amText:"am",firstDay:0,dateText:"วัน",timeText:"เวลา",today:"วันนี้",prevMonthText:"เดือนก่อนหน้า",nextMonthText:"เดือนถัดไป",prevYearText:"ปีก่อนหน้า",nextYearText:"ปีถัดไป",closeText:"ปิด",eventText:"เหตุการณ์",eventsText:"เหตุการณ์",allDayText:"ตลอดวัน",noEventsText:"ไม่มีกิจกรรม",moreEventsText:"อีก {count} กิจกรรม",fromText:"จาก",toText:"ถึง",wholeText:"ทั้งหมด",fractionText:"เศษส่วน",unitText:"หน่วย",labels:["ปี","เดือน","วัน","ชั่วโมง","นาที","วินาที",""],labelsShort:["ปี","เดือน","วัน","ชั่วโมง","นาที","วินาที",""],startText:"เริ่ม",stopText:"หยุด",resetText:"รีเซ็ต",lapText:"รอบ",hideText:"ซ่อน",backText:"ย้อนกลับ",undoText:"เลิกทา",offText:"ปิด",onText:"เปิด",decimalSeparator:",",thousandsSeparator:" "},t.i18n.tr={setText:"Seç",cancelText:"İptal",clearText:"Temizleyin",selectedText:"{count} seçilmiş",dateFormat:"dd.mm.yy",dayNames:["Pazar","Pazartesi","Salı","Çarşamba","Perşembe","Cuma","Cumartesi"],dayNamesShort:["Paz","Pzt","Sal","Çar","Per","Cum","Cmt"],dayNamesMin:["P","P","S","Ç","P","C","C"],dayText:"Gün",delimiter:".",hourText:"Saat",minuteText:"Dakika",monthNames:["Ocak","Şubat","Mart","Nisan","Mayıs","Haziran","Temmuz","Ağustos","Eylül","Ekim","Kasım","Aralık"],monthNamesShort:["Oca","Şub","Mar","Nis","May","Haz","Tem","Ağu","Eyl","Eki","Kas","Ara"],monthText:"Ay",secText:"Saniye",timeFormat:"HH:ii",yearText:"Yıl",nowText:"Şimdi",pmText:"pm",amText:"am",todayText:"Bugün",firstDay:1,dateText:"Tarih",timeText:"Zaman",closeText:"Kapatmak",allDayText:"Tüm gün",noEventsText:"Etkinlik Yok",eventText:"Etkinlik",eventsText:"Etkinlikler",moreEventsText:"{count} tane daha",fromText:"Başla",toText:"Son",wholeText:"Tam",fractionText:"Kesir",unitText:"Birim",labels:["Yıl","Ay","Gün","Saat","Dakika","Saniye",""],labelsShort:["Yıl","Ay","Gün","Sa","Dak","Sn",""],startText:"Başla",stopText:"Durdur",resetText:"Sıfırla",lapText:"Tur",hideText:"Gizle",backText:"Geri",undoText:"Geri Al",offText:"O",onText:"I",decimalSeparator:",",thousandsSeparator:"."},t.i18n.ua={setText:"встановити",cancelText:"відміна",clearText:"очистити",selectedText:"{count} вибрані",dateFormat:"dd.mm.yy",dayNames:["неділя","понеділок","вівторок","середа","четвер","п’ятниця","субота"],dayNamesShort:["нед","пнд","вів","срд","чтв","птн","сбт"],dayNamesMin:["Нд","Пн","Вт","Ср","Чт","Пт","Сб"],dayText:"День",delimiter:".",hourText:"година",minuteText:"хвилина",monthNames:["Січень","Лютий","Березень","Квітень","Травень","Червень","Липень","Серпень","Вересень","Жовтень","Листопад","Грудень"],monthNamesShort:["Січ","Лют","Бер","Кві","Тра","Чер","Лип","Сер","Вер","Жов","Лис","Гру"],monthText:"Місяць",secText:"Секунд",timeFormat:"H:ii",yearText:"Рік",nowText:"Зараз",pmText:"pm",amText:"am",firstDay:1,dateText:"дата",timeText:"Час",todayText:"Сьогодні",prevMonthText:"Попередній місяць",nextMonthText:"Наступного місяця",prevYearText:"Попередній рік",nextYearText:"Наступного року",closeText:"Закрити",eventText:"подія",eventsText:"події",allDayText:"Увесь день",noEventsText:"Жодної події",moreEventsText:"та ще {count}",fromText:"від",toText:"кінець",wholeText:"всі",fractionText:"фракція",unitText:"одиниця",labels:["Рік","Місяць","День","година","хвилина","Секунд",""],labelsShort:["Рік","Місяць","День","година","хвилина","Секунд",""],startText:"Початок",stopText:"СТОП",resetText:"скинути",lapText:"коло",hideText:"сховати",backText:"назад",undoText:"відмінити",offText:"Вимикати",onText:"Увімкнути",decimalSeparator:",",thousandsSeparator:" "},t.i18n.vi={setText:"Đặt",cancelText:"Hủy bò",clearText:"Xóa",selectedText:"{count} chọn",dateFormat:"dd/mm/yy",dayNames:["Chủ Nhật","Thứ Hai","Thứ Ba","Thứ Tư","Thứ Năm","Thứ Sáu","Thứ Bảy"],dayNamesShort:["CN","T2","T3","T4","T5","T6","T7"],dayNamesMin:["CN","T2","T3","T4","T5","T6","T7"],dayText:"",delimiter:"/",hourText:"Giờ",minuteText:"Phút",monthNames:["Tháng Một","Tháng Hai","Tháng Ba","Tháng Tư","Tháng Năm","Tháng Sáu","Tháng Bảy","Tháng Tám","Tháng Chín","Tháng Mười","Tháng Mười Một","Tháng Mười Hai"],monthNamesShort:["Tháng 1","Tháng 2","Tháng 3","Tháng 4","Tháng 5","Tháng 6","Tháng 7","Tháng 8","Tháng 9","Tháng 10","Tháng 11","Tháng 12"],monthText:"Tháng",secText:"Giây",timeFormat:"H:ii",yearText:"Năm",nowText:"Bây giờ",pmText:"pm",amText:"am",firstDay:0,dateText:"Ngày",timeText:"Hồi",todayText:"Hôm nay",prevMonthText:"Tháng trước",nextMonthText:"Tháng tới",prevYearText:"Măm trước",nextYearText:"Năm tới",closeText:"Đóng",eventText:"Sự kiện",eventsText:"Sự kiện",allDayText:"Cả ngày",noEventsText:"Không có sự kiện",moreEventsText:"{count} thẻ khác",fromText:"Từ",toText:"Tới",wholeText:"Toàn thể",fractionText:"Phân số",unitText:"đơn vị",labels:["Năm","Tháng","Ngày","Giờ","Phút","Giây",""],labelsShort:["Năm","Tháng","Ngày","Giờ","Phút","Giây",""],startText:"Bắt đầu",stopText:"Dừng",resetText:"Đặt lại",lapText:"Vòng",hideText:"Giấu",backText:"Quay lại",undoText:"Hoàn tác",offText:"Tất",onText:"Bật",decimalSeparator:",",thousandsSeparator:" "},t.i18n.zh={setText:"确定",cancelText:"取消",clearText:"明确",selectedText:"{count} 选",dateFormat:"yy年mm月d日",dayNames:["周日","周一","周二","周三","周四","周五","周六"],dayNamesShort:["日","一","二","三","四","五","六"],dayNamesMin:["日","一","二","三","四","五","六"],dayText:"日",hourText:"时",minuteText:"分",monthNames:["1月","2月","3月","4月","5月","6月","7月","8月","9月","10月","11月","12月"],monthNamesShort:["一","二","三","四","五","六","七","八","九","十","十一","十二"],monthText:"月",secText:"秒",timeFormat:"HH:ii",yearText:"年",nowText:"当前",pmText:"下午",amText:"上午",yearSuffix:"年",monthSuffix:"月",daySuffix:"日",todayText:"今天",dateText:"日",timeText:"时间",closeText:"关闭",allDayText:"全天",noEventsText:"无事件",eventText:"活动",eventsText:"活动",moreEventsText:"他 {count} 件",fromText:"开始时间",toText:"结束时间",wholeText:"合计",fractionText:"分数",unitText:"单位",labels:["年","月","日","小时","分钟","秒",""],labelsShort:["年","月","日","点","分","秒",""],startText:"开始",stopText:"停止",resetText:"重置",lapText:"圈",hideText:"隐藏",backText:"返回",undoText:"复原",offText:"关闭",onText:"开启",decimalSeparator:",",thousandsSeparator:" "},en=t.themes,en.frame.bootstrap={disabledClass:"disabled",selectedClass:"btn-primary",selectedTabClass:"active",tabLink:!0,todayClass:"text-primary mbsc-cal-today",onMarkupInserted:function(s){var t=e(s.target),n=e(".mbsc-cal-tabs",t);e(".mbsc-fr-popup",t).addClass("popover"),e(".mbsc-fr-w",t).addClass("popover-content"),e(".mbsc-fr-hdr",t).addClass("popover-title popover-header"),e(".mbsc-fr-arr-i",t).addClass("popover"),e(".mbsc-fr-arr",t).addClass("arrow"),e(".mbsc-fr-btn",t).addClass("btn btn-default btn-secondary"),e(".mbsc-fr-btn-s .mbsc-fr-btn",t).removeClass("btn-default btn-secondary").addClass("btn btn-primary"),n.addClass("nav nav-tabs"),n.find(".mbsc-cal-tab").addClass("nav-item"),n.find("a").addClass("nav-link"),n.find(".mbsc-cal-tab.active .nav-link").addClass("active"),e(".mbsc-cal-picker",t).addClass("popover"),e(".mbsc-range-btn",t).addClass("btn btn-sm btn-small btn-default"),e(".mbsc-np-btn",t).addClass("btn btn-default"),e(".mbsc-sel-filter-cont",t).removeClass("mbsc-input"),e(".mbsc-sel-filter-input",t).addClass("form-control")},onTabChange:function(n,t){e(".mbsc-cal-tabs .nav-link",t._markup).removeClass("active"),e(".mbsc-cal-tab.active .nav-link",t._markup).addClass("active")},onPosition:function(t){setTimeout(function(){e(".mbsc-fr-bubble-top, .mbsc-fr-bubble-top .mbsc-fr-arr-i",t.target).removeClass("bottom bs-popover-bottom").addClass("top bs-popover-top"),e(".mbsc-fr-bubble-bottom, .mbsc-fr-bubble-bottom .mbsc-fr-arr-i",t.target).removeClass("top bs-popover-top").addClass("bottom  bs-popover-bottom")},10)}},en.scroller.bootstrap=n({},en.frame.bootstrap,{dateDisplay:"Mddyy",btnCalPrevClass:"mbsc-ic mbsc-ic-arrow-left5",btnCalNextClass:"mbsc-ic mbsc-ic-arrow-right5",btnPlusClass:"mbsc-ic mbsc-ic-arrow-down5 btn-light",btnMinusClass:"mbsc-ic mbsc-ic-arrow-up5 btn-light",selectedLineHeight:!0,onEventBubbleShow:function(n){var t=e(n.eventList);e(".mbsc-cal-event-list",t).addClass("list-group"),e(".mbsc-cal-event",t).addClass("list-group-item")}}),en.navigation.bootstrap={wrapperClass:"popover panel panel-default",groupClass:"btn-group",activeClass:"btn-primary",disabledClass:"disabled",itemClass:"btn btn-default"},en.form.bootstrap={},ec=t.themes;function eV(t,i){var a=o(i,"X",!0),l=o(i,"Y",!0),s=t.offset(),r=a-s.left,c=l-s.top,d=Math.max(r,t[0].offsetWidth-r),u=Math.max(c,t[0].offsetHeight-c),n=2*Math.sqrt(Math.pow(d,2)+Math.pow(u,2));e3(Q),Q=e('<span class="mbsc-ripple"></span>').css({width:n,height:n,top:l-s.top-n/2,left:a-s.left-n/2}).appendTo(t),setTimeout(function(){Q.addClass("mbsc-ripple-scaled mbsc-ripple-visible")},10)}function e3(e){setTimeout(function(){e&&(e.removeClass("mbsc-ripple-visible"),setTimeout(function(){e.remove()},2e3))},100)}function nn(i,t,a,r){var n,s;i.off(".mbsc-ripple").on("touchstart.mbsc-ripple mousedown.mbsc-ripple",t,function(t){H(t,this)&&(n=o(t,"X"),s=o(t,"Y"),(R=e(this)).hasClass(a)||R.hasClass(r)?R=null:eV(R,t))}).on("touchmove.mbsc-ripple mousemove.mbsc-ripple",t,function(e){(R&&9<Math.abs(o(e,"X")-n)||9<Math.abs(o(e,"Y")-s))&&(e3(Q),R=null)}).on("touchend.mbsc-ripple touchcancel.mbsc-ripple mouseleave.mbsc-ripple mouseup.mbsc-ripple",t,function(){R&&(setTimeout(function(){e3(Q)},100),R=null)})}return ec.frame.ios={display:"bottom",headerText:!1,btnWidth:!1,deleteIcon:"ios-backspace",scroll3d:"wp"!=h&&("android"!=h||7<x)},ec.scroller.ios=n({},ec.frame.ios,{rows:5,height:34,minWidth:55,selectedLineHeight:!0,selectedLineBorder:1,showLabel:!1,useShortLabels:!0,btnPlusClass:"mbsc-ic mbsc-ic-arrow-down5",btnMinusClass:"mbsc-ic mbsc-ic-arrow-up5",checkIcon:"ion-ios7-checkmark-empty",filterClearIcon:"ion-close-circled",dateDisplay:"MMdyy",btnCalPrevClass:"mbsc-ic mbsc-ic-arrow-left5",btnCalNextClass:"mbsc-ic mbsc-ic-arrow-right5"}),ec.listview.ios={leftArrowClass:"mbsc-ic-ion-ios7-arrow-back",rightArrowClass:"mbsc-ic-ion-ios7-arrow-forward"},ec.form.ios={},ee=t.themes,ee.frame.material={headerText:!1,btnWidth:!1,deleteIcon:"material-backspace",onMarkupReady:function(t){nn(e(t.target),".mbsc-fr-btn-e","mbsc-disabled","mbsc-fr-btn-nhl")}},ee.scroller.material=n({},ee.frame.material,{showLabel:!1,selectedLineBorder:2,weekDays:"min",icon:{filled:"material-star",empty:"material-star-outline"},checkIcon:"material-check",btnPlusClass:"mbsc-ic mbsc-ic-material-keyboard-arrow-down",btnMinusClass:"mbsc-ic mbsc-ic-material-keyboard-arrow-up",btnCalPrevClass:"mbsc-ic mbsc-ic-material-keyboard-arrow-left",btnCalNextClass:"mbsc-ic mbsc-ic-material-keyboard-arrow-right"}),ee.listview.material={leftArrowClass:"mbsc-ic-material-keyboard-arrow-left",rightArrowClass:"mbsc-ic-material-keyboard-arrow-right",onItemActivate:function(t){eV(e(t.target),t.domEvent)},onItemDeactivate:function(){e3(Q)},onSlideStart:function(t){e(".mbsc-ripple",t.target).remove()},onSortStart:function(t){e(".mbsc-ripple",t.target).remove()}},ee.navigation.material={onInit:function(){nn(e(this),".mbsc-ms-item.mbsc-btn-e","mbsc-disabled","mbsc-btn-nhl")},onMarkupInit:function(){e(".mbsc-ripple",this).remove()},onDestroy:function(){e(this).off(".mbsc-ripple")}},ee.form.material={addRipple:function(e,t){eV(e,t)},removeRipple:function(){e3(Q)}},em=t.themes,em.frame.windows={headerText:!1,deleteIcon:"backspace4",btnReverse:!0},em.scroller.windows=n({},em.frame.windows,{rows:6,minWidth:88,height:44,btnPlusClass:"mbsc-ic mbsc-ic-arrow-down5",btnMinusClass:"mbsc-ic mbsc-ic-arrow-up5",checkIcon:"material-check",dateDisplay:"MMdyy",showLabel:!1,showScrollArrows:!0,btnCalPrevClass:"mbsc-ic mbsc-ic-arrow-left5",btnCalNextClass:"mbsc-ic mbsc-ic-arrow-right5",dayNamesShort:["Su","Mo","Tu","We","Th","Fr","Sa"],useShortLabels:!0}),em.form.windows={},t.customTheme("ios-dark","ios"),t.customTheme("material-dark","material"),t.customTheme("mobiscroll-dark","mobiscroll"),t.customTheme("windows-dark","windows"),nf=t.themes,N="mobiscroll","android"==h?N="material":"ios"==h?N="ios":"wp"==h&&(N="windows"),e.each(nf.frame,function(e,n){if(N&&n.baseTheme==N&&e!=N+"-dark")return t.autoTheme=e,!1;e==N&&(t.autoTheme=e)}),t.customTheme("ios-gray","ios"),t.customTheme("material-indigo","material"),t.customTheme("mobiscroll-lime","mobiscroll"),t.customTheme("windows-yellow","windows"),t})